/**@name LIBRARY: gcsAnalogDelayedAlarm.ctl

@author: Francisco Mico (IT-CO)

Creation Date: 17 06 2003

Modification History:
	$Log: gcsAnalogDelayedAlarm.ctl,v $
	Revision 1.17  2009/03/30 12:14:48  gamx
	fixed bug in animation: priority of "N" is higher than "O"
	
	Revision 1.16  2009/01/28 14:05:58  gamx
	Fixes in _WidgetDisconnection function:
	A standard "unGenericObject_WidgetDisconnections" function cannot be used for ADA or DDA objects (due to additional graphical objects on the widget panel).
	A custom disconnection function has been introduced.

	Revision 1.15  2008/11/12 08:22:17  gthomas
	-removed function calls to _ButtonUserAccessGcs and _ButtonUserAccessUn

	Revision 1.14  2008/07/18 07:16:55  gthomas
	added new Access control functionality for unicos-4-1
	modified  xx_ButtonUserAccess function.

	Revision 1.13  2008/06/18 14:54:51  aabadz
	Changes done following the recomendations from AB-CO (from Herve)

	Revision 1.12  2008/03/17 10:52:36  aabadz
	Fixed a problem found in animation when we have not database.

	Revision 1.11  2008/01/22 14:46:36  aabadz
	Fixed:
	- Naming
	- Call to other functions to reuse code
	- Some error check

	Revision 1.10  2008/01/14 09:58:25  aabadz
	Changes after code review and before tests

	Revision 1.9  2008/01/14 09:48:47  aabadz
	After tests

	Revision 1.8  2007/10/30 16:12:14  gthomas
	Introduced the CVS keyword $Log to track changes made in the library

	25.09.2006 Kulman Nikolay
		- Porting to Unicos 3.6
	25.11.2004 Kulman Nikolay
		- function gcsAnalogDelayedAlarm_HandleMenu modified to deselect widget automaticly
		- function gcsAnalogDelayedAlarm_WidgetAnimationCB modified
	03.11.2004 Kulman Nikolay
		- function gcsAnalogDelayedAlarm_HandleMenu modified to handle Mask/UnMask menu commands
	05/05/2004: Herve AB/CO
		- function gcsAnalogDelayedAlarm_MenuConfiguration modified: signature modified and add the creation of the menu
		- function gcsAnalogDelayedAlarm_HandleMenu added: this function handles the rigth click menu
		- function gcsAnalogDelayedAlarm_ButtonSetState modified: modification of the signature of the function
		- function gcsAnalogDelayedAlarm_UserLoginGetButtonState added: called when user logs in, returns the state of the buttons
		- function gcsAnalogDelayedAlarm_ButtonRegister modified : add the set Manreg01 to 0 in case of dpConnect
		- function gcsAnalogDelayedAlarm_ObjectListGetValueTime added: called in object list.
		- function gcsAnalogDelayedAlarm_ButtonAnimationCB modified: signature of the function gcsAnalogDelayedAlarm_ButtonSetState was modified
		- function gcsAnalogDelayedAlarm_FaceplateConnect, gcsAnalogDelayedAlarm_FaceplateDisconnect and gcsAnalogDelayedAlarm_WidgetRegisterCB
		bug in case the PLC is not defined and if the PVSS system name contains '-',
		call unGenericObject_GetPLCNameFromDpName to get the PLC DPName instead (this function corrects the bugs).

version 1.0

External Function :
	. gcsAnalogDelayedAlarm_AcknowledgeAlarm : animate AnalogDelayedAlarm alarms
	. gcsAnalogDelayedAlarm_FaceplateStatusDisconnection : animate faceplate disconnection
	. gcsAnalogDelayedAlarm_ButtonUserAccess : User access for AnalogDelayedAlarm buttons
	. gcsAnalogDelayedAlarm_ButtonSetState : set button state
	. gcsAnalogDelayedAlarm_WidgetDisconnection : animate widget disconnection
	. gcsAnalogDelayedAlarm_MenuConfiguration : configuration of popup menu


Internal Functions :
	. gcsAnalogDelayedAlarm_FaceplateStatusRegisterCB : register callback function
	. gcsAnalogDelayedAlarm_FaceplateConnect : dpConnect to the DataPoint
	. gcsAnalogDelayedAlarm_FaceplateDisconnect : dpDisconnect to the DataPoint
	. gcsAnalogDelayedAlarm_FaceplateStatusAnimationCB : animate faceplate
	. gcsAnalogDelayedAlarm_ButtonRegisterCB : register callback function
	. gcsAnalogDelayedAlarm_ButtonAnimationCB : animate buttons
	. gcsAnalogDelayedAlarm_ButtonDisconnection : animate buttons disconnection
	. gcsAnalogDelayedAlarm_WidgetRegisterCB : register callback function
	. gcsAnalogDelayedAlarm_WidgetConnect : dpConnect to the DataPoint
	. gcsAnalogDelayedAlarm_WidgetDisconnect : dpDisconnect to the DataPoint
	. gcsAnalogDelayedAlarm_WidgetAnimationCB : animate widget

Purpose: This library contains AnalogDelayedAlarm object functions.

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. AnalogDelayedAlarm object (DPType etc.)
	. Global variables and $parameters defined in AnalogDelayedAlarm faceplate and widget
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

//@{

//---------------------------------------------------------------------------------------------------------------------------------------

// gcsAnalogDelayedAlarm_ObjectListGetValueTime
/**
Purpose: Function called from the objectList to return the time and value

Parameters:
	- sDeviceName, string, input, device name
	- sDeviceType, string, input, unicos object
	- dsReturnData, dyn_string, output, return value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
gcsAnalogDelayedAlarm_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
	float value;
	string sTime, sStatus,sColor;
	bool bBad;

	dpGet(sDeviceName+".ProcessInput.PosAl", value,
	      sDeviceName+".ProcessInput.PosAl:_online.._stime", sTime,
	      sDeviceName+".ProcessInput.PosAl:_online.._bad", bBad);

	dsReturnData[1] = sTime;
	dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(sDeviceName+".ProcessInput.PosAl"), value) + " "
							+ dpGetUnit(sDeviceName+".ProcessInput.PosAl");

	// Optionally (when no stsReg* exists)
	// Status
	dsReturnData[3] = bBad;

	// Status to visualize
	sStatus="OK";
	dsReturnData[4] = sStatus;
	// Colors
	sColor="unEventList_ForeColor";
	dsReturnData[5]=sColor;
}

//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogDelayedAlarm_AcknowledgeAlarm
/**
Purpose: acknowledge AnalogDelayedAlarm alarms

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsNeedAck, dyn_string, output, DPEs names that need acknowledgement.
									 The PLC acknowledgement is specified using UN_ACKNOWLEDGE_PLC string

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in OnOff faceplate
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
gcsAnalogDelayedAlarm_AcknowledgeAlarm(string sDpName, string sDpType, dyn_string &dsNeedAck)
{
	dsNeedAck = makeDynString("ProcessInput.PosAl", "ProcessInput.WSt", UN_ACKNOWLEDGE_PLC);
}


//------------------------------------------------------------------------------------------------------------
// gcsAnalogDelayedAlarm_MenuConfiguration
/**
Purpose: configuration of popup menu

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsMenuConfig, dyn_string, output, configuration of menu. Use defined constants UN_POPUPMENU_***

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
gcsAnalogDelayedAlarm_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{
	dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_ACK_TEXT, UN_MENU_AA_MASK, UN_POPUPMENU_DIAG_INFO_TEXT,
		UN_POPUPMENU_TREND_TEXT, UN_MENU_MAIL_SMS);

	gcsGenericObject_addMaskToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	unGenericObject_addUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	unGenericObject_addDiagnosticAndInfoToActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	unGenericObject_addTrendActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogDelayedAlarm_HandleMenu
/**
Purpose: handle the answer of the popup menu

Parameters:
	- deviceName, string, input, the device dp name
	- sDpType, string, input, the device dp type
	- menuList, dyn_string, input, list of requested action
	- menuAnswer, int, input, selected action from the menuList

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
gcsAnalogDelayedAlarm_HandleMenu(string deviceName, string sDpType, dyn_string menuList, int menuAnswer)
{
	dyn_string exceptionInfo;

	gcsGenericObject_HandleEventMaskMenu(deviceName, menuAnswer, makeDynString(".ProcessInput.evStsReg01", ".ProcessInput.PosAl", ".ProcessInput.WSt"), exceptionInfo);
	unGenericObject_HandleUnicosMenu(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);
	gcsGenericObject_HandleAlarmMaskMenu(deviceName, menuList, menuAnswer, exceptionInfo);
	if (dynlen(exceptionInfo) > 0)
		unSendMessage_toExpertException("Right Click",exceptionInfo);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogDelayedAlarm_ButtonUserAccess
/**
Purpose: User access for AnalogDelayedAlarm buttons

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, dptype
	- dsAccess, dyn_string, output, list of allowed buttons

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
gcsAnalogDelayedAlarm_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
	
	bool operator, expert, admin;
	dyn_bool dbPermissions;
	dyn_string exceptionInfo;
	string sOperAction, sExpAction, sAdminAction;
	bool bActionDefined = false;
	
	unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo);
	if((sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME) ||
	(sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME))
	bActionDefined=true;
	
	operator = dbPermissions[2];
	expert = dbPermissions[3];
	admin = dbPermissions[4];
	dsAccess = makeDynString();
	
	if (operator){
		if(bActionDefined)
			dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
		else
			dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_ACK_ALARM));
	}
	// to check why Block/unblock action is missing !
	if (expert){
		if(bActionDefined)
			dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
		else
			dynAppend(dsAccess, makeDynString(UN_MENU_MAIL_SMS, UN_FACEPLATE_MASKEVENT));
	}
	if(admin){
		if(bActionDefined)
			dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
		else
        		dynAppend(dsAccess, makeDynString(UN_MENU_ALARM_SET_MASK, UN_MENU_ALARM_RESET_MASK));
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogDelayedAlarm_WidgetRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in AnalogDelayedAlarm widget
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
gcsAnalogDelayedAlarm_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, sPlcName, sSystemName;
	int iAction;
	string sPlcDpName;
	dyn_string exceptionInfo;
	bool bRemote;

	deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
	if (deviceName == "")	// In case of disconnection
	{
		deviceName = g_sDpName;
	}

	sSystemName = unGenericDpFunctions_getSystemName(deviceName);

	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;

	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	gcsConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)
		g_bUnSystemAlarmPlc = true;
	else
		g_bUnSystemAlarmPlc = false;


	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);

	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			gcsAnalogDelayedAlarm_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPCONNECT:
			g_sDpName = deviceName;

			if(dpTypeName(deviceName) == GCS_CONFIG_ANALOGDELAYEDALARM_DPT_NAME)
				gcsAnalogDelayedAlarm_WidgetConnect(deviceName, sPlcName, sSystemName);
			else
			{
				g_bUnSystemAlarmPlc = false;
				gcsAnalogDelayedAlarm_WidgetDisconnection(g_sWidgetType);
			}
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			gcsAnalogDelayedAlarm_WidgetDisconnect(deviceName, sPlcName, sSystemName);
			gcsAnalogDelayedAlarm_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_NOTHING: break;
		default:
			break;
		}
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogDelayedAlarm_WidgetConnect
/**
23.04.2004
Teodora Miteva

Purpose: do dpConnect

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in AnalogDelayedAlarm faceplate
	. PVSS version: 2.12.1
	. operating system: XP and W2000, but tested only under XP.
	. distributed system: yes.
*/
gcsAnalogDelayedAlarm_WidgetConnect(string deviceName, string sPlcName, string sSystemName)
{
	int iRes;
	if(g_bUnSystemAlarmPlc)
	{
		iRes = dpConnect("gcsAnalogDelayedAlarm_WidgetAnimationCB",
			deviceName + ".statusInformation.selectedManager:_lock._original._locked",
			deviceName + ".statusInformation.selectedManager",
			deviceName + ".ProcessInput.StsReg01",
			deviceName + ".ProcessInput.StsReg01:_online.._invalid",
			sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
			sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
			deviceName + ".eventMask");
		g_bCallbackConnected = (iRes >= 0);
	}
	else
		gcsAnalogDelayedAlarm_WidgetDisconnection(g_sWidgetType);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogDelayedAlarm_WidgetDisconnect
/**
23.04.2004
Teodora Miteva

Purpose: do dpDisconnect

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in AnalogDelayedAlarm faceplate
	. PVSS version: 2.12.1
	. operating system: XP and W2000, but tested only under XP.
	. distributed system: yes.
*/
gcsAnalogDelayedAlarm_WidgetDisconnect(string deviceName, string sPlcName, string sSystemName)
{
	int iRes;
	if (g_bUnSystemAlarmPlc)
	{
		iRes = dpDisconnect("gcsAnalogDelayedAlarm_WidgetAnimationCB",
			deviceName + ".statusInformation.selectedManager:_lock._original._locked",
			deviceName + ".statusInformation.selectedManager",
			deviceName + ".ProcessInput.StsReg01",
			deviceName + ".ProcessInput.StsReg01:_online.._invalid",
			sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
			sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
			deviceName + ".eventMask");
		g_bCallbackConnected = !(iRes >= 0);
	}
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogDelayedAlarm_WidgetAnimationCB
/**
Purpose: Animate widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in AnalogDelayedAlarm faceplate
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
gcsAnalogDelayedAlarm_WidgetAnimationCB(string sDpLock, bool bLocked, string sDpManager, string SelectedManager,
   		string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid,
		string sDpAl, int iSystemIntegrityAlarmValue, string sDpEnabled, bool bSystemIntegrityAlarmEnabled,
		string sEvtMask, int iMask)
{
	string sBody1Color, sBody1BColor = "", sWarningColor = "",sWarningLetter = "";
	string sAlarmTypeLetter = "", sAlarmTypeColor = "", sAlarmTextLetter = "", sAlarmTextColor = "";

	sBody1Color = "unDataNotValid";

	if (!bStsReg01Invalid)
	{
// all is OK - so alarm is green
		sBody1Color = "gcsAlarm_OK";
		sBody1BColor = "";

		if (getBit(bit32StsReg01, UN_STSREG01_AST) == 1)		//	Alarm is active
		{
			sBody1Color  = "gcsAlarm_Alarm";
			sBody1BColor = "gcsAlarm_Alarm";
		}
		else if (getBit(bit32StsReg01, UN_STSREG01_WST) == 1)		//	Warning is active
		{
			sBody1Color  = "gcsAlarm_Warning";
		}

//	Animate alarm type notification (H, HH, L, LL)
			if(getBit(bit32StsReg01, UN_STSREG01_HST) == 1)	//	H
			{	sAlarmTypeLetter = UN_WIDGET_ALARM_HIGH;
				sAlarmTypeColor  = "gcsAlarm_Warning";
			}
			if(getBit(bit32StsReg01, UN_STSREG01_LST) == 1)	//	L
			{	sAlarmTypeLetter = UN_WIDGET_ALARM_LOW;
				sAlarmTypeColor  = "gcsAlarm_Warning";
			}
			if(getBit(bit32StsReg01, UN_STSREG01_HHST) == 1)	//	HH
			{	sAlarmTypeLetter = UN_WIDGET_ALARM_HIGHHIGH;
				sAlarmTypeColor  = "gcsAlarm_Alarm";
			}
			if(getBit(bit32StsReg01, UN_STSREG01_LLST) == 1)	//	LL
			{	sAlarmTypeLetter = UN_WIDGET_ALARM_LOWLOW;
				sAlarmTypeColor  = "gcsAlarm_Alarm";
			}

// Alarm not acknowledged (higher priority)
			if (getBit(bit32StsReg01, UN_STSREG01_ALUNACK) == 1)
			{	sBody1Color  = "gcsAlarm_AlarmNotAck";
				if (getBit(bit32StsReg01, UN_STSREG01_AST) == 1)
					sBody1BColor = "gcsAlarm_AlarmNotAck";
				else if (getBit(bit32StsReg01, UN_STSREG01_WST) == 1)
					sBody1Color  = "gcsAlarm_WarningNotAck";
			}

			if (getBit(bit32StsReg01, UN_STSREG01_MAMSKST) == 1)			// Alarm masked
			{
				sAlarmTextLetter = UN_WIDGET_TEXT_ALARM_BLOCKED;
				sAlarmTextColor = "unAlarmMasked";
				sBody1Color  = "gcsAlarm_OK";
				sBody1BColor = "";
			}

			unGenericObject_WidgetWarningAnimation(bit32StsReg01,
				makeDynInt(/*UN_STSREG01_AUMRW,*/ UN_STSREG01_IOSIMUW, UN_STSREG01_IOERRORW),
				makeDynString(/*UN_WIDGET_TEXT_WARNING_DEFAULT,*/ UN_WIDGET_TEXT_WARNING_SIMU, UN_WIDGET_TEXT_WARNING_ERROR),
				sWarningLetter, sWarningColor);
	}

	//data quality animation
	if((!bSystemIntegrityAlarmEnabled)||(iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value))
	{
		sWarningLetter = UN_WIDGET_TEXT_OLD_DATA;
		sWarningColor = "unDataNotValid";
	}
	
	if (bStsReg01Invalid){
		sWarningLetter = UN_WIDGET_TEXT_INVALID;
		sWarningColor = "unDataNotValid";	
	}

	if(g_bSystemConnected)
	{
		setMultiValue("WarningText", "text", sWarningLetter, "WarningText", "foreCol", sWarningColor,
			"AlarmType", "text", sAlarmTypeLetter, "AlarmType", "foreCol", sAlarmTypeColor,
			"AlarmText", "text", sAlarmTextLetter, "AlarmText","foreCol", sAlarmTextColor,
			"ControlStateText", "text", "", "SelectArea", "foreCol", "",
			"LockBmp", "visible", false, "Body1", "fill", "[solid]",
			"Body1", "foreCol", sBody1Color, "Body1", "backCol", sBody1BColor,
			"WidgetArea", "visible", true);
		if(iMask == 0)
			eventState.visible	= true;
		else
			eventState.visible	= false;
	}
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsAnalogDelayedAlarm_WidgetDisconnection
/**
Purpose: Animate widget disconnection

Parameters:
	- sWidgetType, string, input, widget type. ex: "heater"

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
gcsAnalogDelayedAlarm_WidgetDisconnection(string sWidgetType)
{
	setMultiValue("Body1", "foreCol", "unDataNoAccess", "Body1", "backCol", "unWidget_Background", "Body1", "fill", "[solid]",
	   "WarningText", "text", "", "WidgetArea", "visible", false, "AlarmType", "text", "", "AlarmText", "text", "");
	eventState.visible = false;
}




