/**@name LIBRARY: gcsConfigAnalogDelayedAlarm.ctl

@author: Nikolay Kulman (IT-CO-FE)
@author: Francisco Mico (IT-CO-FE)
@date 24.07.2003
@brief This library contains the function used to set AnalogDelayedAlarm configuration.
Modifications History:
  $Log: gcsConfigAnalogDelayedAlarm.ctl,v $
  Revision 1.18  2010/09/14 16:34:00  amerezhi
  the check for boolean GCS_CONFIG_ANALOGDELAYEDALARM_ACKNOWLEDGE_TYPE is fixed

  Revision 1.17  2010/04/22 14:26:19  gthomas
  JS fixed bug #60589 (bad address mapping of ADA)

  Revision 1.17  2010/04/22 09:24:00  jszkutni
  -UN_STSREG01_POSAL replaced by UN_STSREG01_AST

  Revision 1.16  2009/03/20 13:57:23  gthomas
  -updated variable names <_GCSDELAYEDALARM_CLASS> with <_ALERT_CLASS_>

  Revision 1.15  2008/09/04 14:44:03  gthomas
  -moved back to version 1.12

  Revision 1.14  2008/06/18 14:54:51  aabadz
  Changes done following the recomendations from AB-CO (from Herve)

  Revision 1.13  2008/06/09 14:54:26  aabadz
  Result after move some constants, only use use inside this file, from gcsObjects_declarations.

  Revision 1.12  2008/01/22 14:46:36  aabadz
  Fixed:
  - Naming
  - Call to other functions to reuse code
  - Some error check

  Revision 1.11  2007/11/15 08:38:52  ikoblik
  gcsConfigGenericFunctions code review comments implemented and tested for syntax errors

  Revision 1.10  2007/11/14 16:39:31  ikoblik
  gcsConfigGenericFunctions code review comments implementation

  Revision 1.9  2007/10/30 16:20:38  gthomas
  Introduced the CVS keyword $Log to track changes made in the library

*/
/*
Internal Functions :
  gcsConfigAnalogDelayedAlarm_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  _UnPlc_AnalogDelayedAlarm_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  S7_PLC_AnalogDelayedAlarm_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  gcsConfigAnalogDelayedAlarm_setCommonConfig (string plcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
  gcsConfigAnalogDelayedAlarm_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  _UnPlc_AnalogDelayedAlarm_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  S7_PLC_AnalogDelayedAlarm_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  gcsConfigAnalogDelayedAlarm_checkCommonConfig (string plcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
*/

// Constants
const string GCS_CONFIG_ANALOGDELAYEDALARM_DPT_NAME = "AnalogDelayedAlarm";
// Constants
const unsigned GCS_CONFIG_ANALOGDELAYEDALARM_LENGTH         = 7;

const unsigned GCS_CONFIG_ANALOGDELAYEDALARM_NORMAL_POSITION    = 1;
const unsigned GCS_CONFIG_ANALOGDELAYEDALARM_ADDRESS_STSREG01     = 2;
const unsigned GCS_CONFIG_ANALOGDELAYEDALARM_ADDRESS_EVSTSREG01   = 3;
const unsigned GCS_CONFIG_ANALOGDELAYEDALARM_ADDRESS_MANREG01     = 4;
const unsigned GCS_CONFIG_ANALOGDELAYEDALARM_DESCRIPTION      = 5;
const unsigned GCS_CONFIG_ANALOGDELAYEDALARM_ACKNOWLEDGE_TYPE   = 6;
const unsigned GCS_CONFIG_ANALOGDELAYEDALARM_MASKEVENT        = 7;

const unsigned UN_CONFIG_ANALOGDELAYEDALARM_LENGTH          = 7;
const unsigned UN_CONFIG_ANALOGDELAYEDALARM_ADDITIONAL_LENGTH   = 0;





//------------------------------------------------------------------------------------------------------------------------
// gcsConfigAnalogDelayedAlarm_setConfig
/**
Set configuration of given AnalogDelayedAlarm datapoint

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
    dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
    are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo All possible errors will be returned here
*/
gcsConfigAnalogDelayedAlarm_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string sDpName, sDescription;

// 1. Initialization : checking + dpCreate
  unConfigGenericFunctions_checkAll(dsConfigs, "AnalogDelayedAlarm", exceptionInfo);
  if (dynlen(exceptionInfo) <= 0)   // Good inputs
  {
    sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
    unConfigGenericFunctions_createDp(sDpName, "AnalogDelayedAlarm", exceptionInfo);
    if (dynlen(exceptionInfo) <= 0) // Creation ok or dp already exists
    {
// Parameters
      unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
// call the device check function
      unConfigGenericFunctions_setDeviceConfig(dsConfigs, "AnalogDelayedAlarm", exceptionInfo);
    }
  }
}


//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_AnalogDelayedAlarm_setConfig
/**
Set configuration of given AnalogDelayedAlarm datapoint for _UnPlc front-end type

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
_UnPlc_AnalogDelayedAlarm_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  gcsConfigAnalogDelayedAlarm_setCommonConfig(UN_PLC_DPTYPE, dsConfigs, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_AnalogDelayedAlarm_setConfig
/**
Set configuration of given AnalogDelayedAlarm datapoint for S7_PLC front-end type

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
S7_PLC_AnalogDelayedAlarm_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  gcsConfigAnalogDelayedAlarm_setCommonConfig(S7_PLC_DPTYPE, dsConfigs, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// gcsConfigAnalogDelayedAlarm_setCommonConfig
/**
Set configuration of given AnalogDelayedAlarm datapoint for both unPLc and S7_PLC front-end type

  @param[in]  plcType     PLC type. must be UN_PLC_DPTYPE or S7_PLC_DPTYPE for the moment
  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
gcsConfigAnalogDelayedAlarm_setCommonConfig(string plcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string str, sDpName, sDescription, sDeviceNamePosAl, sDeviceNameWSt;
  string alarmParameter, alarmExists, alarmActive, className;
  bool bOkRange, bSMS, bType;
  string sMask = "0";
  int iMode, iMask = 0;

  if(!gcsConfigGenericFunctions_checkPlcType(plcType))
  {
    fwException_raise(exceptionInfo,"ERROR","gcsConfigAnalogDelayedAlarm_setCommonConfig: " + getCatStr("gcsGeneration","PLCUNKNOWN") + plcType,"");
    return;
  }
  sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];

// if archive name are in the config->overwrite setting from import panel.
  gcsConfigGenericFunctions_setArchives(GCS_CONFIG_ANALOGDELAYEDALARM_LENGTH, dsConfigs);

  sMask = dsConfigs[GCS_CONFIG_ANALOGDELAYEDALARM_MASKEVENT + UN_CONFIG_ALLCOMMON_LENGTH];
  sscanf(sMask, "%d", iMask);

// Registers
  if(plcType == UN_PLC_DPTYPE)
    str = " unConfigGenericFunctions_setAllRegisters(dsConfigs, \n";
  if(plcType == S7_PLC_DPTYPE)
    str = " unConfigGenericFunctions_setAllRegistersS7(dsConfigs, \n";
  str += "      makeDynString(\"StsReg01\"), \n";
  str += "      makeDynInt(GCS_CONFIG_ANALOGDELAYEDALARM_ADDRESS_STSREG01), \n";
  str += "      makeDynInt(GCS_CONFIG_ANALOGDELAYEDALARM_ADDRESS_EVSTSREG01), \n";
  str += "      makeDynString(\"ManReg01\"), \n";
  str += "      makeDynInt(GCS_CONFIG_ANALOGDELAYEDALARM_ADDRESS_MANREG01), \n";
  str += "      makeDynString(\"PosAl;WSt\"), \n";
  str += "      makeDynString(UN_STSREG01_AST + \";\" + UN_STSREG01_WST), \n";
  str += "      GCS_CONFIG_ANALOGDELAYEDALARM_NORMAL_POSITION, \n";
  str += "      exceptionInfo); \n";

  evalScript(exceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string exceptionInfo; " + str + " return exceptionInfo;}",
    makeDynString(), dsConfigs);

// 3. additional config
  unConfigGenericFunctions_setMaskEvent(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], iMask,
    makeDynString(".ProcessInput.evStsReg01", ".ProcessInput.PosAl", ".ProcessInput.WSt"), exceptionInfo);

// Description for the Recipes
// check if an alarm is in the datapoint
  alarmParameter = dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + GCS_CONFIG_ANALOGDELAYEDALARM_NORMAL_POSITION];
  unConfigGenericFunctions_getNormalPosition(alarmParameter, alarmExists, alarmActive, bOkRange, iMode, bSMS);

  sDeviceNamePosAl = (sDpName + ".ProcessInput.PosAl");
  sDeviceNameWSt = (sDpName + ".ProcessInput.WSt");

  // if an alarm exists set the corresponding string from the csv file
  if (alarmExists)
  {
    sDescription = dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_ANALOGDELAYEDALARM_DESCRIPTION];
    gcsConfigGenericFunctions_checkBool(dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_ANALOGDELAYEDALARM_ACKNOWLEDGE_TYPE], bType, exceptionInfo);

    if(bSMS)  className = bType ? GCS_CONFIG_ALERT_CLASS_ALARM_ACK_MAIL : GCS_CONFIG_ALERT_CLASS_ALARM_NOACK_MAIL;
    else    className = bType ? GCS_CONFIG_ALERT_CLASS_ALARM_ACK : GCS_CONFIG_ALERT_CLASS_ALARM_NOACK;
    gcsConfigGenericFunctions_setDigitalAlert(dsConfigs, sDeviceNamePosAl, sDescription, bOkRange, alarmActive, bSMS, className, exceptionInfo);

  //  no SMS on warnings
    className = bType ? GCS_CONFIG_ALERT_CLASS_WARNING_ACK : GCS_CONFIG_ALERT_CLASS_WARNING_NOACK;
    gcsConfigGenericFunctions_setDigitalAlert(dsConfigs, sDeviceNameWSt, sDescription, bOkRange, alarmActive, FALSE, className, exceptionInfo);
  }
  else
  {
    fwAlertConfig_deleteDigital(sDeviceNamePosAl, exceptionInfo);
    fwAlertConfig_deleteDigital(sDeviceNameWSt, exceptionInfo);
  }
}


//------------------------------------------------------------------------------------------------------------------------
// gcsConfigAnalogDelayedAlarm_checkConfig
/**
Check data before configuration

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
gcsConfigAnalogDelayedAlarm_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
unsigned uUnsigned;

  if(dynlen(dsConfigs) >= UN_CONFIG_COMMON_LENGTH)
  {
// Common
    unConfigGenericFunctions_checkParameters(dsConfigs, "AnalogDelayedAlarm", exceptionInfo);
	if (dynlen(dsConfigs) >= UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_ANALOGDELAYEDALARM_MASKEVENT) {
		unConfigGenericFunctions_checkUnsigned(dsConfigs[UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_ANALOGDELAYEDALARM_MASKEVENT], uUnsigned, exceptionInfo);
	}
// call the device check function
    unConfigGenericFunctions_checkDeviceConfig(dsConfigs, "AnalogDelayedAlarm", exceptionInfo);
  }
  else
  {
    fwException_raise(exceptionInfo,"ERROR","gcsConfigAnalogDelayedAlarm_checkConfig: " + getCatStr("gcsGeneration","BADLENGTH"),"");
  }
}


//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_AnalogDelayedAlarm_checkConfig
/**
Check data before configuration for _UnPlc front-end type

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
_UnPlc_AnalogDelayedAlarm_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  gcsConfigAnalogDelayedAlarm_checkCommonConfig(UN_PLC_DPTYPE, dsConfigs, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_AnalogDelayedAlarm_checkConfig
/**
Check data before configuration for S7_PLC front-end type

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
S7_PLC_AnalogDelayedAlarm_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
    gcsConfigAnalogDelayedAlarm_checkCommonConfig(S7_PLC_DPTYPE, dsConfigs, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// gcsConfigAnalogDelayedAlarm_checkCommonConfig
/**
Check configuration of given AnalogDelayedAlarm datapoint for both unPLc and S7_PLC front-end type

  @param[in]  plcType     PLC type. must be UN_PLC_DPTYPE or S7_PLC_DPTYPE for the moment
  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
gcsConfigAnalogDelayedAlarm_checkCommonConfig(string plcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string str;
  bool bValue;

  if(!gcsConfigGenericFunctions_checkPlcType(plcType))
  {
    fwException_raise(exceptionInfo,"ERROR","gcsConfigAnalogDelayedAlarm_checkCommonConfig: " + getCatStr("gcsGeneration","PLCUNKNOWN") + plcType,"");
    return;
  }
  bool bHasArchive = false;
  if(!gcsConfigGenericFunctions_checkConfigLength(GCS_CONFIG_ANALOGDELAYEDALARM_LENGTH, dsConfigs, bHasArchive))
  {
    fwException_raise(exceptionInfo,"ERROR","with PLC=" + plcType + ". gcsConfigAnalogDelayedAlarm_checkCommonConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
    return;
  }
  if (bHasArchive){
    gcsConfigGenericFunctions_checkArchiveConfig(GCS_CONFIG_ANALOGDELAYEDALARM_LENGTH, dsConfigs, exceptionInfo);
    if (dynlen(exceptionInfo) != 0){
      fwException_raise(exceptionInfo,"ERROR","with PLC=" + plcType + ". gcsConfigAnalogDelayedAlarm_checkCommonConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
      return;
    }
  }
  gcsConfigGenericFunctions_checkBool(dsConfigs[UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_ANALOGDELAYEDALARM_ACKNOWLEDGE_TYPE], bValue, exceptionInfo);
  if (dynlen(exceptionInfo) != 0){
    return;
  }

  // 2. Float block
  if(plcType == UN_PLC_DPTYPE)
    str = " unConfigGenericFunctions_checkAllRegisters(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
  if(plcType == S7_PLC_DPTYPE)
    str = " unConfigGenericFunctions_checkAllRegistersS7(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
  str += "    makeDynInt(GCS_CONFIG_ANALOGDELAYEDALARM_ADDRESS_STSREG01), \n";
  str += "    makeDynInt(GCS_CONFIG_ANALOGDELAYEDALARM_ADDRESS_EVSTSREG01), \n";
  str += "    makeDynInt(GCS_CONFIG_ANALOGDELAYEDALARM_ADDRESS_MANREG01), \n";
  str += "    GCS_CONFIG_ANALOGDELAYEDALARM_NORMAL_POSITION, \n";
  str += "    exceptionInfo); \n";
  evalScript(exceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string exceptionInfo; " + str + " return exceptionInfo;}",
    makeDynString(), dsConfigs);
}


//@}
