/**@name LIBRARY: gcsConfigWord2Word2AnalogStatus.ctl

@author: Geraldine Thomas (EN-ICE/SIC)
Modifications History:
  $Log: gcsConfigWord2AnalogStatus.ctl,v $
  Revision 1.2  2010/09/27 11:25:05  amerezhi
  The checking for SCALE is changed from RANGE to 2 checkFloat

  Revision 1.1  2009/01/15 15:13:49  gthomas
  Added new functions to configure new type: Word2AnalogStatus

  
*/
/*
Internal Functions :
  gcsConfigWord2AnalogStatus_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  _UnPlc_Word2AnalogStatus_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  S7_PLC_Word2AnalogStatus_setConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  gcsConfigWord2AnalogStatus_setCommonConfig (string sPlcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
  gcsConfigWord2AnalogStatus_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  _UnPlc_Word2AnalogStatus_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  S7_PLC_Word2AnalogStatus_checkConfig (dyn_string dsConfigs, dyn_string &exceptionInfo)
  gcsConfigWord2AnalogStatus_checkCommonConfig (string sPlcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
*/


// Constants
const string GCS_CONFIG_WORD2ANALOGSTATUS_DPT_NAME = "Word2AnalogStatus";

//Constants below indicate position number in the configuration line.
//Specified positions are relative, first 10 positions are reserved for standard UNICOS fields.
const unsigned GCS_CONFIG_WORD2ANALOGSTATUS_LENGTH       = 11;

const unsigned GCS_CONFIG_WORD2ANALOGSTATUS_UNIT         = 1;
const unsigned GCS_CONFIG_WORD2ANALOGSTATUS_FORMAT       = 2;
const unsigned GCS_CONFIG_WORD2ANALOGSTATUS_DEADBAND    = 3;
const unsigned GCS_CONFIG_WORD2ANALOGSTATUS_DEADBAND_TYPE = 4;
const unsigned GCS_CONFIG_WORD2ANALOGSTATUS_SCALEMAX       = 5;
const unsigned GCS_CONFIG_WORD2ANALOGSTATUS_SCALEMIN       = 6;
const unsigned GCS_CONFIG_WORD2ANALOGSTATUS_RANGEMAX       = 7;
const unsigned GCS_CONFIG_WORD2ANALOGSTATUS_RANGEMIN       = 8;
const unsigned GCS_CONFIG_WORD2ANALOGSTATUS_ARCHIVE_ACTIVE = 9;
const unsigned GCS_CONFIG_WORD2ANALOGSTATUS_ARCHIVE_TIME_FILTER = 10;
const unsigned GCS_CONFIG_WORD2ANALOGSTATUS_ADDRESS_AUPOSRST = 11;

const unsigned UN_CONFIG_WORD2ANALOGSTATUS_LENGTH       = 14;
const unsigned UN_CONFIG_WORD2ANALOGSTATUS_ADDITIONAL_LENGTH       = 0;
const unsigned UN_CONFIG_WORD2ANALOGSTATUS_ARCHIVE_ACTIVE = 9;

//------------------------------------------------------------------------------------------------------------------------
// gcsConfigWord2AnalogStatus_setConfig
/**
Creates dp (or checks if it is already presented)and sets configuration for Word2AnalogStatus object.

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
    dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
    are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo All possible errors will be returned here
*/
gcsConfigWord2AnalogStatus_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
//DebugN("gcsConfigWord2AnalogStatus_setConfig starts");//set by injector
  string sDpName;

  unConfigGenericFunctions_checkAll(dsConfigs, GCS_CONFIG_WORD2ANALOGSTATUS_DPT_NAME, exceptionInfo);
//DebugN("exceptionInfo: "+exceptionInfo);//set by injector
  if (dynlen(exceptionInfo) <= 0)   // Good inputs
  {
    sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
    unConfigGenericFunctions_createDp(sDpName, GCS_CONFIG_WORD2ANALOGSTATUS_DPT_NAME, exceptionInfo);
//DebugN("exceptionInfo: "+exceptionInfo);//set by injector

    if (dynlen(exceptionInfo) <= 0)
    {
      unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
//DebugN("exceptionInfo: "+exceptionInfo);//set by injector
      if (dynlen(exceptionInfo) <= 0)
      {
      	unConfigGenericFunctions_setDeviceConfig(dsConfigs, GCS_CONFIG_WORD2ANALOGSTATUS_DPT_NAME, exceptionInfo);
//DebugN("exceptionInfo: "+exceptionInfo);//set by injector
      }
    }
  }
//DebugN("gcsConfigWord2AnalogStatus_setConfig ends");//set by injector
}


//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_Word2AnalogStatus_setConfig
/**
Set configuration of given Word2AnalogStatus datapoint for _UnPlc front-end type

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
    dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
    are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo All possible errors will be returned here
*/
_UnPlc_Word2AnalogStatus_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
//DebugN("_UnPlc_Word2AnalogStatus_setConfig starts");//set by injector
  //DebugN("start _UnPlc_Word2AnalogStatus_setConfig");
  gcsConfigWord2AnalogStatus_setCommonConfig(UN_PLC_DPTYPE, dsConfigs, exceptionInfo);
//DebugN("exceptionInfo: "+exceptionInfo);//set by injector
  //DebugN("End _UnPlc_Word2AnalogStatus_setConfig");
//DebugN("_UnPlc_Word2AnalogStatus_setConfig ends");//set by injector
}


//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_Word2AnalogStatus_setConfig
/**
Set configuration of given Word2AnalogStatus datapoint for S7_PLC front-end type

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
    dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
    are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
  @param[out] exceptionInfo All possible errors will be returned here
*/
S7_PLC_Word2AnalogStatus_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
//DebugN("S7_PLC_Word2AnalogStatus_setConfig starts");//set by injector
  gcsConfigWord2AnalogStatus_setCommonConfig(S7_PLC_DPTYPE, dsConfigs, exceptionInfo);
//DebugN("exceptionInfo: "+exceptionInfo);//set by injector
//DebugN("S7_PLC_Word2AnalogStatus_setConfig ends");//set by injector
}


//------------------------------------------------------------------------------------------------------------------------
// gcsConfigWord2AnalogStatus_setCommonConfig
/**
Set configuration of given Word2AnalogStatus datapoint for both unPLc and S7_PLC front-end type

  @param[in]  sPlcType     PLC type. must be UN_PLC_DPTYPE or S7_PLC_DPTYPE for the moment
  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
gcsConfigWord2AnalogStatus_setCommonConfig(string sPlcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
{
//DebugN("gcsConfigWord2AnalogStatus_setCommonConfig starts");//set by injector
 
 string sDpName, str, dataType;
 int iScaleMax, iScaleMin, iRes;
 float fConv;
 
  if(!gcsConfigGenericFunctions_checkPlcType(sPlcType))
  {
    fwException_raise(exceptionInfo,"ERROR","gcsConfigWord2AnalogStatus_setCommonConfig: " + getCatStr("gcsGeneration","PLCUNKNOWN") + sPlcType,"");
//DebugN("exceptionInfo: "+exceptionInfo);//set by injector
    return;
  }

  //this check is required just to avoid any problems with PVSS software as itself
  if (dynlen(dsConfigs) >= UN_CONFIG_OBJECTGENERAL_DPNAME) sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];

  //if archive name are in the config->overwrite setting from import panel.
  gcsConfigGenericFunctions_setArchives(GCS_CONFIG_WORD2ANALOGSTATUS_LENGTH, dsConfigs);

  // 2.2. Registers
  //if(sPlcType == UN_PLC_DPTYPE) str = " unConfigGenericFunctions_setAllRegisters(dsConfigs, \n";
  //if(sPlcType == S7_PLC_DPTYPE) str = " unConfigGenericFunctions_setAllRegistersS7(dsConfigs, \n";
  //str += "    makeDynString(), makeDynInt(), makeDynInt(), makeDynString(), \n";
  //str += "    makeDynInt(), makeDynString(), makeDynString(), \n";
  //str += "    makeDynInt(), \n";  //  GCS_CONFIG_WORD2ANALOGSTATUS_NORMAL_POSITION,
  //str += "    exceptionInfo); \n";
  //evalScript(exceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string exceptionInfo; " + str + " return exceptionInfo;}", makeDynString(), dsConfigs);


// 2.3. Float values
  dataType = "_INT16"; //data type should be the same as what comes from PLC (in this case we have a WS(16 bit value) in PLC)
  
  if(sPlcType == UN_PLC_DPTYPE) str = " unConfigGenericFunctions_setFloatIntBlock(dsConfigs,  PVSS_MODBUS" + dataType + ", \n";
  else if(sPlcType == S7_PLC_DPTYPE) str = " unConfigGenericFunctions_setFloatIntBlockS7(dsConfigs,  PVSS_S7" + dataType + ", \n";
  else
  {
  	dynAppend(exceptionInfo, "gcsConfigWord2AnalogStatus_setCommonConfig: plc type is unknown - " + sPlcType);
//DebugN("exceptionInfo: "+exceptionInfo);//set by injector
  	return;
  }

  str += "    makeDynString(\"AuPosRSt\"), \n";
  str += "    makeDynInt(GCS_CONFIG_WORD2ANALOGSTATUS_ADDRESS_AUPOSRST), \n";
  str += "    makeDynString(), makeDynInt(), \n";
  str += "    makeDynInt(GCS_CONFIG_WORD2ANALOGSTATUS_UNIT, GCS_CONFIG_WORD2ANALOGSTATUS_FORMAT, GCS_CONFIG_WORD2ANALOGSTATUS_DEADBAND, GCS_CONFIG_WORD2ANALOGSTATUS_DEADBAND_TYPE), \n";
  str += "    makeDynInt(GCS_CONFIG_WORD2ANALOGSTATUS_RANGEMIN, GCS_CONFIG_WORD2ANALOGSTATUS_RANGEMAX), \n";
  str += "    makeDynString(\"AuPosRSt\"), \n";
  str += "    makeDynInt(GCS_CONFIG_WORD2ANALOGSTATUS_ARCHIVE_ACTIVE, GCS_CONFIG_WORD2ANALOGSTATUS_ARCHIVE_TIME_FILTER), \n";
  str += "    exceptionInfo); \n";

  evalScript(exceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string exceptionInfo; " + str + " return exceptionInfo;}",
    makeDynString(), dsConfigs);
//DebugN("gcsConfigWord2AnalogStatus_setCommonConfig ends");//set by injector

//set Conversion 
  if (dynlen(exceptionInfo) <= 0)
	{
		iScaleMax = dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_WORD2ANALOGSTATUS_SCALEMAX];
		//DebugN("iScaleMax: " +iScaleMax);
		iScaleMin = dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_WORD2ANALOGSTATUS_SCALEMIN];
		//DebugN("iScaleMin: "+iScaleMin);
		
		if (iScaleMin==0)
			fConv = iScaleMax;
		else
			fConv = ((float)iScaleMin)/iScaleMax;
		
		//DebugN("fConv: "+fConv);
		string dpe = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] + ".ProcessInput.AuPosRSt";
		iRes = dpSetWait(dpe + ":_msg_conv.._type", DPCONFIG_CONVERSION_RAW_TO_ENG_MAIN,
		  		 dpe + ":_msg_conv.1._type", DPDETAIL_CONV_POLY,
		      		 dpe + ":_msg_conv.1._poly_grade", 1,
		      		 dpe + ":_msg_conv.1._poly_a", 0,
		  	         dpe + ":_msg_conv.1._poly_b", fConv);
		if(iRes != 0)
		  {
		     fwException_raise(exceptionInfo,"ERROR","gcsConfigWord2AnalogStatus_setConfig: error in convertion" + getCatStr("unGeneration","BADINPUTS"),"");
		      return;
    		   }
			
	}

}

//------------------------------------------------------------------------------------------------------------------------
// gcsConfigWord2AnalogStatus_checkConfig
/**
Check data before configuration

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
gcsConfigWord2AnalogStatus_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  if(dynlen(dsConfigs) >= UN_CONFIG_COMMON_LENGTH)
  {
  // 1. Common
    unConfigGenericFunctions_checkParameters(dsConfigs, GCS_CONFIG_WORD2ANALOGSTATUS_DPT_NAME, exceptionInfo);
  // 2. call the device check function
    unConfigGenericFunctions_checkDeviceConfig(dsConfigs, GCS_CONFIG_WORD2ANALOGSTATUS_DPT_NAME, exceptionInfo);
  }
  else
  {
    fwException_raise(exceptionInfo,"ERROR","gcsConfigWord2AnalogStatus_checkConfig: " + getCatStr("gcsGeneration","BADLENGTH"),"");
  }
}


//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_Word2AnalogStatus_checkConfig
/**
Check data before configuration for _UnPlc front-end type

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
_UnPlc_Word2AnalogStatus_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
//DebugN("_UnPlc_Word2AnalogStatus_checkConfig starts");//set by injector
  gcsConfigWord2AnalogStatus_checkCommonConfig(UN_PLC_DPTYPE, dsConfigs, exceptionInfo);
//DebugN("exceptionInfo: "+exceptionInfo);//set by injector
//DebugN("_UnPlc_Word2AnalogStatus_checkConfig ends");//set by injector
}


//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_Word2AnalogStatus_checkConfig
/**
Check data before configuration for S7_PLC front-end type

  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
S7_PLC_Word2AnalogStatus_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
//DebugN("S7_PLC_Word2AnalogStatus_checkConfig starts");//set by injector
    gcsConfigWord2AnalogStatus_checkCommonConfig(S7_PLC_DPTYPE, dsConfigs, exceptionInfo);
//DebugN("exceptionInfo: "+exceptionInfo);//set by injector
//DebugN("S7_PLC_Word2AnalogStatus_checkConfig ends");//set by injector
}


//------------------------------------------------------------------------------------------------------------------------
// gcsConfigWord2AnalogStatus_checkCommonConfig
/**
Check configuration of given Word2AnalogStatus datapoint for both unPLc and S7_PLC front-end type

  @param[in]  sPlcType     PLC type. must be UN_PLC_DPTYPE or S7_PLC_DPTYPE for the moment
  @param[in]  dsConfigs   Configs (see dyn_string indexes in constants defined)
  @param[out] exceptionInfo All possible errors will be returned here
*/
gcsConfigWord2AnalogStatus_checkCommonConfig(string sPlcType, dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string str;

  if(!gcsConfigGenericFunctions_checkPlcType(sPlcType))
  {
    fwException_raise(exceptionInfo,"ERROR","gcsConfigWord2AnalogStatus_checkCommonConfig: " + getCatStr("gcsGeneration","PLCUNKNOWN") + sPlcType,"");
    return;
  }
 
  bool bHasArchive = false;
  if(!gcsConfigGenericFunctions_checkConfigLength(GCS_CONFIG_WORD2ANALOGSTATUS_LENGTH, dsConfigs, bHasArchive))
  {
    fwException_raise(exceptionInfo,"ERROR","with PLC=" + sPlcType + ". gcsConfigWord2AnalogStatus_checkCommonConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
    return;
  }
  if (bHasArchive){
    gcsConfigGenericFunctions_checkArchiveConfig(GCS_CONFIG_WORD2ANALOGSTATUS_LENGTH, dsConfigs, exceptionInfo);
    if (dynlen(exceptionInfo) != 0){
      fwException_raise(exceptionInfo,"ERROR","with PLC=" + sPlcType + ". gcsConfigWord2AnalogStatus_checkCommonConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
      return;
    }
  }

  // 2. Float block
  if(sPlcType == UN_PLC_DPTYPE)
	str = " unConfigGenericFunctions_checkFloatIntBlock(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
  if(sPlcType == S7_PLC_DPTYPE)
	str = " unConfigGenericFunctions_checkFloatIntBlockS7(dsConfigs, UN_CONFIG_COMMON_LENGTH, \n";
  str += "    makeDynInt(GCS_CONFIG_WORD2ANALOGSTATUS_ADDRESS_AUPOSRST), \n";
  str += "    makeDynInt(),  \n";
  str += "    makeDynInt(GCS_CONFIG_WORD2ANALOGSTATUS_UNIT, GCS_CONFIG_WORD2ANALOGSTATUS_FORMAT, GCS_CONFIG_WORD2ANALOGSTATUS_DEADBAND, GCS_CONFIG_WORD2ANALOGSTATUS_DEADBAND_TYPE), \n";
  str += "    makeDynInt(GCS_CONFIG_WORD2ANALOGSTATUS_RANGEMIN, GCS_CONFIG_WORD2ANALOGSTATUS_RANGEMAX), \n";
  str += "    makeDynInt(GCS_CONFIG_WORD2ANALOGSTATUS_ARCHIVE_ACTIVE, GCS_CONFIG_WORD2ANALOGSTATUS_ARCHIVE_TIME_FILTER), \n";
  str += "    exceptionInfo); \n";
  evalScript(exceptionInfo, "dyn_string main(dyn_string dsConfigs) {\n dyn_string exceptionInfo; " + str + " return exceptionInfo;}",
    makeDynString(), dsConfigs);
//DebugN("gcsConfigWord2AnalogStatus_checkCommonConfig ends");//set by injector

//4.check additional parameters (Scale Max and Scale Min)
  if (dynlen(exceptionInfo) <= 0)
  {
    float sMin, sMax;
	unConfigGenericFunctions_checkFloat(dsConfigs[UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_WORD2ANALOGSTATUS_SCALEMIN], sMin, exceptionInfo);
    unConfigGenericFunctions_checkFloat(dsConfigs[UN_CONFIG_COMMON_LENGTH + GCS_CONFIG_WORD2ANALOGSTATUS_SCALEMAX], sMax, exceptionInfo);
  }
}

//@}
