/**@name LIBRARY: gcsDigitalDelayedAlarm.ctl

@author: Francisco Mico (IT-CO)

Creation Date: 18 06 2003

Modifications History:
	$Log: gcsDigitalDelayedAlarm.ctl,v $
	Revision 1.17  2009/03/30 12:14:48  gamx
	fixed bug in animation: priority of "N" is higher than "O"
	
	Revision 1.15  2009/01/28 14:06:39  gamx
	Fixes in _WidgetDisconnection function:
	A standard "unGenericObject_WidgetDisconnections" function cannot be used for ADA or DDA objects (due to additional graphical objects on the widget panel).
	A custom disconnection function has been introduced.

	Revision 1.14  2008/11/12 08:25:13  gthomas
	-removed function calls to _ButtonUserAccessGcs and _ButtonUserAccessUn

	Revision 1.13  2008/10/20 13:55:47  gthomas
	-removed unused XX_EventList_Configure_YY functions

	Revision 1.12  2008/07/18 07:16:55  gthomas
	added new Access control functionality for unicos-4-1
	modified  xx_ButtonUserAccess function.

	Revision 1.11  2008/06/18 14:54:51  aabadz
	Changes done following the recomendations from AB-CO (from Herve)

	Revision 1.10  2008/01/22 14:46:36  aabadz
	Fixed:
	- Naming
	- Call to other functions to reuse code
	- Some error check

	Revision 1.9  2008/01/14 10:12:21  aabadz
	Changes after code review and before tests

	Revision 1.8  2007/10/30 16:21:41  gthomas
	Introduced the CVS keyword $Log to track changes made in the library

	25.09.2006 Kulman Nikolay
		- Porting to Unicos 3.6
	25.11.2004 Kulman Nikolay
		- function gcsAnalogAlarm_HandleMenu modified to deselect widget automaticly
		- function gcsAnalogAlarm_WidgetAnimationCB modified
	03.11.2004 Kulman Nikolay
		- function gcsDigitalDelayedAlarm_HandleMenu modified to handle Mask/UnMask menu commands
	11/05/2004: Herve AB/CO
		- function gcsDigitalDelayedAlarm_MenuConfiguration modified: signature modified add the creation of the menu
		- function gcsDigitalDelayedAlarm_HandleMenu added: this function handles the rigth click menu
		- function gcsDigitalDelayedAlarm_ObjectListGetValueTime added: called in object list.
		- function gcsDigitalDelayedAlarm_ButtonSetState modified: modification of the signature of the function
		- function gcsDigitalDelayedAlarm_ButtonAnimationCB modified: signature of the function gcsDigitalDelayedAlarm_ButtonSetState was modified
		- function gcsDigitalDelayedAlarm_UserLoginGetButtonState added: called when user logs in, returns the state of the buttons
		- function gcsDigitalDelayedAlarm_ButtonRegister modified : add the set Manreg01 to 0 in case of dpConnect
		- function gcsDigitalDelayedAlarm_FaceplateConnect, gcsDigitalDelayedAlarm_FaceplateDisconnect and gcsDigitalDelayedAlarm_WidgetRegisterCB
		bug in case the PLC is not defined and if the PVSS system name contains '-',
		call unGenericObject_GetPLCNameFromDpName to get the PLC DPName instead (this function corrects the bugs).

version 1.0

External Function :
	. gcsDigitalDelayedAlarm_AcknowledgeAlarm : animate Digital Alarm alarms
	. gcsDigitalDelayedAlarm_FaceplateStatusDisconnection : animate faceplate disconnection
	. gcsDigitalDelayedAlarm_ButtonUserAccess : User access for DigitalDelayedAlarm buttons
	. gcsDigitalDelayedAlarm_ButtonSetState : set button state
	. gcsDigitalDelayedAlarm_WidgetDisconnection : animate widget disconnection
	. gcsDigitalDelayedAlarm_MenuConfiguration : configuration of popup menu

Internal Functions :
	. gcsDigitalDelayedAlarm_FaceplateStatusRegisterCB : register callback function
	. gcsDigitalDelayedAlarm_FaceplateConnect : dpConnect to the DataPoint
	. gcsDigitalDelayedAlarm_FaceplateDisconnect : dpDisconnect to the DataPoint
	. gcsDigitalDelayedAlarm_FaceplateStatusAnimationCB_NoRangeNoAlarm : animate faceplate
	. gcsDigitalDelayedAlarm_FaceplateStatusAnimationCB_NoAlarm : animate faceplate
	. gcsDigitalDelayedAlarm_FaceplateStatusAnimationCB_NoRange : animate faceplate
	. gcsDigitalDelayedAlarm_FaceplateStatusAnimationCB : animate faceplate
	. gcsDigitalDelayedAlarm_ButtonRegisterCB : register callback function
	. gcsDigitalDelayedAlarm_ButtonAnimationCB : animate buttons
	. gcsDigitalDelayedAlarm_ButtonDisconnection : animate buttons disconnection
	. gcsDigitalDelayedAlarm_WidgetRegisterCB : register callback function
	. gcsDigitalDelayedAlarm_WidgetConnect : dpConnect to the DataPoint
	. gcsDigitalDelayedAlarm_WidgetDisconnect : dpDisconnect to the DataPoint
	. gcsDigitalDelayedAlarm_WidgetAnimationCB : animate widget

Purpose: This library contains DigitalDelayedAlarm object functions.

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. DigitalDelayedAlarm object (DPType etc.)
	. Global variables and $parameters defined in DigitalDelayedAlarm faceplate and widget
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

//@{

//---------------------------------------------------------------------------------------------------------------------------------------

// gcsDigitalDelayedAlarm_ObjectListGetValueTime
/**
Purpose: Function called from the objectList to return the time and value

Parameters:
	- sDeviceName, string, input, device name
	- sDeviceType, string, input, unicos object
	- dsReturnData, dyn_string, output, return value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
gcsDigitalDelayedAlarm_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
	float value;
	string sTime, sStatus,sColor;
	bool bBad;

	dpGet(sDeviceName+".ProcessInput.PosAl", value,
	      sDeviceName+".ProcessInput.PosAl:_online.._stime", sTime,
	      sDeviceName+".ProcessInput.PosAl:_online.._bad", bBad);

	dsReturnData[1] = sTime;
	dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(sDeviceName+".ProcessInput.PosAl"), value) + " "
							+ dpGetUnit(sDeviceName+".ProcessInput.PosAl");

	// Optionally (when no stsReg* exists)
	// Status
	dsReturnData[3] = bBad;

	// Status to visualize
	sStatus="OK";
	dsReturnData[4] = sStatus;
	// Colors
	sColor="unEventList_ForeColor";
	dsReturnData[5]=sColor;
}

//---------------------------------------------------------------------------------------------------------------------------------------
// gcsDigitalDelayedAlarm_AcknowledgeAlarm
/**
Purpose: acknowledge Digital Alarm alarms

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsNeedAck, dyn_string, output, DPEs names that need acknowledgement.
									 The PLC acknowledgement is specified using UN_ACKNOWLEDGE_PLC string

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Local faceplate
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
gcsDigitalDelayedAlarm_AcknowledgeAlarm(string sDpName, string sDpType, dyn_string &dsNeedAck)
{
	dsNeedAck = makeDynString("ProcessInput.PosAl", UN_ACKNOWLEDGE_PLC);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsDigitalDelayedAlarm_MenuConfiguration
/**
Purpose: configuration of popup menu

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsMenuConfig, dyn_string, output, configuration of menu. Use defined constants UN_POPUPMENU_***

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
gcsDigitalDelayedAlarm_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{
	dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_ACK_TEXT, UN_MENU_DA_MASK, UN_POPUPMENU_DIAG_INFO_TEXT,
		UN_POPUPMENU_TREND_TEXT, UN_MENU_MAIL_SMS/*, UN_POPUPMENU_FACEPLATE_TEXT*/);

	gcsGenericObject_addMaskToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	unGenericObject_addUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	unGenericObject_addDiagnosticAndInfoToActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	unGenericObject_addTrendActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsDigitalDelayedAlarm_HandleMenu
/**
Purpose: handle the answer of the popup menu

Parameters:
	- deviceName, string, input, the device dp name
	- sDpType, string, input, the device dp type
	- menuList, dyn_string, input, list of requested action
	- menuAnswer, int, input, selected action from the menuList

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
gcsDigitalDelayedAlarm_HandleMenu(string deviceName, string sDpType, dyn_string menuList, int menuAnswer)
{
	dyn_string exceptionInfo;
	gcsGenericObject_HandleEventMaskMenu(deviceName, menuAnswer, makeDynString(".ProcessInput.evStsReg01", ".ProcessInput.PosAl"), exceptionInfo);
	unGenericObject_HandleUnicosMenu(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);
	gcsGenericObject_HandleAlarmMaskMenu(deviceName, menuList, menuAnswer, exceptionInfo);
	if (dynlen(exceptionInfo) > 0)
		unSendMessage_toExpertException("Right Click",exceptionInfo);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsDigitalDelayedAlarm_ButtonUserAccess
/**
Purpose: User access for DigitalDelayedAlarm buttons

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, dptype
	- dsAccess, dyn_string, output, list of allowed buttons

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
gcsDigitalDelayedAlarm_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{	
	bool operator, expert, admin;
	dyn_bool dbPermissions;
	dyn_string exceptionInfo;
	string sOperAction, sExpAction, sAdminAction;
	bool bActionDefined = false;
	
	unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo);
	if((sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME) ||
	(sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME))
	bActionDefined=true;
	
	operator = dbPermissions[2];
	expert = dbPermissions[3];
	admin = dbPermissions[4];
	dsAccess = makeDynString();
	
	if (operator){
		if(bActionDefined)
			dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
		else
			dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_ACK_ALARM));
	}
	// to check why Block/unblock action is missing !
	if (expert){
		if(bActionDefined)
			dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
		else
			dynAppend(dsAccess, makeDynString(UN_MENU_MAIL_SMS, UN_FACEPLATE_MASKEVENT));
	}
	if(admin){
		if(bActionDefined)
			dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
		else
        		dynAppend(dsAccess, makeDynString(UN_MENU_ALARM_SET_MASK, UN_MENU_ALARM_RESET_MASK));
	}

}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsDigitalDelayedAlarm_WidgetRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in DigitalDelayedAlarm widget
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
gcsDigitalDelayedAlarm_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, sPlcName, sPlcDpName, sSystemName;
	int iAction;
	dyn_string exceptionInfo;
	bool bRemote;

	deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
	if (deviceName == "")	// In case of disconnection
	{
		deviceName = g_sDpName;
	}

	sSystemName = unGenericDpFunctions_getSystemName(deviceName);

	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;

	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	gcsConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)
		g_bUnSystemAlarmPlc = true;
	else
		g_bUnSystemAlarmPlc = false;

	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			gcsDigitalDelayedAlarm_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPCONNECT:
			g_sDpName = deviceName;
			if(dpTypeName(deviceName) == "DigitalDelayedAlarm")
				gcsDigitalDelayedAlarm_WidgetConnect(deviceName, sPlcName, sSystemName);
			else{
				g_bUnSystemAlarmPlc = false;
				gcsDigitalDelayedAlarm_WidgetDisconnection(g_sWidgetType);
			}
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			gcsDigitalDelayedAlarm_WidgetDisconnect (deviceName, sPlcName, sSystemName);
			gcsDigitalDelayedAlarm_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}


//--------------------------------------------------------------------------------------------------------------------------------------
// gcsDigitalDelayedAlarm_WidgetConnect
/**
Purpose: do dpConnect

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in DigitalDelayedAlarm faceplate
	. PVSS version: 2.12.1
	. operating system: XP and W2000, but tested only under XP
	. distributed system: yes.
*/
gcsDigitalDelayedAlarm_WidgetConnect(string deviceName, string sPlcName, string sSystemName)
{
	int iRes;

	if (g_bUnSystemAlarmPlc)
	{
		iRes = dpConnect("gcsDigitalDelayedAlarm_WidgetAnimationCB",
			deviceName + ".statusInformation.selectedManager:_lock._original._locked",
			deviceName + ".statusInformation.selectedManager",
			deviceName + ".ProcessInput.StsReg01",
			deviceName + ".ProcessInput.StsReg01:_online.._invalid",
			sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
			sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
			deviceName + ".eventMask");
		g_bCallbackConnected = (iRes >= 0);
	}
	else
		gcsDigitalDelayedAlarm_WidgetDisconnection (g_sWidgetType);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsDigitalDelayedAlarm_WidgetDisconnect
/**
Purpose: do dpDisconnect

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in DigitalDelayedAlarm faceplate
	. PVSS version: 2.12.1
	. operating system: XP and W2000, but tested only under XP.
	. distributed system: yes.
*/
gcsDigitalDelayedAlarm_WidgetDisconnect(string deviceName, string sPlcName, string sSystemName)
{
	int iRes;

	if (g_bUnSystemAlarmPlc)
	{
		iRes = dpDisconnect("gcsDigitalDelayedAlarm_WidgetAnimationCB",
			deviceName + ".statusInformation.selectedManager:_lock._original._locked",
			deviceName + ".statusInformation.selectedManager",
			deviceName + ".ProcessInput.StsReg01",
			deviceName + ".ProcessInput.StsReg01:_online.._invalid",
			sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
			sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
			deviceName + ".eventMask");
		g_bCallbackConnected = !(iRes >= 0);
	}
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsDigitalDelayedAlarm_WidgetAnimationCB
/**
Purpose: Animate widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in DigitalDelayedAlarm faceplate
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
gcsDigitalDelayedAlarm_WidgetAnimationCB(string sDpLock, bool bLocked, string sDpManager, string SelectedManager,
	string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid,
	string sDpAl, int iSystemIntegrityAlarmValue, string sDpEnabled, bool bSystemIntegrityAlarmEnabled,
	string sEvtMask, int iMask)
{
	string sBody1Color, sBody1BColor = "", sColorSelect = "", sWarningColor = "",sWarningLetter = "";
	string sAlarmTypeLetter = "", sAlarmTypeColor = "", sAlarmTextLetter = "", sAlarmTextColor = "";
	sBody1Color = "unDataNotValid";
	bool bPreMask = (getBit(bit32StsReg01, UN_STSREG01_PREMASKALARM) == 1)?true:false;
	bool bAlarmSt = (getBit(bit32StsReg01, UN_STSREG01_AST) == 1)?true:false;

	if (!bStsReg01Invalid)
	{
		if(bPreMask == true || bAlarmSt == true) sBody1Color = "gcsAlarm_Alarm";
		else sBody1Color = "gcsAlarm_OK";
		sBody1BColor = "";

// Alarm not acknowledged (higher priority)
		if (getBit(bit32StsReg01, UN_STSREG01_ALUNACK) == 1)
			sBody1Color = "gcsAlarm_AlarmNotAck";

		if(bPreMask == true || bAlarmSt == true) 						//	Alarm is active - show letter "A"
		{	sAlarmTypeLetter = UN_WIDGET_ALARM;
			sAlarmTypeColor  = "gcsAlarm_Alarm";
			sBody1BColor = sBody1Color;
		}
		if (getBit(bit32StsReg01, UN_STSREG01_MAMSKST) == 1)			// Alarm masked
		{
			sAlarmTextLetter = UN_WIDGET_TEXT_ALARM_BLOCKED;
			sAlarmTextColor = "unAlarmMasked";
			if(bPreMask == true || bAlarmSt == true)						//	Alarm is active - show letter "A"
			{	sAlarmTypeLetter = UN_WIDGET_ALARM;
				sAlarmTypeColor  = "gcsAlarm_Alarm";
			}
			sBody1Color = "gcsAlarm_OK";
			sBody1BColor = "";
		}

		// Warning animation
		unGenericObject_WidgetWarningAnimation(bit32StsReg01,
		   makeDynInt(/*UN_STSREG01_AUMRW,*/ UN_STSREG01_IOSIMUW, UN_STSREG01_IOERRORW),
		   makeDynString(/*UN_WIDGET_TEXT_WARNING_DEFAULT,*/ UN_WIDGET_TEXT_WARNING_SIMU, UN_WIDGET_TEXT_WARNING_ERROR),
		   sWarningLetter, sWarningColor);
	}

	//data quality indicators
	if((!bSystemIntegrityAlarmEnabled)||(iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value))
	{
		sWarningLetter = UN_WIDGET_TEXT_OLD_DATA;
		sWarningColor = "unDataNotValid";
	}
	
	if (bStsReg01Invalid){
		sWarningLetter = UN_WIDGET_TEXT_INVALID;
		sWarningColor = "unDataNotValid";	
	}
	

	if(g_bSystemConnected)
	{
		setMultiValue("WarningText", "text", sWarningLetter, "WarningText", "foreCol", sWarningColor,
			"AlarmType", "text", sAlarmTypeLetter, "AlarmType", "foreCol", sAlarmTypeColor,	//	"A"
			"AlarmText", "text", sAlarmTextLetter, "AlarmText", "foreCol", sAlarmTextColor,	//	"B"
			"ControlStateText", "text", "", "SelectArea", "foreCol", "",
			"LockBmp", "visible", false, "Body1","fill","[solid]",
			"Body1", "foreCol", sBody1Color, "Body1", "backCol", sBody1BColor,
			"WidgetArea", "visible", true);
		if(iMask == 0)	eventState.visible	= true;
		else			eventState.visible	= false;
	}
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsDigitalDelayedAlarm_WidgetDisconnection
/**
Purpose: Animate widget disconnection

Parameters:
	- sWidgetType, string, input, widget type. ex: "heater"

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
gcsDigitalDelayedAlarm_WidgetDisconnection(string sWidgetType)
{
	setMultiValue("Body1", "foreCol", "unDataNoAccess", "Body1", "backCol", "unWidget_Background", "Body1", "fill", "[solid]",
	   "WarningText", "text", "", "WidgetArea", "visible", false, "AlarmType", "text", "", "AlarmText", "text", "");
	eventState.visible = false;
}




//@}
