/**@name LIBRARY: gcsExportDevice.ctl

@author: Nikolay KULMAN (IT/CO)

Creation Date: 06 04 2006

Modification History:
	$Log: gcsExportDevice.ctl,v $
	Revision 1.33  2010/09/01 11:10:32 amerezhi
	Exporting default value for WordParameter is fixed
	Exporting achive time filter is fixed
	Added *_MAIL alert classes to the gcsExportDevice_getAlarmIsAck 
	Added PV_RANGE to the WordComputed export results.
	Changed ARCHIVE_TIME_FILTER's exported value from UN_CONFIG_EXPORT_UNDEF to ""
	Fixed exporting archive time filter for PUMP and MFC

	Revision 1.32  2010/03/25 15:30:58  gthomas
	Fixed #53414 (bad dpe name reference, renamed ASt-->PosAl)
	
	Revision 1.31  2009/03/20 13:59:43  gthomas
	-updated variable names <_GCSDELAYEDALARM_CLASS> with <_ALERT_CLASS_>
	-update variable <GCS_CONFIG_EXPORT_UNDEF> with <UN_CONFIG_EXPORT_UNDEF>

	Revision 1.30  2009/03/17 10:11:45  gthomas
	-added MFC and Pump Export functions from 1.21 version

	Revision 1.29  2009/03/10 15:28:24  gthomas
	added UNICOS Changes made on 17/02/2009
	-Function _UnPlc_DigitalParameter_ExportConfig exports the default value in text mode (True instead of TRUE)

	Revision 1.28  2009/01/15 15:13:17  gthomas
	Added functions to export new type: Word2AnalogStatus

	Revision 1.27  2008/07/23 08:41:25  gthomas
	changed CVS log comment in previous version (1.26)

	Revision 1.26  2008/07/18 07:16:55  gthomas
	added functions for other gcs types

	Revision 1.25  2008/06/20 07:48:46  aabadz
	Some auxiliary functions were deleted because the were not going to be used in the new package to UNICOS.

	Revision 1.24  2008/06/20 06:36:46  gthomas
	introduced CVS key Log to track changes

	20/08/2007: enrique Blanco
	_UnPlc_AnalogStatus_ExportConfig has been modified. In case of no archive defined it shoud give "" instead of UN_CONFIG_EXPORT_UNDEF

version 1.0

External Function : 
	._UnPlc_Com_ExportConfig
	._UnPlc_ComExtended_ExportConfig

Internal Functions :
	_UnPlc_AnalogParameter_ExportConfig
	S7_PLC_AnalogParameter_ExportConfig
	_UnPlc_DigitalParameter_ExportConfig
	S7_PLC_DigitalParameter_ExportConfig
	_UnPlc_WordParameter_ExportConfig
	S7_PLC_WordParameter_ExportConfig
	_UnPlc_AnalogStatus_ExportConfig
	S7_PLC_AnalogStatus_ExportConfig
	_UnPlc_WordStatus_ExportConfig
	S7_PLC_WordStatus_ExportConfig
	_UnPlc_Word2AnalogStatus_ExportConfig
	S7_PLC_Word2AnalogStatus_ExportConfig
	_UnPlc_AnalogComputed_ExportConfig
	S7_PLC_AnalogComputed_ExportConfig
	_UnPlc_WordComputed_ExportConfig
	S7_PLC_WordComputed_ExportConfig
	_UnPlc_AnalogDelayedAlarm_ExportConfig
	S7_PLC_AnalogDelayedAlarm_ExportConfig
	_UnPlc_DigitalDelayedAlarm_ExportConfig
	S7_PLC_DigitalDelayedAlarm_ExportConfig
	_UnPlc_Pump_ExportConfig
	S7_PLC_Pump_ExportConfig
	_UnPlc_MassFlowController_ExportConfig
	S7_PLC_MassFlowController_ExportConfig	
	gcsExportDevice_getArchiveParameters
	gcsExportDevice_getAddressConfig
	gcsExportDevice_getArchiveNames
	gcsExportDevice_getArchiveName
	
*/



//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_AnalogParameter_ExportConfig
/**
Purpose: Export AnalogParameter Devices for _UnPlc front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void _UnPlc_AnalogParameter_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
dyn_string addressDpParameters, dsDpParameters, dsArchiveNames;
string sObject, sUnit, sFormatConverted, sArchiveActive, sMin, sMax, currentDP, element;
float fMinValue, fMaxValue, fArchiveTimeFilter, fDefault,  fDeadband, fDeadbandTemp, fTimeInterval;
int i, iLeni,  iSmoothProcedure, iDeadBandType;
bool bDoesExist, bNegateRange, bIgnoreOutside,  bInclusiveMin , bInclusiveMax, bIsSmoothDefined;

		sObject="AnalogParameter";
		g_iAlertRange = 0;
		
		iLeni=dynlen(dsDpList);
		for(i=1; i<=iLeni; i++)
		{									
			if(!dpExists(dsDpList[i])) continue;

			currentDP = dsDpList[i];
			element = currentDP + ".ProcessInput.CurValSt";

			dynClear(dsDpParameters);
			dynClear(dsArchiveNames);
					
//	Get all common Parameters
			unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);
				
			sUnit =  dpGetUnit(element);	//	Get DP unit
			sFormatConverted = unExportDevice_convertFormat(dpGetFormat(element), "#");	// Get Format
//	Get RangeMax and RangeMin
			fwPvRange_get(element, bDoesExist , fMinValue , fMaxValue , bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax , exceptionInfo);
			sMax = unExportDevice_floatFormat(fMaxValue);			
			sMin = unExportDevice_floatFormat(fMinValue);
//	Get smoothProcedure, fDeadband & timeInterval, Get unicos DeadBand
			unExportDevice_fwSmoothing_getTemp(element, bIsSmoothDefined, iSmoothProcedure, fDeadbandTemp, fTimeInterval);
			unExportDevice_getDeadBand(fDeadbandTemp, bIsSmoothDefined, sMax, sMin, iSmoothProcedure, fDeadband, iDeadBandType );	
//	Get Archive Parameters
			gcsExportDevice_getArchiveParameters(currentDP + ".ProcessInput.CurValSt", sObject, sArchiveActive, fArchiveTimeFilter);
			gcsExportDevice_getArchiveNames(currentDP,
				makeDynString(), makeDynString(".ProcessInput.CurValSt"), makeDynString(), dsArchiveNames);
//	Get address config		
			gcsExportDevice_getAddressConfig(sObject, currentDP,
					makeDynString("ValRqAck", "CurValSt", "ValRq"),
					makeDynBool(true, true, false), addressDpParameters);

//	Write to file
			dynAppend(dsDpParameters, sUnit);
			dynAppend(dsDpParameters, sFormatConverted);
			if (iDeadBandType==DPCONFIG_NONE)
				dynAppend(dsDpParameters, UN_CONFIG_EXPORT_UNDEF);
			else
				dynAppend(dsDpParameters, fDeadband);
			dynAppend(dsDpParameters, iDeadBandType);
			dynAppend(dsDpParameters, sMax);
			dynAppend(dsDpParameters, sMin);
			dynAppend(dsDpParameters, sArchiveActive);
			if (fArchiveTimeFilter < 0)
				dynAppend(dsDpParameters, "");
			else	
				dynAppend(dsDpParameters, fArchiveTimeFilter);
			dynAppend(dsDpParameters, addressDpParameters);
			dpGet(currentDP + ".ProcessOutput.DfltVal:_online.._value", fDefault);
//			dynAppend(dsDpParameters, dpGetDescription(currentDP + ".ProcessOutput.ValRq"));
			dynAppend(dsDpParameters, fDefault);

			for(int k = 1; k <= dynlen(dsArchiveNames); k++)
				dynAppend(dsDpParameters, dsArchiveNames[k]);

			unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
		}//end for
}

//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_AnalogParameter_ExportConfig
/**
Purpose: Export AnalogParameter Devices for S7_PLC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void S7_PLC_AnalogParameter_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	_UnPlc_AnalogParameter_ExportConfig(dsDpList, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_DigitalParameter_ExportConfig
/**
Purpose: Export DigitalParameter Devices for _UnPlc front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void _UnPlc_DigitalParameter_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
dyn_string addressDpParameters, dsDpParameters, dsArchiveNames;
string sObject, sArchiveActive, currentDP;
float fMinValue, fMaxValue, fArchiveTimeFilter ;
int i, iLeni;
bool bNegateRange, bIgnoreOutside,  bInclusiveMin , bInclusiveMax, bDefault;

		sObject="DigitalParameter";
		g_iAlertRange = 0;
		
		iLeni=dynlen(dsDpList);
		for(i=1; i<=iLeni; i++)
		{									
			if(!dpExists(dsDpList[i])) continue;

			currentDP = dsDpList[i];
			dynClear(dsDpParameters);
			dynClear(dsArchiveNames);
					
//	Get all common Parameters
			unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);
//	Get Archive Parameters
			gcsExportDevice_getArchiveParameters(currentDP + ".ProcessInput.CurValSt", sObject, sArchiveActive, fArchiveTimeFilter);
			gcsExportDevice_getArchiveNames(currentDP,
				makeDynString(".ProcessInput.CurValSt"), makeDynString(), makeDynString(), dsArchiveNames);
//	Get address config		
			gcsExportDevice_getAddressConfig(sObject, currentDP,
					makeDynString("ValRqAck", "CurValSt", "ValRq"),
					makeDynBool(true, true, false), addressDpParameters);

//	Write to file
			dynAppend(dsDpParameters, sArchiveActive);
			dynAppend(dsDpParameters, addressDpParameters);
			dpGet(currentDP + ".ProcessOutput.DfltVal:_online.._value", bDefault);
			dynAppend(dsDpParameters, bDefault);
			for(int k = 1; k <= dynlen(dsArchiveNames); k++)
				dynAppend(dsDpParameters, dsArchiveNames[k]);

			unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
		}//	end for
}

//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_DigitalParameter_ExportConfig
/**
Purpose: Export DigitalParameter Devices for S7_PLC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void S7_PLC_DigitalParameter_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	_UnPlc_DigitalParameter_ExportConfig(dsDpList, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_WordParameter_ExportConfig
/**
Purpose: Export WordParameter Devices for _UnPlc front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void _UnPlc_WordParameter_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
dyn_string addressDpParameters, dsDpParameters, dsArchiveNames;
string sObject, sUnit, sFormatConverted, sArchiveActive, sMin, sMax, currentDP, element, sDefaultValue;
float fMinValue, fMaxValue, fArchiveTimeFilter, fDeadbandTemp,fTimeInterval,fDeadband ;
int i, iLeni, iDefault,iSmoothProcedure,iDeadBandType ;
bool bDoesExist, bNegateRange, bIgnoreOutside,  bInclusiveMin , bInclusiveMax, bIsSmoothDefined, bDefault;

		sObject="WordParameter";
		g_iAlertRange = 0;
		
		iLeni=dynlen(dsDpList);
		for(i=1; i<=iLeni; i++)
		{									
			if(!dpExists(dsDpList[i])) continue;

			currentDP = dsDpList[i];
			element = currentDP + ".ProcessInput.CurValSt";

			dynClear(dsDpParameters);
			dynClear(dsArchiveNames);

//	Get all common Parameters
			unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);

			sUnit =  dpGetUnit(element);	//	Get DP unit
			sFormatConverted = unExportDevice_convertFormat(dpGetFormat(element), "#");	// Get Format
//	Get RangeMax and RangeMin
			fwPvRange_get(element, bDoesExist , fMinValue , fMaxValue , bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax , exceptionInfo);
			sMax = unExportDevice_floatFormat(fMaxValue);			
			sMin = unExportDevice_floatFormat(fMinValue);
//	Get unicos DeadBand
			unExportDevice_fwSmoothing_getTemp(element, bIsSmoothDefined, iSmoothProcedure, fDeadbandTemp, fTimeInterval);
			unExportDevice_getDeadBand(fDeadbandTemp, bIsSmoothDefined, sMax, sMin, iSmoothProcedure, fDeadband, iDeadBandType);	
//	Get Archive Parameters
			gcsExportDevice_getArchiveParameters(currentDP + ".ProcessInput.CurValSt", sObject, sArchiveActive, fArchiveTimeFilter);
			gcsExportDevice_getArchiveNames(currentDP,
				makeDynString(".ProcessInput.CurValSt"), makeDynString(), makeDynString(), dsArchiveNames);
//	Get address config		
			gcsExportDevice_getAddressConfig(sObject, currentDP,
					makeDynString("ValRqAck", "CurValSt", "ValRq"),
					makeDynBool(true, true, false), addressDpParameters);

//	Write to file
			dynAppend(dsDpParameters, sUnit);
			dynAppend(dsDpParameters, sFormatConverted);
			if (iDeadBandType==DPCONFIG_NONE)
				dynAppend(dsDpParameters, UN_CONFIG_EXPORT_UNDEF);
			else
				dynAppend(dsDpParameters, fDeadband);
			dynAppend(dsDpParameters, iDeadBandType);
			dynAppend(dsDpParameters, sMax);
			dynAppend(dsDpParameters, sMin);
			dynAppend(dsDpParameters, sArchiveActive);
			dynAppend(dsDpParameters, addressDpParameters);
			dpGet(currentDP + ".ProcessOutput.DfltVal:_online.._value", bDefault);
			if (bDefault)
				sDefaultValue = "True";
			else
				sDefaultValue = "False";
			dpGet(currentDP + ".ProcessOutput.DfltVal", iDefault);
			dynAppend(dsDpParameters, iDefault);
			for(int k = 1; k <= dynlen(dsArchiveNames); k++)
				dynAppend(dsDpParameters, dsArchiveNames[k]);

			unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
		}//end for
}

//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_WordParameter_ExportConfig
/**
Purpose: Export WordParameter Devices for S7_PLC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void S7_PLC_WordParameter_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	_UnPlc_WordParameter_ExportConfig(dsDpList, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_AnalogStatus_ExportConfig
/**
Purpose: Export AnalogStatus Devices for _UnPlc front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void _UnPlc_AnalogStatus_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
dyn_string addressDpParameters, dsDpParameters, dsArchiveNames;
string sObject, sUnit, sFormatConverted, sArchiveActive, currentDP, element, sMax, sMin;
float fArchiveTimeFilter, fDeadbandTemp, fDeadband, fTimeInterval, fMinValue , fMaxValue;
int i, iLeni, iSmoothProcedure, iDeadBandType;
bool bIsSmoothDefined, bDoesExist, bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax;
		sObject="AnalogStatus";
		g_iAlertRange = 0;
		
		iLeni=dynlen(dsDpList);
		for(i=1; i<=iLeni; i++)
		{									
			if(!dpExists(dsDpList[i])) continue;

			currentDP = dsDpList[i];
			element = currentDP + ".ProcessInput.AuPosRSt";

			dynClear(dsDpParameters);
			dynClear(dsArchiveNames);

//	Get all common Parameters
			unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);

			sUnit =  dpGetUnit(element);	//	Get DP unit
			sFormatConverted = unExportDevice_convertFormat(dpGetFormat(element), "#");	// Get Format
//	Get RangeMax and RangeMin
			fwPvRange_get(element, bDoesExist , fMinValue , fMaxValue , bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax , exceptionInfo);
			sMax = unExportDevice_floatFormat(fMaxValue);			
			sMin = unExportDevice_floatFormat(fMinValue);
//	Get unicos DeadBand
			unExportDevice_fwSmoothing_getTemp(element, bIsSmoothDefined, iSmoothProcedure, fDeadbandTemp, fTimeInterval);
			unExportDevice_getDeadBand(fDeadbandTemp, bIsSmoothDefined, sMax, sMin, iSmoothProcedure, fDeadband, iDeadBandType);	
//	Get Archive Parameters
			gcsExportDevice_getArchiveParameters(currentDP + ".ProcessInput.AuPosRSt", sObject, sArchiveActive, fArchiveTimeFilter);
			gcsExportDevice_getArchiveNames(currentDP,
				makeDynString(), makeDynString(".ProcessInput.AuPosRSt"), makeDynString(), dsArchiveNames);
//	Get address config		
			gcsExportDevice_getAddressConfig(sObject, currentDP,
					makeDynString("AuPosRSt"), makeDynBool(true), addressDpParameters);

//	Write to file
			dynAppend(dsDpParameters, sUnit);
			dynAppend(dsDpParameters, sFormatConverted);
			if (iDeadBandType==DPCONFIG_NONE)
				dynAppend(dsDpParameters, UN_CONFIG_EXPORT_UNDEF);
			else
				dynAppend(dsDpParameters, fDeadband);
			dynAppend(dsDpParameters, iDeadBandType);
			dynAppend(dsDpParameters, sMax);
			dynAppend(dsDpParameters, sMin);
			dynAppend(dsDpParameters, sArchiveActive);
			if (fArchiveTimeFilter < 0)
				dynAppend(dsDpParameters, "");
			else	
				dynAppend(dsDpParameters, fArchiveTimeFilter);
			dynAppend(dsDpParameters, addressDpParameters);
			for(int k = 1; k <= dynlen(dsArchiveNames); k++)
				dynAppend(dsDpParameters, dsArchiveNames[k]);

			unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
		}//end for
}

//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_AnalogStatus_ExportConfig
/**
Purpose: Export AnalogStatus Devices for S7_PLC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void S7_PLC_AnalogStatus_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	_UnPlc_AnalogStatus_ExportConfig(dsDpList, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_WordStatus_ExportConfig
/**
Purpose: Export WordStatus Devices for _UnPlc front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void _UnPlc_WordStatus_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
dyn_string addressDpParameters, dsDpParameters, dsArchiveNames;
string sObject, sUnit, sFormatConverted, sArchiveActive, currentDP, element,sMax, sMin;
float fArchiveTimeFilter, fDeadbandTemp, fDeadband, fTimeInterval,fMinValue , fMaxValue;
int i, iLeni, iSmoothProcedure, iDeadBandType;
bool bIsSmoothDefined, bDoesExist, bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax;

		sObject="WordStatus";
		g_iAlertRange = 0;
		
		iLeni=dynlen(dsDpList);
		for(i=1; i<=iLeni; i++)
		{									
			if(!dpExists(dsDpList[i])) continue;

			currentDP = dsDpList[i];
			element = currentDP + ".ProcessInput.AuPosRSt";
			dynClear(dsDpParameters);
			dynClear(dsArchiveNames);

//	Get all common Parameters
			unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);

			sUnit =  dpGetUnit(element);	//	Get DP unit
			sFormatConverted = unExportDevice_convertFormat(dpGetFormat(element), "#");	// Get Format
//	Get RangeMax and RangeMin
			fwPvRange_get(element, bDoesExist , fMinValue , fMaxValue , bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax , exceptionInfo);
			sMax = unExportDevice_floatFormat(fMaxValue);			
			sMin = unExportDevice_floatFormat(fMinValue);
//	Get unicos DeadBand
			unExportDevice_fwSmoothing_getTemp(element, bIsSmoothDefined, iSmoothProcedure, fDeadbandTemp, fTimeInterval);
			unExportDevice_getDeadBand(fDeadbandTemp, bIsSmoothDefined, sMax, sMin, iSmoothProcedure, fDeadband, iDeadBandType);	

//		- unConfigGenericFunctions_getArchiving: implement the following:
//			use archive=Y/N/O/A 
//			N=No,
//			Y=Yes if archiveTimeFilter=0 ==> no smoothing else time smoothing
//			O=Yes if archiveTimeFilter=0 => only old/new else old/new or time smoothing
//			A=Yes if archiveTimeFilter=0 => only old/new else old/new and time smoothing
//			new parameter type of smoothing and smoothing procedure

//	Get Archive Parameters
			gcsExportDevice_getArchiveParameters(currentDP + ".ProcessInput.AuPosRSt", sObject, sArchiveActive, fArchiveTimeFilter);
			gcsExportDevice_getArchiveNames(currentDP,
				makeDynString(".ProcessInput.AuPosRSt"), makeDynString(), makeDynString(), dsArchiveNames);
//	Get address config		
			gcsExportDevice_getAddressConfig(sObject, currentDP,
					makeDynString("AuPosRSt"), makeDynBool(true), addressDpParameters);

//	Write to file
			dynAppend(dsDpParameters, sUnit);
			dynAppend(dsDpParameters, sFormatConverted);
			if (iDeadBandType==DPCONFIG_NONE)
				dynAppend(dsDpParameters, UN_CONFIG_EXPORT_UNDEF);
			else
				dynAppend(dsDpParameters, fDeadband);
			dynAppend(dsDpParameters, iDeadBandType);
			dynAppend(dsDpParameters, sMax);
			dynAppend(dsDpParameters, sMin);
			dynAppend(dsDpParameters, sArchiveActive);
			dynAppend(dsDpParameters, addressDpParameters);
			dynAppend(dsDpParameters, dpGetDescription(element));
			for(int k = 1; k <= dynlen(dsArchiveNames); k++)
				dynAppend(dsDpParameters, dsArchiveNames[k]);

			unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
		}//end for
}

//
//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_Word2AnalogStatus_ExportConfig
/**
Purpose: Export Word2AnalogStatus Devices for _UnPlc front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void _UnPlc_Word2AnalogStatus_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
dyn_string addressDpParameters, dsDpParameters, dsArchiveNames;
string sObject, sUnit, sFormatConverted, sArchiveActive, currentDP, element, sMax, sMin;
float fArchiveTimeFilter, fDeadbandTemp, fDeadband, fTimeInterval, fMinValue , fMaxValue;
dyn_float arguments;
int i, iLeni, iSmoothProcedure, iDeadBandType, conversionType,order, iScaleMin, iScaleMax;
bool bIsSmoothDefined, bDoesExist, bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax, configExists;

		sObject="Word2AnalogStatus";
		g_iAlertRange = 0;
		
		iLeni=dynlen(dsDpList);
		for(i=1; i<=iLeni; i++)
		{									
			if(!dpExists(dsDpList[i])) continue;

			currentDP = dsDpList[i];
			element = currentDP + ".ProcessInput.AuPosRSt";

			dynClear(dsDpParameters);
			dynClear(dsArchiveNames);

//	Get all common Parameters
			unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);

			sUnit =  dpGetUnit(element);	//	Get DP unit
			sFormatConverted = unExportDevice_convertFormat(dpGetFormat(element), "#");	// Get Format

// 	Get conversion parameters to set back Scale Max % Scale Min
			fwConfigConversion_get(element, configExists, DPCONFIG_CONVERSION_RAW_TO_ENG_MAIN, conversionType, order,arguments, exceptionInfo);
			//DebugN("configExists: "+configExists);
			//DebugN("conversionType: "+conversionType);
			//DebugN("order: "+order);
			//DebugN("arguments: "+arguments);
					
			if (arguments[2]!= 0 && arguments[2] <1){
				iScaleMax = 1/arguments[2];
				iScaleMin = 1;
			}
			else{			
				iScaleMax = (int)arguments[2];
				iScaleMin = 0;
			}
									
//	Get RangeMax and RangeMin
			fwPvRange_get(element, bDoesExist , fMinValue , fMaxValue , bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax , exceptionInfo);
			sMax = unExportDevice_floatFormat(fMaxValue);			
			sMin = unExportDevice_floatFormat(fMinValue);
//	Get unicos DeadBand
			unExportDevice_fwSmoothing_getTemp(element, bIsSmoothDefined, iSmoothProcedure, fDeadbandTemp, fTimeInterval);
			unExportDevice_getDeadBand(fDeadbandTemp, bIsSmoothDefined, sMax, sMin, iSmoothProcedure, fDeadband, iDeadBandType);	
//	Get Archive Parameters
			gcsExportDevice_getArchiveParameters(currentDP + ".ProcessInput.AuPosRSt", sObject, sArchiveActive, fArchiveTimeFilter);
			gcsExportDevice_getArchiveNames(currentDP,
				makeDynString(), makeDynString(".ProcessInput.AuPosRSt"), makeDynString(), dsArchiveNames);
//	Get address config		
			gcsExportDevice_getAddressConfig(sObject, currentDP,
					makeDynString("AuPosRSt"), makeDynBool(true), addressDpParameters);

//	Write to file
			dynAppend(dsDpParameters, sUnit);
			dynAppend(dsDpParameters, sFormatConverted);
			if (iDeadBandType==DPCONFIG_NONE)
				dynAppend(dsDpParameters, UN_CONFIG_EXPORT_UNDEF);
			else
				dynAppend(dsDpParameters, fDeadband);
			dynAppend(dsDpParameters, iDeadBandType);
			dynAppend(dsDpParameters, iScaleMax);
			dynAppend(dsDpParameters, iScaleMin);
			dynAppend(dsDpParameters, sMax);
			dynAppend(dsDpParameters, sMin);
			dynAppend(dsDpParameters, sArchiveActive);
			if (fArchiveTimeFilter < 0)
				dynAppend(dsDpParameters, "");
			else	
				dynAppend(dsDpParameters, fArchiveTimeFilter);
			dynAppend(dsDpParameters, addressDpParameters);
			for(int k = 1; k <= dynlen(dsArchiveNames); k++)
				dynAppend(dsDpParameters, dsArchiveNames[k]);

			unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
		}//end for
}

//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_Word2AnalogStatus_ExportConfig
/**
Purpose: Export Word2AnalogStatus Devices for S7_PLC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void S7_PLC_Word2AnalogStatus_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	_UnPlc_Word2AnalogStatus_ExportConfig(dsDpList, exceptionInfo);
}


/*********/

//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_WordStatus_ExportConfig
/**
Purpose: Export WordStatus Devices for S7_PLC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void S7_PLC_WordStatus_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	_UnPlc_WordStatus_ExportConfig(dsDpList, exceptionInfo);
}





////////////////////////////////////////////
////////////////////////////////////////////
//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_AnalogComputed_ExportConfig
/**
Purpose: Export AnalogComputed Devices for _UnPlc front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void _UnPlc_AnalogComputed_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
dyn_string addressDpParameters, dsDpParameters, dsArchiveNames;
string sObject, sArchiveActive, currentDP, sMax, sMin, sMask, sDescription, element, sUnit, sFormatConverted;
float fMinValue, fMaxValue, fArchiveTimeFilter, fDeadband, fDeadbandTemp, fTimeInterval;
int i, iLeni, iNormalPosition, iMask, iSmoothProcedure, iDeadBandType;
bool bDoesExist, bNegateRange, bIgnoreOutside, bInclusiveMin, bInclusiveMax, bIsSmoothDefined;

		sObject="AnalogComputed";

		iLeni=dynlen(dsDpList);
		for(i=1; i<=iLeni; i++)
		{
			if(!dpExists(dsDpList[i])) continue;

			currentDP = dsDpList[i];
			element = currentDP + ".ProcessInput.PosOV";
			dynClear(dsDpParameters);
			dynClear(dsArchiveNames);
					
//	Get all common Parameters
			unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);
//	Unit and Format
			sUnit =  dpGetUnit(element);	//	Get DP unit
			sFormatConverted = unExportDevice_convertFormat(dpGetFormat(element), "#");	// Get Format
//	Get RangeMax and RangeMin
			fwPvRange_get(element, bDoesExist , fMinValue , fMaxValue , bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax , exceptionInfo);
			sMax = unExportDevice_floatFormat(fMaxValue);			
			sMin = unExportDevice_floatFormat(fMinValue);
//	Get unicos DeadBand and DeadBandType
			unExportDevice_fwSmoothing_getTemp(element, bIsSmoothDefined, iSmoothProcedure, fDeadbandTemp, fTimeInterval);
			unExportDevice_getDeadBand(fDeadbandTemp, bIsSmoothDefined, sMax, sMin, iSmoothProcedure, fDeadband, iDeadBandType);	
//	Get Archive Parameters
			gcsExportDevice_getArchiveParameters(currentDP + ".ProcessInput.PosOV", sObject, sArchiveActive, fArchiveTimeFilter);
			gcsExportDevice_getArchiveNames(currentDP,
				makeDynString(".ProcessInput.PosOV"), makeDynString(), makeDynString(), dsArchiveNames);

//	Get address config		
			gcsExportDevice_getAddressConfig(sObject, currentDP,
					makeDynString("StsReg01", "evStsReg01", "PosOV", "AuPosRSt", "ManReg01", "MPosR"),
					makeDynBool(true, true, true, true, false, false), addressDpParameters);
//	Mask
			unGenericDpFunctions_getMaskEvent(currentDP, iMask);
				sprintf(sMask, "%d", iMask);

//	Write to file
			dynAppend(dsDpParameters, sUnit);
			dynAppend(dsDpParameters, sFormatConverted);
			dynAppend(dsDpParameters, sMax);
			dynAppend(dsDpParameters, sMin);
			if (iDeadBandType==DPCONFIG_NONE)
				dynAppend(dsDpParameters, UN_CONFIG_EXPORT_UNDEF);
			else
				dynAppend(dsDpParameters, fDeadband);
			dynAppend(dsDpParameters, iDeadBandType);
			dynAppend(dsDpParameters, sArchiveActive);
			if (fArchiveTimeFilter < 0)
				dynAppend(dsDpParameters, "");
			else	
				dynAppend(dsDpParameters, fArchiveTimeFilter);
			dynAppend(dsDpParameters, addressDpParameters);
			dynAppend(dsDpParameters, sMask);
			for(int k = 1; k <= dynlen(dsArchiveNames); k++)
				dynAppend(dsDpParameters, dsArchiveNames[k]);

			unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
		}//	end for
}

//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_AnalogComputed_ExportConfig
/**
Purpose: Export AnalogComputed Devices for S7_PLC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void S7_PLC_AnalogComputed_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	_UnPlc_AnalogComputed_ExportConfig(dsDpList, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_WordComputed_ExportConfig
/**
Purpose: Export WordComputed Devices for _UnPlc front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void _UnPlc_WordComputed_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
dyn_string addressDpParameters, dsDpParameters, dsArchiveNames;
string sObject, sArchiveActive, currentDP, sMask, sUnit, element, sFormatConverted, sMax, sMin;
float fMinValue, fMaxValue, fArchiveTimeFilter, fDeadbandTemp, fTimeInterval, fDeadband;
int i, iLeni, iNormalPosition, iMask, iSmoothProcedure, iDeadBandType;
bool bIsSmoothDefined, bDoesExist, bNegateRange, bIgnoreOutside, bInclusiveMin, bInclusiveMax;

		sObject="WordComputed";

		iLeni=dynlen(dsDpList);
		for(i=1; i<=iLeni; i++)
		{
			if(!dpExists(dsDpList[i])) continue;

			currentDP = dsDpList[i];
			element = currentDP + ".ProcessInput.PosOV";
			dynClear(dsDpParameters);
			dynClear(dsArchiveNames);
					
//	Get all common Parameters
			unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);
//	Unit and Format
			sUnit =  dpGetUnit(element);	//	Get DP unit
			sFormatConverted = unExportDevice_convertFormat(dpGetFormat(element), "#");	// Get Format
//	Get RangeMax and RangeMin
			fwPvRange_get(element, bDoesExist , fMinValue , fMaxValue , bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax , exceptionInfo);
			if (dynlen(exceptionInfo) != 0)
			{
				fwException_raise(exceptionInfo,"ERROR","gcsExportDevice_UnPlc_WordComputed_ExportConfig: fwPvRange_get error - ","");
				return;
			}
			sMax = unExportDevice_floatFormat(fMaxValue);
			sMin = unExportDevice_floatFormat(fMinValue);
//	Get unicos DeadBand and DeadBandType
			unExportDevice_fwSmoothing_getTemp(element, bIsSmoothDefined, iSmoothProcedure, fDeadbandTemp, fTimeInterval);
			unExportDevice_getDeadBand(fDeadbandTemp, bIsSmoothDefined, sMax, sMin, iSmoothProcedure, fDeadband, iDeadBandType);	
//	Get Archive Parameters
			gcsExportDevice_getArchiveParameters(currentDP + ".ProcessInput.PosOV", sObject, sArchiveActive, fArchiveTimeFilter);
			gcsExportDevice_getArchiveNames(currentDP,
				makeDynString(".ProcessInput.PosOV"), makeDynString(), makeDynString(), dsArchiveNames);
//	Get address config		
			gcsExportDevice_getAddressConfig(sObject, currentDP,
					makeDynString("StsReg01", "evStsReg01", "PosOV", "AuPosRSt", "ManReg01", "MPosR"),
					makeDynBool(true, true, true, true, false, false), addressDpParameters);
//	Mask
			unGenericDpFunctions_getMaskEvent(currentDP, iMask);
				sprintf(sMask, "%d", iMask);

//	Write to file
			dynAppend(dsDpParameters, sUnit);
			dynAppend(dsDpParameters, sFormatConverted);
			dynAppend(dsDpParameters, sMax);
			dynAppend(dsDpParameters, sMin);
			if (iDeadBandType==DPCONFIG_NONE)
				dynAppend(dsDpParameters, UN_CONFIG_EXPORT_UNDEF);
			else
				dynAppend(dsDpParameters, fDeadband);
			dynAppend(dsDpParameters, iDeadBandType);
			dynAppend(dsDpParameters, sArchiveActive);
			dynAppend(dsDpParameters, addressDpParameters);
			dynAppend(dsDpParameters, dpGetDescription(element));
			dynAppend(dsDpParameters, sMask);
			for(int k = 1; k <= dynlen(dsArchiveNames); k++)
				dynAppend(dsDpParameters, dsArchiveNames[k]);

			unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
		}//	end for
}

//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_WordComputed_ExportConfig
/**
Purpose: Export WordComputed Devices for S7_PLC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void S7_PLC_WordComputed_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	_UnPlc_WordComputed_ExportConfig(dsDpList, exceptionInfo);
}
























//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_AnalogDelayedAlarm_ExportConfig
/**
Purpose: Export AnalogDelayedAlarm Devices for _UnPlc front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void _UnPlc_AnalogDelayedAlarm_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
dyn_string addressDpParameters, dsDpParameters, dsArchiveNames;
string sObject, sArchiveActive, currentDP;
string sMask, sDescription, sAlarmType, sAlertText, className, AckAl;
float fMinValue, fMaxValue, fArchiveTimeFilter ;
int i, iLeni, iNormalPosition, iMask, iRes, iAlertType;
bool bOkRange;

	sObject="AnalogDelayedAlarm";
		
	iLeni=dynlen(dsDpList);
	for(i=1; i<=iLeni; i++)
	{
		if(!dpExists(dsDpList[i])) continue;

		currentDP = dsDpList[i];
		dynClear(dsDpParameters);
		dynClear(dsArchiveNames);
					
//	Get all common Parameters
		unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);

//	Get address config		
		gcsExportDevice_getAddressConfig(sObject, currentDP,
			makeDynString("StsReg01", "evStsReg01", "ManReg01"),
			makeDynBool(true, true, false), addressDpParameters);

		gcsExportDevice_getDigitalNormalPosition(currentDP, sObject, makeDynString(".ProcessInput.PosAl"), iNormalPosition);

//	Get Archive Parameters
		gcsExportDevice_getArchiveParameters(currentDP + ".ProcessInput.evStsReg01", sObject, sArchiveActive, fArchiveTimeFilter);
		gcsExportDevice_getArchiveNames(currentDP, makeDynString(".ProcessInput.evStsReg01"), 
			makeDynString(), makeDynString(), dsArchiveNames);

//	Mask
		unGenericDpFunctions_getMaskEvent(currentDP, iMask);
			sprintf(sMask, "%d", iMask);

//	Get alarm Description.
		sDescription = gcsExportDevice_getAlarmDescription(currentDP + ".ProcessInput.PosAl");

//	Get if Alarm is acknowledgeble
		AckAl =gcsExportDevice_getAlarmIsAck(currentDP + ".ProcessInput.PosAl");

//	Write to file
		dynAppend(dsDpParameters, iNormalPosition);
		dynAppend(dsDpParameters, addressDpParameters);
		dynAppend(dsDpParameters, sDescription);
		dynAppend(dsDpParameters, AckAl);
		dynAppend(dsDpParameters, sMask);
		for(int k = 1; k <= dynlen(dsArchiveNames); k++)
			dynAppend(dsDpParameters, dsArchiveNames[k]);

		unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
	}//	end for
}


//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_AnalogDelayedAlarm_ExportConfig
/**
Purpose: Export AnalogDelayedAlarm Devices for S7_PLC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void S7_PLC_AnalogDelayedAlarm_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	_UnPlc_AnalogDelayedAlarm_ExportConfig(dsDpList, exceptionInfo);
}





//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_DigitalDelayedAlarm_ExportConfig
/**
Purpose: Export DigitalDelayedAlarm Devices for _UnPlc front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void _UnPlc_DigitalDelayedAlarm_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
dyn_string addressDpParameters, dsDpParameters, dsArchiveNames;
string sObject, sArchiveActive, currentDP;
string sMask, sDescription, sAlarmType, sAlertText, className, AckAl;
float fMinValue, fMaxValue, fArchiveTimeFilter ;
int i, iLeni, iNormalPosition, iMask, iRes, iAlertType;
bool bOkRange;

	sObject="DigitalDelayedAlarm";
		
	iLeni=dynlen(dsDpList);
	for(i=1; i<=iLeni; i++)
	{
		if(!dpExists(dsDpList[i])) continue;

		currentDP = dsDpList[i];
		dynClear(dsDpParameters);
		dynClear(dsArchiveNames);
					
//	Get all common Parameters
		unExportDevice_getAllCommonParameters(currentDP, sObject, dsDpParameters);

//	Get address config		
		gcsExportDevice_getAddressConfig(sObject, currentDP,
			makeDynString("StsReg01", "evStsReg01", "ManReg01"),
			makeDynBool(true, true, false), addressDpParameters);

		gcsExportDevice_getDigitalNormalPosition(currentDP, sObject, makeDynString(".ProcessInput.PosAl"), iNormalPosition);

//	Get Archive Parameters
		gcsExportDevice_getArchiveParameters(currentDP + ".ProcessInput.evStsReg01", sObject, sArchiveActive, fArchiveTimeFilter);
		gcsExportDevice_getArchiveNames(currentDP, makeDynString(".ProcessInput.evStsReg01"), 
			makeDynString(), makeDynString(), dsArchiveNames);

//	Mask
		unGenericDpFunctions_getMaskEvent(currentDP, iMask);
			sprintf(sMask, "%d", iMask);

//	Get alarm Description.
		sDescription = gcsExportDevice_getAlarmDescription(currentDP + ".ProcessInput.PosAl");

//	Get if Alarm is acknowledgeble
		AckAl =gcsExportDevice_getAlarmIsAck(currentDP + ".ProcessInput.PosAl");

//	Write to file
		dynAppend(dsDpParameters, iNormalPosition);
		dynAppend(dsDpParameters, addressDpParameters);
		dynAppend(dsDpParameters, sDescription);
		dynAppend(dsDpParameters, AckAl);
		dynAppend(dsDpParameters, sMask);
		for(int k = 1; k <= dynlen(dsArchiveNames); k++)
			dynAppend(dsDpParameters, dsArchiveNames[k]);

		unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
	}//	end for
}


//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_DigitalDelayedAlarm_ExportConfig
/**
Purpose: Export DigitalDelayedAlarm Devices for S7_PLC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void S7_PLC_DigitalDelayedAlarm_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	_UnPlc_AnalogDelayedAlarm_ExportConfig(dsDpList, exceptionInfo);
}



//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_Pump_ExportConfig
/**
Export Pump Devices for _UnPlc front-end

	@param[in]	dsDpList		List of all DPs of Pump type
	@param[out]	exceptionInfo	All possible errors returnted here
*/
void _UnPlc_Pump_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
dyn_string addressDpParameters, dsDpParameters, dsArchiveNames;
string sDeviceType, sUnit, sFormatConverted, sArchiveActive, currentDP, element,sMax, sMin, sMask;
float fArchiveTimeFilter, fDeadbandTemp, fDeadband, fTimeInterval,fMinValue , fMaxValue;
int i, iLengthDS, iSmoothProcedure, iDeadBandType, iNormalPosition, iMask;
bool bIsSmoothDefined, bDoesExist, bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax;

		sDeviceType="Pump";
		g_iAlertRange = 0;

		iLengthDS=dynlen(dsDpList);
		for(i=1; i<=iLengthDS; i++)
		{
			currentDP = dsDpList[i];
			if(!dpExists(dsDpList[i]))
			{
				fwException_raise(exceptionInfo,"ERROR","gcsExportDevice_UnPlc_Pump_ExportConfig: DataPoint doesn't exist - ","");
				continue;
			}

			currentDP = dsDpList[i];
			element = currentDP + ".ProcessInput.PosSt";
			dynClear(dsDpParameters);
			dynClear(dsArchiveNames);

//	Get all common Parameters
			unExportDevice_getAllCommonParameters(currentDP, sDeviceType, dsDpParameters);

//////// setpoint parameters
			sUnit =  dpGetUnit(element);	//	Get DP unit
			sFormatConverted = unExportDevice_convertFormat(dpGetFormat(element), "#");	// Get Format
//	Get RangeMax and RangeMin
			fwPvRange_get(element, bDoesExist , fMinValue , fMaxValue , bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax , exceptionInfo);
			if (dynlen(exceptionInfo) != 0)
			{
				fwException_raise(exceptionInfo,"ERROR","gcsExportDevice_UnPlc_Pump_ExportConfig: fwPvRange_get error - ","");
				return;
			}
			sMax = unExportDevice_floatFormat(fMaxValue);
			sMin = unExportDevice_floatFormat(fMinValue);
//	Get unicos DeadBand
			unExportDevice_fwSmoothing_getTemp(element, bIsSmoothDefined, iSmoothProcedure, fDeadbandTemp, fTimeInterval);
			unExportDevice_getDeadBand(fDeadbandTemp, bIsSmoothDefined, sMax, sMin, iSmoothProcedure, fDeadband, iDeadBandType);
//	Get Archive Parameters
			gcsExportDevice_getArchiveParameters(element, sDeviceType, sArchiveActive, fArchiveTimeFilter, makeDynString(), exceptionInfo);
			if (dynlen(exceptionInfo) != 0)
			{
				fwException_raise(exceptionInfo,"ERROR","gcsExportDevice_UnPlc_Pump_ExportConfig: getting the archive parameters - ","");
				return;
			}

			dynAppend(dsDpParameters, sUnit);
			dynAppend(dsDpParameters, sFormatConverted);
			if (iDeadBandType==DPCONFIG_NONE)
				dynAppend(dsDpParameters, UN_CONFIG_EXPORT_UNDEF);
			else
				dynAppend(dsDpParameters, fDeadband);
			dynAppend(dsDpParameters, iDeadBandType);
			dynAppend(dsDpParameters, sMax);
			dynAppend(dsDpParameters, sMin);
			dynAppend(dsDpParameters, sArchiveActive);
			if (fArchiveTimeFilter < 0)
				dynAppend(dsDpParameters, "");
			else	
				dynAppend(dsDpParameters, fArchiveTimeFilter);

//////// Load parameters
			element = currentDP + ".ProcessInput.LoadSt";

			sUnit =  dpGetUnit(element);	//	Get DP unit
			sFormatConverted = unExportDevice_convertFormat(dpGetFormat(element), "#");	// Get Format
//	Get RangeMax and RangeMin
			fwPvRange_get(element, bDoesExist , fMinValue , fMaxValue , bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax , exceptionInfo);
			if (dynlen(exceptionInfo) != 0)
			{
				fwException_raise(exceptionInfo,"ERROR","gcsExportDevice_UnPlc_Pump_ExportConfig: fwPvRange_get error - ","");
				return;
			}
			sMax = unExportDevice_floatFormat(fMaxValue);
			sMin = unExportDevice_floatFormat(fMinValue);
//	Get unicos DeadBand
			unExportDevice_fwSmoothing_getTemp(element, bIsSmoothDefined, iSmoothProcedure, fDeadbandTemp, fTimeInterval);
			unExportDevice_getDeadBand(fDeadbandTemp, bIsSmoothDefined, sMax, sMin, iSmoothProcedure, fDeadband, iDeadBandType);
//	Get Archive Parameters
			gcsExportDevice_getArchiveParameters(element, sDeviceType, sArchiveActive, fArchiveTimeFilter, makeDynString(),exceptionInfo);
			if (dynlen(exceptionInfo) != 0)
			{
				fwException_raise(exceptionInfo,"ERROR","gcsExportDevice_UnPlc_Pump_ExportConfig: getting the archive parameters - ","");
				return;
			}

			dynAppend(dsDpParameters, sUnit);
			dynAppend(dsDpParameters, sFormatConverted);
			if (iDeadBandType==DPCONFIG_NONE)
				dynAppend(dsDpParameters, UN_CONFIG_EXPORT_UNDEF);
			else
				dynAppend(dsDpParameters, fDeadband);
			dynAppend(dsDpParameters, iDeadBandType);
			dynAppend(dsDpParameters, sMax);
			dynAppend(dsDpParameters, sMin);
			dynAppend(dsDpParameters, sArchiveActive);
			if (fArchiveTimeFilter < 0)
				dynAppend(dsDpParameters, "");
			else	
				dynAppend(dsDpParameters, fArchiveTimeFilter);

			gcsExportDevice_getDigitalNormalPosition(currentDP, sDeviceType, makeDynString(".ProcessInput.StartISt", ".ProcessInput.StopISt"), iNormalPosition, exceptionInfo);
			dynAppend(dsDpParameters, iNormalPosition);
			if (dynlen(exceptionInfo) != 0)
			{
				fwException_raise(exceptionInfo,"ERROR","gcsExportDevice_UnPlc_Pump_ExportConfig: getting the digital normal position - ","");
				return;
			}

//	Get Archive Parameters
			gcsExportDevice_getArchiveNames(currentDP, makeDynString(".ProcessInput.evStsReg01", ".ProcessInput.evStsReg02"),
				makeDynString(".ProcessInput.PosSt", ".ProcessInput.AOutOVSt"), makeDynString(), dsArchiveNames, exceptionInfo);
			if (dynlen(exceptionInfo) != 0)
			{
				fwException_raise(exceptionInfo,"ERROR","gcsExportDevice_UnPlc_Pump_ExportConfig: getting the archive names - ","");
				return;
			}

//	Get address config
			gcsExportDevice_getAddressConfig(sDeviceType, currentDP,
			makeDynString("StsReg01", "StsReg02", "evStsReg01", "evStsReg02", "PosSt", "LoadSt", "AuPosRSt", "AOutOVSt",
			"ManReg01", "MPosR"),
			makeDynBool(true, true, true, true, true, true, true, true, false, false), addressDpParameters, exceptionInfo);
			if (dynlen(exceptionInfo) != 0)
			{
				fwException_raise(exceptionInfo,"ERROR","gcsExportDevice_UnPlc_Pump_ExportConfig: getting the address config - ","");
				return;
			}

//	Mask
			unGenericDpFunctions_getMaskEvent(currentDP, iMask);
			if (sprintf(sMask, "%d", iMask) <0)
			{
				fwException_raise(exceptionInfo,"ERROR","gcsExportDevice_UnPlc_Pump_ExportConfig: formating mask - ","");
				return;
			}

//	Write to file
			dynAppend(dsDpParameters, addressDpParameters);
			dynAppend(dsDpParameters, sMask);
			for(int k = 1; k <= dynlen(dsArchiveNames); k++)
				dynAppend(dsDpParameters, "");//dsArchiveNames[k]);

			unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
		}//end for
}

//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_Pump_ExportConfig
/**
Export Pump Devices for S7_PLC front-end

	@param[in]	dsDpList		List of all DPs of Pump type
	@param[out]	exceptionInfo	All possible errors returnted here
*/
void S7_PLC_Pump_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	_UnPlc_Pump_ExportConfig(dsDpList, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_MassFlowController_ExportConfig
/**
Export MassFlowController Devices for _UnPlc front-end

	@param[in]	dsDpList		List of all DPs of MassFlowController type
	@param[out]	exceptionInfo	All possible errors returnted here
*/
void _UnPlc_MassFlowController_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
dyn_string addressDpParameters, dsDpParameters, dsArchiveNames;
string sDeviceType, sUnit, sFormatConverted, sArchiveActive, currentDP, element,sMax, sMin, sMask;
float fArchiveTimeFilter, fDeadbandTemp, fDeadband, fTimeInterval,fMinValue , fMaxValue;
int i, iLengthDS, iSmoothProcedure, iDeadBandType, iNormalPosition, iMask;
bool bIsSmoothDefined, bDoesExist, bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax;

		sDeviceType="MassFlowController";
		g_iAlertRange = 0;

		iLengthDS=dynlen(dsDpList);
		for(i=1; i<=iLengthDS; i++)
		{
			currentDP = dsDpList[i];
			if(!dpExists(dsDpList[i]))
			{
				fwException_raise(exceptionInfo,"ERROR","gcsExportDevice_UnPlc_MassFlowController_ExportConfig: DataPoint doesn't exist - ","");
				continue;
			}


			currentDP = dsDpList[i];
			element = currentDP + ".ProcessInput.ApSpSt";
			dynClear(dsDpParameters);
			dynClear(dsArchiveNames);

//	Get all common Parameters
			unExportDevice_getAllCommonParameters(currentDP, sDeviceType, dsDpParameters);

//////// flow and setpoint parameters
			sUnit =  dpGetUnit(element);	//	Get DP unit
			sFormatConverted = unExportDevice_convertFormat(dpGetFormat(element), "#");	// Get Format
//	Get RangeMax and RangeMin
			fwPvRange_get(element, bDoesExist , fMinValue , fMaxValue , bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax ,exceptionInfo);
			if (dynlen(exceptionInfo) != 0)
			{
				fwException_raise(exceptionInfo,"ERROR","gcsExportDevice_UnPlc_MassFlowController_ExportConfig: fwPvRange_get error - ","");
				return;
			}
			sMax = unExportDevice_floatFormat(fMaxValue);
			sMin = unExportDevice_floatFormat(fMinValue);
//	Get unicos DeadBand


			unExportDevice_fwSmoothing_getTemp(element, bIsSmoothDefined, iSmoothProcedure, fDeadbandTemp, fTimeInterval);
			unExportDevice_getDeadBand(fDeadbandTemp, bIsSmoothDefined, sMax, sMin, iSmoothProcedure, fDeadband, iDeadBandType);
//	Get Archive Parameters
			gcsExportDevice_getArchiveParameters(currentDP + ".ProcessInput.SpOV", sDeviceType, sArchiveActive, fArchiveTimeFilter, makeDynString(), exceptionInfo);

			if (dynlen(exceptionInfo) != 0)
			{
				fwException_raise(exceptionInfo,"ERROR","gcsExportDevice_UnPlc_MassFlowController_ExportConfig: getting the archive parameters - ","");
				return;
			}

			dynAppend(dsDpParameters, sUnit);
			dynAppend(dsDpParameters, sFormatConverted);
			if (iDeadBandType==DPCONFIG_NONE)
				dynAppend(dsDpParameters, UN_CONFIG_EXPORT_UNDEF);
			else
				dynAppend(dsDpParameters, fDeadband);

			dynAppend(dsDpParameters, iDeadBandType);
			dynAppend(dsDpParameters, sMax);
			dynAppend(dsDpParameters, sMin);
			dynAppend(dsDpParameters, sArchiveActive);

			if (fArchiveTimeFilter < 0)
				dynAppend(dsDpParameters, "");
			else	
				dynAppend(dsDpParameters, fArchiveTimeFilter);

//////// totalizer parameters
			element = currentDP + ".ProcessInput.TotSt";

			sUnit =  dpGetUnit(element);	//	Get DP unit
			sFormatConverted = unExportDevice_convertFormat(dpGetFormat(element), "#");	// Get Format
//	Get unicos DeadBand
			unExportDevice_fwSmoothing_getTemp(element, bIsSmoothDefined, iSmoothProcedure, fDeadbandTemp, fTimeInterval);

			unExportDevice_getDeadBand(fDeadbandTemp, bIsSmoothDefined, sMax, sMin, iSmoothProcedure, fDeadband, iDeadBandType);
//	Get Archive Parameters
			gcsExportDevice_getArchiveParameters(element, sDeviceType, sArchiveActive, fArchiveTimeFilter, makeDynString(), exceptionInfo);
			if (dynlen(exceptionInfo) != 0)
			{
				fwException_raise(exceptionInfo,"ERROR","gcsExportDevice_UnPlc_MassFlowController_ExportConfig: getting the archive parameters - ","");
				return;
			}

			dynAppend(dsDpParameters, sUnit);

			dynAppend(dsDpParameters, sFormatConverted);
			if (iDeadBandType==DPCONFIG_NONE)
				dynAppend(dsDpParameters, UN_CONFIG_EXPORT_UNDEF);

			else
				dynAppend(dsDpParameters, fDeadband);

			dynAppend(dsDpParameters, iDeadBandType);
			dynAppend(dsDpParameters, sArchiveActive);

			if (fArchiveTimeFilter < 0)
				dynAppend(dsDpParameters, "");
			else
				dynAppend(dsDpParameters, fArchiveTimeFilter);

			gcsExportDevice_getDigitalNormalPosition(currentDP, sDeviceType, makeDynString(".ProcessInput.StartISt", ".ProcessInput.StopISt"), iNormalPosition, exceptionInfo);
			dynAppend(dsDpParameters, iNormalPosition);
			if (dynlen(exceptionInfo) != 0)
			{
				fwException_raise(exceptionInfo,"ERROR","gcsExportDevice_UnPlc_MassFlowController_ExportConfig: getting the digital normal position - ","");
				return;
			}

//	Get Archive Parameters
			gcsExportDevice_getArchiveNames(currentDP, makeDynString(".ProcessInput.evStsReg01"),
				makeDynString(".ProcessInput.SpOV", ".ProcessInput.FlSt"), makeDynString(), dsArchiveNames, exceptionInfo);
			if (dynlen(exceptionInfo) != 0)
			{
				fwException_raise(exceptionInfo,"ERROR","gcsExportDevice_UnPlc_MassFlowController_ExportConfig: getting the archive names - ","");
				return;
			}

//	Get address config
			gcsExportDevice_getAddressConfig(sDeviceType, currentDP,
			makeDynString("StsReg01", "evStsReg01", "ApSpSt", "AuSpRSt", "SpOV", "DrMoSt", "AuDrMRSt", "DrMoOV",
			"CcSt", "AuCcRSt", "CcOV", "TotMoSt", "AuTotMoRSt", "TotMoOV", "MaxFlSt", "FlSt", "TotSt",
			"ManReg01", "MSpR", "MDrMoR", "MCcR", "MTotMoR"),
			makeDynBool(true, true, true, true, true, true, true, true, true, true, true, true, true, true, true, true, true,
			false, false, false, false, false), addressDpParameters, exceptionInfo);
			if (dynlen(exceptionInfo) != 0)
			{
				fwException_raise(exceptionInfo,"ERROR","gcsExportDevice_UnPlc_MassFlowController_ExportConfig: getting the address config - ","");
				return;
			}
//	Mask
			unGenericDpFunctions_getMaskEvent(currentDP, iMask);
			if (sprintf(sMask, "%d", iMask) <0)
			{
				fwException_raise(exceptionInfo,"ERROR","gcsExportDevice_UnPlc_MassFlowController_ExportConfig: formating mask - ","");
				return;
			}

//	Write to file

			dynAppend(dsDpParameters, addressDpParameters);
			dynAppend(dsDpParameters, dpGetDescription(currentDP + ".ProcessInput.CcSt"));	//	calibration curve
			dynAppend(dsDpParameters, sMask);

			for(int k = 1; k <= dynlen(dsArchiveNames); k++)
				dynAppend(dsDpParameters, "");

			unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
		}//end for
}

//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_MassFlowController_ExportConfig
/**
Export MassFlowController Devices for S7_PLC front-end

	@param[in]	dsDpList		List of all DPs of MassFlowController type
	@param[out]	exceptionInfo	All possible errors returnted here
*/
void S7_PLC_MassFlowController_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	_UnPlc_MassFlowController_ExportConfig(dsDpList, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------
// gcsExportDevice_getArchiveParameters
/**
Purpose:  Get archive parameters of a dp

Parameters:

Usage: Internal function

PVSS manager usage: NG, NV
*/
void gcsExportDevice_getArchiveParameters(string sDp, string sObject, string &sArchiveActive,  float &fArchiveTimeFilter,string sVersion="")
{
	bool bArchiveActive;
 //bool bArchiveActive = false;
	//sArchiveActive = "N";
	//fArchiveTimeFilter = -1;

	if(!dpExists(sDp + ":_archive.._archive")) return;
	dpGet(sDp + ":_archive.._archive" , bArchiveActive);
	//
	if(sVersion != "") {
		unExportDevice_getArchiveTemp(sDp, bArchiveActive, sArchiveActive, fArchiveTimeFilter);
		if(sArchiveActive == "")
		{
			sArchiveActive = "N";
			fArchiveTimeFilter = -1;
		}
	}
	else {
		if (bArchiveActive)
		{
			unExportDevice_getArchiveTemp(sDp, bArchiveActive, sArchiveActive, fArchiveTimeFilter);
		}
		else
		{	
			sArchiveActive = "N";
			fArchiveTimeFilter = -1;
		}		
	}
	//
	
	//if (bArchiveActive)
	//{
	//	DebugN("boolean archive");
  //		unExportDevice_getArchiveTemp(sDp, bArchiveActive, sArchiveActive, fArchiveTimeFilter);
	//}
}



//------------------------------------------------------------------------------------------------------------------------
// gcsExportDevice_getAddressConfig
/**
Purpose:  Get address config parameters of a dp

Parameters:

Usage: Internal function

PVSS manager usage: NG, NV
*/
void gcsExportDevice_getAddressConfig(string sObject, string sDP, dyn_string dsDpElements, dyn_bool dbIsInput, dyn_string &addressDpParameters)
{
//	we don't need sObject at the moment, as unicos functions need it, but not used. we can pass "".
	int i=0, iLen;
	dynClear(addressDpParameters);
	iLen=dynlen(dsDpElements);
	for(i = 1; i <= iLen; i++)
	{
		string temp = gcsExportDevice_getAddress(sDP, dsDpElements[i], sObject, dbIsInput[i]);
		dynAppend(addressDpParameters, temp);
	}
}


string gcsExportDevice_getAddress(string sDP, string sDPE, string sObject, bool isInput)
{
	string sFunction, temp = "";

	string process = (isInput == true) ? ".ProcessInput." : ".ProcessOutput.";
	string fullDPE = sDP + process + sDPE;

	dpGet(fullDPE + ":_address.._drv_ident" , temp);
	sFunction = "DRV_" + temp + "_convertToUnicosAddress";
	if(!isFunctionDefined(sFunction)) return UN_CONFIG_EXPORT_UNDEF;
	
	evalScript(temp,	"string main(string sDeviceDpeName, string sDeviceType)"+
										"{	string temp;"+
										"	temp = "+sFunction+"(sDeviceDpeName, sDeviceType);"+
										"	return temp;"+
										"}", makeDynString(), fullDPE, sObject);
	return temp;
}








//------------------------------------------------------------------------------------------------------------------------
// gcsExportDevice_getArchiveNames
/**
Purpose: Get Archives names (Bool, Analog and Event values)

Parameters:

Usage: Internal function

PVSS manager usage: NG, NV
*/
void gcsExportDevice_getArchiveNames(string sDp, dyn_string boolArch, dyn_string analogArch, dyn_string eventArch, dyn_string &dsNames)
{
	string str;
	dynClear(dsNames);

	str = gcsExportDevice_getArchiveName(sDp, boolArch, UN_CONFIG_EXPORT_ARCHIVE_1);
	dynAppend(dsNames, str);
	str = gcsExportDevice_getArchiveName(sDp, analogArch, UN_CONFIG_EXPORT_ARCHIVE_2);
	dynAppend(dsNames, str);
	str = gcsExportDevice_getArchiveName(sDp, eventArch, UN_CONFIG_EXPORT_ARCHIVE_3);
	dynAppend(dsNames, str);
}


//------------------------------------------------------------------------------------------------------------------------
// gcsExportDevice_getArchiveName
/**
Purpose: Get Archive name for paticular archive type.

Parameters:

Usage: Internal function

PVSS manager usage: NG, NV
*/
string gcsExportDevice_getArchiveName(string sDp, dyn_string dsArchive, string sUdnef)
{
	int i, iLen;
	string sName = "", sArchiveClass = "";
	dyn_string dsCheck;

	iLen=dynlen(dsArchive);
	if(iLen == 0) return sUdnef;
	for (i = 1; i <= iLen; i++)
	{
		dpGet(sDp + dsArchive[i] + ":_archive.1._class", sArchiveClass);
		if(dpExists(sArchiveClass + ".general.arName"))
	 		dpGet(sArchiveClass + ".general.arName:_online.._value", sName);
	 	else
			sName = sUdnef;	
	 	dynAppend(dsCheck, sName);
	}
	dynUnique(dsCheck);

	if(dynlen(dsCheck) ==1 )		//check if same archive for same type
		return dsCheck[1];	
	else return UN_CONFIG_EXPORT_UNDEF;	
}


/////////////////////////////////////////////////////
void gcsExportDevice_getDigitalNormalPosition(string sDP, string sObject, dyn_string dsDpElements, int &iNormalPosition)
{
//	we don't need sObject at the moment, as unicos functions need it, but not used. we can pass "".
bool bAlarmMask, bOkRange, bBoolAlert, bSMS = false;
dyn_int diAlertClass;
int iAlertType, iRes, i;
string sDataPoint, sAlertClass1, sAlertClass2;

	iNormalPosition = -1;
	for(i = 1; i <= dynlen(dsDpElements); i++)
	{
		iRes = dpGet(sDP + dsDpElements[i] + ":_alert_hdl.._type", iAlertType);
		dynAppend(diAlertClass, iAlertType);
	}
	i = dynUnique(diAlertClass);
	
	if(i != 1) iAlertType = DPCONFIG_NONE;
	else iAlertType = diAlertClass[1];

	if(iAlertType == DPCONFIG_NONE)
	{
		iNormalPosition = 2;
		return;
	}

	sDataPoint = sDP + dsDpElements[1];
	bBoolAlert = ((iRes >= 0) && (iAlertType == DPCONFIG_ALERT_BINARYSIGNAL)); 
	if (bBoolAlert) 
	{ 
		iRes = dpGet(sDataPoint + ":_alert_hdl.._ok_range", bOkRange);
	}
		
	iRes = dpGet(sDataPoint + ":_alert_hdl.._active",  bAlarmMask);
	iRes = dpGet(sDataPoint + ":_alert_hdl.1._class",  sAlertClass1);
	iRes = dpGet(sDataPoint + ":_alert_hdl.2._class",  sAlertClass2);
		
	if((patternMatch(UN_PROCESSALARM_PATTERN,sAlertClass1))||(patternMatch(UN_PROCESSALARM_PATTERN,sAlertClass2)))
		bSMS = true;
	else bSMS = false;		
	
	if(bAlarmMask)
	{ //	Alarm unmasked
		if(bOkRange) iNormalPosition = 1;	//	ON
		else iNormalPosition = 0;			//	OFF
	}else
	{ //	Alarm masked
		if(bOkRange)iNormalPosition = 4;	//	ON
		else iNormalPosition = 3;			//	OFF
	}
	if(bSMS) iNormalPosition += 100;
} 

//	Get alarm Description.
string gcsExportDevice_getAlarmDescription(string alarmDP)
{
	int iRes, iAlertType;
	bool bOkRange;
	string sDescription = UN_CONFIG_EXPORT_UNDEF;

	iRes = dpGet(alarmDP + ":_alert_hdl.._type", iAlertType);
	if((iRes >= 0) && (iAlertType == DPCONFIG_ALERT_BINARYSIGNAL))
	{
		dpGet(alarmDP + ":_alert_hdl.._ok_range", bOkRange);
		if(bOkRange) dpGet(alarmDP + ":_alert_hdl.._text0", sDescription);
		else		 dpGet(alarmDP + ":_alert_hdl.._text1", sDescription);
	}
	return sDescription;
}


//	Get if Alarm is acknowledgeble
string gcsExportDevice_getAlarmIsAck(string alarmDP)
{
	int iRes, iAlertType;
	string className;

	iRes = dpGet(alarmDP + ":_alert_hdl.._type", iAlertType);
	if((iRes < 0) || (iAlertType == DPCONFIG_NONE)) return UN_CONFIG_EXPORT_UNDEF;

	dpGet(alarmDP + ":_alert_hdl.._class", className);
	if(strpos(className, GCS_CONFIG_ALERT_CLASS_ALARM_ACK) >= 0 || strpos(className, GCS_CONFIG_ALERT_CLASS_ALARM_ACK_MAIL) >= 0) return "Y";
	if(strpos(className, GCS_CONFIG_ALERT_CLASS_ALARM_NOACK) >= 0 || strpos(className, GCS_CONFIG_ALERT_CLASS_ALARM_NOACK_MAIL) >= 0) return "N";
	return UN_CONFIG_EXPORT_UNDEF;
}







//
