/* @name LIBRARY: gcsGenericObject.ctl

@author: Nikolay Kulman (IT-CO-FE)
@author: Francisco Mico (IT-CO-FE)
@date 08.07.2003
@brief This library contains generic functions for Unicos objects.
Modifications History:
	$Log: gcsGenericObject.ctl,v $
  - 23/11/10 quique
      [UCPC-33] WordStatus bit description not correctly shown; Function: gcsGenericObject_GetLabelsFromString() 
      
	- 22/10/10 amerezhi: merged with ABCO version:
		28/01/2010: Herve
		- modify the function: gcsGenericObject_GetLabelsFromString
		add the missing bits in the description if all the bits are not defined.

	Revision 1.18  2008/11/12 09:17:39  gthomas
	-removed _isRoot function
	
	Revision 1.17  2008/09/12 13:45:08  gthomas
	Fixed variable typo in <gcsGenericObject_DisplayValue> fct.

	Revision 1.16  2008/09/04 14:48:11  gthomas
	-moved back to version 1.12

	Revision 1.15  2008/06/13 12:29:47  aabadz
	Modifications asked by UNICOS for their version 4.X

	Revision 1.14  2008/06/09 14:54:26  aabadz
	Result after move some constants, only use use inside this file, from gcsObjects_declarations.

	Revision 1.13  2008/05/26 12:18:19  gthomas
	Merged files with modifications made by ABCO on xS and xP (GCS-ABCO-v4-1)
	to be used with PVSS 3.6 QT

	Revision 1.12  2008/01/22 14:46:36  aabadz
	Fixed:
	- Naming
	- Call to other functions to reuse code
	- Some error check

	Revision 1.11  2008/01/15 09:50:13  aabadz
	Changes after code review and before tests

	Revision 1.10  2007/10/30 16:21:58  gthomas
	Introduced the CVS keyword $Log to track changes made in the library

*/


/*
Internal Functions :
*/


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsGenericObject_WidgetXC_Animation
/**
Animate AC / WC  widget
*/
gcsGenericObject_WidgetXC_Animation(bool bLocked, string sSelectedManager, bit32 bit32StsReg01, bool bStsReg01Invalid,
		string sDpPosOV, int iPosOV, bool bInvalidBody, bool bInvalidLetter,
		int iSystemIntegrityAlarmValue, bool bSystemIntegrityAlarmEnabled, string sEvtMask, int iMask)
{
	string sFormattedValue;
	string sWarningLetter, sWarningColor, sControlLetter, sControlColor, sBody1Text, sBody1Color, sSelectColor;
	bool bLockVisible;

	// Control animation
	sBody1Color = "unWidget_ControlStateAuto";					// Auto mode (default)
	sControlLetter = "";
	if ((!bStsReg01Invalid) && (getBit(bit32StsReg01, UN_STSREG01_FOMOST) == 1))
	{
		sBody1Color = "unWidget_ControlStateForced";				// Forced mode
		sControlLetter = UN_WIDGET_TEXT_CONTROL_FORCED;
		sControlColor = "unWidget_ControlStateForced";
	}
	if ((!bStsReg01Invalid) && (getBit(bit32StsReg01, UN_STSREG01_MMOST) == 1))
	{
		sBody1Color = "unWidget_ControlStateManualLocal";			// Manual mode
		sControlLetter = UN_WIDGET_TEXT_CONTROL_MANUAL;
		sControlColor = "unWidget_ControlStateManualLocal";
	}
	if (!bStsReg01Invalid)
	{
		unGenericObject_WidgetWarningAnimation(bit32StsReg01,
		   makeDynInt(UN_STSREG01_AUMRW, UN_STSREG01_IOSIMUW, UN_STSREG01_IOERRORW),
		   makeDynString(UN_WIDGET_TEXT_WARNING_DEFAULT, UN_WIDGET_TEXT_WARNING_SIMU, UN_WIDGET_TEXT_WARNING_ERROR),
		   sWarningLetter, sWarningColor);
		if (bInvalidLetter)
		{
			sWarningLetter = UN_WIDGET_TEXT_OLD_DATA;
			sWarningColor = "unDataNotValid";
		}
	}// if

	if((!bSystemIntegrityAlarmEnabled)||(iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value))
	{
		sWarningLetter = UN_WIDGET_TEXT_OLD_DATA;
		sWarningColor = "unDataNotValid";
	}

	if((g_bLabelsAvailable == TRUE) && (dynContains(g_diWcValues, iPosOV) >= 1))
		sBody1Text = g_dsWcLabels[dynContains(g_diWcValues, iPosOV)];
	else
	{
		sFormattedValue = unGenericObject_FormatValue(g_sPosOvFormat, iPosOV);
		if(g_sPosOvUnit == "-")
			sBody1Text = sFormattedValue;
		else
			sBody1Text = sFormattedValue + " " + g_sPosOvUnit;
	}

	if (bInvalidBody)
		sBody1Color = "unDataNotValid";

	if (bInvalidLetter)
	{
		sWarningLetter = UN_WIDGET_TEXT_INVALID;
		sWarningColor = "unDataNotValid";
	}

	unGenericObject_WidgetSelectAnimation(bLocked, sSelectedManager, sSelectColor, bLockVisible);
	if(g_bSystemConnected)
	{
		setMultiValue("Body1", "text", sBody1Text, /*"Body1", "fill", "[solid]", */
			"Body1", "foreCol", sBody1Color, /*"Body1", "backCol", "unWidget_Background",*/
			"WarningText", "text", sWarningLetter, "WarningText", "foreCol", sWarningColor,
			"ControlStateText", "text", sControlLetter, "ControlStateText", "foreCol", sControlColor,
			"SelectArea", "foreCol", sSelectColor, "LockBmp", "visible", bLockVisible,
			"WidgetArea", "visible", true);
	}

// display event state
	if(g_bSystemConnected)
	{
		if(iMask == 0)
			eventState.visible	= true;
		else
			eventState.visible	= false;
	}
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsGenericObject_DisplayValue
/**
Animate a display value

	@param[in] sFormat		Datapoint which format has to be used
	@param[in] sUnit		Datapoint which unit has to be used
	@param[in] iValue		Value to format
	@param[in] sShape		Shape that displays the value
	@param[in] sForeCol		Display color
	@param[in] bInvalid		Is value invalid ?
*/
gcsGenericObject_DisplayValue(string sDpFormat, string sDpUnit, int iValue, string sShape, string sForeCol, bool bInvalid)
{
	string sUnit, sFormattedValue, sColor;
	int index;

	sUnit = dpGetUnit(sDpUnit);
	sFormattedValue = unGenericObject_FormatValue(dpGetFormat(sDpFormat), iValue);
	sColor = sForeCol;
	if (bInvalid)
	{
		sColor = "unDataNotValid";
	}

	// if string exists display string
	if ( g_bLabelsAvailable != TRUE)
	{
		if(sUnit == "-")
		{
			// if no string, display the int number without unit
			if(g_bSystemConnected)
				setMultiValue(sShape + ".display", "foreCol", sColor, sShape + ".display", "text", strrtrim(sFormattedValue));
		}
		else
		{
			// if no string, display the int number
			if(g_bSystemConnected)
				setMultiValue(sShape + ".display", "foreCol", sColor, sShape + ".display", "text", strrtrim(sFormattedValue + " " + sUnit));
		}
	}
	else
	{
		// retrieve position of the value in the list of value
		index = dynContains( g_diWcValues, iValue);

		// if the int value match with the one in the dyn_int display the corresponding string
		if ( index >= 1 )
		{
			if(g_bSystemConnected)
				setMultiValue(sShape + ".display", "foreCol", sColor, sShape + ".display", "text", g_dsWcLabels[index]);
//			DebugN("g_dsWcLabels[index]",g_dsWcLabels[index]);
		}
		else
		{
			if(sUnit == "-")
			{
				// if no string, display the int number without unit
				if(g_bSystemConnected)
					setMultiValue(sShape + ".display", "foreCol", sColor, sShape + ".display", "text", strrtrim(sFormattedValue));
			}
			else
			{
				// if no string, display the int number
				if(g_bSystemConnected)
					setMultiValue(sShape + ".display", "foreCol", sColor, sShape + ".display", "text", strrtrim(sFormattedValue + " " + sUnit));
			}
		}
	}
}


// gcsGenericObject_addMaskToMenu
/**
Add "Mask/UnMask" in the AA, DA menu

	@param[in]  deviceName		Device dp name
	@param[in]  sDpType			Device dp type
	@param[in]  dsMenuConfig	List of requested action
	@param[in]  dsAccessOk		List of authorized action
	@param[out] menuList		Output menu
*/
gcsGenericObject_addMaskToMenu(string deviceName, string sDpType, dyn_string dsMenuConfig, dyn_string dsAccessOk, dyn_string &menuList)
{
	string Text = "";
	bit32 stsReg01;
	bool bMasked;
	int iRes, iAccess = 0;
	dyn_string exceptionInfo;

	iRes = dpGet(deviceName + ".ProcessInput.StsReg01", stsReg01);
	if(iRes < 0) return;

	if((dynContains(dsMenuConfig, UN_MENU_AA_MASK) > 0) || (dynContains(dsMenuConfig, UN_MENU_DA_MASK) > 0))
	{
		bMasked = getBit(stsReg01, UN_STSREG01_MAMSKST);
		if (bMasked)
		{	//	masked, so we need to UnMask
			Text = UN_MENU_ALARM_RESET_MASK;
		}
		else
		{	//	not masked, so we need to Mask
			Text = UN_MENU_ALARM_SET_MASK;
		}

		if(dynContains(dsAccessOk, Text) > 0)
			iAccess = 1;
		else
		 	iAccess = 0;

		dynAppend(menuList, "PUSH_BUTTON, " + Text + ", " + UN_MENU_ALARM_MASK_REPLY + ", " + iAccess);
	}
}


// gcsGenericObject_HandleEventMaskMenu
/**
Event mask events handling

	@param[in]  deviceName		Device dp name
	@param[in]  menuAnswer		Menu answer, ID of element, which was pressed
	@param[in]  dsDP			List of DPs
	@param[out] exceptionInfo	All possible errors returned here
*/
bool gcsGenericObject_HandleEventMaskMenu(string deviceName, int menuAnswer, dyn_string dsDP, dyn_string& exceptionInfo)
{
	switch(menuAnswer)
	{
		case UN_POPUPMENU_UNMASKEVENT:				// unmask event
			unGenericObject_setMaskEvent(deviceName, 1, dsDP, exceptionInfo);
			break;
		case UN_POPUPMENU_MASKEVENT:				// mask event
			unGenericObject_setMaskEvent(deviceName, 0, dsDP, exceptionInfo);
			break;
		default:
			return false;
			break;
	}
	return true;
}


// gcsGenericObject_HandleAlarmMaskMenu
/**
Alarm mask events handling

	@param[in]  deviceName		Device dp name
	@param[out] menuList		Menu items list
	@param[in]  menuAnswer		Menu answer, ID of element, which was pressed
	@param[out] exceptionInfo	All possible errors returned here
*/
bool gcsGenericObject_HandleAlarmMaskMenu(string deviceName, dyn_string menuList, int menuAnswer, dyn_string& exceptionInfo)
{
	string s;
	bool bRes;

	if(menuAnswer == UN_MENU_ALARM_MASK_REPLY)			//	mask/unMask of Alarm
	{
		s = "PUSH_BUTTON, " + UN_MENU_ALARM_RESET_MASK + ", " + UN_MENU_ALARM_MASK_REPLY + ", 1";
		if(dynContains(menuList, s) > 0)
		{
			unGenericObject_SendBinaryOrder(deviceName, UN_MANREG01_AMSKRST, "DEBLOCKALARM", exceptionInfo);
		}
		s = "PUSH_BUTTON, " + UN_MENU_ALARM_SET_MASK + ", " + UN_MENU_ALARM_MASK_REPLY + ", 1";

		if(dynContains(menuList, s) > 0)
		{
			unGenericObject_SendBinaryOrder(deviceName, UN_MANREG01_AMSKST, "BLOCKALARM", exceptionInfo);
		}

		unGenericObject_NeedSelect(dpTypeName(deviceName), bRes, exceptionInfo);

		if (bRes)
			unSelectDeselectHMI_select(deviceName, false, exceptionInfo);
		return true;
	}
	return false;
}


// gcsGenericObject_GetLabelsFromString
/**
Split a string into label text and label ID

	@return 	If function finishedf successfully
	@param[in]  sLabels			string with labels information
	@param[out] numbersList		List of number part of labels (IDs)
	@param[out] labelsList		List of texl patrs of labels
	@todo		Check if we use this function. Probably dublication in MFC.
*/
bool gcsGenericObject_GetLabelsFromString(string sLabels, dyn_string& outnumbersList, dyn_string& outlabelsList)
{
	int len, i, iPos;
	dyn_string dsValue, dsLabels;
        string sTemp;
        dyn_string numbersList, labelsList;
        
	dynClear(outnumbersList); dynClear(outlabelsList);
 
  // DebugTN("strpos(sLabels==",strpos(sLabels, ","));
  if ((sLabels != "") && (strpos(sLabels, ",") < 0))    // if potentially only one element is defined
    sLabels=sLabels+",";
 
	if(sLabels == GCS_CONFIG_EMPTY_STRING || sLabels == UN_CONFIG_EXPORT_NONE || strpos(sLabels, ",") < 0 ) return false;
	// Split the configuration field in a list of "n=string,m=string" elements
	dsLabels = strsplit(sLabels, ",");
	len = dynlen(dsLabels);

	for (i = 1; i <= len; i++)
	{
		if(strpos(dsLabels[i], "=") < 0) continue; //	we have incorrect string
		dsValue = strsplit(dsLabels[i], "=");
		if(dynlen(dsValue) != 2) continue;
		dynAppend(numbersList, dsValue[1]);
		dynAppend(labelsList, dsValue[2]);
	}
//DebugN("middle", numbersList, labelsList);
  // add the missing bits.
        for(i=0;i<16;i++)
        {
          sTemp = i;
          dynAppend(outnumbersList, sTemp);
		  iPos = dynContains(numbersList, sTemp);
		  if(iPos > 0) {
			dynAppend(outlabelsList, labelsList[iPos]);
			dynRemove(numbersList, iPos);
			dynRemove(labelsList, iPos);
		  }
		  else
            dynAppend(outlabelsList, "N/A");
        }
        len = dynlen(numbersList);
        for(i=1;i<=len;i++)
        {
		  dynAppend(outnumbersList, numbersList[i]);
		  dynAppend(outlabelsList, labelsList[i]);
        }
//DebugN("end", outnumbersList, outlabelsList);
        
	if(dynlen(outnumbersList) > 0 && dynlen(outlabelsList) > 0 && dynlen(outnumbersList) == dynlen(outlabelsList)) return true;
	dynClear(outnumbersList); dynClear(outlabelsList);
	return false;
}


//@}
