/**@name LIBRARY: gcsMassFlowController.ctl
@author EN/ICE-SIC
@date 18.06.2003
@brief This library contains MassFlowController object functions.
Modifications History:
	$Log: gcsMassFlowController.ctl,v $
	Revision 1.32  2009/04/02 12:11:19  gthomas
	in _ProvideAccessToButtons(): removed exceptionInfo as returns errors in log when monitor or no user are logged in
	
	Revision 1.31  2009/03/30 09:42:24  gamx
	fixed bug in animation: priority of "N" is higher than "O"

	Revision 1.29  2008/11/12 08:27:45  gthomas
	-removed function calls to _ButtonUserAccessGcs and _ButtonUserAccessUn

	Revision 1.28  2008/10/20 14:12:47  gthomas
	-Removed  the check of (g_bSystemConnected) in unGenericObject_WidgetMFCDisconnection() function.

	Revision 1.27  2008/10/20 13:53:13  gthomas
	-removed XX_EventList_Configure_YY functions

	Revision 1.26  2008/10/15 14:53:37  gthomas
	-Removed the <if statement> of the dpConnect in _ButtonRegisterCB function
	to fix problem with select/deselect button animation.

	Revision 1.25  2008/09/29 08:53:57  gthomas
	Reviewed gcsMassFlowController_RangeHandlingCB fct.

	Revision 1.24  2008/07/23 08:53:08  gthomas
	added exceptionInfo in _ProvideAccessToButtons function

	Revision 1.23  2008/07/18 07:16:55  gthomas
	added new Access control functionality for unicos-4-1
	modified  xx_ButtonUserAccess function.

	Revision 1.22  2008/06/18 14:54:51  aabadz
	Changes done following the recomendations from AB-CO (from Herve)

	Revision 1.21  2008/04/25 10:22:46  aabadz
	Fixed problem that we had when the object had not connection to PLC

	Revision 1.20  2008/03/17 10:52:36  aabadz
	Fixed a problem found in animation when we have not database.

	Revision 1.19  2008/01/25 09:38:35  gamx
	an extra exception handling case has been added

	Revision 1.17  2008/01/16 15:57:13  aabadz
	Some sintactic errors fixed

	Revision 1.16  2008/01/15 11:00:33  aabadz
	Modifed ObjectListGetValueTime because the code that we had before return nothing instead of  the time and the value

	Revision 1.15  2008/01/11 12:58:57  gamx
	messages of errors have been improved

	Revision 1.14  2008/01/11 10:19:00  gamx
	a new function is introduced - AnimatePlcDisconnection()

	Revision 1.13  2008/01/10 15:46:07  gamx
	Fixed bug in "ProvideAccessToButtons".

	Revision 1.12  2007/12/09 20:37:18  gamx
	changes after code review:
	- changes to be consistent with naming convention
	- error checks
	- some functions are splited
	- logic changes to check device connection status

	Revision 1.11  2007/10/30 16:22:03  gthomas
	Introduced the CVS keyword $Log to track changes made in the library

*/
/*
Internal Functions :
	gcsMassFlowController_ObjectListGetValueTime (string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
	gcsMassFlowController_AcknowledgeAlarm (string sDpName, string sDpType, dyn_string &dsNeedAck)
	gcsMassFlowController_MenuConfiguration (string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
	gcsMassFlowController_HandleMenu (string sDeviceName, string sDpType, dyn_string menuList, int menuAnswer)
	gcsMassFlowController_FaceplateStatusRegisterCB (string sDp, bool bSystemConnected)
	gcsMassFlowController_FaceplateConnect (string sDeviceName)
	gcsMassFlowController_FaceplateDisconnect (string sDeviceName)
	gcsMassFlowController_FaceplateStatusAnimationCB_NoAlarm (string sStsReg01, bit32 bit32StsReg01, string sDp2, ... )
	gcsMassFlowController_FaceplateStatusAnimationCB (string sStsReg01, bit32 bit32StsReg01, string sDp2, ... )
	gcsMassFlowController_AnimatePlcDisconnection()
	gcsMassFlowController_ButtonRegisterCB (string sDp, bool bSystemConnected)
	gcsMassFlowController_ButtonAnimationCB (string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager, string sDpStsReg01, bit32 stsReg01Value, string sDpStsReg01Bad, bool bStsReg01Bad)
	gcsMassFlowController_ButtonUserAccess (string sDpName, string sDpType, dyn_string &dsAccess)
	gcsMassFlowController_UserLoginGetButtonState (string sDeviceName, string sType, dyn_string &dsReturnData)
	gcsMassFlowController_ButtonSetState (string sDeviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
	gcsMassFlowController_ButtonDisconnection ()
	gcsMassFlowController_WidgetRegisterCB (string sDp, bool bSystemConnected)
	gcsMassFlowController_WidgetConnect (string sDeviceName, string sPlcName, string sSystemName)
	gcsMassFlowController_WidgetDisconnect (string sDeviceName, string sPlcName, string sSystemName)
	gcsMassFlowController_WidgetAnimationCB (string sDpLock, bool bLocked, string sDpManager, string sSelectedManager, ... )
	gcsMassFlowController_WidgetDisconnection (string sWidgetType)
	unGenericObject_WidgetMFCDisconnection ()
	gcsMassFlowController_EventList_Configure_Init (dyn_string &exceptionInfo, const bool bUpdate=false)
	gcsMassFlowController_EventList_Configure_Clic (int iRow, string sColumn, string sValue, dyn_string &exceptionInfo)
	gcsMassFlowController_EventList_Configure_Save (dyn_string &exceptionInfo)
	gcsMassFlowController_CheckParametersAccess (string sUser, string sDeviceName, bool &granted)
	gcsMassFlowController_DisplayLabelFromInt (int iValue, int iMode, string sShape, string sColor, bool bInvalid)
	gcsMassFlowController_getLabelsFromDP (string sDP, dyn_string &ds, dyn_int &di)
	bool  gcsMassFlowController_getStringFromLabels (dyn_string ds, dyn_int di, int index, string &label)
*/


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsMassFlowController_ObjectListGetValueTime
/**
Function called from the objectList to return the time and value

	@param[in]	sDeviceName		Device name
	@param[in]	sDeviceType		Unicos object
	@param[out]	dsReturnData	Return value
*/

gcsMassFlowController_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
	float value;
	string sTime, sStatus,sColor;
	bool bBad;

	dpGet(sDeviceName+".ProcessInput.TotSt", value,
	      sDeviceName+".ProcessInput.TotSt:_online.._stime", sTime,
	      sDeviceName+".ProcessInput.TotSt:_online.._bad", bBad);

	dsReturnData[1] = sTime;
	dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(sDeviceName+".ProcessInput.TotSt"), value) + " "
							+ dpGetUnit(sDeviceName+".ProcessInput.TotSt");

	// Optionally (when no stsReg* exists)
	// Status
	dsReturnData[3] = bBad;

	// Status to visualize
	sStatus="OK";
	dsReturnData[4] = sStatus;
	// Colors
	sColor="unEventList_ForeColor";
	dsReturnData[5]=sColor;
}

//---------------------------------------------------------------------------------------------------------------------------------------
// gcsMassFlowController_AcknowledgeAlarm
/**
Acknowledge AnalogDelayedAlarm alarms
	The PLC acknowledgement is specified using UN_ACKNOWLEDGE_PLC string

	@param[in]	 sDpName	Device name
	@param[in]	 sDpType	Unicos object
	@param[out]	 dsNeedAck	DPEs names that need acknowledgement.
*/
gcsMassFlowController_AcknowledgeAlarm(string sDpName, string sDpType, dyn_string &dsNeedAck)
{
	dsNeedAck = makeDynString("ProcessInput.StartISt", "ProcessInput.StopISt", UN_ACKNOWLEDGE_PLC);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsMassFlowController_MenuConfiguration
/**
Configuration of popup menu

	@param[in]  sDpName Device name
	@param[in]	sDpType Unicos object
	@param[in]	dsAccessOk	List of authorized action
	@param[out] menuList Configuration of menu. Use defined constants UN_POPUPMENU_***
*/
gcsMassFlowController_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &dsMenuList)
{
	dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, UN_POPUPMENU_ACK_TEXT,
		UN_POPUPMENU_FACEPLATE_TEXT, UN_POPUPMENU_TREND_TEXT);

	unGenericObject_addSelectToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, dsMenuList);
	unGenericObject_addUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, dsMenuList);
	unGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, dsMenuList);
	unGenericObject_addTrendActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, dsMenuList);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsMassFlowController_HandleMenu
/**
Handle the answer of the popup menu

	@param[in]	sDeviceName 	The device dp name
	@param[in]	sDpType 	The device dp type
	@param[in]	menuList 	List of requested action
	@param[in]	menuAnswer 	Selected action from the menuList
*/
gcsMassFlowController_HandleMenu(string sDeviceName, string sDpType, dyn_string dsMenuList, int iMenuAnswer)
{
	dyn_string exceptionInfo;
	gcsGenericObject_HandleEventMaskMenu(sDeviceName, iMenuAnswer, makeDynString(".ProcessInput.evStsReg01"), exceptionInfo);
	if (dynlen(exceptionInfo) <= 0) unGenericObject_HandleUnicosMenu(sDeviceName, sDpType, dsMenuList, iMenuAnswer, exceptionInfo);

	//in case of errors in _HandleEventMaskMenu or _HandleUnicosMenu
	if (dynlen(exceptionInfo) > 0) unSendMessage_toExpertException("Right Click", exceptionInfo);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsMassFlowController_FaceplateStatusRegisterCB
/**
Register faceplate callback function

	@param[in]	sSystemStatus 		Parameters of the distributed system
	@param[in] 	bSystemConnected 	Flag
*/
gcsMassFlowController_FaceplateStatusRegisterCB(string sSystemStatus, bool bSystemConnected)
{
	string sDeviceName;
	string sSystemName;
	string sPlcName;
	string sPlcDpName;
	int iAction;
	dyn_string exceptionInfo;
	bool bRemote;

	sDeviceName = unGenericDpFunctions_getDpName($sDpName);
	if (sDeviceName)
	{
		sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
		unDistributedControl_isRemote(bRemote, sSystemName);

		if(bRemote) g_bSystemConnected = bSystemConnected;
		else g_bSystemConnected = true;

		sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);
		gcsConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
		if (dynlen(exceptionInfo) <= 0) g_bUnSystemAlarmPlc = true;
		else g_bUnSystemAlarmPlc = false;

		unGenericObject_Connection(sDeviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);

		switch (iAction)
		{
			case UN_ACTION_DISCONNECT:
				gcsMassFlowController_AnimatePlcDisconnection();
				break;
			case UN_ACTION_DPCONNECT:
				gcsMassFlowController_FaceplateConnect (sDeviceName);
				break;
			case UN_ACTION_DPDISCONNECT_DISCONNECT:
				gcsMassFlowController_FaceplateDisconnect (sDeviceName);
				gcsMassFlowController_AnimatePlcDisconnection();
				break;
			case UN_ACTION_NOTHING:
			default:
				break;
		}

	}
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsMassFlowController_FaceplateConnect
/**
Connects callback functions to animate a faceplate.
As it is not possible to pass constant values in callback functions and to get them inside callback function,
global constants are used.
	@param[in]	sDeviceName	Device DP name
*/
gcsMassFlowController_FaceplateConnect(string sDeviceName)
{
	int iRes = -1;
	int iAlertType1, iAlertType2;
	int iAlertRes;
	dyn_string exceptionInfo;
	string sPlcName, sSystemName;
	string sPlcDpName;

	sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
	g_bCallbackConnected = false;

	sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);
	gcsConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0) g_bUnSystemAlarmPlc = true;
	else g_bUnSystemAlarmPlc = false;

	//to be checked if StartISt and StopISt are used
	iAlertRes = dpGet(sDeviceName + ".ProcessInput.StartISt:_alert_hdl.._type", iAlertType1, sDeviceName + ".ProcessInput.StopISt:_alert_hdl.._type", iAlertType2);
	if (iAlertRes >= 0) g_bBoolAlert = ((iAlertType1 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType2 == DPCONFIG_ALERT_BINARYSIGNAL));
	else
	{
		dynAppend(exceptionInfo, "gcsMassFlowController_FaceplateConnect: problems to get alert handling data, object:" + sDeviceName);
	}

	if (g_bUnSystemAlarmPlc)
	{
		g_sApSpStFormat = dpGetFormat(sDeviceName + ".ProcessInput.ApSpSt");
		g_sApSpStUnit = dpGetUnit(sDeviceName + ".ProcessInput.ApSpSt");
		g_sAuSpRStFormat = dpGetFormat(sDeviceName + ".ProcessInput.AuSpRSt");
		g_sAuSpRStUnit = dpGetUnit(sDeviceName + ".ProcessInput.AuSpRSt");
		g_sSpOVFormat = dpGetFormat(sDeviceName + ".ProcessInput.SpOV");
		g_sSpOVUnit = dpGetUnit(sDeviceName + ".ProcessInput.SpOV");
		g_sMaxFlStFormat = dpGetFormat(sDeviceName + ".ProcessInput.MaxFlSt");
		g_sMaxFlStUnit = dpGetUnit(sDeviceName + ".ProcessInput.MaxFlSt");
		g_sFlStFormat = dpGetFormat(sDeviceName + ".ProcessInput.FlSt");
		g_sFlStUnit = dpGetUnit(sDeviceName + ".ProcessInput.FlSt");
		g_sTotStFormat = dpGetFormat(sDeviceName + ".ProcessInput.TotSt");
		g_sTotStUnit = dpGetUnit(sDeviceName + ".ProcessInput.TotSt");

		gcsMassFlowController_getLabelsFromDP(sDeviceName + ".ProcessInput.CcSt", g_dsMFCalibrCurve, g_diMFCalibrCurve, exceptionInfo);
		gcsMassFlowController_getLabelsFromDP(sDeviceName + ".ProcessInput.TotMoSt", g_dsMFCTotalizerMode, g_diMFCTotalizerMode, exceptionInfo);
		gcsMassFlowController_getLabelsFromDP(sDeviceName + ".ProcessInput.DrMoSt", g_dsMFCDriveMode, g_diMFCDriveMode, exceptionInfo);

		//in case there are problems to extract states description, an error should be just reported.

		if (g_bBoolAlert)
		{
			iRes = dpConnect("gcsMassFlowController_FaceplateAnimationCB",
				 sDeviceName + ".ProcessInput.StsReg01", 	sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
				 sDeviceName + ".ProcessInput.ApSpSt",		sDeviceName + ".ProcessInput.ApSpSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.AuSpRSt",		sDeviceName + ".ProcessInput.AuSpRSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.SpOV",		sDeviceName + ".ProcessInput.SpOV:_online.._invalid",
				 sDeviceName + ".ProcessInput.DrMoSt",		sDeviceName + ".ProcessInput.DrMoSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.AuDrMRSt",	sDeviceName + ".ProcessInput.AuDrMRSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.DrMoOV",		sDeviceName + ".ProcessInput.DrMoOV:_online.._invalid",
				 sDeviceName + ".ProcessInput.TotMoSt",		sDeviceName + ".ProcessInput.TotMoSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.AuTotMoRSt",	sDeviceName + ".ProcessInput.AuTotMoRSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.TotMoOV",		sDeviceName + ".ProcessInput.TotMoOV:_online.._invalid",
				 sDeviceName + ".ProcessInput.CcSt",		sDeviceName + ".ProcessInput.CcSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.AuCcRSt",		sDeviceName + ".ProcessInput.AuCcRSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.CcOV",		sDeviceName + ".ProcessInput.CcOV:_online.._invalid",
				 sDeviceName + ".ProcessInput.FlSt",		sDeviceName + ".ProcessInput.FlSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.MaxFlSt",		sDeviceName + ".ProcessInput.MaxFlSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.TotSt",		sDeviceName + ".ProcessInput.TotSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state",
				 sDeviceName + ".ProcessInput.StopISt:_alert_hdl.._act_state",
				 sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
				 sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
				 sDeviceName + ".eventMask");
		}
		else
		{
			iRes = dpConnect("gcsMassFlowController_FaceplateAnimationCB_NoAlarm",
				 sDeviceName + ".ProcessInput.StsReg01", 	sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
				 sDeviceName + ".ProcessInput.ApSpSt",		sDeviceName + ".ProcessInput.ApSpSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.AuSpRSt",		sDeviceName + ".ProcessInput.AuSpRSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.SpOV",			sDeviceName + ".ProcessInput.SpOV:_online.._invalid",
				 sDeviceName + ".ProcessInput.DrMoSt",		sDeviceName + ".ProcessInput.DrMoSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.AuDrMRSt",		sDeviceName + ".ProcessInput.AuDrMRSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.DrMoOV",		sDeviceName + ".ProcessInput.DrMoOV:_online.._invalid",
				 sDeviceName + ".ProcessInput.TotMoSt",		sDeviceName + ".ProcessInput.TotMoSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.AuTotMoRSt",	sDeviceName + ".ProcessInput.AuTotMoRSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.TotMoOV",		sDeviceName + ".ProcessInput.TotMoOV:_online.._invalid",
				 sDeviceName + ".ProcessInput.CcSt",			sDeviceName + ".ProcessInput.CcSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.AuCcRSt",		sDeviceName + ".ProcessInput.AuCcRSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.CcOV",			sDeviceName + ".ProcessInput.CcOV:_online.._invalid",
				 sDeviceName + ".ProcessInput.FlSt",			sDeviceName + ".ProcessInput.FlSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.MaxFlSt",		sDeviceName + ".ProcessInput.MaxFlSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.TotSt",		sDeviceName + ".ProcessInput.TotSt:_online.._invalid",
				 sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
				 sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
				 sDeviceName + ".eventMask");
		}

		g_bCallbackConnected = (iRes >= 0);
	}
	else gcsMassFlowController_AnimatePlcDisconnection();
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsMassFlowController_FaceplateDisconnect
/**
Do faceplate dpDisconnect

	@param[in]	sDeviceName	Device DP name
*/
gcsMassFlowController_FaceplateDisconnect(string sDeviceName)
{
	int iRes = -1;
	string sPlcName, sSystemName;


	g_bCallbackConnected = false;

	if (g_bUnSystemAlarmPlc)
	{
		sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
		sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);

		if (g_bBoolAlert)
		{
			iRes = dpDisconnect("gcsMassFlowController_FaceplateAnimationCB",
				 sDeviceName + ".ProcessInput.StsReg01", 	sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
				 sDeviceName + ".ProcessInput.ApSpSt",		sDeviceName + ".ProcessInput.ApSpSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.AuSpRSt",		sDeviceName + ".ProcessInput.AuSpRSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.SpOV",		sDeviceName + ".ProcessInput.SpOV:_online.._invalid",
				 sDeviceName + ".ProcessInput.DrMoSt",		sDeviceName + ".ProcessInput.DrMoSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.AuDrMRSt",	sDeviceName + ".ProcessInput.AuDrMRSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.DrMoOV",		sDeviceName + ".ProcessInput.DrMoOV:_online.._invalid",
				 sDeviceName + ".ProcessInput.TotMoSt",		sDeviceName + ".ProcessInput.TotMoSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.AuTotMoRSt",	sDeviceName + ".ProcessInput.AuTotMoRSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.TotMoOV",		sDeviceName + ".ProcessInput.TotMoOV:_online.._invalid",
				 sDeviceName + ".ProcessInput.CcSt",		sDeviceName + ".ProcessInput.CcSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.AuCcRSt",		sDeviceName + ".ProcessInput.AuCcRSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.CcOV",		sDeviceName + ".ProcessInput.CcOV:_online.._invalid",
				 sDeviceName + ".ProcessInput.FlSt",		sDeviceName + ".ProcessInput.FlSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.MaxFlSt",		sDeviceName + ".ProcessInput.MaxFlSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.TotSt",		sDeviceName + ".ProcessInput.TotSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state",
				 sDeviceName + ".ProcessInput.StopISt:_alert_hdl.._act_state",
				 sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
				 sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
				 sDeviceName + ".eventMask");
		}
		else
		{
			iRes = dpDisconnect("gcsMassFlowController_FaceplateAnimationCB_NoAlarm",
				 sDeviceName + ".ProcessInput.StsReg01", 	sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
				 sDeviceName + ".ProcessInput.ApSpSt",		sDeviceName + ".ProcessInput.ApSpSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.AuSpRSt",		sDeviceName + ".ProcessInput.AuSpRSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.SpOV",		sDeviceName + ".ProcessInput.SpOV:_online.._invalid",
				 sDeviceName + ".ProcessInput.DrMoSt",		sDeviceName + ".ProcessInput.DrMoSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.AuDrMRSt",		sDeviceName + ".ProcessInput.AuDrMRSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.DrMoOV",		sDeviceName + ".ProcessInput.DrMoOV:_online.._invalid",
				 sDeviceName + ".ProcessInput.TotMoSt",		sDeviceName + ".ProcessInput.TotMoSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.AuTotMoRSt",	sDeviceName + ".ProcessInput.AuTotMoRSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.TotMoOV",		sDeviceName + ".ProcessInput.TotMoOV:_online.._invalid",
				 sDeviceName + ".ProcessInput.CcSt",			sDeviceName + ".ProcessInput.CcSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.AuCcRSt",		sDeviceName + ".ProcessInput.AuCcRSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.CcOV",			sDeviceName + ".ProcessInput.CcOV:_online.._invalid",
				 sDeviceName + ".ProcessInput.FlSt",			sDeviceName + ".ProcessInput.FlSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.MaxFlSt",		sDeviceName + ".ProcessInput.MaxFlSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.TotSt",		sDeviceName + ".ProcessInput.TotSt:_online.._invalid",
				 sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
				 sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
				 sDeviceName + ".eventMask");
		}
		g_bCallbackConnected = !(iRes >= 0);
	}

}








//---------------------------------------------------------------------------------------------------------------------------------------

// gcsMassFlowController_FaceplateAnimationCB
/**

*/
gcsMassFlowController_FaceplateAnimationCB(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
		 string sDpApSpSt, float fApSpSt, string sDp4, bool bApSpStInvalid,
		 string sDpAuSpRSt, float fAuSpRSt, string sDp6, bool bAuSpRStInvalid,
		 string sDpSpOV, float fSpOV, string sDp8, bool bSpOVInvalid,
		 string sDpDrMoSt, int iDrMoSt, string sDp10, bool bDrMoStInvalid,
		 string sDpAuDrMRSt, int iAuDrMRSt, string sDp12, bool bAuDrMRStInvalid,
		 string sDpDrMoOV, int iDrMoOV, string sDp14, bool bDrMoOVInvalid,
		 string sDpTotMoSt, int iTotMoSt, string sDp16, bool bTotMoStInvalid,
		 string sDpAuTotMoRSt, int iAuTotMoRSt, string sDp18, bool bAuTotMoRStInvalid,
		 string sDpTotMoOV, int iTotMoOV, string sDp20, bool bTotMoOVInvalid,
		 string sDpCcSt, int iCcSt, string sDp22, bool bCcStInvalid,
		 string sDpAuCcRSt, int iAuCcRSt, string sDp24, bool bAuCcRStInvalid,
		 string sDpCcOV, int iCcOV, string sDp26, bool bCcOVInvalid,
		 string sDpFlSt, float fFlSt, string sDp28, bool bFlStInvalid,
		 string sDpMaxFlSt, float fMaxFlSt, string sDp30, bool bMaxFlStInvalid,
		 string sDpTotSt, float fTotSt, string sDp32, bool bTotStInvalid,
		 string sDp33, int iStartIStState, string sDp34, int iStopIStState,
		 string sDpAl, int iValAlarm, string sDpEnabled, bool bSystemIntegrityAlarmEnabled,
		 string sDpEvent, int iMask)
{
			gcsMassFlowController_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid,
		 							 sDpApSpSt, fApSpSt, sDp4, bApSpStInvalid,
		 							 sDpAuSpRSt, fAuSpRSt, sDp6, bAuSpRStInvalid,
		 							 sDpSpOV, fSpOV, sDp8, bSpOVInvalid,
		 							 sDpDrMoSt, iDrMoSt, sDp10, bDrMoStInvalid,
		 							 sDpAuDrMRSt, iAuDrMRSt, sDp12, bAuDrMRStInvalid,
		 							 sDpDrMoOV, iDrMoOV, sDp14, bDrMoOVInvalid,
		 							 sDpTotMoSt, iTotMoSt, sDp16, bTotMoStInvalid,
		 							 sDpAuTotMoRSt, iAuTotMoRSt, sDp18, bAuTotMoRStInvalid,
		 							 sDpTotMoOV, iTotMoOV, sDp20, bTotMoOVInvalid,
		 							 sDpCcSt, iCcSt, sDp22, bCcStInvalid,
		 							 sDpAuCcRSt, iAuCcRSt, sDp24, bAuCcRStInvalid,
		 							 sDpCcOV, iCcOV, sDp26, bCcOVInvalid,
		 							 sDpFlSt, fFlSt, sDp28, bFlStInvalid,
		 							 sDpMaxFlSt, fMaxFlSt, sDp30, bMaxFlStInvalid,
		 							 sDpTotSt, fTotSt, sDp32, bTotStInvalid,
		 							 sDp33, iStartIStState, sDp34, iStopIStState,
		 							 sDpAl, iValAlarm, sDpEnabled, bSystemIntegrityAlarmEnabled,
		 							 sDpEvent, iMask);
}











//---------------------------------------------------------------------------------------------------------------------------------------

// gcsMassFlowController_FaceplateStatusAnimationCB_NoAlarm
/**
Animate faceplate in case of no alarms.
The second function for NoAlarms case is required as it is not possible to send constants to CB function.
*/
gcsMassFlowController_FaceplateStatusAnimationCB_NoAlarm(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
		 string sDpApSpSt, float fApSpSt, string sDp4, bool bApSpStInvalid,
		 string sDpAuSpRSt, float fAuSpRSt, string sDp6, bool bAuSpRStInvalid,
		 string sDpSpOV, float fSpOV, string sDp8, bool bSpOVInvalid,
		 string sDpDrMoSt, int iDrMoSt, string sDp10, bool bDrMoStInvalid,
		 string sDpAuDrMRSt, int iAuDrMRSt, string sDp12, bool bAuDrMRStInvalid,
		 string sDpDrMoOV, int iDrMoOV, string sDp14, bool bDrMoOVInvalid,
		 string sDpTotMoSt, int iTotMoSt, string sDp16, bool bTotMoStInvalid,
		 string sDpAuTotMoRSt, int iAuTotMoRSt, string sDp18, bool bAuTotMoRStInvalid,
		 string sDpTotMoOV, int iTotMoOV, string sDp20, bool bTotMoOVInvalid,
		 string sDpCcSt, int iCcSt, string sDp22, bool bCcStInvalid,
		 string sDpAuCcRSt, int iAuCcRSt, string sDp24, bool bAuCcRStInvalid,
		 string sDpCcOV, int iCcOV, string sDp26, bool bCcOVInvalid,
		 string sDpFlSt, float fFlSt, string sDp28, bool bFlStInvalid,
		 string sDpMaxFlSt, float fMaxFlSt, string sDp30, bool bMaxFlStInvalid,
		 string sDpTotSt, float fTotSt, string sDp32, bool bTotStInvalid,
		 string sDpAl, int iValAlarm, string sDpEnabled, bool bPlcEnabled,
		 string sDpEvent, int iMask)
{
 	gcsMassFlowController_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid,
		 sDpApSpSt, fApSpSt, sDp4, bApSpStInvalid,
		 sDpAuSpRSt, fAuSpRSt, sDp6, bAuSpRStInvalid,
		 sDpSpOV, fSpOV, sDp8, bSpOVInvalid,
		 sDpDrMoSt, iDrMoSt, sDp10, bDrMoStInvalid,
		 sDpAuDrMRSt, iAuDrMRSt, sDp12, bAuDrMRStInvalid,
		 sDpDrMoOV, iDrMoOV, sDp14, bDrMoOVInvalid,
		 sDpTotMoSt, iTotMoSt, sDp16, bTotMoStInvalid,
		 sDpAuTotMoRSt, iAuTotMoRSt, sDp18, bAuTotMoRStInvalid,
		 sDpTotMoOV, iTotMoOV, sDp20, bTotMoOVInvalid,
		 sDpCcSt, iCcSt, sDp22, bCcStInvalid,
		 sDpAuCcRSt, iAuCcRSt, sDp24, bAuCcRStInvalid,
		 sDpCcOV, iCcOV, sDp26, bCcOVInvalid,
		 sDpFlSt, fFlSt, sDp28, bFlStInvalid,
		 sDpMaxFlSt, fMaxFlSt, sDp30, bMaxFlStInvalid,
		 sDpTotSt, fTotSt, sDp32, bTotStInvalid,
		 "", 0, "", 0,
		 sDpAl, iValAlarm, sDpEnabled, bPlcEnabled, sDpEvent, iMask);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsMassFlowController_FaceplateStatusAnimationCB
/**
Animate faceplate
*/
gcsMassFlowController_FaceplateStatusAnimationCB(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
		 string sDpApSpSt, float fApSpSt, string sDp4, bool bApSpStInvalid,
		 string sDpAuSpRSt, float fAuSpRSt, string sDp6, bool bAuSpRStInvalid,
		 string sDpSpOV, float fSpOV, string sDp8, bool bSpOVInvalid,
		 string sDpDrMoSt, int iDrMoSt, string sDp10, bool bDrMoStInvalid,
		 string sDpAuDrMRSt, int iAuDrMRSt, string sDp12, bool bAuDrMRStInvalid,
		 string sDpDrMoOV, int iDrMoOV, string sDp14, bool bDrMoOVInvalid,
		 string sDpTotMoSt, int iTotMoSt, string sDp16, bool bTotMoStInvalid,
		 string sDpAuTotMoRSt, int iAuTotMoRSt, string sDp18, bool bAuTotMoRStInvalid,
		 string sDpTotMoOV, int iTotMoOV, string sDp20, bool bTotMoOVInvalid,
		 string sDpCcSt, int iCcSt, string sDp22, bool bCcStInvalid,
		 string sDpAuCcRSt, int iAuCcRSt, string sDp24, bool bAuCcRStInvalid,
		 string sDpCcOV, int iCcOV, string sDp26, bool bCcOVInvalid,
		 string sDpFlSt, float fFlSt, string sDp28, bool bFlStInvalid,
		 string sDpMaxFlSt, float fMaxFlSt, string sDp30, bool bMaxFlStInvalid,
		 string sDpTotSt, float fTotSt, string sDp32, bool bTotStInvalid,
		 string sDp33, int iStartIStState, string sDp34, int iStopIStState,
		 string sDpAl, int iValAlarm, string sDpEnabled, bool bSystemIntegrityAlarmEnabled,
		 string sDpEvent, int iMask)
{
// Animate display values. These values should be float values
	if(g_bSystemConnected)
		unGenericObject_DisplayValue(g_sApSpStFormat, g_sApSpStUnit, fApSpSt, "StatusSetPoint", "unDisplayValue_Status",
			bApSpStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_DisplayValue(g_sAuSpRStFormat, g_sAuSpRStUnit, fAuSpRSt, "RequestAutoSetPoint", "unDisplayValue_Request",
			bAuSpRStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_DisplayValue(g_sSpOVFormat, g_sSpOVUnit, fSpOV, "RequestActiveSetPoint", "unDisplayValue_Output",
			bSpOVInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_DisplayValue(g_sMaxFlStFormat, g_sMaxFlStUnit, fMaxFlSt, "StatusTemperature", "unDisplayValue_Status",
			bMaxFlStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_DisplayValue(g_sFlStFormat, g_sFlStUnit, fFlSt, "StatusFlow", "unDisplayValue_Status",
			bFlStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_DisplayValue(g_sTotStFormat, g_sTotStUnit, fTotSt, "StatusTotSt", "unDisplayValue_Status",
			bTotStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
// DriveMode
	if(g_bSystemConnected)
		gcsMassFlowController_DisplayLabelFromInt(iDrMoSt, GCS_MFC_DRIVEMODE, "StatusDriveMode", "unDisplayValue_Status",
			bDrMoStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		gcsMassFlowController_DisplayLabelFromInt(iAuDrMRSt, GCS_MFC_DRIVEMODE, "RequestAutoDrive", "unDisplayValue_Request",
			bAuDrMRStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		gcsMassFlowController_DisplayLabelFromInt(iDrMoOV, GCS_MFC_DRIVEMODE, "RequestActiveDrive", "unDisplayValue_Output",
			bDrMoOVInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
//	CC
	if(g_bSystemConnected)
		gcsMassFlowController_DisplayLabelFromInt(iCcSt, GCS_MFC_CALIBRATION, "StatusCalibrationCurve", "unDisplayValue_Status",
			bCcStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		gcsMassFlowController_DisplayLabelFromInt(iAuCcRSt, GCS_MFC_CALIBRATION, "RequestAutoC", "unDisplayValue_Request",
			bAuCcRStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		gcsMassFlowController_DisplayLabelFromInt(iCcOV, GCS_MFC_CALIBRATION, "RequestActiveC", "unDisplayValue_Output",
			bCcOVInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
// Animate colorboxes
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("OperationMode_Auto", getBit(bit32StsReg01, UN_STSREG01_AUMOST), 1, "unFaceplate_OperationModeActive",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("OperationMode_Manual", getBit(bit32StsReg01, UN_STSREG01_MMOST), 1, "unFaceplate_OperationModeActive",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("OperationMode_Forced", getBit(bit32StsReg01, UN_STSREG01_FOMOST), 1, "unFaceplate_OperationModeActive",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("Warning_IOError", getBit(bit32StsReg01, UN_STSREG01_IOERRORW), 1, "unFaceplate_WarningActive",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("Warning_IOSimulated", getBit(bit32StsReg01, UN_STSREG01_IOSIMUW), 1, "unFaceplate_WarningActive",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("Warning_ManualAuto", getBit(bit32StsReg01, UN_STSREG01_AUMRW), 1, "unFaceplate_WarningActive",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("Warning_PositionWarning", getBit(bit32StsReg01, UN_STSREG01_POSALW), 1, "unFaceplate_WarningActive",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("Status_Failsafe", getBit(bit32StsReg01, UN_STSREG01_PFSPOSONST_MFC), getBit(bit32StsReg01, UN_STSREG01_PFSPOSONST_MFC), "unFaceplate_StatusActive",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
// Animate alarm colorboxes
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimateAlarm("Alarm_StartISt", iStartIStState, g_bBoolAlert,
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimateAlarm("Alarm_StopISt", iStopIStState, g_bBoolAlert,
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("Alarm_UnAck", getBit(bit32StsReg01, UN_STSREG01_ALUNACK), 1, "unFaceplate_AlarmNotAck",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("eventState", iMask, 1, "",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value),
			"unFaceplate_OperationModeActive");
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsMassFlowController_AnimatePlcDisconnection
/**
Animate faceplate disconnection
*/
gcsMassFlowController_AnimatePlcDisconnection()
{
	unGenericObject_ColorBoxDisconnect(makeDynString("Alarm_StartISt", "Alarm_StopISt","Alarm_UnAck", "OperationMode_Auto",
		 "OperationMode_Forced", "OperationMode_Manual", "Warning_IOError",
		 "Warning_IOSimulated", "Warning_ManualAuto", "Warning_PositionWarning",
		 "Status_Failsafe", "eventState"));

	unGenericObject_DisplayValueDisconnect(makeDynString("StatusTemperature", "StatusFlow", "StatusTotSt",
		 "StatusSetPoint", "StatusDriveMode",
		 "StatusCalibrationCurve", "RequestAutoC", "RequestActiveC",
		 "RequestAutoDrive",
		 "RequestActiveDrive", "RequestAutoSetPoint", "RequestActiveSetPoint"));
}


//---------------------------------------------------------------------------------------------------------------------
// gcsMassFlowController_ButtonRegisterCB
/**
Register callback function

	@param[in]	sSystemStatus		Status of a distributed system.
	@param[in] 	bSystemConnected 	Flag if system is connected or not.
*/
gcsMassFlowController_ButtonRegisterCB(string sSystemStatus, bool bSystemConnected)
{
	string sDeviceName, sDpType, sFunction;
	int iAction, iRes, iManReg01;
	int iManRegSet;
	dyn_string exceptionInfo, dsFunctions;

	sDeviceName = unGenericDpFunctions_getDpName($sDpName);

	if (sDeviceName)
	{
		unGenericObject_Connection(sDeviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);

		switch (iAction)
		{
				case UN_ACTION_DISCONNECT:
					gcsMassFlowController_ButtonDisconnection();
					break;
				case UN_ACTION_DPCONNECT:
					// Send 0 to ManReg01 if necessary
					iRes = dpGet(sDeviceName + ".ProcessOutput.ManReg01", iManReg01);
					if ((iRes < 0) || (iManReg01 != 0))
					{
						iManRegSet = dpSet(sDeviceName + ".ProcessOutput.ManReg01", 0);
						if (iManRegSet < 0)
						{
							dynAppend(exceptionInfo, "gcsMassFlowController_ButtonRegisterCB: problems to set value for: " + sDeviceName + ".ProcessOutput.ManReg01");
							break;
						}
					}


					// call the gcsMassFlowControllerUserAccess to get the list of allowed buttons.
					sDpType=GCS_CONFIG_MFC_DPT_NAME;
					unGenericObject_GetFunctions(sDpType, dsFunctions, exceptionInfo);

					if (dynlen(exceptionInfo) <= 0) gcsMassFlowController_ProvideAccessToButtons(sDeviceName, sDpType, dsFunctions, exceptionInfo);
					gcsMassFlowController_ButtonsConnect(sDeviceName);
				
					break;
				case UN_ACTION_DPDISCONNECT_DISCONNECT:
					gcsMassFlowController_ButtonsDisconnect(sDeviceName);
					gcsMassFlowController_ButtonDisconnection();
					break;
				case UN_ACTION_NOTHING:
					break;
				default:
					break;

		}
	}
	else dynAppend(exceptionInfo, "gcsMassFlowController_ButtonRegisterCB: device name is empty");

}










//---------------------------------------------------------------------------------------------------------------------------------------
// gcsMassFlowController_ProvideAccessToButtons
/**
Defines the list of buttons available for a user. The result is passed in global variable.
	@param[in]	sDeviceName	Name of the device
	@param[in]	sDpType		Device Type
	@param[in] 	dsFunctions	A list of functions for a device, specified in device internal configuration.
	@param[out]	exceptionInfo	A list of exceptions.
*/
gcsMassFlowController_ProvideAccessToButtons(string sDeviceName, string sDpType, dyn_string dsFunctions, dyn_string &exceptionInfo)
{
	string sFunction;
	dyn_string dsUserAccess;

	if (dynlen(dsFunctions) >= UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS)
		sFunction = dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS];
	else
	{
		dynAppend(exceptionInfo, "gcsMassFlowController_ButtonRegisterCB: the list of functions is not correct - number of functions is less than expected");
		return;
	}

	if (sFunction != "")
	{
		evalScript(dsUserAccess, "dyn_string main(string sDeviceNameTemp, string sDpTypeTemp) {" +
					 "dyn_string dsUserAccessReturn;" +
					 "if (isFunctionDefined(\"" + sFunction  + "\"))" +
					 "    {" +
					 	sFunction  + "(sDeviceNameTemp, sDpTypeTemp, dsUserAccessReturn);" +
					 "    }" +
					 "return dsUserAccessReturn; }", makeDynString(), sDeviceName, sDpType);
		g_dsUserAccess=dsUserAccess;
	}
	else
		fwException_raise(exceptionInfo,"ERROR", "gcsMassFlowController_ButtonRegisterCB:" + getCatStr("unGeneration","UNKNOWNFUNCTION"), "EMPTY");
}







//---------------------------------------------------------------------------------------------------------------------------------------
// gcsMassFlowController_ButtonsConnect
/**
Connects callback function to animate buttons states
	@param[in] 	sDeviceName	The name of the device
*/
gcsMassFlowController_ButtonsConnect(string sDeviceName)
{
	int iRes = -1;

	iRes = dpConnect("gcsMassFlowController_ButtonAnimationCB",
				sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
				sDeviceName + ".statusInformation.selectedManager",
				sDeviceName + ".ProcessInput.StsReg01",
				sDeviceName + ".ProcessInput.StsReg01:_online.._aut_inv");

	g_bCallbackConnected = (iRes >= 0);
}












//---------------------------------------------------------------------------------------------------------------------------------------
// gcsMassFlowController_ButtonsDisconnect
/**
Disconnects callback function to animate buttons states
	@param[in] 	sDeviceName	The name of the device
*/
gcsMassFlowController_ButtonsDisconnect(string sDeviceName)
{
	int iRes = -1;

	iRes = dpDisconnect("gcsMassFlowController_ButtonAnimationCB",
				sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
				sDeviceName + ".statusInformation.selectedManager",
				sDeviceName + ".ProcessInput.StsReg01",
				sDeviceName + ".ProcessInput.StsReg01:_online.._aut_inv");

	g_bCallbackConnected = !(iRes >= 0);
}










//---------------------------------------------------------------------------------------------------------------------------------------
// gcsMassFlowController_ButtonAnimationCB
/**
Animate buttons

@note This function works with an unicos button panel and needs global variables declared in this panel -
		"g_dsUserAccess" (dyn_string). Button identifier is added to this variable if user has access.
*/
gcsMassFlowController_ButtonAnimationCB(string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager,
									    string sDpStsReg01, bit32 stsReg01Value, string sDpStsReg01Bad, bool bStsReg01Bad)
{
	gcsMassFlowController_ButtonSetState(unGenericDpFunctions_getDpName(sDpStsReg01), GCS_CONFIG_MFC_DPT_NAME, g_dsUserAccess,
							 	 makeDynString(bLocked, sSelectedManager, stsReg01Value, bStsReg01Bad, 0, 0));
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsMassFlowController_ButtonUserAccess
/**
User access for buttons

	@param[in] 		sDpName	Device name
	@param[in] 		sDpType	dpType
	@param[out] 	dsAccess	List of allowed buttons
	
*/
gcsMassFlowController_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
	bool operator, expert, admin;
	dyn_bool dbPermissions;
	dyn_string exceptionInfo;
	string sOperAction, sExpAction, sAdminAction;
	bool bActionDefined = false;
	
	unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo);
	if((sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME) ||
	(sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME))
	bActionDefined=true;
	
	operator = dbPermissions[2];
	expert = dbPermissions[3];
	admin = dbPermissions[4];
	dsAccess = makeDynString();
	
	if (operator){
		if(bActionDefined)
			dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
		else
			dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_MANUAL_MODE,
							UN_FACEPLATE_BUTTON_ACK_ALARM, UN_FACEPLATE_BUTTON_SELECT,UN_FACEPLATE_BUTTON_MFC_SUBPANEL_APPLY));
	}
	if (expert){
		if(bActionDefined)
			dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
		else
			dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_FORCED_MODE, UN_FACEPLATE_MASKEVENT));
	}
	if(admin){
		if(bActionDefined)
			dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
        }
	
}

//---------------------------------------------------------------------------------------------------------------------------------------
// gcsMassFlowController_UserLoginGetButtonState
/**
Function is called before the button set state when a user logs in.

	@param[in]	sDeviceName		The device name
	@param[in]	sType			Device type <b>(Currently not used)</b>
	@param[out]	dsReturnData	The data are returned here
	@note	This library works with an unicos button panel and need global variables declared in this panel
		- g_dsUserAccess, dyn_string, button identifier is added to this variable if user access
		- $sDpName, string, the device DP name
		- g_sFaceplateButtonType, string, faceplate type
*/
gcsMassFlowController_UserLoginGetButtonState(string sDeviceName, string sType, dyn_string &dsReturnData)
{
	unGenericObject_UserCBGetButtonState(sDeviceName, sType, dsReturnData);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsMassFlowController_ButtonSetState
/**
Set button state

	@param[in]	 sDeviceName		The device name
	@param[in]	 sDpType		Device type
	@param[in]	 dsUserAccess	User access
	@param[in]	 dsData			List of various parameters
*/
gcsMassFlowController_ButtonSetState(string sDeviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
	bool bSelected, bButtonEnabled, bit1, bit2;
	dyn_string dsButtons;
	int i, iLength;
	string localManager;
	bool bLocked, bStsReg01Bad;
	string sSelectedUIManager;
	bit32 bit32StsReg01Value;

	if (dynlen(dsData) >= 4)
	{
		bLocked = dsData[1];
		sSelectedUIManager = dsData[2];
		bit32StsReg01Value = dsData[3];
		bStsReg01Bad = dsData[4];
	}


	dsButtons = makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_MANUAL_MODE,
							  UN_FACEPLATE_BUTTON_ACK_ALARM, UN_FACEPLATE_BUTTON_FORCED_MODE); // Each button except "Select" button
	iLength = dynlen(dsButtons);
	// 1. Selection state
	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedUIManager);
	bSelected = (localManager == "S");
	// 2. Button state
	for(i = 1; i <= iLength; i++)
		{
		bButtonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0);	// User access
		switch (dsButtons[i])
			{
			case UN_FACEPLATE_BUTTON_FORCED_MODE:
				bit1 = !(getBit(bit32StsReg01Value, UN_STSREG01_FOMOST) == 0) && !(bStsReg01Bad);
				bit2 = !(getBit(bit32StsReg01Value, UN_STSREG01_AUINHFMO) == 0) && !(bStsReg01Bad);
				bButtonEnabled = bButtonEnabled && bSelected && !(bit1 || bit2);
				break;
			case UN_FACEPLATE_BUTTON_MANUAL_MODE:
				bit1 = !(getBit(bit32StsReg01Value, UN_STSREG01_MMOST) == 0) && !(bStsReg01Bad);
				bButtonEnabled = bButtonEnabled && bSelected && !(bit1);
				break;
			case UN_FACEPLATE_BUTTON_AUTO_MODE:
				bit1 = !(getBit(bit32StsReg01Value, UN_STSREG01_AUMOST) == 0) && !(bStsReg01Bad);
				bit2 = !(getBit(bit32StsReg01Value, UN_STSREG01_AUINHMMO) == 0) && !(bStsReg01Bad);
				bButtonEnabled = bButtonEnabled && bSelected && !(bit1 || bit2);
				break;
			case UN_FACEPLATE_BUTTON_ACK_ALARM:
			default:
				break;
			}
		setValue(UN_FACEPLATE_BUTTON_PREFIX + dsButtons[i], "enabled", bButtonEnabled);
		}
// 3. Button Select state
	unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsMassFlowController_ButtonDisconnection
/**
Animate disconnection
*/
gcsMassFlowController_ButtonDisconnection()
{
	unGenericObject_ButtonDisconnect(makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_MANUAL_MODE,
		   UN_FACEPLATE_BUTTON_ACK_ALARM, UN_FACEPLATE_BUTTON_FORCED_MODE,
		   UN_FACEPLATE_BUTTON_SELECT));
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsMassFlowController_WidgetRegisterCB
/**
Register callback function

	@param[in]	sDp 	The device dp
	@param[in] 	bSystemConnected 	Flag
*/
gcsMassFlowController_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
	string sDeviceName, sSystemName, sPlcName;
	string sPlcDpName;
	int iAction;
	dyn_string exceptionInfo;
	bool bRemote;

	sDeviceName = unGenericDpFunctions_getWidgetDpName ($sIdentifier);
	if (sDeviceName == "") //in case of disconnection
	{
		sDeviceName = g_sDpName;
	}

	sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);

	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote) g_bSystemConnected = bSystemConnected;
	else g_bSystemConnected = true;

	sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);
	gcsConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0) g_bUnSystemAlarmPlc = true;
	else g_bUnSystemAlarmPlc = false;


	unGenericObject_Connection(sDeviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
	{
		case UN_ACTION_DISCONNECT:
			gcsMassFlowController_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPCONNECT:
			g_sDpName = sDeviceName;
			if(dpTypeName(sDeviceName) == GCS_CONFIG_MFC_DPT_NAME)
			{
				gcsMassFlowController_WidgetConnect(sDeviceName, sPlcName, sSystemName);
			}
			else{
				g_bUnSystemAlarmPlc = false;
				gcsMassFlowController_WidgetDisconnection(g_sWidgetType);
			}
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			gcsMassFlowController_WidgetDisconnect(sDeviceName, sPlcName, sSystemName);
			gcsMassFlowController_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_NOTHING:
			break;
		default:
			break;
	}

}


//-----------------------------------------------------------------------------------------------------------------------------------------
// gcsMassFlowController_WidgetConnect
/**
dpConnect to the DataPoint

	@param[in]	sDeviceName	Device DP name
	@param[in]	sPlcName	PLC name
	@param[in]	sSystemName	System name
*/
gcsMassFlowController_WidgetConnect(string sDeviceName, string sPlcName, string sSystemName)
{
	int iRes = -1;

	if (g_bUnSystemAlarmPlc)
	{
		g_sSpOVFormat = dpGetFormat(sDeviceName + ".ProcessInput.SpOV");
		g_sSpOVUnit = dpGetUnit(sDeviceName + ".ProcessInput.SpOV");
		g_sFlStFormat = dpGetFormat(sDeviceName + ".ProcessInput.FlSt");
		g_sFlStUnit = dpGetUnit(sDeviceName + ".ProcessInput.FlSt");

		iRes = dpConnect("gcsMassFlowController_WidgetAnimationCB",
			sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
			sDeviceName + ".statusInformation.selectedManager",
			sDeviceName + ".ProcessInput.StsReg01",
			sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
			sDeviceName + ".ProcessInput.FlSt",
			sDeviceName + ".ProcessInput.FlSt:_online.._invalid",
			sDeviceName + ".ProcessInput.SpOV",
			sDeviceName + ".ProcessInput.SpOV:_online.._invalid",
			sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
			sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
			sDeviceName + ".eventMask");
		g_bCallbackConnected = (iRes >= 0);
	}
	else gcsMassFlowController_WidgetDisconnection(g_sWidgetType);
}


//--------------------------------------------------------------------------------------------------------------------------
// gcsMassFlowController_WidgetDisconnect
/**
dpDisconnect from the DataPoint

	@param[in]	sDeviceName	Device DP name
	@param[in]	sPlcName	PLC name
	@param[in]	sSystemName	System name
*/
gcsMassFlowController_WidgetDisconnect(string sDeviceName, string sPlcName, string sSystemName)
{
	int iRes = -1;

	if (g_bUnSystemAlarmPlc)
	{
		string sPlcDpName = "";

		dyn_string dsPlcDpNameExceptions;
		gcsConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, dsPlcDpNameExceptions);

		if (dynlen(dsPlcDpNameExceptions) == 0)
		{
			iRes = dpDisconnect("gcsMassFlowController_WidgetAnimationCB",
				sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
				sDeviceName + ".statusInformation.selectedManager",
				sDeviceName + ".ProcessInput.StsReg01",
				sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
				sDeviceName + ".ProcessInput.FlSt",
				sDeviceName + ".ProcessInput.FlSt:_online.._invalid",
				sDeviceName + ".ProcessInput.SpOV",
				sDeviceName + ".ProcessInput.SpOV:_online.._invalid",
				sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
				sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
				sDeviceName + ".eventMask");
			g_bCallbackConnected = !(iRes >= 0);
		}
	}
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsMassFlowController_WidgetAnimationCB
/**
Animate widget
*/
gcsMassFlowController_WidgetAnimationCB(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
	   string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid,
	   string sDFlSt, float fFlSt, string sDp6, bool bFlStInvalid,
	   string sDpSpOV, float fSpOV, string sDp8, bool bSpOVInvalid,
	   string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled, string sEvtMask, int iMask)
{
	string value1, value2;
	string AlarmLetter = "", AlarmColor = "";
	string ControlLetter = "", ControlColor = "";
	string WarningLetter, WarningColor;
	string display1Color, display2Color, ColorSelect, Color, ColorBk, Color2Bk;
	bool bLockVisible;

	WarningLetter = UN_WIDGET_TEXT_INVALID; WarningColor = "unDataNotValid";
	Color = "unDataNotValid";
	ColorBk = "unWidget_Background"; Color2Bk = "unWidget_Background";

	if (!bStsReg01Invalid)
	{
		//modes animation
		Color = "unWidget_ControlStateAuto";
		if (getBit(bit32StsReg01, UN_STSREG01_MMOST) == 1) Color = "unWidget_ControlStateManualLocal";
		if (getBit(bit32StsReg01, UN_STSREG01_FOMOST) == 1) Color = "unWidget_ControlStateForced";

		// 1.3. Alarm (higher priority)
		if ((getBit(bit32StsReg01, UN_STSREG01_STARTIST) == 1)||(getBit(bit32StsReg01, UN_STSREG01_STOPIST) == 1))
			Color = "unFaceplate_AlarmActive";

		if (getBit(bit32StsReg01, UN_STSREG01_ALUNACK) == 1)
			Color = "unWidget_AlarmNotAck";

		ColorBk = Color; Color2Bk = "unWidget_Background";
		if (getBit(bit32StsReg01, UN_STSREG01_ONST) == 1 && getBit(bit32StsReg01, UN_STSREG01_OFFST) == 0)
		{
			Color2Bk = Color;
		}
		if (getBit(bit32StsReg01, UN_STSREG01_ONST) == 0 && getBit(bit32StsReg01, UN_STSREG01_OFFST) == 1)
		{
			ColorBk = "unWidget_Background"; Color2Bk = "unWidget_Background";
		}

		// 2. Alarm animation
		unGenericObject_WidgetAlarmTextAnimation(bit32StsReg01, makeDynInt(UN_STSREG01_STARTIST, UN_STSREG01_STOPIST),
			makeDynString(UN_WIDGET_TEXT_ALARM_STARTIST, UN_WIDGET_TEXT_ALARM_STOPIST), AlarmLetter, AlarmColor);

		// 3. Control state animation
		unGenericObject_WidgetControlStateAnimation(bit32StsReg01, makeDynInt(UN_STSREG01_MMOST, UN_STSREG01_FOMOST),
			makeDynString(UN_WIDGET_TEXT_CONTROL_MANUAL, UN_WIDGET_TEXT_CONTROL_FORCED), ControlLetter, ControlColor);

		// 4. Warning animation I/O Error, I/O Simulated, Manual Auto, Position Warning
		unGenericObject_WidgetWarningAnimation(bit32StsReg01, makeDynInt(UN_STSREG01_AUMRW, UN_STSREG01_POSALW, UN_STSREG01_IOSIMUW, UN_STSREG01_IOERRORW),
			makeDynString(UN_WIDGET_TEXT_WARNING_DEFAULT, UN_WIDGET_TEXT_WARNING_DEFAULT, UN_WIDGET_TEXT_WARNING_SIMU, UN_WIDGET_TEXT_WARNING_ERROR),
			WarningLetter, WarningColor);
	}

	
	//data quality animation
	if ((!bPlcEnabled) || (iAlValue > c_unSystemIntegrity_no_alarm_value))
	{
		WarningLetter = UN_WIDGET_TEXT_OLD_DATA;
		WarningColor = "unDataNotValid";
	}
	
	if (bStsReg01Invalid || bSpOVInvalid || bFlStInvalid)
	{
		WarningLetter = UN_WIDGET_TEXT_INVALID;
		WarningColor = "unDataNotValid";
	}	
	

	// 6. Animate analog values
	// 6.1. SpOV
	display1Color = "unDisplayValue_Output";
	if (bStsReg01Invalid || bSpOVInvalid) display1Color = "unDataNotValid";
	value1 = unGenericObject_FormatValue(g_sSpOVFormat, fSpOV) + " " + g_sSpOVUnit;

	//	 6.2. FlSt
	display2Color = "unDisplayValue_Status";
	if (bStsReg01Invalid || bFlStInvalid) display2Color = "unDataNotValid";
	value2 = unGenericObject_FormatValue(g_sFlStFormat, fFlSt) + " " + g_sFlStUnit;

	// 7. Invalid
	if (bSpOVInvalid || bFlStInvalid) // unWidgetHMI_invalid is run if bStsReg01Invalid
	{
		WarningLetter = UN_WIDGET_TEXT_INVALID; WarningColor = "unDataNotValid";
		Color = "unDataNotValid"; ColorBk = Color; Color2Bk = "";
		AlarmLetter = ""; AlarmColor = ""; ControlLetter = ""; ControlColor = "";
	}

	unGenericObject_WidgetSelectAnimation(bLocked, sSelectedManager, ColorSelect, bLockVisible);
	if(g_bSystemConnected)
	{
		setMultiValue("WarningText", "text", WarningLetter, "WarningText", "foreCol", WarningColor,
			"AlarmText", "text", AlarmLetter, "AlarmText", "foreCol", AlarmColor,
			"ControlStateText", "text", ControlLetter, "ControlStateText", "foreCol", ControlColor,
			"Body2", "foreCol", Color,  "Body2", "backCol", ColorBk, "Body2", "fill", "[solid]",
			"Body4", "foreCol", Color,  "Body4", "backCol", ColorBk, "Body4", "fill", "[solid]",
			"Body3", "foreCol", Color, "Body3", "backCol", Color2Bk, "Body3", "fill", "[solid]",
			"Body5", "foreCol", Color, "Body5", "backCol", Color2Bk, "Body5", "fill", "[solid]",
			"Body1", "foreCol", Color, "Body1", "backCol", "",
			"Line4", "foreCol", Color, "Line5", "foreCol", Color, "Line6", "foreCol", Color,
			"Display1", "text", value1, "Display1", "foreCol", display1Color,
			"Display2", "text", value2, "Display2", "foreCol", display2Color,
			"SelectArea", "foreCol", ColorSelect, "LockBmp", "visible", bLockVisible, "WidgetArea", "visible", true);
		// display event state
		if(iMask == 0)	eventState.visible	= true;
		else			eventState.visible	= false;
	}
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsMassFlowController_WidgetDisconnection
/**
Animate widget disconnection

	@param[in] sWidgetType Widget type. ex: "heater"
*/
gcsMassFlowController_WidgetDisconnection(string sWidgetType)
{
	unGenericObject_WidgetDisconnection(sWidgetType, GCS_CONFIG_MFC_DPT_NAME);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// unGenericObject_WidgetMFCDisconnection
/**
Animate Mass Flow Controller disconnection
*/
unGenericObject_WidgetMFCDisconnection()
{
		setMultiValue("Body1", "foreCol", "unDataNoAccess",
				"Body2","backCol","unWidget_Background", "Body2", "fill", "[solid]",
				"Body3","backCol","unWidget_Background", "Body3", "fill", "[solid]",
				"Body4","foreCol","unDataNoAccess", "Body4","backCol","unWidget_Background",  "Body4", "fill", "[solid]",
				"Body5","foreCol","unDataNoAccess", "Body5","backCol","unWidget_Background",  "Body5", "fill", "[solid]",
				"Line4","foreCol","unDataNoAccess", "Line5","foreCol","unDataNoAccess", "Line6","foreCol","unDataNoAccess",
				"Display1", "foreCol", "unDataNoAccess", "Display2", "foreCol", "unDataNoAccess");

		eventState.visible = false;
}

//---------------------------------------------------------------------------------------------------------------------------------------
// gcsMassFlowController_CheckParametersAccess
/**
Write ApSpSt min and max ranges, according to "temp" variable.

	@param[in]	sUser		Current user
	@param[in]	sDeviceName	Device DP name
	@param[out]	granted		Is access granted?
*/
gcsMassFlowController_CheckParametersAccess(string sUser, string sDeviceName, bool &bGranted)
{
	string sDomain;
	dyn_string dsParameters, exceptionInfo;
	bGranted = false;

	//	gcsAccessControl should be installed
	if(!isFunctionDefined("gcsAccessControlGeneric_isPrivelegeExist")) return;

	unGenericDpFunctions_getParameters(sDeviceName, dsParameters);
	if (dynlen(dsParameters) >= UN_PARAMETER_DOMAIN) sDomain = dsParameters[UN_PARAMETER_DOMAIN];

	if(!gcsAccessControlGeneric_isPrivelegeExist(sDomain, GCS_PRIVELEGES_3)) return;
	fwAccessControl_checkUserPrivilege (sUser, sDomain, GCS_PRIVELEGES_3, bGranted, exceptionInfo);
}


//------------------------------------------------------------------------------------------------------------------------------
// gcsMassFlowController_DisplayLabelFromInt
/**
Animate Part of MFC faceplate. Display values of type int (Totalizer, calibration curve and DriveMode).

	@param[in]	 iValue		Value coming from the different elements
	@param[in]	 iMode		Indicate kind of "tab" - Totalizer mode, Drive mode, Calibration curve
		iMode = {GCS_MFC_TOTALIZER, GCS_MFC_DRIVEMODE, GCS_MFC_CALIBRATION}
	@param[in]	 sShape		Name of the shape in the colorbox
	@param[in]	 sColor		Name of the color corresponding to the specified shape
	@param[in]	 bInvalid	Invalidity flag
*/
gcsMassFlowController_DisplayLabelFromInt(int iValue, int iMode, string sShape, string sColor, bool bInvalid)
{
	string sValue = "";
	bool iRes = false;

	//	get label according to int value (not index)
	switch(iMode)
	{
		case GCS_MFC_TOTALIZER:
			iRes = gcsMassFlowController_getStringFromLabels(g_dsMFCTotalizerMode, g_diMFCTotalizerMode, iValue, sValue); break;
		case GCS_MFC_DRIVEMODE:
			iRes = gcsMassFlowController_getStringFromLabels(g_dsMFCDriveMode, g_diMFCDriveMode, iValue, sValue); break;
		case GCS_MFC_CALIBRATION:
			iRes = gcsMassFlowController_getStringFromLabels(g_dsMFCalibrCurve, g_diMFCalibrCurve, iValue, sValue); break;
	}

	//data is valid but value is out of pattern
	if(iRes != true)
	{
		sValue = GCS_MFC_INVALID_DATA;
	}
	if (bInvalid)
	{
		sColor = "unDataNotValid";
	}

	if(g_bSystemConnected)
	{
		setMultiValue(sShape + ".display", "foreCol", sColor, sShape + ".display", "text", sValue);
	}
}

//-----------------------------------------------------------------------------------------------------------------------
// gcsMassFlowController_getLabelsFromDP
/**
Gets description of states for a corresponding data point.
The format of the states description: "1=start,2=stop,5=run".
At least one symbol of "=" in the description indicates that it is the set of states.

	@param[in]	sDp	Device DP name
	@param[out]	dsLabels	Labels list
	@param[out]	diIndexes	Indexes list
	@sa	gcsMassFlowController_getIndexFromLabels,
		gcsMassFlowController_getIntFromLabels, gcsMassFlowController_getStringFromLabels
*/
gcsMassFlowController_getLabelsFromDP(string sDp, dyn_string &dsLabels, dyn_int &diIndexes, dyn_string &exceptionInfo)
{
	string sLabels;
	dyn_string dsValueLabelDoublet;
	dyn_string dsCurLabels;
	int i;

	dynClear(dsLabels);
	dynClear(diIndexes);

	sLabels = dpGetDescription(sDp);
	if(strpos(sLabels, "=") < 0) dynAppend(exceptionInfo, "gcsMassFlowController_getLabelsFromDP: states description is not correct for the object" + sDp);

	//If there is at least one symbol "=" in the description,
	//it is processed as description of states
	dsCurLabels = strsplit(sLabels, ",");

	for (i = 1; i <= dynlen(dsCurLabels); i++)
	{
		dynClear(dsValueLabelDoublet);
		if(strpos(dsCurLabels[i], "=") < 0)
		{
			dynAppend(exceptionInfo, "gcsMassFlowController_getLabelsFromDP: description of states is not correct for the object: " + sDp);
			continue;
		}

		dsValueLabelDoublet = strsplit(dsCurLabels[i], "=");
		dynAppend(diIndexes, dsValueLabelDoublet[1]);
		dynAppend(dsLabels, dsValueLabelDoublet[2]);
	}
}

//-----------------------------------------------------------------------------------------------------------------------
// gcsMassFlowController_getStringFromLabels
/**
Get information about labels based on index number.
If we have labels like this: "1=stop 5=start 7=reset" and index == 5, we get "start".

	@param[in]	dsLabels	Labels list
	@param[in]	diIndexes	Indexes list
	@param[in]	iIndex	Index part of label
	@param[out]	sLabel	Label, corresponding to the index
	@sa	gcsMassFlowController_getLabelsFromDP, gcsMassFlowController_getIndexFromLabels,
		gcsMassFlowController_getIntFromLabels
*/
bool gcsMassFlowController_getStringFromLabels(dyn_string dsLabels, dyn_int diIndexes, int iIndex, string& sLabel)
{
	sLabel = "";
	if(dynlen(dsLabels) != dynlen(diIndexes)) return false;

	int iPos = dynContains(diIndexes, iIndex);
	if(iPos <= 0) return false;
	sLabel = dsLabels[iPos];
	return true;
}





//-----------------------------------------------------------------------------------------------------------------------
// gcsMassFlowController_getIndexFromLabels
/**
Get information about label position in a labels list, based on index number.
If we have labels like this: "1=stop 5=start 7=reset" and index == 5, we get 2

	@param[in]	dsLabels	Labels list
	@param[in]	diIndexes	Indexes list
	@param[in]	iIndex		Index part of label
	@param[out]	iPosition	Position of label with index "index" inside list of labels
	@sa	gcsMassFlowController_getLabelsFromDP,
		gcsMassFlowController_getIntFromLabels, gcsMassFlowController_getStringFromLabels
*/
bool gcsMassFlowController_getIndexFromLabels(dyn_string dsLabels, dyn_int diIndexes, int iIndex, int& iPosition)
{
	iPosition = -1;
	if(dynlen(dsLabels) != dynlen(diIndexes)) return false;
	int iTemp = dynContains(diIndexes, iIndex);
	if(iTemp <= 0) return false;
	iPosition = iTemp;
	return true;
}













//-----------------------------------------------------------------------------------------------------------------------
// gcsMassFlowController_getIntFromLabels
/**
Get text part of a label, based on index number.
If we have labels like this: "1=stop 5=start 7=reset" and label == start, we get 5

	@param[in]	dsLabels	Labels list
	@param[in]	diIndexes	Indexes list
	@param[in]	sLabel	Text part of label
	@param[out]	iIndex	Position of label with index "index" inside list of labels
	@sa	gcsMassFlowController_getLabelsFromDP, gcsMassFlowController_getIndexFromLabels,
		gcsMassFlowController_getStringFromLabels
*/
bool gcsMassFlowController_getIntFromLabels(dyn_string dsLabels, dyn_int diIndexes, string sLabel, int& iIndex)
{
	iIndex = -1;
	if(dynlen(dsLabels) != dynlen(diIndexes)) return false;
	int iTemp = dynContains(dsLabels, sLabel);
	if(iTemp <= 0) return false;
	iIndex = diIndexes[iTemp];
	return true;
}



//@}
