/**@name LIBRARY: gcsConstants.ctl

@author: Nikolay Kulman (IT-CO-FE)
@author: Francisco Mico (IT-CO-FE)
@date 17.06.2003
@brief This library contains gcs constants.
Modifications History:
	$Log: gcsObjects_declarations.ctl,v $
	Revision 1.22  2009/03/20 14:00:10  gthomas
	-Updated variable names <_GCSDELAYEDALARM_CLASS> with <_ALERT_CLASS_>
	
	Revision 1.21  2008/09/04 14:54:31  gthomas
	-moved back to version 1.15  and added "MFCSetParameters" constant

	Revision 1.20  2008/06/23 08:57:31  gthomas
	added variable const string GCS_CONFIG_EMPTY_STRING = "-";

	Revision 1.19  2008/06/23 08:29:10  gthomas
	Kept only variables for xStatus and xParameters

	Revision 1.18  2008/06/19 12:24:25  gthomas
	uncommented 2 variables used in xParameters lib and pnl

	Revision 1.17  2008/06/13 15:16:38  aabadz
	UN_MENU_ALARM_MASK_REPLY commented
	because is already declared in another file.

	Revision 1.16  2008/06/09 14:54:27  aabadz
	Result after move some constants, only use use inside this file, from gcsObjects_declarations.

	Revision 1.15  2007/10/30 16:22:09  gthomas
	Introduced the CVS keyword $Log to track changes made in the library

*/
/*
Internal Functions :
*/
// StsReg01 constants
const unsigned UN_STSREG01_AST =			0;				    /// Alarm Status AA (for PosAl DPE; un_POSAL = 9)
const unsigned UN_STSREG01_WST =			1;				    /// Warning Status AnalogAlarm
const unsigned UN_STSREG01_AuMoSt =			2;				    /// Auto Mode
const unsigned UN_STSREG01_MaMoSt =			3;				    /// Manual Mode
const unsigned UN_STSREG01_FoMoSt =			4;				    /// Forced mode
const unsigned UN_STSREG01_PREMASKALARM =	8;					/// Premask alarm for DA
const unsigned UN_STSREG01_LLST =			8;					/// ANALOG ALARM -LL
const unsigned UN_STSREG01_LST=			 	9;					/// ANALOG ALARM -L
const unsigned UN_STSREG01_HST =			10;					/// ANALOG ALARM -H
const unsigned UN_STSREG01_HHST =			11;					/// ANALOG ALARM -HH

const unsigned UN_STSREG01_MAMSKST =		15;					/// STSREG info alarm setMask
const unsigned UN_STSREG01_PFSPOSONST_MFC = 5;					/// Auto inhibit manual mode (MFC)
const unsigned UN_STSREG01_POSOV =			11;					/// DC Status Position		????

const unsigned UN_MANREG01_AMSKST =			11;					/// AlarmMaskOn (Off is when is not bit 11) AA, DA. Output
const unsigned UN_MANREG01_AMSKRST =		12;					/// AalarmMaskOff (Reset alarm AA, DA) Output.

// ManReg01 constants --- seems we not use any more
const unsigned UN_MANREG01_MPOSRST =	6;						/// Manual Position request trigger bit in ManReg01

// Button constants
const string UN_FACEPLATE_BUTTON_AC_SET_VALUE = "ACSetValue";			/// Set Value AnalogComputed
const string UN_FACEPLATE_BUTTON_WC_SET_VALUE   = "WCSetValue";			/// Set Value WordComputed
const string UN_FACEPLATE_BUTTON_WC_SET_STRING   = "WCSetString";		/// Set Value WordComputed

const string UN_FACEPLATE_BUTTON_PARAMETER_SET_VALUE	=		"Parameter_SetValue"; 			/// Set Value for AnalogParameter and WordParameter
const string UN_FACEPLATE_BUTTON_DIGITALPARAMETER_SET_VALUE	=	"DigitalParameter_SetValue"; 	/// Set Value DigitalParameter
const string UN_FACEPLATE_BUTTON_PUMP_SET_FREQUENCY   = 		"SetFrecuency";					/// Set Frecuency Pump

const string UN_FACEPLATE_BUTTON_AA_SUBPANEL_SET_ALARM   = "SetAlarm";			/// Test subpanel Analog Alarm
const string UN_FACEPLATE_BUTTON_AA_SUBPANEL_RESET_ALARM   = "ResetAlarm";		/// Test subpanel Analog Alarm
const string UN_FACEPLATE_BUTTON_AA_SUBPANEL_SET_WARNING   = "SetWarning";		/// Test subpanel Analog Alarm
const string UN_FACEPLATE_BUTTON_AA_SUBPANEL_RESET_WARNING   = "ResetWarning";	/// Test subpanel Analog Alarm

const string UN_FACEPLATE_BUTTON_MFC_SUBPANEL_APPLY ="MFCSetParameters";

//	AA and DA RBMenu
const unsigned UN_MENU_ALARM_MASK_REPLY		 = 55;			///Menu Choice result for Digital Alarm and Analog Alarm
const unsigned UN_MENU_RECIPES_REPLY		 = 56;			///Menu Choice result for Recipes from PCO
const unsigned UN_MENU_CURRNTPARAMS_REPLY	 = 57;			///Menu Choice result for Current Parameters from PCO
const string UN_POPUPMENU_RECIPES  			 = "Recipes";	///Menu Item for calling Recipes from PCO
const string UN_POPUPMENU_CURRENTPARAMS 	 = "Current Parameters";	///Menu Item for calling Current Parameters from PCO

const string UN_MENU_AA_MASK   = "Mask_AA";					///Mask Menu Item for Analog Alarm
const string UN_MENU_DA_MASK   = "Mask_DA";					///Mask Menu Item for Digital Alarm

const string UN_MENU_ALARM_SET_MASK	 =	"SetMask";			///Set Mask Analog/Digital Alarm - text
const string UN_MENU_ALARM_RESET_MASK   =	"ResetMask";	///Reset Mask Analog/Digital Alarm - text

const string UN_POPUPMENU_DIAG_INFO_TEXT = "Info_Diagnostic";	///	Unicos "info-diagnostic" menu-text
const string UN_MENU_MAIL_SMS = "Mail-SMS";						///	SMS-mail menu identifier.
const string UN_MENU_MAIL_SMS_TEXT = "Mail-SMS";				///	SMS-mail menu text.
const string GCS_ALARMS_CATEGORY = "";							/// 

// MFC constants
const string UN_MFC_CURRENT_MODE_PATH = ".ProcessInput.DrMoSt";					/// Current Mode DPE
const string UN_MFC_ACTIVE_REQUEST_MODE_PATH = ".ProcessInput.DrMoOVSt";		/// Active Request DPE
const string UN_MFC_AUTO_REQUEST_MODE_PATH = ".ProcessInput.AuDrMoRSt";			/// Auto Request DPE

const unsigned UN_MANREG01_MSPR 		= 6;					///Manual Set Point Request trigger
const unsigned UN_MANREG01_MDRMODE 		= 7;					///Manual Drive Mode Request Trigger 
const unsigned UN_MANREG01_MTOTMRT	 	= 8;					///Manual Totalizer Mode Request Trigger 
const unsigned UN_MANREG01_MCCRT   		= 9;					///Manual Calibration Curve Mode Request Trigger 

const unsigned GCS_MFC_TOTALIZER		= 100;					///For faceplate animation: TotMode items
const unsigned GCS_MFC_DRIVEMODE   		= 101;					///For faceplate animation: DriveMode items
const unsigned GCS_MFC_CALIBRATION  	= 102;					///For faceplate animation: CC items
const string   GCS_MFC_INVALID_DATA 	= "Invalid Data";		///Display in sace of errors in labels

// widget
const string GCS_WIDGET_PATH = "objects/GCS_OBJECTS/";			/// GCS widget path
const string GCS_WIDGET_PREFIX = "gcsWidget_";					///	GCS widget prefix
const string GCS_DPNAME_PREFIX = "gcs";							/// GCS prefix
const unsigned GCS_DPNAME_FIELD_PREFIX = 1;						/// Field prefix

// Config constants
const unsigned GCS_CONFIG_STRING_VALUE = 9;							/// String values separeted by comas
const string GCS_CONFIG_EMPTY_STRING = "-";							/// A dummy string to be put in description when no actual string is valid
const string GCS_CONFIG_DEFAULT_CALIBRATION_CURVE = "1=default";	/// default string when there is not string for the calibration curve

// Alarm
const string UN_WIDGET_ALARM_HIGH		= "H";						///High alarm AnalogAlarm
const string UN_WIDGET_ALARM_HIGHHIGH	= "HH";						///High High alarm AnalogAlarm
const string UN_WIDGET_ALARM_LOW		= "L";						///Low alarm AnalogAlarm
const string UN_WIDGET_ALARM_LOWLOW		= "LL";						///Low Low alarm - for AnalogAlarm
const string UN_WIDGET_ALARM			= "A";						///Alarm for Digital Alarm


//	access control
const int GCS_ACCESS_CONTROL_UNICOS_DOMAIN		=	1;				///	Position (ID) of domain "UNICOS"
const string GCS_PRIVELEGES_1 = "Browsing";							///	Priveledge #1 name
const string GCS_PRIVELEGES_2 = "Operation";						///	Priveledge #2 name
const string GCS_PRIVELEGES_3 = "Tuning";							///	Priveledge #3 name
const string GCS_PRIVELEGES_4 = "Support";							///	Priveledge #4 name


// recipes and current parameters
const string c_sAckFieldName 		= "ProcessInput.ValRqAck";		///	Path to ValRqAck DPE
const string c_sInputFieldName 		= "ProcessInput.CurValSt";		///	Path to CurValSt DPE
const string c_sOutputFieldName 	= "ProcessOutput.ValRq";		///	Path to ValRq DPE


// Mail-SMS alert class constants
const string GCS_CONFIG_ALERT_CLASS_ALARM_ACK		= "_gcsAlAck.";			/// Name for alarm alertclass, which is acknowledgeble and SMS is disabled
const string GCS_CONFIG_ALERT_CLASS_ALARM_NOACK		= "_gcsAlNoAck.";		/// Name for alarm alertclass, which is not acknowledgeble and SMS is disabled
const string GCS_CONFIG_ALERT_CLASS_WARNING_ACK		= "_gcsWrnAck.";		/// Name for warning alertclass, which is acknowledgeble (SMS is disabled)
const string GCS_CONFIG_ALERT_CLASS_WARNING_NOACK	= "_gcsWrnNoAck.";		/// Name for warning alertclass, which is not acknowledgeble (SMS is disabled)
const string GCS_CONFIG_ALERT_CLASS_ALARM_ACK_MAIL	= "_gcsAlAck_Mail.";	/// Name for alarm alertclass, which is acknowledgeble and SMS is enabled
const string GCS_CONFIG_ALERT_CLASS_ALARM_NOACK_MAIL	= "_gcsAlNoAck_Mail.";	/// Name for alarm alertclass, which is not acknowledgeble and SMS is enabled




