/**@name LIBRARY: gcsPump.ctl

@author: Artem Burmyakov (IT-CO-FE)
@author: Nikolay Kulman (IT-CO-FE)
@author: Teodora Miteva (IT-CO-FE)
@author: Francisco Mico (IT-CO-FE)
@date 17.06.2003
@brief This library contains the set of functions to animate widget, faceplate, popup-menu of Pump objects.
Modifications History:
	$Log: gcsPump.ctl,v $
	Revision 1.25  2009/04/06 12:19:28  gthomas
	in _WidgetAnimationCB(): fixed animation of body and letters for invalid bits and data quality priorites
			        Added body color animation when Manual or Local mode
	
	Revision 1.23  2008/11/12 08:29:18  gthomas
	-removed function calls to _ButtonUserAccessGcs and _ButtonUserAccessUn

	Revision 1.22  2008/10/20 13:54:20  gthomas
	-removed unused XX_EventList_Configure_YY functions

	Revision 1.21  2008/10/15 14:59:23  gthomas
	-Removed the <if statement> of the dpConnect in _ButtonRegisterCB function
	to fix problem with select/deselect button animation.

	Revision 1.20  2008/07/23 10:02:37  gthomas
	-removed commented code (for error handling)

	Revision 1.19  2008/07/21 13:12:41  gthomas
	added missing braket in animation CB function

	Revision 1.18  2008/07/18 07:16:55  gthomas
	added new Access control functionality for unicos-4-1
	modified  xx_ButtonUserAccess function.

	Revision 1.17  2008/06/18 14:54:51  aabadz
	Changes done following the recomendations from AB-CO (from Herve)

	Revision 1.16  2008/05/15 09:36:45  gamx
	fixed a bug related to exceptions handling (undeclared variable)

	Revision 1.15  2008/03/17 10:52:36  aabadz
	Fixed a problem found in animation when we have not database.

	Revision 1.14  2008/01/23 15:17:24  gamx
	fixed bug in animation of a faceplate,
	the version after the final test

	Revision 1.13  2008/01/16 15:57:13  aabadz
	Some sintactic errors fixed

	Revision 1.12  2008/01/15 10:06:23  aabadz
	Modifed gcsPump_ObjectListGetValueTime because the code that we had before return nothing instead of  the time and the value

	Revision 1.11  2007/11/22 20:13:11  gamx
	extra checks of possible exceptional cases have been added

	Revision 1.10  2007/11/22 10:05:11  gamx
	gcsConfigGenericFunctions_getPlcDpName is used now to define plc dp name

	Revision 1.9  2007/11/20 10:53:19  gamx
	sintactic bugs are fixed

	Revision 1.8  2007/11/18 19:37:29  gamx
	fixes after code review

	Revision 1.7  2007/10/30 16:22:27  gthomas
	Introduced the CVS keyword $Log to track changes made in the library

*/
/*
Internal Functions :
	gcsPump_ObjectListGetValueTime (string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
	gcsPump_AcknowledgeAlarm (string sDpName, string sDpType, dyn_string &dsNeedAck)
	gcsPump_MenuConfiguration (string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
	gcsPump_HandleMenu (string deviceName, string sDpType, dyn_string menuList, int menuAnswer)
	gcsPump_FaceplateStatusRegisterCB (string sDp, bool bSystemConnected)
	gcsPump_FaceplateConnect (string deviceName)
	gcsPump_FaceplateDisconnect (string deviceName)
	gcsPump_FaceplateStatusAnimationCB_NoAlarm (string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid, ... )
	gcsPump_FaceplateStatusAnimationCB (string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid, ... )
	gcsPump_FaceplateAnimatePlcDisconnection ()
	gcsPump_ButtonRegisterCB (string sDp, bool bSystemConnected)
	gcsPump_ButtonAnimationCB (string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager, string sDpStsReg01, ... )
	gcsPump_ButtonUserAccess (string sDpName, string sDpType, dyn_string &dsAccess)
	gcsPump_UserLoginGetButtonState (string deviceName, string sType, dyn_string &dsReturnData)
	gcsPump_ButtonSetState (string deviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
	gcsPump_ButtonDisconnection ()
	gcsPump_WidgetRegisterCB (string sDp, bool bSystemConnected)
	gcsPump_WidgetConnect (string deviceName, string sPlcName, string sSystemName)
	gcsPump_WidgetDisconnect (string deviceName, string sPlcName, string sSystemName)
	gcsPump_WidgetAnimationCB (string sDpLock, bool bLocked, string sDpManager, string sSelectedManager, string sDpStsReg01, bit32 bit32StsReg01, ... )
	gcsPump_WidgetDisconnection (string sWidgetType)
	unGenericObject_WidgetPmpDisconnection ()
	gcsPump_EventList_Configure_Init (dyn_string &exceptionInfo, const bool bUpdate=false)
	gcsPump_EventList_Configure_Clic (int iRow, string sColumn, string sValue, dyn_string &exceptionInfo)
	gcsPump_EventList_Configure_Save (dyn_string &exceptionInfo)
*/


//---------------------------------------------------------------------------------------------------------------------------------------

// gcsPump_ObjectListGetValueTime
/**
Function called from the objectList to return the time and value

	@param[in]	sDeviceName		Device name
	@param[in]	sDeviceType		Unicos object
	@param[out]	dsReturnData	Return value
*/

gcsPump_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
	float value;
	string sTime, sStatus,sColor;
	bool bBad;

	dpGet(sDeviceName+".ProcessInput.AuPosSt", value,
	      sDeviceName+".ProcessInput.AuPosSt:_online.._stime", sTime,
	      sDeviceName+".ProcessInput.AuPosSt:_online.._bad", bBad);

	dsReturnData[1] = sTime;
	dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(sDeviceName+".ProcessInput.AuPosSt"), value) + " "
							+ dpGetUnit(sDeviceName+".ProcessInput.AuPosSt");

	// Optionally (when no stsReg* exists)
	// Status
	dsReturnData[3] = bBad;

	// Status to visualize
	sStatus="OK";
	dsReturnData[4] = sStatus;
	// Colors
	sColor="unEventList_ForeColor";
	dsReturnData[5]=sColor;
}
//---------------------------------------------------------------------------------------------------------------------------------------
// gcsPump_AcknowledgeAlarm
/**
Acknowledge Pump alarms

	@param[in]	 sDpName 	Device name
	@param[in]	 sDpType 	Unicos object
	@param[out]	 dsNeedAck 	DPEs names that need acknowledgement.

The PLC acknowledgement is specified using UN_ACKNOWLEDGE_PLC string
*/
// used in the unGenericObject_Acknowledge
gcsPump_AcknowledgeAlarm(string sDpName, string sDpType, dyn_string &dsNeedAck)
{
	dsNeedAck = makeDynString("ProcessInput.StartISt", "ProcessInput.StopISt", UN_ACKNOWLEDGE_PLC);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsPump_MenuConfiguration
/**
Configuration of popup menu

	@param[in]  sDpName Device name
	@param[in]	sDpType Unicos object
	@param[in]	dsAccessOk	List of authorized action
	@param[out] menuList Configuration of menu. Use defined constants UN_POPUPMENU_***
*/
gcsPump_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &dsMenuList)
{
	dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, UN_POPUPMENU_ACK_TEXT, UN_POPUPMENU_FACEPLATE_TEXT, UN_POPUPMENU_TREND_TEXT);

	unGenericObject_addSelectToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, dsMenuList);
	unGenericObject_addUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, dsMenuList);
	unGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, dsMenuList);
	unGenericObject_addTrendActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, dsMenuList);
}













//---------------------------------------------------------------------------------------------------------------------------------------
// gcsPump_HandleMenu
/**
Handle the answer of the popup menu

	@param[in]	deviceName 	The device dp name
	@param[in]	sDpType 	The device dp type
	@param[in]	menuList 	List of available actions for this type of objects
	@param[in]	menuAnswer 	Selected action by an operator
*/
gcsPump_HandleMenu(string sDeviceName, string sDpType, dyn_string dsMenuList, int iMenuAnswer)
{
	dyn_string dsExceptionInfo;

	gcsGenericObject_HandleEventMaskMenu(sDeviceName, iMenuAnswer, makeDynString(".ProcessInput.evStsReg01"), dsExceptionInfo);
	if (dynlen(dsExceptionInfo) <= 0) unGenericObject_HandleUnicosMenu(sDeviceName, sDpType, dsMenuList, iMenuAnswer, dsExceptionInfo);

	if (dynlen(dsExceptionInfo) > 0)
	{
		unSendMessage_toExpertException("Right Click", dsExceptionInfo);
	}
}

















//---------------------------------------------------------------------------------------------------------------------------------------
// gcsPump_FaceplateStatusRegisterCB
/**
Register faceplate callback function

	@param[in]	sSystemStatus 	The device dp
	@param[in] 	bSystemConnected 	Flag
*/
gcsPump_FaceplateStatusRegisterCB(string sSystemStatus, bool bSystemConnected)
{
	string sDeviceName;
	string sSystemName;
	string sPlcName;
	string sPlcDpName;
	int iAction;
	dyn_string dsExceptionInfo;
	bool bRemote;


	sDeviceName = unGenericDpFunctions_getDpName($sDpName);

	if (sDeviceName != "")
	{
		sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);


		unDistributedControl_isRemote(bRemote, sSystemName);
		if(bRemote) g_bSystemConnected = bSystemConnected;
		else g_bSystemConnected = true;


		sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);


		gcsConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, dsExceptionInfo);
		if (dynlen(dsExceptionInfo) <= 0) g_bUnSystemAlarmPlc = true;
		else g_bUnSystemAlarmPlc = false;





		unGenericObject_Connection(sDeviceName, g_bCallbackConnected, bSystemConnected, iAction, dsExceptionInfo);

		switch (iAction)
		{
			case UN_ACTION_DISCONNECT:
				gcsPump_FaceplateAnimatePlcDisconnection();
				break;
			case UN_ACTION_DPCONNECT:
				gcsPump_FaceplateConnect(sDeviceName, dsExceptionInfo);
				break;
			case UN_ACTION_DPDISCONNECT_DISCONNECT:
				gcsPump_FaceplateDisconnect(sDeviceName, dsExceptionInfo);
				gcsPump_FaceplateAnimatePlcDisconnection();
				break;
			case UN_ACTION_NOTHING:
			default:
				break;
		}
	}

}


//-------------------------------------------------------------------------------------------------------------------------------
// function - gcsPump_FaceplateConnect
/**
Do faceplate dpConnect

	@param[in]	deviceName	Device DP name
*/
gcsPump_FaceplateConnect(string sDeviceName, dyn_string &exceptionInfo)
{
	int iAction;
	int iRes = -1;
	int iAlertType1;
	int iAlertType2;
	int iRangeType;
	string sPlcName;
	string sPlcDpName;
	string sSystemName;


	sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
	sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);


	gcsConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0) g_bUnSystemAlarmPlc = true;
	else {g_bUnSystemAlarmPlc = false; return;}


	iRes = dpGet(sDeviceName + ".ProcessInput.StartISt:_alert_hdl.._type", iAlertType1, sDeviceName + ".ProcessInput.StopISt:_alert_hdl.._type", iAlertType2);
	if (iRes < 0)
	{
    		dynAppend(exceptionInfo, "gcsPump_FaceplateConnect: missed dpes (StartISt/StopISt:_alert_hdl.._type) for the device " + sDeviceName);
    		return;
	}
	else
	{
		g_bBoolAlert = ((iAlertType1 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType2 == DPCONFIG_ALERT_BINARYSIGNAL));

		if (g_bUnSystemAlarmPlc)
		{
			g_sLoadStFormat = dpGetFormat(sDeviceName + ".ProcessInput.LoadSt");
			g_sLoadStUnit = dpGetUnit(sDeviceName + ".ProcessInput.LoadSt");
			g_sPosStFormat = dpGetFormat(sDeviceName + ".ProcessInput.PosSt");
			g_sPosStUnit = dpGetUnit(sDeviceName + ".ProcessInput.PosSt");
			g_sAOutOVStFormat = dpGetFormat(sDeviceName + ".ProcessInput.AOutOVSt");
			g_sAOutOVStUnit = dpGetUnit(sDeviceName + ".ProcessInput.AOutOVSt");
			g_sAuPosRStFormat = dpGetFormat(sDeviceName + ".ProcessInput.AuPosRSt");
			g_sAuPosRStUnit = dpGetUnit(sDeviceName + ".ProcessInput.AuPosRSt");

			if(g_bBoolAlert)
			{
			   iRes = dpConnect("gcsPump_FaceplateStatusAnimationCB",
			 	sDeviceName + ".ProcessInput.StsReg01",  sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
			 	sDeviceName + ".ProcessInput.StsReg02",  sDeviceName + ".ProcessInput.StsReg02:_online.._invalid",
			 	sDeviceName + ".ProcessInput.PosSt",	 sDeviceName + ".ProcessInput.PosSt:_online.._invalid",
			 	sDeviceName + ".ProcessInput.LoadSt",	 sDeviceName + ".ProcessInput.LoadSt:_online.._invalid",
			 	sDeviceName + ".ProcessInput.AuPosRSt",	 sDeviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
			 	sDeviceName + ".ProcessInput.AOutOVSt",	 sDeviceName + ".ProcessInput.AOutOVSt:_online.._invalid",
			 	sDeviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state",
			 	sDeviceName + ".ProcessInput.StopISt:_alert_hdl.._act_state",
			 	sPlcDpName + ".alarm",
			 	sPlcDpName + ".enabled",
			 	sDeviceName + ".eventMask");
			}
			else
			{
			    iRes = dpConnect("gcsPump_FaceplateStatusAnimationCB_NoAlarm",
			 	sDeviceName + ".ProcessInput.StsReg01",  sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
			 	sDeviceName + ".ProcessInput.StsReg02",  sDeviceName + ".ProcessInput.StsReg02:_online.._invalid",
			 	sDeviceName + ".ProcessInput.PosSt",	 sDeviceName + ".ProcessInput.PosSt:_online.._invalid",
			 	sDeviceName + ".ProcessInput.LoadSt",	 sDeviceName + ".ProcessInput.LoadSt:_online.._invalid",
			 	sDeviceName + ".ProcessInput.AuPosRSt",	 sDeviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
			 	sDeviceName + ".ProcessInput.AOutOVSt",	 sDeviceName + ".ProcessInput.AOutOVSt:_online.._invalid",
			 	sPlcDpName + ".alarm",
			 	sPlcDpName + ".enabled",
			 	sDeviceName + ".eventMask");
			}
		}
		else gcsPump_FaceplateAnimatePlcDisconnection();

		if (iRes < 0)
		{
			dynAppend(exceptionInfo, "gcsPump_FaceplateConnect: problems to set callback animation function");
			g_bCallbackConnected = false;
		}
		else g_bCallbackConnected = true;
	}

}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsPump_FaceplateDisconnect
/**
Do faceplate dpDisconnect

	@param[in]	deviceName	Device DP name
*/
gcsPump_FaceplateDisconnect(string sDeviceName, dyn_string &exceptionInfo)
{
	int iRes = -1;
	string sPlcName, sSystemName;
	string sPlcDpName;

	g_bCallbackConnected = false;

	sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
	sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);

	gcsConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0) g_bUnSystemAlarmPlc = true;
	else g_bUnSystemAlarmPlc = false;


	if (g_bUnSystemAlarmPlc)
	{
		sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
		sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);

		gcsConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);

		if (dynlen(exceptionInfo) <= 0)
		{
			if (g_bBoolAlert)
			{
			iRes = dpDisconnect("gcsPump_FaceplateStatusAnimationCB",
				sDeviceName + ".ProcessInput.StsReg01",  sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
				sDeviceName + ".ProcessInput.StsReg02",  sDeviceName + ".ProcessInput.StsReg02:_online.._invalid",
				sDeviceName + ".ProcessInput.PosSt",	 sDeviceName + ".ProcessInput.PosSt:_online.._invalid",
				sDeviceName + ".ProcessInput.LoadSt",	 sDeviceName + ".ProcessInput.LoadSt:_online.._invalid",
				sDeviceName + ".ProcessInput.AuPosRSt",	 sDeviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
				sDeviceName + ".ProcessInput.AOutOVSt",	 sDeviceName + ".ProcessInput.AOutOVSt:_online.._invalid",
				sDeviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state",
				sDeviceName + ".ProcessInput.StopISt:_alert_hdl.._act_state",
				sPlcDpName + ".alarm",
				sPlcDpName + ".enabled",
				sDeviceName + ".eventMask");
			}
			else
			{
			iRes = dpDisconnect("gcsPump_FaceplateStatusAnimationCB_NoAlarm",
				sDeviceName + ".ProcessInput.StsReg01",  sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
				sDeviceName + ".ProcessInput.StsReg02",  sDeviceName + ".ProcessInput.StsReg02:_online.._invalid",
				sDeviceName + ".ProcessInput.PosSt",	 sDeviceName + ".ProcessInput.PosSt:_online.._invalid",
				sDeviceName + ".ProcessInput.LoadSt",	 sDeviceName + ".ProcessInput.LoadSt:_online.._invalid",
				sDeviceName + ".ProcessInput.AuPosRSt",	 sDeviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
				sDeviceName + ".ProcessInput.AOutOVSt",	 sDeviceName + ".ProcessInput.AOutOVSt:_online.._invalid",
				sPlcDpName + ".alarm",
				sPlcDpName + ".enabled",
				sDeviceName + ".eventMask");
			}
		}
	}

	if (iRes < 0)
	{
		dynAppend(exceptionInfo, "gcsPump_FaceplateDisconnect: problems to disconnect a callback animation function");
		g_bCallbakcConnected = true;
	}
	else g_bCallbackConnected = false;
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsPump_FaceplateStatusAnimationCB_NoAlarm
/**
Animate faceplate
*/
gcsPump_FaceplateStatusAnimationCB_NoAlarm(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
			string sStsReg02, bit32 bit32StsReg02, string sDp4, bool bStsReg02Invalid,
			string sDpPosSt, float fPosSt, string sDp6, bool bPosStInvalid,
			string sDpLoadSt, float fLoadSt, string sDp12, bool bLoadStInvalid,
			string sDpAuto, float fAuPosRSt, string sDp8, bool bAuPosRStInvalid,
			string sDpActive, float fAOutOVSt, string sDp10, bool bAOutOVStInvalid,
			string sDpAl, int iValAlarm, string sDpEnabled, bool bPlcEnabled, string sDpEvent, int iMask)
{
	gcsPump_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid,
			                                sStsReg02, bit32StsReg02, sDp4, bStsReg02Invalid,
                                     sDpPosSt, fPosSt, sDp6, bPosStInvalid,
                                     sDpLoadSt, fLoadSt, sDp12, bLoadStInvalid,
                                     sDpAuto, fAuPosRSt, sDp8, bAuPosRStInvalid,
                                     sDpActive, fAOutOVSt, sDp10, bAOutOVStInvalid,"", 0, "", 0,
                                     sDpAl, iValAlarm, sDpEnabled, bPlcEnabled, sDpEvent, iMask);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsPump_FaceplateStatusAnimationCB
/**
Animate faceplate
*/
gcsPump_FaceplateStatusAnimationCB(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
		string sStsReg02, bit32 bit32StsReg02, string sDp4, bool bStsReg02Invalid,
		string sDpPosSt, float fPosSt, string sDp6, bool bPosStInvalid,
		string sDpLoadSt, float fLoadSt, string sDp14, bool bLoadStInvalid,
		string sDpAuto, float fAuPosRSt, string sDp8, bool bAuPosRStInvalid,
		string sDpActive, float fAOutOVSt, string sDp10, bool bAOutOVStInvalid,
		string sDp11, int iStartIStState, string sDp12, int iStopIStState,
		string sDpAl, int iValAlarm, string sDpEnabled, bool bSystemIntegrityAlarmEnabled, string sDpEvent, int iMask)
{
		// 1. Animate display values
	if(g_bSystemConnected)
		unGenericObject_DisplayValue(g_sLoadStFormat, g_sLoadStUnit, fLoadSt, "LoadStatus", "unDisplayValue_Status", bLoadStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, fPosSt, "StatusFrecuency", "unDisplayValue_Status", bPosStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_DisplayValue(g_sAuPosRStFormat,	g_sAuPosRStUnit, fAuPosRSt, "RequestAutoFrecSetPoint", "unDisplayValue_Request", bAuPosRStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_DisplayValue(g_sAOutOVStFormat, g_sAOutOVStUnit, fAOutOVSt, "RequestActiveFrecSetPoint", "unDisplayValue_Output", bAOutOVStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));

		// 2. Animate colorboxes
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("OperationMode_Auto", getBit(bit32StsReg01, UN_STSREG01_AUMOST), 1, "unFaceplate_OperationModeActive",
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("OperationMode_Manual", getBit(bit32StsReg01, UN_STSREG01_MMOST), 1, "unFaceplate_OperationModeActive",
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("OperationMode_Forced", getBit(bit32StsReg01, UN_STSREG01_FOMOST), 1, "unFaceplate_OperationModeActive",
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("OperationMode_Local", getBit(bit32StsReg01, UN_STSREG01_LDST), 1, "unFaceplate_OperationModeActive",
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("Warning_IOError", getBit(bit32StsReg01, UN_STSREG01_IOERRORW), 1, "unFaceplate_WarningActive",
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("Warning_IOSimulated", getBit(bit32StsReg01, UN_STSREG01_IOSIMUW), 1, "unFaceplate_WarningActive",
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("Warning_ManualAuto", getBit(bit32StsReg01, UN_STSREG01_AUMRW), 1, "unFaceplate_WarningActive",
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("Warning_PositionWarning", getBit(bit32StsReg01, UN_STSREG01_POSALW), 1, "unFaceplate_WarningActive",
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("Status_OnOpened", getBit(bit32StsReg02, UN_STSREG02_PHFONST), 1, "unFaceplate_StatusActive",
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("Status_OffClosed", getBit(bit32StsReg02, UN_STSREG02_PHFOFFST), 1, "unFaceplate_StatusActive",
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("Request_OnAuto", getBit(bit32StsReg02, UN_STSREG02_AUONRST), 1, "unFaceplate_RequestActive",
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("Request_OnActive", getBit(bit32StsReg02, UN_STSREG02_MONRST), 1, "unFaceplate_RequestActive",
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("Request_OffAuto", getBit(bit32StsReg02, UN_STSREG02_AUOFFRST), 1, "unFaceplate_RequestActive",
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("Request_OffActive", getBit(bit32StsReg02, UN_STSREG02_MOFFRST), 1, "unFaceplate_RequestActive",
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));

		// 3. Animate alarm colorboxes
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimateAlarm("Alarm_StartISt", iStartIStState, g_bBoolAlert,
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimateAlarm("Alarm_StopISt", iStopIStState, g_bBoolAlert,
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("Alarm_UnAck", getBit(bit32StsReg01, UN_STSREG01_ALUNACK), 1, "unFaceplate_AlarmNotAck",
			bStsReg01Invalid || bStsReg02Invalid  || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("eventState", iMask, 1, "",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value),
			"unFaceplate_OperationModeActive");
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsPump_FaceplateAnimatePlcDisconnection
/**
Animate faceplate disconnection
*/
gcsPump_FaceplateAnimatePlcDisconnection()
{
	unGenericObject_ColorBoxDisconnect(makeDynString("Alarm_StartISt", "Alarm_StopISt", "Alarm_UnAck", "OperationMode_Auto",
		 "OperationMode_Forced", "OperationMode_Local", "OperationMode_Manual",
		 "Warning_IOError", "Warning_IOSimulated", "Warning_ManualAuto",
		 "Warning_PositionWarning", "Status_OnOpened", "Status_OffClosed",
		 "Request_OnAuto", "Request_OnActive", "Request_OffAuto", "Request_OffActive",
		 "eventState"));
	unGenericObject_DisplayValueDisconnect(makeDynString("StatusFrecuency", "LoadStatus", "RequestAutoFrecSetPoint",
		 "RequestActiveFrecSetPoint"));
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsPump_ButtonRegisterCB
/**
Register callback function

	@param[in]	sDp 	The device dp
	@param[in] 	bSystemConnected 	Flag
*/
gcsPump_ButtonRegisterCB(string sDp, bool bSystemConnected)
{
	string sDeviceName;
	string sDpType;
	int iAction;
	int iRes;
	int iAlertType1;
	int iAlertType2;
	int iManReg01;
	dyn_string exceptionInfo;


	sDeviceName = unGenericDpFunctions_getDpName($sDpName);
	unGenericObject_Connection(sDeviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);

	switch (iAction)
	{
		case UN_ACTION_DISCONNECT:
			gcsPump_ButtonDisconnection();
			break;

		case UN_ACTION_DPCONNECT:
			iRes = dpGet(sDeviceName + ".ProcessOutput.ManReg01", iManReg01);
			if ((iRes < 0) || (iManReg01 != 0))
			{
				dpSet(sDeviceName + ".ProcessOutput.ManReg01", 0);
			}

			if (dynlen(exceptionInfo) <= 0)gcsPump_ApplyAccessToButtons(sDeviceName, exceptionInfo);
			iRes = dpConnect("gcsPump_ButtonAnimationCB",
					 sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
					 sDeviceName + ".statusInformation.selectedManager",
					 sDeviceName + ".ProcessInput.StsReg01",
					 sDeviceName + ".ProcessInput.StsReg01:_online.._aut_inv",
					 sDeviceName + ".ProcessInput.StsReg02",
					 sDeviceName + ".ProcessInput.StsReg02:_online.._aut_inv");
			g_bCallbackConnected = (iRes >= 0);
							
			break;

		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			iRes = dpDisconnect("gcsPump_ButtonAnimationCB",
				 sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
				 sDeviceName + ".statusInformation.selectedManager",
				 sDeviceName + ".ProcessInput.StsReg01",
				 sDeviceName + ".ProcessInput.StsReg01:_online.._aut_inv",
				 sDeviceName + ".ProcessInput.StsReg02",
				 sDeviceName + ".ProcessInput.StsReg02:_online.._aut_inv");
			if (iRes >= 0)
			{
				g_bCallbackConnected = false;
				gcsPump_ButtonDisconnection();
			}
			else
			{
				g_bCallbackConnected = true;
				dynAppend(exceptionInfo, "gcsPump_ButtonRegisterCB: problems to disconnect animation callback function");
			}
			break;

		case UN_ACTION_NOTHING:
		default:
			break;
	}
}









//---------------------------------------------------------------------------------------------------------------------------------------
// gcsPump_ApplyAccessToButtons
/**
    Via unicos function, a callback function is defined to provide access control to faceplate buttons.

    @param[out]	exceptionInfo	 The list of exceptions
*/
gcsPump_ApplyAccessToButtons(string sDeviceName, dyn_string &exceptionInfo)
{
	string sFunction = "";
	dyn_string dsFunctions;
	dyn_string dsUserAccess;
	int iRes;


	unGenericObject_GetFunctions(GCS_CONFIG_PUMP_DPT_NAME, dsFunctions, exceptionInfo);

	//to avoid problems with PVSS
	if (dynlen(dsFunctions) >= UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS)
		sFunction = dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS];

	if (sFunction != "")
	{
		iRes = evalScript(dsUserAccess, "dyn_string main(string deviceNameTemp, string sDpTypeTemp) {" +
					 "dyn_string dsUserAccessReturn;" +
					 "if (isFunctionDefined(\"" + sFunction  + "\"))" +
					 "    {" +
			 			sFunction  + "(deviceNameTemp, sDpTypeTemp, dsUserAccessReturn);" +
					 "    }" +
					 "return dsUserAccessReturn; }", makeDynString(), sDeviceName, GCS_CONFIG_PUMP_DPT_NAME);

    g_dsUserAccess=dsUserAccess;
	}
	else dynAppend(exceptionInfo, "gcsPump_ButtonRegisterCB:" + getCatStr("unGeneration","UNKNOWNFUNCTION"));
}











//---------------------------------------------------------------------------------------------------------------------------------------
// gcsPump_ButtonAnimationCB
/**
Animate buttons

@note This function works with an unicos button panel and needs global variables declared in this panel -
		"g_dsUserAccess" (dyn_string). Button identifier is added to this variable if user has access.
*/
gcsPump_ButtonAnimationCB(string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager,
	   string sDpStsReg01, bit32 stsReg01Value, string sDpStsReg01Bad, bool bStsReg01Bad,
	   string sDpStsReg02, bit32 stsReg02Value, string sDpStsReg02Bad, bool bstsReg02Bad)
{
	gcsPump_ButtonSetState(unGenericDpFunctions_getDpName(sDpStsReg01), "Pump", g_dsUserAccess,
			makeDynString(bLocked, sSelectedManager, stsReg01Value, bStsReg01Bad, stsReg02Value, bstsReg02Bad));
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsPump_ButtonUserAccess
/**
User access for buttons

	@param[in] 		sDpName	Device name
	@param[in] 		sDpType	dpType
	@param[out] 	dsAccess	List of allowed buttons
*/
gcsPump_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
	
	bool operator, expert, admin;
	dyn_bool dbPermissions;
	dyn_string exceptionInfo;
	string sOperAction, sExpAction, sAdminAction;
	bool bActionDefined = false;
	
	unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo);
	if((sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME) ||
	(sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME))
	bActionDefined=true;
	
	operator = dbPermissions[2];
	expert = dbPermissions[3];
	admin = dbPermissions[4];
	dsAccess = makeDynString();
	
	if (operator){
		if(bActionDefined)
			dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
		else
			dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_MANUAL_MODE,
			UN_FACEPLATE_BUTTON_ONOPEN_REQUEST, UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST, UN_FACEPLATE_BUTTON_PUMP_SET_FREQUENCY,
			UN_FACEPLATE_BUTTON_ACK_ALARM, UN_FACEPLATE_BUTTON_CONTROLLED_STOP_REQUEST, UN_FACEPLATE_BUTTON_SELECT));
	}
	if (expert){
		if(bActionDefined)
			dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
		else
			dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_FORCED_MODE, UN_FACEPLATE_MASKEVENT));
	}
	if(admin){
		if(bActionDefined)
			dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
        }
	
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsPump_UserLoginGetButtonState
/**
Function is called before the button set state when a user logs in.

	@param[in]	deviceName		The device name
	@param[in]	sType			Device type <b>(Currently not used)</b>
	@param[out]	dsReturnData	The data are returned here
	@note	This library works with an unicos button panel and need global variables declared in this panel
		- g_dsUserAccess, dyn_string, button identifier is added to this variable if user access
		- $sDpName, string, the device DP name
		- g_sFaceplateButtonType, string, faceplate type
*/
gcsPump_UserLoginGetButtonState(string sDeviceName, string sType, dyn_string &dsReturnData)
{
	unGenericObject_UserCBGetButtonState(sDeviceName, sType, dsReturnData);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsPump_ButtonSetState
/**
Set button state

	@param[in]	 deviceName		The device name
	@param[in]	 sDpType		Device type
	@param[in]	 dsUserAccess	User access
	@param[in]	 dsData			List of various parameters
*/
gcsPump_ButtonSetState(string sDeviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
	bool bSelected;
	bool bLocked;
	bool bStsReg01Bad;
	bool bStsReg02Bad;
	dyn_string dsButtons;
	int iButtonsNumber;
	string sLocalManager;
	string sSelectedManager;
	bit32 stsReg01Value;
	bit32 stsReg02Value;


	bLocked = dsData[1];
	sSelectedManager = dsData[2];
	stsReg01Value = dsData[3];
	bStsReg01Bad = dsData[4];
	stsReg02Value = dsData[5];
	bStsReg02Bad = dsData[6];

	dsButtons = makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_MANUAL_MODE,
							  UN_FACEPLATE_BUTTON_ONOPEN_REQUEST, UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST,
							  UN_FACEPLATE_BUTTON_FORCED_MODE, UN_FACEPLATE_BUTTON_PUMP_SET_FREQUENCY,
							  UN_FACEPLATE_BUTTON_CONTROLLED_STOP_REQUEST, UN_FACEPLATE_BUTTON_ACK_ALARM); // Each button except "Select" button
	//I have replaced Ack alarm interlock for ack alarm
	//ack alarm interlock constant is in the unicosObjects_declarations but is not active?????????
	iButtonsNumber = dynlen(dsButtons);

	// 1. Selection state
	sLocalManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
	bSelected = (sLocalManager == "S");

	// 2. Button state
	for(int i=1;i<=iButtonsNumber;i++)
	{
		bool bBit1Active;
		bool bBit2Active;
		bool bBit3Active;
		bool bButtonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0);	// User access

		switch (dsButtons[i])
			{
			case UN_FACEPLATE_BUTTON_FORCED_MODE:
				bBit1Active = !(getBit(stsReg01Value, UN_STSREG01_FOMOST) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				bBit2Active = !(getBit(stsReg01Value, UN_STSREG01_AUINHFMO) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				bBit3Active = !(getBit(stsReg01Value, UN_STSREG01_LDST) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				bButtonEnabled = bButtonEnabled && bSelected && !(bBit1Active || bBit2Active || bBit3Active);
				break;
			case UN_FACEPLATE_BUTTON_MANUAL_MODE:
				bBit1Active = !(getBit(stsReg01Value, UN_STSREG01_MMOST) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				bBit2Active = !(getBit(stsReg01Value, UN_STSREG01_AUINHMMO) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				bBit3Active = !(getBit(stsReg01Value, UN_STSREG01_LDST) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				bButtonEnabled = bButtonEnabled && bSelected && !(bBit1Active || bBit2Active || bBit3Active);
				break;

			case UN_FACEPLATE_BUTTON_PUMP_SET_FREQUENCY:
				bBit1Active = !(getBit(stsReg01Value, UN_STSREG01_AUMOST) == 0) && !(bStsReg01Bad || bStsReg02Bad); // ==1
				bBit2Active = !(getBit(stsReg01Value, UN_STSREG01_AUINHMMO) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				bBit3Active = !(getBit(stsReg01Value, UN_STSREG01_LDST) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				bButtonEnabled = bButtonEnabled && bSelected && !(bBit1Active || bBit2Active || bBit3Active);
				break;

			case UN_FACEPLATE_BUTTON_ONOPEN_REQUEST:
			case UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST:
			case UN_FACEPLATE_BUTTON_AUTO_MODE:
				bBit1Active = !(getBit(stsReg01Value, UN_STSREG01_AUMOST) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				bBit2Active = !(getBit(stsReg01Value, UN_STSREG01_LDST) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				bButtonEnabled = bButtonEnabled && bSelected && !(bBit1Active || bBit2Active);
				break;
			case UN_FACEPLATE_BUTTON_CONTROLLED_STOP_REQUEST:
				bBit1Active = !(getBit(stsReg01Value, UN_STSREG01_AUMOST) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				bButtonEnabled = bButtonEnabled && bSelected && !(bBit1Active || bBit2Active);
				break;
			case UN_FACEPLATE_BUTTON_ACK_ALARM:
			default:
				break;
			}
		setValue(UN_FACEPLATE_BUTTON_PREFIX + dsButtons[i], "enabled", bButtonEnabled);
	}

	//3. Button Select state
	unGenericObject_ButtonAnimateSelect(sLocalManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsPump_ButtonDisconnection
/**
Animate disconnection
*/
gcsPump_ButtonDisconnection()
{
	unGenericObject_ButtonDisconnect(makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_MANUAL_MODE,
		   UN_FACEPLATE_BUTTON_ONOPEN_REQUEST, UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST,
		   UN_FACEPLATE_BUTTON_FORCED_MODE, UN_FACEPLATE_BUTTON_PUMP_SET_FREQUENCY,
		   UN_FACEPLATE_BUTTON_ACK_ALARM, UN_FACEPLATE_BUTTON_CONTROLLED_STOP_REQUEST,
		   UN_FACEPLATE_BUTTON_SELECT));
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsPump_WidgetRegisterCB
/**
Register callback function

	@param[in]	sDp 	The device dp
	@param[in] 	bSystemConnected 	Flag
*/
gcsPump_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
	
	string sDeviceName;
	string sSystemName;
	string sPlcName;
	int iAction;
	dyn_string dsExceptionInfo;
	bool bRemote;


	sDeviceName = unGenericDpFunctions_getWidgetDpName ($sIdentifier);
	//case of disconnection
	if (sDeviceName == "")
	{
		sDeviceName = g_sDpName;
	}

	sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
	sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);

	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote) g_bSystemConnected = bSystemConnected;
	else g_bSystemConnected = true;

	if(sPlcName != "")
	{
		if(dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName))
		{	g_bUnSystemAlarmPlc = true;
		}
		else g_bUnSystemAlarmPlc = false;
	}
	else	g_bUnSystemAlarmPlc = false;


	unGenericObject_Connection(sDeviceName, g_bCallbackConnected, bSystemConnected, iAction, dsExceptionInfo);
	//DebugN("sDeviceName: "+sDeviceName + " iAction: "+iAction);
	switch (iAction)
	{
		case UN_ACTION_DISCONNECT:
			gcsPump_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPCONNECT:
			g_sDpName = sDeviceName;
			if(dpTypeName(sDeviceName) == GCS_CONFIG_PUMP_DPT_NAME) {
				gcsPump_WidgetConnect(sDeviceName, sPlcName, sSystemName);
			} else{
				g_bUnSystemAlarmPlc = false;
				gcsPump_WidgetDisconnection(g_sWidgetType);
			}
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			gcsPump_WidgetDisconnect(sDeviceName, sPlcName, sSystemName);
			gcsPump_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_NOTHING: break;
		default:
			break;
	}
}


//-----------------------------------------------------------------------------------------------------------------------------------------
// gcsPump_WidgetConnect
/**
dpConnect to the DataPoint

	@param[in]	deviceName	Device DP name
	@param[in]	sPlcName	PLC name
	@param[in]	sSystemName	System name
*/
gcsPump_WidgetConnect(string sDeviceName, string sPlcName, string sSystemName)
{
	int iRes;
 //DebugN("gcsPump_WidgetConnect");
 //DebugN("sDeviceName: "+sDeviceName +" g_bUnSystemAlarmPlc: "+g_bUnSystemAlarmPlc);
	if (g_bUnSystemAlarmPlc)
	{
		g_sLoadStFormat = dpGetFormat(sDeviceName + ".ProcessInput.LoadSt");
		g_sLoadStUnit = dpGetUnit(sDeviceName + ".ProcessInput.LoadSt");
		g_sPosStFormat = dpGetFormat(sDeviceName + ".ProcessInput.PosSt");
		g_sPosStUnit = dpGetUnit(sDeviceName + ".ProcessInput.PosSt");
		g_sAOutOVStFormat = dpGetFormat(sDeviceName + ".ProcessInput.AOutOVSt");
		g_sAOutOVStUnit = dpGetUnit(sDeviceName + ".ProcessInput.AOutOVSt");

		iRes = dpConnect("gcsPump_WidgetAnimationCB",
			sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
			sDeviceName + ".statusInformation.selectedManager",
			sDeviceName + ".ProcessInput.StsReg01",
			sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
			sDeviceName + ".ProcessInput.StsReg02",
			sDeviceName + ".ProcessInput.StsReg02:_online.._invalid",
			sDeviceName + ".ProcessInput.PosSt",
			sDeviceName + ".ProcessInput.PosSt:_online.._invalid",
			sDeviceName + ".ProcessInput.AOutOVSt",
			sDeviceName + ".ProcessInput.AOutOVSt:_online.._invalid",
			sDeviceName + ".ProcessInput.LoadSt",
			sDeviceName + ".ProcessInput.LoadSt:_online.._invalid",
			sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
			sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
			sDeviceName + ".eventMask");
		g_bCallbackConnected = (iRes >= 0);
	}
	else gcsPump_WidgetDisconnection(g_sWidgetType);
}


//--------------------------------------------------------------------------------------------------------------------------
// gcsPump_WidgetDisconnect
/**
dpDisconnect from the DataPoint

	@param[in]	sDeviceName	Device DP name
	@param[in]	sPlcName	PLC name
	@param[in]	sSystemName	System name
*/
gcsPump_WidgetDisconnect(string sDeviceName, string sPlcName, string sSystemName)
{
	int iRes;

	if (g_bUnSystemAlarmPlc)
	{
		iRes = dpDisconnect("gcsPump_WidgetAnimationCB",
				sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
				sDeviceName + ".statusInformation.selectedManager",
				sDeviceName + ".ProcessInput.StsReg01",
				sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
				sDeviceName + ".ProcessInput.StsReg02",
				sDeviceName + ".ProcessInput.StsReg02:_online.._invalid",
				sDeviceName + ".ProcessInput.PosSt",
				sDeviceName + ".ProcessInput.PosSt:_online.._invalid",
				sDeviceName + ".ProcessInput.AOutOVSt",
				sDeviceName + ".ProcessInput.AOutOVSt:_online.._invalid",
				sDeviceName + ".ProcessInput.LoadSt",
				sDeviceName + ".ProcessInput.LoadSt:_online.._invalid",
				sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
				sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled",
				sDeviceName + ".eventMask");
		g_bCallbackConnected = !(iRes >= 0);
	}
}


//------------------------------------------------------------------------------------------------------------------------------
// gcsPump_WidgetAnimationCB
/**
Animate widget
*/
gcsPump_WidgetAnimationCB(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
			 string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid,
			 string sDpStsReg02, bit32 bit32StsReg02, string sDp6, bool bStsReg02Invalid,
			 string sDpPosSt, float fPosSt, string sDp8, bool bPosStInvalid,
			 string sDpAOutOVSt, float fAOutOVSt, string sDp10, bool bAOutOVStInvalid,
			 string sDpLoadSt, float fLoadSt, string sDp12, bool bLoadStInvalid,
			 string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled, string sEvtMask, int iMask)
{
	string sAlarmLetter = "";
	string sAlarmColor = "";
	string sControlLetter = "";
	string sControlColor = "";
	string sWarningLetter = UN_WIDGET_TEXT_INVALID;
	string sWarningColor = "unDataNotValid";
	string sSelectionFrameColor = "";
	string sBodyColor = "unDataNotValid";
	string sBody1BkColor = "unWidget_Background";
	string sBody2BkColor = "unWidget_Background";
	bool bLockVisible;



	// A. Animate widget bodies and letters
	
	
	if (!bStsReg01Invalid && !bStsReg02Invalid)
	{
		//DebugN("if bStsReg01Invalid &  bStsReg02Invalid are not invalid set the corresponding  Warning letters!");
		// 1. Body animation
		sBodyColor = "unWidget_ControlStateAuto";
		// 1.2. Forced mode (lower medium priority)
    	if (getBit(bit32StsReg01, UN_STSREG01_MMOST) == 1)sBodyColor = "unWidget_ControlStateManualLocal";
    	if (getBit(bit32StsReg01, UN_STSREG01_LDST) == 1)sBodyColor = "unWidget_ControlStateManualLocal";
    	if (getBit(bit32StsReg01, UN_STSREG01_FOMOST) == 1)sBodyColor = "unWidget_ControlStateForced";

		// 1.3. Alarm (higher priority)
		if ((getBit(bit32StsReg01, UN_STSREG01_STARTIST) == 1) ||(getBit(bit32StsReg01, UN_STSREG01_STOPIST) == 1))
			sBodyColor = "unFaceplate_AlarmActive";

		if (getBit(bit32StsReg01, UN_STSREG01_ALUNACK) == 1)
			sBodyColor = "unWidget_AlarmNotAck";

		sBody2BkColor = sBodyColor;	//	1 1    or 0 0
		if (getBit(bit32StsReg01, UN_STSREG01_ONST) == 1 && getBit(bit32StsReg01, UN_STSREG01_OFFST) == 0)
		{
			//pump is ON
			sBody1BkColor = sBodyColor; 
			sBody2BkColor = sBodyColor;
			sBodyColor = "unWidget_AlternativeColor";
		}
		if (getBit(bit32StsReg01, UN_STSREG01_ONST) == 0 && getBit(bit32StsReg01, UN_STSREG01_OFFST) == 1)
		{
			//pump is OFF
			sBody1BkColor = "unWidget_Background"; 
			sBody2BkColor = "unWidget_Background";
		}

		// 2. Alarm animation
		unGenericObject_WidgetAlarmTextAnimation(bit32StsReg01, makeDynInt(UN_STSREG01_STARTIST, UN_STSREG01_STOPIST),
			 makeDynString(UN_WIDGET_TEXT_ALARM_STARTIST, UN_WIDGET_TEXT_ALARM_STOPIST), sAlarmLetter, sAlarmColor);

		// 3. Control state animation
		unGenericObject_WidgetControlStateAnimation(bit32StsReg01, makeDynInt(UN_STSREG01_MMOST, UN_STSREG01_LDST, UN_STSREG01_FOMOST),
  		    makeDynString(UN_WIDGET_TEXT_CONTROL_MANUAL, UN_WIDGET_TEXT_CONTROL_LOCAL,UN_WIDGET_TEXT_CONTROL_FORCED), sControlLetter, sControlColor);

		// 4. Warning animation I/O Error, I/O Simulated, Manual Auto, Position Warning
		unGenericObject_WidgetWarningAnimation(bit32StsReg01, makeDynInt(UN_STSREG01_AUMRW, UN_STSREG01_POSALW, UN_STSREG01_IOSIMUW, UN_STSREG01_IOERRORW),
		  				 makeDynString(UN_WIDGET_TEXT_WARNING_DEFAULT, UN_WIDGET_TEXT_WARNING_DEFAULT, UN_WIDGET_TEXT_WARNING_SIMU, UN_WIDGET_TEXT_WARNING_ERROR),
		   					sWarningLetter, sWarningColor);
	}


	string sColor1 = "unDisplayValue_Status";
	string sColor2 = "unDisplayValue_Output";
	string sColor3 = "unDisplayValue_Status";
	
	// 2.1 if the device is not correctly running ( the iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value)
	// or the cheking is not enabled then set the letter to O and the color to not valid data (O is of less priority than N)
	if ((!bPlcEnabled) || (iAlValue > c_unSystemIntegrity_no_alarm_value))
	{
		sWarningLetter = UN_WIDGET_TEXT_OLD_DATA;
		sWarningColor = "unDataNotValid";
	}

	
	if (bStsReg01Invalid || bStsReg02Invalid || bPosStInvalid || bAOutOVStInvalid || bLoadStInvalid)
	{
		sWarningLetter = UN_WIDGET_TEXT_INVALID;
		sWarningColor = "unDataNotValid";
	}
	
	
	if(bStsReg01Invalid || bStsReg02Invalid) sBodyColor = "unDataNotValid";
	if(bPosStInvalid) sColor1 = "unDataNotValid";
	if(bAOutOVStInvalid) sColor2 = "unDataNotValid";
	if(bLoadStInvalid)  sColor3 = "unDataNotValid";


	string sFormattedValue1 = unGenericObject_FormatValue(g_sPosStFormat, fPosSt) + "  " + g_sPosStUnit;
	string sFormattedValue2 = unGenericObject_FormatValue(g_sAOutOVStFormat, fAOutOVSt) + "  " + g_sAOutOVStUnit;
	string sFormattedValue3 = unGenericObject_FormatValue(g_sLoadStFormat, fLoadSt) + "  " + g_sLoadStUnit;

	unGenericObject_WidgetSelectAnimation(bLocked, sSelectedManager, sSelectionFrameColor, bLockVisible);
	if(g_bSystemConnected)
	{
		setMultiValue("WarningText", "text", sWarningLetter, "WarningText", "foreCol", sWarningColor,
			"AlarmText", "text", sAlarmLetter, "AlarmText", "foreCol", sAlarmColor,
			"ControlStateText", "text", sControlLetter, "ControlStateText", "foreCol", sControlColor,
			"Body1","fill","[solid]", "Body1", "foreCol", sBodyColor, "Body1", "backCol", sBody1BkColor,
			"Body2","fill","[solid]", "Body2", "foreCol", sBodyColor, "Body2", "backCol", sBody2BkColor,
			"Display1","foreCol",sColor1, "Display1","text", sFormattedValue1,
			"Display2","foreCol",sColor2, "Display2","text", sFormattedValue2,
			"Display3","foreCol",sColor3, "Display3","text", sFormattedValue3,
			"SelectArea", "foreCol", sSelectionFrameColor, "LockBmp", "visible", bLockVisible, "WidgetArea", "visible", true);
	}

	// display event state: small "e" on the widget
	if(g_bSystemConnected)
	{
		if(iMask == 0) eventState.visible = true;
		else eventState.visible	= false;
	}
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsPump_WidgetDisconnection
/**
Animate widget disconnection

	@param[in] sWidgetType Widget type. ex: "heater"
*/
gcsPump_WidgetDisconnection(string sWidgetType)
{
	//DebugN("gcsPump_WidgetDisconnection");
	unGenericObject_WidgetDisconnection(sWidgetType, "Pump");
	eventState.visible = false;
}


//---------------------------------------------------------------------------------------------------------------------------------------
// unGenericObject_WidgetPmpDisconnection
/**
Animate Pump disconnection
*/
unGenericObject_WidgetPmpDisconnection()
{
	setMultiValue("Body1", "foreCol", "unDataNoAccess", "Body1", "backCol", "unWidget_Background", "Body1", "fill", "[solid]",
				"Body2", "foreCol", "unDataNoAccess", "Body2", "backCol", "unWidget_Background", "Body2", "fill", "[solid]",
				"Display1", "foreCol", "unDataNoAccess", "Display2", "foreCol", "unDataNoAccess",  "Display3", "foreCol", "unDataNoAccess");
}








//@}
