/**@name LIBRARY: gcsWordComputed.ctl

@author: Artem Burmyakov (IT-CO-FE)
@author: Nikolay Kulman (IT-CO-FE)
@author: Francisco Mico (IT-CO-FE)
@date 17.06.2003
@brief This library contains functions to animate WordComputed objects.
Modifications History:
	$Log: gcsWordComputed.ctl,v $
	Revision 1.21  2009/04/02 12:12:37  gthomas
	in _ProvideAccessToButtons(): removed exceptionInfo as returns errors in log when monitor or no user are logged in
	
	Revision 1.20  2009/03/25 15:48:44  gamx
	Bug in animation fixed (test cases 2-5 of widget animation):
	Standard Unicos function to animate widget disconnection cannot be used as AnalogComputed does not have "AlarmText" graphical object
	on the widget panel.

	Revision 1.19  2008/11/12 08:30:08  gthomas
	-removed function calls to _ButtonUserAccessGcs and _ButtonUserAccessUn

	Revision 1.18  2008/10/15 14:54:48  gthomas
	-Removed the <if statement> of the dpConnect in _ButtonRegisterCB function
	to fix problem with select/deselect button animation.

	Revision 1.17  2008/10/15 07:29:23  gthomas
	-removed unused EventList config functions

	Revision 1.16  2008/09/12 13:38:06  gthomas
	Fixed typo in <.ProcessInput.StsReg01:_online.._invalid> attribute declaration
	in Faceplate connect fct. which caused error in faceplate animation in PVSS 3.6.

	Revision 1.15  2008/07/23 09:13:31  gthomas
	added missing ExceptionInfo in _ProvideAccessToButtons() function

	Revision 1.14  2008/07/18 07:16:55  gthomas
	added new Access control functionality for unicos-4-1
	modified  xx_ButtonUserAccess function.

	Revision 1.13  2008/06/18 14:54:51  aabadz
	Changes done following the recomendations from AB-CO (from Herve)

	Revision 1.12  2008/03/17 10:52:36  aabadz
	Fixed a problem found in animation when we have not database.

	Revision 1.11  2008/01/22 12:53:24  aabadz
	Fixed error mesage text.

	Revision 1.10  2008/01/21 14:04:07  gamx
	version after the final test:
	bugs related to plc dp name definition are fixed

	Revision 1.9  2008/01/16 15:57:13  aabadz
	Some sintactic errors fixed

	Revision 1.8  2008/01/15 11:00:34  aabadz
	Modifed ObjectListGetValueTime because the code that we had before return nothing instead of  the time and the value

	Revision 1.7  2008/01/11 10:18:20  gamx
	Reviewed:
	1) functions organisation (complex ones are splitted);
	2) naming;
	3) exception handling;
	4) documention;

	Revision 1.6  2007/10/30 16:22:33  gthomas
	Introduced the CVS keyword $Log to track changes made in the library

*/
/*
Internal Functions :
	gcsWordComputed_ObjectListGetValueTime (string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
	gcsWordComputed_MenuConfiguration (string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
	gcsWordComputed_HandleMenu (string sDeviceName, string sDpType, dyn_string menuList, int iMenuAnswer)
	gcsWordComputed_FaceplateStatusRegisterCB (string sDp, bool bSystemConnected)
	gcsWordComputed_FaceplateConnect (string sDeviceName)
	gcsWordComputed_FaceplateDisconnect (string sDeviceName)
	gcsWordComputed_FaceplateStatusAnimationCB (string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid, ... )
	gcsWordComputed_AnimatePlcDisconnection ()
	gcsWordComputed_ButtonRegisterCB (string sDp, bool bSystemConnected)
	gcsWordComputed_ButtonAnimationCB (string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager, string sDpStsReg01, bit32 stsReg01Value, string sDpStsReg01Bad, bool bStsReg01Bad)
	gcsWordComputed_ButtonUserAccess (string sDpName, string sDpType, dyn_string &dsAccess)
	gcsWordComputed_UserLoginGetButtonState (string sDeviceName, string sType, dyn_string &dsReturnData)
	gcsWordComputed_ButtonSetState (string sDeviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
	gcsWordComputed_ButtonDisconnection ()
	gcsWordComputed_WidgetRegisterCB (string sDp, bool bSystemConnected)
	gcsWordComputed_WidgetConnect (string sDeviceName, string sPlcName, string sSystemName)
	gcsWordComputed_WidgetDisconnect (string sDeviceName, string sPlcName, string sSystemName)
	gcsWordComputed_WidgetAnimationCB (string sDpLock, bool bLocked, string sDpManager, string sSelectedManager, string sDpStsReg01, bit32 bit32StsReg01, ... )
	gcsWordComputed_WidgetDisconnection (string sWidgetType)
	
*/


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordComputed_ObjectListGetValueTime
/**
Function called from the objectList to return the time and value

	@param[in]	sDeviceName		Device name
	@param[in]	sDeviceType		Unicos object
	@param[out]	dsReturnData	Return value
*/
gcsWordComputed_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
	//object list is going to disappear, function will be removed
	dsReturnData = makeDynString("", "");
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordComputed_MenuConfiguration
/**
Configuration of popup menu

	@param[in]  	sDpName 	Device name
	@param[in]	sDpType 	Unicos object
	@param[in]	dsAccessOk	List of authorized action
	@param[out] 	menuList 	Configuration of menu. Use defined constants UN_POPUPMENU_***
*/
gcsWordComputed_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{
	dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT,
		 UN_POPUPMENU_FACEPLATE_TEXT, UN_POPUPMENU_TREND_TEXT);

	unGenericObject_addSelectToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	unGenericObject_addUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	unGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	unGenericObject_addTrendActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordComputed_HandleMenu
/**
Handle the answer of the popup menu

	@param[in]	sDeviceName 	The device dp name
	@param[in]	sDpType 	The device dp type
	@param[in]	dsMenuList 	List of requested action
	@param[in]	iMenuAnswer 	Selected action from the dsMenuList
*/
gcsWordComputed_HandleMenu(string sDeviceName, string sDpType, dyn_string dsMenuList, int iMenuAnswer)
{
	dyn_string exceptionInfo;

	gcsGenericObject_HandleEventMaskMenu(sDeviceName, iMenuAnswer, makeDynString(".ProcessInput.evStsReg01"), exceptionInfo);
	if (dynlen(exceptionInfo) <= 0) unGenericObject_HandleUnicosMenu(sDeviceName, sDpType, dsMenuList, iMenuAnswer, exceptionInfo);

	if (dynlen(exceptionInfo) > 0) unSendMessage_toExpertException("Right Click",exceptionInfo);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordComputed_FaceplateStatusRegisterCB
/**
Registers faceplate callback function

	@param[in]	sSystemStatus 		The device dp
	@param[in] 	bSystemConnected 	Flag
*/
gcsWordComputed_FaceplateStatusRegisterCB(string sSystemStatus, bool bSystemConnected)
{
	string sDeviceName;
	string sSystemName;
	string sPlcName;
	string sPlcDpName;
	int iAction;
	dyn_string exceptionInfo;
	bool bRemote;

	sDeviceName = unGenericDpFunctions_getDpName($sDpName);
	if (sDeviceName)
	{
		sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
		unDistributedControl_isRemote(bRemote, sSystemName);

		if(bRemote) g_bSystemConnected = bSystemConnected;
		else g_bSystemConnected = true;

		sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);
		gcsConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
		if (dynlen(exceptionInfo) <= 0) g_bUnSystemAlarmPlc = true;
		else g_bUnSystemAlarmPlc = false;

		unGenericObject_Connection(sDeviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
		switch (iAction)
		{
			case UN_ACTION_DISCONNECT:
				gcsWordComputed_AnimatePlcDisconnection();
				break;
			case UN_ACTION_DPCONNECT:
				gcsWordComputed_FaceplateConnect(sDeviceName);
				break;
			case UN_ACTION_DPDISCONNECT_DISCONNECT:
				gcsWordComputed_FaceplateDisconnect(sDeviceName);
				gcsWordComputed_AnimatePlcDisconnection();
				break;
			case UN_ACTION_NOTHING:
			default:
				break;
		}
	}
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordComputed_FaceplateConnect
/**
Connects callback functions to animate a faceplate.
As it is not possible to pass constant values in callback functions and to get them inside callback function,
global constants are used.

	@param[in]	sDeviceName	Device DP name
*/
gcsWordComputed_FaceplateConnect(string sDeviceName)
{
	int iAction;
	int iRes = -1;
	dyn_string exceptionInfo;
	string sPlcName;
	string sPlcDpName;
	string sSystemName;

	sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
	g_bCallbackConnected = false;

	sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);
	gcsConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0) g_bUnSystemAlarmPlc = true;
	else g_bUnSystemAlarmPlc = false;

	if (g_bUnSystemAlarmPlc)
	{
		g_sPosOvFormat = dpGetFormat(sDeviceName + ".ProcessInput.PosOV");
		g_sPosOvUnit = dpGetUnit(sDeviceName + ".ProcessInput.PosOV");
		g_sAuPosRStFormat = dpGetFormat(sDeviceName + ".ProcessInput.AuPosRSt");
		g_sAuPosRStUnit = dpGetUnit(sDeviceName + ".ProcessInput.AuPosRSt");

		iRes = dpConnect("gcsWordComputed_FaceplateStatusAnimationCB",
			sDeviceName + ".ProcessInput.StsReg01",
			sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
			sDeviceName + ".ProcessInput.PosOV",
			sDeviceName + ".ProcessInput.PosOV:_online.._invalid",
			sDeviceName + ".ProcessInput.AuPosRSt",
			sDeviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
			sPlcDpName + ".alarm",
			sPlcDpName + ".enabled",
			sDeviceName + ".eventMask");
	}
	else gcsWordComputed_AnimatePlcDisconnection();

	g_bCallbackConnected = (iRes >= 0);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordComputed_FaceplateDisconnect
/**
Do faceplate dpDisconnect

	@param[in]	sDeviceName	Device DP name
*/
gcsWordComputed_FaceplateDisconnect(string sDeviceName)
{
	int iRes = -1;
	string sPlcName, sSystemName;
	string sPlcDpName;
	dyn_string exceptionInfo;

	g_bCallbackConnected = false;
	sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
	sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);

	gcsConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0) g_bUnSystemAlarmPlc = true;
	else g_bUnSystemAlarmPlc = false;

	if (g_bUnSystemAlarmPlc)
	{
		iRes = dpDisconnect("gcsWordComputed_FaceplateStatusAnimationCB",
			 sDeviceName + ".ProcessInput.StsReg01",
			 sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
			 sDeviceName + ".ProcessInput.PosOV",
			 sDeviceName + ".ProcessInput.PosOV:_online.._invalid",
			 sDeviceName + ".ProcessInput.AuPosRSt",
			 sDeviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
			 sPlcDpName + ".alarm",
			 sPlcDpName + ".enabled",
			 sDeviceName + ".eventMask");

		g_bCallbackConnected = !(iRes >= 0);
	}
	else
	{
		gcsWordComputed_AnimatePlcDisconnection();
		g_bCallbackConnected = false;
	}
}




//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordComputed_FaceplateStatusAnimationCB
/**
Animate faceplate
*/
gcsWordComputed_FaceplateStatusAnimationCB(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
	  string sDpPosOV, int iPosOV, string sDp4, bool bPosOVInvalid,
	  string sDpAuto, int iAuPosRSt, string sDp6, bool bAuPosRStInvalid,
	  string sDpAl, int iValAlarm, string sDpEnabled, bool bSystemIntegrityAlarmEnabled,
	  string sDpEvent, int iMask)
{
	if(g_bSystemConnected)
		gcsGenericObject_DisplayValue(g_sPosOvFormat, g_sPosOvUnit, iPosOV, "StatusPosition", "unDisplayValue_Status",
			bPosOVInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		gcsGenericObject_DisplayValue(g_sAuPosRStFormat, g_sAuPosRStUnit, iAuPosRSt, "RequestAuto", "unDisplayValue_Request",
			bAuPosRStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("OperationMode_Auto", getBit(bit32StsReg01, UN_STSREG01_AUMOST), 1, "unFaceplate_OperationModeActive",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("OperationMode_Manual", getBit(bit32StsReg01, UN_STSREG01_MMOST), 1, "unFaceplate_OperationModeActive",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("OperationMode_Forced", getBit(bit32StsReg01, UN_STSREG01_FOMOST), 1, "unFaceplate_OperationModeActive",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("Warning_IOError", getBit(bit32StsReg01, UN_STSREG01_IOERRORW), 1, "unFaceplate_WarningActive",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("Warning_IOSimulated", getBit(bit32StsReg01, UN_STSREG01_IOSIMUW), 1, "unFaceplate_WarningActive",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("Warning_ManualAuto", getBit(bit32StsReg01, UN_STSREG01_AUMRW), 1, "unFaceplate_WarningActive",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("eventState", iMask, 1, "",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value),
			"unFaceplate_OperationModeActive");
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordComputed_AnimatePlcDisconnection
/**
Animate faceplate disconnection
*/
gcsWordComputed_AnimatePlcDisconnection()
{
	unGenericObject_ColorBoxDisconnect(makeDynString("OperationMode_Auto", "OperationMode_Forced", "OperationMode_Manual",
		 "Warning_IOError", "Warning_IOSimulated", "Warning_ManualAuto", "eventState"));
	unGenericObject_DisplayValueDisconnect(makeDynString("RequestAuto", "StatusPosition"));
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordComputed_ButtonRegisterCB
/**
Register callback function

	@param[in]	sSystemStatus 	The device dp
	@param[in] 	bSystemConnected 	Flag
*/
gcsWordComputed_ButtonRegisterCB(string sSystemStatus, bool bSystemConnected)
{
	string sDeviceName, sDpType, sFunction;
	int iAction, iRes, iManReg01;
	dyn_string exceptionInfo, dsFunctions;

	sDeviceName = unGenericDpFunctions_getDpName($sDpName);

	if (sDeviceName)
	{
		unGenericObject_Connection(sDeviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
		switch (iAction)
		{
			case UN_ACTION_DISCONNECT:
				gcsWordComputed_ButtonDisconnection();
				break;
			case UN_ACTION_DPCONNECT:
				iRes = dpGet(sDeviceName + ".ProcessOutput.ManReg01", iManReg01);
				if ((iRes < 0) || (iManReg01 != 0))
				{
					dpSet(sDeviceName + ".ProcessOutput.ManReg01", 0);
				}

				sDpType=GCS_CONFIG_WORDCOMPUTED_DPT_NAME;
				unGenericObject_GetFunctions(sDpType, dsFunctions, exceptionInfo);
				if (dynlen(exceptionInfo) <= 0) gcsWordComputed_ProvideAccessToButtons(sDeviceName, sDpType, dsFunctions, exceptionInfo);
				gcsWordComputed_ButtonsConnect(sDeviceName);
				break;
			case UN_ACTION_DPDISCONNECT_DISCONNECT:
				gcsWordComputed_ButtonsDisconnect(sDeviceName);
				gcsWordComputed_ButtonDisconnection();
				break;
			case UN_ACTION_NOTHING:break;
			default:
				break;
		}

	}
}





//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordComputed_ProvideAccessToButtons
/**
Defines the list of buttons available for a user. The result is passed in global variable.
	@param[in]	sDeviceName	Name of the device
	@param[in]	sDpType		Device Type
	@param[in] 	dsFunctions	A list of functions for a device, specified in device internal configuration.
	@param[out]	exceptionInfo	A list of exceptions.
*/
gcsWordComputed_ProvideAccessToButtons(string sDeviceName, string sDpType, dyn_string dsFunctions, dyn_string &exceptionInfo)
{
	string sFunction;
	dyn_string dsUserAccess;

	if (dynlen(dsFunctions) >= UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS)
		sFunction = dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS];
	else
	{
		dynAppend(exceptionInfo, "gcsWordComputed_ProvideAccessToButtons: the list of functions is not correct - number of functions is less than expected");
		return;
	}

	if (sFunction != "")
	{
		evalScript(dsUserAccess, "dyn_string main(string deviceNameTemp, string sDpTypeTemp) {" +
					 "dyn_string dsUserAccessReturn;" +
					 "if (isFunctionDefined(\"" + sFunction  + "\"))" +
					 "    {" +
						sFunction  + "(deviceNameTemp, sDpTypeTemp, dsUserAccessReturn);" +
					 "    }" +
					 "return dsUserAccessReturn; }", makeDynString(), sDeviceName, sDpType);

		g_dsUserAccess=dsUserAccess;
	}
	else
		fwException_raise(exceptionInfo,"ERROR", "WordComputed_ButtonRegisterCB:" + getCatStr("unGeneration","UNKNOWNFUNCTION"), "EMPTY");
}






//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordComputed_ButtonsConnect
/**
Connects callback function to animate buttons states
	@param[in] 	sDeviceName	The name of the device
*/
gcsWordComputed_ButtonsConnect(string sDeviceName)
{
	int iRes = -1;

	iRes = dpConnect("gcsWordComputed_ButtonAnimationCB",
			 sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
			 sDeviceName + ".statusInformation.selectedManager",
			 sDeviceName + ".ProcessInput.StsReg01",
			 sDeviceName + ".ProcessInput.StsReg01:_online.._aut_inv");

	g_bCallbackConnected = (iRes >= 0);
}





//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordComputed_ButtonsDisconnect
/**
Disconnects callback function to animate buttons states
	@param[in] 	sDeviceName	The name of the device
*/
gcsWordComputed_ButtonsDisconnect(string sDeviceName)
{
	int iRes = -1;

	iRes = dpDisconnect("gcsWordComputed_ButtonAnimationCB",
			 sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
			 sDeviceName + ".statusInformation.selectedManager",
			 sDeviceName + ".ProcessInput.StsReg01",
			 sDeviceName + ".ProcessInput.StsReg01:_online.._aut_inv");

	g_bCallbackConnected = !(iRes >= 0);
}











//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordComputed_ButtonAnimationCB
/**
Animate buttons

@note This function works with an unicos button panel and needs global variables declared in this panel -
		"g_dsUserAccess" (dyn_string). Button identifier is added to this variable if user has access.
*/
gcsWordComputed_ButtonAnimationCB(string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager,
    string sDpStsReg01, bit32 stsReg01Value, string sDpStsReg01Bad, bool bStsReg01Bad)
{
	gcsWordComputed_ButtonSetState(unGenericDpFunctions_getDpName(sDpStsReg01), GCS_CONFIG_WORDCOMPUTED_DPT_NAME, g_dsUserAccess,
		 makeDynString(bLocked, sSelectedManager, stsReg01Value, bStsReg01Bad, 0, 0));
}




//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordComputed_ButtonUserAccess
/**
User access for buttons

	@param[in] 		sDpName	Device name
	@param[in] 		sDpType	dpType
	@param[out] 	dsAccess	List of allowed buttons
*/
gcsWordComputed_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
	
	bool operator, expert, admin;
	dyn_bool dbPermissions;
	dyn_string exceptionInfo;
	string sOperAction, sExpAction, sAdminAction;
	bool bActionDefined = false;
	
	unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo);
	if((sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME) ||
	(sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME))
	bActionDefined=true;
	operator = dbPermissions[2];
	expert = dbPermissions[3];
	admin = dbPermissions[4];
	dsAccess = makeDynString();
	
	if (operator){
		if(bActionDefined)
			dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
		else
			dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_MANUAL_MODE,
			UN_FACEPLATE_BUTTON_WC_SET_VALUE, UN_FACEPLATE_BUTTON_SELECT, UN_FACEPLATE_BUTTON_WC_SET_STRING));
	}
	if (expert){
		if(bActionDefined)
			dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
		else
			dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_FORCED_MODE, UN_FACEPLATE_MASKEVENT));
	}
	if(admin){
		if(bActionDefined)
			dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
        }

	
}

//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordComputed_UserLoginGetButtonState
/**
Function is called before the button set state when a user logs in.

	@param[in]	sDeviceName		The device name
	@param[in]	sType			Device type <b>(Currently not used)</b>
	@param[out]	dsReturnData	The data are returned here
	@note	This library works with an unicos button panel and need global variables declared in this panel
		- g_dsUserAccess, dyn_string, button identifier is added to this variable if user access
		- $sDpName, string, the device DP name
		- g_sFaceplateButtonType, string, faceplate type
*/
gcsWordComputed_UserLoginGetButtonState(string sDeviceName, string sType, dyn_string &dsReturnData)
{
	unGenericObject_UserCBGetButtonState(sDeviceName, sType, dsReturnData);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordComputed_ButtonSetState
/**
Set button state

	@param[in]	 sDeviceName		The device name
	@param[in]	 sDpType		Device type
	@param[in]	 dsUserAccess	User access
	@param[in]	 dsData			List of various parameters
**/
gcsWordComputed_ButtonSetState(string sDeviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
	bool bSelected, bButtonEnabled, bit1, bit2, bit3;
	dyn_string dsButtons;
	int i, iLength;
	string sLocalManager;
	bool bLocked, bStsReg01Bad, bStsReg02Bad;
	string sSelectedManager;
	bit32 bit32StsReg01Value, bit32StsReg02Value;

	if (dynlen(dsData) >= 4)
	{
		bLocked = dsData[1];
		sSelectedManager = dsData[2];
		bit32StsReg01Value = dsData[3];
		bStsReg01Bad = dsData[4];
		bit32StsReg02Value = dsData[5];
		bStsReg02Bad = dsData[6];
	}

	dsButtons = makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_MANUAL_MODE, UN_FACEPLATE_BUTTON_WC_SET_VALUE,
		  			UN_FACEPLATE_BUTTON_FORCED_MODE); // Each button except "Select" button
	iLength = dynlen(dsButtons);
	// 1. Selection state
	sLocalManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
	bSelected = (sLocalManager == "S");
	// 2. Button state
	for(i = 1; i <= iLength; i++)
	{
		bButtonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0);	// User access
		switch (dsButtons[i])
			{
			case UN_FACEPLATE_BUTTON_FORCED_MODE:
				bit1 = !(getBit(bit32StsReg01Value, UN_STSREG01_FOMOST) == 0) && !(bStsReg01Bad);
				bit2 = !(getBit(bit32StsReg01Value, UN_STSREG01_AUINHFMO) == 0) && !(bStsReg01Bad);
				bButtonEnabled = bButtonEnabled && bSelected && !(bit1 || bit2);
				break;
			case UN_FACEPLATE_BUTTON_MANUAL_MODE:
				bit1 = !(getBit(bit32StsReg01Value, UN_STSREG01_MMOST) == 0) && !(bStsReg01Bad);
				bit2 = !(getBit(bit32StsReg01Value, UN_STSREG01_AUINHMMO) == 0) && !(bStsReg01Bad);
				bButtonEnabled = bButtonEnabled && bSelected && !(bit1 || bit2);
				break;
			case UN_FACEPLATE_BUTTON_AUTO_MODE:
				bit1 = !(getBit(bit32StsReg01Value, UN_STSREG01_AUMOST) == 0) && !(bStsReg01Bad);
				bButtonEnabled = bButtonEnabled && bSelected && !(bit1);
				break;
			case UN_FACEPLATE_BUTTON_WC_SET_VALUE:
				bit1 = !(getBit(bit32StsReg01Value, UN_STSREG01_AUMOST) == 0) && !(bStsReg01Bad);
				bit2 = !(getBit(bit32StsReg01Value, UN_STSREG01_AUINHFMO) == 0) && !(bStsReg01Bad);
				bit3 = !(getBit(bit32StsReg01Value, UN_STSREG01_AUINHMMO) == 0) && !(bStsReg01Bad);
				bButtonEnabled = bButtonEnabled && bSelected && !(bit1 || bit2 || bit3);
				break;
			default:
				break;
			}
		setValue(UN_FACEPLATE_BUTTON_PREFIX + dsButtons[i], "enabled", bButtonEnabled);
		}
	// 3. Button Select state
	unGenericObject_ButtonAnimateSelect(sLocalManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordComputed_ButtonDisconnection
/**
Animate disconnection
**/
gcsWordComputed_ButtonDisconnection()
{
	unGenericObject_ButtonDisconnect(makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_MANUAL_MODE, UN_FACEPLATE_BUTTON_WC_SET_VALUE,
		   UN_FACEPLATE_BUTTON_FORCED_MODE, UN_FACEPLATE_BUTTON_SELECT));
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordComputed_WidgetRegisterCB
/**
Register callback function

	@param[in]	sDp 	The device dp
	@param[in] 	bSystemConnected 	Flag
*/
gcsWordComputed_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
	string sDeviceName, sPlcName, sSystemName;
	string sPlcDpName;
	int iAction;
	dyn_string exceptionInfo;
	bool bRemote;

	sDeviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
	if (sDeviceName == "")	// In case of disconnection
	{
		sDeviceName = g_sDpName;
	}

	sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);

	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote) g_bSystemConnected = bSystemConnected;
	else g_bSystemConnected = true;

	sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);
	gcsConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0) g_bUnSystemAlarmPlc = true;
	else g_bUnSystemAlarmPlc = false;

	unGenericObject_Connection(sDeviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
	{
		case UN_ACTION_DISCONNECT:
			gcsWordComputed_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPCONNECT:
			g_sDpName = sDeviceName;
			if(dpTypeName(sDeviceName) == GCS_CONFIG_WORDCOMPUTED_DPT_NAME)
			{
				gcsWordComputed_WidgetConnect(sDeviceName, sPlcDpName, sSystemName);
			}
			else
			{
				g_bUnSystemAlarmPlc = false;
				gcsWordComputed_WidgetDisconnection(g_sWidgetType);
			}
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			gcsWordComputed_WidgetDisconnect(sDeviceName, sPlcDpName, sSystemName);
			gcsWordComputed_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
	}
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordComputed_WidgetConnect
/**
dpConnect to the DataPoint

	@param[in]	sDeviceName	Device DP name
	@param[in]	sPlcDpName	PLC dp name
	@param[in]	sSystemName	System name
*/
gcsWordComputed_WidgetConnect(string sDeviceName, string sPlcDpName, string sSystemName)
{
	int iRes = -1;

	if(g_bUnSystemAlarmPlc)
	{
		string sDP = sDeviceName + ".ProcessInput.PosOV";
		g_sPosOvFormat = dpGetFormat(sDP);
		g_sPosOvUnit = dpGetUnit(sDP);
		g_bLabelsAvailable = gcsGenericObject_GetLabelsFromString(dpGetDescription(sDP), g_diWcValues, g_dsWcLabels);

		iRes = dpConnect("gcsWordComputed_WidgetAnimationCB",
			sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
			sDeviceName + ".statusInformation.selectedManager",
			sDeviceName + ".ProcessInput.StsReg01",
			sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
			sDeviceName + ".ProcessInput.PosOV",
			sDeviceName + ".ProcessInput.PosOV:_online.._invalid",
			sPlcDpName + ".alarm",
			sPlcDpName + ".enabled",
			sDeviceName + ".eventMask");
		g_bCallbackConnected = (iRes >= 0);
	}
	else gcsWordComputed_WidgetDisconnection (g_sWidgetType);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordComputed_WidgetDisconnect
/**
dpDisconnect from the DataPoint

	@param[in]	sDeviceName	Device DP name
	@param[in]	sPlcDpName	PLC dp name
	@param[in]	sSystemName	System name
*/
gcsWordComputed_WidgetDisconnect(string sDeviceName, string sPlcDpName, string sSystemName)
{
	int iRes = -1;

	if (g_bUnSystemAlarmPlc)
	{
		iRes = dpDisconnect("gcsWordComputed_WidgetAnimationCB",
			sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
			sDeviceName + ".statusInformation.selectedManager",
			sDeviceName + ".ProcessInput.StsReg01",
			sDeviceName + ".ProcessInput.StsReg01:_online.._invalid",
			sDeviceName + ".ProcessInput.PosOV",
			sDeviceName + ".ProcessInput.PosOV:_online.._invalid",
			sPlcDpName + ".alarm",
			sPlcDpName + ".enabled",
			sDeviceName + ".eventMask");
		g_bCallbackConnected = !(iRes >= 0);
	}
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordComputed_WidgetAnimationCB
/**
Animate widget
*/
gcsWordComputed_WidgetAnimationCB(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
		  string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid,
		  string sDpPosOV, int iPosOV, string sDp6, bool bPosOVInvalid,
		  string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled,
		  string sEventMask, int iMask)
{
	gcsGenericObject_WidgetXC_Animation(bLocked, sSelectedManager, bit32StsReg01, bStsReg01Invalid,
		  sDpPosOV, iPosOV, bPosOVInvalid, bStsReg01Invalid || bPosOVInvalid,
		  iAlValue, bPlcEnabled, sEventMask, iMask);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordComputed_WidgetDisconnection
/**
Animate widget disconnection

	@param[in] sWidgetType Widget type. ex: "heater"
*/
gcsWordComputed_WidgetDisconnection(string sWidgetType)
{
	//Standard Unicos function to animate widget disconnection cannot be used
	//as WordComputed does not have "AlarmText" graphical object
	//on the widget panel.
	//unGenericObject_WidgetDisconnection(sWidgetType, GCS_CONFIG_WORDCOMPUTED_DPT_NAME);
	
	setMultiValue("Body1", "foreCol", "unDataNoAccess",
		      "Body1", "backCol", "unWidget_Background",
		      "Body1", "fill", "[solid]",
		      "WarningText", "text", "", 
		      "ControlStateText", "text", "",
		      "SelectArea", "foreCol", "",
		      "LockBmp", "visible", false,
		      "WidgetArea", "visible", false);	
	
	eventState.visible	= false;
}



//@}
