/**@name LIBRARY: gcsWordParameter.ctl

@author: Nikolay Kulman (IT-CO-FE)
@author: Teodora Miteva (IT-CO-FE)
@date 01.11.2004
@brief This library contains WordParameter object functions.
Modifications History:
	$Log: gcsWordParameter.ctl,v $
	Revision 1.29  2009/03/31 15:06:14  gthomas
	in _WidgetDisconnection(): removed AlertText animation
	in _FaceplateStatusAnimationCB(): removed animation check of PLC disconnection for Ranges and the Default value (not required as no PLC connection)
	in _ProvideAccessToButtons(): removed exceptionInfo as returns errors in log when monitor or no user are logged in.
	
	Revision 1.28  2008/11/12 08:33:24  gthomas
	-removed function calls to _ButtonUserAccessGcs and _ButtonUserAccessUn

	Revision 1.27  2008/10/15 14:47:04  gthomas
	-Removed the <if statement> of the dpConnect in gcsAnalogParameter_ButtonRegisterCB function
	to fix problem with select/deselect button animation.

	Revision 1.26  2008/09/12 13:55:40  gthomas
	Fixed bad function call to "xx_ProvideAccessToButtons" fct in <gcsWordParameter_ButtonRegisterCB>

	Revision 1.25  2008/07/30 14:21:02  gamx
	Changes in "WidgetDisconnection": objects "ControlStateText" and "eventState" have been removed as they are not presented on the widget panel

	Revision 1.24  2008/07/23 09:26:09  gthomas
	-added missing ExceptionInfo in _ProvideAccessToButtons() function
	-removed comments

	Revision 1.23  2008/06/19 12:09:26  gthomas
	integrated new Access Control of UNICOS 4-1 (by instance)
	removed functions _ButtonUserAccessUn and _ButtonUserAccessGcs

	Revision 1.22  2008/06/18 14:54:51  aabadz
	Changes done following the recomendations from AB-CO (from Herve)

	Revision 1.21  2008/06/13 12:29:47  aabadz
	Modifications asked by UNICOS for their version 4.X

	Revision 1.20  2008/05/26 12:18:19  gthomas
	Merged files with modifications made by ABCO on xS and xP (GCS-ABCO-v4-1)
	to be used with PVSS 3.6 QT

	21/04/08	Enrique (AB/CO)
		- Delete functions:
		WordParameter_EventList_Configure_Init()
		WordParameter_EventList_Configure_Clic()
		WordParameter_EventList_Configure_Save()
	Revision 1.19  2008/03/17 10:52:36  aabadz
	Fixed a problem found in animation when we have not database.

	Revision 1.18  2008/01/23 15:27:54  aabadz
	Changed name of funciont FaceplateStatusDisconnection to AnimateFaceplateDisconnection

	Revision 1.17  2008/01/22 12:52:45  aabadz
	Fixed:
	- Naming
	- Call to other functions to reuse code
	- Some error check

	Revision 1.16  2008/01/10 15:24:07  aabadz
	Changes after code review and before tests

	Revision 1.15  2007/10/30 16:22:40  gthomas
	Introduced the CVS keyword $Log to track changes made in the library

	21/08/2007: Enrique AB/CO
		- Function gcsWordParameter_ObjectListGetValueTime defined
	14/09/2006: Kulman Nikolay
		- Ported to work with UNICOS-JCOP-3-6 version

*/
/*
Internal Functions :
	gcsWordParameter_MenuConfiguration (string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &dsMenuList)
	gcsWordParameter_HandleMenu (string sDeviceName, string sDpType, dyn_string dsMenuList, int menuAnswer)
	gcsWordParameter_WidgetRegisterCB (string sSystemStatus, bool bSystemConnected)
	gcsWordParameter_WidgetConnect (string sDeviceName, string sPlcName, string sSystemName)
	gcsWordParameter_WidgetDisconnect (string sDeviceName, string sPlcName, string sSystemName)
	gcsWordParameter_WidgetAnimationCB (string sDpLock, bool bLocked, string sDpManager, string sSelectedManager, ... )
	gcsWordParameter_WidgetDisconnection (string sWidgetType)
	gcsWordParameter_FaceplateStatusRegisterCB (string sSystemStatus, bool bSystemConnected)
	gcsWordParameter_FaceplateConnect (string sDeviceName)
	gcsWordParameter_FaceplateDisconnect (string sDeviceName)
	gcsWordParameter_FaceplateStatusAnimationCB (string sDpValRqAck, int iValRqAck, string sDp4, bool bValRqAckInvalid, ... )
	gcsWordParameter_AnimatePlcDisconnection ()
	gcsWordParameter_ButtonRegisterCB (string sSystemStatus, bool bSystemConnected)
	gcsWordParameter_ButtonAnimationCB (string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager, string sDpValRqAck, int iValRqAckValue, string sDpValRqAckBad, bool bValRqAckBad)
	gcsWordParameter_ButtonUserAccess (string sDpName, string sDpType, dyn_string &dsAccess)
	gcsWordParameter_UserLoginGetButtonState (string sDeviceName, string sType, dyn_string &dsReturnData)
	gcsWordParameter_ButtonSetState (string sDeviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
	gcsWordParameter_ButtonDisconnection ()
	WordParameter_EventList_Configure_Init (dyn_string &exceptionInfo, const bool bUpdate=false)
	WordParameter_EventList_Configure_Clic (int iRow, string sColumn, string sValue, dyn_string &exceptionInfo)
	WordParameter_EventList_Configure_Save (dyn_string &exceptionInfo)
*/


//------------------------------------------------------------------------------------------------------------
// gcsWordParameter_MenuConfiguration
/**
Configuration of popup menu

	@param[in]  sDpName Device name
	@param[in]	sDpType Unicos object
	@param[in]	dsAccessOk	List of authorized action
	@param[out] dsMenuList Configuration of menu. Use defined constants UN_POPUPMENU_***
*/
gcsWordParameter_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &dsMenuList)
{
	dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT,
			 UN_POPUPMENU_FACEPLATE_TEXT, UN_POPUPMENU_TREND_TEXT);

	unGenericObject_addSelectToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, dsMenuList);
	unGenericObject_addUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, dsMenuList);
	unGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, dsMenuList);
	unGenericObject_addTrendActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, dsMenuList);
}


//------------------------------------------------------------------------------------------------------------------
// gcsWordParameter_HandleMenu
/**
Handle the answer of the popup menu

	@param[in]	sDeviceName 	The device dp name
	@param[in]	sDpType 	The device dp type
	@param[in]	dsMenuList 	List of requested action
	@param[in]	menuAnswer 	Selected action from the dsMenuList
*/
gcsWordParameter_HandleMenu(string sDeviceName, string sDpType, dyn_string dsMenuList, int menuAnswer)
{
	dyn_string exceptionInfo;

	unGenericObject_HandleUnicosMenu(sDeviceName, sDpType, dsMenuList, menuAnswer, exceptionInfo);
	if (dynlen(exceptionInfo) > 0)
	{
		unSendMessage_toExpertException("Right Click",exceptionInfo);
	}
}


//------------------------------------------------------------------------------------
//gcsWordParameter_WidgetRegisterCB
/**
Register callback function

	@param[in]	sSystemStatus 	The device dp
	@param[in] 	bSystemConnected 	Flag
*/
gcsWordParameter_WidgetRegisterCB(string sSystemStatus, bool bSystemConnected)
{
	string sDeviceName, sSystemName, sPlcDpName, sPlcName;
	int iAction;
	dyn_string exceptionInfo;
	bool bRemote;

	sDeviceName = unGenericDpFunctions_getWidgetDpName ($sIdentifier);
	if (sDeviceName == "") // in case of disconnection
	{
		sDeviceName = g_sDpName;
	}

	sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);

	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;

	sPlcName = unGenericObject_GetPLCNameFromDpName (sDeviceName);
	gcsConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)
		g_bUnSystemAlarmPlc = true;
	else
		g_bUnSystemAlarmPlc = false;

	unGenericObject_Connection(sDeviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	

	switch(iAction)
	{
		case UN_ACTION_DISCONNECT:
			gcsWordParameter_WidgetDisconnection(g_sWidgetType);
			break;

		case UN_ACTION_DPCONNECT:
			g_sDpName = sDeviceName;

			if(dpTypeName(sDeviceName) == GCS_CONFIG_WORDPARAMETER_DPT_NAME)
				gcsWordParameter_WidgetConnect(sDeviceName, sPlcName, sSystemName);
			else
			{
				g_bUnSystemAlarmPlc = false;
				gcsWordParameter_WidgetDisconnection(g_sWidgetType);
			}
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			gcsWordParameter_WidgetDisconnect (sDeviceName, sPlcName, sSystemName);
			gcsWordParameter_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_NOTHING: break;
			default:
				break;
	}
}


//------------------------------------------------------------------------------------------------------------
//gcsWordParameter_WidgetConnect
/**
dpConnect to the DataPoint

	@param[in]	sDeviceName	Device DP name
	@param[in]	sPlcName	PLC name
	@param[in]	sSystemName	System name
*/
gcsWordParameter_WidgetConnect(string sDeviceName, string sPlcName, string sSystemName)
{
	int iRes;

	if(g_bUnSystemAlarmPlc)
	{
		g_sValRqAckFormat = dpGetFormat(sDeviceName + ".ProcessInput.ValRqAck");
		g_sValRqAckUnit = dpGetUnit(sDeviceName + ".ProcessInput.ValRqAck");
		g_sCurValStFormat = dpGetFormat(sDeviceName + ".ProcessInput.CurValSt");
		g_sCurValStUnit = dpGetUnit(sDeviceName + ".ProcessInput.CurValSt");

		iRes = dpConnect ("gcsWordParameter_WidgetAnimationCB",
				sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
				sDeviceName + ".statusInformation.selectedManager",
				sDeviceName + ".ProcessInput.ValRqAck",
				sDeviceName + ".ProcessInput.ValRqAck:_online.._invalid",
				sDeviceName + ".ProcessInput.CurValSt",
				sDeviceName + ".ProcessInput.CurValSt:_online.._invalid",
				sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
				sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled");
		g_bCallbackConnected = (iRes >= 0);
	}
	else
		gcsWordParameter_WidgetDisconnection(g_sWidgetType);
}

//------------------------------------------------------------------------------------------------------------
//gcsWordParameter_WidgetDisconnect
/**
dpDisconnect to the DataPoint

	@param[in]	sDeviceName	Device DP name
	@param[in]	sPlcName	PLC name
	@param[in]	sSystemName	System name
*/
gcsWordParameter_WidgetDisconnect(string sDeviceName, string sPlcName, string sSystemName)
{
	int iRes;

	if(g_bUnSystemAlarmPlc)
	{
		iRes = dpDisconnect("gcsWordParameter_WidgetAnimationCB",
				sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
				sDeviceName + ".statusInformation.selectedManager",
				sDeviceName + ".ProcessInput.ValRqAck",
				sDeviceName + ".ProcessInput.ValRqAck:_online.._invalid",
				sDeviceName + ".ProcessInput.CurValSt",
				sDeviceName + ".ProcessInput.CurValSt:_online.._invalid",
				sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
				sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled");
		g_bCallbackConnected = !(iRes >= 0);
	}
}


//------------------------------------------------------------------------------------------------------------
//gcsWordParameter_WidgetAnimationCB
/**
Animate widget
*/
gcsWordParameter_WidgetAnimationCB(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
			 string sDpValRqAck, string sValRqAck, string sDp4, bool bValRqAckInvalid,
			 string sDpCurValSt, string sCurValSt, string sDp6, bool bCurValStInvalid,
			 string sDpAl, int iSystemIntegrityAlarmValue, string sDpEnabled, bool bSystemIntegrityAlarmEnabled)
{
	string sFormattedValue, sFormattedValueBody, sColorSelect;
	string sBody1Text, sBody1Color, sDisplay1Text, sDisplay1Color, sWarningLetter, sWarningColor;
	bool bLockVisible;

	if((!bSystemIntegrityAlarmEnabled)||(iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value))
	{
		sWarningLetter = UN_WIDGET_TEXT_OLD_DATA;
		sWarningColor = "unDataNotValid";
	}

	sFormattedValueBody = unGenericObject_FormatValue(g_sValRqAckFormat, sValRqAck);
	sBody1Text = sFormattedValueBody + " " + g_sValRqAckUnit;
	//control animation
	if (bValRqAckInvalid)
	{
		sBody1Color = "unDataNotValid";
		sWarningLetter = UN_WIDGET_TEXT_INVALID;
		sWarningColor = "unDataNotValid";
	}
	else
	{
		sBody1Color = "unDisplayValue_Output";
	}

	sFormattedValue = unGenericObject_FormatValue(g_sCurValStFormat, sCurValSt);
	sDisplay1Text = sFormattedValue + " " + g_sCurValStUnit;
	//control animation
	if (bCurValStInvalid)
	{
		sDisplay1Color = "unDataNotValid";
		sWarningLetter = UN_WIDGET_TEXT_INVALID;
		sWarningColor = "unDataNotValid";
	}
	else
	{
		sDisplay1Color = "unDisplayValue_Status";
	}

	unGenericObject_WidgetSelectAnimation(bLocked, sSelectedManager, sColorSelect, bLockVisible);
	if(g_bSystemConnected)
	{
		setMultiValue ("Body1", "text", sBody1Text, /*"Body1", "fill", "[solid]",*/
				"Body1", "foreCol", sBody1Color, /*"Body1", "backCol", "unWidget_Background",*/
				"Display1", "text", sDisplay1Text, /*"Display1", "fill", "[solid]",*/
				"Display1", "foreCol", sDisplay1Color, /*"Display1", "backCol", "unWidget_Background",*/
				"WarningText", "text", sWarningLetter, "WarningText", "foreCol", sWarningColor,
				"SelectArea", "foreCol", sColorSelect, "LockBmp", "visible", bLockVisible,
				"WidgetArea", "visible", true);
	}
}


//------------------------------------------------------------------------------------------------------------
//gcsWordParameter_WidgetDisconnection
/**
Animate widget disconnection

	@param[in] sWidgetType Widget type. ex: "heater"
*/
gcsWordParameter_WidgetDisconnection(string sWidgetType)
{
		setMultiValue("Body1", "foreCol", "unDataNoAccess", 
                 "Display1","foreCol","unDataNoAccess", 
                 "WarningText", "text", "", 
                 "SelectArea", "foreCol", "", 
                 "LockBmp", "visible", false, 
                 "WidgetArea", "visible", false);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordParameter_FaceplateStatusRegisterCB
/**
Register faceplate callback function

	@param[in]	sSystemStatus 	The device dp
	@param[in] 	bSystemConnected 	Flag
*/
gcsWordParameter_FaceplateStatusRegisterCB(string sSystemStatus, bool bSystemConnected)
{
	string sPlcName, sPlcDpName, sDeviceName,sSystemName;
	int iAction;
	dyn_string exceptionInfo;
	bool bRemote;

	sDeviceName = unGenericDpFunctions_getDpName($sDpName);
	if (sDeviceName)
	{
		sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
		unDistributedControl_isRemote(bRemote, sSystemName);
		if(bRemote)
			g_bSystemConnected = bSystemConnected;
		else
			g_bSystemConnected = true;
         		sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);
			if(sPlcName != "")
			{
				gcsConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
				if (dynlen(exceptionInfo) <= 0)
					g_bUnSystemAlarmPlc = true;
				else g_bUnSystemAlarmPlc = false;
			}
			else
				g_bUnSystemAlarmPlc = false;
				unGenericObject_Connection(sDeviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
		
				switch (iAction)
				{
				case UN_ACTION_DISCONNECT:
					gcsWordParameter_AnimatePlcDisconnection();
					break;
				case UN_ACTION_DPCONNECT:
					gcsWordParameter_FaceplateConnect (sDeviceName);
					break;
				case UN_ACTION_DPDISCONNECT_DISCONNECT:
					gcsWordParameter_FaceplateDisconnect (sDeviceName);
					gcsWordParameter_AnimatePlcDisconnection();
					break;
				case UN_ACTION_NOTHING:
					break;
		
				default:
					break;
				}
	}
	
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordParameter_FaceplateConnect
/**
Do faceplate dpConnect

	@param[in]	sDeviceName	Device DP name
*/
gcsWordParameter_FaceplateConnect(string sDeviceName)
{
	int iRes=0;
	dyn_string exceptionInfo;
	string sPlcName, sPlcDpName, sSystemName;

	sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
// get the PLC name
	sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);
	gcsConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0) g_bUnSystemAlarmPlc = true;
	else g_bUnSystemAlarmPlc = false;

	if (g_bUnSystemAlarmPlc)
	{
		g_sValRqAckFormat = dpGetFormat(sDeviceName + ".ProcessInput.ValRqAck");
		g_sValRqAckUnit = dpGetUnit(sDeviceName + ".ProcessInput.ValRqAck");
		g_sCurValStFormat = dpGetFormat(sDeviceName + ".ProcessInput.CurValSt");
		g_sCurValStUnit = dpGetUnit(sDeviceName + ".ProcessInput.CurValSt");

		iRes = dpConnect("gcsWordParameter_FaceplateStatusAnimationCB",
				 sDeviceName + ".ProcessInput.ValRqAck",
				 sDeviceName + ".ProcessInput.ValRqAck:_online.._invalid",
				 sDeviceName + ".ProcessInput.CurValSt",
				 sDeviceName + ".ProcessInput.CurValSt:_online.._invalid",
				 sDeviceName + ".ProcessInput.CurValSt:_pv_range.._min",
				 sDeviceName + ".ProcessInput.CurValSt:_pv_range.._max",
				 sDeviceName + ".ProcessOutput.DfltVal",
				 sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
				 sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled");
		g_bCallbackConnected = (iRes >= 0);
	}
	else
		gcsWordParameter_AnimatePlcDisconnection();
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordParameter_FaceplateDisconnect
/**
Do faceplate dpDisconnect

	@param[in]	sDeviceName	Device DP name
*/
gcsWordParameter_FaceplateDisconnect (string sDeviceName)
{
	int iRes=0;
	string sPlcName, sSystemName;

	if (g_bUnSystemAlarmPlc)
	{
		sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
		sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);

		iRes = dpDisconnect("gcsWordParameter_FaceplateStatusAnimationCB",
			 sDeviceName + ".ProcessInput.ValRqAck",
			 sDeviceName + ".ProcessInput.ValRqAck:_online.._invalid",
			 sDeviceName + ".ProcessInput.CurValSt",
			 sDeviceName + ".ProcessInput.CurValSt:_online.._invalid",
			 sDeviceName + ".ProcessInput.CurValSt:_pv_range.._min",
			 sDeviceName + ".ProcessInput.CurValSt:_pv_range.._max",
			 sDeviceName + ".ProcessOutput.DfltVal",
			 sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".alarm",
			 sSystemName + c_unSystemAlarm_dpPattern + PLC_DS_pattern + sPlcName + ".enabled");
		g_bCallbackConnected = !(iRes >= 0);
	}
	else
	{
		gcsWordParameter_WidgetDisconnection(g_sWidgetType);
		g_bCallbackConnected = false;
	}
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordParameter_FaceplateStatusAnimationCB
/**
Animate faceplate
*/
gcsWordParameter_FaceplateStatusAnimationCB(string sDpValRqAck, int iValRqAck, string sDp4, bool bValRqAckInvalid,
			 string sDpCurValSt, int iCurValSt, string sDp6, bool bCurValStInvalid,
			 string sDp7, int iCurValStMin, string sDp8, int iCurValStMax,string sDp10, int iDefaultVal,
			 string sDpAl, int iValAlarm, string sDpEnabled, bool bSystemIntegrityAlarmEnabled)
{
	if(g_bSystemConnected)
		unGenericObject_DisplayValue(g_sValRqAckFormat,	g_sValRqAckUnit, iValRqAck, "RqValue", "unDisplayValue_Output",
			bValRqAckInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bSystemConnected)
		unGenericObject_DisplayValue(g_sCurValStFormat,	g_sCurValStUnit, iCurValSt, "CurVal_Status", "unDisplayValue_Status",
			bCurValStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));

	if(g_bSystemConnected)
		unGenericObject_DisplayValue(g_sCurValStFormat,	g_sCurValStUnit, iDefaultVal, "DefaultValue", "unDisplayValue_Setpoint",0);
	if(g_bSystemConnected)
	    unGenericObject_DisplayValue(g_sCurValStFormat,	g_sCurValStUnit, iCurValStMin, "RangeCurValStMin", "unDisplayValue_Parameter",0);
	if(g_bSystemConnected)
	    unGenericObject_DisplayValue(g_sCurValStFormat,	g_sCurValStUnit, iCurValStMax, "RangeCurValStMax", "unDisplayValue_Parameter",0);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordParameter_AnimatePlcDisconnection
/**
Animate faceplate disconnection
*/
gcsWordParameter_AnimatePlcDisconnection()
{
	unGenericObject_DisplayValueDisconnect(makeDynString("RqValue", "CurVal_Status", "DefaultValue", "RangeCurValStMin", "RangeCurValStMax"));
}

//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordParameter_ProvideAccessToButtons
/**
Defines the list of buttons available for a user. The result is passed in global variable.
	@param[in]	sDeviceName	Name of the device
	@param[in]	sDpType		Device Type
	@param[in] 	dsFunctions	A list of functions for a device, specified in device internal configuration.
	@param[out]	exceptionInfo	A list of exceptions.
*/
gcsWordParameter_ProvideAccessToButtons(string sDeviceName, string sDpType, dyn_string dsFunctions, dyn_string &exceptionInfo)
{
	string sFunction;
	dyn_string dsUserAccess;

	if (dynlen(dsFunctions) >= UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS)
		sFunction = dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS];
	else
	{
		dynAppend(exceptionInfo, "gcsWordParameter_ProvideAccessToButtons: the list of functions is not correct - number of functions is less than expected");
		return;
	}

	if (sFunction != "")
	{
		evalScript(dsUserAccess, "dyn_string main(string deviceNameTemp, string sDpTypeTemp) {" +
					 "dyn_string dsUserAccessReturn;" +
					 "if (isFunctionDefined(\"" + sFunction  + "\"))" +
					 "    {" +
			 			sFunction  + "(deviceNameTemp, sDpTypeTemp, dsUserAccessReturn);" +
					 "    }" +
					 "return dsUserAccessReturn; }", makeDynString(), sDeviceName, sDpType);

			g_dsUserAccess=dsUserAccess;
	}
	else
		fwException_raise(exceptionInfo,"ERROR", "gcsWordParameter_ProvideAccessToButtons:" + getCatStr("unGeneration","UNKNOWNFUNCTION"), "EMPTY");

}

//---------------------------------------------------------------------------------------------------------------------
// gcsWordParameter_ButtonRegisterCB
/**
Register callback function

	@param[in]	sSystemStatus 	The device dp
	@param[in] 	bSystemConnected 	Flag
*/
gcsWordParameter_ButtonRegisterCB(string sSystemStatus, bool bSystemConnected)
{
	string sDeviceName, sDpType, sFunction;
	int iAction, iRes;
	dyn_string exceptionInfo, dsFunctions, dsUserAccess;

	sDeviceName = unGenericDpFunctions_getDpName($sDpName);
	unGenericObject_Connection(sDeviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	
	switch (iAction)
	{
		case UN_ACTION_DISCONNECT:
			gcsWordParameter_ButtonDisconnection();
			break;
		case UN_ACTION_DPCONNECT:

// call the gcsWordParameter_ButtonUserAccess to get the list of allowed buttons.
			sDpType="WordParameter";
			unGenericObject_GetFunctions(sDpType, dsFunctions, exceptionInfo);

			unGenericObject_GetFunctions(sDpType, dsFunctions, exceptionInfo);
			if (dynlen(exceptionInfo) <= 0)	gcsWordParameter_ProvideAccessToButtons(sDeviceName, sDpType, dsFunctions, exceptionInfo);
			iRes = dpConnect("gcsWordParameter_ButtonAnimationCB",
					sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
					sDeviceName + ".statusInformation.selectedManager",
					sDeviceName + ".ProcessInput.ValRqAck",
					sDeviceName + ".ProcessInput.ValRqAck:_online.._aut_inv");
			g_bCallbackConnected = (iRes >= 0);
			
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			gcsWordParameter_ButtonsDisconnect(sDeviceName);
			gcsWordParameter_ButtonDisconnection();
			break;
		case UN_ACTION_NOTHING: break;
		default:
			break;
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------
//gcsWordParameter_ButtonsDisconnect
/**
Disconnects callback function to animate buttons states
	@param[in] 	sDeviceName	The name of the device
*/
gcsWordParameter_ButtonsDisconnect(string sDeviceName)
{
	int iRes = -1;

	iRes = dpDisconnect("gcsWordParameter_ButtonAnimationCB",
			sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
			sDeviceName + ".statusInformation.selectedManager",
			sDeviceName + ".ProcessInput.ValRqAck",
			sDeviceName + ".ProcessInput.ValRqAck:_online.._aut_inv");
	g_bCallbackConnected = !(iRes >= 0);
}

//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordParameter_ButtonAnimationCB
/**
Animate buttons

@note This function works with an unicos button panel and needs global variables declared in this panel -
		"g_dsUserAccess" (dyn_string). Button identifier is added to this variable if user has access.
*/
gcsWordParameter_ButtonAnimationCB(string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager,
		string sDpValRqAck, int iValRqAckValue, string sDpValRqAckBad, bool bValRqAckBad)
{
	gcsWordParameter_ButtonSetState(unGenericDpFunctions_getDpName(sDpValRqAck), "AnalogParameter", g_dsUserAccess,
			makeDynString(bLocked, sSelectedManager, iValRqAckValue, bValRqAckBad, 0, 0));
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordParameter_ButtonUserAccess
/**
User access for buttons

	@param[in] 		sDpName	Device name
	@param[in] 		sDpType	dpType
	@param[out] 	dsAccess	List of allowed buttons
*/
gcsWordParameter_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
	bool operator, expert, admin;
		dyn_bool dbPermissions;
		dyn_string exceptionInfo;
		string sOperAction, sExpAction, sAdminAction;
		bool bActionDefined = false;
		
		unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo);
		if((sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME) ||
		(sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME))
		bActionDefined=true;
		operator = dbPermissions[2];
		expert = dbPermissions[3];
		admin = dbPermissions[4];
		dsAccess = makeDynString();
		if (operator){
			if(bActionDefined)
				dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
			else
			    dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_SELECT));
		}
		if (expert){
			if(bActionDefined)
		            dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
		        else
			    dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_PARAMETER_SET_VALUE));
		}
	        if(admin){
		       	if(bActionDefined)
		           dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
       	}
}

//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordParameter_UserLoginGetButtonState
/**
Function is called before the button set state when a user logs in.

	@param[in]	sDeviceName		The device name
	@param[in]	sType			Device type <b>(Currently not used)</b>
	@param[out]	dsReturnData	The data are returned here
	@note	This library works with an unicos button panel and need global variables declared in this panel
		- g_dsUserAccess, dyn_string, button identifier is added to this variable if user access
		- $sDpName, string, the device DP name
		- g_sFaceplateButtonType, string, faceplate type
*/
gcsWordParameter_UserLoginGetButtonState(string sDeviceName, string sType, dyn_string &dsReturnData)
{
	int iRes = -1;
	string sDpLock, sDpSelectedManager, sSelectedManager;
	bool bLocked;

	sDpLock = sDeviceName + ".statusInformation.selectedManager:_lock._original._locked";
	sDpSelectedManager = sDeviceName + ".statusInformation.selectedManager";
	if (dpExists(sDeviceName))
		iRes = dpGet(sDpLock, bLocked, sDpSelectedManager, sSelectedManager);

	if (iRes < 0) // Errors during dpget
	{
		sDpLock = "";
		bLocked = false;
		sDpSelectedManager = "";
		sSelectedManager = "";
	}
	dsReturnData=makeDynString(bLocked, sSelectedManager, 0, 0, 0, 0);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordParameter_ButtonSetState
/**
Set button state

	@param[in]	 sDeviceName		The device name
	@param[in]	 sDpType		Device type
	@param[in]	 dsUserAccess	User access
	@param[in]	 dsData			List of various parameters
*/
gcsWordParameter_ButtonSetState(string sDeviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
	bool bSelected, buttonEnabled;
	dyn_string dsButtons;
	int i, iDsButtonsLength;
	string localManager;
	bool bLocked, bStsReg01Bad;
	string sSelectedManager;
	bit32 stsReg01Value;

	bLocked = dsData[1];
	sSelectedManager = dsData[2];
	stsReg01Value = dsData[3];
	bStsReg01Bad = dsData[4];

	dsButtons = makeDynString(UN_FACEPLATE_BUTTON_PARAMETER_SET_VALUE);// Each button except "Select" button
	iDsButtonsLength = dynlen(dsButtons);
// 1. Selection state
	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
	bSelected = (localManager == "S");
// 2. Button state
	for(i = 1; i <= iDsButtonsLength; i++)
	{
		buttonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0);	// User access
		setValue(UN_FACEPLATE_BUTTON_PREFIX + dsButtons[i], "enabled", buttonEnabled && bSelected);
	}
// 3. Button Select state
	unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordParameter_ButtonDisconnection
/**
Animate disconnection
*/
gcsWordParameter_ButtonDisconnection()
{
	unGenericObject_ButtonDisconnect(makeDynString(UN_FACEPLATE_BUTTON_SELECT, UN_FACEPLATE_BUTTON_PARAMETER_SET_VALUE));
}



// gcsWordParameter_ObjectListGetValueTime
/**
Purpose: Function called from the objectList to return the time and value

Parameters:
	- sDeviceName, string, input, device name
	- sDeviceType, string, input, unicos object
	- dsReturnData, dyn_string, output, return value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 3.1
	. operating system: XP.
	. distributed system: yes.
*/

gcsWordParameter_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
	float value, valueRequested;
	string sTime, sStatus,sColor;
	bool bBad;

	dpGet(sDeviceName+".ProcessInput.CurValSt", value,
	      sDeviceName+".ProcessInput.CurValSt:_online.._stime", sTime,
	      sDeviceName+".ProcessInput.CurValSt:_online.._bad", bBad,
	      sDeviceName+".ProcessInput.ValRqAck", valueRequested);

	dsReturnData[1] = sTime;
	dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(sDeviceName+".ProcessInput.CurValSt"), value) + " "
							+ dpGetUnit(sDeviceName+".ProcessInput.CurValSt");

	// Optionally (when no stsReg* exists)
	// Status
	dsReturnData[3] = bBad;

	if (value==valueRequested)
		{
			sStatus="OK";
			sColor="unEventList_ForeColor";
		}
	else
		{
			sStatus="WPar value not activated";
			sColor="unEventList_Warning";
		}
	// Status to visualize
	dsReturnData[4] = sStatus;
	// Colors
	dsReturnData[5]=sColor;
}
//----------------------------------------------
