/**@name LIBRARY: gcsWordStatus.ctl

@author: Nikolay Kulman (IT-CO-FE)
@author: Francisco Mico (IT-CO-FE)
@date 31.07.2003
@brief This library contains WordStatus object functions.
Modifications History:
		
  Id: $Id: gcsWordStatus.ctl 47601 2010-11-05 16:51:25Z amerezhi $
  no changes made
  
  new test
  
  $Log: gcsWordStatus.ctl,v $
  22/10/10 amerezhi: merged with ABCO version:
	28/01/2010: Herce
	- in gcsWordStatus_WidgetAnimationCB and gcsWordStatus_WidgetDisconnection: 
	check if the WidgetArea object is in the widget before setting it.
  
  Revision 1.20  2009/03/27 10:33:23  gthomas
  -in _WidgetRegisterCB() under UN_ACTION_DPCONNECT: added evaluation on DP type
  -in _WidgetAnimationCB()under
  if(g_sWidgetType != "WordStatusBit"): changed conditions of animation when data does not correspond to any labels
  if(g_sWidgetType == "WordStatusBit"): added evaluation when no labels are defined
  -in _WidgetDisconnection(): added animation of color boxes for WordStatusBitBig widget

  Revision 1.19  2009/03/11 14:33:49  gthomas
  added missing variable declaration in _WidgetAnimationCB
  and added modifs from ABCO_v4-3 in _FaceplateAnimationCB

  Revision 1.18  2008/07/23 09:41:29  gthomas
  removed exceptionInfo after unGenericObject_Connection function.

  Revision 1.17  2008/06/23 11:58:22  gthomas
  fixed bad variable call ("WordStatus" replaced with  "WS") in WidgetAnimationCB fct

  Revision 1.16  2008/06/13 12:29:47  aabadz
  Modifications asked by UNICOS for their version 4.X

  Revision 1.15  2008/05/26 12:18:20  gthomas
  Merged files with modifications made by ABCO on xS and xP (GCS-ABCO-v4-1)
  to be used with PVSS 3.6 QT

  22/04/2008 Quique AB/CO
  	If no message is associated with the input value (specs definition) then the value itself is visualized

  21/04/2008 Quique AB/CO
   	-Delete the functions:
		WordStatus_EventList_Configure_Init()
		WordStatus_EventList_Configure_Clic()
		WordStatus_EventList_Configure_Save()
  Revision 1.14  2008/03/17 10:52:36  aabadz
  Fixed a problem found in animation when we have not database.

  Revision 1.13  2007/11/27 10:47:11  ikoblik
  Tested + bug fixes according to the November, 21st code review.

  Revision 1.12  2007/11/27 09:25:19  ikoblik
  Bug fixes according to the November, 21st code review.

  Revision 1.11  2007/10/30 16:22:45  gthomas
  Introduced the CVS keyword $Log to track changes made in the library

  11/05/2004: Herve AB/CO
  	- function gcsWordStatus_WidgetRegisterCB
  	bug in case the PLC is not defined and if the PVSS system name contains '-',
  	call unGenericObject_GetPLCNameFromDpName to get the PLC DPName instead (this function corrects the bugs).


*/
/*
External Functions :
	. gcsWordStatus_WidgetDisconnection : animate widget disconnection
	
Internal Functions :
	. gcsWordStatus_WidgetRegisterCB : register callback function
	. gcsWordStatus_WidgetConnect: dpConnect to the datapoints
	. gcsWordStatus_WidgetDisconnect: dpDisconnect to the datapoints
	. gcsWordStatus_WidgetAnimationCB : animate widget
	. gcsWordStatus_WidgetDisconnection: Disconnect the widget

Purpose: This library contains WordStatus object functions.

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. WordStatus object (DPType etc.)
	. Global variables and $parameters defined in WordStatus faceplate and widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/


//------------------------------------------------------------------------------------------------------------
// gcsWordStatus_MenuConfiguration
/**
Configuration of popup menu

  @param[in]  sDpName Device name
  @param[in]  sDpType Unicos object
  @param[in]  dsAccessOk  List of authorized action
  @param[out] menuList Configuration of menu. Use defined constants UN_POPUPMENU_***
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
gcsWordStatus_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{
  dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_DIAG_INFO_TEXT, UN_POPUPMENU_TREND_TEXT);

  unGenericObject_addDiagnosticAndInfoToActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
  unGenericObject_addTrendActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
}


//------------------------------------------------------------------------------------------------------------------
// gcsWordStatus_HandleMenu
/**
Handle the answer of the popup menu

  @param[in]  deviceName  The device dp name
  @param[in]  sDpType   The device dp type
  @param[in]  menuList  List of requested action
  @param[in]  menuAnswer  Selected action from the menuList
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
gcsWordStatus_HandleMenu(string sDeviceName, string sDpType, dyn_string dsMenuList, int iMenuAnswer)
{
  dyn_string exceptionInfo;

  unGenericObject_HandleUnicosMenu(sDeviceName, sDpType, dsMenuList, iMenuAnswer, exceptionInfo);
  if (dynlen(exceptionInfo) > 0)
  {
    unSendMessage_toExpertException("Right Click",exceptionInfo);
  }
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordStatus_WidgetRegisterCB
/**
Register callback function

  @param[in]  sConnectionStatus   Distributed system connection status string
  @param[in]  bSystemConnected  Flag
Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in WordStatus widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
gcsWordStatus_WidgetRegisterCB(string sConnectionStatus, bool bSystemConnected)
{
  string sDeviceName = "", sPlcName = "", sSystemName = "";
  int iAction = UN_ACTION_NOTHING;
  dyn_string exceptionInfo;
  bool bRemote = false;

  if ("" == (sDeviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier))) // In case of disconnection
  {
    sDeviceName = g_sDpName;
  }


// get the systemName
  sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
// get the PLC name
  sPlcName = unGenericObject_GetPLCNameFromDpName(sDeviceName);
  if ("" != sSystemName)
    unDistributedControl_isRemote(bRemote, sSystemName);
  if(bRemote)
    g_bSystemConnected = bSystemConnected;
  else
    g_bSystemConnected = true;

  if(sPlcName != "")
  {
    string sPlcDpName = "";
    dyn_string dsPlcDpNameExceptions;
    gcsConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, dsPlcDpNameExceptions);
    if (0 == dynlen(dsPlcDpNameExceptions))
    {
      g_bUnSystemAlarmPlc = true;
    }
    else
    {
      g_bUnSystemAlarmPlc = false;
    }
  }
  else  g_bUnSystemAlarmPlc = false;

  unGenericObject_Connection(sDeviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
  
  switch (iAction)
  {
      case UN_ACTION_DISCONNECT:
       //DebugN("gcsWordStatus_WidgetRegisterCB: UN_ACTION_DISCONNECT");
        gcsWordStatus_WidgetDisconnection(g_sWidgetType);
        break;
      case UN_ACTION_DPCONNECT:
        //DebugN("gcsWordStatus_WidgetRegisterCB: UN_ACTION_DPCONNECT");
        g_sDpName = sDeviceName;
        if(dpTypeName(sDeviceName) != GCS_CONFIG_WORDSTATUS_DPT_NAME) {
			g_bUnSystemAlarmPlc = false;
			gcsWordStatus_WidgetDisconnection(g_sWidgetType);
		} else {
			switch(g_sWidgetType)
			  { 
				case "WS":
				case "WordStatusBit":
				case "WordStatusBitBig":
					gcsWordStatus_WidgetConnect(sDeviceName, sPlcName, sSystemName);
					break;
				case "step_PLC": 
					gcsWordStatus_step_PLC_WidgetConnect(g_sDpName);
					break;
				case "transition_PLC": 
					gcsWordStatus_transition_PLC_WidgetConnect(g_sDpName);
					break;
				default:
					g_bUnSystemAlarmPlc = false;
					gcsWordStatus_WidgetDisconnection(g_sWidgetType);
					break;
			  }
		}
        break;
      case UN_ACTION_DPDISCONNECT_DISCONNECT:
			switch(g_sWidgetType)
			  {
				case "WS":
				case "WordStatusBit":
				case "WordStatusBitBig":
					gcsWordStatus_WidgetDisconnect(sDeviceName, sPlcName, sSystemName);
					gcsWordStatus_WidgetDisconnection(g_sWidgetType);
					break;
				case "step_PLC":
					gcsWordStatus_step_PLC_WidgetDisconnect(g_sDpName, iRes);
					break;
				case "transition_PLC":
					gcsWordStatus_transition_PLC_WidgetDisconnect(g_sDpName, iRes);
					break;
				default:
					break;
			  }
        break;
      default:
        break;
  }

}

//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordStatus_WidgetConnect
/**
Purpose: dpConnect to the DataPoint

  @param[in]  deviceName  Device DP name
  @param[in]  sPlcName  PLC name
  @param[in]  sSystemName System name
  
Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in WordStatus faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
gcsWordStatus_WidgetConnect(string sDeviceName, string sPlcName, string sSystemName)
{
  int iRes;
  if (false == dpExists(sDeviceName))
    return;
  if(g_bUnSystemAlarmPlc)
  {
    string sDp = sDeviceName + ".ProcessInput.AuPosRSt";
    g_sAuPosRStFormat = dpGetFormat(sDp);
    g_sAuPosRStUnit = dpGetUnit(sDp);
	if ((g_sWidgetType!="step_PLC") && (g_sWidgetType!="transition_PLC")) {
		g_bLabelsAvailable = gcsGenericObject_GetLabelsFromString(dpGetDescription(sDp), g_diWsValues, g_dsWsLabels);
	}

    string sPlcDpName = "";
    dyn_string dsPlcDpNameExceptions;
    gcsConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, dsPlcDpNameExceptions);
    if (0 == dynlen(dsPlcDpNameExceptions)) {
      iRes = dpConnect("gcsWordStatus_WidgetAnimationCB",
          sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
          sDeviceName + ".statusInformation.selectedManager",
          sDeviceName + ".ProcessInput.AuPosRSt",
          sDeviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
          sPlcDpName+".alarm",
          sPlcDpName+".enabled");
		g_bCallbackConnected = (iRes >= 0);
    }
  }
  else gcsWordStatus_WidgetDisconnection(g_sWidgetType);
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordStatus_WidgetDisconnect
/**
Purpose: dpDisconnect to the DataPoint

  @param[in]  deviceName  Device DP name
  @param[in]  sPlcName  PLC name
  @param[in]  sSystemName System name

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in WordStatus faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.

*/
gcsWordStatus_WidgetDisconnect(string sDeviceName, string sPlcName, string sSystemName)
{
  int iRes;
  if (false == dpExists(sDeviceName))
    return;
  if(g_bUnSystemAlarmPlc)
  {
    string sPlcDpName = "";
    dyn_string dsPlcDpNameExceptions;
    gcsConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, dsPlcDpNameExceptions);
    if (0 == dynlen(dsPlcDpNameExceptions)) {
      iRes = dpDisconnect("gcsWordStatus_WidgetAnimationCB",
          sDeviceName + ".statusInformation.selectedManager:_lock._original._locked",
          sDeviceName + ".statusInformation.selectedManager",
          sDeviceName + ".ProcessInput.AuPosRSt",
          sDeviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
          sPlcDpName+".alarm",
          sPlcDpName+".enabled");
		g_bCallbackConnected = !(iRes >= 0);
    }
  }
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordStatus_WidgetAnimationCB
/**
Purpose: Animate widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in WordStatus faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
gcsWordStatus_WidgetAnimationCB(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
                  string sDpAuPosRSt, int iAuPosRSt, string sDp4, bool bAuPosRStInvalid,
                  string sDpAl, int iSystemIntegrityAlarmValue, string sDpEnabled, bool bSystemIntegrityAlarmEnabled)
{
  string sWarningLetter, sWarningColor;
  int iIndex, iLength, iBitCounter,iBitPos,j;
  string sFormattedValue;
  string sBody1Text, sBody1Color, sTempBodyText;

  // 1.1 if the device is not correclty running (the iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value)
  // or the checking is not enabled then set the letter to O and the color to not valid data
  if ((!bSystemIntegrityAlarmEnabled) || (iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value))
  {
    sWarningLetter = UN_WIDGET_TEXT_OLD_DATA;
    sWarningColor = "unDataNotValid";
  }

  // 1.2 Control animation. Check if the datapoint is invalid, then will be displayed in blue colors
  if (bAuPosRStInvalid)
  {
	  if(g_sWidgetType == "WS")
	  {
      		if (g_bLabelsAvailable != TRUE)
      		{
        			//"-" char represents an empty string
        			if (g_sAuPosRStUnit != "-")
      				  sBody1Text = iAuPosRSt + " " + g_sAuPosRStUnit;
        			else
        				  sBody1Text = iAuPosRSt;
        				  sBody1Color = "unDataNotValid";
      		}
    		  else
		      {
		        	sBody1Text = "Invalid Data";
		        	sBody1Color = "unDataNotValid";
		      }
	  }			
	  if(g_sWidgetType == "WordStatusBit")
	    {
		      sBody1Text = "Invalid Data";
		      sBody1Color = "unDataNotValid";
	    }
	  if(g_sWidgetType == "WordStatusBitBig")
	  {
	    for(int j=0;j<=dynlen(g_dsWsLabels)-1;j++)
	  	  {
		    unGenericObject_ColoredSquareAnimate("bit" + j, true , true, "unDataNotValid", "unDataNotValid", true);
		    }
		  sBody1Text = "Invalid Data";
		  sBody1Color = "unDataNotValid";
	  }
    sWarningLetter = UN_WIDGET_TEXT_INVALID;
    sWarningColor = "unDataNotValid";
  }
  else
    {//When it's valid
	      if(g_sWidgetType != "WordStatusBit")
      	{
          // DebugN("WordStatus Widget animation");
	      	// g_sAuPosRStFormat
	      	// 2. AuPosRSt value
      		// display the int value with or without the unit, or the corresponding string to this value
      		if (g_bLabelsAvailable != TRUE || dynContains( g_diWsValues, iAuPosRSt) == 0 || g_dsWsLabels[dynContains( g_diWsValues, iAuPosRSt)] == "N/A")
      		{
             //DebugN("g_bLabelsAvailable: "+g_bLabelsAvailable); 
      			  if (g_sAuPosRStUnit != "-")
      		  		sBody1Text = iAuPosRSt + " " + g_sAuPosRStUnit;
      			  else
      		  		sBody1Text = iAuPosRSt;
      		  		sBody1Color = "unWidget_ControlStateAuto";
      		}
      		else
      		{
            // retrieve position of the value in the list of value
            //DebugN("g_diWsValues: " +g_diWsValues);
            //DebugN("g_dsWsLabels: "+g_dsWsLabels);       
          	  iIndex = dynContains( g_diWsValues, iAuPosRSt);
      			  iLength = dynlen(g_dsWsLabels);

      			  // if the value is defined in the common of the AuPosRSt, display the corresponding string
      			  if (iIndex != 0)
      			  {
              	 sBody1Text = g_dsWsLabels[iIndex];
      				    sBody1Color = "unWidget_ControlStateAuto";
      			  }
	  		      else  // otherwise visualize the value itself
	  		      {
                //if value does not correspond to a string, should display invalid data	            	                
               // DebugN("no string associated to the value---> displays as invalid data!");                
                        sBody1Text = "Invalid Data";
                  				sBody1Color = "unWidget_ControlStateAuto";
             }
     	    }
	      }
	
        if(g_sWidgetType == "WordStatusBit")
        {
               // DebugN("WordStatusBit Widget animation");
            			for(int j=0;j<=15;j++)
            				{
              					if(getBit(iAuPosRSt,j)==1)
                						{
                          
                							if(iBitCounter==0)
                								{
                									iBitPos = j;
                								}
                							iBitCounter = iBitCounter + 1 ;
                						}
              				}
			            if(iBitCounter==1)
            				{
                    //DebugN("if only one bit is set it should display the corresponding string");
                    if (g_bLabelsAvailable != TRUE) 
                        {
                          sBody1Color = "unWidget_ControlStateAuto";
                          sBody1Text = "No message attributed";
                        }
                     else
                       {
                      				 sBody1Color = "unWidget_ControlStateAuto";
                      					sBody1Text =  sBody1Text + g_dsWsLabels[iBitPos+1];
                        }
             				}
                    
			            else if(iBitCounter==0)
			            	{
			                		sBody1Color = "unWidget_ControlStateAuto";
			                		sBody1Text = "No Status";
			            	}
			            else
			             	{
			                		sBody1Color = "unWidget_ControlStateAuto";
			                  sBody1Text =  "Multiple Status";
				             }
		       }
        
	if(g_sWidgetType == "WordStatusBitBig")
	{
    //DebugN("WordStatusBitBig Widget animation");
		  sBody1Text = "";
		  sTempBodyText = "";
  
			for(j = 0; j < dynlen(g_diWsValues); j++)
				{
					iIndex = dynContains(g_diWsValues, j);
					if(iIndex >0)
						{
							sTempBodyText = g_dsWsLabels[iIndex];
						}
					else
						{
							sTempBodyText = "No message attributed";
						}
        			sBody1Color = "unWidget_ControlStateAuto";
      				sBody1Text =  sBody1Text + sTempBodyText + "\n";
            unGenericObject_ColoredSquareAnimate("bit" + j, getBit(iAuPosRSt,j), TRUE, "unFaceplate_StatusActive", "unFaceplate_StatusActive", FALSE);
				}
		   	
     if(g_bSystemConnected)
		   	setMultiValue("Body1", "text", sBody1Text, "Body1", "fill", "[solid]",
                				"Body1", "foreCol", sBody1Color, "Body1", "backCol", "unWidget_Background",
                				"WarningText", "text", sWarningLetter, "WarningText", "foreCol", sWarningColor);
		   
    }
  }
  // 2. Animate widget
  if(g_bSystemConnected) {
	if ((g_sWidgetType!="step_PLC") &&  (g_sWidgetType!="transition_PLC")) {
		setMultiValue("Body1",        "text",     sBody1Text, //"Body1", "fill", "[solid]",
					  "Body1",        "foreCol",  sBody1Color, "Body1", "backCol", "unWidget_Background",
					  "WarningText",  "text",     sWarningLetter,  "WarningText", "foreCol", sWarningColor);
		if(shapeExists("WidgetArea"))
		  setValue("WidgetArea", "visible", true);
	} else {
		if(g_sWidgetType == "step_PLC") {
			setValue("step_Plc", "backCol", sBody1Color); 	   		
		} else if(g_sWidgetType == "transition_PLC") {
			setValue("line_transition", "foreCol", sBody1Color); 		
		}
	}
  }
}


//------------------------------------------------------------------------------------------------------------------------------
// gcsWordStatus_WidgetDisconnection
/**
Purpose: Animate widget disconnection

Parameters:
  @param[in] sWidgetType Widget type. ex: "heater"

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
gcsWordStatus_WidgetDisconnection(string sWidgetType)
{
  int j;
  dyn_string dsSquareName=makeDynString("");
  
  
    switch(sWidgetType) {
		case "WS":
		case "WordStatusBit":
		case "WordStatusBitBig":
			setMultiValue("Body1", "foreCol", "unDataNoAccess", "Body1", "backCol", "unWidget_Background", "Body1", "fill", "[solid]",
			   "WarningText", "text", "");
			if(shapeExists("WidgetArea")) 
 			   setValue("WidgetArea", "visible", false);
			if(sWidgetType == "WordStatusBitBig")
			{
			  for(j=0;j<=15;j++)     
			   {
				  dynAppend(dsSquareName,"bit"+j);
				  setValue("bit"+j +".border", "backCol", "unDataNoAccess");
				  setValue("bit"+j +".body", "backCol", "unWidget_Background");
			   }
			}
			break;
		case "step_PLC":
			gcsWordStatus_step_PLC_disconnection();
			break;
		case "transition_PLC":
			gcsWordStatus_transition_PLC_disconnection();
			break;
		default:
			break;
	}
}
// gcsWordStatus_FaceplateStatusRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
gcsWordStatus_FaceplateStatusRegisterCB(string sDp, bool bSystemConnected)
{
  //DebugN("gcsWordStatus_FaceplateStatusRegisterCB ");
	string deviceName, sSystemName;
	int iAction, iRes;
	dyn_string exceptionInfo;
	bool bRemote;

	deviceName = unGenericDpFunctions_getDpName($sDpName);
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = TRUE;


	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);

	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
    //DebugN("gcsWordStatus_FaceplateStatusRegisterCB: UN_ACTION_DISCONNECT");
			gcsWordStatus_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
     //DebugN("gcsWordStatus_FaceplateStatusRegisterCB: UN_ACTION_DISCONNECT");
			 gcsWordStatus_FaceplateConnect(deviceName);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
      //DebugN("gcsWordStatus_FaceplateStatusRegisterCB: UN_ACTION_DPDISCONNECT_DISCONNECT");
			gcsWordStatus_FaceplateDisconnect(deviceName); //function not defined ????
			gcsWordStatus_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}
// gcsWordStatus_FaceplateStatusAnimationCB
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in gcsWordStatus faceplate
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
gcsWordStatus_FaceplateStatusAnimationCB(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
 							    string sDpAuPosRSt, int iAuPosRSt, string sDp4, bool bAuPosRStInvalid,
 							    string sDpAl, int iSystemIntegrityAlarmValue, string sDpEnabled, bool bSystemIntegrityAlarmEnabled)
{
	c_unSystemIntegrity_no_alarm_value;
	bool bOkData=TRUE;
	string deviceName, sTempBodyText;
	string body1Text, body1Color, sWarningLetter, sWarningColor,sWarningLetter2, sWarningColor2, sUniqueMessageTextColor, sUniqueMessageText, sDecimalValueColor, sDecimalValueText;
	int j, index, length;
	dyn_string dsValue;
	dyn_int diBits;

	// 1.1 if the device is not correclty running (the iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value)
	// or the checking is not enabled then set the letter to O and the color to not valid data
	if ((!bSystemIntegrityAlarmEnabled) || (iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value))
	{
		sWarningLetter = UN_WIDGET_TEXT_OLD_DATA;
		sWarningColor = "unDataNotValid";
		sWarningLetter2 = UN_WIDGET_TEXT_OLD_DATA;
		sWarningColor2 = "unDataNotValid";
	}

	// 1.2 Control animation. Check if the datapoint is invalid, then will be displayed in blue colors
	if (bAuPosRStInvalid==false)
	{
		if(g_sWidgetType == "WordStatusBit") {
			  body1Text = "";
			  sTempBodyText = "";

				for(j = 0; j < dynlen(g_diWsValues); j++){
						index = dynContains(g_diWsValues, j);
						if(index >0){
							sTempBodyText = g_dsWsLabels[index];
						}
						else{
							sTempBodyText = "No message attributed";
						}
						body1Color = "unWidget_ControlStateAuto";
						body1Text =  body1Text + sTempBodyText + "\n";
						if (g_bSystemConnected)
							unGenericObject_ColoredSquareAnimate("bit" + j, getBit(iAuPosRSt,j), TRUE, "unFaceplate_StatusActive", "unFaceplate_StatusActive", FALSE);
				}
		}
		else{
			sUniqueMessageTextColor = "unWidget_ControlStateAuto";
			index = dynContains(g_diWsValues, iAuPosRSt);

			if(index>0){
			  sUniqueMessageText =  body1Text + g_dsWsLabels[index] + "\n";
			  sDecimalValueColor = "unWidget_ControlStateAuto";
			  sDecimalValueText = iAuPosRSt;
			}
			else{
				sUniqueMessageText = "No Description";
			  sDecimalValueColor = "unWidget_ControlStateAuto";
			  sDecimalValueText = iAuPosRSt;
			}
		}
	}
	else{
		//if data is invalid
			sWarningLetter = UN_WIDGET_TEXT_INVALID;
			sWarningColor = "unDataNotValid";
			sWarningLetter2 = UN_WIDGET_TEXT_INVALID;
			sWarningColor2 = "unDataNotValid";
			sUniqueMessageText = "Invalid Data";
			sUniqueMessageTextColor = "unDataNotValid";
			
			if(g_sWidgetType =="WordStatusBit")
			{
			    for(j=0;j<=dynlen(g_diWsValues)-1;j++){
					if (g_bSystemConnected)
						unGenericObject_ColoredSquareAnimate("bit" + j, true , true, "unDataNotValid", "unDataNotValid", true);
				}
			}
		}
		if(g_sWidgetType == "WordStatusBit") {
			if(g_bSystemConnected)
				setMultiValue("Body1", "text", body1Text, "Body1", "fill", "[solid]",
						  "Body1", "foreCol", body1Color, "Body1", "backCol", "unWidget_Background",
						  "WarningText", "text", sWarningLetter, "WarningText", "foreCol", sWarningColor);
		}
		else{
			if(g_bSystemConnected)
				setMultiValue(//"Body1", "text", body1Text, "Body1", "fill", "[solid]",
						          "UniqueMessageText", "text", sUniqueMessageText, "UniqueMessageText", "fill", "[solid]",
        						  "UniqueMessageText", "foreCol", sUniqueMessageTextColor, "UniqueMessageText", "backCol", "unWidget_Background",
         					  "DecimalValue", "text", sDecimalValueText, "DecimalValue", "fill", "[solid]",
						          "DecimalValue", "foreCol", sDecimalValueColor, "DecimalValue", "backCol", "unWidget_Background",
						          "WarningText2", "text", sWarningLetter2, "WarningText2", "foreCol", sWarningColor2);
		}
}
// gcsWordStatus_FaceplateConnect
/**
Purpose: connect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in WordStatus faceplate
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
gcsWordStatus_FaceplateConnect(string deviceName)
{
	int iRes, iAlertType1;
	dyn_string exceptionInfo;
	string sPlcName, sSystemName, sPlcDpName;

	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	
	gcsConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0) g_bUnSystemAlarmPlc = true;
	else g_bUnSystemAlarmPlc = false;

	if(g_bUnSystemAlarmPlc) {

		string sDP = deviceName + ".ProcessInput.AuPosRSt";
		g_sAuPosRStFormat = dpGetFormat(sDP);
		g_sAuPosRStUnit = dpGetUnit(sDP);
		g_bLabelsAvailable = gcsGenericObject_GetLabelsFromString(dpGetDescription(sDP), g_diWsValues, g_dsWsLabels);

			iRes = dpConnect("gcsWordStatus_FaceplateStatusAnimationCB",
							deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							deviceName + ".statusInformation.selectedManager",
							deviceName + ".ProcessInput.AuPosRSt",
						 	deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
						 	sSystemName + c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							sSystemName + c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled");
		g_bCallbackConnected = (iRes >= 0);
	} else {
		gcsWordStatus_FaceplateStatusDisconnection();
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordStatus_FaceplateDisconnect
/**
Do faceplate dpDisconnect

	@param[in]	deviceName	Device DP name
*/
gcsWordStatus_FaceplateDisconnect(string deviceName)
{
	int iRes, iAlertType1;
	dyn_string exceptionInfo;
	string sPlcName, sSystemName, sPlcDpName;

	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	
	gcsConfigGenericFunctions_getPlcDpName(sSystemName, sPlcName, sPlcDpName, exceptionInfo);
	if (dynlen(exceptionInfo) <= 0) g_bUnSystemAlarmPlc = true;
	else g_bUnSystemAlarmPlc = false;

	if(g_bUnSystemAlarmPlc) {
		string sDP = deviceName + ".ProcessInput.AuPosRSt";
		iRes = dpDisconnect("gcsWordStatus_FaceplateStatusAnimationCB",
						deviceName + ".statusInformation.selectedManager:_lock._original._locked",
						deviceName + ".statusInformation.selectedManager",
						deviceName + ".ProcessInput.AuPosRSt",
						deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
						sSystemName + c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
						sSystemName + c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled");
		g_bCallbackConnected = !(iRes >= 0);
	} else {
		gcsWordStatus_FaceplateStatusDisconnection();
	}
}

// gcsWordStatus_FaceplateStatusDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.1
	. operating system: XP.
	. distributed system: yes.
*/
gcsWordStatus_FaceplateStatusDisconnection()
{
	int j;
	if(g_sWidgetType == "WordStatusBit") {
		setMultiValue("Body1", "foreCol", "unDataNoAccess",
					  "WarningText", "foreCol", "unDataNoAccess");
		for(j=0;j<=dynlen(g_diWsValues)-1;j++){
			setValue("bit"+j +".border", "backCol", "unDataNoAccess");
			setValue("bit"+j +".body", "backCol", "unWidget_Background");
		}
	} else {
		setMultiValue("UniqueMessageText", "foreCol", "unDataNoAccess",
					  "DecimalValue", "foreCol", "unDataNoAccess",
					  "WarningText2", "foreCol", "unDataNoAccess");
	}
}
//@}
// gcsWordStatus__ObjectListGetValueTime
/**
Purpose: Function called from the objectList to return the time and value

Parameters:
	- sDeviceName, string, input, device name
	- sDeviceType, string, input, unicos object
	- dsReturnData, dyn_string, output, return value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate

*/
gcsWordStatus_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
	float value;
	string sTime, sStatus,sColor;
	bool bBad;

	dpGet(sDeviceName+".ProcessInput.AuPosRSt", value,
	      sDeviceName+".ProcessInput.AuPosRSt:_online.._stime", sTime,
	      sDeviceName+".ProcessInput.AuPosRSt:_online.._bad", bBad);

	dsReturnData[1] = sTime;
	dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(sDeviceName+".ProcessInput.AuPosRSt"), value) + " "
							+ dpGetUnit(sDeviceName+".ProcessInput.AuPosRSt");

	// Optionally (when no stsReg* exists)
	// Status
	dsReturnData[3] = bBad;

	// Status to visualize
	sStatus="OK";
	dsReturnData[4] = sStatus;
	// Colors
	sColor="unEventList_ForeColor";
	dsReturnData[5]=sColor;
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordStatus_step_PLC_CB
/**
Purpose: callback function for step PLC

Parameters:
	- sDp1, string, input, datapoint nane
	- fPosSt, float, input, value of datapoint
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

gcsWordStatus_step_PLC_CB (string sDp1, int iPosSt,
			 string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled)
{
	string sColor;

	if ((!bPlcEnabled) || (iAlValue > c_unSystemIntegrity_no_alarm_value))
		{
		sColor = "unDataNotValid";
		//DebugN("StepPLC unDataNotValid");
	}
	else {
		
		if (iPosSt==g_iValue)
			sColor=g_sColor_Active;
		else
			sColor=g_sColor_Passive;
	}
	if(g_bSystemConnected){
		setValue("step_Plc","backCol",sColor); 
	}

}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordStatus_step_PLC_WidgetDisconnect
/**
Purpose: Animate Step widget disconnect

Parameters:
	- deviceName, string, Device Name to disconnect
	- iRes, int, output, Disconnection result

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
gcsWordStatus_step_PLC_WidgetDisconnect(string deviceName, int &iRes)
{
string sPlcName, sSystemName;

	if(g_bUnSystemAlarmPlc) {
// get the systemName
		sSystemName = unGenericDpFunctions_getSystemName(deviceName);
		// get the PLC name 
		sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
		iRes = dpDisconnect("gcsWordStatus_step_PLC_CB", deviceName + ".ProcessInput.AuPosRSt",
												sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
												sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled");
		}		
}

//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordStatus_step_PLC_disconnection
/**
Purpose: Animate Step widget disconnection

Parameters:

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

gcsWordStatus_step_PLC_disconnection()
{

	step_Plc.backCol = "unDataNoAccess";	
}


//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordStatus_step_PLC_WidgetConnect
/**
Purpose: Animate Transition widget connection

Parameters:
	- iRes, string, input, device Name to connect

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

gcsWordStatus_step_PLC_WidgetConnect(string deviceName)
{
	int iRes, iAlertType1, iRangeType;
	dyn_string exceptionInfo;
	string sPlcName, sSystemName;

		
// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	if(sPlcName != "") {
		if (dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName)) 
			{
			g_bUnSystemAlarmPlc = true;
			
			iRes = dpConnect("gcsWordStatus_step_PLC_CB", deviceName + ".ProcessInput.AuPosRSt",
												sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
												sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled");
			}
		else 
			{
			g_bUnSystemAlarmPlc = false;
			gcsWordStatus_step_PLC_disconnection();
			}
		}	
	else 
		{
		g_bUnSystemAlarmPlc = false;
		gcsWordStatus_step_PLC_disconnection();
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordStatus_transition_PLC_WidgetConnect
/**
Purpose: Animate Transition widget connection

Parameters:
	- iRes, string, input, device Name to connect

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

gcsWordStatus_transition_PLC_WidgetConnect(string deviceName)
{
int iRes, iAlertType1, iRangeType;
dyn_string exceptionInfo;
string sPlcName, sSystemName;
			
// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	
	if (g_iBit > UN_TRANSITION_PLC_MAX_POSITION)
		sPlcName="";
		
	if (sPlcName != "") {
		if (dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName)) 
			{
			g_bUnSystemAlarmPlc = true;
			iRes = dpConnect("gcsWordStatus_transition_PLC_CB", deviceName + ".ProcessInput.AuPosRSt",
												sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
												sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled");
			}
		else 
			{
			g_bUnSystemAlarmPlc = false;
			gcsWordStatus_transition_PLC_disconnection();
			}
		}
	else {
		g_bUnSystemAlarmPlc = false;
		gcsWordStatus_transition_PLC_disconnection();
		}
				
}

//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordStatus_transition_PLC_CB
/**
Purpose: callback function for step PLC

Parameters:
	- sDp1, string, input, datapoint nane
	- iPosSt, iloat, input, value of datapoint (int to convert in bit32)
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

gcsWordStatus_transition_PLC_CB (string sDp1, int iPosSt,
			 string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled)
{
	string sColor;
	bit32 bitPost;
	
	bitPost=iPosSt;
	
	if ((!bPlcEnabled) || (iAlValue > c_unSystemIntegrity_no_alarm_value)){
		//DebugN("transitionPLC unDataNotValid");
		sColor = "unDataNotValid";
	}
	else {
		if (getBit(bitPost,g_iBit))
			sColor=g_sColor_Active;
		else
			sColor=g_sColor_Passive;
	}
	if(g_bSystemConnected)
	setValue("line_transition","foreCol",sColor); 
}
 
//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordStatus_transition_PLC_WidgetDisconnect
/**
Purpose: Animate Transition widget disconnect

Parameters:
	- deviceName, string, input, device name to disconnect
	- iRes, int, output, Disconnection result

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

gcsWordStatus_transition_PLC_WidgetDisconnect(string deviceName, int &iRes)
{
string sPlcName, sSystemName;
	
	if(g_bUnSystemAlarmPlc) {
// get the systemName
		sSystemName = unGenericDpFunctions_getSystemName(deviceName);
		// get the PLC name 
		sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
		iRes = dpDisconnect("gcsWordStatus_transition_PLC_CB", deviceName + ".ProcessInput.PosRSt",
												sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
												sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled");
		}
		
}

//---------------------------------------------------------------------------------------------------------------------------------------
// gcsWordStatus_transition_PLC_disconnection
/**
Purpose: Animate Transition widget disconnection

Parameters:

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

gcsWordStatus_transition_PLC_disconnection()
{
	line_transition.foreCol = "unDataNoAccess";
}

