/**@name LIBRARY: unAlarm.ctl

@author: Vincent Forest (AB-CO-IS)

Creation Date: 05 06 2003

Modification History:
  27/03/2009: Frederic
	- ..._WidgetRegisterCB(): fix problem with deviceType check during disconnection
        
  21/11/2008: Quique
    - New widget animation 
    
  07/05/2008: Herve
    - add faceplate in RC menu
    
  28/12/2007: Herve
    - device acccess control
    
	02/05/2006: Herve
		- implementation of maskEvent in widget, faceplate and rigthClick
		
	11/04/2006: Herve
		- remove dynAppend(menuList, "SEPARATOR"); in unAlarm_MenuConfiguration function

	14/12/2005: Celine
	- unAlarm_WidgetRegisterCB,unAlarm_WidgetAnimationCB: add g_bSystemConnected

	17/06/2005: Celine
		- add mail/sms in functions unAlarm_ButtonUserAccess, unAlarm_MenuConfiguration

	22/04/2005: Herve
		- use function unGenericObject_addDiagnosticAndInfoToActionToMenu in unAlarm_MenuConfiguration
		
	15/04/2005: Herve
		- unGenericObject_GetPLCNameFromDpName: return the PLCtype+"_"+plcName, in case of _UnPlc it returns _unPlc_+plcName
		
	05/11/2004: Herve
		- in the WidgetRegisterCB: use the systemName of the $sIdentifier to dpConnect to the PLC

	05/05/2004: Frederic
		- in unAlarm_MenuConfiguration function, add links to Diagnostic and Info
	
	03/05/2004: Herve
		- during WidgetRegister call unGenericObject_GetPLCNameFromDpName to get the PLC DPName
		- during WidgetRegister check if the dpType is correct
		
	28/04/2004: Herve
		- add function unAlarm_ObjectListGetValueTime

	14/04/2004: Herve
		- add the function unAlarm_HandleMenu

	13/04/2004: Herve
		- modification of the signature unAlarm_MenuConfiguration and add the creation of the menu.
		
	16/01/2004: Herve
		remove the dpConnect, dpDisconnect and callback function in the case of no PLC systemAlarm. Ease the development 
		and test.
		
	10/12/2003: Herve
		- add in all dpConnect and dpDisconnect the _unSystemAlarm_PLC_DS dp if the PLC is configured for systemIntegrity. 
		The g_bUnSystemAlarmPlc is set to true if the PLC is configured for systemIntegrity
		change the dpConnect and dpDisconnect of _aut_inv to _invalid

version 1.0

External Function : 
	. unAlarm_AcknowledgeAlarm : acknowledge Alarm alarms
	. unAlarm_ButtonUserAccess : User access for Alarm buttons
	. unAlarm_WidgetDisconnection : animate widget disconnection
	. unAlarm_MenuConfiguration : configuration of popup menu

Internal Functions :
	. unAlarm_WidgetRegisterCB : register callback function
	. unAlarm_WidgetAnimationCB : animate widget
	. unAlarm_WidgetAnimationCB_NoAlarm : animate widget
	
Purpose: This library contains Alarm object functions.

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Alarm object (DPType etc.)
	. Global variables and $parameters defined in Alarm faceplate and widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

//@{

//---------------------------------------------------------------------------------------------------------------------------------------

// unAlarm_ObjectListGetValueTime
/**
Purpose: Function called from the objectList to return the time and value

Parameters:
	- sDeviceName, string, input, device name
	- sDeviceType, string, input, unicos object
	- dsReturnData, dyn_string, output, return value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAlarm_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
	float value;
	string sTime;

	dpGet(sDeviceName+".ProcessInput.PosAl", value, sDeviceName+".ProcessInput.PosAl:_online.._stime", sTime);
	dsReturnData[1] = sTime;
	dsReturnData[2] = value;
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAlarm_AcknowledgeAlarm
/**
Purpose: acknowledge Alarm alarms

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsNeedAck, dyn_string, output, DPEs names that need acknowledgement. 
									 The PLC acknowledgement is specified using UN_ACKNOWLEDGE_PLC string

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAlarm_AcknowledgeAlarm(string sDpName, string sDpType, dyn_string &dsNeedAck)
{
	dsNeedAck = makeDynString("ProcessInput.PosAl", UN_ACKNOWLEDGE_PLC);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAlarm_MenuConfiguration
/**
Purpose: configuration of popup menu

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsAccessOk, dyn_string, input, authorized action
	- menuList, dyn_string, output, menu to pop-up
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAlarm_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{
  dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_ACK_TEXT, UN_POPUPMENU_MASK_POSAL_TEXT, UN_POPUPMENU_BLOCK_PLC_TEXT,"Mail-SMS", UN_POPUPMENU_FACEPLATE_TEXT);

  unGenericObject_addUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
  unGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAlarm_HandleMenu
/**
Purpose: handle the answer of the popup menu

Parameters:
	- deviceName, string, input, the device dp name
	- sDpType, string, input, the device dp type
	- menuList, dyn_string, input, list of requested action
	- menuAnswer, int, input, selected action from the menuList
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unAlarm_HandleMenu(string deviceName, string sDpType, dyn_string menuList, int menuAnswer)
{
	dyn_string exceptionInfo;

	switch(menuAnswer) {
		case UN_POPUPMENU_UNMASKEVENT:				// unmask event
			unGenericObject_setMaskEvent(deviceName, 1, 
																							makeDynString(".ProcessInput.evStsReg01", ".ProcessInput.PosAl"), exceptionInfo);
			break;
		case UN_POPUPMENU_MASKEVENT:				// mask event
			unGenericObject_setMaskEvent(deviceName, 0, 
																							makeDynString(".ProcessInput.evStsReg01", ".ProcessInput.PosAl"), exceptionInfo);
			break;
		default:
			unGenericObject_HandleUnicosMenu(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);
			break;
	}
	if (dynlen(exceptionInfo) > 0)
	{
		unSendMessage_toExpertException("Right Click",exceptionInfo);
	}

}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAlarm_ButtonUserAccess
/**
Purpose: User access for Alarm buttons

Parameters: 
	- sDpName, string, input, device name
	- sDpType, string, input, dptype
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAlarm_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
	bool operator, expert, admin;
	int i, length;
	dyn_bool dbPermissions;
	dyn_string exceptionInfo;
        string sOperAction, sExpAction, sAdminAction;
        bool bActionDefined = false;

        unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo);
        if((sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME) ||
           (sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME))
          bActionDefined=true;
	operator = dbPermissions[2];
	expert = dbPermissions[3];
	admin = dbPermissions[4];
	dsAccess = makeDynString();
	if (operator)
	{
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
          else
            dynAppend(dsAccess,makeDynString(UN_FACEPLATE_BUTTON_ACK_ALARM, UN_FACEPLATE_BUTTON_MASK_ALARM));
	}
	if (expert)
	{
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
          else
            dynAppend(dsAccess,makeDynString(UN_FACEPLATE_BUTTON_BLOCK_ALARM_ACTION,"Mail-SMS",UN_FACEPLATE_MASKEVENT));
	}
        if(admin){
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
        }
//DebugN(sDpName, dsAccess, sOperAction, sExpAction, sAdminAction);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAlarm_WidgetRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAlarm_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName;
	int iAction, iRes, iAlertType1;
	dyn_string exceptionInfo, split;
	string sPlcName, sSystemName;
	bool bRemote;
		
	deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
	if (deviceName == "")	// In case of disconnection
		{
		deviceName = g_sDpName;
		}
// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;

	if(sPlcName != "") {
		if (dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;

//  check if the device name is of type "Alarm" if not like of no PLC. -> not available.
  if (deviceName != "")
		{
    if(dpTypeName(deviceName) != "Alarm")
	    g_bUnSystemAlarmPlc = false;
    }
        
	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			unAlarm_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPCONNECT:
			g_sDpName = deviceName;
							
			if(g_bUnSystemAlarmPlc) {
				iRes = dpGet(deviceName + ".ProcessInput.PosAl:_alert_hdl.._type", iAlertType1);
				g_bBoolAlert = ((iRes >= 0) && (iAlertType1 == DPCONFIG_ALERT_BINARYSIGNAL));

				if (g_bBoolAlert)
				{
					iRes = dpConnect("unAlarm_WidgetAnimationCB", 
								deviceName + ".statusInformation.selectedManager:_lock._original._locked",
								deviceName + ".statusInformation.selectedManager",
								deviceName + ".ProcessInput.StsReg01",
								deviceName + ".ProcessInput.StsReg01:_online.._invalid",
								deviceName + ".ProcessInput.PosAl:_alert_hdl.._active",
								sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
								sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
								deviceName + ".eventMask");
				}
				else
				{
					iRes = dpConnect("unAlarm_WidgetAnimationCB_NoAlarm", 
								deviceName + ".statusInformation.selectedManager:_lock._original._locked",
								deviceName + ".statusInformation.selectedManager",
								deviceName + ".ProcessInput.StsReg01",
								deviceName + ".ProcessInput.StsReg01:_online.._invalid",
								sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
								sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
								deviceName + ".eventMask");				
				}
			g_bCallbackConnected = (iRes >= 0);
      }
			else 
				unAlarm_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			if(g_bUnSystemAlarmPlc) {
				if (g_bBoolAlert)
				{
					iRes = dpDisconnect("unAlarm_WidgetAnimationCB", 
								deviceName + ".statusInformation.selectedManager:_lock._original._locked",
								deviceName + ".statusInformation.selectedManager",
								deviceName + ".ProcessInput.StsReg01",
								deviceName + ".ProcessInput.StsReg01:_online.._invalid",
								deviceName + ".ProcessInput.PosAl:_alert_hdl.._active",
								sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
								sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
								deviceName + ".eventMask");
				}
				else
				{
					iRes = dpDisconnect("unAlarm_WidgetAnimationCB_NoAlarm", 
								deviceName + ".statusInformation.selectedManager:_lock._original._locked",
								deviceName + ".statusInformation.selectedManager",
								deviceName + ".ProcessInput.StsReg01",
								deviceName + ".ProcessInput.StsReg01:_online.._invalid",
								sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
								sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
								deviceName + ".eventMask");				
				}
			}
			g_bCallbackConnected = !(iRes >= 0);
			unAlarm_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAlarm_WidgetAnimationCB_NoAlarm
/**
Purpose: Animate widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Alarm faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAlarm_WidgetAnimationCB_NoAlarm(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
						   		  string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid,
						   		  string sDpAl, int iValAlarm, string sDpEnabled, bool bPlcEnabled,
						   		  string sEvtMask, int iMask)
{
	unAlarm_WidgetAnimationCB(sDpLock, bLocked, sDpManager, sSelectedManager, sDpStsReg01, bit32StsReg01, 
								sDp4, bStsReg01Invalid, "", 0, sDpAl, iValAlarm, sDpEnabled, bPlcEnabled, sEvtMask, iMask);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAlarm_WidgetAnimationCB
/**
Purpose: Animate widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Alarm faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAlarm_WidgetAnimationCB(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
						   string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid,
						   string sDp5, bool bIsAlarmActive, string sDpAl, int iValAlarm, string sDpEnabled, bool bPlcEnabled,
						   string sEvtMask, int iMask)
{
	string warningLetter = "", warningColor = "", alarmLetter = "", alarmColor = "", controlLetter = "", controlColor = "";
	string body1Color, body1BackColor, selectColor = "";
	bool lockVisible = false;
	
	if (!g_bBoolAlert)
		{
		alarmLetter = UN_WIDGET_TEXT_ALARM_DOES_NOT_EXIST;
		alarmColor = "unAlarmDoesNotExist";
		}
	else
		{
		if (!bIsAlarmActive)
			{
			alarmLetter = UN_WIDGET_TEXT_ALARM_MASKED;
			alarmColor = "unAlarmMasked";
			}
		}
	if (bStsReg01Invalid)
		{
		warningLetter = UN_WIDGET_TEXT_INVALID;
		warningColor = "unDataNotValid";
		body1Color = "unDataNotValid";
		body1BackColor = "unWidget_Background";
		}
	else
		{
		// body1Color = "unFaceplate_AlarmActive";
                body1Color = "unAlarm_Ok";
                
                // Warning letter
                if (getBit(bit32StsReg01,UN_STSREG01_IOERRORW)==1)    // IOerror
                        {
                        warningLetter=UN_WIDGET_TEXT_WARNING_ERROR;
                        warningColor= "unFaceplate_WarningActive";   
                        }
                if (getBit(bit32StsReg01,UN_STSREG01_IOSIMUW)==1)    // IOerror
                        {
                        warningLetter=UN_WIDGET_TEXT_WARNING_SIMU;
                        warningColor= "unFaceplate_WarningActive";   
                        }
                if (getBit(bit32StsReg01,UN_STSREG01_POSALW)==1)    // Position warning
                        {
                        warningLetter=UN_WIDGET_TEXT_WARNING_DEFAULT;
                        warningColor= "unFaceplate_WarningActive";   
                        }
                // Ack
		if (getBit(bit32StsReg01, UN_STSREG01_ALUNACK) == 1)
			{
			body1Color = "unWidget_AlarmNotAck";
			}
                else if (getBit(bit32StsReg01, UN_STSREG01_POSST) == 1)
                        {
			body1Color = "unFaceplate_AlarmActive";
			}
                // Block
		if (getBit(bit32StsReg01, UN_STSREG01_ALMSKST) == 1)    // Alarm BLOCKED in PLC
			{
			alarmLetter = UN_WIDGET_TEXT_ALARM_BLOCKED;
			alarmColor = "unAlarmMasked";
                        body1Color="unAlarmMasked";
			}	
		body1BackColor = "unWidget_Background";
		if (getBit(bit32StsReg01, UN_STSREG01_POSST) == 1)
			{
			body1BackColor = body1Color;
			}
		}

// 6.1 if the device is not correclty running (the iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value) 
// or the checking is not enabled then set the letter to O and the color to not valid data

	if ((!bPlcEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value))
		{
		warningLetter = UN_WIDGET_TEXT_OLD_DATA;
		warningColor = "unDataNotValid";
		}

	if(g_bSystemConnected)
	setMultiValue("Body1", "foreCol", body1Color, "Body1", "fill", "[solid]", "Body1", "backCol", body1BackColor,
				  "WarningText", "text", warningLetter, "WarningText", "foreCol", warningColor,
				  "AlarmText", "text", alarmLetter, "AlarmText", "foreCol", alarmColor,
				  "ControlStateText", "text", controlLetter, "ControlStateText", "foreCol", controlColor,
				  "SelectArea", "foreCol", selectColor, "LockBmp", "visible", lockVisible,
				  "WidgetArea", "visible", true);

// display event state
	if(g_bSystemConnected) {
		if(iMask == 0)
			eventState.visible	= true;
		else
			eventState.visible	= false;
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAlarm_WidgetDisconnection
/**
Purpose: Animate widget disconnection

Parameters:
	- sWidgetType, string, input, widget type. ex: "heater"

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAlarm_WidgetDisconnection(string sWidgetType)
{
	unGenericObject_WidgetDisconnection(sWidgetType, "Alarm");

	// set event state
	eventState.visible	= false;
}
//@}
