/**@name LIBRARY: unAnaDig.ctl

@author: Vincent Forest (AB-CO)

Creation Date: 21 05 2003

Modification History:

  15/11/2010: Quique
  - [UCPC-154] Inhibit manual/forced mode must be shown in faceplate and widget (letter "h")
  
  27/03/2009: Frederic
	  - ..._ButtonRegisterCB(): remove fwException_raise(...) before g_dsUserAccess
        
  13/10/2008: Quique
    - Include stsReg02 UN_STSREG02_ALBW bit to be taken into account in the widget warning letter animation / Also faceplate 
            
  28/12/2007: Herve
    - device acccess control
    
	29/06/2007: Herve
		- AnaDig widget: 2 letter RR
		
	14/12/2006: Herve
		- new param in unGenericDpFunctions_checkDeviceType
		
	08/05/2006: Herve
		- Implementation of the R
	
	02/05/2006: Herve
		- implementation of maskEvent in widget, faceplate and rigthClick
		
	11/04/2006: Herve
		- remove dynAppend(menuList, "SEPARATOR"); in unAnaDig_MenuConfiguration function

	15/12/2005: Celine
		- unAnaDig_WidgetAnimationCB, unAnaDig_WidgetRegisterCB, unAnaDig_FaceplateStatusAnimationCB,
		unAnaDig_FaceplateStatusRegisterCB : add g_bSystemConnected checking
		
	09/08/2005: Frederic
		- unAnaDig_ButtonRegisterCB(), add evalScript to run dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS] function
  
	15/04/2005: Herve
		- unGenericObject_GetPLCNameFromDpName: return the PLCtype+"_"+plcName, in case of _UnPlc it returns _unPlc_+plcName
		
	05/11/2004: Herve
		- in the WidgetRegisterCB, faceplateRegiterCB: use the systemName of the $sIdentifier to dpConnect to the PLC

	26/10/2004: Herve
		- in the unAnaDig_ButtonRegisterCB function: call the unAnaDig_ButtonUserAccess to get the list of allowed buttons. This list is 
		reseted if the user logs in/out while the remote system is off.

	22/10/2004: Herve
		- modify the unGenericObject_FormatValue, unGenericObject_WidgetDisplayValueAnimation, unGenericObject_DisplayValue call

	18/10/2004: Herve
		- add to unAnaDig_MenuConfiguration the function call unGenericObject_addTrendActionToMenu

	03/05/2004: Herve
		- during WidgetRegister and FaceplateRegister call unGenericObject_GetPLCNameFromDpName to get the PLC DPName
		- during WidgetRegister check if the dpType is correct

	28/04/2004: Herve
		- add function unAnaDig_ObjectListGetValueTime

	14/04/2004: Herve
		- add the function unAnaDig_HandleMenu
		- add the function unAnaDig_UserLoginGetButtonState
		
	13/04/2004: Herve
		- in function unAnaDig_ButtonRegisterCB add the set Manreg01 to 0 in case of dpConnect.
		- modification of the signature unAnaDig_MenuConfiguration and add the creation of the menu.
		
	08/04/2004: Herve 
		modification of the signature of the function unAnaDig_ButtonSetState

	16/01/2004: Herve
		remove the dpConnect, dpDisconnect and callback function in the case of no PLC systemAlarm. Ease the development 
		and test.
		
	11/12/2003: Herve
		- add in all dpConnect and dpDisconnect the _unSystemAlarm_PLC_DS dp if the PLC is configured for systemIntegrity. 
		The g_bUnSystemAlarmPlc is set to true if the PLC is configured for systemIntegrity
		change the dpConnect and dpDisconnect of _aut_inv to _invalid

version 1.0

External Function : 
	. unAnaDig_AcknowledgeAlarm : acknowledge AnaDig alarms
	. unAnaDig_FaceplateStatusDisconnection : animate faceplate disconnection
	. unAnaDig_ButtonUserAccess : User access for AnaDig buttons
	. unAnaDig_ButtonSetState : set button state
	. unAnaDig_WidgetDisconnection : animate widget disconnection
	. unAnaDig_MenuConfiguration : configuration of popup menu

Internal Functions :
	. unAnaDig_FaceplateStatusRegisterCB : register callback function
	. unAnaDig_FaceplateStatusAnimationCB_NoRangeNoAlarm : animate faceplate
	. unAnaDig_FaceplateStatusAnimationCB_NoAlarm : animate faceplate
	. unAnaDig_FaceplateStatusAnimationCB_NoRange : animate faceplate
	. unAnaDig_FaceplateStatusAnimationCB : animate faceplate
	. unAnaDig_ButtonRegisterCB : register callback function
	. unAnaDig_ButtonAnimationCB : animate buttons
	. unAnaDig_ButtonDisconnection : animate buttons disconnection
	. unAnaDig_WidgetRegisterCB : register callback function
	. unAnaDig_WidgetAnimationCB : animate widget

Purpose: This library contains AnaDig object functions.

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. AnaDig object (DPType etc.)
	. Global variables and $parameters defined in AnaDig faceplate and widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

//@{

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnaDig_ObjectListGetValueTime
/**
Purpose: Function called from the objectList to return the time and value

Parameters:
	- sDeviceName, string, input, device name
	- sDeviceType, string, input, unicos object
	- dsReturnData, dyn_string, output, return value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnaDig_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
	float value;
	string sTime;
	
	dpGet(sDeviceName+".ProcessInput.PosSt", value, sDeviceName+".ProcessInput.PosSt:_online.._stime", sTime);
	dsReturnData[1] = sTime;
	dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(sDeviceName+".ProcessInput.PosSt"), value) + " " 
							+ dpGetUnit(sDeviceName+".ProcessInput.PosSt");
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnaDig_AcknowledgeAlarm
/**
Purpose: acknowledge AnaDig alarms

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsNeedAck, dyn_string, output, DPEs names that need acknowledgement. 
									 The PLC acknowledgement is specified using UN_ACKNOWLEDGE_PLC string
	
Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in AnaDig faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnaDig_AcknowledgeAlarm(string sDpName, string sDpType, dyn_string &dsNeedAck)
{
	dsNeedAck = makeDynString("ProcessInput.StartISt", "ProcessInput.StopISt", UN_ACKNOWLEDGE_PLC);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnaDig_MenuConfiguration
/**
Purpose: configuration of popup menu

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsAccessOk, dyn_string, input, authorized action
	- menuList, dyn_string, output, menu to pop-up
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnaDig_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{
	dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, UN_POPUPMENU_ACK_TEXT, UN_POPUPMENU_FACEPLATE_TEXT, UN_POPUPMENU_TREND_TEXT);

	unGenericObject_addSelectToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	unGenericObject_addUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	unGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	unGenericObject_addTrendActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnaDig_HandleMenu
/**
Purpose: handle the answer of the popup menu

Parameters:
	- deviceName, string, input, the device dp name
	- sDpType, string, input, the device dp type
	- menuList, dyn_string, input, list of requested action
	- menuAnswer, int, input, selected action from the menuList
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unAnaDig_HandleMenu(string deviceName, string sDpType, dyn_string menuList, int menuAnswer)
{
	dyn_string exceptionInfo;

	switch(menuAnswer) {
		case UN_POPUPMENU_UNMASKEVENT:				// unmask event
			unGenericObject_setMaskEvent(deviceName, 1, 
																							makeDynString(".ProcessInput.evStsReg01", ".ProcessInput.evStsReg02",
																							".ProcessInput.StartISt", ".ProcessInput.StopISt"), exceptionInfo);
			break;
		case UN_POPUPMENU_MASKEVENT:				// mask event
			unGenericObject_setMaskEvent(deviceName, 0, 
																							makeDynString(".ProcessInput.evStsReg01", ".ProcessInput.evStsReg02",
																							".ProcessInput.StartISt", ".ProcessInput.StopISt"), exceptionInfo);
			break;
		default:
			unGenericObject_HandleUnicosMenu(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);
			break;
	}
	if (dynlen(exceptionInfo) > 0)
	{
		unSendMessage_toExpertException("Right Click",exceptionInfo);
	}

}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnaDig_FaceplateStatusRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in AnaDig faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnaDig_FaceplateStatusRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, sSystemName;
	int iAction;
	dyn_string exceptionInfo;
	bool bRemote;
		
	deviceName = unGenericDpFunctions_getDpName($sDpName);
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);

	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;

	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			unAnaDig_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
			unAnaDig_FaceplateConnect(deviceName);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			unAnaDig_FaceplateDisconnect(deviceName);
			unAnaDig_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnaDig_FaceplateConnect
/**
Purpose: connect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in AnaDig faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unAnaDig_FaceplateConnect(string deviceName)
{
	int iAction, iRes, iAlertType1, iAlertType2, iRangeType;
	dyn_string exceptionInfo;
	string sPlcName, sSystemName;

// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	if(sPlcName != "") {
		if (dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;
		
	iRes = dpGet(deviceName + ".ProcessInput.StartISt:_alert_hdl.._type", iAlertType1,
						 deviceName + ".ProcessInput.StopISt:_alert_hdl.._type", iAlertType2);
	g_bBoolAlert = ((iRes >= 0) && (iAlertType1 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType2 == DPCONFIG_ALERT_BINARYSIGNAL));
	iRes = dpGet(deviceName + ".ProcessInput.PosSt:_pv_range.._type", iRangeType);
	if ((iRes >= 0) && (iRangeType == DPCONFIG_MINMAX_PVSS_RANGECHECK))
	{
		g_iRangeType = DPCONFIG_MINMAX_PVSS_RANGECHECK;
	}
	else
	{
		g_iRangeType = DPCONFIG_NONE;
	}

	g_sPosStFormat=dpGetFormat(deviceName + ".ProcessInput.PosSt");
	g_sPosStUnit=dpGetUnit(deviceName + ".ProcessInput.PosSt");
	g_sAuPosRStFormat = dpGetFormat(deviceName + ".ProcessInput.AuPosRSt");
	g_sAuPosRStUnit = dpGetUnit(deviceName + ".ProcessInput.AuPosRSt");
	g_sOutOVStFormat = dpGetFormat(deviceName + ".ProcessInput.OutOVSt");
	g_sOutOVStUnit=dpGetUnit(deviceName + ".ProcessInput.OutOVSt");
	g_sMPosRStFormat=dpGetFormat(deviceName + ".ProcessInput.MPosRSt");
	g_sMPosRStUnit=dpGetUnit(deviceName + ".ProcessInput.MPosRSt");
	g_sPLiOnFormat=dpGetFormat(deviceName + ".ProcessOutput.PLiOn");
	g_sPLiOnUnit=dpGetUnit(deviceName + ".ProcessOutput.PLiOn");
	g_sPLiOffFormat=dpGetFormat(deviceName + ".ProcessOutput.PLiOff");
	g_sPLiOffUnit=dpGetUnit(deviceName + ".ProcessOutput.PLiOff");

	if(g_bUnSystemAlarmPlc) {
		if ((g_bBoolAlert) && (g_iRangeType == DPCONFIG_MINMAX_PVSS_RANGECHECK))
		{
				iRes = dpConnect("unAnaDig_FaceplateStatusAnimationCB",
								 deviceName + ".ProcessInput.StsReg01", 
								 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
								 deviceName + ".ProcessInput.StsReg02", 
								 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
								 deviceName + ".ProcessInput.PosSt",
								 deviceName + ".ProcessInput.PosSt:_online.._invalid",
								 deviceName + ".ProcessInput.AuPosRSt",
								 deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
								 deviceName + ".ProcessInput.OutOVSt",
								 deviceName + ".ProcessInput.OutOVSt:_online.._invalid",
								 deviceName + ".ProcessInput.MPosRSt",
								 deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
								 deviceName + ".ProcessOutput.PLiOn",
								 deviceName + ".ProcessOutput.PLiOn:_online.._invalid",
								 deviceName + ".ProcessOutput.PLiOff",
								 deviceName + ".ProcessOutput.PLiOff:_online.._invalid",
								 deviceName + ".ProcessInput.PosSt:_pv_range.._min",
								 deviceName + ".ProcessInput.PosSt:_pv_range.._max",
								 deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state",
								 deviceName + ".ProcessInput.StopISt:_alert_hdl.._act_state",
								 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
								 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
								 deviceName + ".eventMask");
		}
		if ((!g_bBoolAlert) && (g_iRangeType == DPCONFIG_MINMAX_PVSS_RANGECHECK))
		{
			iRes = dpConnect("unAnaDig_FaceplateStatusAnimationCB_NoAlarm",
								 deviceName + ".ProcessInput.StsReg01", 
								 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
								 deviceName + ".ProcessInput.StsReg02", 
								 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
								 deviceName + ".ProcessInput.PosSt",
								 deviceName + ".ProcessInput.PosSt:_online.._invalid",
								 deviceName + ".ProcessInput.AuPosRSt",
								 deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
								 deviceName + ".ProcessInput.OutOVSt",
								 deviceName + ".ProcessInput.OutOVSt:_online.._invalid",
								 deviceName + ".ProcessInput.MPosRSt",
								 deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
								 deviceName + ".ProcessOutput.PLiOn",
								 deviceName + ".ProcessOutput.PLiOn:_online.._invalid",
								 deviceName + ".ProcessOutput.PLiOff",
								 deviceName + ".ProcessOutput.PLiOff:_online.._invalid",
								 deviceName + ".ProcessInput.PosSt:_pv_range.._min",
								 deviceName + ".ProcessInput.PosSt:_pv_range.._max",
								 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
								 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
								 deviceName + ".eventMask");
		}	
		if ((g_bBoolAlert) && (g_iRangeType != DPCONFIG_MINMAX_PVSS_RANGECHECK))
		{
			iRes = dpConnect("unAnaDig_FaceplateStatusAnimationCB_NoRange",
								 deviceName + ".ProcessInput.StsReg01", 
								 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
								 deviceName + ".ProcessInput.StsReg02", 
								 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
								 deviceName + ".ProcessInput.PosSt",
								 deviceName + ".ProcessInput.PosSt:_online.._invalid",
								 deviceName + ".ProcessInput.AuPosRSt",
								 deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
								 deviceName + ".ProcessInput.OutOVSt",
								 deviceName + ".ProcessInput.OutOVSt:_online.._invalid",
								 deviceName + ".ProcessInput.MPosRSt",
								 deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
								 deviceName + ".ProcessOutput.PLiOn",
								 deviceName + ".ProcessOutput.PLiOn:_online.._invalid",
								 deviceName + ".ProcessOutput.PLiOff",
								 deviceName + ".ProcessOutput.PLiOff:_online.._invalid",
								 deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state",
								 deviceName + ".ProcessInput.StopISt:_alert_hdl.._act_state",
								 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
								 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
								 deviceName + ".eventMask");				
		}
		if ((!g_bBoolAlert) && (g_iRangeType != DPCONFIG_MINMAX_PVSS_RANGECHECK))
		{
			iRes = dpConnect("unAnaDig_FaceplateStatusAnimationCB_NoRangeNoAlarm",
								 deviceName + ".ProcessInput.StsReg01", 
								 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
								 deviceName + ".ProcessInput.StsReg02", 
								 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
								 deviceName + ".ProcessInput.PosSt",
								 deviceName + ".ProcessInput.PosSt:_online.._invalid",
								 deviceName + ".ProcessInput.AuPosRSt",
								 deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
								 deviceName + ".ProcessInput.OutOVSt",
								 deviceName + ".ProcessInput.OutOVSt:_online.._invalid",
								 deviceName + ".ProcessInput.MPosRSt",
								 deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
								 deviceName + ".ProcessOutput.PLiOn",
								 deviceName + ".ProcessOutput.PLiOn:_online.._invalid",
								 deviceName + ".ProcessOutput.PLiOff",
								 deviceName + ".ProcessOutput.PLiOff:_online.._invalid",
								 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
								 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
								 deviceName + ".eventMask");				
		}
	}
	else
		unAnaDig_FaceplateStatusDisconnection();
	g_bCallbackConnected = (iRes >= 0);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnaDig_FaceplateDisconnect
/**
Purpose: disconnect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in AnaDig faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unAnaDig_FaceplateDisconnect(string deviceName)
{
	int iRes;
	string sPlcName, sSystemName;

	if(g_bUnSystemAlarmPlc) {
// get the systemName
		sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
		sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);

		if ((g_bBoolAlert) && (g_iRangeType == DPCONFIG_MINMAX_PVSS_RANGECHECK))
		{
			iRes = dpDisconnect("unAnaDig_FaceplateStatusAnimationCB",
								 deviceName + ".ProcessInput.StsReg01", 
								 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
								 deviceName + ".ProcessInput.StsReg02", 
								 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
								 deviceName + ".ProcessInput.PosSt",
								 deviceName + ".ProcessInput.PosSt:_online.._invalid",
								 deviceName + ".ProcessInput.AuPosRSt",
								 deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
								 deviceName + ".ProcessInput.OutOVSt",
								 deviceName + ".ProcessInput.OutOVSt:_online.._invalid",
								 deviceName + ".ProcessInput.MPosRSt",
								 deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
								 deviceName + ".ProcessOutput.PLiOn",
								 deviceName + ".ProcessOutput.PLiOn:_online.._invalid",
								 deviceName + ".ProcessOutput.PLiOff",
								 deviceName + ".ProcessOutput.PLiOff:_online.._invalid",
								 deviceName + ".ProcessInput.PosSt:_pv_range.._min",
								 deviceName + ".ProcessInput.PosSt:_pv_range.._max",
								 deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state",
								 deviceName + ".ProcessInput.StopISt:_alert_hdl.._act_state",
								 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
								 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
								 deviceName + ".eventMask");
		}
		if ((!g_bBoolAlert) && (g_iRangeType == DPCONFIG_MINMAX_PVSS_RANGECHECK))
		{
			iRes = dpDisconnect("unAnaDig_FaceplateStatusAnimationCB_NoAlarm",
								 deviceName + ".ProcessInput.StsReg01", 
								 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
								 deviceName + ".ProcessInput.StsReg02", 
								 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
								 deviceName + ".ProcessInput.PosSt",
								 deviceName + ".ProcessInput.PosSt:_online.._invalid",
								 deviceName + ".ProcessInput.AuPosRSt",
								 deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
								 deviceName + ".ProcessInput.OutOVSt",
								 deviceName + ".ProcessInput.OutOVSt:_online.._invalid",
								 deviceName + ".ProcessInput.MPosRSt",
								 deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
								 deviceName + ".ProcessOutput.PLiOn",
								 deviceName + ".ProcessOutput.PLiOn:_online.._invalid",
								 deviceName + ".ProcessOutput.PLiOff",
								 deviceName + ".ProcessOutput.PLiOff:_online.._invalid",
								 deviceName + ".ProcessInput.PosSt:_pv_range.._min",
								 deviceName + ".ProcessInput.PosSt:_pv_range.._max",
								 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
								 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
								 deviceName + ".eventMask");
		}	
		if ((g_bBoolAlert) && (g_iRangeType != DPCONFIG_MINMAX_PVSS_RANGECHECK))
		{
			iRes = dpDisconnect("unAnaDig_FaceplateStatusAnimationCB_NoRange",
								 deviceName + ".ProcessInput.StsReg01", 
								 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
								 deviceName + ".ProcessInput.StsReg02", 
								 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
								 deviceName + ".ProcessInput.PosSt",
								 deviceName + ".ProcessInput.PosSt:_online.._invalid",
								 deviceName + ".ProcessInput.AuPosRSt",
								 deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
								 deviceName + ".ProcessInput.OutOVSt",
								 deviceName + ".ProcessInput.OutOVSt:_online.._invalid",
								 deviceName + ".ProcessInput.MPosRSt",
								 deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
								 deviceName + ".ProcessOutput.PLiOn",
								 deviceName + ".ProcessOutput.PLiOn:_online.._invalid",
								 deviceName + ".ProcessOutput.PLiOff",
								 deviceName + ".ProcessOutput.PLiOff:_online.._invalid",
								 deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state",
								 deviceName + ".ProcessInput.StopISt:_alert_hdl.._act_state",
								 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
								 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
								 deviceName + ".eventMask");				
		}
		if ((!g_bBoolAlert) && (g_iRangeType != DPCONFIG_MINMAX_PVSS_RANGECHECK))
		{
			iRes = dpDisconnect("unAnaDig_FaceplateStatusAnimationCB_NoRangeNoAlarm",
								 deviceName + ".ProcessInput.StsReg01", 
								 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
								 deviceName + ".ProcessInput.StsReg02", 
								 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
								 deviceName + ".ProcessInput.PosSt",
								 deviceName + ".ProcessInput.PosSt:_online.._invalid",
								 deviceName + ".ProcessInput.AuPosRSt",
								 deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
								 deviceName + ".ProcessInput.OutOVSt",
								 deviceName + ".ProcessInput.OutOVSt:_online.._invalid",
								 deviceName + ".ProcessInput.MPosRSt",
								 deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
								 deviceName + ".ProcessOutput.PLiOn",
								 deviceName + ".ProcessOutput.PLiOn:_online.._invalid",
								 deviceName + ".ProcessOutput.PLiOff",
								 deviceName + ".ProcessOutput.PLiOff:_online.._invalid",
								 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
								 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
								 deviceName + ".eventMask");				
		}
	}
	g_bCallbackConnected = !(iRes >= 0);

}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnaDig_FaceplateStatusAnimationCB_NoRangeNoAlarm
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnaDig_FaceplateStatusAnimationCB_NoRangeNoAlarm(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
									string sStsReg02, bit32 bit32StsReg02, string sDp4, bool bStsReg02Invalid,
									string sDpPosSt, float fPosSt, string sDp6, bool bPosStInvalid, 
				 			        string sDpAuto, float fAuPosRSt, string sDp8, bool bAuPosRStInvalid,
				 			        string sDpOutput, float fOutOVSt, string sDp10, bool bOutOVStInvalid,
				 			        string sDpMPosRSt, float fMPosRSt, string sDp12, bool bMPosRStInvalid,
				 			        string sDpLimitOn, float fPLiOn, string sDp14, bool bPLiOnInvalid,
				 			        string sDpLimitOff, float fPLiOff, string sDp16, bool bPLiOffInvalid,
				 			        string sDpAl, int iValAlarm, string sDpEnabled, bool bPlcEnabled,
				 			        string sDpEvt, int iMask)
{
	unAnaDig_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid, 
									    sStsReg02, bit32StsReg02, sDp4, bStsReg02Invalid,
										sDpPosSt, fPosSt, sDp6, bPosStInvalid,
					 			        sDpAuto, fAuPosRSt, sDp8, bAuPosRStInvalid,
					 			        sDpOutput, fOutOVSt, sDp10, bOutOVStInvalid,
					 			        sDpMPosRSt, fMPosRSt, sDp12, bMPosRStInvalid,
					 			        sDpLimitOn, fPLiOn, sDp14, bPLiOnInvalid,
					 			        sDpLimitOff, fPLiOff, sDp16, bPLiOffInvalid,
					 			        "", 0, "", 0, "", 0, "", 0,
					 			        sDpAl, iValAlarm, sDpEnabled, bPlcEnabled, sDpEvt, iMask);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnaDig_FaceplateStatusAnimationCB_NoRange
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnaDig_FaceplateStatusAnimationCB_NoRange(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
									string sStsReg02, bit32 bit32StsReg02, string sDp4, bool bStsReg02Invalid,
									string sDpPosSt, float fPosSt, string sDp6, bool bPosStInvalid, 
				 			        string sDpAuto, float fAuPosRSt, string sDp8, bool bAuPosRStInvalid,
				 			        string sDpOutput, float fOutOVSt, string sDp10, bool bOutOVStInvalid,
				 			        string sDpMPosRSt, float fMPosRSt, string sDp12, bool bMPosRStInvalid,
				 			        string sDpLimitOn, float fPLiOn, string sDp14, bool bPLiOnInvalid,
				 			        string sDpLimitOff, float fPLiOff, string sDp16, bool bPLiOffInvalid,
									string sDp19, int iStartIStState, string sDp20, int iStopIStState,
				 			        string sDpAl, int iValAlarm, string sDpEnabled, bool bPlcEnabled,
				 			        string sDpEvt, int iMask)
{
	unAnaDig_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid, 
									    sStsReg02, bit32StsReg02, sDp4, bStsReg02Invalid,
										sDpPosSt, fPosSt, sDp6, bPosStInvalid,
					 			        sDpAuto, fAuPosRSt, sDp8, bAuPosRStInvalid,
					 			        sDpOutput, fOutOVSt, sDp10, bOutOVStInvalid,
					 			        sDpMPosRSt, fMPosRSt, sDp12, bMPosRStInvalid,
					 			        sDpLimitOn, fPLiOn, sDp14, bPLiOnInvalid,
					 			        sDpLimitOff, fPLiOff, sDp16, bPLiOffInvalid,
					 			        "", 0, "", 0, sDp19, iStartIStState, sDp19, iStopIStState,
					 			        sDpAl, iValAlarm, sDpEnabled, bPlcEnabled, sDpEvt, iMask);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnaDig_FaceplateStatusAnimationCB_NoAlarm
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnaDig_FaceplateStatusAnimationCB_NoAlarm(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
									string sStsReg02, bit32 bit32StsReg02, string sDp4, bool bStsReg02Invalid,
									string sDpPosSt, float fPosSt, string sDp6, bool bPosStInvalid, 
				 			        string sDpAuto, float fAuPosRSt, string sDp8, bool bAuPosRStInvalid,
				 			        string sDpOutput, float fOutOVSt, string sDp10, bool bOutOVStInvalid,
				 			        string sDpMPosRSt, float fMPosRSt, string sDp12, bool bMPosRStInvalid,
				 			        string sDpLimitOn, float fPLiOn, string sDp14, bool bPLiOnInvalid,
				 			        string sDpLimitOff, float fPLiOff, string sDp16, bool bPLiOffInvalid,
									string sDp17, float minPosSt, string sDp18, float maxPosSt,
				 			        string sDpAl, int iValAlarm, string sDpEnabled, bool bPlcEnabled,
				 			        string sDpEvt, int iMask)
{
	unAnaDig_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid, 
									    sStsReg02, bit32StsReg02, sDp4, bStsReg02Invalid,
										sDpPosSt, fPosSt, sDp6, bPosStInvalid,
					 			        sDpAuto, fAuPosRSt, sDp8, bAuPosRStInvalid,
					 			        sDpOutput, fOutOVSt, sDp10, bOutOVStInvalid,
					 			        sDpMPosRSt, fMPosRSt, sDp12, bMPosRStInvalid,
					 			        sDpLimitOn, fPLiOn, sDp14, bPLiOnInvalid,
					 			        sDpLimitOff, fPLiOff, sDp16, bPLiOffInvalid,
					 			        sDp17, minPosSt, sDp18, maxPosSt, "", 0, "", 0,
					 			        sDpAl, iValAlarm, sDpEnabled, bPlcEnabled, sDpEvt, iMask);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnaDig_FaceplateStatusAnimationCB
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in AnaDig faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnaDig_FaceplateStatusAnimationCB(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
									string sStsReg02, bit32 bit32StsReg02, string sDp4, bool bStsReg02Invalid,
									string sDpPosSt, float fPosSt, string sDp6, bool bPosStInvalid, 
				 			        string sDpAuto, float fAuPosRSt, string sDp8, bool bAuPosRStInvalid,
				 			        string sDpOutput, float fOutOVSt, string sDp10, bool bOutOVStInvalid,
				 			        string sDpMPosRSt, float fMPosRSt, string sDp12, bool bMPosRStInvalid,
				 			        string sDpLimitOn, float fPLiOn, string sDp14, bool bPLiOnInvalid,
				 			        string sDpLimitOff, float fPLiOff, string sDp16, bool bPLiOffInvalid,
									string sDp17, float minPosSt, string sDp18, float maxPosSt,
									string sDp19, int iStartIStState, string sDp20, int iStopIStState,
				 			        string sDpAl, int iValAlarm, string sDpEnabled, bool bSystemIntegrityAlarmEnabled,
				 			        string sDpEvt, int iMask)
{
// 1. Animate display values
if(g_bSystemConnected)
	unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, fPosSt, "StatusPosition", "unDisplayValue_Status", 
						bPosStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_DisplayValue(g_sAuPosRStFormat, g_sAuPosRStUnit, fAuPosRSt, "RequestAuto", "unDisplayValue_Request", 
						bAuPosRStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_DisplayValue(g_sOutOVStFormat, g_sOutOVStUnit, fOutOVSt, "RequestOutput", "unDisplayValue_Output", 
						bOutOVStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_DisplayValue(g_sMPosRStFormat, g_sMPosRStUnit, fMPosRSt, "RequestManual", "unDisplayValue_Request", 
						bMPosRStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_DisplayValue(g_sPLiOnFormat, g_sPLiOnUnit, fPLiOn, "StatusLimitOn", "unDisplayValue_Parameter", 
						bPLiOnInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_DisplayValue(g_sPLiOffFormat, g_sPLiOffUnit, fPLiOff, "StatusLimitOff", "unDisplayValue_Parameter", 
						bPLiOffInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_DisplayRange(g_iRangeType, g_sPosStFormat, g_sPosStUnit, minPosSt, maxPosSt, "RangeMin", "RangeMax", "unDisplayValue_Parameter");
// 2. Animate colorboxes
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OperationMode_Auto", getBit(bit32StsReg01, UN_STSREG01_AUMOST), 1, "unFaceplate_OperationModeActive", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OperationMode_Manual", getBit(bit32StsReg01, UN_STSREG01_MMOST), 1, "unFaceplate_OperationModeActive", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OperationMode_Forced", getBit(bit32StsReg01, UN_STSREG01_FOMOST), 1, "unFaceplate_OperationModeActive", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OperationMode_Local", getBit(bit32StsReg01, UN_STSREG01_LDST), 1, "unFaceplate_OperationModeActive", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Warning_IOError", getBit(bit32StsReg01, UN_STSREG01_IOERRORW), 1, "unFaceplate_WarningActive", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Warning_IOSimulated", getBit(bit32StsReg01, UN_STSREG01_IOSIMUW), 1, "unFaceplate_WarningActive", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Warning_AlBW", getBit(bit32StsReg02, UN_STSREG02_ALBW), 1, "unFaceplate_WarningActive", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Warning_ManualAuto", getBit(bit32StsReg01, UN_STSREG01_AUMRW), 1, "unFaceplate_WarningActive", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Warning_PositionWarning", getBit(bit32StsReg01, UN_STSREG01_POSALW), 1, "unFaceplate_WarningActive", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Status_Failsafe", getBit(bit32StsReg01, UN_STSREG01_PFSPOSONST), getBit(bit32StsReg01, UN_STSREG01_PFSPOSONST), 
			"unFaceplate_StatusActive", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Request_Off", getBit(bit32StsReg02, UN_STSREG02_DOUTOFFO), 1, "unFaceplate_RequestActive", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Request_On", getBit(bit32StsReg02, UN_STSREG02_DOUTONO), 1, "unFaceplate_RequestActive", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
// 3. Animate alarm colorboxes
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimateAlarm("Alarm_StartISt", iStartIStState, g_bBoolAlert, 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimateAlarm("Alarm_StopISt", iStopIStState, g_bBoolAlert, 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Alarm_UnAck", getBit(bit32StsReg01, UN_STSREG01_ALUNACK), 1, "unFaceplate_AlarmNotAck", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));

// [UCPC-154] Auto Inhibit Manual/Forced mode  UN_STSREG01_AUINHFMO,UN_STSREG01_AUINHMMO
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Status_AuInhFor", getBit(bit32StsReg01, UN_STSREG01_AUINHFMO), 1, "unFaceplate_OperationModeActive", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Status_AuInhMan", getBit(bit32StsReg01, UN_STSREG01_AUINHMMO), 1, "unFaceplate_OperationModeActive", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));


// display event state
	if(g_bSystemConnected) {
		unGenericObject_ColorBoxAnimate("eventState", iMask, 1, "", 
				bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value),
				"unFaceplate_OperationModeActive");
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnaDig_FaceplateStatusDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnaDig_FaceplateStatusDisconnection()
{
	unGenericObject_ColorBoxDisconnect(makeDynString("Alarm_StartISt", "Alarm_StopISt", "Alarm_UnAck", "OperationMode_Auto",
													 "OperationMode_Forced", "OperationMode_Local", "OperationMode_Manual",
													 "Warning_IOError", "Warning_IOSimulated", "Warning_ManualAuto", 
													 "Warning_PositionWarning", "Request_Off", "Request_On", "Status_Failsafe"));
	unGenericObject_DisplayValueDisconnect(makeDynString("RangeMax", "RangeMin", "RequestAuto", "RequestManual", "RequestOutput",
														 "StatusLimitOff", "StatusLimitOn", "StatusPosition"));
// set eventState
	unGenericObject_ColorBoxDisconnect(makeDynString("eventState"));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnaDig_ButtonRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in AnaDig faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnaDig_ButtonRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, sDpType, sFunction;
	int iAction, iRes, iManReg01;
	dyn_string exceptionInfo, dsUserAccess, dsFunctions;
	
	deviceName = unGenericDpFunctions_getDpName($sDpName);
	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			unAnaDig_ButtonDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
// Send 0 to ManReg01 if necessary
			iRes = dpGet(deviceName + ".ProcessOutput.ManReg01", iManReg01);
			if ((iRes < 0) || (iManReg01 != 0))
			{
				dpSet(deviceName + ".ProcessOutput.ManReg01", 0);
			}
		
// call the unAnaDig_ButtonUserAccess to get the list of allowed buttons.
//			unAnaDig_ButtonUserAccess(deviceName, "AnaDig", g_dsUserAccess);

			sDpType="AnaDig";
			unGenericObject_GetFunctions(sDpType, dsFunctions, exceptionInfo);
			sFunction = dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS];
						 
			if (sFunction != "")
				{
				evalScript(dsUserAccess, "dyn_string main(string deviceNameTemp, string sDpTypeTemp) {" + 
							 "dyn_string dsUserAccessReturn;" +
							 "if (isFunctionDefined(\"" + sFunction  + "\"))" +
							 "    {" +
					 			sFunction  + "(deviceNameTemp, sDpTypeTemp, dsUserAccessReturn);" +
							 "    }" +
							 "return dsUserAccessReturn; }", makeDynString(), deviceName, sDpType);

				g_dsUserAccess=dsUserAccess;	
				}
			else
				fwException_raise(exceptionInfo,"ERROR", "unAnaDig_ButtonRegisterCB: " + getCatStr("unGeneration","UNKNOWNFUNCTION"), "EMPTY");
					

			iRes = dpConnect("unAnaDig_ButtonAnimationCB",
							 deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							 deviceName + ".statusInformation.selectedManager",
							 deviceName + ".ProcessInput.StsReg01",
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
							 deviceName + ".ProcessInput.StsReg02",
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid");
			g_bCallbackConnected = (iRes >= 0);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			iRes = dpDisconnect("unAnaDig_ButtonAnimationCB",
							 deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							 deviceName + ".statusInformation.selectedManager",
							 deviceName + ".ProcessInput.StsReg01",
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
							 deviceName + ".ProcessInput.StsReg02",
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid");
			g_bCallbackConnected = !(iRes >= 0);
			unAnaDig_ButtonDisconnection();
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnaDig_ButtonAnimationCB
/**
Purpose: animate buttons

Parameters: 
	- dbPermissions, dyn_bool, input, list of permissions, length = 4, order : 1=monitor, 2=operator, 3=expert, 4=admin
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnaDig_ButtonAnimationCB(string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager, 
						   string sDpStsReg01, bit32 stsReg01Value, string sDpStsReg01Bad, bool bStsReg01Bad,
						   string sDpStsReg02, bit32 stsReg02Value, string sDpStsReg02Bad, bool bStsReg02Bad)
{
	unAnaDig_ButtonSetState(unGenericDpFunctions_getDpName(sDpStsReg01), "AnaDig", g_dsUserAccess, 
						 	makeDynString(bLocked, sSelectedManager, stsReg01Value, bStsReg01Bad, stsReg02Value, bStsReg02Bad));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnaDig_ButtonUserAccess
/**
Purpose: User access for AnaDig buttons

Parameters: 
	- sDpName, string, input, device name
	- sDpType, string, input, dptype
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnaDig_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
	bool operator, expert, admin;
	int i, length;
	dyn_bool dbPermissions;
	dyn_string exceptionInfo;
        string sOperAction, sExpAction, sAdminAction;
        bool bActionDefined = false;

        unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo);
        if((sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME) ||
           (sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME))
          bActionDefined=true;
	operator = dbPermissions[2];
	expert = dbPermissions[3];
	admin = dbPermissions[4];
	dsAccess = makeDynString();
	if (operator)
	{
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
          else
            dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_MANUAL_MODE, UN_FACEPLATE_BUTTON_SET_VALUE, 
							UN_FACEPLATE_BUTTON_ONOPEN_REQUEST, UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST, UN_FACEPLATE_BUTTON_ACK_ALARM,
							UN_FACEPLATE_BUTTON_INC_VALUE, UN_FACEPLATE_BUTTON_DEC_VALUE, UN_FACEPLATE_BUTTON_SET_STATUS_LIMITS,
							UN_FACEPLATE_BUTTON_SELECT));
	}
	if (expert)
	{
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
          else
	    dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_FORCED_MODE, UN_FACEPLATE_MASKEVENT));
	}
        if(admin){
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
        }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnaDig_UserLoginGetButtonState
/**
Purpose: function called before the button set state when a user logs in. 

Parameters:
	deviceName: string, input, the device name
	dsReturnData: dyn_string, output, the data are returned here

Usage: Internal function

PVSS manager usage:  Ctrl, NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
		-> $sDpName, string, the device DP name
		-> g_sFaceplateButtonType, string, faceplate type
	. PVSS version: 2.12.1 
	. operating system: NT, W2000 and Linux, but tested only under W2000.
	. distributed system: yes but not tested.
*/

unAnaDig_UserLoginGetButtonState(string deviceName, string sType, dyn_string &dsReturnData)
{
	unGenericObject_UserCBGetButtonState(deviceName, sType, dsReturnData);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnaDig_ButtonSetState
/**
Purpose: set button state

Parameters: 
	- deviceName, string, input, device
	- sDpType, string, input, dptype
	- dsUserAccess, dyn_string, input, user access
	- dsData, dyn_string, parameter
		
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnaDig_ButtonSetState(string deviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
	bool bSelected, buttonEnabled, bit1, bit2, bit3;
	dyn_string dsButtons;
	int i, length;
	string localManager;
	bool bLocked, bStsReg01Bad, bStsReg02Bad;
	string sSelectedManager;
	bit32 stsReg01Value, stsReg02Value; 

	bLocked = dsData[1];
	sSelectedManager = dsData[2];
	stsReg01Value = dsData[3];
	bStsReg01Bad = dsData[4];
	stsReg02Value = dsData[5];
	bStsReg02Bad = dsData[6];

//DebugN("AnaDig_SetState", bLocked, sSelectedManager, stsReg01Value, bStsReg01Bad, stsReg02Value, bStsReg02Bad);
	
	dsButtons = makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_MANUAL_MODE, UN_FACEPLATE_BUTTON_SET_VALUE, 
							  UN_FACEPLATE_BUTTON_ONOPEN_REQUEST, UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST, UN_FACEPLATE_BUTTON_ACK_ALARM,
							  UN_FACEPLATE_BUTTON_INC_VALUE, UN_FACEPLATE_BUTTON_DEC_VALUE, UN_FACEPLATE_BUTTON_SET_STATUS_LIMITS,
							  UN_FACEPLATE_BUTTON_FORCED_MODE); // Each button except "Select" button
	length = dynlen(dsButtons);
// 1. Selection state
	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
	bSelected = (localManager == "S");
// 2. Button state
	for(i=1;i<=length;i++)
		{
		buttonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0);	// User access
		switch (dsButtons[i])
			{
			case UN_FACEPLATE_BUTTON_FORCED_MODE:
				bit1 = !(getBit(stsReg01Value, UN_STSREG01_FOMOST) == 0) && !(bStsReg01Bad || bStsReg02Bad);	
				bit2 = !(getBit(stsReg01Value, UN_STSREG01_AUINHFMO) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				bit3 = !(getBit(stsReg01Value, UN_STSREG01_LDST) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				buttonEnabled = buttonEnabled && bSelected && !(bit1 || bit2 || bit3);
				break;
			case UN_FACEPLATE_BUTTON_MANUAL_MODE:
				bit1 = !(getBit(stsReg01Value, UN_STSREG01_MMOST) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				bit2 = !(getBit(stsReg01Value, UN_STSREG01_AUINHMMO) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				bit3 = !(getBit(stsReg01Value, UN_STSREG01_LDST) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				buttonEnabled = buttonEnabled && bSelected && !(bit1 || bit2 || bit3);
				break;
			case UN_FACEPLATE_BUTTON_INC_VALUE:
			case UN_FACEPLATE_BUTTON_DEC_VALUE:
			case UN_FACEPLATE_BUTTON_ONOPEN_REQUEST:
			case UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST:
			case UN_FACEPLATE_BUTTON_AUTO_MODE:
			case UN_FACEPLATE_BUTTON_SET_VALUE:
				bit1 = !(getBit(stsReg01Value, UN_STSREG01_AUMOST) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				bit2 = !(getBit(stsReg01Value, UN_STSREG01_LDST) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				buttonEnabled = buttonEnabled && bSelected && !(bit1 || bit2);
				break;
			case UN_FACEPLATE_BUTTON_SET_STATUS_LIMITS:
				buttonEnabled = buttonEnabled && bSelected;
				break;
			case UN_FACEPLATE_BUTTON_ACK_ALARM:
			default:
				break;
			}
		setValue(UN_FACEPLATE_BUTTON_PREFIX + dsButtons[i], "enabled", buttonEnabled);
		}
// 3. Button Select state
	unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnaDig_ButtonDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnaDig_ButtonDisconnection()
{
	unGenericObject_ButtonDisconnect(makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_MANUAL_MODE, 
												   UN_FACEPLATE_BUTTON_SET_VALUE, UN_FACEPLATE_BUTTON_ONOPEN_REQUEST, 
												   UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST, UN_FACEPLATE_BUTTON_ACK_ALARM,
												   UN_FACEPLATE_BUTTON_INC_VALUE, UN_FACEPLATE_BUTTON_DEC_VALUE, 
												   UN_FACEPLATE_BUTTON_SET_STATUS_LIMITS, UN_FACEPLATE_BUTTON_FORCED_MODE,
												   UN_FACEPLATE_BUTTON_SELECT));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnaDig_WidgetRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in AnaDig widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnaDig_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName;
	int iAction, iRes;
	dyn_string exceptionInfo;
	string sPlcName, sSystemName;
	bool bRemote;

	deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
	if (deviceName == "")	// In case of disconnection
		{
		deviceName = g_sDpName;
		}
// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	
	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;

	if(sPlcName != "") {
		if (dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;

	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			unAnaDig_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPCONNECT:
			g_sDpName = deviceName;
			unAnaDig_WidgetConnect(deviceName, sPlcName, sSystemName);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			unAnaDig_WidgetDisconnect(deviceName, sPlcName, sSystemName);
			unAnaDig_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnaDig_WidgetConnect
/**
Purpose: Connect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in AnaDig faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unAnaDig_WidgetConnect(string deviceName, string sPlcName, string sSystemName)
{
	int iRes;
	string sDeviceLink;
	
//  check if the device name is of type "AnaDig" if not like of no PLC. -> not available.
	if(dpTypeName(deviceName) != "AnaDig")
		g_bUnSystemAlarmPlc = false;
				
	if(g_bUnSystemAlarmPlc)
	{
		g_sPosStFormat=dpGetFormat(deviceName + ".ProcessInput.PosSt");
		g_sOutputFormat=dpGetFormat(deviceName + ".ProcessInput.OutOVSt");
		unGenericDpFunctions_getDeviceLink(deviceName, sDeviceLink);
		g_dsCtrl = unAnaDig_getCtrlLink(sSystemName, strsplit(sDeviceLink, UN_CONFIG_GROUP_SEPARATOR));
		g_iRegulation = dynlen(g_dsCtrl);
//DebugN(g_dsCtrl);
		
		if(g_iRegulation <= 0) {
			iRes = dpConnect("unAnaDig_WidgetAnimationCB", 
								deviceName + ".statusInformation.selectedManager:_lock._original._locked",
								deviceName + ".statusInformation.selectedManager",
								deviceName + ".ProcessInput.StsReg01",
								deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                                                deviceName + ".ProcessInput.StsReg02",
								deviceName + ".ProcessInput.StsReg02:_online.._invalid",
								deviceName + ".ProcessInput.PosSt",
								deviceName + ".ProcessInput.PosSt:_online.._invalid",
								deviceName + ".ProcessInput.OutOVSt",
								deviceName + ".ProcessInput.OutOVSt:_online.._invalid",
								deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
								deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
								sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
								sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
								deviceName + ".eventMask");
		}
		else if(g_iRegulation == 1) {
			iRes = dpConnect("unAnaDig_WidgetAnimationCB_1Ctrl", 
								deviceName + ".statusInformation.selectedManager:_lock._original._locked",
								deviceName + ".statusInformation.selectedManager",
								deviceName + ".ProcessInput.StsReg01",
								deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                                                deviceName + ".ProcessInput.StsReg02",
								deviceName + ".ProcessInput.StsReg02:_online.._invalid",
								deviceName + ".ProcessInput.PosSt",
								deviceName + ".ProcessInput.PosSt:_online.._invalid",
								deviceName + ".ProcessInput.OutOVSt",
								deviceName + ".ProcessInput.OutOVSt:_online.._invalid",
								deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
								deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
								g_dsCtrl[1] + ".ProcessInput.StsReg01", 
								sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
								sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
								deviceName + ".eventMask");
		}
		else {
			iRes = dpConnect("unAnaDig_WidgetAnimationCB_2Ctrl", 
								deviceName + ".statusInformation.selectedManager:_lock._original._locked",
								deviceName + ".statusInformation.selectedManager",
								deviceName + ".ProcessInput.StsReg01",
								deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                                                deviceName + ".ProcessInput.StsReg02",
								deviceName + ".ProcessInput.StsReg02:_online.._invalid",
								deviceName + ".ProcessInput.PosSt",
								deviceName + ".ProcessInput.PosSt:_online.._invalid",
								deviceName + ".ProcessInput.OutOVSt",
								deviceName + ".ProcessInput.OutOVSt:_online.._invalid",
								deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
								deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
								g_dsCtrl[1] + ".ProcessInput.StsReg01", 
								g_dsCtrl[2] + ".ProcessInput.StsReg01", 
								sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
								sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
								deviceName + ".eventMask");
		}
		g_bCallbackConnected = (iRes >= 0);
	}
	else
		unAnaDig_WidgetDisconnection(g_sWidgetType);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnaDig_getCtrlLink
/**
Purpose: get the list of Controller

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in AnaDig faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
dyn_string unAnaDig_getCtrlLink(string sSystemName, dyn_string dsDeviceName)
{
	dyn_string dsCtrl;
	string sDp;
	int i, len, iPos;
	
	len = dynlen(dsDeviceName);
	for(i=1;i<=len;i++) {
		iPos = strpos(dsDeviceName[i], ":");
		if(iPos <=0) 
			dsDeviceName[i] = sSystemName+dsDeviceName[i];
		sDp = unGenericDpFunctions_getWidgetDpName(dsDeviceName[i]);
		if(sSystemName == unGenericDpFunctions_getSystemName(sDp)) {
			if(dpExists(sDp)) {
				if(dpTypeName(sDp) == "Controller") {
					dynAppend(dsCtrl, sDp);
				}
			}
		}
	}
	
	return dsCtrl;
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnaDig_WidgetDisconnect
/**
Purpose: Disconnect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in AnaDig faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unAnaDig_WidgetDisconnect(string deviceName, string sPlcName, string sSystemName)
{
	int iRes;
	
	if(g_bUnSystemAlarmPlc)
	{
		if(g_iRegulation <= 0) {
			iRes = dpDisconnect("unAnaDig_WidgetAnimationCB", 
								deviceName + ".statusInformation.selectedManager:_lock._original._locked",
								deviceName + ".statusInformation.selectedManager",
								deviceName + ".ProcessInput.StsReg01",
								deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                                                deviceName + ".ProcessInput.StsReg02",
								deviceName + ".ProcessInput.StsReg02:_online.._invalid",
								deviceName + ".ProcessInput.PosSt",
								deviceName + ".ProcessInput.PosSt:_online.._invalid",
								deviceName + ".ProcessInput.OutOVSt",
								deviceName + ".ProcessInput.OutOVSt:_online.._invalid",
								deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
								deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
								sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
								sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
								deviceName + ".eventMask");
		}
		else if(g_iRegulation == 1) {
			iRes = dpDisconnect("unAnaDig_WidgetAnimationCB_1Ctrl", 
								deviceName + ".statusInformation.selectedManager:_lock._original._locked",
								deviceName + ".statusInformation.selectedManager",
								deviceName + ".ProcessInput.StsReg01",
								deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                                                deviceName + ".ProcessInput.StsReg02",
								deviceName + ".ProcessInput.StsReg02:_online.._invalid",
								deviceName + ".ProcessInput.PosSt",
								deviceName + ".ProcessInput.PosSt:_online.._invalid",
								deviceName + ".ProcessInput.OutOVSt",
								deviceName + ".ProcessInput.OutOVSt:_online.._invalid",
								deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
								deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
								g_dsCtrl[1] + ".ProcessInput.StsReg01", 
								sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
								sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
								deviceName + ".eventMask");
		}
		else {
			iRes = dpDisconnect("unAnaDig_WidgetAnimationCB_2Ctrl", 
								deviceName + ".statusInformation.selectedManager:_lock._original._locked",
								deviceName + ".statusInformation.selectedManager",
								deviceName + ".ProcessInput.StsReg01",
								deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                                                deviceName + ".ProcessInput.StsReg02",
								deviceName + ".ProcessInput.StsReg02:_online.._invalid",
								deviceName + ".ProcessInput.PosSt",
								deviceName + ".ProcessInput.PosSt:_online.._invalid",
								deviceName + ".ProcessInput.OutOVSt",
								deviceName + ".ProcessInput.OutOVSt:_online.._invalid",
								deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
								deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
								g_dsCtrl[1] + ".ProcessInput.StsReg01", 
								g_dsCtrl[2] + ".ProcessInput.StsReg01", 
								sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
								sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
								deviceName + ".eventMask");
		}
		g_bCallbackConnected = !(iRes >= 0);
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnaDig_WidgetAnimationCB
/**
Purpose: Animate widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in AnaDig faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnaDig_WidgetAnimationCB(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
						   	string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid,
						   	string sDpStsReg02, bit32 bit32StsReg02, string sDp5, bool bStsReg02Invalid,
						   	string sDpPosSt, float fPosSt, string sDp7, bool bPosStInvalid,
						   	string sDpOutput, float fOutOVSt, string sDp9, bool bOutOVStInvalid,
						   	string sDp10, bool bAuPosRStInvalid, string sDp11, bool bMPosRStInvalid,
								string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled,
								string sEvtMask, int iMask)
{
	unAnaDig_WidgetAnimationCB_2Ctrl(sDpLock, bLocked, sDpManager, sSelectedManager,
							   	sDpStsReg01, bit32StsReg01, sDp4, bStsReg01Invalid,
							   	sDpStsReg02, bit32StsReg02, sDp5, bStsReg02Invalid,
							   	sDpPosSt, fPosSt, sDp7, bPosStInvalid,
							   	sDpOutput, fOutOVSt, sDp9, bOutOVStInvalid,
							   	sDp10, bAuPosRStInvalid, sDp11, bMPosRStInvalid,
							   	"", 0, "", 0, 
									sDpAl, iAlValue, sDpEnabled, bPlcEnabled,
									sEvtMask, iMask);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnaDig_WidgetAnimationCB_1Ctrl
/**
Purpose: Animate widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in AnaDig faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnaDig_WidgetAnimationCB_1Ctrl(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
						   	string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid,
						   	string sDpStsReg02, bit32 bit32StsReg02, string sDp5, bool bStsReg02Invalid,
						   	string sDpPosSt, float fPosSt, string sDp7, bool bPosStInvalid,
						   	string sDpOutput, float fOutOVSt, string sDp9, bool bOutOVStInvalid,
						   	string sDp10, bool bAuPosRStInvalid, string sDp11, bool bMPosRStInvalid,
						   	string sDpCtrl1, bit32 b32StsRegCtrl1, 
								string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled,
								string sEvtMask, int iMask)
{
	unAnaDig_WidgetAnimationCB_2Ctrl(sDpLock, bLocked, sDpManager, sSelectedManager,
							   	sDpStsReg01, bit32StsReg01, sDp4, bStsReg01Invalid,
							   	sDpStsReg02, bit32StsReg02, sDp5, bStsReg02Invalid,
							   	sDpPosSt, fPosSt, sDp7, bPosStInvalid,
							   	sDpOutput, fOutOVSt, sDp9, bOutOVStInvalid,
							   	sDp10, bAuPosRStInvalid, sDp11, bMPosRStInvalid,
							   	sDpCtrl1, b32StsRegCtrl1, "", 0, 
									sDpAl, iAlValue, sDpEnabled, bPlcEnabled,
									sEvtMask, iMask);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnaDig_WidgetAnimationCB_2Ctrl
/**
Purpose: Animate widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in AnaDig faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnaDig_WidgetAnimationCB_2Ctrl(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
						   	string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid,
						   	string sDpStsReg02, bit32 bit32StsReg02, string sDp5, bool bStsReg02Invalid,
						   	string sDpPosSt, float fPosSt, string sDp7, bool bPosStInvalid,
						   	string sDpOutput, float fOutOVSt, string sDp9, bool bOutOVStInvalid,
						   	string sDp10, bool bAuPosRStInvalid, string sDp11, bool bMPosRStInvalid,
						   	string sDpCtrl1, bit32 b32StsRegCtrl1, string sDpCtrl2, bit32 b32StsRegCtrl2, 
								string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled,
								string sEvtMask, int iMask)
{
// 1. Generic animation
	unGenericObject_WidgetAnimationDoubleStsReg(g_sWidgetType, "AnaDig", 
		bLocked, sSelectedManager, bit32StsReg01,bit32StsReg02,
		makeDynInt(UN_STSREG01_AUMRW, UN_STSREG01_POSALW, UN_STSREG01_IOSIMUW, UN_STSREG01_IOERRORW),
                makeDynInt(UN_STSREG02_ALBW),
		makeDynString(UN_WIDGET_TEXT_WARNING_DEFAULT, UN_WIDGET_TEXT_WARNING_DEFAULT, UN_WIDGET_TEXT_WARNING_SIMU, UN_WIDGET_TEXT_WARNING_ERROR),
                makeDynString(UN_WIDGET_TEXT_WARNING_BLOCKED),
		makeDynInt(UN_STSREG01_STARTIST, UN_STSREG01_STOPIST), 
		makeDynString(UN_WIDGET_TEXT_ALARM_STARTIST, UN_WIDGET_TEXT_ALARM_STOPIST),
		makeDynInt(UN_STSREG01_AUINHFMO,UN_STSREG01_AUINHMMO,UN_STSREG01_MMOST, UN_STSREG01_LDST, UN_STSREG01_FOMOST),
		makeDynString(UN_WIDGET_TEXT_CONTROL_AUINHFMO,UN_WIDGET_TEXT_CONTROL_AUINHMMO,UN_WIDGET_TEXT_CONTROL_MANUAL, UN_WIDGET_TEXT_CONTROL_LOCAL, UN_WIDGET_TEXT_CONTROL_FORCED),
		UN_STSREG01_ONST, UN_STSREG01_OFFST, UN_STSREG01_ALUNACK,
		bStsReg01Invalid || bStsReg02Invalid || bPosStInvalid || bOutOVStInvalid || bMPosRStInvalid || bAuPosRStInvalid,
		bStsReg01Invalid || bStsReg02Invalid, iAlValue, bPlcEnabled);
// 2. Display values
	if(g_bSystemConnected)
		unGenericObject_WidgetDisplayValueAnimation("Display1", g_sPosStFormat, fPosSt, "unDisplayValue_Status", bPosStInvalid);
	if(g_bSystemConnected)	
		unGenericObject_WidgetDisplayValueAnimation("Display2", g_sOutputFormat, fOutOVSt, "unDisplayValue_Output", bOutOVStInvalid);

// display event state
	if(g_bSystemConnected) {
		if(iMask == 0)
			eventState.visible	= true;
		else
			eventState.visible	= false;
	}
	
	unAnaDig_WidgetRegulationState(b32StsRegCtrl1, b32StsRegCtrl2);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnaDig_WidgetRegulationState
/**
Purpose: Animate RR letter in widgt

Parameters:

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unAnaDig_WidgetRegulationState(int b32StsRegCtrl1, int b32StsRegCtrl2)
{
	dyn_string dsR = makeDynString("", "");
	int i, len=g_iRegulation;
	dyn_int di = makeDynInt(b32StsRegCtrl1, b32StsRegCtrl2);
	bit32 b32;
	
	if(len > 2)
		len = 2;
	for(i=1;i<=len;i++) {
		b32 = di[i];
		if(getBit(b32, UN_STSREG01_REMOST))
			dsR[i] = "R";
		else if(getBit(b32, UN_STSREG01_AUMOST))
			dsR[i] = "A";
		else if(getBit(b32, UN_STSREG01_MMOST))
			dsR[i] = "M";
		else if(getBit(b32, UN_STSREG01_FOMOST))
			dsR[i] = "F";		
		else if(getBit(b32, UN_STSREG01_TST))
			dsR[i] = "T";
		else
			dsR[i] = "";
	}
	
//DebugN(di, len, dsR);
// display R letter
	if(g_bSystemConnected && (g_iRegulation> 0)) {
		RegulationText1.visible = true;
		RegulationText1.text = dsR[1];
		RegulationText2.visible = true;
		RegulationText2.text = dsR[2];
	}
	else {
		RegulationText1.visible = false;
		RegulationText1.text = "";
		RegulationText2.visible = false;
		RegulationText2.text = "";
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnaDig_WidgetDisconnection
/**
Purpose: Animate widget disconnection

Parameters:
	- sWidgetType, string, input, widget type. ex: "heater"

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unAnaDig_WidgetDisconnection(string sWidgetType)
{
	unGenericObject_WidgetDisconnection(sWidgetType, "AnaDig");
	unGenericObject_WidgetDisplayValueDisconnect(makeDynString("Display1", "Display2"));
	
	// set event state
	eventState.visible	= false;

	// set Regulation visibility.
	RegulationText1.visible = false;
	RegulationText2.visible = false;
}
//@}
