/**@name LIBRARY: unAnalog.ctl

@author: Vincent Forest (AB-CO)

Creation Date: 21 05 2003

Modification History:
  15/11/2010: Quique
  - [UCPC-154] Inhibit manual/forced mode must be shown in faceplate and widget (letter "h")
  
        28/04/2009: Quique
                - Include animation for Warning_IOSimulated. it was cutted from the last modification (bug)
				
  27/03/2009: Frederic
	  - ..._ButtonRegisterCB(): remove fwException_raise(...) before g_dsUserAccess
        
  12/10/2008: Quique
    - New animation for the stsReg02 bit 13 : Dependent Alarm blocked warning
    - Faceplate animation: Include in the callbacks the stsReg02
    - Widget animation : "B" warning letter low priority
                  
  28/12/2007: Herve
    - device acccess control
    
	29/06/2007: Herve
		- animate widget with Ctrl if any
		
	14/12/2006: Herve
		- new param in unGenericDpFunctions_checkDeviceType
		
	08/05/2006: Herve
		- Implementation of the R
	
	02/05/2006: Herve
		- implementation of maskEvent in widget, faceplate and rigthClick
		
	11/04/2006: Herve
		- remove dynAppend(menuList, "SEPARATOR"); in unAnalog_MenuConfiguration function

	14/12/2005: Celine
		- unAnalog_WidgetRegisterCB, unAnalog_FaceplateStatusRegisterCB, unAnalog_FaceplateStatusAnimationCB, unAnalog_WidgetAnimationCB: 
		add g_bSystemConnected
		
	09/08/2005: Frederic
		- unAnalog_ButtonRegisterCB(), add evalScript to run dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS] function
 
	15/04/2005: Herve
		- unGenericObject_GetPLCNameFromDpName: return the PLCtype+"_"+plcName, in case of _UnPlc it returns _unPlc_+plcName
		
	05/11/2004: Herve
		- in the WidgetRegisterCB, faceplateRegiterCB: use the systemName of the $sIdentifier to dpConnect to the PLC

	26/10/2004: Herve
		- in the unAnalog_ButtonRegisterCB function: call the unAnalog_ButtonUserAccess to get the list of allowed buttons. This list is 
		reseted if the user logs in/out while the remote system is off.

	22/10/2004: Herve
		- modify the unGenericObject_FormatValue, unGenericObject_WidgetDisplayValueAnimation, unGenericObject_DisplayValue call

	18/10/2004: Herve
		- add to unAnalog_MenuConfiguration the function call unGenericObject_addTrendActionToMenu

	03/05/2004: Herve
		- during WidgetRegister and FaceplateRegister call unGenericObject_GetPLCNameFromDpName to get the PLC DPName
		- during WidgetRegister check if the dpType is correct

	28/04/2004: Herve
		- add function unAnalog_ObjectListGetValueTime

	14/04/2004: Herve
		- add the function unAnalog_HandleMenu
		- add the function unAnalog_UserLoginGetButtonState

	13/04/2004: Herve
		- in function unAnalog_ButtonRegisterCB add the set Manreg01 to 0 in case of dpConnect.
		- modification of the signature unAnalog_MenuConfiguration and add the creation of the menu.

	08/04/2004: Herve 
		modification of the signature of the function unAnalog_ButtonSetState

	16/01/2004: Herve
		remove the dpConnect, dpDisconnect and callback function in the case of no PLC systemAlarm. Ease the development 
		and test.
		
	01/01/2004: Herve
		- add in all dpConnect and dpDisconnect the _unSystemAlarm_PLC_DS dp if the PLC is configured for systemIntegrity. 
		The g_bUnSystemAlarmPlc is set to true if the PLC is configured for systemIntegrity
		change the dpConnect and dpDisconnect of _aut_inv to _invalid

version 1.0

External Function : 
	. unAnalog_AcknowledgeAlarm : acknowledge Analog alarms
	. unAnalog_FaceplateStatusDisconnection : animate faceplate disconnection
	. unAnalog_ButtonUserAccess : User access for Analog buttons
	. unAnalog_ButtonSetState : set button state
	. unAnalog_WidgetDisconnection : animate widget disconnection
	. unAnalog_MenuConfiguration : configuration of popup menu

Internal Functions :
	. unAnalog_FaceplateStatusRegisterCB : register callback function
	. unAnalog_FaceplateStatusAnimationCB_NoRangeNoAlarm : animate faceplate
	. unAnalog_FaceplateStatusAnimationCB_NoAlarm : animate faceplate
	. unAnalog_FaceplateStatusAnimationCB_NoRange : animate faceplate
	. unAnalog_FaceplateStatusAnimationCB : animate faceplate
	. unAnalog_ButtonRegisterCB : register callback function
	. unAnalog_ButtonAnimationCB : animate buttons
	. unAnalog_ButtonDisconnection : animate buttons disconnection
	. unAnalog_WidgetRegisterCB : register callback function
	. unAnalog_WidgetAnimationCB : animate widget
		
Purpose: This library contains Analog object functions.

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Analog object (DPType etc.)
	. Global variables and $parameters defined in Analog faceplate and widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

//@{

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalog_ObjectListGetValueTime
/**
Purpose: Function called from the objectList to return the time and value

Parameters:
	- sDeviceName, string, input, device name
	- sDeviceType, string, input, unicos object
	- dsReturnData, dyn_string, output, return value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalog_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
	float value;
	string sTime;
	
	dpGet(sDeviceName+".ProcessInput.PosSt", value, sDeviceName+".ProcessInput.PosSt:_online.._stime", sTime);
	dsReturnData[1] = sTime;
	dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(sDeviceName+".ProcessInput.PosSt"), value) + " " 
							+ dpGetUnit(sDeviceName+".ProcessInput.PosSt");
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalog_AcknowledgeAlarm
/**
Purpose: acknowledge Analog alarms

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsNeedAck, dyn_string, output, DPEs names that need acknowledgement. 
									 The PLC acknowledgement is specified using UN_ACKNOWLEDGE_PLC string
	
Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Analog faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalog_AcknowledgeAlarm(string sDpName, string sDpType, dyn_string &dsNeedAck)
{
	dsNeedAck = makeDynString("ProcessInput.StartISt", "ProcessInput.StopISt", UN_ACKNOWLEDGE_PLC);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalog_MenuConfiguration
/**
Purpose: configuration of popup menu

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsAccessOk, dyn_string, input, authorized action
	- menuList, dyn_string, output, menu to pop-up
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalog_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{
	dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, UN_POPUPMENU_ACK_TEXT, UN_POPUPMENU_FACEPLATE_TEXT, UN_POPUPMENU_TREND_TEXT);

	unGenericObject_addSelectToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	unGenericObject_addUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	unGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	unGenericObject_addTrendActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalog_HandleMenu
/**
Purpose: handle the answer of the popup menu

Parameters:
	- deviceName, string, input, the device dp name
	- sDpType, string, input, the device dp type
	- menuList, dyn_string, input, list of requested action
	- menuAnswer, int, input, selected action from the menuList
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unAnalog_HandleMenu(string deviceName, string sDpType, dyn_string menuList, int menuAnswer)
{
	dyn_string exceptionInfo;

	switch(menuAnswer) {
		case UN_POPUPMENU_UNMASKEVENT:				// unmask event
			unGenericObject_setMaskEvent(deviceName, 1, 
																							makeDynString(".ProcessInput.evStsReg01", 
																								".ProcessInput.StartISt", ".ProcessInput.StopISt"), exceptionInfo);
			break;
		case UN_POPUPMENU_MASKEVENT:				// mask event
			unGenericObject_setMaskEvent(deviceName, 0, 
																							makeDynString(".ProcessInput.evStsReg01", 
																								".ProcessInput.StartISt", ".ProcessInput.StopISt"), exceptionInfo);
			break;
		default:
			unGenericObject_HandleUnicosMenu(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);
			break;
	}

	if (dynlen(exceptionInfo) > 0)
	{
		unSendMessage_toExpertException("Right Click",exceptionInfo);
	}

}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalog_FaceplateStatusRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Analog faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalog_FaceplateStatusRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName,sSystemName;
	int iAction;
	dyn_string exceptionInfo;
	bool bRemote;
	
	deviceName = unGenericDpFunctions_getDpName($sDpName);
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;

	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			unAnalog_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
			unAnalog_FaceplateConnect(deviceName);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			unAnalog_FaceplateDisconnect(deviceName);
			unAnalog_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalog_FaceplateConnect
/**
Purpose: connect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Analog faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unAnalog_FaceplateConnect(string deviceName)
{
	int iAction, iRes, iAlertType1, iAlertType2, iRangeType;
	dyn_string exceptionInfo;
	string sPlcName, sSystemName;

// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	if(sPlcName != "") {
		if (dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;

	iRes = dpGet(deviceName + ".ProcessInput.StartISt:_alert_hdl.._type", iAlertType1,
				 deviceName + ".ProcessInput.StopISt:_alert_hdl.._type", iAlertType2);
	g_bBoolAlert = ((iRes >= 0) && (iAlertType1 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType2 == DPCONFIG_ALERT_BINARYSIGNAL));
	iRes = dpGet(deviceName + ".ProcessInput.PosSt:_pv_range.._type", iRangeType);
	if ((iRes >= 0) && (iRangeType == DPCONFIG_MINMAX_PVSS_RANGECHECK))
		{
		g_iRangeType = DPCONFIG_MINMAX_PVSS_RANGECHECK;
		}
	else
		{
		g_iRangeType = DPCONFIG_NONE;
		}

	g_sPosStFormat=dpGetFormat(deviceName + ".ProcessInput.PosSt");
	g_sPosStUnit=dpGetUnit(deviceName + ".ProcessInput.PosSt");
	g_sAuPosRStFormat=dpGetFormat(deviceName + ".ProcessInput.AuPosRSt");
	g_sAuPosRStUnit=dpGetUnit(deviceName + ".ProcessInput.AuPosRSt");
	g_sOutOVStFormat=dpGetFormat(deviceName + ".ProcessInput.OutOVSt");
	g_sOutOVStUnit=dpGetUnit(deviceName + ".ProcessInput.OutOVSt");
	g_sMPosRStFormat=dpGetFormat(deviceName + ".ProcessInput.MPosRSt");
	g_sMPosRStUnit=dpGetUnit(deviceName + ".ProcessInput.MPosRSt");
	g_sPLiOnFormat=dpGetFormat(deviceName + ".ProcessOutput.PLiOn");
	g_sPLiOnUnit=dpGetUnit(deviceName + ".ProcessOutput.PLiOn");
	g_sPLiOffFormat=dpGetFormat(deviceName + ".ProcessOutput.PLiOff");
	g_sPLiOffUnit=dpGetUnit(deviceName + ".ProcessOutput.PLiOff");

	if(g_bUnSystemAlarmPlc) {
		if ((g_bBoolAlert) && (g_iRangeType == DPCONFIG_MINMAX_PVSS_RANGECHECK))
			{
			iRes = dpConnect("unAnalog_FaceplateStatusAnimationCB",
							 deviceName + ".ProcessInput.StsReg01", 
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                                         deviceName + ".ProcessInput.StsReg02", 
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
							 deviceName + ".ProcessInput.PosSt",
							 deviceName + ".ProcessInput.PosSt:_online.._invalid",
							 deviceName + ".ProcessInput.AuPosRSt",
							 deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
							 deviceName + ".ProcessInput.OutOVSt",
							 deviceName + ".ProcessInput.OutOVSt:_online.._invalid",
							 deviceName + ".ProcessInput.MPosRSt",
							 deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
							 deviceName + ".ProcessOutput.PLiOn",
							 deviceName + ".ProcessOutput.PLiOn:_online.._invalid",
							 deviceName + ".ProcessOutput.PLiOff",
							 deviceName + ".ProcessOutput.PLiOff:_online.._invalid",
							 deviceName + ".ProcessInput.PosSt:_pv_range.._min",
							 deviceName + ".ProcessInput.PosSt:_pv_range.._max",
							 deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state",
							 deviceName + ".ProcessInput.StopISt:_alert_hdl.._act_state",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							 deviceName + ".eventMask");
			}
		if ((!g_bBoolAlert) && (g_iRangeType == DPCONFIG_MINMAX_PVSS_RANGECHECK))
			{
			iRes = dpConnect("unAnalog_FaceplateStatusAnimationCB_NoAlarm",
							 deviceName + ".ProcessInput.StsReg01", 
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                                         deviceName + ".ProcessInput.StsReg02", 
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
							 deviceName + ".ProcessInput.PosSt",
							 deviceName + ".ProcessInput.PosSt:_online.._invalid",
							 deviceName + ".ProcessInput.AuPosRSt",
							 deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
							 deviceName + ".ProcessInput.OutOVSt",
							 deviceName + ".ProcessInput.OutOVSt:_online.._invalid",
							 deviceName + ".ProcessInput.MPosRSt",
							 deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
							 deviceName + ".ProcessOutput.PLiOn",
							 deviceName + ".ProcessOutput.PLiOn:_online.._invalid",
							 deviceName + ".ProcessOutput.PLiOff",
							 deviceName + ".ProcessOutput.PLiOff:_online.._invalid",
							 deviceName + ".ProcessInput.PosSt:_pv_range.._min",
							 deviceName + ".ProcessInput.PosSt:_pv_range.._max",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							 deviceName + ".eventMask");
			}	
		if ((g_bBoolAlert) && (g_iRangeType != DPCONFIG_MINMAX_PVSS_RANGECHECK))
			{
			iRes = dpConnect("unAnalog_FaceplateStatusAnimationCB_NoRange",
							 deviceName + ".ProcessInput.StsReg01", 
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                                         deviceName + ".ProcessInput.StsReg02", 
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
							 deviceName + ".ProcessInput.PosSt",
							 deviceName + ".ProcessInput.PosSt:_online.._invalid",
							 deviceName + ".ProcessInput.AuPosRSt",
							 deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
							 deviceName + ".ProcessInput.OutOVSt",
							 deviceName + ".ProcessInput.OutOVSt:_online.._invalid",
							 deviceName + ".ProcessInput.MPosRSt",
							 deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
							 deviceName + ".ProcessOutput.PLiOn",
							 deviceName + ".ProcessOutput.PLiOn:_online.._invalid",
							 deviceName + ".ProcessOutput.PLiOff",
							 deviceName + ".ProcessOutput.PLiOff:_online.._invalid",
							 deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state",
							 deviceName + ".ProcessInput.StopISt:_alert_hdl.._act_state",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							 deviceName + ".eventMask");				
			}
		if ((!g_bBoolAlert) && (g_iRangeType != DPCONFIG_MINMAX_PVSS_RANGECHECK))
			{
			iRes = dpConnect("unAnalog_FaceplateStatusAnimationCB_NoRangeNoAlarm",
							 deviceName + ".ProcessInput.StsReg01", 
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                                         deviceName + ".ProcessInput.StsReg02", 
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
							 deviceName + ".ProcessInput.PosSt",
							 deviceName + ".ProcessInput.PosSt:_online.._invalid",
							 deviceName + ".ProcessInput.AuPosRSt",
							 deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
							 deviceName + ".ProcessInput.OutOVSt",
							 deviceName + ".ProcessInput.OutOVSt:_online.._invalid",
							 deviceName + ".ProcessInput.MPosRSt",
							 deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
							 deviceName + ".ProcessOutput.PLiOn",
							 deviceName + ".ProcessOutput.PLiOn:_online.._invalid",
							 deviceName + ".ProcessOutput.PLiOff",
							 deviceName + ".ProcessOutput.PLiOff:_online.._invalid",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							 deviceName + ".eventMask");				
			}
	}
	else
		unAnalog_FaceplateStatusDisconnection();
	g_bCallbackConnected = (iRes >= 0);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalog_FaceplateDisconnect
/**
Purpose: disconnect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Analog faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unAnalog_FaceplateDisconnect(string deviceName)
{
	int iRes;
	string sPlcName, sSystemName;

	if(g_bUnSystemAlarmPlc) {
// get the systemName
		sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
		sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);

		if ((g_bBoolAlert) && (g_iRangeType == DPCONFIG_MINMAX_PVSS_RANGECHECK))
			{
			iRes = dpDisconnect("unAnalog_FaceplateStatusAnimationCB",
							 deviceName + ".ProcessInput.StsReg01", 
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                                         deviceName + ".ProcessInput.StsReg02", 
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
							 deviceName + ".ProcessInput.PosSt",
							 deviceName + ".ProcessInput.PosSt:_online.._invalid",
							 deviceName + ".ProcessInput.AuPosRSt",
							 deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
							 deviceName + ".ProcessInput.OutOVSt",
							 deviceName + ".ProcessInput.OutOVSt:_online.._invalid",
							 deviceName + ".ProcessInput.MPosRSt",
							 deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
							 deviceName + ".ProcessOutput.PLiOn",
							 deviceName + ".ProcessOutput.PLiOn:_online.._invalid",
							 deviceName + ".ProcessOutput.PLiOff",
							 deviceName + ".ProcessOutput.PLiOff:_online.._invalid",
							 deviceName + ".ProcessInput.PosSt:_pv_range.._min",
							 deviceName + ".ProcessInput.PosSt:_pv_range.._max",
							 deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state",
							 deviceName + ".ProcessInput.StopISt:_alert_hdl.._act_state",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							 deviceName + ".eventMask");
			}
		if ((!g_bBoolAlert) && (g_iRangeType == DPCONFIG_MINMAX_PVSS_RANGECHECK))
			{
			iRes = dpDisconnect("unAnalog_FaceplateStatusAnimationCB_NoAlarm",
							 deviceName + ".ProcessInput.StsReg01", 
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                                         deviceName + ".ProcessInput.StsReg02", 
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
							 deviceName + ".ProcessInput.PosSt",
							 deviceName + ".ProcessInput.PosSt:_online.._invalid",
							 deviceName + ".ProcessInput.AuPosRSt",
							 deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
							 deviceName + ".ProcessInput.OutOVSt",
							 deviceName + ".ProcessInput.OutOVSt:_online.._invalid",
							 deviceName + ".ProcessInput.MPosRSt",
							 deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
							 deviceName + ".ProcessOutput.PLiOn",
							 deviceName + ".ProcessOutput.PLiOn:_online.._invalid",
							 deviceName + ".ProcessOutput.PLiOff",
							 deviceName + ".ProcessOutput.PLiOff:_online.._invalid",
							 deviceName + ".ProcessInput.PosSt:_pv_range.._min",
							 deviceName + ".ProcessInput.PosSt:_pv_range.._max",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							 deviceName + ".eventMask");
			}	
		if ((g_bBoolAlert) && (g_iRangeType != DPCONFIG_MINMAX_PVSS_RANGECHECK))
			{
			iRes = dpDisconnect("unAnalog_FaceplateStatusAnimationCB_NoRange",
							 deviceName + ".ProcessInput.StsReg01", 
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                                         deviceName + ".ProcessInput.StsReg02", 
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
							 deviceName + ".ProcessInput.PosSt",
							 deviceName + ".ProcessInput.PosSt:_online.._invalid",
							 deviceName + ".ProcessInput.AuPosRSt",
							 deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
							 deviceName + ".ProcessInput.OutOVSt",
							 deviceName + ".ProcessInput.OutOVSt:_online.._invalid",
							 deviceName + ".ProcessInput.MPosRSt",
							 deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
							 deviceName + ".ProcessOutput.PLiOn",
							 deviceName + ".ProcessOutput.PLiOn:_online.._invalid",
							 deviceName + ".ProcessOutput.PLiOff",
							 deviceName + ".ProcessOutput.PLiOff:_online.._invalid",
							 deviceName + ".ProcessInput.StartISt:_alert_hdl.._act_state",
							 deviceName + ".ProcessInput.StopISt:_alert_hdl.._act_state",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							 deviceName + ".eventMask");				
			}
		if ((!g_bBoolAlert) && (g_iRangeType != DPCONFIG_MINMAX_PVSS_RANGECHECK))
			{
			iRes = dpDisconnect("unAnalog_FaceplateStatusAnimationCB_NoRangeNoAlarm",
							 deviceName + ".ProcessInput.StsReg01", 
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                                         deviceName + ".ProcessInput.StsReg02", 
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
							 deviceName + ".ProcessInput.PosSt",
							 deviceName + ".ProcessInput.PosSt:_online.._invalid",
							 deviceName + ".ProcessInput.AuPosRSt",
							 deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
							 deviceName + ".ProcessInput.OutOVSt",
							 deviceName + ".ProcessInput.OutOVSt:_online.._invalid",
							 deviceName + ".ProcessInput.MPosRSt",
							 deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
							 deviceName + ".ProcessOutput.PLiOn",
							 deviceName + ".ProcessOutput.PLiOn:_online.._invalid",
							 deviceName + ".ProcessOutput.PLiOff",
							 deviceName + ".ProcessOutput.PLiOff:_online.._invalid",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							 deviceName + ".eventMask");				
			}
	}
	g_bCallbackConnected = !(iRes >= 0);
}
//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalog_FaceplateStatusAnimationCB_NoRangeNoAlarm
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalog_FaceplateStatusAnimationCB_NoRangeNoAlarm(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
                                                   string sStsReg02, bit32 bit32StsReg02, string sDp3, bool bStsReg02Invalid,
						   string sDpPosSt, float fPosSt, string sDp4, bool bPosStInvalid, 
				 		   string sDpAuto, float fAuPosRSt, string sDp6, bool bAuPosRStInvalid,
				 		   string sDpOutput, float fOutOVSt, string sDp8, bool bOutOVStInvalid,
				 		   string sDpMPosRSt, float fMPosRSt, string sDp10, bool bMPosRStInvalid,
				 		   string sDpLimitOn, float fPLiOn, string sDp12, bool bPLiOnInvalid,
				 		   string sDpLimitOff, float fPLiOff, string sDp14, bool bPLiOffInvalid,
				 		   string sDpAl, int iValAlarm, string sDpEnabled, bool bPlcEnabled,
				 		   string sDpEvt, int iMask)
{
	unAnalog_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid,
                                            sStsReg02, bit32StsReg02, sDp3, bStsReg02Invalid,  
									    sDpPosSt, fPosSt, sDp4, bPosStInvalid,
					 			        sDpAuto, fAuPosRSt, sDp6, bAuPosRStInvalid,
					 			        sDpOutput, fOutOVSt, sDp8, bOutOVStInvalid,
					 			        sDpMPosRSt, fMPosRSt, sDp10, bMPosRStInvalid,
					 			        sDpLimitOn, fPLiOn, sDp12, bPLiOnInvalid,
					 			        sDpLimitOff, fPLiOff, sDp14, bPLiOffInvalid,
					 			        "", 0, "", 0, "", 0, "", 0,
					 			        sDpAl, iValAlarm, sDpEnabled, bPlcEnabled, sDpEvt, iMask);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalog_FaceplateStatusAnimationCB_NoRange
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalog_FaceplateStatusAnimationCB_NoRange(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
                                            string sStsReg02, bit32 bit32StsReg02, string sDp3, bool bStsReg02Invalid,
					    string sDpPosSt, float fPosSt, string sDp4, bool bPosStInvalid, 
				 	    string sDpAuto, float fAuPosRSt, string sDp6, bool bAuPosRStInvalid,
				 	    string sDpOutput, float fOutOVSt, string sDp8, bool bOutOVStInvalid,
				 	    string sDpMPosRSt, float fMPosRSt, string sDp10, bool bMPosRStInvalid,
				 	    string sDpLimitOn, float fPLiOn, string sDp12, bool bPLiOnInvalid,
				 	    string sDpLimitOff, float fPLiOff, string sDp14, bool bPLiOffInvalid,
					    string sDp17, int iStartIStState, string sDp18, int iStopIStState,
				 	    string sDpAl, int iValAlarm, string sDpEnabled, bool bPlcEnabled,
				 	    string sDpEvt, int iMask)
{
	unAnalog_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid, 
                                            sStsReg02, bit32StsReg02, sDp3, bStsReg02Invalid, 
									    sDpPosSt, fPosSt, sDp4, bPosStInvalid,
					 			        sDpAuto, fAuPosRSt, sDp6, bAuPosRStInvalid,
					 			        sDpOutput, fOutOVSt, sDp8, bOutOVStInvalid,
					 			        sDpMPosRSt, fMPosRSt, sDp10, bMPosRStInvalid,
					 			        sDpLimitOn, fPLiOn, sDp12, bPLiOnInvalid,
					 			        sDpLimitOff, fPLiOff, sDp14, bPLiOffInvalid,
					 			        "", 0, "", 0, sDp17, iStartIStState, sDp18, iStopIStState,
					 			        sDpAl, iValAlarm, sDpEnabled, bPlcEnabled, sDpEvt, iMask);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalog_FaceplateStatusAnimationCB_NoAlarm
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalog_FaceplateStatusAnimationCB_NoAlarm(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
                                            string sStsReg02, bit32 bit32StsReg02, string sDp3, bool bStsReg02Invalid,
									string sDpPosSt, float fPosSt, string sDp4, bool bPosStInvalid, 
				 			        string sDpAuto, float fAuPosRSt, string sDp6, bool bAuPosRStInvalid,
				 			        string sDpOutput, float fOutOVSt, string sDp8, bool bOutOVStInvalid,
				 			        string sDpMPosRSt, float fMPosRSt, string sDp10, bool bMPosRStInvalid,
				 			        string sDpLimitOn, float fPLiOn, string sDp12, bool bPLiOnInvalid,
				 			        string sDpLimitOff, float fPLiOff, string sDp14, bool bPLiOffInvalid,
									string sDp15, float minPosSt, string sDp16, float maxPosSt,
				 			        string sDpAl, int iValAlarm, string sDpEnabled, bool bPlcEnabled,
				 			        string sDpEvt, int iMask)
{
	unAnalog_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid,
                                            sStsReg02, bit32StsReg02, sDp3, bStsReg02Invalid, 
									    sDpPosSt, fPosSt, sDp4, bPosStInvalid,
					 			        sDpAuto, fAuPosRSt, sDp6, bAuPosRStInvalid,
					 			        sDpOutput, fOutOVSt, sDp8, bOutOVStInvalid,
					 			        sDpMPosRSt, fMPosRSt, sDp10, bMPosRStInvalid,
					 			        sDpLimitOn, fPLiOn, sDp12, bPLiOnInvalid,
					 			        sDpLimitOff, fPLiOff, sDp14, bPLiOffInvalid,
					 			        sDp15, minPosSt, sDp16, maxPosSt, "", 0, "", 0,
					 			        sDpAl, iValAlarm, sDpEnabled, bPlcEnabled, sDpEvt, iMask);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalog_FaceplateStatusAnimationCB
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Analog faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalog_FaceplateStatusAnimationCB(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
                                    string sStsReg02, bit32 bit32StsReg02, string sDp3, bool bStsReg02Invalid,
				    string sDpPosSt, float fPosSt, string sDp4, bool bPosStInvalid, 
				    string sDpAuto, float fAuPosRSt, string sDp6, bool bAuPosRStInvalid,
				    string sDpOutput, float fOutOVSt, string sDp8, bool bOutOVStInvalid,
				    string sDpMPosRSt, float fMPosRSt, string sDp10, bool bMPosRStInvalid,
				    string sDpLimitOn, float fPLiOn, string sDp12, bool bPLiOnInvalid,
				    string sDpLimitOff, float fPLiOff, string sDp14, bool bPLiOffInvalid,
				    string sDp15, float minPosSt, string sDp16, float maxPosSt,
				    string sDp17, int iStartIStState, string sDp18, int iStopIStState,
				    string sDpAl, int iValAlarm, string sDpEnabled, bool bSystemIntegrityAlarmEnabled,
				    string sDpEvt, int iMask)
{
// 1. Animate display values
if(g_bSystemConnected)
	unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, fPosSt, "StatusPosition", "unDisplayValue_Status", 
				bPosStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_DisplayValue(g_sAuPosRStFormat, g_sAuPosRStUnit, fAuPosRSt, "RequestAuto", "unDisplayValue_Request", 
				bAuPosRStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_DisplayValue(g_sOutOVStFormat, g_sOutOVStUnit, fOutOVSt, "RequestOutput", "unDisplayValue_Output", 
				bOutOVStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_DisplayValue(g_sMPosRStFormat, g_sMPosRStUnit, fMPosRSt, "RequestManual", "unDisplayValue_Request", 
				bMPosRStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_DisplayValue(g_sPLiOnFormat, g_sPLiOnUnit, fPLiOn, "StatusLimitOn", "unDisplayValue_Parameter", 
				bPLiOnInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_DisplayValue(g_sPLiOffFormat, g_sPLiOffUnit, fPLiOff, "StatusLimitOff", "unDisplayValue_Parameter", 
				bPLiOffInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_DisplayRange (g_iRangeType, g_sPosStFormat, g_sPosStUnit, minPosSt, maxPosSt, "RangeMin", "RangeMax", "unDisplayValue_Parameter");
// 2. Animate colorboxes
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OperationMode_Auto", getBit(bit32StsReg01, UN_STSREG01_AUMOST), 1, "unFaceplate_OperationModeActive", 
				bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OperationMode_Manual", getBit(bit32StsReg01, UN_STSREG01_MMOST), 1, "unFaceplate_OperationModeActive", 
				bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OperationMode_Forced", getBit(bit32StsReg01, UN_STSREG01_FOMOST), 1, "unFaceplate_OperationModeActive", 
				bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OperationMode_Local", getBit(bit32StsReg01, UN_STSREG01_LDST), 1, "unFaceplate_OperationModeActive", 
				bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Warning_IOError", getBit(bit32StsReg01, UN_STSREG01_IOERRORW), 1, "unFaceplate_WarningActive", 
				bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
// The bStsReg01Invalid is taken to show invalidity to be compatible with version which do not have stsReg02 in the analog object
if(g_bSystemConnected)
         unGenericObject_ColorBoxAnimate("Warning_IOSimulated", getBit(bit32StsReg01,UN_STSREG01_IOSIMUW), 1, "unFaceplate_WarningActive",
                                bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Warning_AlBW", getBit(bit32StsReg02, UN_STSREG02_ALBW), 1, "unFaceplate_WarningActive", 
				bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Warning_ManualAuto", getBit(bit32StsReg01, UN_STSREG01_AUMRW), 1, "unFaceplate_WarningActive", 
				bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Warning_PositionWarning", getBit(bit32StsReg01, UN_STSREG01_POSALW), 1, "unFaceplate_WarningActive", 
				bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Status_Failsafe", getBit(bit32StsReg01, UN_STSREG01_PFSPOSONST), getBit(bit32StsReg01, UN_STSREG01_PFSPOSONST), 
				"unFaceplate_StatusActive", bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
// 3. Animate alarm colorboxes
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimateAlarm("Alarm_StartISt", iStartIStState, g_bBoolAlert, 
				bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimateAlarm("Alarm_StopISt", iStopIStState, g_bBoolAlert, 
				bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Alarm_UnAck", getBit(bit32StsReg01, UN_STSREG01_ALUNACK), 1, "unFaceplate_AlarmNotAck", 
				bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));

// display event state
	if(g_bSystemConnected) {
		unGenericObject_ColorBoxAnimate("eventState", iMask, 1, "", 
				bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value),
				"unFaceplate_OperationModeActive"); 
	}
  
 // [UCPC-154] Auto Inhibit Manual/Forced mode  UN_STSREG01_AUINHFMO,UN_STSREG01_AUINHMMO
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Status_AuInhFor", getBit(bit32StsReg01, UN_STSREG01_AUINHFMO), 1, "unFaceplate_OperationModeActive", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Status_AuInhMan", getBit(bit32StsReg01, UN_STSREG01_AUINHMMO), 1, "unFaceplate_OperationModeActive", 
			bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
 
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalog_FaceplateStatusDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalog_FaceplateStatusDisconnection()
{
	unGenericObject_ColorBoxDisconnect(makeDynString("Alarm_StartISt", "Alarm_StopISt", "Alarm_UnAck", "OperationMode_Auto",
													 "OperationMode_Forced", "OperationMode_Local", "OperationMode_Manual",
													 "Warning_IOError", "Warning_IOSimulated", "Warning_ManualAuto", 
													 "Warning_PositionWarning", "Status_Failsafe"));
	unGenericObject_DisplayValueDisconnect(makeDynString("RangeMax", "RangeMin", "RequestAuto", "RequestManual", "RequestOutput",
														 "StatusLimitOff", "StatusLimitOn", "StatusPosition"));
// set eventState
	unGenericObject_ColorBoxDisconnect(makeDynString("eventState"));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalog_ButtonRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Analog faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalog_ButtonRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, sDpType, sFunction;
	int iAction, iRes, iManReg01;
	dyn_string exceptionInfo, dsUserAccess, dsFunctions;
	
	deviceName = unGenericDpFunctions_getDpName($sDpName);
	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			unAnalog_ButtonDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
// Send 0 to ManReg01 if necessary
			iRes = dpGet(deviceName + ".ProcessOutput.ManReg01", iManReg01);
			if ((iRes < 0) || (iManReg01 != 0))
			{
				dpSet(deviceName + ".ProcessOutput.ManReg01", 0);
			}
			
// call the unAnalog_ButtonUserAccess to get the list of allowed buttons.
//			unAnalog_ButtonUserAccess(deviceName, "Analog", g_dsUserAccess);

			sDpType="Analog";
			unGenericObject_GetFunctions(sDpType, dsFunctions, exceptionInfo);
			sFunction = dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS];
						 
			if (sFunction != "")
				{
				evalScript(dsUserAccess, "dyn_string main(string deviceNameTemp, string sDpTypeTemp) {" + 
							 "dyn_string dsUserAccessReturn;" +
							 "if (isFunctionDefined(\"" + sFunction  + "\"))" +
							 "    {" +
					 			sFunction  + "(deviceNameTemp, sDpTypeTemp, dsUserAccessReturn);" +
							 "    }" +
							 "return dsUserAccessReturn; }", makeDynString(), deviceName, sDpType);
						
				g_dsUserAccess=dsUserAccess;	
				}
			else
				fwException_raise(exceptionInfo,"ERROR", "unAnalog_ButtonRegisterCB: " + getCatStr("unGeneration","UNKNOWNFUNCTION"), "EMPTY");
					
			
			iRes = dpConnect("unAnalog_ButtonAnimationCB",
							 deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							 deviceName + ".statusInformation.selectedManager",
							 deviceName + ".ProcessInput.StsReg01",
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                                         deviceName + ".ProcessInput.StsReg02",
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid");
			g_bCallbackConnected = (iRes >= 0);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			iRes = dpDisconnect("unAnalog_ButtonAnimationCB",
							 deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							 deviceName + ".statusInformation.selectedManager",
							 deviceName + ".ProcessInput.StsReg01",
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                                         deviceName + ".ProcessInput.StsReg02",
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid");
			g_bCallbackConnected = !(iRes >= 0);
			unAnalog_ButtonDisconnection();
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalog_ButtonAnimationCB
/**
Purpose: animate buttons

Parameters: 
	- dbPermissions, dyn_bool, input, list of permissions, length = 4, order : 1=monitor, 2=operator, 3=expert, 4=admin
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalog_ButtonAnimationCB(string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager, 
		           string sDpStsReg01, bit32 stsReg01Value, string sDpStsReg01Bad, bool bStsReg01Bad,
                           string sDpStsReg02, bit32 stsReg02Value, string sDpStsReg02Bad, bool bStsReg02Bad)
{
	unAnalog_ButtonSetState(unGenericDpFunctions_getDpName(sDpStsReg01), "Analog", g_dsUserAccess, 
						 	makeDynString(bLocked, sSelectedManager, stsReg01Value, bStsReg01Bad, stsReg02Value, bStsReg02Bad));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalog_ButtonUserAccess
/**
Purpose: User access for Analog buttons

Parameters: 
	- sDpName, string, input, device name
	- sDpType, string, input, dptype
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalog_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
	bool operator, expert, admin;
	int i, length;
	dyn_bool dbPermissions;
	dyn_string exceptionInfo;
        string sOperAction, sExpAction, sAdminAction;
        bool bActionDefined = false;

        unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo);
        if((sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME) ||
           (sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME))
          bActionDefined=true;
	operator = dbPermissions[2];
	expert = dbPermissions[3];
	admin = dbPermissions[4];
	dsAccess = makeDynString();
	if (operator)
	{
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
          else
  	    dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_MANUAL_MODE, UN_FACEPLATE_BUTTON_SET_VALUE, 
							UN_FACEPLATE_BUTTON_ONOPEN_REQUEST, UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST, UN_FACEPLATE_BUTTON_ACK_ALARM,
							UN_FACEPLATE_BUTTON_INC_VALUE, UN_FACEPLATE_BUTTON_DEC_VALUE, UN_FACEPLATE_BUTTON_SET_STATUS_LIMITS,
							UN_FACEPLATE_BUTTON_SELECT));
	}
	if (expert)
	{
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
          else
	    dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_FORCED_MODE, UN_FACEPLATE_MASKEVENT));
	}
        if(admin){
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
        }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalog_UserLoginGetButtonState
/**
Purpose: function called before the button set state when a user logs in. 

Parameters:
	deviceName: string, input, the device name
	dsReturnData: dyn_string, output, the data are returned here

Usage: Internal function

PVSS manager usage:  Ctrl, NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
		-> $sDpName, string, the device DP name
		-> g_sFaceplateButtonType, string, faceplate type
	. PVSS version: 2.12.1 
	. operating system: NT, W2000 and Linux, but tested only under W2000.
	. distributed system: yes but not tested.
*/

unAnalog_UserLoginGetButtonState(string deviceName, string sType, dyn_string &dsReturnData)
{
	unGenericObject_UserCBGetButtonState(deviceName, sType, dsReturnData);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalog_ButtonSetState
/**
Purpose: set button state

Parameters: 
	- deviceName, string, input, device
	- sDpType, string, input, dptype
	- dsUserAccess, dyn_string, input, user access
	- dsData, dyn_string, parameter
			
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalog_ButtonSetState(string deviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
	bool bSelected, buttonEnabled, bit1, bit2, bit3;
	dyn_string dsButtons;
	int i, length;
	string localManager;
	bool bLocked, bStsReg01Bad, bStsReg02Bad;
	string sSelectedManager;
	bit32 stsReg01Value, stsReg02Value; 

	bLocked = dsData[1];
	sSelectedManager = dsData[2];
	stsReg01Value = dsData[3];
	bStsReg01Bad = dsData[4];
	stsReg02Value = dsData[5];
	bStsReg02Bad = dsData[6];

//DebugN("Analog_SetState", bLocked, sSelectedManager, stsReg01Value, bStsReg01Bad, stsReg02Value, bStsReg02Bad);

	dsButtons = makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_MANUAL_MODE, UN_FACEPLATE_BUTTON_SET_VALUE, 
							  UN_FACEPLATE_BUTTON_ONOPEN_REQUEST, UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST, UN_FACEPLATE_BUTTON_ACK_ALARM,
							  UN_FACEPLATE_BUTTON_INC_VALUE, UN_FACEPLATE_BUTTON_DEC_VALUE, UN_FACEPLATE_BUTTON_SET_STATUS_LIMITS,
							  UN_FACEPLATE_BUTTON_FORCED_MODE); // Each button except "Select" button
	length = dynlen(dsButtons);
// 1. Selection state
	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
	bSelected = (localManager == "S");
// 2. Button state
	for(i=1;i<=length;i++)
		{
		buttonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0);	// User access
		switch (dsButtons[i])
			{
			case UN_FACEPLATE_BUTTON_FORCED_MODE:
				bit1 = !(getBit(stsReg01Value, UN_STSREG01_FOMOST) == 0) && !(bStsReg01Bad || bStsReg02Bad);	
				bit2 = !(getBit(stsReg01Value, UN_STSREG01_AUINHFMO) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				bit3 = !(getBit(stsReg01Value, UN_STSREG01_LDST) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				buttonEnabled = buttonEnabled && bSelected && !(bit1 || bit2 || bit3);
				break;
			case UN_FACEPLATE_BUTTON_MANUAL_MODE:
				bit1 = !(getBit(stsReg01Value, UN_STSREG01_MMOST) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				bit2 = !(getBit(stsReg01Value, UN_STSREG01_AUINHMMO) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				bit3 = !(getBit(stsReg01Value, UN_STSREG01_LDST) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				buttonEnabled = buttonEnabled && bSelected && !(bit1 || bit2 || bit3);
				break;
			case UN_FACEPLATE_BUTTON_INC_VALUE:
			case UN_FACEPLATE_BUTTON_DEC_VALUE:
			case UN_FACEPLATE_BUTTON_ONOPEN_REQUEST:
			case UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST:
			case UN_FACEPLATE_BUTTON_AUTO_MODE:
			case UN_FACEPLATE_BUTTON_SET_VALUE:
				bit1 = !(getBit(stsReg01Value, UN_STSREG01_AUMOST) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				bit2 = !(getBit(stsReg01Value, UN_STSREG01_LDST) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				buttonEnabled = buttonEnabled && bSelected && !(bit1 || bit2);
				break;
			case UN_FACEPLATE_BUTTON_SET_STATUS_LIMITS:
				buttonEnabled = buttonEnabled && bSelected;
				break;
			case UN_FACEPLATE_BUTTON_ACK_ALARM:
			default:
				break;
			}
		setValue(UN_FACEPLATE_BUTTON_PREFIX + dsButtons[i], "enabled", buttonEnabled);
		}
// 3. Button Select state
	unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalog_ButtonDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalog_ButtonDisconnection()
{
	unGenericObject_ButtonDisconnect(makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_MANUAL_MODE, 
												   UN_FACEPLATE_BUTTON_SET_VALUE, UN_FACEPLATE_BUTTON_ONOPEN_REQUEST, 
												   UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST, UN_FACEPLATE_BUTTON_ACK_ALARM,
												   UN_FACEPLATE_BUTTON_INC_VALUE, UN_FACEPLATE_BUTTON_DEC_VALUE, 
												   UN_FACEPLATE_BUTTON_SET_STATUS_LIMITS, UN_FACEPLATE_BUTTON_FORCED_MODE,
												   UN_FACEPLATE_BUTTON_SELECT));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalog_WidgetRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Analog widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalog_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName;
	int iAction;
	dyn_string exceptionInfo;
	string sPlcName, sSystemName;
	bool bRemote;
	
	deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
	if (deviceName == "")	// In case of disconnection
		{
		deviceName = g_sDpName;
		}
// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	if(sPlcName != "") {
		if (dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;
	
	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;

	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			unAnalog_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPCONNECT:
			g_sDpName = deviceName;
			unAnalog_WidgetConnect(deviceName, sPlcName, sSystemName);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			unAnalog_WidgetDisconnect(deviceName, sPlcName, sSystemName);
			unAnalog_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalog_WidgetConnect
/**
Purpose: Connect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Analog faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unAnalog_WidgetConnect(string deviceName, string sPlcName, string sSystemName)
{
	int iRes;
	string sDeviceLink;
	
//  check if the device name is of type "Analog" if not like of no PLC. -> not available.
	if(dpTypeName(deviceName) != "Analog")
		g_bUnSystemAlarmPlc = false;
				
	if(g_bUnSystemAlarmPlc)
	{
		g_sPosStFormat = dpGetFormat(deviceName + ".ProcessInput.PosSt");
		g_sOutOVStFormat = dpGetFormat(deviceName + ".ProcessInput.OutOVSt");
		unGenericDpFunctions_getDeviceLink(deviceName, sDeviceLink);
		g_dsCtrl = unAnaDig_getCtrlLink(sSystemName, strsplit(sDeviceLink, UN_CONFIG_GROUP_SEPARATOR));
		g_iRegulation = dynlen(g_dsCtrl);
//DebugN(g_dsCtrl);
		if(g_iRegulation <= 0) {
			iRes = dpConnect("unAnalog_WidgetAnimationCB", 
							deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							deviceName + ".statusInformation.selectedManager",
							deviceName + ".ProcessInput.StsReg01",
							deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                                        deviceName + ".ProcessInput.StsReg02",
							deviceName + ".ProcessInput.StsReg02:_online.._invalid",
							deviceName + ".ProcessInput.PosSt",
							deviceName + ".ProcessInput.PosSt:_online.._invalid",
							deviceName + ".ProcessInput.OutOVSt",
							deviceName + ".ProcessInput.OutOVSt:_online.._invalid",
							deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
							deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
							sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							deviceName + ".eventMask");
		}
		else if(g_iRegulation == 1) {
			iRes = dpConnect("unAnalog_WidgetAnimationCB_1Ctrl", 
							deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							deviceName + ".statusInformation.selectedManager",
							deviceName + ".ProcessInput.StsReg01",
							deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                                        deviceName + ".ProcessInput.StsReg02",
							deviceName + ".ProcessInput.StsReg02:_online.._invalid",
							deviceName + ".ProcessInput.PosSt",
							deviceName + ".ProcessInput.PosSt:_online.._invalid",
							deviceName + ".ProcessInput.OutOVSt",
							deviceName + ".ProcessInput.OutOVSt:_online.._invalid",
							deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
							deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
							g_dsCtrl[1] + ".ProcessInput.StsReg01", 
							sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							deviceName + ".eventMask");
		}
		else {
			iRes = dpConnect("unAnalog_WidgetAnimationCB_2Ctrl", 
							deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							deviceName + ".statusInformation.selectedManager",
							deviceName + ".ProcessInput.StsReg01",
							deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                                        deviceName + ".ProcessInput.StsReg02",
							deviceName + ".ProcessInput.StsReg02:_online.._invalid",
							deviceName + ".ProcessInput.PosSt",
							deviceName + ".ProcessInput.PosSt:_online.._invalid",
							deviceName + ".ProcessInput.OutOVSt",
							deviceName + ".ProcessInput.OutOVSt:_online.._invalid",
							deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
							deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
							g_dsCtrl[1] + ".ProcessInput.StsReg01", 
							g_dsCtrl[2] + ".ProcessInput.StsReg01", 
							sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							deviceName + ".eventMask");
		}
		g_bCallbackConnected = (iRes >= 0);
	}
	else
		unAnalog_WidgetDisconnection(g_sWidgetType);
}
//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalog_WidgetDisconnect
/**
Purpose: Disconnect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Analog faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unAnalog_WidgetDisconnect(string deviceName, string sPlcName, string sSystemName)
{
	int iRes;
	
	if(g_bUnSystemAlarmPlc)
	{
		if(g_iRegulation <= 0) {
			iRes = dpDisconnect("unAnalog_WidgetAnimationCB", 
							deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							deviceName + ".statusInformation.selectedManager",
							deviceName + ".ProcessInput.StsReg01",
							deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                                        deviceName + ".ProcessInput.StsReg02",
							deviceName + ".ProcessInput.StsReg02:_online.._invalid",
							deviceName + ".ProcessInput.PosSt",
							deviceName + ".ProcessInput.PosSt:_online.._invalid",
							deviceName + ".ProcessInput.OutOVSt",
							deviceName + ".ProcessInput.OutOVSt:_online.._invalid",
							deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
							deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
							sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							deviceName + ".eventMask");
		}
		else if(g_iRegulation == 1) {
			iRes = dpDisconnect("unAnalog_WidgetAnimationCB_1Ctrl", 
							deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							deviceName + ".statusInformation.selectedManager",
							deviceName + ".ProcessInput.StsReg01",
							deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                                        deviceName + ".ProcessInput.StsReg02",
							deviceName + ".ProcessInput.StsReg02:_online.._invalid",
							deviceName + ".ProcessInput.PosSt",
							deviceName + ".ProcessInput.PosSt:_online.._invalid",
							deviceName + ".ProcessInput.OutOVSt",
							deviceName + ".ProcessInput.OutOVSt:_online.._invalid",
							deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
							deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
							g_dsCtrl[1] + ".ProcessInput.StsReg01", 
							sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							deviceName + ".eventMask");
		}
		else {
			iRes = dpDisconnect("unAnalog_WidgetAnimationCB_2Ctrl", 
							deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							deviceName + ".statusInformation.selectedManager",
							deviceName + ".ProcessInput.StsReg01",
							deviceName + ".ProcessInput.StsReg01:_online.._invalid",
                                                        deviceName + ".ProcessInput.StsReg02",
							deviceName + ".ProcessInput.StsReg02:_online.._invalid",
							deviceName + ".ProcessInput.PosSt",
							deviceName + ".ProcessInput.PosSt:_online.._invalid",
							deviceName + ".ProcessInput.OutOVSt",
							deviceName + ".ProcessInput.OutOVSt:_online.._invalid",
							deviceName + ".ProcessInput.AuPosRSt:_online.._invalid",
							deviceName + ".ProcessInput.MPosRSt:_online.._invalid",
							g_dsCtrl[1] + ".ProcessInput.StsReg01", 
							g_dsCtrl[2] + ".ProcessInput.StsReg01", 
							sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							deviceName + ".eventMask");
		}
		g_bCallbackConnected = !(iRes >= 0);
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalog_WidgetAnimationCB
/**
Purpose: Animate widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Analog faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unAnalog_WidgetAnimationCB(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
						   	string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid,
                                                        string sDpStsReg02, bit32 bit32StsReg02, string sDp5, bool bStsReg02Invalid,
						   	string sDpPosSt, float fPosSt, string sDp6, bool bPosStInvalid,
						   	string sDpOutput, float fOutOVSt, string sDp7, bool bOutOVStInvalid,
						   	string sDp8, bool bAuPosRStInvalid, string sDp9, bool bMPosRStInvalid,
								string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled,
								string sEvtMask, int iMask)
{
	unAnalog_WidgetAnimationCB_2Ctrl(sDpLock, bLocked, sDpManager, sSelectedManager,
					 sDpStsReg01, bit32StsReg01, sDp4, bStsReg01Invalid,
                                         sDpStsReg02, bit32StsReg02, sDp5, bStsReg02Invalid,
					 sDpPosSt, fPosSt, sDp6, bPosStInvalid,
					 sDpOutput, fOutOVSt, sDp7, bOutOVStInvalid,
					 sDp8, bAuPosRStInvalid, sDp9, bMPosRStInvalid,
					 "", 0, "", 0, 
					 sDpAl, iAlValue, sDpEnabled, bPlcEnabled,
					 sEvtMask, iMask);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalog_WidgetAnimationCB_1Ctrl
/**
Purpose: Animate widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Analog faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unAnalog_WidgetAnimationCB_1Ctrl(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
						   	string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid,
                                                        string sDpStsReg02, bit32 bit32StsReg02, string sDp5, bool bStsReg02Invalid,
						   	string sDpPosSt, float fPosSt, string sDp6, bool bPosStInvalid,
						   	string sDpOutput, float fOutOVSt, string sDp7, bool bOutOVStInvalid,
						   	string sDp8, bool bAuPosRStInvalid, string sDp9, bool bMPosRStInvalid,
						   	string sDpCtrl1, bit32 b32StsRegCtrl1, 
								string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled,
								string sEvtMask, int iMask)
{
	unAnalog_WidgetAnimationCB_2Ctrl(sDpLock, bLocked, sDpManager, sSelectedManager,
							   	sDpStsReg01, bit32StsReg01, sDp4, bStsReg01Invalid,
                                                                sDpStsReg02, bit32StsReg02, sDp5, bStsReg02Invalid,
							   	sDpPosSt, fPosSt, sDp6, bPosStInvalid,
							   	sDpOutput, fOutOVSt, sDp7, bOutOVStInvalid,
							   	sDp8, bAuPosRStInvalid, sDp9, bMPosRStInvalid,
							   	sDpCtrl1, b32StsRegCtrl1, "", 0, 
									sDpAl, iAlValue, sDpEnabled, bPlcEnabled,
									sEvtMask, iMask);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalog_WidgetAnimationCB_2Ctrl
/**
Purpose: Animate widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Analog faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unAnalog_WidgetAnimationCB_2Ctrl(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
						   	string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid,
                                                        string sDpStsReg02, bit32 bit32StsReg02, string sDp5, bool bStsReg02Invalid,
						   	string sDpPosSt, float fPosSt, string sDp6, bool bPosStInvalid,
						   	string sDpOutput, float fOutOVSt, string sDp7, bool bOutOVStInvalid,
						   	string sDp8, bool bAuPosRStInvalid, string sDp9, bool bMPosRStInvalid,
						   	string sDpCtrl1, bit32 b32StsRegCtrl1, string sDpCtrl2, bit32 b32StsRegCtrl2, 
								string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled,
								string sEvtMask, int iMask)
{
// 1. Generic animation (Common to Analog/OnOff/AnaDig and PCO)
  	unGenericObject_WidgetAnimationDoubleStsReg(g_sWidgetType, "Analog", 
		bLocked, sSelectedManager, bit32StsReg01,bit32StsReg02,
		makeDynInt(UN_STSREG01_AUMRW, UN_STSREG01_POSALW, UN_STSREG01_IOSIMUW, UN_STSREG01_IOERRORW),
                makeDynInt(UN_STSREG02_ALBW),
		makeDynString(UN_WIDGET_TEXT_WARNING_DEFAULT, UN_WIDGET_TEXT_WARNING_DEFAULT, UN_WIDGET_TEXT_WARNING_SIMU, UN_WIDGET_TEXT_WARNING_ERROR),
                makeDynString(UN_WIDGET_TEXT_WARNING_BLOCKED),
		makeDynInt(UN_STSREG01_STARTIST, UN_STSREG01_STOPIST), 
		makeDynString(UN_WIDGET_TEXT_ALARM_STARTIST, UN_WIDGET_TEXT_ALARM_STOPIST),
		makeDynInt(UN_STSREG01_AUINHFMO,UN_STSREG01_AUINHMMO,UN_STSREG01_MMOST, UN_STSREG01_LDST, UN_STSREG01_FOMOST),
		makeDynString(UN_WIDGET_TEXT_CONTROL_AUINHFMO,UN_WIDGET_TEXT_CONTROL_AUINHMMO,UN_WIDGET_TEXT_CONTROL_MANUAL, UN_WIDGET_TEXT_CONTROL_LOCAL, UN_WIDGET_TEXT_CONTROL_FORCED),
		UN_STSREG01_ONST, UN_STSREG01_OFFST, UN_STSREG01_ALUNACK,
		bStsReg01Invalid || bPosStInvalid || bOutOVStInvalid || bMPosRStInvalid || bAuPosRStInvalid,
		bStsReg01Invalid, iAlValue, bPlcEnabled);
        /*
	unGenericObject_WidgetAnimation(g_sWidgetType, "Analog", 
		bLocked, sSelectedManager, bit32StsReg01,
		makeDynInt(UN_STSREG01_AUMRW, UN_STSREG01_POSALW, UN_STSREG01_IOSIMUW, UN_STSREG01_IOERRORW),
		makeDynString(UN_WIDGET_TEXT_WARNING_DEFAULT, UN_WIDGET_TEXT_WARNING_DEFAULT, UN_WIDGET_TEXT_WARNING_SIMU, UN_WIDGET_TEXT_WARNING_ERROR),
		makeDynInt(UN_STSREG01_STARTIST, UN_STSREG01_STOPIST), 
		makeDynString(UN_WIDGET_TEXT_ALARM_STARTIST, UN_WIDGET_TEXT_ALARM_STOPIST),
		makeDynInt(UN_STSREG01_MMOST, UN_STSREG01_LDST, UN_STSREG01_FOMOST),
		makeDynString(UN_WIDGET_TEXT_CONTROL_MANUAL, UN_WIDGET_TEXT_CONTROL_LOCAL, UN_WIDGET_TEXT_CONTROL_FORCED),
		UN_STSREG01_ONST, UN_STSREG01_OFFST, UN_STSREG01_ALUNACK,
		bStsReg01Invalid || bPosStInvalid || bOutOVStInvalid || bMPosRStInvalid || bAuPosRStInvalid,
		bStsReg01Invalid, iAlValue, bPlcEnabled);
        */
// 2. Display values
	if (g_bSystemConnected)
		unGenericObject_WidgetDisplayValueAnimation("Display1", g_sPosStFormat, fPosSt, "unDisplayValue_Status", bPosStInvalid);
	if (g_bSystemConnected)	
		unGenericObject_WidgetDisplayValueAnimation("Display2", g_sOutOVStFormat, fOutOVSt, "unDisplayValue_Output", bOutOVStInvalid);

// display event state
	if(g_bSystemConnected) {
		if(iMask == 0)
			eventState.visible	= true;
		else
			eventState.visible	= false;
	}
	
	unAnaDig_WidgetRegulationState(b32StsRegCtrl1, b32StsRegCtrl2);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalog_WidgetDisconnection
/**
Purpose: Animate widget disconnection

Parameters:
	- sWidgetType, string, input, widget type. ex: "heater"
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalog_WidgetDisconnection(string sWidgetType)
{
	unGenericObject_WidgetDisconnection(sWidgetType, "Analog");
	unGenericObject_WidgetDisplayValueDisconnect(makeDynString("Display1", "Display2"));

	// set event state
	eventState.visible	= false;

	// set Regulation visibility.
	RegulationText1.visible = false;
	RegulationText2.visible = false;
}
//@}
