/**@name LIBRARY: unAnalogInput.ctl

@author: Vincent Forest (AB-CO)

Creation Date: 21 05 2003

Modification History:
  27/03/2009: Frederic
	  - ..._WidgetRegisterCB(): fix problem with deviceType check during disconnection
	  - ..._ButtonRegisterCB(): remove fwException_raise(...) before g_dsUserAccess
 
  03/09/2008: Herve
    - allow rigth click if lock taken.
    
  28/12/2007: Herve
    - device acccess control
    
	20/07/2007: Herve
		in faceplate: 
			- case alarm masked show the alarm limits 
			- when no alarm limits show "" instead of "???"
		
	26/06/2007: Herve
		- move all bargraph function to unGenericWidget
		
	21/11/2006: Herve
		- all bargraph function: new widget bargraphSimple, identical as bargraph without the range, alarm and value
		
	02/05/2006: Herve
		- implementation of maskEvent in widget, faceplate and rigthClick
		
	11/04/2006: Herve
		- remove dynAppend(menuList, "SEPARATOR"); in unAnalogInput_MenuConfiguration function

	13/12/2005: Celine
		unAnalogInput_FaceplateStatusRegisterCB,
		unAnalogInput_FaceplateStatusAnimationCB,
		unAnalogInput_WidgetRegisterCB,
		unAnalogInput_bargraph_rangeAlertCB,
		unAnalogInput_step_PLC_CB,
		unAnalogInput_transition_PLC_CB : add g_bSystemConnected checking.
		
	09/08/2005: Frederic
		- unAnalogInput_ButtonRegisterCB(), add evalScript to run dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS] function

	22/06/2005:
		- unAnalogInput_FaceplateStatusAnimationCB, unAnalogInput_FaceplateStatusDisconnection() :add Mail/SMS colorbox in the faceplate
	15/04/2005: Herve
		- unGenericObject_GetPLCNameFromDpName: return the PLCtype+"_"+plcName, in case of _UnPlc it returns _unPlc_+plcName
		
	10/11/2004: Herve
		- bargraph callback: if fMax-fMin = 0.0 set the scale to 0.

	05/11/2004: Herve
		- in the WidgetRegisterCB, faceplateRegiterCB: use the systemName of the $sIdentifier to dpConnect to the PLC

	26/10/2004: Herve
		- in the unAnalogInput_ButtonRegisterCB function: call the unAnalogInput_ButtonUserAccess to get the list of allowed buttons. This list is 
		reseted if the user logs in/out while the remote system is off.
		
	22/10/2004: Herve
		- modify the unGenericObject_FormatValue call
		- modify the unGenericObject_DisplayValue, unGenericObject_DisplayRange, unGenericObject_WidgetAIAOAnimation call

	18/10/2004: Herve
		- add to unAnalogInput_MenuConfiguration the function call unGenericObject_addTrendActionToMenu

	11/05/2004: Frederic
		- update unAnalogInput_transition_PLC_WidgetConnect to add checking of g_iBit <= Position Max
		- update unAnalogInput_transition_PLC_WidgetDisconnect to call unGenericObject_GetPLCNameFromDpName
		- update unAnalogInput_step_PLC_WidgetDisconnect to call unGenericObject_GetPLCNameFromDpName
	
	03/05/2004: Herve
		- during WidgetRegister and FaceplateRegister call unGenericObject_GetPLCNameFromDpName to get the PLC DPName
		- during WidgetRegister check if the dpType is correct

	28/04/2004: Herve
		- add function unAnalogInput_ObjectListGetValueTime

	14/04/2004: Herve
		- add the function unAnalogInput_HandleMenu
		- add the function unAnalogInput_UserLoginGetButtonState

	13/04/2004: Herve
		- in function unAnalogInput_ButtonRegisterCB add the set Manreg01 to 0 in case of dpConnect.
		- modification of the signature unAnalogInput_MenuConfiguration and add the creation of the menu.

	08/04/2004: Herve 
		modification of the signature of the function unAnalogInput_ButtonSetState

	25/03/2004: Herve
		in the function unAnalogInput_ButtonUserAccess, the set_alarm_limit is accessible only for expert (was before also operator)
		add HH, H, L and LL, L, H case

	27/01/2004: Frederic
		- include Step_PLC and Transition_PLC Widget into the lib
	
	15/01/2004: Herve
		- remove the dpConnect, dpDisconnect and callback function in the case of no PLC systemAlarm. Ease the development 
		and test.
		- include the bargraph function into the lib

	26/11/2003: Herve
		- add in all dpConnect and dpDisconnect the _unSystemAlarm_PLC_DS dp if the PLC is configured for systemIntegrity. 
		The g_bUnSystemAlarmPlc is set to true if the PLC is configured for systemIntegrity
		change the dpConnect and dpDisconnect of h_aut_inv to _invalid
		- widget: remove the dpConnect on pv_range, not anymore needed, old version 2.3
		
	06/10/2003: Herve
		add dpConnect on the _lock._alert_hdl._locked in order to detect when the alert_hdl config modified.
		
	22/10/2003: Herve:
		add the following cases for alarm: H&L, HH&H, LL&L, H, L

version 1.0

External Function : 
	. unAnalogInput_AcknowledgeAlarm : acknowledge AnalogInput alarms
	. unAnalogInput_FaceplateStatusDisconnection : animate faceplate disconnection
	. unAnalogInput_ButtonUserAccess : User access for AnalogInput buttons
	. unAnalogInput_ButtonSetState : set button state
	. unAnalogInput_WidgetDisconnection : animate widget disconnection
	. unAnalogInput_MenuConfiguration : configuration of popup menu

Internal Functions :
	. unAnalogInput_FaceplateStatusRegisterCB : register callback function
	. unAnalogInput_FaceplateLockCallBack : set up the lock callback function for the faceplate
	. unAnalogInput_FaceplateConnect: Connect function of the faceplate data
	. unAnalogInput_FaceplateDisconnect: Disconnect function of the faceplate data
	. unAnalogInput_FaceplateStatusAnimationCB_NoRangeNoAlarm : animate faceplate
	. unAnalogInput_FaceplateStatusAnimationCB_NoAlarm : animate faceplate
	. unAnalogInput_FaceplateStatusAnimationCB_NoRange : animate faceplate
	. unAnalogInput_FaceplateStatusAnimationCB : animate faceplate
	. unAnalogInput_ButtonRegisterCB : register callback function
	. unAnalogInput_ButtonAnimationCB : animate buttons
	. unAnalogInput_ButtonDisconnection : animate buttons disconnection
	. unAnalogInput_WidgetRegisterCB : register callback function
	. unAnalogInput_WidgetLockCallBack : set up the lock callback function for the widget
	. unAnalogInput_WidgetConnect : Connect function of the widget data
	. unAnalogInput_WidgetDisconnect : Disconnect function of the widget data
	. unAnalogInput_WidgetAnimationCB_NoRangeNoAlarm : animate widget
	. unAnalogInput_WidgetAnimationCB_NoRange : animate widget
	. unAnalogInput_WidgetAnimationCB_NoAlarm : animate widget
	. unAnalogInput_WidgetAnimationCB : animate widget
	

Purpose: This library contains AnalogInput object functions.

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. AnalogInput object (DPType etc.)
	. Global variables and $parameters defined in AnalogInput faceplate and widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

//@{

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogInput_ObjectListGetValueTime
/**
Purpose: Function called from the objectList to return the time and value

Parameters:
	- sDeviceName, string, input, device name
	- sDeviceType, string, input, unicos object
	- dsReturnData, dyn_string, output, return value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogInput_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
	float value;
	string sTime;
	
	dpGet(sDeviceName+".ProcessInput.PosSt", value, sDeviceName+".ProcessInput.PosSt:_online.._stime", sTime);
	dsReturnData[1] = sTime;
	dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(sDeviceName+".ProcessInput.PosSt"), value) + " " 
							+ dpGetUnit(sDeviceName+".ProcessInput.PosSt");
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogInput_AcknowledgeAlarm
/**
Purpose: acknowledge AnalogInput alarms

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsNeedAck, dyn_string, output, DPEs names that need acknowledgement. 
									 The PLC acknowledgement is specified using UN_ACKNOWLEDGE_PLC string
	
Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in AnalogInput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogInput_AcknowledgeAlarm(string sDpName, string sDpType, dyn_string &dsNeedAck)
{
	dsNeedAck = makeDynString("ProcessInput.PosSt");
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogInput_MenuConfiguration
/**
Purpose: configuration of popup menu

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsAccessOk, dyn_string, input, authorized action
	- menuList, dyn_string, output, menu to pop-up
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogInput_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{
	dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, UN_POPUPMENU_MASK_POSST_TEXT, 
								 UN_POPUPMENU_ACK_TEXT, UN_POPUPMENU_FACEPLATE_TEXT, UN_POPUPMENU_TREND_TEXT);

	unGenericObject_addSelectToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	unGenericObject_addUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	unGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	unGenericObject_addTrendActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogInput_HandleMenu
/**
Purpose: handle the answer of the popup menu

Parameters:
	- deviceName, string, input, the device dp name
	- sDpType, string, input, the device dp type
	- menuList, dyn_string, input, list of requested action
	- menuAnswer, int, input, selected action from the menuList
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unAnalogInput_HandleMenu(string deviceName, string sDpType, dyn_string menuList, int menuAnswer)
{
	dyn_string exceptionInfo;

	switch(menuAnswer) {
		case UN_POPUPMENU_UNMASKEVENT:				// unmask event
			unGenericObject_setMaskEvent(deviceName, 1, 
																							makeDynString(".ProcessInput.evStsReg01"), exceptionInfo);
			break;
		case UN_POPUPMENU_MASKEVENT:				// mask event
			unGenericObject_setMaskEvent(deviceName, 0, 
																							makeDynString(".ProcessInput.evStsReg01"), exceptionInfo);
			break;
		default:
			unGenericObject_HandleUnicosMenu(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);
			break;
	}
	if (dynlen(exceptionInfo) > 0)
	{
		unSendMessage_toExpertException("Right Click",exceptionInfo);
	}

}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogInput_FaceplateStatusRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in AnalogInput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogInput_FaceplateStatusRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, sSystemName, sPlcName;
	int iAction, iRes, iAlertType1, iRangeType;
	dyn_string exceptionInfo;
	bool bRemote;
	
	deviceName = unGenericDpFunctions_getDpName($sDpName);
// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	
	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;
			
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	if(sPlcName != "") {
		if (dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;

	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			unAnalogInput_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
			g_bDisconnectCB = false;
			if(g_bUnSystemAlarmPlc) {
				iRes = dpConnect("unAnalogInput_FaceplateLockCallBack", deviceName + ".ProcessInput.PosSt:_lock._alert_hdl._locked");

				g_bCallbackConnected = (iRes >= 0);
			}
			else
				unAnalogInput_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			if(g_bUnSystemAlarmPlc) 
				iRes = dpDisconnect("unAnalogInput_FaceplateLockCallBack", deviceName + ".ProcessInput.PosSt:_lock._alert_hdl._locked");

			if(g_bDisconnectCB)
				unAnalogInput_FaceplateDisconnect(deviceName, iRes);

			g_bCallbackConnected = !(iRes >= 0);
			g_bDisconnectCB = false;
			unAnalogInput_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------
// unAnalogInput_FaceplateLockCallBack
/**
Purpose: set up the lock callback function for the faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in AnalogInput widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unAnalogInput_FaceplateLockCallBack(string sdpe, bool bLockState)
{
	int iRes;
	string deviceName;
//DebugN("unAnalogInput_FaceplateLockCallBack", bLockState, g_bDisconnectCB);

	deviceName = unGenericDpFunctions_getDpName(sdpe);
	if(bLockState) { // lock set so disconnect if not already disconnected
// no need to check if it was already set before because it is not possible to do 2 dpSet on the lock the Ev reject the message
		if(g_bDisconnectCB) {
			unAnalogInput_FaceplateDisconnect(deviceName, iRes);
//DebugN("unAnalogInput_FaceplateLockCallBack: disconnect");
			g_bDisconnectCB = false;
		}
		unAnalogInput_FaceplateStatusDisconnection();
	}
	else { // lock not set so connect if not already connected
// no need to check if it was already set before because it is not possible to do 2 dpSet on the lock the Ev reject the message
		g_bDisconnectCB = true;
		unAnalogInput_FaceplateConnect(deviceName);
//DebugN("unAnalogInput_FaceplateLockCallBack: connect");
	}
}


//---------------------------------------------------------------------------------------------------------------------------------------
// unAnalogInput_FaceplateConnect
/**
Purpose: Connect function of the faceplate data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in AnalogInput widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unAnalogInput_FaceplateConnect(string deviceName)
{
	int iRes, iAlertType1, iRangeType;
	dyn_string exceptionInfo;
	string sPlcName, sSystemName;
		
// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	if(sPlcName != "") {
		if (dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;
		
	unGenericObject_CheckAnalogAlarmType(deviceName + ".ProcessInput.PosSt", iAlertType1, exceptionInfo);
	g_bBoolAlert = (iAlertType1 == DPCONFIG_ALERT_NONBINARYSIGNAL);
	iRes = dpGet(deviceName + ".ProcessInput.PosSt:_pv_range.._type", iRangeType);
	
	g_sPosStFormat = dpGetFormat(deviceName + ".ProcessInput.PosSt");
	g_sPosStUnit = dpGetUnit(deviceName + ".ProcessInput.PosSt");
	g_sHFStOrAuPosRStFormat = dpGetFormat(deviceName + ".ProcessInput.HFStOrAuPosRSt");
	g_sHFStOrAuPosRStUnit = dpGetUnit(deviceName + ".ProcessInput.HFStOrAuPosRSt");
	
	if ((iRes >= 0) && (iRangeType == DPCONFIG_MINMAX_PVSS_RANGECHECK))
		{
		g_iRangeType = DPCONFIG_MINMAX_PVSS_RANGECHECK;
		}
	else
		{
		g_iRangeType = DPCONFIG_NONE;
		}
	if(g_bUnSystemAlarmPlc) {
		if ((g_bBoolAlert) && (g_iRangeType == DPCONFIG_MINMAX_PVSS_RANGECHECK))
		{
			switch(g_iAlertRange) {
				case UN_AIAO_ALARM_5_HH_H_L_LL:
					iRes = dpConnect("unAnalogInput_FaceplateStatusAnimationCB",
									 deviceName + ".ProcessInput.StsReg01", 
									 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
									 deviceName + ".ProcessInput.PosSt",
									 deviceName + ".ProcessInput.PosSt:_online.._invalid",
									 deviceName + ".ProcessInput.HFStOrAuPosRSt",
									 deviceName + ".ProcessInput.HFStOrAuPosRSt:_online.._invalid",
									 deviceName + ".ProcessInput.PosSt:_pv_range.._min",
									 deviceName + ".ProcessInput.PosSt:_pv_range.._max",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._active",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._act_range",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.5._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.4._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.2._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.1._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._ack_possible",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.5._l_limit",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.4._l_limit",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.2._u_limit",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.1._u_limit",
									 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
									 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
									 deviceName + ".eventMask");
					break;
				case UN_AIAO_ALARM_4_HH_H_L:
				case UN_AIAO_ALARM_4_H_L_LL:
					iRes = dpConnect("unAnalogInput_FaceplateStatusAnimationCB_4_ranges",
									 deviceName + ".ProcessInput.StsReg01", 
									 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
									 deviceName + ".ProcessInput.PosSt",
									 deviceName + ".ProcessInput.PosSt:_online.._invalid",
									 deviceName + ".ProcessInput.HFStOrAuPosRSt",
									 deviceName + ".ProcessInput.HFStOrAuPosRSt:_online.._invalid",
									 deviceName + ".ProcessInput.PosSt:_pv_range.._min",
									 deviceName + ".ProcessInput.PosSt:_pv_range.._max",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._active",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._act_range",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.4._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.3._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.2._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.1._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._ack_possible",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.4._l_limit",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.2._u_limit",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.1._u_limit",
									 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
									 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
									 deviceName + ".eventMask");
					break;
				case UN_AIAO_ALARM_3_H_L:
				case UN_AIAO_ALARM_3_HH_H:
				case UN_AIAO_ALARM_3_LL_L:
					iRes = dpConnect("unAnalogInput_FaceplateStatusAnimationCB_3_ranges",
									 deviceName + ".ProcessInput.StsReg01", 
									 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
									 deviceName + ".ProcessInput.PosSt",
									 deviceName + ".ProcessInput.PosSt:_online.._invalid",
									 deviceName + ".ProcessInput.HFStOrAuPosRSt",
									 deviceName + ".ProcessInput.HFStOrAuPosRSt:_online.._invalid",
									 deviceName + ".ProcessInput.PosSt:_pv_range.._min",
									 deviceName + ".ProcessInput.PosSt:_pv_range.._max",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._active",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._act_range",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.3._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.2._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.1._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._ack_possible",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.3._l_limit",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.1._u_limit",
									 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
									 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
									 deviceName + ".eventMask");
					break;
				case UN_AIAO_ALARM_2_H:
				case UN_AIAO_ALARM_2_L:
					iRes = dpConnect("unAnalogInput_FaceplateStatusAnimationCB_2_ranges",
									 deviceName + ".ProcessInput.StsReg01", 
									 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
									 deviceName + ".ProcessInput.PosSt",
									 deviceName + ".ProcessInput.PosSt:_online.._invalid",
									 deviceName + ".ProcessInput.HFStOrAuPosRSt",
									 deviceName + ".ProcessInput.HFStOrAuPosRSt:_online.._invalid",
									 deviceName + ".ProcessInput.PosSt:_pv_range.._min",
									 deviceName + ".ProcessInput.PosSt:_pv_range.._max",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._active",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._act_range",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.2._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.1._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._ack_possible",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.1._u_limit",
									 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
									 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
									 deviceName + ".eventMask");
					break;
			}
		}
		if ((!g_bBoolAlert) && (g_iRangeType == DPCONFIG_MINMAX_PVSS_RANGECHECK))
		{
			iRes = dpConnect("unAnalogInput_FaceplateStatusAnimationCB_NoAlarm",
							 deviceName + ".ProcessInput.StsReg01", 
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
							 deviceName + ".ProcessInput.PosSt",
							 deviceName + ".ProcessInput.PosSt:_online.._invalid",
							 deviceName + ".ProcessInput.HFStOrAuPosRSt",
							 deviceName + ".ProcessInput.HFStOrAuPosRSt:_online.._invalid",
							 deviceName + ".ProcessInput.PosSt:_pv_range.._min",
							 deviceName + ".ProcessInput.PosSt:_pv_range.._max",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							 deviceName + ".eventMask");
		}	
		if ((g_bBoolAlert) && (g_iRangeType != DPCONFIG_MINMAX_PVSS_RANGECHECK))
		{
			switch(g_iAlertRange) {
				case UN_AIAO_ALARM_5_HH_H_L_LL:
					iRes = dpConnect("unAnalogInput_FaceplateStatusAnimationCB_NoRange",
									 deviceName + ".ProcessInput.StsReg01", 
									 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
									 deviceName + ".ProcessInput.PosSt",
									 deviceName + ".ProcessInput.PosSt:_online.._invalid",
									 deviceName + ".ProcessInput.HFStOrAuPosRSt",
									 deviceName + ".ProcessInput.HFStOrAuPosRSt:_online.._invalid",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._active",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._act_range",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.5._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.4._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.2._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.1._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._ack_possible",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.5._l_limit",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.4._l_limit",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.2._u_limit",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.1._u_limit",
									 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
									 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
									 deviceName + ".eventMask");	
					break;
				case UN_AIAO_ALARM_4_HH_H_L:
				case UN_AIAO_ALARM_4_H_L_LL:
					iRes = dpConnect("unAnalogInput_FaceplateStatusAnimationCB_4_ranges_NoRange",
									 deviceName + ".ProcessInput.StsReg01", 
									 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
									 deviceName + ".ProcessInput.PosSt",
									 deviceName + ".ProcessInput.PosSt:_online.._invalid",
									 deviceName + ".ProcessInput.HFStOrAuPosRSt",
									 deviceName + ".ProcessInput.HFStOrAuPosRSt:_online.._invalid",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._active",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._act_range",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.4._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.3._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.2._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.1._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._ack_possible",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.4._l_limit",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.2._u_limit",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.1._u_limit",
									 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
									 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
									 deviceName + ".eventMask");
					break;
				case UN_AIAO_ALARM_3_H_L:
				case UN_AIAO_ALARM_3_HH_H:
				case UN_AIAO_ALARM_3_LL_L:
					iRes = dpConnect("unAnalogInput_FaceplateStatusAnimationCB_3_ranges_NoRange",
									 deviceName + ".ProcessInput.StsReg01", 
									 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
									 deviceName + ".ProcessInput.PosSt",
									 deviceName + ".ProcessInput.PosSt:_online.._invalid",
									 deviceName + ".ProcessInput.HFStOrAuPosRSt",
									 deviceName + ".ProcessInput.HFStOrAuPosRSt:_online.._invalid",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._active",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._act_range",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.3._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.2._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.1._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._ack_possible",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.3._l_limit",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.1._u_limit",
									 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
									 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
									 deviceName + ".eventMask");	
					break;
				case UN_AIAO_ALARM_2_H:
				case UN_AIAO_ALARM_2_L:
					iRes = dpConnect("unAnalogInput_FaceplateStatusAnimationCB_2_ranges_NoRange",
									 deviceName + ".ProcessInput.StsReg01", 
									 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
									 deviceName + ".ProcessInput.PosSt",
									 deviceName + ".ProcessInput.PosSt:_online.._invalid",
									 deviceName + ".ProcessInput.HFStOrAuPosRSt",
									 deviceName + ".ProcessInput.HFStOrAuPosRSt:_online.._invalid",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._active",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._act_range",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.2._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.1._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._ack_possible",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.1._u_limit",
									 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
									 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
									 deviceName + ".eventMask");	
					break;
			}
		}
		if ((!g_bBoolAlert) && (g_iRangeType != DPCONFIG_MINMAX_PVSS_RANGECHECK))
		{
			iRes = dpConnect("unAnalogInput_FaceplateStatusAnimationCB_NoRangeNoAlarm",
							 deviceName + ".ProcessInput.StsReg01", 
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
							 deviceName + ".ProcessInput.PosSt",
							 deviceName + ".ProcessInput.PosSt:_online.._invalid",
							 deviceName + ".ProcessInput.HFStOrAuPosRSt",
							 deviceName + ".ProcessInput.HFStOrAuPosRSt:_online.._invalid",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							 deviceName + ".eventMask");		
		}
	}
	else
		unAnalogInput_FaceplateStatusDisconnection();
}
//---------------------------------------------------------------------------------------------------------------------------------------
// unAnalogInput_FaceplateDisconnect
/**
Purpose: Disconnect function of the faceplate data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in AnalogInput widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unAnalogInput_FaceplateDisconnect(string deviceName, int &iRes)
{
	string sPlcName, sSystemName;

	if(g_bUnSystemAlarmPlc) {
// get the systemName
		sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
		sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);

		if ((g_bBoolAlert) && (g_iRangeType == DPCONFIG_MINMAX_PVSS_RANGECHECK))
		{
			switch(g_iAlertRange) {
				case UN_AIAO_ALARM_5_HH_H_L_LL:
					iRes = dpDisconnect("unAnalogInput_FaceplateStatusAnimationCB",
									 deviceName + ".ProcessInput.StsReg01", 
									 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
									 deviceName + ".ProcessInput.PosSt",
									 deviceName + ".ProcessInput.PosSt:_online.._invalid",
									 deviceName + ".ProcessInput.HFStOrAuPosRSt",
									 deviceName + ".ProcessInput.HFStOrAuPosRSt:_online.._invalid",
									 deviceName + ".ProcessInput.PosSt:_pv_range.._min",
									 deviceName + ".ProcessInput.PosSt:_pv_range.._max",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._active",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._act_range",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.5._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.4._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.2._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.1._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._ack_possible",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.5._l_limit",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.4._l_limit",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.2._u_limit",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.1._u_limit",
									 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
									 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
									 deviceName + ".eventMask");
					break;
				case UN_AIAO_ALARM_4_HH_H_L:
				case UN_AIAO_ALARM_4_H_L_LL:
					iRes = dpDisconnect("unAnalogInput_FaceplateStatusAnimationCB_4_ranges",
									 deviceName + ".ProcessInput.StsReg01", 
									 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
									 deviceName + ".ProcessInput.PosSt",
									 deviceName + ".ProcessInput.PosSt:_online.._invalid",
									 deviceName + ".ProcessInput.HFStOrAuPosRSt",
									 deviceName + ".ProcessInput.HFStOrAuPosRSt:_online.._invalid",
									 deviceName + ".ProcessInput.PosSt:_pv_range.._min",
									 deviceName + ".ProcessInput.PosSt:_pv_range.._max",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._active",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._act_range",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.4._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.3._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.2._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.1._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._ack_possible",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.4._l_limit",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.2._u_limit",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.1._u_limit",
									 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
									 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
									 deviceName + ".eventMask");
					break;
				case UN_AIAO_ALARM_3_H_L:
				case UN_AIAO_ALARM_3_HH_H:
				case UN_AIAO_ALARM_3_LL_L:
					iRes = dpDisconnect("unAnalogInput_FaceplateStatusAnimationCB_3_ranges",
									 deviceName + ".ProcessInput.StsReg01", 
									 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
									 deviceName + ".ProcessInput.PosSt",
									 deviceName + ".ProcessInput.PosSt:_online.._invalid",
									 deviceName + ".ProcessInput.HFStOrAuPosRSt",
									 deviceName + ".ProcessInput.HFStOrAuPosRSt:_online.._invalid",
									 deviceName + ".ProcessInput.PosSt:_pv_range.._min",
									 deviceName + ".ProcessInput.PosSt:_pv_range.._max",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._active",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._act_range",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.3._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.2._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.1._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._ack_possible",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.3._l_limit",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.1._u_limit",
									 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
									 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
									 deviceName + ".eventMask");
					break;
				case UN_AIAO_ALARM_2_H:
				case UN_AIAO_ALARM_2_L:
					iRes = dpDisconnect("unAnalogInput_FaceplateStatusAnimationCB_2_ranges",
									 deviceName + ".ProcessInput.StsReg01", 
									 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
									 deviceName + ".ProcessInput.PosSt",
									 deviceName + ".ProcessInput.PosSt:_online.._invalid",
									 deviceName + ".ProcessInput.HFStOrAuPosRSt",
									 deviceName + ".ProcessInput.HFStOrAuPosRSt:_online.._invalid",
									 deviceName + ".ProcessInput.PosSt:_pv_range.._min",
									 deviceName + ".ProcessInput.PosSt:_pv_range.._max",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._active",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._act_range",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.2._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.1._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._ack_possible",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.1._u_limit",
									 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
									 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
									 deviceName + ".eventMask");
					break;
			}
		}
		if ((!g_bBoolAlert) && (g_iRangeType == DPCONFIG_MINMAX_PVSS_RANGECHECK))
		{
			iRes = dpDisconnect("unAnalogInput_FaceplateStatusAnimationCB_NoAlarm",
							 deviceName + ".ProcessInput.StsReg01", 
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
							 deviceName + ".ProcessInput.PosSt",
							 deviceName + ".ProcessInput.PosSt:_online.._invalid",
							 deviceName + ".ProcessInput.HFStOrAuPosRSt",
							 deviceName + ".ProcessInput.HFStOrAuPosRSt:_online.._invalid",
							 deviceName + ".ProcessInput.PosSt:_pv_range.._min",
							 deviceName + ".ProcessInput.PosSt:_pv_range.._max",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							 deviceName + ".eventMask");
		}	
		if ((g_bBoolAlert) && (g_iRangeType != DPCONFIG_MINMAX_PVSS_RANGECHECK))
		{
			switch(g_iAlertRange) {
				case UN_AIAO_ALARM_5_HH_H_L_LL:
					iRes = dpDisconnect("unAnalogInput_FaceplateStatusAnimationCB_NoRange",
									 deviceName + ".ProcessInput.StsReg01", 
									 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
									 deviceName + ".ProcessInput.PosSt",
									 deviceName + ".ProcessInput.PosSt:_online.._invalid",
									 deviceName + ".ProcessInput.HFStOrAuPosRSt",
									 deviceName + ".ProcessInput.HFStOrAuPosRSt:_online.._invalid",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._active",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._act_range",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.5._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.4._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.2._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.1._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._ack_possible",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.5._l_limit",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.4._l_limit",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.2._u_limit",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.1._u_limit",
									 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
									 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
									 deviceName + ".eventMask");	
					break;
				case UN_AIAO_ALARM_4_HH_H_L:
				case UN_AIAO_ALARM_4_H_L_LL:
					iRes = dpDisconnect("unAnalogInput_FaceplateStatusAnimationCB_4_ranges_NoRange",
									 deviceName + ".ProcessInput.StsReg01", 
									 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
									 deviceName + ".ProcessInput.PosSt",
									 deviceName + ".ProcessInput.PosSt:_online.._invalid",
									 deviceName + ".ProcessInput.HFStOrAuPosRSt",
									 deviceName + ".ProcessInput.HFStOrAuPosRSt:_online.._invalid",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._active",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._act_range",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.4._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.3._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.2._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.1._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._ack_possible",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.4._l_limit",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.2._u_limit",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.1._u_limit",
									 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
									 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
									 deviceName + ".eventMask");
					break;
				case UN_AIAO_ALARM_3_H_L:
				case UN_AIAO_ALARM_3_HH_H:
				case UN_AIAO_ALARM_3_LL_L:
					iRes = dpDisconnect("unAnalogInput_FaceplateStatusAnimationCB_3_ranges_NoRange",
									 deviceName + ".ProcessInput.StsReg01", 
									 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
									 deviceName + ".ProcessInput.PosSt",
									 deviceName + ".ProcessInput.PosSt:_online.._invalid",
									 deviceName + ".ProcessInput.HFStOrAuPosRSt",
									 deviceName + ".ProcessInput.HFStOrAuPosRSt:_online.._invalid",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._active",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._act_range",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.3._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.2._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.1._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._ack_possible",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.3._l_limit",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.1._u_limit",
									 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
									 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
									 deviceName + ".eventMask");	
					break;
				case UN_AIAO_ALARM_2_H:
				case UN_AIAO_ALARM_2_L:
					iRes = dpDisconnect("unAnalogInput_FaceplateStatusAnimationCB_2_ranges_NoRange",
									 deviceName + ".ProcessInput.StsReg01", 
									 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
									 deviceName + ".ProcessInput.PosSt",
									 deviceName + ".ProcessInput.PosSt:_online.._invalid",
									 deviceName + ".ProcessInput.HFStOrAuPosRSt",
									 deviceName + ".ProcessInput.HFStOrAuPosRSt:_online.._invalid",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._active",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._act_range",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.2._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.1._act_state",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.._ack_possible",
									 deviceName + ".ProcessInput.PosSt:_alert_hdl.1._u_limit",
									 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
									 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
									 deviceName + ".eventMask");	
					break;
			}
		}
		if ((!g_bBoolAlert) && (g_iRangeType != DPCONFIG_MINMAX_PVSS_RANGECHECK))
		{
			iRes = dpDisconnect("unAnalogInput_FaceplateStatusAnimationCB_NoRangeNoAlarm",
							 deviceName + ".ProcessInput.StsReg01", 
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
							 deviceName + ".ProcessInput.PosSt",
							 deviceName + ".ProcessInput.PosSt:_online.._invalid",
							 deviceName + ".ProcessInput.HFStOrAuPosRSt",
							 deviceName + ".ProcessInput.HFStOrAuPosRSt:_online.._invalid",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							 deviceName + ".eventMask");		
		}
	}
}
//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogInput_FaceplateStatusAnimationCB_NoRangeNoAlarm
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogInput_FaceplateStatusAnimationCB_NoRangeNoAlarm(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
										 		 string sDpPosSt, float fPosSt, string sDp4, bool bPosStInvalid, 
										 		 string sDpHFStOrAuPosRSt, float fHFStOrAuPosRSt, string sDp6, bool bHFStOrAuPosRStInvalid,
										 		 string sDpAl, int iValAlarm, string sDpEnabled, bool bPlcEnabled, string sDpEvt, int iMask)
{
	unAnalogInput_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid, 
									    	 sDpPosSt, fPosSt, sDp4, bPosStInvalid,
					 			        	 sDpHFStOrAuPosRSt, fHFStOrAuPosRSt, sDp6, bHFStOrAuPosRStInvalid,
					 			        	 "", 0, "", 0,
					 			        	 "", 0, "", 0, "", 0, "", 0, "", 0, "", 0, "", 0, "", 0, "", 0, "", 0, "", 0, 
					 			        	 sDpAl, iValAlarm, sDpEnabled, bPlcEnabled, sDpEvt, iMask);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogInput_FaceplateStatusAnimationCB_NoRange
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogInput_FaceplateStatusAnimationCB_NoRange(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
										 		 string sDpPosSt, float fPosSt, string sDp4, bool bPosStInvalid, 
										 		 string sDpHFStOrAuPosRSt, float fHFStOrAuPosRSt, string sDp6, bool bHFStOrAuPosRStInvalid, 
						 			        	 string sDp9, bool bIsAlarmActive, string sDp10, int iActRange,
						 			        	 string sDp11, int iHHActRange, string sDp12, int iHActRange, 
						 			        	 string sDp13, int iLActRange, string sDp14, int iLLActRange,
						 			        	 string sDp15, bool bAckPossible,
						 			        	 string sDp16, float fHH, string sDp17, float fH, 
						 			        	 string sDp18, float fL, string sDp19, float fLL,
						 			        	 string sDpAl, int iValAlarm, string sDpEnabled, bool bPlcEnabled, string sDpEvt, int iMask)
{
	unAnalogInput_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid, 
									    	 sDpPosSt, fPosSt, sDp4, bPosStInvalid,
					 			        	 sDpHFStOrAuPosRSt, fHFStOrAuPosRSt, sDp6, bHFStOrAuPosRStInvalid,
					 			        	 "", 0, "", 0,
					 			        	 sDp9, bIsAlarmActive, sDp10, iActRange, sDp11, iHHActRange, sDp12, iHActRange, 
						 			         sDp13, iLActRange, sDp14, iLLActRange, sDp15, bAckPossible,
						 			         sDp16, fHH, sDp17, fH, sDp18, fL, sDp19, fLL, sDpAl, iValAlarm, sDpEnabled, bPlcEnabled,
						 			         sDpEvt, iMask);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogInput_FaceplateStatusAnimationCB_4_ranges_NoRange
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogInput_FaceplateStatusAnimationCB_4_ranges_NoRange(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
										 		 string sDpPosSt, float fPosSt, string sDp4, bool bPosStInvalid, 
										 		 string sDpHFStOrAuPosRSt, float fHFStOrAuPosRSt, string sDp6, bool bHFStOrAuPosRStInvalid, 
						 			        	 string sDp9, bool bIsAlarmActive, string sDp10, int iActRange,
						 			        	 string sDp11, int i4ActRange, string sDp12, int i3ActRange, 
						 			        	 string sDp13, int i2ActRange, string sDp14, int i1ActRange,
						 			        	 string sDp15, bool bAckPossible,
						 			        	 string sDp17, float f3, 
						 			        	 string sDp18, float f2, string sDp19, float f1,
						 			        	 string sDpAl, int iValAlarm, string sDpEnabled, bool bPlcEnabled, string sDpEvt, int iMask)
{
	switch(g_iAlertRange) {
		case UN_AIAO_ALARM_4_HH_H_L:
			unAnalogInput_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid, 
											    	 sDpPosSt, fPosSt, sDp4, bPosStInvalid,
							 			        	 sDpHFStOrAuPosRSt, fHFStOrAuPosRSt, sDp6, bHFStOrAuPosRStInvalid,
							 			        	 "", 0, "", 0,
							 			        	 sDp9, bIsAlarmActive, sDp10, iActRange, 
							 			        	 sDp11, i4ActRange, sDp12, i3ActRange, 
								 			         sDp13, i1ActRange, "", 0, sDp15, bAckPossible,
								 			         sDp17, f3, sDp18, f2, sDp19, f1, "", 0, sDpAl, iValAlarm, sDpEnabled, bPlcEnabled,
								 			         sDpEvt, iMask);
			break;
		case UN_AIAO_ALARM_4_H_L_LL:
			unAnalogInput_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid, 
											    	 sDpPosSt, fPosSt, sDp4, bPosStInvalid,
							 			        	 sDpHFStOrAuPosRSt, fHFStOrAuPosRSt, sDp6, bHFStOrAuPosRStInvalid,
							 			        	 "", 0, "", 0,
							 			        	 sDp9, bIsAlarmActive, sDp10, iActRange, 
							 			        	 "", 0, sDp11, i4ActRange, 
								 			         sDp13, i2ActRange, sDp14, i1ActRange, sDp15, bAckPossible,
								 			         "", 0, sDp17, f3, sDp18, f2, sDp19, f1, sDpAl, iValAlarm, sDpEnabled, bPlcEnabled,
								 			         sDpEvt, iMask);
			break;
	} 
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogInput_FaceplateStatusAnimationCB_3_ranges_NoRange
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogInput_FaceplateStatusAnimationCB_3_ranges_NoRange(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
										 		 string sDpPosSt, float fPosSt, string sDp4, bool bPosStInvalid, 
										 		 string sDpHFStOrAuPosRSt, float fHFStOrAuPosRSt, string sDp6, bool bHFStOrAuPosRStInvalid, 
						 			        	 string sDp9, bool bIsAlarmActive, string sDp10, int iActRange,
						 			        	 string sDp11, int i3ActRange, 
						 			        	 string sDp12, int i2ActRange, 
						 			        	 string sDp13, int i1ActRange,
						 			        	 string sDp15, bool bAckPossible,
						 			        	 string sDp16, float f3, 
						 			        	 string sDp17, float f1, string sDpAl, int iValAlarm, string sDpEnabled, bool bPlcEnabled,
						 			        	 string sDpEvt, int iMask)
{
	switch(g_iAlertRange) {
		case UN_AIAO_ALARM_3_H_L:
			unAnalogInput_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid, 
									    	 sDpPosSt, fPosSt, sDp4, bPosStInvalid,
					 			        	 sDpHFStOrAuPosRSt, fHFStOrAuPosRSt, sDp6, bHFStOrAuPosRStInvalid,
					 			        	 "", 0, "", 0,
					 			        	 sDp9, bIsAlarmActive, sDp10, iActRange, "", 0, sDp11, i3ActRange, 
						 			         sDp13, i1ActRange, "", 0, sDp15, bAckPossible,
						 			         "", 0, sDp16, f3, sDp17, f1, "", 0, sDpAl, iValAlarm, sDpEnabled, bPlcEnabled,
						 			         sDpEvt, iMask);
		break;
		case UN_AIAO_ALARM_3_HH_H:
			unAnalogInput_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid, 
									    	 sDpPosSt, fPosSt, sDp4, bPosStInvalid,
					 			        	 sDpHFStOrAuPosRSt, fHFStOrAuPosRSt, sDp6, bHFStOrAuPosRStInvalid,
					 			        	 "", 0, "", 0,
					 			        	 sDp9, bIsAlarmActive, sDp10, iActRange, sDp11, i3ActRange, sDp12, i2ActRange, 
						 			         "", 0, "", 0, sDp15, bAckPossible,
						 			         sDp16, f3, sDp17, f1, "", 0, "", 0, sDpAl, iValAlarm, sDpEnabled, bPlcEnabled,
						 			         sDpEvt, iMask);
		break;
		case UN_AIAO_ALARM_3_LL_L:
			unAnalogInput_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid, 
									    	 sDpPosSt, fPosSt, sDp4, bPosStInvalid,
					 			        	 sDpHFStOrAuPosRSt, fHFStOrAuPosRSt, sDp6, bHFStOrAuPosRStInvalid,
					 			        	 "", 0, "", 0,
					 			        	 sDp9, bIsAlarmActive, sDp10, iActRange, "", 0, "", 0, 
						 			         sDp12, i2ActRange, sDp13, i1ActRange, sDp15, bAckPossible,
						 			         "", 0, "", 0, sDp16, f3, sDp17, f1, sDpAl, iValAlarm, sDpEnabled, bPlcEnabled,
						 			         sDpEvt, iMask);
		break;
	}		
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogInput_FaceplateStatusAnimationCB_2_ranges_NoRange
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogInput_FaceplateStatusAnimationCB_2_ranges_NoRange(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
										 		 string sDpPosSt, float fPosSt, string sDp4, bool bPosStInvalid, 
										 		 string sDpHFStOrAuPosRSt, float fHFStOrAuPosRSt, string sDp6, bool bHFStOrAuPosRStInvalid, 
						 			        	 string sDp9, bool bIsAlarmActive, string sDp10, int iActRange,
						 			        	 string sDp12, int i2ActRange, 
						 			        	 string sDp13, int i1ActRange,
						 			        	 string sDp15, bool bAckPossible,
						 			        	 string sDp17, float f1, string sDpAl, int iValAlarm, string sDpEnabled, bool bPlcEnabled,
						 			        	 string sDpEvt, int iMask)
{
	switch(g_iAlertRange) {
		case UN_AIAO_ALARM_2_H:
			unAnalogInput_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid, 
									    	 sDpPosSt, fPosSt, sDp4, bPosStInvalid,
					 			        	 sDpHFStOrAuPosRSt, fHFStOrAuPosRSt, sDp6, bHFStOrAuPosRStInvalid,
					 			        	 "", 0, "", 0,
					 			        	 sDp9, bIsAlarmActive, sDp10, iActRange, "", 0, sDp12, i2ActRange, 
						 			         "", 0, "", 0, sDp15, bAckPossible,
						 			         "", 0, sDp17, f1, "", 0, "", 0, sDpAl, iValAlarm, sDpEnabled, bPlcEnabled,
						 			         sDpEvt, iMask);
		break;
		case UN_AIAO_ALARM_2_L:
			unAnalogInput_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid, 
									    	 sDpPosSt, fPosSt, sDp4, bPosStInvalid,
					 			        	 sDpHFStOrAuPosRSt, fHFStOrAuPosRSt, sDp6, bHFStOrAuPosRStInvalid,
					 			        	 "", 0, "", 0,
					 			        	 sDp9, bIsAlarmActive, sDp10, iActRange, "", 0, "", 0, 
						 			         sDp13, i1ActRange, "", 0, sDp15, bAckPossible,
						 			         "", 0, "", 0, sDp17, f1, "", 0, sDpAl, iValAlarm, sDpEnabled, bPlcEnabled,
						 			         sDpEvt, iMask);
		break;
	}		
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogInput_FaceplateStatusAnimationCB_NoAlarm
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogInput_FaceplateStatusAnimationCB_NoAlarm(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
										 		 string sDpPosSt, float fPosSt, string sDp4, bool bPosStInvalid, 
										 		 string sDpHFStOrAuPosRSt, float fHFStOrAuPosRSt, string sDp6, bool bHFStOrAuPosRStInvalid, 
				 			        			 string sDp7, float fMin, string sDp8, float fMax, 
				 			        			 string sDpAl, int iValAlarm, string sDpEnabled, bool bPlcEnabled,
				 			        			 string sDpEvt, int iMask)
{
	unAnalogInput_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid, 
									    	 sDpPosSt, fPosSt, sDp4, bPosStInvalid,
					 			        	 sDpHFStOrAuPosRSt, fHFStOrAuPosRSt, sDp6, bHFStOrAuPosRStInvalid,
					 			        	 sDp7, fMin, sDp8, fMax,
					 			        	 "", 0, "", 0, "", 0, "", 0, "", 0, "", 0, "", 0, "", 0, "", 0, "", 0, "", 0, 
					 			        	 sDpAl, iValAlarm, sDpEnabled, bPlcEnabled, sDpEvt, iMask);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogInput_FaceplateStatusAnimationCB_4_ranges
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in AnalogInput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogInput_FaceplateStatusAnimationCB_4_ranges(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
										 string sDpPosSt, float fPosSt, string sDp4, bool bPosStInvalid, 
										 string sDpHFStOrAuPosRSt, float fHFStOrAuPosRSt, string sDp6, bool bHFStOrAuPosRStInvalid, 
				 			        	 string sDp7, float fMin, string sDp8, float fMax,
				 			        	 string sDp9, bool bIsAlarmActive, string sDp10, int iActRange,
				 			        	 string sDp11, int i4ActRange, string sDp12, int i3ActRange, 
				 			        	 string sDp13, int i2ActRange, string sDp14, int i1ActRange,
				 			        	 string sDp15, bool bAckPossible,
				 			        	 string sDp17, float f3, 
				 			        	 string sDp18, float f2, string sDp19, float f1,
				 			        	 string sDpAl, int iValAlarm, string sDpEnabled, bool bPlcEnabled, string sDpEvt, int iMask)
{
	switch(g_iAlertRange) {
		case UN_AIAO_ALARM_4_HH_H_L:
			unAnalogInput_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid, 
											    	 sDpPosSt, fPosSt, sDp4, bPosStInvalid,
							 			        	 sDpHFStOrAuPosRSt, fHFStOrAuPosRSt, sDp6, bHFStOrAuPosRStInvalid,
							 			        	 sDp7, fMin, sDp8, fMax,
							 			        	 sDp9, bIsAlarmActive, sDp10, iActRange, 
							 			        	 sDp11, i4ActRange, sDp12, i3ActRange, 
								 			         sDp13, i1ActRange, "", 0, sDp15, bAckPossible,
								 			         sDp17, f3, sDp18, f2, sDp19, f1, "", 0, sDpAl, iValAlarm, sDpEnabled, bPlcEnabled,
								 			         sDpEvt, iMask);
			break;
		case UN_AIAO_ALARM_4_H_L_LL:
			unAnalogInput_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid, 
											    	 sDpPosSt, fPosSt, sDp4, bPosStInvalid,
							 			        	 sDpHFStOrAuPosRSt, fHFStOrAuPosRSt, sDp6, bHFStOrAuPosRStInvalid,
							 			        	 sDp7, fMin, sDp8, fMax,
							 			        	 sDp9, bIsAlarmActive, sDp10, iActRange, 
							 			        	 "", 0, sDp11, i4ActRange, 
								 			         sDp13, i2ActRange, sDp14, i1ActRange, sDp15, bAckPossible,
								 			         "", 0, sDp17, f3, sDp18, f2, sDp19, f1, sDpAl, iValAlarm, sDpEnabled, bPlcEnabled,
								 			         sDpEvt, iMask);
			break;
	} 
}
//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogInput_FaceplateStatusAnimationCB_3_ranges
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in AnalogInput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogInput_FaceplateStatusAnimationCB_3_ranges(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
										 string sDpPosSt, float fPosSt, string sDp4, bool bPosStInvalid, 
										 string sDpHFStOrAuPosRSt, float fHFStOrAuPosRSt, string sDp6, bool bHFStOrAuPosRStInvalid, 
				 			        	 string sDp7, float fMin, string sDp8, float fMax,
				 			        	 string sDp9, bool bIsAlarmActive, string sDp10, int iActRange,
				 			        	 string sDp11, int i3ActRange, 
				 			        	 string sDp12, int i2ActRange,
				 			        	 string sDp13, int i1ActRange,
				 			        	 string sDp15, bool bAckPossible,
				 			        	 string sDp16, float f3, 
				 			        	 string sDp17, float f1, string sDpAl, int iValAlarm, string sDpEnabled, bool bPlcEnabled,
				 			        	 string sDpEvt, int iMask)
{
	switch(g_iAlertRange) {
		case UN_AIAO_ALARM_3_H_L:
			unAnalogInput_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid, 
											    	 sDpPosSt, fPosSt, sDp4, bPosStInvalid,
							 			        	 sDpHFStOrAuPosRSt, fHFStOrAuPosRSt, sDp6, bHFStOrAuPosRStInvalid,
							 			        	 sDp7, fMin, sDp8, fMax,
							 			        	 sDp9, bIsAlarmActive, sDp10, iActRange, "", 0, sDp11, i3ActRange, 
								 			         sDp13, i1ActRange, "", 0, sDp15, bAckPossible,
								 			         "", 0, sDp16, f3, sDp17, f1, "", 0, sDpAl, iValAlarm, sDpEnabled, bPlcEnabled,
								 			         sDpEvt, iMask);
			break;
		case UN_AIAO_ALARM_3_HH_H:
			unAnalogInput_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid, 
											    	 sDpPosSt, fPosSt, sDp4, bPosStInvalid,
							 			        	 sDpHFStOrAuPosRSt, fHFStOrAuPosRSt, sDp6, bHFStOrAuPosRStInvalid,
							 			        	 sDp7, fMin, sDp8, fMax,
							 			        	 sDp9, bIsAlarmActive, sDp10, iActRange, sDp11, i3ActRange, sDp12, i2ActRange, 
								 			         "", 0, "", 0, sDp15, bAckPossible,
								 			         sDp16, f3, sDp17, f1, "", 0, "", 0, sDpAl, iValAlarm, sDpEnabled, bPlcEnabled,
								 			         sDpEvt, iMask);
			break;
		case UN_AIAO_ALARM_3_LL_L:
			unAnalogInput_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid, 
											    	 sDpPosSt, fPosSt, sDp4, bPosStInvalid,
							 			        	 sDpHFStOrAuPosRSt, fHFStOrAuPosRSt, sDp6, bHFStOrAuPosRStInvalid,
							 			        	 sDp7, fMin, sDp8, fMax,
							 			        	 sDp9, bIsAlarmActive, sDp10, iActRange, "", 0, "", 0, 
								 			         sDp12, i2ActRange, sDp13, i1ActRange, sDp15, bAckPossible,
								 			         "", 0, "", 0, sDp16, f3, sDp17, f1, sDpAl, iValAlarm, sDpEnabled, bPlcEnabled,
								 			         sDpEvt, iMask);
			break;
	} 
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogInput_FaceplateStatusAnimationCB_2_ranges
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in AnalogInput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogInput_FaceplateStatusAnimationCB_2_ranges(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
										 string sDpPosSt, float fPosSt, string sDp4, bool bPosStInvalid, 
										 string sDpHFStOrAuPosRSt, float fHFStOrAuPosRSt, string sDp6, bool bHFStOrAuPosRStInvalid, 
				 			        	 string sDp7, float fMin, string sDp8, float fMax,
				 			        	 string sDp9, bool bIsAlarmActive, string sDp10, int iActRange,
				 			        	 string sDp12, int i2ActRange,
				 			        	 string sDp13, int i1ActRange,
				 			        	 string sDp15, bool bAckPossible,
				 			        	 string sDp17, float f1, string sDpAl, int iValAlarm, string sDpEnabled, bool bPlcEnabled,
				 			        	 string sDpEvt, int iMask)
{
	switch(g_iAlertRange) {
		case UN_AIAO_ALARM_2_H:
			unAnalogInput_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid, 
											    	 sDpPosSt, fPosSt, sDp4, bPosStInvalid,
							 			        	 sDpHFStOrAuPosRSt, fHFStOrAuPosRSt, sDp6, bHFStOrAuPosRStInvalid,
							 			        	 sDp7, fMin, sDp8, fMax,
							 			        	 sDp9, bIsAlarmActive, sDp10, iActRange, "", 0, sDp12, i2ActRange, 
								 			         "", 0, "", 0, sDp15, bAckPossible,
								 			         "", 0, sDp17, f1, "", 0, "", 0, sDpAl, iValAlarm, sDpEnabled, bPlcEnabled,
								 			         sDpEvt, iMask);
			break;
		case UN_AIAO_ALARM_2_L:
			unAnalogInput_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid, 
											    	 sDpPosSt, fPosSt, sDp4, bPosStInvalid,
							 			        	 sDpHFStOrAuPosRSt, fHFStOrAuPosRSt, sDp6, bHFStOrAuPosRStInvalid,
							 			        	 sDp7, fMin, sDp8, fMax,
							 			        	 sDp9, bIsAlarmActive, sDp10, iActRange, "", 0, "", 0, 
								 			         sDp13, i1ActRange, "", 0, sDp15, bAckPossible,
								 			         "", 0, "", 0, sDp17, f1, "", 0, sDpAl, iValAlarm, sDpEnabled, bPlcEnabled,
								 			         sDpEvt, iMask);
			break;
	} 
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogInput_FaceplateStatusAnimationCB
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in AnalogInput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogInput_FaceplateStatusAnimationCB(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
										 string sDpPosSt, float fPosSt, string sDp4, bool bPosStInvalid, 
										 string sDpHFStOrAuPosRSt, float fHFStOrAuPosRSt, string sDp6, bool bHFStOrAuPosRStInvalid, 
				 			        	 string sDp7, float fMin, string sDp8, float fMax,
				 			        	 string sDp9, bool bIsAlarmActive, string sDp10, int iActRange,
				 			        	 string sDp11, int iHHActRange, string sDp12, int iHActRange, 
				 			        	 string sDp13, int iLActRange, string sDp14, int iLLActRange,
				 			        	 string sDp15, bool bAckPossible,
				 			        	 string sDp16, float fHH, string sDp17, float fH, 
				 			        	 string sDp18, float fL, string sDp19, float fLL,
				 			        	 string sDpAl, int iValAlarm, string sDpEnabled, bool bSystemIntegrityAlarmEnabled,
				 			        	 string sDpEvt, int iMask)
{
	string sColor;
	bool bNoHH = false;
	dyn_bool isDpConfigured;

//DebugN("unAnalogInput_FaceplateStatusAnimationCB", g_iAlertRange, iHHActRange, fHH, iHActRange, fH, iLActRange, fL, iLLActRange, fLL);
// 1. Animate display values
 if(g_bSystemConnected)
		unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, fPosSt, "Request_Active", "unDisplayValue_Status", 
			bPosStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	if(g_bAnalogInput)
	{
	 if(g_bSystemConnected)
		unGenericObject_DisplayValue(g_sHFStOrAuPosRStFormat, g_sHFStOrAuPosRStUnit, fHFStOrAuPosRSt, "Request_Process", "unDisplayValue_Status", 
					bHFStOrAuPosRStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	}
	else
	{
	 if(g_bSystemConnected)
		unGenericObject_DisplayValue(g_sHFStOrAuPosRStFormat, g_sHFStOrAuPosRStUnit, fHFStOrAuPosRSt, "Request_Auto", "unDisplayValue_Request", 
					bHFStOrAuPosRStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	}				
 if(g_bSystemConnected)
	unGenericObject_DisplayRange(g_iRangeType, g_sPosStFormat, g_sPosStUnit, fMin, fMax, "RangeMin", "RangeMax", "unDisplayValue_Parameter");
// 2. Animate colorboxes
 if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OperationMode_Auto", getBit(bit32StsReg01, UN_STSREG01_AUMOST), 1, "unFaceplate_OperationModeActive", 
					bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
 if(g_bSystemConnected)				
	unGenericObject_ColorBoxAnimate("OperationMode_Forced", getBit(bit32StsReg01, UN_STSREG01_FOMOST), 1, "unFaceplate_OperationModeActive", 
					bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
 if(g_bSystemConnected)					
	unGenericObject_ColorBoxAnimate("Warning_IOError", getBit(bit32StsReg01, UN_STSREG01_IOERRORW), 1, "unFaceplate_WarningActive", 
					bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
 if(g_bSystemConnected)					
	unGenericObject_ColorBoxAnimate("Warning_IOSimulated", getBit(bit32StsReg01, UN_STSREG01_IOSIMUW), 1, "unFaceplate_WarningActive", 
					bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
 if(g_bSystemConnected)					
	unGenericObject_ColorBoxAnimate("Warning_ForcedProcess", getBit(bit32StsReg01, UN_STSREG01_FODIPRO), 1, "unFaceplate_WarningActive", 
					bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));

// 3. Animate alarm
// 3.1. Alarm exists
	if (g_bBoolAlert)
	{
//		sColor = bIsAlarmActive ? "unDisplayValue_Parameter" : "unFaceplate_AlarmMasked";
// always show alarm limits even if alarm is masked
		sColor = "unDisplayValue_Parameter";
// 3.1.1 display limit value
		switch(g_iAlertRange) {
			case UN_AIAO_ALARM_5_HH_H_L_LL:
			 if(g_bSystemConnected)			
				unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, fHH, "ValueHH", sColor, false);
			 if(g_bSystemConnected)
				unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, fLL, "ValueLL", sColor, false);
			 if(g_bSystemConnected)
				unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, fH, "ValueH", sColor, false);
			 if(g_bSystemConnected)				
				unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, fL, "ValueL", sColor, false);
				break;
			case  UN_AIAO_ALARM_4_HH_H_L:
			 if(g_bSystemConnected)
				setMultiValue("ValueLL.display", "text", "", "ValueLL.display", "foreCol", "unAlarmDoesNotExist");
			 if(g_bSystemConnected)
				unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, fHH, "ValueHH", sColor, false);
			 if(g_bSystemConnected)				
				unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, fH, "ValueH", sColor, false);
			 if(g_bSystemConnected)				
				unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, fL, "ValueL", sColor, false);
				break;
			case  UN_AIAO_ALARM_4_H_L_LL:
			 if(g_bSystemConnected)				
				setMultiValue("ValueHH.display", "text", "", "ValueHH.display", "foreCol", "unAlarmDoesNotExist");
			 if(g_bSystemConnected)	
				unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, fH, "ValueH", sColor, false);
			 if(g_bSystemConnected)				
				unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, fL, "ValueL", sColor, false);
			 if(g_bSystemConnected)				
				unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, fLL, "ValueLL", sColor, false);
				break;
			case  UN_AIAO_ALARM_3_H_L:
			 if(g_bSystemConnected)
				setMultiValue("ValueHH.display", "text", "", "ValueHH.display", "foreCol", "unAlarmDoesNotExist",
								"ValueLL.display", "text", "", "ValueLL.display", "foreCol", "unAlarmDoesNotExist");
			 if(g_bSystemConnected)
				unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, fH, "ValueH", sColor, false);
			 if(g_bSystemConnected)				
				unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, fL, "ValueL", sColor, false);
				break;
			case  UN_AIAO_ALARM_3_HH_H:
			 if(g_bSystemConnected)			
				setMultiValue("ValueL.display", "text", "", "ValueL.display", "foreCol", "unAlarmDoesNotExist",
								"ValueLL.display", "text", "", "ValueLL.display", "foreCol", "unAlarmDoesNotExist");
			 if(g_bSystemConnected)
				unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, fH, "ValueH", sColor, false);
			 if(g_bSystemConnected)				
				unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, fHH, "ValueHH", sColor, false);
				break;
			case  UN_AIAO_ALARM_3_LL_L:
			 if(g_bSystemConnected)
				setMultiValue("ValueH.display", "text", "", "ValueH.display", "foreCol", "unAlarmDoesNotExist",
								"ValueHH.display", "text", "", "ValueHH.display", "foreCol", "unAlarmDoesNotExist");
			 if(g_bSystemConnected)
				unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, fL, "ValueL", sColor, false);
			 if(g_bSystemConnected)				
				unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, fLL, "ValueLL", sColor, false);
				break;
			case  UN_AIAO_ALARM_2_H:
			 if(g_bSystemConnected)
				setMultiValue("ValueHH.display", "text", "", "ValueHH.display", "foreCol", "unAlarmDoesNotExist",
								"ValueL.display", "text", "", "ValueL.display", "foreCol", "unAlarmDoesNotExist",
								"ValueLL.display", "text", "", "ValueLL.display", "foreCol", "unAlarmDoesNotExist");
			 if(g_bSystemConnected)
				unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, fH, "ValueH", sColor, false);
				break;
			case  UN_AIAO_ALARM_2_L:
			 if(g_bSystemConnected)
				setMultiValue("ValueHH.display", "text", "", "ValueHH.display", "foreCol", "unAlarmDoesNotExist",
								"ValueH.display", "text", "", "ValueH.display", "foreCol", "unAlarmDoesNotExist",
								"ValueLL.display", "text", "", "ValueLL.display", "foreCol", "unAlarmDoesNotExist");
			 if(g_bSystemConnected)
				unGenericObject_DisplayValue(g_sPosStFormat, g_sPosStUnit, fL, "ValueL", sColor, false);
				break;
			default:
				break;
		}
// 3.1.1 animate color box
		if (bIsAlarmActive)
		{
			switch(g_iAlertRange) {
				case UN_AIAO_ALARM_5_HH_H_L_LL:
				 if(g_bSystemConnected)
					unGenericObject_ColorBoxAnimateAlarm("Alarm_H", iHActRange, g_bBoolAlert, false);
				 if(g_bSystemConnected)					
					unGenericObject_ColorBoxAnimateAlarm("Alarm_L", iLActRange, g_bBoolAlert, false);
				 if(g_bSystemConnected)
					unGenericObject_ColorBoxAnimateAlarm("Alarm_HH", iHHActRange, g_bBoolAlert, false);
				 if(g_bSystemConnected)					
					unGenericObject_ColorBoxAnimateAlarm("Alarm_LL", iLLActRange, g_bBoolAlert, false);
					break;
				case  UN_AIAO_ALARM_4_HH_H_L:
				 if(g_bSystemConnected)
					unGenericObject_ColorBoxAnimateAlarm("Alarm_LL", 0, false, false);
				 if(g_bSystemConnected)
					unGenericObject_ColorBoxAnimateAlarm("Alarm_HH", iHHActRange, g_bBoolAlert, false);
				 if(g_bSystemConnected)					
					unGenericObject_ColorBoxAnimateAlarm("Alarm_H", iHActRange, g_bBoolAlert, false);
				 if(g_bSystemConnected)					
					unGenericObject_ColorBoxAnimateAlarm("Alarm_L", iLActRange, g_bBoolAlert, false);
					break;
				case  UN_AIAO_ALARM_4_H_L_LL:
				 if(g_bSystemConnected)
					unGenericObject_ColorBoxAnimateAlarm("Alarm_HH", 0, false, false);
				 if(g_bSystemConnected)
					unGenericObject_ColorBoxAnimateAlarm("Alarm_H", iHActRange, g_bBoolAlert, false);
				 if(g_bSystemConnected)
					unGenericObject_ColorBoxAnimateAlarm("Alarm_L", iLActRange, g_bBoolAlert, false);
				 if(g_bSystemConnected)				
					unGenericObject_ColorBoxAnimateAlarm("Alarm_LL", iLLActRange, g_bBoolAlert, false);
					break;
				case  UN_AIAO_ALARM_3_H_L:
				 if(g_bSystemConnected)
					unGenericObject_ColorBoxAnimateAlarm("Alarm_HH", 0, false, false);
				 if(g_bSystemConnected)					
					unGenericObject_ColorBoxAnimateAlarm("Alarm_LL", 0, false, false);
				 if(g_bSystemConnected)
					unGenericObject_ColorBoxAnimateAlarm("Alarm_H", iHActRange, g_bBoolAlert, false);
				 if(g_bSystemConnected)					
					unGenericObject_ColorBoxAnimateAlarm("Alarm_L", iLActRange, g_bBoolAlert, false);
					break;
				case  UN_AIAO_ALARM_3_HH_H:
				 if(g_bSystemConnected)
					unGenericObject_ColorBoxAnimateAlarm("Alarm_L", 0, false, false);
				 if(g_bSystemConnected)					
					unGenericObject_ColorBoxAnimateAlarm("Alarm_LL", 0, false, false);
				 if(g_bSystemConnected)
					unGenericObject_ColorBoxAnimateAlarm("Alarm_H", iHActRange, g_bBoolAlert, false);
				 if(g_bSystemConnected)					
					unGenericObject_ColorBoxAnimateAlarm("Alarm_HH", iHHActRange, g_bBoolAlert, false);
					break;
				case  UN_AIAO_ALARM_3_LL_L:
				 if(g_bSystemConnected)				
					unGenericObject_ColorBoxAnimateAlarm("Alarm_H", 0, false, false);
				 if(g_bSystemConnected)					
					unGenericObject_ColorBoxAnimateAlarm("Alarm_HH", 0, false, false);
				 if(g_bSystemConnected)
					unGenericObject_ColorBoxAnimateAlarm("Alarm_L", iLActRange, g_bBoolAlert, false);
				 if(g_bSystemConnected)					
					unGenericObject_ColorBoxAnimateAlarm("Alarm_LL", iLLActRange, g_bBoolAlert, false);
					break;
				case  UN_AIAO_ALARM_2_H:
				 if(g_bSystemConnected)				
					unGenericObject_ColorBoxAnimateAlarm("Alarm_HH", 0, false, false);
				 if(g_bSystemConnected)					
					unGenericObject_ColorBoxAnimateAlarm("Alarm_L", 0, false, false);
				 if(g_bSystemConnected)	
					unGenericObject_ColorBoxAnimateAlarm("Alarm_LL", 0, false, false);
				 if(g_bSystemConnected)
					unGenericObject_ColorBoxAnimateAlarm("Alarm_H", iHActRange, g_bBoolAlert, false);
					break;
				case  UN_AIAO_ALARM_2_L:
				 if(g_bSystemConnected)
					unGenericObject_ColorBoxAnimateAlarm("Alarm_HH", 0, false, false);
				 if(g_bSystemConnected)					
					unGenericObject_ColorBoxAnimateAlarm("Alarm_H", 0, false, false);
				 if(g_bSystemConnected)					
					unGenericObject_ColorBoxAnimateAlarm("Alarm_LL", 0, false, false);
				 if(g_bSystemConnected)
					unGenericObject_ColorBoxAnimateAlarm("Alarm_L", iLActRange, g_bBoolAlert, false);
					break;
				default:
					break;
			}
		}
		else
		{
			switch(g_iAlertRange) {
				case UN_AIAO_ALARM_5_HH_H_L_LL:
				 if(g_bSystemConnected)	
					unGenericObject_ColorBoxAnimate("Alarm_HH", false, false, "", false);
				 if(g_bSystemConnected)					
					unGenericObject_ColorBoxAnimate("Alarm_LL", false, false, "", false);
				 if(g_bSystemConnected)
					unGenericObject_ColorBoxAnimate("Alarm_H", false, false, "", false);
				 if(g_bSystemConnected)					
					unGenericObject_ColorBoxAnimate("Alarm_L", false, false, "", false);
					break;
				case  UN_AIAO_ALARM_4_HH_H_L:
				 if(g_bSystemConnected)				
					unGenericObject_ColorBoxAnimateAlarm("Alarm_LL", 0, false, false);
////
				 if(g_bSystemConnected)
					unGenericObject_ColorBoxAnimate("Alarm_HH", false, false, "", false);
				 if(g_bSystemConnected)					
					unGenericObject_ColorBoxAnimate("Alarm_H", false, false, "", false);
				 if(g_bSystemConnected)						
					unGenericObject_ColorBoxAnimate("Alarm_L", false, false, "", false);
					break;
				case  UN_AIAO_ALARM_4_H_L_LL:
				 if(g_bSystemConnected)
					unGenericObject_ColorBoxAnimateAlarm("Alarm_HH", 0, false, false);
				 if(g_bSystemConnected)
					unGenericObject_ColorBoxAnimate("Alarm_H", false, false, "", false);
				 if(g_bSystemConnected)
					unGenericObject_ColorBoxAnimate("Alarm_L", false, false, "", false);
				 if(g_bSystemConnected)					
					unGenericObject_ColorBoxAnimate("Alarm_LL", false, false, "", false);////////
					break;
				case  UN_AIAO_ALARM_3_H_L:
				 if(g_bSystemConnected)	
					unGenericObject_ColorBoxAnimateAlarm("Alarm_HH", 0, false, false);
				 if(g_bSystemConnected)					
					unGenericObject_ColorBoxAnimateAlarm("Alarm_LL", 0, false, false);
				 if(g_bSystemConnected)
					unGenericObject_ColorBoxAnimate("Alarm_H", false, false, "", false);
				 if(g_bSystemConnected)					
					unGenericObject_ColorBoxAnimate("Alarm_L", false, false, "", false);					
					break;
				case  UN_AIAO_ALARM_3_HH_H:
				 if(g_bSystemConnected)
					unGenericObject_ColorBoxAnimateAlarm("Alarm_L", 0, false, false);
				 if(g_bSystemConnected)					
					unGenericObject_ColorBoxAnimateAlarm("Alarm_LL", 0, false, false);
				 if(g_bSystemConnected)
					unGenericObject_ColorBoxAnimate("Alarm_H", false, false, "", false);
				 if(g_bSystemConnected)					
					unGenericObject_ColorBoxAnimate("Alarm_HH", false, false, "", false);					
					break;
				case  UN_AIAO_ALARM_3_LL_L:
				 if(g_bSystemConnected)
					unGenericObject_ColorBoxAnimateAlarm("Alarm_H", 0, false, false);
				 if(g_bSystemConnected)					
					unGenericObject_ColorBoxAnimateAlarm("Alarm_HH", 0, false, false);
				 if(g_bSystemConnected)
					unGenericObject_ColorBoxAnimate("Alarm_L", false, false, "", false);
				 if(g_bSystemConnected)					
					unGenericObject_ColorBoxAnimate("Alarm_LL", false, false, "", false);					
					break;
				case  UN_AIAO_ALARM_2_H:
				 if(g_bSystemConnected)
					unGenericObject_ColorBoxAnimateAlarm("Alarm_HH", 0, false, false);
				 if(g_bSystemConnected)					
					unGenericObject_ColorBoxAnimateAlarm("Alarm_L", 0, false, false);
				 if(g_bSystemConnected)					
					unGenericObject_ColorBoxAnimateAlarm("Alarm_LL", 0, false, false);
				 if(g_bSystemConnected)
					unGenericObject_ColorBoxAnimate("Alarm_H", false, false, "", false);
					break;
				case  UN_AIAO_ALARM_2_L:
				 if(g_bSystemConnected)
					unGenericObject_ColorBoxAnimateAlarm("Alarm_HH", 0, false, false);
				 if(g_bSystemConnected)					
					unGenericObject_ColorBoxAnimateAlarm("Alarm_H", 0, false, false);
				 if(g_bSystemConnected)					
					unGenericObject_ColorBoxAnimateAlarm("Alarm_LL", 0, false, false);
				 if(g_bSystemConnected)
					unGenericObject_ColorBoxAnimate("Alarm_L", false, false, "", false);
					break;
				default:
					break;
			}
		}
	 if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("Alarm_UnAck", bAckPossible, bIsAlarmActive, "unFaceplate_AlarmNotAck", false);
	
		unProcessAlarm_isDpConfigured(substr(sDpPosSt,0,strpos(sDpPosSt,":_online.._value")),DPCONFIG_ALERT_NONBINARYSIGNAL,g_iAlertRange,isDpConfigured);					
	 if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("MailSMS", dynContains(isDpConfigured, 1)>0, bIsAlarmActive,"unFaceplate_OperationModeActive", bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));	

	}
// 3.2. Alarm doesn't exist
	else
	{
	 if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimateAlarm("Alarm_HH", 0, false, false);
	 if(g_bSystemConnected)		
		unGenericObject_ColorBoxAnimateAlarm("Alarm_H", 0, false, false);
	 if(g_bSystemConnected)		
		unGenericObject_ColorBoxAnimateAlarm("Alarm_L", 0, false, false);
	 if(g_bSystemConnected)		
		unGenericObject_ColorBoxAnimateAlarm("Alarm_LL", 0, false, false);
	 if(g_bSystemConnected)		
		unGenericObject_ColorBoxAnimateAlarm("Alarm_UnAck", 0, false, false);
	 if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimateAlarm("MailSMS", 0, false, false);
	 if(g_bSystemConnected)		
		setMultiValue("ValueHH.display", "text", "", "ValueHH.display", "foreCol", "unAlarmDoesNotExist",
					  "ValueH.display", "text", "", "ValueH.display", "foreCol", "unAlarmDoesNotExist",
					  "ValueL.display", "text", "", "ValueL.display", "foreCol", "unAlarmDoesNotExist",
					  "ValueLL.display", "text", "", "ValueLL.display", "foreCol", "unAlarmDoesNotExist");
	}

// display event state
	if(g_bSystemConnected) {
		unGenericObject_ColorBoxAnimate("eventState", iMask, 1, "", 
				bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value),
				"unFaceplate_OperationModeActive");
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogInput_FaceplateStatusDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogInput_FaceplateStatusDisconnection()
{
	unGenericObject_ColorBoxDisconnect(makeDynString("Alarm_HH", "Alarm_H", "Alarm_UnAck", "Alarm_LL", "Alarm_L",
													 "OperationMode_Auto", "OperationMode_Forced", 
													 "Warning_IOError", "Warning_IOSimulated", "Warning_ForcedProcess","MailSMS"));
	if(g_bAnalogInput)
		unGenericObject_DisplayValueDisconnect(makeDynString("RangeMax", "RangeMin", "Request_Process", "Request_Active",
														 "ValueHH", "ValueH", "ValueL", "ValueLL"));
	else
		unGenericObject_DisplayValueDisconnect(makeDynString("RangeMax", "RangeMin", "Request_Auto", "Request_Active",
														 "ValueHH", "ValueH", "ValueL", "ValueLL"));
// set eventState
	unGenericObject_ColorBoxDisconnect(makeDynString("eventState"));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogInput_ButtonRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in AnalogInput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogInput_ButtonRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, sDpType, sFunction;
	int iAction, iRes, iManReg01;
	dyn_string exceptionInfo, dsFunctions, dsUserAccess;
	
	deviceName = unGenericDpFunctions_getDpName($sDpName);
	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			unAnalogInput_ButtonDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
// Send 0 to ManReg01 if necessary
			iRes = dpGet(deviceName + ".ProcessOutput.ManReg01", iManReg01);
			if ((iRes < 0) || (iManReg01 != 0))
			{
				dpSet(deviceName + ".ProcessOutput.ManReg01", 0);
			}
// call the unAnalogInput_ButtonUserAccess to get the list of allowed buttons.
//			unAnalogInput_ButtonUserAccess(deviceName, "AnalogInput", g_dsUserAccess);


			sDpType="AnalogInput";
			unGenericObject_GetFunctions(sDpType, dsFunctions, exceptionInfo);
			sFunction = dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS];
						 
			if (sFunction != "")
				{
				evalScript(dsUserAccess, "dyn_string main(string deviceNameTemp, string sDpTypeTemp) {" + 
							 "dyn_string dsUserAccessReturn;" +
							 "if (isFunctionDefined(\"" + sFunction  + "\"))" +
							 "    {" +
					 			sFunction  + "(deviceNameTemp, sDpTypeTemp, dsUserAccessReturn);" +
							 "    }" +
							 "return dsUserAccessReturn; }", makeDynString(), deviceName, sDpType);
						
				g_dsUserAccess=dsUserAccess;	
				}
			else
				fwException_raise(exceptionInfo,"ERROR", "unAnalogInput_ButtonRegisterCB:" + getCatStr("unGeneration","UNKNOWNFUNCTION"), "EMPTY");


			iRes = dpConnect("unAnalogInput_ButtonAnimationCB",
							 deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							 deviceName + ".statusInformation.selectedManager",
							 deviceName + ".ProcessInput.StsReg01",
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid");
			g_bCallbackConnected = (iRes >= 0);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			iRes = dpDisconnect("unAnalogInput_ButtonAnimationCB",
							 deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							 deviceName + ".statusInformation.selectedManager",
							 deviceName + ".ProcessInput.StsReg01",
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid");
			g_bCallbackConnected = !(iRes >= 0);
			unAnalogInput_ButtonDisconnection();
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogInput_ButtonAnimationCB
/**
Purpose: animate buttons

Parameters: 
	- dbPermissions, dyn_bool, input, list of permissions, length = 4, order : 1=monitor, 2=operator, 3=expert, 4=admin
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogInput_ButtonAnimationCB(string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager, 
						   		string sDpStsReg01, bit32 stsReg01Value, string sDpStsReg01Bad, bool bStsReg01Bad)
{
	unAnalogInput_ButtonSetState(unGenericDpFunctions_getDpName(sDpStsReg01), "AnalogInput", g_dsUserAccess, 
									makeDynString(bLocked, sSelectedManager, stsReg01Value, bStsReg01Bad, 0, 0));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogInput_ButtonUserAccess
/**
Purpose: User access for AnalogInput buttons

Parameters: 
	- sDpName, string, input, device name
	- sDpType, string, input, dptype
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogInput_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
	bool operator, expert, admin;
	int i, length;
	dyn_bool dbPermissions;
	dyn_string exceptionInfo;
        string sOperAction, sExpAction, sAdminAction;
        bool bActionDefined = false;

        unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo);
        if((sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME) ||
           (sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME))
          bActionDefined=true;
	operator = dbPermissions[2];
	expert = dbPermissions[3];
	admin = dbPermissions[4];
	dsAccess = makeDynString();
	if (operator)
	{
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
          else
	    dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_SET_VALUE, UN_FACEPLATE_BUTTON_MASK_ALARM,
							UN_FACEPLATE_BUTTON_ACK_ALARM, UN_FACEPLATE_BUTTON_SELECT));
	}
	if (expert)
	{
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
          else
	    dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_FORCED_MODE, UN_FACEPLATE_BUTTON_SET_ALARM_LIMITS, UN_FACEPLATE_MASKEVENT));
	}
        if(admin){
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
        }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogInput_UserLoginGetButtonState
/**
Purpose: function called before the button set state when a user logs in. 

Parameters:
	deviceName: string, input, the device name
	dsReturnData: dyn_string, output, the data are returned here

Usage: Internal function

PVSS manager usage:  Ctrl, NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
		-> $sDpName, string, the device DP name
		-> g_sFaceplateButtonType, string, faceplate type
	. PVSS version: 2.12.1 
	. operating system: NT, W2000 and Linux, but tested only under W2000.
	. distributed system: yes but not tested.
*/

unAnalogInput_UserLoginGetButtonState(string deviceName, string sType, dyn_string &dsReturnData)
{
	unGenericObject_UserCBGetButtonState(deviceName, sType, dsReturnData);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogInput_ButtonSetState
/**
Purpose: set button state

Parameters: 
	- deviceName, string, input, device
	- sDpType, string, input, dptype
	- dsUserAccess, dyn_string, input, user access
	- dsData, dyn_string, parameter
		
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogInput_ButtonSetState(string deviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
	bool bSelected, buttonEnabled, bit1, bit2;
	dyn_string dsButtons;
	int i, length, alertType;
	string localManager;
	bool bLocked, bStsReg01Bad, bStsReg02Bad;
	string sSelectedManager;
	bit32 stsReg01Value, stsReg02Value; 

	bLocked = dsData[1];
	sSelectedManager = dsData[2];
	stsReg01Value = dsData[3];
	bStsReg01Bad = dsData[4];
	stsReg02Value = dsData[5];
	bStsReg02Bad = dsData[6];

//DebugN("AnalogInput_SetState", bLocked, sSelectedManager, stsReg01Value, bStsReg01Bad, stsReg02Value, bStsReg02Bad);

	dsButtons = makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_SET_VALUE, UN_FACEPLATE_BUTTON_ACK_ALARM, 
							  UN_FACEPLATE_BUTTON_SET_ALARM_LIMITS, UN_FACEPLATE_BUTTON_FORCED_MODE); // Each button except "Select"
	length = dynlen(dsButtons);
// 1. Selection state
	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
	bSelected = (localManager == "S");
// 2. Button state
	for(i=1;i<=length;i++)
		{
		buttonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0);	// User access
		switch (dsButtons[i])
			{
			case UN_FACEPLATE_BUTTON_FORCED_MODE:
				bit1 = !(getBit(stsReg01Value, UN_STSREG01_FOMOST) == 0) && !(bStsReg01Bad || bStsReg02Bad);	
				bit2 = !(getBit(stsReg01Value, UN_STSREG01_AUINHFMO) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				buttonEnabled = buttonEnabled && bSelected && !(bit1 || bit2);
				break;
			case UN_FACEPLATE_BUTTON_AUTO_MODE:
			case UN_FACEPLATE_BUTTON_SET_VALUE:
				bit1 = !(getBit(stsReg01Value, UN_STSREG01_AUMOST) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				buttonEnabled = buttonEnabled && bSelected && !(bit1);
				break;
			case UN_FACEPLATE_BUTTON_SET_ALARM_LIMITS:
			case UN_FACEPLATE_BUTTON_ACK_ALARM:
			default:
				break;
			}
		setValue(UN_FACEPLATE_BUTTON_PREFIX + dsButtons[i], "enabled", buttonEnabled);
		}
// 3. Button Select state
	unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogInput_ButtonDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogInput_ButtonDisconnection()
{
	unGenericObject_ButtonDisconnect(makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_SET_VALUE,  
												   UN_FACEPLATE_BUTTON_ACK_ALARM, UN_FACEPLATE_BUTTON_SET_ALARM_LIMITS, 
												   UN_FACEPLATE_BUTTON_FORCED_MODE, UN_FACEPLATE_BUTTON_SELECT));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogInput_WidgetRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in AnalogInput widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogInput_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, deviceType, sSystemName, sPlcName;
	int iAction, iRes;
	dyn_string exceptionInfo;
	bool bRemote;
	
	deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
	if (deviceName == "")	// In case of disconnection
		{
		deviceName = g_sDpName;
		}

// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	
	//////////////// 
	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;
	////////////////
	if(sPlcName != "") {
		if (dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;

	//  check if the device name is of type "AnalogInput" or "AnalogOutput" if not like of no PLC. -> not available.
  if(deviceName!="")
    {    
    deviceType = dpTypeName(deviceName);
	  if((deviceType != "AnalogInput") && (deviceType != "AnalogOutput"))        
      g_bUnSystemAlarmPlc = false;
    }
                            		
	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			unAnalogInput_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPCONNECT:
//DebugN("unAnalogInput_WidgetRegisterCB: connect");

			g_sDpName = deviceName;
			g_bDisconnectCB = false;

		if(g_bUnSystemAlarmPlc) {
				iRes = dpConnect("unAnalogInput_WidgetLockCallBack", deviceName + ".ProcessInput.PosSt:_lock._alert_hdl._locked");
				g_bCallbackConnected = (iRes >= 0);
			}
			else
					unAnalogInput_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
//DebugN("unAnalogInput_WidgetRegisterCB: disconnect");
			if(g_bUnSystemAlarmPlc) 
				iRes = dpDisconnect("unAnalogInput_WidgetLockCallBack", deviceName + ".ProcessInput.PosSt:_lock._alert_hdl._locked");

			if(g_bDisconnectCB) {
				switch(g_sWidgetType) {
					case "AIAO":
						unAnalogInput_AIAO_WidgetDisconnect(g_sDpName, iRes);
						break;
					case "bargraph":
					case "bargraphSimple":
						unGenericWidget_bargraph_WidgetDisconnect(g_sDpName, iRes);
						break;
					case "step_PLC":
						unAnalogInput_step_PLC_WidgetDisconnect(g_sDpName, iRes);
						break;
					case "transition_PLC":
						unAnalogInput_transition_PLC_WidgetDisconnect(g_sDpName, iRes);
						break;
					default:
						break;
				}
			}
			g_bCallbackConnected = !(iRes >= 0);
			g_bDisconnectCB = false;

			unAnalogInput_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------
// unAnalogInput_WidgetLockCallBack
/**
Purpose: set up the lock callback function for the widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in AnalogInput widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unAnalogInput_WidgetLockCallBack(string sdpe, bool bLockState)
{
	int iRes;

	if(bLockState) { // lock set so disconnect if not already disconnected
// no need to check if it was already set before because it is not possible to do 2 dpSet on the lock the Ev reject the message
		if(g_bDisconnectCB) {
			switch(g_sWidgetType) {
				case "AIAO":
					unAnalogInput_AIAO_WidgetDisconnect(g_sDpName, iRes);
					break;
				case "bargraph":
				case "bargraphSimple":
					unGenericWidget_bargraph_WidgetDisconnect(g_sDpName, iRes);
					break;
				case "step_PLC":
					unAnalogInput_step_PLC_WidgetDisconnect(g_sDpName, iRes);
					break;
				case "transition_PLC":
					unAnalogInput_transition_PLC_WidgetDisconnect(g_sDpName, iRes);
					break;
				default:
					break;
			}
//DebugN("unAnalogInput_WidgetLockCallBack: disconnect");
			g_bDisconnectCB = false;
		}
		unAnalogInput_WidgetDisconnection(g_sWidgetType);
                setValue("WidgetArea", "visible", true);
	}
	else { // lock not set so connect if not already connected
// no need to check if it was already set before because it is not possible to do 2 dpSet on the lock the Ev reject the message
		g_bDisconnectCB = true;

		switch(g_sWidgetType) {
			case "AIAO":
				unAnalogInput_AIAO_WidgetConnect(g_sDpName);
				break;
			case "bargraph":
			case "bargraphSimple":
				unGenericWidget_bargraph_WidgetConnect(g_sDpName);
				break;
			case "step_PLC":
				unAnalogInput_step_PLC_WidgetConnect(g_sDpName);
				break;
			case "transition_PLC":
				unAnalogInput_transition_PLC_WidgetConnect(g_sDpName);
				break;
			default:
				break;
		}

	}
}

//---------------------------------------------------------------------------------------------------------------------------------------
// unAnalogInput_AIAO_WidgetConnect
/**
Purpose: Connect function of the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in AnalogInput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unAnalogInput_AIAO_WidgetConnect(string deviceName)
{
	int iRes;
	dyn_string exceptionInfo;
	string sPlcName, sSystemName;
		
// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	if(sPlcName != "") {
		if (dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;
		
	unGenericObject_CheckAnalogAlarmType(deviceName + ".ProcessInput.PosSt", g_iAlertType, exceptionInfo);

	g_sPosStFormat = dpGetFormat(deviceName + ".ProcessInput.PosSt");
	g_sPosStUnit = dpGetUnit(deviceName + ".ProcessInput.PosSt");

	if(g_bUnSystemAlarmPlc){
		if (g_iAlertType == DPCONFIG_ALERT_NONBINARYSIGNAL)
		{
			iRes = dpConnect("unAnalogInput_WidgetAnimationCB", 
								deviceName + ".statusInformation.selectedManager:_lock._original._locked",
								deviceName + ".statusInformation.selectedManager",
								deviceName + ".ProcessInput.StsReg01",
								deviceName + ".ProcessInput.StsReg01:_online.._invalid",
								deviceName + ".ProcessInput.PosSt",
								deviceName + ".ProcessInput.PosSt:_online.._invalid",
								deviceName + ".ProcessInput.HFStOrAuPosRSt:_online.._invalid",
								deviceName + ".ProcessInput.PosSt:_alert_hdl.._active",
								deviceName + ".ProcessInput.PosSt:_alert_hdl.._act_range",
								deviceName + ".ProcessInput.PosSt:_alert_hdl.._ack_possible",
								sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
								sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
								deviceName + ".eventMask");
		}
		else
		{
			iRes = dpConnect("unAnalogInput_WidgetAnimationCB_NoAlarm", 
							deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							deviceName + ".statusInformation.selectedManager",
							deviceName + ".ProcessInput.StsReg01",
							deviceName + ".ProcessInput.StsReg01:_online.._invalid",
							deviceName + ".ProcessInput.PosSt",
							deviceName + ".ProcessInput.PosSt:_online.._invalid",
							deviceName + ".ProcessInput.HFStOrAuPosRSt:_online.._invalid",
							sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							deviceName + ".eventMask");
		}
	}
	else
		unAnalogInput_WidgetDisconnection(g_sWidgetType);

}
//---------------------------------------------------------------------------------------------------------------------------------------
// unAnalogInput_AIAO_WidgetDisconnect
/**
Purpose: Disconnect function of the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in AnalogInput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unAnalogInput_AIAO_WidgetDisconnect(string deviceName, int &iRes)
{
	string sPlcName, sSystemName;
		
	if(g_bUnSystemAlarmPlc) {
// get the systemName
		sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
		sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);

		if (g_iAlertType == DPCONFIG_ALERT_NONBINARYSIGNAL)
		{
			iRes = dpDisconnect("unAnalogInput_WidgetAnimationCB", 
						deviceName + ".statusInformation.selectedManager:_lock._original._locked",
						deviceName + ".statusInformation.selectedManager",
						deviceName + ".ProcessInput.StsReg01",
						deviceName + ".ProcessInput.StsReg01:_online.._invalid",
						deviceName + ".ProcessInput.PosSt",
						deviceName + ".ProcessInput.PosSt:_online.._invalid",
						deviceName + ".ProcessInput.HFStOrAuPosRSt:_online.._invalid",
						deviceName + ".ProcessInput.PosSt:_alert_hdl.._active",
						deviceName + ".ProcessInput.PosSt:_alert_hdl.._act_range",
						deviceName + ".ProcessInput.PosSt:_alert_hdl.._ack_possible",
						sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
						sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
						deviceName + ".eventMask");
		}
		else
		{
			iRes = dpDisconnect("unAnalogInput_WidgetAnimationCB_NoAlarm", 
						deviceName + ".statusInformation.selectedManager:_lock._original._locked",
						deviceName + ".statusInformation.selectedManager",
						deviceName + ".ProcessInput.StsReg01",
						deviceName + ".ProcessInput.StsReg01:_online.._invalid",
						deviceName + ".ProcessInput.PosSt",
						deviceName + ".ProcessInput.PosSt:_online.._invalid",
						deviceName + ".ProcessInput.HFStOrAuPosRSt:_online.._invalid",
						sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
						sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
						deviceName + ".eventMask");
		}	
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogInput_WidgetAnimationCB_NoAlarm
/**
Purpose: Animate widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in AnalogInput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogInput_WidgetAnimationCB_NoAlarm(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
							   		    string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid,
								        string sDpPosSt, float fPosSt, string sDp6, bool bPosStInvalid,
								        string sDp7, bool bHFStOrAuPosRStInvalid,
										string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled, string sEvtMask, int iMask)
{
	unAnalogInput_WidgetAnimationCB(sDpLock, bLocked, sDpManager, sSelectedManager, sDpStsReg01, bit32StsReg01, sDp4, bStsReg01Invalid,
									sDpPosSt, fPosSt, sDp6, bPosStInvalid, sDp7, bHFStOrAuPosRStInvalid,
									"", 0, "", 0, "", 0, sDpAl, iAlValue, sDpEnabled, bPlcEnabled, sEvtMask, iMask);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogInput_WidgetAnimationCB
/**
Purpose: Animate widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in AnalogInput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogInput_WidgetAnimationCB(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
						   		string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid,
							    string sDpPosSt, float fPosSt, string sDp6, bool bPosStInvalid,
							    string sDp7, bool bHFStOrAuPosRStInvalid,
							    string sDp10, bool bIsAlarmActive, string sDp11, int iActRange, string sDp12, bool bAckPossible,
								string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled, string sEvtMask, int iMask)
{
	unGenericObject_WidgetAIAOAnimation(bLocked, sSelectedManager, bit32StsReg01, bStsReg01Invalid,
										g_sPosStFormat, g_sPosStUnit, fPosSt,
										g_iAlertType, bIsAlarmActive, iActRange, bAckPossible, 
										bPosStInvalid, bStsReg01Invalid || bPosStInvalid || bHFStOrAuPosRStInvalid, 
										g_iAlertRange, iAlValue, bPlcEnabled);

// display event state
	if(g_bSystemConnected) {
		if(iMask == 0)
			eventState.visible	= true;
		else
			eventState.visible	= false;
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogInput_WidgetDisconnection
/**
Purpose: Animate widget disconnection

Parameters:
	- sWidgetType, string, input, widget type. ex: "heater"

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unAnalogInput_WidgetDisconnection(string sWidgetType)
{
	switch(sWidgetType) {
		case "AIAO":
			unGenericObject_WidgetDisconnection(sWidgetType, "AnalogInput");
	// set event state
			eventState.visible	= false;
			break;
		case "bargraph":
		case "bargraphSimple":
			unGenericWidget_bargraph_disconnection();
			break;
		case "step_PLC":
			unAnalogInput_step_PLC_disconnection();
			break;
		case "transition_PLC":
			unAnalogInput_transition_PLC_disconnection();
			break;
		default:
			break;
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------
// unAnalogInput_step_PLC_WidgetConnect
/**
Purpose: Animate Step widget connection

Parameters:
	- deviceName, string, input, device Name to connect

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogInput_step_PLC_WidgetConnect(string deviceName)
{
	int iRes, iAlertType1, iRangeType;
	dyn_string exceptionInfo;
	string sPlcName, sSystemName;
	
//DebugN("unAnalogInput_step_PLC_WidgetConnect: " + deviceName);
		
// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	if(sPlcName != "") {
		if (dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName)) 
			{
			g_bUnSystemAlarmPlc = true;
			iRes = dpConnect("unAnalogInput_step_PLC_CB", deviceName + ".ProcessInput.PosSt",
												sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
												sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled");
			}
		else 
			{
			g_bUnSystemAlarmPlc = false;
			unAnalogInput_step_PLC_disconnection();
			}
		}	
	else 
		{
		g_bUnSystemAlarmPlc = false;
		unAnalogInput_step_PLC_disconnection();
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------
// unAnalogInput_step_PLC_CB
/**
Purpose: callback function for step PLC

Parameters:
	- sDp1, string, input, datapoint nane
	- fPosSt, float, input, value of datapoint
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogInput_step_PLC_CB (string sDp1, int iPosSt,
			 string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled)
{
string sColor;

	if ((!bPlcEnabled) || (iAlValue > c_unSystemIntegrity_no_alarm_value))
		sColor = "unDataNotValid";
	else {
		if (iPosSt==g_iValue)
			sColor=g_sColor_Active;
		else
			sColor=g_sColor_Passive;
	}
	if(g_bSystemConnected)
	setValue("step_Plc","backCol",sColor); 
}


//---------------------------------------------------------------------------------------------------------------------------------------
// unAnalogInput_step_PLC_WidgetDisconnect
/**
Purpose: Animate Step widget disconnect

Parameters:
	- deviceName, string, Device Name to disconnect
	- iRes, int, output, Disconnection result

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unAnalogInput_step_PLC_WidgetDisconnect(string deviceName, int &iRes)
{
string sPlcName, sSystemName;

//DebugN("ds unAnalogInput_step_PLC_WidgetDisconnect");
	if(g_bUnSystemAlarmPlc) {
// get the systemName
		sSystemName = unGenericDpFunctions_getSystemName(deviceName);
		// get the PLC name 
		sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
		iRes = dpDisconnect("unAnalogInput_step_PLC_CB", deviceName + ".ProcessInput.PosSt",
												sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
												sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled");
		}		
}

//---------------------------------------------------------------------------------------------------------------------------------------
// unAnalogInput_step_PLC_disconnection
/**
Purpose: Animate Step widget disconnection

Parameters:

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogInput_step_PLC_disconnection()
{
//DebugN("ds unAnalogInput_step_PLC_disconnection");
	step_Plc.backCol = "unDataNoAccess";	
}

//---------------------------------------------------------------------------------------------------------------------------------------
// unAnalogInput_transition_PLC_WidgetConnect
/**
Purpose: Animate Transition widget connection

Parameters:
	- iRes, string, input, device Name to connect

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogInput_transition_PLC_WidgetConnect(string deviceName)
{
int iRes, iAlertType1, iRangeType;
dyn_string exceptionInfo;
string sPlcName, sSystemName;
	
//DebugN("unAnalogInput_transition_PLC_WidgetConnect: " + deviceName);
		
// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	
	if (g_iBit > UN_TRANSITION_PLC_MAX_POSITION)
		sPlcName="";
		
	if (sPlcName != "") {
		if (dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName)) 
			{
			g_bUnSystemAlarmPlc = true;
			iRes = dpConnect("unAnalogInput_transition_PLC_CB", deviceName + ".ProcessInput.PosSt",
												sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
												sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled");
			}
		else 
			{
			g_bUnSystemAlarmPlc = false;
			unAnalogInput_transition_PLC_disconnection();
			}
		}
	else {
		g_bUnSystemAlarmPlc = false;
		unAnalogInput_transition_PLC_disconnection();
		}
				
}

//---------------------------------------------------------------------------------------------------------------------------------------
// unAnalogInput_transition_PLC_CB
/**
Purpose: callback function for step PLC

Parameters:
	- sDp1, string, input, datapoint nane
	- iPosSt, iloat, input, value of datapoint (int to convert in bit32)
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogInput_transition_PLC_CB (string sDp1, int iPosSt,
			 string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled)
{
	string sColor;
	bit32 bitPost;
	
	bitPost=iPosSt;

	if ((!bPlcEnabled) || (iAlValue > c_unSystemIntegrity_no_alarm_value))
		sColor = "unDataNotValid";
	else {
		if (getBit(bitPost,g_iBit))
			sColor=g_sColor_Active;
		else
			sColor=g_sColor_Passive;
	}
	if(g_bSystemConnected)
	setValue("line_transition","foreCol",sColor); 
}
 
//---------------------------------------------------------------------------------------------------------------------------------------
// unAnalogInput_transition_PLC_WidgetDisconnect
/**
Purpose: Animate Transition widget disconnect

Parameters:
	- deviceName, string, input, device name to disconnect
	- iRes, int, output, Disconnection result

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogInput_transition_PLC_WidgetDisconnect(string deviceName, int &iRes)
{
string sPlcName, sSystemName;

//DebugN("ds unAnalogInput_transition_PLC_WidgetDisconnect");		
	
	if(g_bUnSystemAlarmPlc) {
// get the systemName
		sSystemName = unGenericDpFunctions_getSystemName(deviceName);
		// get the PLC name 
		sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
		iRes = dpDisconnect("unAnalogInput_transition_PLC_CB", deviceName + ".ProcessInput.PosSt",
												sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
												sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled");
		}
		
}

//---------------------------------------------------------------------------------------------------------------------------------------
// unAnalogInput_transition_PLC_disconnection
/**
Purpose: Animate Transition widget disconnection

Parameters:

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogInput_transition_PLC_disconnection()
{
//DebugN("ds unAnalogInput_transition_PLC_disconnection");		
	line_transition.foreCol = "unDataNoAccess";
}

//@}


