/**@name LIBRARY: unAnalogOutput.ctl

@author: Vincent Forest (AB-CO)

Creation Date: 21 05 2003

Modification History: 
	28/04/2004: Herve
		- add function unAnalogOutput_ObjectListGetValueTime

	14/04/2004: Herve
		- add the function unAnalogOutput_HandleMenu
		- add the function unAnalogOutput_UserLoginGetButtonState

	13/04/2004: Herve
		- modification of the signature unAnalogOutput_MenuConfiguration.

	08/04/2004: Herve 
		modification of the signature of the function unAnalogOutput_ButtonSetState

	12/11/2003: Herve
		for the faceplate CB function, call the analogInput function.
		
	06/10/2003: Herve
		add dpConnect on the _lock._alert_hdl._locked in order to detect when the alert_hdl config modified.

version 1.0

External Function : 
	. unAnalogOutput_AcknowledgeAlarm : acknowledge AnalogOutput alarms
	. unAnalogOutput_FaceplateStatusRegisterCB : register callback function
	. unAnalogInput_FaceplateStatusDisconnection : animate faceplate disconnection
	. unAnalogOutput_ButtonRegisterCB : register callback function
	. unAnalogOutput_ButtonSetState : set button state
	. unAnalogOutput_ButtonDisconnection : animate buttons disconnection
	. unAnalogOutput_ButtonUserAccess : User access for AnalogOutput buttons
	. unAnalogOutput_WidgetRegisterCB : register callback function
	. unAnalogOutput_WidgetDisconnection : animate widget disconnection
	. unAnalogOutput_MenuConfiguration : configuration of popup menu

Internal Functions :

Purpose: This library contains AnalogOutput object functions.

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. AnalogOutput object (DPType etc.)
	. Global variables and $parameters defined in AnalogOutput faceplate and widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

//@{

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogOutput_ObjectListGetValueTime
/**
Purpose: Function called from the objectList to return the time and value

Parameters:
	- sDeviceName, string, input, device name
	- sDeviceType, string, input, unicos object
	- dsReturnData, dyn_string, output, return value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogOutput_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
	unAnalogInput_ObjectListGetValueTime(sDeviceName, sDeviceType, dsReturnData);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogOutput_AcknowledgeAlarm
/**
Purpose: acknowledge AnalogOutput alarms

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsNeedAck, dyn_string, output, DPEs names that need acknowledgement. 
									 The PLC acknowledgement is specified using UN_ACKNOWLEDGE_PLC string
	
Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in AnalogOutput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogOutput_AcknowledgeAlarm(string sDpName, string sDpType, dyn_string &dsNeedAck)
{
	unAnalogInput_AcknowledgeAlarm(sDpName, sDpType, dsNeedAck);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogOutput_MenuConfiguration
/**
Purpose: configuration of popup menu

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsAccessOk, dyn_string, input, authorized action
	- menuList, dyn_string, output, menu to pop-up
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogOutput_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{
	unAnalogInput_MenuConfiguration(sDpName, sDpType, dsAccessOk, menuList);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogOutput_HandleMenu
/**
Purpose: handle the answer of the popup menu

Parameters:
	- deviceName, string, input, the device dp name
	- sDpType, string, input, the device dp type
	- menuList, dyn_string, input, list of requested action
	- menuAnswer, int, input, selected action from the menuList
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unAnalogOutput_HandleMenu(string deviceName, string sDpType, dyn_string menuList, int menuAnswer)
{
	unAnalogInput_HandleMenu(deviceName, sDpType, menuList, menuAnswer);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogOutput_FaceplateStatusRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in AnalogOutput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogOutput_FaceplateStatusRegisterCB(string sDp, bool bSystemConnected)
{
	unAnalogInput_FaceplateStatusRegisterCB(sDp, bSystemConnected);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogOutput_FaceplateStatusDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogOutput_FaceplateStatusDisconnection()
{
	unAnalogInput_FaceplateStatusDisconnection();
}
//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogOutput_ButtonRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in AnalogOutput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogOutput_ButtonRegisterCB(string sDp, bool bSystemConnected)
{
	unAnalogInput_ButtonRegisterCB(sDp, bSystemConnected);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogOutput_ButtonUserAccess
/**
Purpose: User access for AnalogOutput buttons

Parameters: 
	- sDpName, string, input, device name
	- sDpType, string, input, dptype
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogOutput_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
	unAnalogInput_ButtonUserAccess(sDpName, sDpType, dsAccess);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogOutput_UserLoginGetButtonState
/**
Purpose: function called before the button set state when a user logs in. 

Parameters:
	deviceName: string, input, the device name
	dsReturnData: dyn_string, output, the data are returned here

Usage: Internal function

PVSS manager usage:  Ctrl, NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
		-> $sDpName, string, the device DP name
		-> g_sFaceplateButtonType, string, faceplate type
	. PVSS version: 2.12.1 
	. operating system: NT, W2000 and Linux, but tested only under W2000.
	. distributed system: yes but not tested.
*/

unAnalogOutput_UserLoginGetButtonState(string deviceName, string sType, dyn_string &dsReturnData)
{
	unAnalogInput_UserLoginGetButtonState(deviceName, sType, dsReturnData);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogOutput_ButtonSetState
/**
Purpose: set button state

Parameters: 
	- deviceName, string, input, device
	- sDpType, string, input, dptype
	- dsUserAccess, dyn_string, input, user access
	- dsData, dyn_string, parameter
		
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogOutput_ButtonSetState(string deviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
	unAnalogInput_ButtonSetState(deviceName, sDpType, dsUserAccess, dsData);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogOutput_ButtonDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogOutput_ButtonDisconnection()
{
	unAnalogInput_ButtonDisconnection();
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogOutput_WidgetRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in AnalogOutput widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogOutput_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
	unAnalogInput_WidgetRegisterCB(sDp, bSystemConnected);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unAnalogOutput_WidgetDisconnection
/**
Purpose: Animate widget disconnection

Parameters:
	- sWidgetType, string, input, widget type. ex: "heater"

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unAnalogOutput_WidgetDisconnection(string sWidgetType)
{
	unGenericObject_WidgetDisconnection(sWidgetType, "AnalogOutput");
}
//@}

