/**@file

// unCPCExportDevice.ctl
This library contains the export functions of the package.

@par Creation Date
  02/05/2011

@par Modification History
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@author 
  H. Milcent (EN-ICE)
*/

//@{

//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_AnalogInput_ExportConfig
/**
Purpose: Export AnalogInput Devices for _UnPlc front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void _UnPlc_AnalogInput_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
dyn_string dsAllCommonParameters, dsSmoothDpParameters, dsLimits , dsDpParameters, dsArchiveNames;
string sObject, sUnit, sFormatToConvert, sFormatConverted, sArchiveActive, sMin, sMax, sArchiveName;
float fMinValue, fMaxValue, fDeadband, fDeadbandTemp, fTimeInterval, fArchiveTimeFilter ;
int iSmoothProcedure, iDeadBandType, i, j, iNormalPosition, iLeni, iLenj;
bool bDoesExist, bNegateRange, bIgnoreOutside,  bInclusiveMin , bInclusiveMax , bIsSmoothDefined ;
string sVersion, sMask;
int iMask;

		sVersion = frontEndVersion.text();
		sObject="AnalogInput";
		g_iAlertRange = 0;
		
		iLeni=dynlen(dsDpList);
		for(i=1; i<=iLeni; i++)
			{									
			if (dpExists(dsDpList[i]))
				{
				dynClear(dsAllCommonParameters);
				dynClear(dsSmoothDpParameters);
				dynClear(dsLimits);		
				dynClear(dsDpParameters);
				dynClear(dsArchiveNames);
					
				//Get all common Parameters
				unExportDevice_getAllCommonParameters(dsDpList[i], sObject ,  dsAllCommonParameters);
				dsDpParameters = dsAllCommonParameters;
						
				//Get DP unit
				sUnit =  dpGetUnit(dsDpList[i] + ".ProcessInput.PosSt" );
				
				//Get Format
				sFormatToConvert = dpGetFormat(dsDpList[i] + ".ProcessInput.PosSt");
				sFormatConverted = unExportDevice_convertFormat(sFormatToConvert, "#");
				
				//Get RangeMax and RangeMin
				fwPvRange_get( dsDpList[i] + ".ProcessInput.PosSt", bDoesExist , fMinValue , fMaxValue , bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax , exceptionInfo);
				
				sMax = unExportDevice_floatFormat(fMaxValue);			
				sMin = unExportDevice_floatFormat(fMinValue);
				
				//Get smoothProcedure, fDeadband & timeInterval
				unExportDevice_fwSmoothing_getTemp(dsDpList[i] + ".ProcessInput.PosSt", bIsSmoothDefined, iSmoothProcedure, fDeadbandTemp, fTimeInterval);
				
				//for Max and Min: force convert from string to float
				//Get unicos DeadBand
				unExportDevice_getDeadBand(fDeadbandTemp, bIsSmoothDefined, sMax, sMin, iSmoothProcedure, fDeadband, iDeadBandType );	
								
				//Get Limits
				unExportDevice_getLimits(dsDpList[i], dsLimits, iNormalPosition, sMax, sMin);
				
				//Get Archive Parameters
				unExportDevice_getArchiveParameters(dsDpList[i], sObject, sArchiveActive, fArchiveTimeFilter);
											
				//Get Archive Names
				unCPCExportDevice_getArchiveNames(dsDpList[i], sObject, dsArchiveNames);
				
				//Get address config		
				unCPCExportDevice_getAddressConfig(dsDpList[i], sObject, dsSmoothDpParameters );
		
				dynAppend(dsDpParameters, sUnit);
				dynAppend(dsDpParameters, sFormatConverted);
				dynAppend(dsDpParameters, sMax);
				dynAppend(dsDpParameters, sMin);
								
				dynAppend(dsDpParameters, dsLimits[1]);
				dynAppend(dsDpParameters, dsLimits[2]);
				dynAppend(dsDpParameters, dsLimits[3]);
				dynAppend(dsDpParameters, dsLimits[4]);
								
				if (iDeadBandType==DPCONFIG_NONE)
					dynAppend(dsDpParameters, "0");
				else
					dynAppend(dsDpParameters, fDeadband);
				
				dynAppend(dsDpParameters, iDeadBandType);
				dynAppend(dsDpParameters, sArchiveActive);
				
				if(fArchiveTimeFilter < 0)
					dynAppend(dsDpParameters, "");
				else	
					dynAppend(dsDpParameters, fArchiveTimeFilter);
				
				dynAppend(dsDpParameters, iNormalPosition);
				
				iLenj=dynlen(dsSmoothDpParameters);
				for(j=1; j<=iLenj ; j++)
					{
					dynAppend(dsDpParameters, dsSmoothDpParameters[j]);
					}		
				
				iLenj=dynlen(dsArchiveNames);
				for(j=1; j<=iLenj ; j++)
					{
					dynAppend(dsDpParameters, dsArchiveNames[j]);
					}	
				if(sVersion != "") {
					unGenericDpFunctions_getMaskEvent(dsDpList[i], iMask);
					sprintf(sMask, "%d", iMask);
					dynAppend(dsDpParameters, sMask);
				}
				unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
				}//dp doesn't exist
			}//end for
}

//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_AnalogInput_ExportConfig
/**
Purpose: Export AnalogInput Devices for S7_PLC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void S7_PLC_AnalogInput_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	_UnPlc_AnalogInput_ExportConfig(dsDpList, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_AnalogOutput_ExportConfig
/**
Purpose: Export AnalogOutput Devices for _UnPlc front-end  

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void _UnPlc_AnalogOutput_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
dyn_string dsAllCommonParameters, dsSmoothDpParameters, dsLimits , dsDpParameters, dsArchiveNames;
string sObject, sUnit, sFormatToConvert, sFormatConverted, sArchiveActive, sMin, sMax, sArchiveName;
float fMinValue, fMaxValue, fDeadband, fDeadbandTemp, fTimeInterval, fArchiveTimeFilter ;
int iSmoothProcedure, iDeadBandType, i, j, iNormalPosition, iLeni, iLenj;
bool bDoesExist, bNegateRange, bIgnoreOutside,  bInclusiveMin , bInclusiveMax , bIsSmoothDefined ;
string sVersion, sMask;
int iMask;

		sVersion = frontEndVersion.text();
		sObject="AnalogOutput";
		g_iAlertRange = 0;
		
		iLeni=dynlen(dsDpList);
		for(i=1; i<=iLeni; i++)
			{									
			if (dpExists(dsDpList[i]))
				{
				dynClear(dsAllCommonParameters);
				dynClear(dsSmoothDpParameters);
				dynClear(dsLimits);		
				dynClear(dsDpParameters);
				dynClear(dsArchiveNames);
					
				//Get all common Parameters
				unExportDevice_getAllCommonParameters(dsDpList[i], sObject ,  dsAllCommonParameters);
				dsDpParameters = dsAllCommonParameters;
						
				//Get DP unit
				sUnit =  dpGetUnit(dsDpList[i] + ".ProcessInput.PosSt" );
				
				//Get Format
				sFormatToConvert = dpGetFormat(dsDpList[i] + ".ProcessInput.PosSt");
				sFormatConverted = unExportDevice_convertFormat(sFormatToConvert, "#");
				
				//Get RangeMax and RangeMin
				fwPvRange_get( dsDpList[i] + ".ProcessInput.PosSt", bDoesExist , fMinValue , fMaxValue , bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax , exceptionInfo);
				
				sMax = unExportDevice_floatFormat(fMaxValue);	
				sMin= unExportDevice_floatFormat(fMinValue);
				
				//Get smoothProcedure, fDeadband & timeInterval
				unExportDevice_fwSmoothing_getTemp(dsDpList[i] + ".ProcessInput.PosSt", bIsSmoothDefined, iSmoothProcedure, fDeadbandTemp, fTimeInterval);
				
				//for Max and Min: force convert from string to float
				//Get unicos DeadBand
				unExportDevice_getDeadBand(fDeadbandTemp, bIsSmoothDefined, sMax, sMin, iSmoothProcedure, fDeadband, iDeadBandType );	
								
				//Get Limits
				unExportDevice_getLimits(dsDpList[i], dsLimits, iNormalPosition, sMax, sMin);
				
				//Get Archive Parameters
				unExportDevice_getArchiveParameters(dsDpList[i], sObject, sArchiveActive, fArchiveTimeFilter);
				
				//Get Archive Names
				unCPCExportDevice_getArchiveNames(dsDpList[i], sObject, dsArchiveNames);
											
				//Get address config		
				unCPCExportDevice_getAddressConfig(dsDpList[i], sObject, dsSmoothDpParameters );
		
	
				dynAppend(dsDpParameters, sUnit);
				dynAppend(dsDpParameters, sFormatConverted);
				dynAppend(dsDpParameters, sMax);
				dynAppend(dsDpParameters, sMin);
								
				dynAppend(dsDpParameters, dsLimits[1]);
				dynAppend(dsDpParameters, dsLimits[2]);
				dynAppend(dsDpParameters, dsLimits[3]);
				dynAppend(dsDpParameters, dsLimits[4]);
								
				if (iDeadBandType==DPCONFIG_NONE)
					dynAppend(dsDpParameters, "0");
				else
					dynAppend(dsDpParameters, fDeadband);
				
				dynAppend(dsDpParameters, iDeadBandType);
				dynAppend(dsDpParameters, sArchiveActive);
				
				if(fArchiveTimeFilter < 0)
					dynAppend(dsDpParameters, UN_CONFIG_EXPORT_UNDEF);
				else	
					dynAppend(dsDpParameters, fArchiveTimeFilter);
				
				dynAppend(dsDpParameters, iNormalPosition);
				
				iLenj=dynlen(dsSmoothDpParameters);
				for(j=1; j<=iLenj ; j++)
					{
					dynAppend(dsDpParameters, dsSmoothDpParameters[j]);
					}		
				
				iLenj=dynlen(dsArchiveNames);
				for(j=1; j<=iLenj ; j++)
					{
					dynAppend(dsDpParameters, dsArchiveNames[j]);
					}	
				if(sVersion != "") {
					unGenericDpFunctions_getMaskEvent(dsDpList[i], iMask);
					sprintf(sMask, "%d", iMask);
					dynAppend(dsDpParameters, sMask);
				}
				unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
				}//dp doesn't exist
			}//end for
}

//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_AnalogOutput_ExportConfig
/**
Purpose: Export AnalogOutput Devices for S7_PLC front-end  

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void S7_PLC_AnalogOutput_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	_UnPlc_AnalogOutput_ExportConfig(dsDpList, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_DigitalInput_ExportConfig
/**
Purpose: Export DigitalInput Devices for _UnPlc front-end  

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
_UnPlc_DigitalInput_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
string sDp, sArchiveActive, sObject, sArchiveName;
dyn_string dsAllCommonParameters, smoothDpParameters, dsDpParameters, dsArchiveNames;
float fArchiveTimeFilter ;
int i, j, iNormalPosition, iLeni, iLenj;
string sVersion, sMask;
int iMask;

		sVersion = frontEndVersion.text();
		sObject="DigitalInput";
		
		iLeni=dynlen(dsDpList);
		for(i=1; i<=iLeni; i++)
			{									
			if (dpExists(dsDpList[i]))
				{
				dynClear(dsAllCommonParameters);
				dynClear(smoothDpParameters);
				dynClear(dsDpParameters);
				dynClear(dsArchiveNames);
				
				//Get all common Parameters
				unExportDevice_getAllCommonParameters(dsDpList[i], sObject,  dsAllCommonParameters);
				dsDpParameters = dsAllCommonParameters;
												
				//Get Archive Parameters
				unExportDevice_getArchiveParameters(dsDpList[i], sObject, sArchiveActive, fArchiveTimeFilter, sVersion);
				
				//Get Archive Names
				unCPCExportDevice_getArchiveNames(dsDpList[i], sObject, dsArchiveNames);
											
				//Get address config		
				unCPCExportDevice_getAddressConfig(dsDpList[i], sObject, smoothDpParameters );
				
				//Get Normal Position
				unCPCExportDevice_getDigitalNormalPosition(dsDpList[i], sObject, iNormalPosition);
				
				dynAppend(dsDpParameters, sArchiveActive);
				
				dynAppend(dsDpParameters, iNormalPosition);
				
				iLenj=dynlen(smoothDpParameters);
				for(j=1; j<=iLenj ; j++)
					{
					dynAppend(dsDpParameters, smoothDpParameters[j]);
					}
					
				iLenj=dynlen(dsArchiveNames);
				for(j=1; j<=iLenj ; j++)
					{
					dynAppend(dsDpParameters, dsArchiveNames[j]);
					}	
				if(sVersion != "") {
					unGenericDpFunctions_getMaskEvent(dsDpList[i], iMask);
					sprintf(sMask, "%d", iMask);
					dynAppend(dsDpParameters, sMask);
				}
				unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
				}
			}	
}


//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_DigitalInput_ExportConfig
/**
Purpose: Export DigitalInput Devices for S7_PLC front-end  

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
S7_PLC_DigitalInput_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	_UnPlc_DigitalInput_ExportConfig(dsDpList, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_DigitalOutput_ExportConfig
/**
Purpose: Export DigitalOutput Devices for _UnPlc front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
_UnPlc_DigitalOutput_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
string sDp, sArchiveActive, sObject, sArchiveName;
dyn_string dsAllCommonParameters, smoothDpParameters, dsDpParameters, dsArchiveNames;
float fArchiveTimeFilter ;
int i, j, iNormalPosition, iLeni, iLenj;
string sVersion, sMask;
int iMask;

		sVersion = frontEndVersion.text();
		sObject="DigitalOutput";
				
		iLeni=dynlen(dsDpList);
		for(i=1; i<=iLeni; i++)
			{									
			if (dpExists(dsDpList[i]))
				{
				dynClear(dsAllCommonParameters);
				dynClear(smoothDpParameters);
				dynClear(dsDpParameters);
				dynClear(dsArchiveNames);
				
				//Get all common Parameters
				unExportDevice_getAllCommonParameters(dsDpList[i], sObject,  dsAllCommonParameters);
				dsDpParameters = dsAllCommonParameters;
												
				//Get Archive Parameters
				unExportDevice_getArchiveParameters(dsDpList[i], sObject, sArchiveActive, fArchiveTimeFilter, sVersion);
				
				//Get Archive Names
				unCPCExportDevice_getArchiveNames(dsDpList[i], sObject, dsArchiveNames);
											
				//Get address config		
				unCPCExportDevice_getAddressConfig(dsDpList[i], sObject, smoothDpParameters );
				
				//Get Normal Position
				unCPCExportDevice_getDigitalNormalPosition(dsDpList[i], sObject, iNormalPosition);
				
				dynAppend(dsDpParameters, sArchiveActive);
								
				dynAppend(dsDpParameters, iNormalPosition);
				
				iLenj=dynlen(smoothDpParameters);
				for(j=1; j<=iLenj ; j++)
					{
					dynAppend(dsDpParameters, smoothDpParameters[j]);
					}
					
				iLenj=dynlen(dsArchiveNames);
				for(j=1; j<=iLenj ; j++)
					{
					dynAppend(dsDpParameters, dsArchiveNames[j]);
					}	
				if(sVersion != "") {
					unGenericDpFunctions_getMaskEvent(dsDpList[i], iMask);
					sprintf(sMask, "%d", iMask);
					dynAppend(dsDpParameters, sMask);
				}
				unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
				}
			}	
}

//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_DigitalOutput_ExportConfig
/**
Purpose: Export DigitalOutput Devices for S7_PLC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
S7_PLC_DigitalOutput_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	_UnPlc_DigitalOutput_ExportConfig(dsDpList, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_Alarm_ExportConfig
/**
Purpose: Export Alarm Devices for _UnPlc front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
_UnPlc_Alarm_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
string sArchiveActive, sObject;
dyn_string dsAllCommonParameters, smoothDpParameters, dsDpParameters, dsArchiveNames;
int i, j, iLeni, iLenj, iNormalPosition;
string sVersion, sMask, sMasterTypeName,sNormalPos;
int iMask;

		sVersion = frontEndVersion.text();
		sObject="Alarm";
				
		iLeni=dynlen(dsDpList);
		for(i=1; i<=iLeni; i++)
			{									
			if (dpExists(dsDpList[i]))
				{
				dynClear(dsAllCommonParameters);
				dynClear(smoothDpParameters);	
				dynClear(dsDpParameters);
				dynClear(dsArchiveNames);
				
				//Get all common Parameters
				unExportDevice_getAllCommonParameters(dsDpList[i], sObject,  dsAllCommonParameters);
				dsDpParameters = dsAllCommonParameters;
																						
				//Get address config		
				unCPCExportDevice_getAddressConfig(dsDpList[i], sObject, smoothDpParameters );
				
				//Get Archive Names
				unCPCExportDevice_getArchiveNames(dsDpList[i], sObject, dsArchiveNames);
				
				//Add Normal position
				unCPCExportDevice_getDigitalNormalPosition(dsDpList[i], sObject, iNormalPosition);
                                
                                // Get Master and Type 
                                sNormalPos=iNormalPosition;       // back compatibility
				unExportDevice_getAlarmMasterType(dsDpList[i], sMasterTypeName);
                                if (sMasterTypeName !="")  // Defined Master/Type (or just ",")
                                  sNormalPos=sNormalPos +","+sMasterTypeName;
                                
				dynAppend(dsDpParameters, sNormalPos);
				
				iLenj=dynlen(smoothDpParameters);
				for(j=1; j<=iLenj ; j++)
					{
					dynAppend(dsDpParameters, smoothDpParameters[j]);
					}
				
				iLenj=dynlen(dsArchiveNames);
				for(j=1; j<=iLenj ; j++)
					{
					dynAppend(dsDpParameters, dsArchiveNames[j]);
					}	
				if(sVersion != "") {
					unGenericDpFunctions_getMaskEvent(dsDpList[i], iMask);
					sprintf(sMask, "%d", iMask);
					dynAppend(dsDpParameters, sMask);
				}
				unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
				}
			}		
}

//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_Alarm_ExportConfig
/**
Purpose: Export Alarm Devices for S7_PLC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
S7_PLC_Alarm_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	_UnPlc_Alarm_ExportConfig(dsDpList, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_Local_ExportConfig
/**
Purpose: Export Local Devices for _UnPlc front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
_UnPlc_Local_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
string sArchiveActive, sObject;
dyn_string dsAllCommonParameters, smoothDpParameters, dsDpParameters, dsArchiveNames;
int i, j, iLeni, iLenj, iNormalPosition;
string sVersion, sMask;
int iMask;

		sVersion = frontEndVersion.text();
		sObject="Local";
				
		iLeni=dynlen(dsDpList);
		for(i=1; i<=iLeni; i++)
			{									
			if (dpExists(dsDpList[i]))
				{
				dynClear(dsAllCommonParameters);
				dynClear(smoothDpParameters);	
				dynClear(dsDpParameters);
				dynClear(dsArchiveNames);
				
				//Get all common Parameters
				unExportDevice_getAllCommonParameters(dsDpList[i], sObject,  dsAllCommonParameters);
				dsDpParameters = dsAllCommonParameters;
																						
				//Get address config		
				unCPCExportDevice_getAddressConfig(dsDpList[i], sObject, smoothDpParameters );
				
				//Get Archive Names
				unCPCExportDevice_getArchiveNames(dsDpList[i], sObject, dsArchiveNames);
				
				//Add Normal position
				unCPCExportDevice_getDigitalNormalPosition(dsDpList[i], sObject, iNormalPosition);
				dynAppend(dsDpParameters, iNormalPosition);
				
				iLenj=dynlen(smoothDpParameters);
				for(j=1; j<= iLenj ; j++)
					{
					dynAppend(dsDpParameters, smoothDpParameters[j]);
					}
				
				iLenj=dynlen(dsArchiveNames);
				for(j=1; j<=iLenj ; j++)
					{
					dynAppend(dsDpParameters, dsArchiveNames[j]);
					}	
				if(sVersion != "") {
					unGenericDpFunctions_getMaskEvent(dsDpList[i], iMask);
					sprintf(sMask, "%d", iMask);
					dynAppend(dsDpParameters, sMask);
				}
				unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
				}
			}		
}

//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_Local_ExportConfig
/**
Purpose: Export Local Devices for S7_PLC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
S7_PLC_Local_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	_UnPlc_Local_ExportConfig(dsDpList, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_Analog_ExportConfig
/**
Purpose: Export Analog Devices for _UnPlc front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
_UnPlc_Analog_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
string sObject, sUnit, sFormatToConvert, sFormatConverted, sArchiveActive, sLimitOn, sLimitOff, sMax, sMin, sArchiveName;
dyn_string dsAllCommonParameters, smoothDpParameters, dsDpParameters, dsArchiveNames;
bool bDoesExist, bNegateRange, bIgnoreOutside,  bInclusiveMin , bInclusiveMax , bIsSmoothDefined ;
float fMinValue, fMaxValue, fDeadband, fDeadbandTemp, fTimeInterval, fArchiveTimeFilter ;
int iSmoothProcedure, iDeadBandType, i, j, iLeni, iLenj, iNormalPosition;
string sVersion, sMask;
int iMask;
bool bStsReg02Exist=false;
int iTypeAddress;

		sVersion = frontEndVersion.text();
		sObject="Analog";
						
		iLeni=dynlen(dsDpList);
		for(i=1; i<=iLeni; i++)
			{									
			if (dpExists(dsDpList[i]))
				{
				dynClear(dsAllCommonParameters);
				dynClear(smoothDpParameters);	
				dynClear(dsDpParameters);
				dynClear(dsArchiveNames);
				
				//Get all common Parameters
				unExportDevice_getAllCommonParameters(dsDpList[i], sObject ,  dsAllCommonParameters);
				dsDpParameters = dsAllCommonParameters;
						
				//Get DP unit
				sUnit =  dpGetUnit(dsDpList[i] + ".ProcessInput.PosSt" );
				
				//Get Format
				sFormatToConvert = dpGetFormat(dsDpList[i] + ".ProcessInput.PosSt");
				sFormatConverted = unExportDevice_convertFormat(sFormatToConvert, "#");
				
				//Get RangeMax and RangeMin
				fwPvRange_get( dsDpList[i] + ".ProcessInput.PosSt", bDoesExist , fMinValue , fMaxValue , bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax , exceptionInfo);
				sMax = unExportDevice_floatFormat(fMaxValue);
				sMin = unExportDevice_floatFormat(fMinValue);
				
				//Get smoothProcedure, fDeadband & timeInterval
				unExportDevice_fwSmoothing_getTemp(dsDpList[i] + ".ProcessInput.PosSt", bIsSmoothDefined, iSmoothProcedure, fDeadbandTemp, fTimeInterval);
				unExportDevice_getDeadBand(fDeadbandTemp, bIsSmoothDefined, sMax, sMin, iSmoothProcedure, fDeadband, iDeadBandType );
									
				//Get Limits On & Off
				dpGet(dsDpList[i] + ".ProcessOutput.PLiOn",  sLimitOn);
				sLimitOn = unExportDevice_floatFormat(sLimitOn);
				
				dpGet(dsDpList[i] + ".ProcessOutput.PLiOff",  sLimitOff);
				sLimitOff = unExportDevice_floatFormat(sLimitOff);
				
				//Get Archive Parameters
				unExportDevice_getArchiveParameters(dsDpList[i], sObject, sArchiveActive, fArchiveTimeFilter);
				
				//Get Archive Names
				unCPCExportDevice_getArchiveNames(dsDpList[i], sObject, dsArchiveNames);
											
				//Get address config		
				unCPCExportDevice_getAddressConfig(dsDpList[i], sObject, smoothDpParameters );
				
				dynAppend(dsDpParameters, sUnit);
				dynAppend(dsDpParameters, sFormatConverted);
				dynAppend(dsDpParameters, sMax);
				dynAppend(dsDpParameters, sMin);
				
				dynAppend(dsDpParameters, sLimitOn);
				dynAppend(dsDpParameters, sLimitOff);
							
				if (iDeadBandType==DPCONFIG_NONE)
					dynAppend(dsDpParameters, "0");
				else
					dynAppend(dsDpParameters, fDeadband);
				
				dynAppend(dsDpParameters, iDeadBandType);
				dynAppend(dsDpParameters, sArchiveActive);
				
				if(fArchiveTimeFilter < 0)
					dynAppend(dsDpParameters, UN_CONFIG_EXPORT_UNDEF);
				else	
					dynAppend(dsDpParameters, fArchiveTimeFilter);
				
				//Add Normal position
				unCPCExportDevice_getDigitalNormalPosition(dsDpList[i], sObject, iNormalPosition);
				dynAppend(dsDpParameters, iNormalPosition);
				
                                // Exporting pheriperial addresses
                                // Case doubled defintion of stsReg01 and evStsReg01 then 2 more components 
                                // Former case: 10 components, New possible case=12
                                // stsReg01 = 1st
                                // stsReg02 = 2st
                                // evStsReg01 = 3nd
                                // evStsReg02 = 4th

                                bStsReg02Exist=false;
                                dpGet(dsDpList[i] + ".ProcessInput.StsReg02:_address.._type" , iTypeAddress);    
                                // DebugN("iTypeAddress",iTypeAddress);  
                
                                if (iTypeAddress != DPCONFIG_NONE)
                                    bStsReg02Exist=true;
                                // DebugN("Export ANALOG: smoothDpParameters=",smoothDpParameters);
                                if (bStsReg02Exist)    // new implementation
                                  {
                                    smoothDpParameters[1]=smoothDpParameters[1]+UN_CONFIG_GROUP_SEPARATOR+smoothDpParameters[2];
                                    smoothDpParameters[3]=smoothDpParameters[3]+UN_CONFIG_GROUP_SEPARATOR+smoothDpParameters[4];
                                    // Eliminate 2nd and 4th fields
                                    dynRemove(smoothDpParameters,4);
                                    dynRemove(smoothDpParameters,2);
                                  }
                                // DebugN("Export ANALOG final: smoothDpParameters=",smoothDpParameters);
                                iLenj=dynlen(smoothDpParameters);
				for(j=1; j<=iLenj ; j++)
					{
					dynAppend(dsDpParameters, smoothDpParameters[j]);
					}	
				iLenj=dynlen(dsArchiveNames);
				for(j=1; j<=iLenj ; j++)
					{
					dynAppend(dsDpParameters, dsArchiveNames[j]);
					}	
				if(sVersion != "") {
					unGenericDpFunctions_getMaskEvent(dsDpList[i], iMask);
					sprintf(sMask, "%d", iMask);
					dynAppend(dsDpParameters, sMask);
				}
				unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
				}	
			}
}

//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_Analog_ExportConfig
/**
Purpose: Export Analog Devices for S7_PLC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
S7_PLC_Analog_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	_UnPlc_Analog_ExportConfig(dsDpList, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_AnaDig_ExportConfig
/**
Purpose: Export Anadig Devices for _UnPlc front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
_UnPlc_AnaDig_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
string sObject, sUnit, sFormatToConvert, sFormatConverted, sArchiveActive, sLimitOn, sLimitOff, sMax, sMin, sArchiveName;
dyn_string dsAllCommonParameters, smoothDpParameters, dsDpParameters, dsArchiveNames;
bool bDoesExist, bNegateRange, bIgnoreOutside,  bInclusiveMin , bInclusiveMax , bIsSmoothDefined ;
float fMinValue, fMaxValue, fDeadband, fDeadbandTemp, fTimeInterval, fArchiveTimeFilter ;
int iSmoothProcedure, iDeadBandType, i, j, iLeni, iLenj, iNormalPosition;
string sVersion, sMask;
int iMask;

		sVersion = frontEndVersion.text();
		sObject="AnaDig";
						
		iLeni=dynlen(dsDpList);
		for(i=1; i<=iLeni; i++)
			{									
			if (dpExists(dsDpList[i]))
				{
				dynClear(dsAllCommonParameters);
				dynClear(smoothDpParameters);	
				dynClear(dsDpParameters);
				dynClear(dsArchiveNames);
				
				//Get all common Parameters
				unExportDevice_getAllCommonParameters(dsDpList[i], sObject ,  dsAllCommonParameters);
				dsDpParameters = dsAllCommonParameters;
						
				//Get DP unit
				sUnit =  dpGetUnit(dsDpList[i] + ".ProcessInput.PosSt" );
				
				//Get Format
				sFormatToConvert = dpGetFormat(dsDpList[i] + ".ProcessInput.PosSt");
				sFormatConverted = unExportDevice_convertFormat(sFormatToConvert, "#");
				
				//Get RangeMax and RangeMin
				fwPvRange_get( dsDpList[i] + ".ProcessInput.PosSt", bDoesExist , fMinValue , fMaxValue , bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax , exceptionInfo);
				
				sMax = unExportDevice_floatFormat(fMaxValue);		
				sMin= unExportDevice_floatFormat(fMinValue);
				
				//Get smoothProcedure, fDeadband & timeInterval
				unExportDevice_fwSmoothing_getTemp(dsDpList[i] + ".ProcessInput.PosSt", bIsSmoothDefined, iSmoothProcedure, fDeadbandTemp, fTimeInterval, exceptionInfo);
				unExportDevice_getDeadBand(fDeadbandTemp, bIsSmoothDefined, sMax, sMin, iSmoothProcedure, fDeadband, iDeadBandType );
									
				//Get Limits On & Off
				dpGet(dsDpList[i] + ".ProcessOutput.PLiOn", sLimitOn);
				sLimitOn = unExportDevice_floatFormat(sLimitOn);
				dpGet(dsDpList[i] + ".ProcessOutput.PLiOff", sLimitOff);
				sLimitOff=unExportDevice_floatFormat(sLimitOff);
				
				//Get Archive Parameters
				unExportDevice_getArchiveParameters(dsDpList[i], sObject, sArchiveActive, fArchiveTimeFilter);
				
				//Get Archive Names
				unCPCExportDevice_getArchiveNames(dsDpList[i], sObject, dsArchiveNames);
											
				//Get address config		
				unCPCExportDevice_getAddressConfig(dsDpList[i], sObject, smoothDpParameters );
				
				dynAppend(dsDpParameters, sUnit);
				dynAppend(dsDpParameters, sFormatConverted);
				dynAppend(dsDpParameters, sMax);
				dynAppend(dsDpParameters, sMin);
				
				dynAppend(dsDpParameters, sLimitOn);
				dynAppend(dsDpParameters, sLimitOff);
							
				if (iDeadBandType==DPCONFIG_NONE)
					dynAppend(dsDpParameters, "0");
				else
					dynAppend(dsDpParameters, fDeadband);
				
				dynAppend(dsDpParameters, iDeadBandType);
				dynAppend(dsDpParameters, sArchiveActive);
				
				if(fArchiveTimeFilter < 0)
					dynAppend(dsDpParameters, "0");
				else	
					dynAppend(dsDpParameters, fArchiveTimeFilter);
				
				//Normal Position
				unCPCExportDevice_getDigitalNormalPosition(dsDpList[i], sObject, iNormalPosition);
				dynAppend(dsDpParameters, iNormalPosition);
				
				iLenj=dynlen(smoothDpParameters);
				for(j=1; j<=iLenj ; j++)
					{
					dynAppend(dsDpParameters, smoothDpParameters[j]);
					}	
				
				iLenj=dynlen(dsArchiveNames);
				for(j=1; j<=iLenj ; j++)
					{
					dynAppend(dsDpParameters, dsArchiveNames[j]);
					}	
				if(sVersion != "") {
					unGenericDpFunctions_getMaskEvent(dsDpList[i], iMask);
					sprintf(sMask, "%d", iMask);
					dynAppend(dsDpParameters, sMask);
				}
				unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
				}	
			}
			
}

//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_AnaDig_ExportConfig
/**
Purpose: Export Anadig Devices for S7_PLC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
S7_PLC_AnaDig_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	_UnPlc_AnaDig_ExportConfig(dsDpList, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_Controller_ExportConfig
/**
Purpose: Export Controller Devices for _UnPlc front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
_UnPlc_Controller_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
string sObject, sUnitSPMV, sUnitOut, sFormatToConvertSPMV, sFormatToConvertOut, sFormatConvertedSPMV, sFormatConvertedOut, sArchiveActive, sPIDName, sMaxOut, sMinOut, sMaxSPMV, sMinSPMV, sArchiveName;
dyn_string dsAllCommonParameters, smoothDpParameters, dsDpParameters, dsPCOParameters, dsArchiveNames;
bool bDoesExist, bNegateRange, bIgnoreOutside,  bInclusiveMin , bInclusiveMax , bIsSmoothDefined ;
float fMinValueSPMV , fMaxValueSPMV, fMinValueOut , fMaxValueOut, fDeadbandOut, fDeadbandMV, fDeadbandTemp, fTimeInterval, fArchiveTimeFilter ;
int iSmoothProcedure, iDeadBandTypeOut, iDeadBandTypeMV, i, j, iLeni, iLenj;
int iTypeAddress;
long lUpdate;
bool bPeriphAddrExist,bDefaultParamSet; 
time tUpdate;

string sVersion, sMask, sFrontEndType;
int iMask;
float fDfltP, fDfltI, fDfltD, fDfltTDs;
float fDfltSp, fDfltSpH,  fDfltSpL;
float fDfltOutH,  fDlftOutL;

string sDeadbandType, sDeadband;
// New archive settings
string sArchiveActiveOUT;
float fArchiveTimeFilterOUT;
string sArchiveComposition;

		sVersion = frontEndVersion.text();
		sObject="Controller";
		sFrontEndType = frontEndType.text();
						
		iLeni=dynlen(dsDpList);
		for(i=1; i<=iLeni; i++)
			{									
			if (dpExists(dsDpList[i]))
				{
				dynClear(dsAllCommonParameters);
				dynClear(smoothDpParameters);	
				dynClear(dsDpParameters);
				dynClear(dsArchiveNames);
				
				//Get all common Parameters
				unExportDevice_getAllCommonParameters(dsDpList[i], sObject ,  dsAllCommonParameters);
				dsDpParameters = dsAllCommonParameters;
								
				//Get PIDName
				unExportDevice_getPIDName(dsDpList[i], sPIDName);
								
				//Get DP unit SPMV
				sUnitSPMV =  dpGetUnit(dsDpList[i] + ".ProcessInput.MV" );
								
				//Get Format SPMV
				sFormatToConvertSPMV = dpGetFormat(dsDpList[i] + ".ProcessInput.MV");
				sFormatConvertedSPMV = unExportDevice_convertFormat(sFormatToConvertSPMV, "#");
								
				//Get DP unitOut
				sUnitOut =  dpGetUnit(dsDpList[i] + ".ProcessInput.OutOVSt" );
				
				//Get Format OutOVSt
				sFormatToConvertOut = dpGetFormat(dsDpList[i] + ".ProcessInput.OutOVSt");
				sFormatConvertedOut = unExportDevice_convertFormat(sFormatToConvertOut, "#");
				
				//Get RangeMax and RangeMin AtSPo
				fwPvRange_get( dsDpList[i] + ".ProcessInput.AtSPo", bDoesExist , fMinValueSPMV , fMaxValueSPMV , bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax , exceptionInfo);
				
				sMaxSPMV = unExportDevice_floatFormat(fMaxValueSPMV);	
				sMinSPMV = unExportDevice_floatFormat(fMinValueSPMV);
				
				//Get RangeMax and RangeMin OutOVSt
				fwPvRange_get( dsDpList[i] + ".ProcessInput.OutOVSt", bDoesExist , fMinValueOut , fMaxValueOut , bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax , exceptionInfo);
				
				sMaxOut = unExportDevice_floatFormat(fMaxValueOut);			
				sMinOut= unExportDevice_floatFormat(fMinValueOut);
				
				//Get smoothProcedure, fDeadband & timeInterval for OutOVSt
				unExportDevice_fwSmoothing_getTemp(dsDpList[i] + ".ProcessInput.OutOVSt", bIsSmoothDefined, iSmoothProcedure, fDeadbandTemp, fTimeInterval);
				unExportDevice_getDeadBand(fDeadbandTemp, bIsSmoothDefined, sMaxOut, sMinOut, iSmoothProcedure, fDeadbandOut, iDeadBandTypeOut );

				//Get smoothProcedure, fDeadband & timeInterval for MV
				unExportDevice_fwSmoothing_getTemp(dsDpList[i] + ".ProcessInput.MV", bIsSmoothDefined, iSmoothProcedure, fDeadbandTemp, fTimeInterval);
				unExportDevice_getDeadBand(fDeadbandTemp, bIsSmoothDefined, sMaxSPMV, sMinSPMV, iSmoothProcedure, fDeadbandMV, iDeadBandTypeMV );
				
				//Get Archive Parameters
				unExportDevice_getArchiveParameters(dsDpList[i], sObject, sArchiveActive, fArchiveTimeFilter);
                                // Get additionally the OutOVSt
				unExportDevice_getArchiveParametersDpe(dsDpList[i]+".ProcessInput.OutOVSt", sArchiveActiveOUT, fArchiveTimeFilterOUT, "");
                                
				//Get Archive Names
				unCPCExportDevice_getArchiveNames(dsDpList[i], sObject, dsArchiveNames);
				 				
				//Get address config		
				unCPCExportDevice_getAddressConfig(dsDpList[i], sObject, smoothDpParameters );
				
				//Get PCO Parameters like OpName1->8 & Allowance1->8
				unExportDevice_getPCOName(dsDpList[i], dsPCOParameters );
				
				dynAppend(dsDpParameters, sPIDName);
				dynAppend(dsDpParameters, sUnitSPMV);
				dynAppend(dsDpParameters, sFormatConvertedSPMV);
				dynAppend(dsDpParameters, sMaxSPMV);
				dynAppend(dsDpParameters, sMinSPMV);
				dynAppend(dsDpParameters, sUnitOut);
				dynAppend(dsDpParameters, sFormatConvertedOut);
				dynAppend(dsDpParameters, sMaxOut);
				dynAppend(dsDpParameters, sMinOut);
				
                                // DebugN("exportFUNTION:: sPIDName=",sPIDName," iDeadBandTypeOut="+iDeadBandTypeOut);
				if (iDeadBandTypeOut==DPCONFIG_NONE) 
                                  {
					sDeadbandType = "0";
					sDeadband = "0";
				  }
				else 
                                  {
					sDeadbandType = iDeadBandTypeOut;
					sDeadband = fDeadbandOut;
				  }
				if (iDeadBandTypeMV==DPCONFIG_NONE) 
                                  {
					sDeadbandType += UN_CONFIG_GROUP_SEPARATOR+"0";
					sDeadband += UN_CONFIG_GROUP_SEPARATOR+"0";
				  }
				else 
                                  {
					sDeadbandType += UN_CONFIG_GROUP_SEPARATOR+iDeadBandTypeMV;
					sDeadband += UN_CONFIG_GROUP_SEPARATOR+fDeadbandMV;
				  }
				
				dynAppend(dsDpParameters, sDeadband);
				dynAppend(dsDpParameters, sDeadbandType);
                                
                                // Treat the archive settings (OUT,MV_SP)
				dynAppend(dsDpParameters, sArchiveActiveOUT+UN_CONFIG_GROUP_SEPARATOR+sArchiveActive);
				
				if(fArchiveTimeFilterOUT < 0)
                                        sArchiveComposition=UN_CONFIG_EXPORT_UNDEF;
				else	
                                        sArchiveComposition=fArchiveTimeFilterOUT;
                                if (fArchiveTimeFilter < 0)
				        sArchiveComposition+=UN_CONFIG_GROUP_SEPARATOR+UN_CONFIG_EXPORT_UNDEF;
                                else
                                        sArchiveComposition+=UN_CONFIG_GROUP_SEPARATOR+fArchiveTimeFilter;
                                
                                dynAppend(dsDpParameters, sArchiveComposition);
				
				iLenj=dynlen(smoothDpParameters);
				for(j=1; j<=iLenj; j++)
					{
					dynAppend(dsDpParameters, smoothDpParameters[j]);
					}	
				
				iLenj=dynlen(dsArchiveNames);
				for(j=1; j<=iLenj ; j++)
					{
					dynAppend(dsDpParameters, dsArchiveNames[j]);
					}	
				if(sVersion != "") 
                                    {
					unGenericDpFunctions_getMaskEvent(dsDpList[i], iMask);
					sprintf(sMask, "%d", iMask);
					dynAppend(dsDpParameters, sMask);
					 
                                        // init values
                                        bPeriphAddrExist=false;
                                        bDefaultParamSet=false;
                                        // Check  implementation S7 with addressing
                                        dpGet(dsDpList[i] + ".ProcessOutput.DfltP:_address.._type" , iTypeAddress);                                            
                                        if (iTypeAddress != DPCONFIG_NONE) bPeriphAddrExist=true;
                                        // Check implementation S7/unPLC without addressing (just default values)
                                        dpGet(dsDpList[i] + ".ProcessOutput.DfltSPo:_online.._stime" , tUpdate);
                                        lUpdate=tUpdate;
                                        if (lUpdate>0) bDefaultParamSet=true;
                                        
                                        // Case S7 with addressing in periphery
					if((sFrontEndType == S7_PLC_DPTYPE) && bPeriphAddrExist)
                                          {
        					unConfigController_getDefault(dsDpList[i], fDfltP, fDfltI, fDfltD, fDfltTDs);					
					
        					dynAppend(dsDpParameters,DRV_S7_convertToUnicosAddress(dsDpList[i]+ ".ProcessOutput.DfltP", sObject));
        					sprintf(sMask, "%f", fDfltP);
        					dynAppend(dsDpParameters, sMask);
					
        					dynAppend(dsDpParameters,DRV_S7_convertToUnicosAddress(dsDpList[i]+ ".ProcessOutput.DfltI", sObject));
        					sprintf(sMask, "%f", fDfltI);
        					dynAppend(dsDpParameters, sMask);
					
        					dynAppend(dsDpParameters,DRV_S7_convertToUnicosAddress(dsDpList[i]+ ".ProcessOutput.DfltD", sObject));
        					sprintf(sMask, "%f", fDfltD);
        					dynAppend(dsDpParameters, sMask);
					
        					dynAppend(dsDpParameters,DRV_S7_convertToUnicosAddress(dsDpList[i]+ ".ProcessOutput.DfltTDs", sObject));
        					sprintf(sMask, "%f", fDfltTDs);
        					dynAppend(dsDpParameters, sMask);
					  }
                                        // Case new parameters (default values without addressing) both S7/unPLC
                                        else if (bDefaultParamSet)  // time stamp = 0
                                          {
                                              unConfigController_getDefaultValuesAll(dsDpList[i], fDfltP, fDfltI, fDfltD, fDfltTDs,
                                                                                     fDfltSp, fDfltSpH,  fDfltSpL,
                                                                                     fDfltOutH,  fDlftOutL);
                                                // PID params
        					sprintf(sMask, "%f", fDfltP);
        					dynAppend(dsDpParameters, sMask);
        					sprintf(sMask, "%f", fDfltI);
        					dynAppend(dsDpParameters, sMask);
        					sprintf(sMask, "%f", fDfltD);
        					dynAppend(dsDpParameters, sMask);
        					sprintf(sMask, "%f", fDfltTDs);
        					dynAppend(dsDpParameters, sMask);
                                                // Sp, SP & OUT limits
                                      		sprintf(sMask, "%f", fDfltSp);
        					dynAppend(dsDpParameters, sMask);
        					sprintf(sMask, "%f", fDfltSpH);
        					dynAppend(dsDpParameters, sMask);
        					sprintf(sMask, "%f", fDfltSpL);
        					dynAppend(dsDpParameters, sMask);
        					sprintf(sMask, "%f", fDfltOutH);
        					dynAppend(dsDpParameters, sMask);
                                                sprintf(sMask, "%f", fDlftOutL);
        					dynAppend(dsDpParameters, sMask);
                                          }
				    }
				unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
			}
		}
}

//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_Controller_ExportConfig
/**
Purpose: Export Controller Devices for S7_PLC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
S7_PLC_Controller_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	_UnPlc_Controller_ExportConfig(dsDpList, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_ProcessControlObject_ExportConfig
/**
Purpose: Export PCO Devices for _UnPlc front-end  

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
_UnPlc_ProcessControlObject_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
string sObject, sPCOName;
dyn_string dsAllCommonParameters, smoothDpParameters, dsDpParameters, dsPCOParameters, dsArchiveNames;
int i, j, k, iLeni, iLenk, iNormalPosition;
string sVersion, sMask;
int iMask;

		sVersion = frontEndVersion.text();
		sObject="ProcessControlObject";
						
		iLeni=dynlen(dsDpList);
		for(i=1; i<=iLeni; i++)
			{									
			if (dpExists(dsDpList[i]))
				{
					dynClear(dsAllCommonParameters);
					dynClear(smoothDpParameters);
					dynClear(dsDpParameters);
					dynClear(dsArchiveNames);
					
					//Get all common Parameters
					unExportDevice_getAllCommonParameters(dsDpList[i], sObject ,  dsAllCommonParameters);
					dsDpParameters = dsAllCommonParameters;
					
					//Get PCOName
					unExportDevice_getPCOName(dsDpList[i], sPCOName);
					 				
					//Get address config		
					unCPCExportDevice_getAddressConfig(dsDpList[i], sObject, smoothDpParameters );
					
					//Get PCO Parameters like OpName1->8 & Allowance1->8
					unExportDevice_getPCOName(dsDpList[i], dsPCOParameters );
					
					dynAppend(dsDpParameters, sPCOName);
					
					//Get Archive Names
					unCPCExportDevice_getArchiveNames(dsDpList[i], sObject, dsArchiveNames);
					
					//Add Normal position
					unCPCExportDevice_getDigitalNormalPosition(dsDpList[i], sObject, iNormalPosition);
					dynAppend(dsDpParameters, iNormalPosition);
					
					for(j=1; j<= dynlen(smoothDpParameters) ; j++)
						{
						dynAppend(dsDpParameters, smoothDpParameters[j]);
						}	
					
					unExportDevice_getPCOParameters(dsDpList[i], dsPCOParameters);
					
					iLenk=dynlen(dsPCOParameters);
					for(k=1;k<=iLenk;k++)
						{
						dynAppend(dsDpParameters, dsPCOParameters[k]);
						}
					
					iLenk=dynlen(dsArchiveNames);
					for(k=1; k<=iLenk ; k++)
						{
						dynAppend(dsDpParameters, dsArchiveNames[k]);
						}	
				if(sVersion != "") {
					unGenericDpFunctions_getMaskEvent(dsDpList[i], iMask);
					sprintf(sMask, "%d", iMask);
					dynAppend(dsDpParameters, sMask);
				}
					unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
				}
			}
}

//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_ProcessControlObject_ExportConfig
/**
Purpose: Export PCO Devices for S7_PLC front-end  

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
S7_PLC_ProcessControlObject_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	_UnPlc_ProcessControlObject_ExportConfig(dsDpList, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// _UnPlc_OnOff_ExportConfig
/**
Purpose: Export OnOff Devices for _UnPlc front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
_UnPlc_OnOff_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
string sObject;
dyn_string dsAllCommonParameters, smoothDpParameters, dsDpParameters, dsArchiveNames;
int i, j, iLeni, iNormalPosition, iLenj;
string sVersion, sMask;
int iMask;

		sVersion = frontEndVersion.text();
		sObject="OnOff";
						
		iLeni=dynlen(dsDpList);
		for(i=1; i<=iLeni; i++)
			{									
			if (dpExists(dsDpList[i]))
				{
				dynClear(dsAllCommonParameters);
				dynClear(smoothDpParameters);		
				dynClear(dsDpParameters);
				dynClear(dsArchiveNames);
				
				//Get all common Parameters
				unExportDevice_getAllCommonParameters(dsDpList[i], sObject,  dsAllCommonParameters);
				dsDpParameters = dsAllCommonParameters;
																						
				//Get address config		
				unCPCExportDevice_getAddressConfig(dsDpList[i], sObject, smoothDpParameters );
				
				//Get Archive Names
				unCPCExportDevice_getArchiveNames(dsDpList[i], sObject, dsArchiveNames);
				
				//Add Normal position
				unCPCExportDevice_getDigitalNormalPosition(dsDpList[i], sObject, iNormalPosition);
				dynAppend(dsDpParameters, iNormalPosition);
				
				iLenj=dynlen(smoothDpParameters);
				for(j=1; j<=iLenj ; j++)
					{
					dynAppend(dsDpParameters, smoothDpParameters[j]);
					}
				
				iLenj=dynlen(dsArchiveNames);
				for(j=1; j<=iLenj ; j++)
					{
					dynAppend(dsDpParameters, dsArchiveNames[j]);
					}	
				if(sVersion != "") {
					unGenericDpFunctions_getMaskEvent(dsDpList[i], iMask);
					sprintf(sMask, "%d", iMask);
					dynAppend(dsDpParameters, sMask);
				}
				unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
				}
			}
}

//------------------------------------------------------------------------------------------------------------------------
// S7_PLC_OnOff_ExportConfig
/**
Purpose: Export OnOff Devices for S7_PLC front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
S7_PLC_OnOff_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	_UnPlc_OnOff_ExportConfig(dsDpList, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// CMW_AnalogInput_ExportConfig
/**
Purpose: Export AnalogInput Devices for _UnPlc front-end

Parameters:

Usage: External function

// 56;;;;CMW;AI;AnalogInput_Small;Unit;#####;40000;0;;;;;0;0;O;10;2;;;2||cryo.S56.CryoStart$WATCHDOG_ARC$value||;;;;
PVSS manager usage: NG, NV
*/

void CMW_AnalogInput_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	dyn_string dsAllCommonParameters, dsSmoothDpParameters, dsLimits , dsDpParameters, dsArchiveNames;
	string sObject, sUnit, sFormatToConvert, sFormatConverted, sArchiveActive, sMin, sMax, sArchiveName;
	float fMinValue, fMaxValue, fDeadband, fDeadbandTemp, fTimeInterval, fArchiveTimeFilter ;
	int iSmoothProcedure, iDeadBandType, i, j, iNormalPosition, iLeni, iLenj;
	bool bDoesExist, bNegateRange, bIgnoreOutside,  bInclusiveMin , bInclusiveMax , bIsSmoothDefined ;
	string sVersion, sMask,sCompleteCMWAddress;
	int iMask;

		sVersion = frontEndVersion.text();
		sObject="AnalogInput";
		g_iAlertRange = 0;
		
		iLeni=dynlen(dsDpList);
		for(i=1; i<=iLeni; i++)
			{									
			if (dpExists(dsDpList[i]))
				{
				dynClear(dsAllCommonParameters);
				dynClear(dsSmoothDpParameters);
				dynClear(dsLimits);		
				dynClear(dsDpParameters);
				dynClear(dsArchiveNames);
					
				//Get all common Parameters
				unExportDevice_getAllCommonParameters(dsDpList[i], sObject ,  dsAllCommonParameters);
				dsDpParameters = dsAllCommonParameters;
						
				//Get DP unit
				sUnit =  dpGetUnit(dsDpList[i] + ".ProcessInput.PosSt" );
				
				//Get Format
				sFormatToConvert = dpGetFormat(dsDpList[i] + ".ProcessInput.PosSt");
				sFormatConverted = unExportDevice_convertFormat(sFormatToConvert, "#");
				
				//Get RangeMax and RangeMin
				fwPvRange_get( dsDpList[i] + ".ProcessInput.PosSt", bDoesExist , fMinValue , fMaxValue , bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax , exceptionInfo);
				
				sMax = unExportDevice_floatFormat(fMaxValue);			
				sMin = unExportDevice_floatFormat(fMinValue);
				
				//Get smoothProcedure, fDeadband & timeInterval
				unExportDevice_fwSmoothing_getTemp(dsDpList[i] + ".ProcessInput.PosSt", bIsSmoothDefined, iSmoothProcedure, fDeadbandTemp, fTimeInterval);
				
				//for Max and Min: force convert from string to float
				//Get unicos DeadBand
				unExportDevice_getDeadBand(fDeadbandTemp, bIsSmoothDefined, sMax, sMin, iSmoothProcedure, fDeadband, iDeadBandType );	
								
				//Get Limits
				unExportDevice_getLimits(dsDpList[i], dsLimits, iNormalPosition, sMax, sMin);
				
				//Get Archive Parameters
				unExportDevice_getArchiveParameters(dsDpList[i], sObject, sArchiveActive, fArchiveTimeFilter);
											
				//Get Archive Names
				unCPCExportDevice_getArchiveNames(dsDpList[i], sObject, dsArchiveNames);
						
				dynAppend(dsDpParameters, sUnit);
				dynAppend(dsDpParameters, sFormatConverted);
				dynAppend(dsDpParameters, sMax);
				dynAppend(dsDpParameters, sMin);
								
				dynAppend(dsDpParameters, dsLimits[1]);
				dynAppend(dsDpParameters, dsLimits[2]);
				dynAppend(dsDpParameters, dsLimits[3]);
				dynAppend(dsDpParameters, dsLimits[4]);
								
				if (iDeadBandType==DPCONFIG_NONE)
					dynAppend(dsDpParameters, UN_CONFIG_EXPORT_UNDEF);
				else
					dynAppend(dsDpParameters, fDeadband);
				
				dynAppend(dsDpParameters, iDeadBandType);
				dynAppend(dsDpParameters, sArchiveActive);
				
				if(fArchiveTimeFilter < 0)
					dynAppend(dsDpParameters, UN_CONFIG_EXPORT_UNDEF);
				else	
					dynAppend(dsDpParameters, fArchiveTimeFilter);
				
				dynAppend(dsDpParameters, iNormalPosition);
				
				// StsReg01
				unConfigCMW_getAddressConfig(dsDpList[i]+".ProcessInput.StsReg01", dsSmoothDpParameters );
				// DebugN("dsSmoothDpParameters=",dsSmoothDpParameters);
				if (dsSmoothDpParameters[4]=="") // no address config
					sCompleteCMWAddress="";
				else
					sCompleteCMWAddress=dsSmoothDpParameters[1]+
															CMW_ADDRESS_CONFIG_SEPARATOR+
															dsSmoothDpParameters[3]+
															CMW_ADDRESS_CONFIG_SEPARATOR+
															dsSmoothDpParameters[4]+"$"+dsSmoothDpParameters[5]+"$"+dsSmoothDpParameters[6]+
															CMW_ADDRESS_CONFIG_SEPARATOR+
															dsSmoothDpParameters[7]+
															CMW_ADDRESS_CONFIG_SEPARATOR+
															dsSmoothDpParameters[8];
				dynAppend(dsDpParameters,sCompleteCMWAddress);
									
				// evStsReg01
				unConfigCMW_getAddressConfig(dsDpList[i]+".ProcessInput.evStsReg01", dsSmoothDpParameters );
				if (dsSmoothDpParameters[4]=="") // no address config
					sCompleteCMWAddress="";
				else
					sCompleteCMWAddress=dsSmoothDpParameters[1]+
															CMW_ADDRESS_CONFIG_SEPARATOR+
															dsSmoothDpParameters[3]+
															CMW_ADDRESS_CONFIG_SEPARATOR+
															dsSmoothDpParameters[4]+"$"+dsSmoothDpParameters[5]+"$"+dsSmoothDpParameters[6]+
															CMW_ADDRESS_CONFIG_SEPARATOR+
															dsSmoothDpParameters[7]+
															CMW_ADDRESS_CONFIG_SEPARATOR+
															dsSmoothDpParameters[8];
				dynAppend(dsDpParameters,sCompleteCMWAddress);
															
				// PosSt		
				unConfigCMW_getAddressConfig(dsDpList[i]+".ProcessInput.PosSt", dsSmoothDpParameters );
				if (dsSmoothDpParameters[4]=="") // no address config
					sCompleteCMWAddress="";
				else
					sCompleteCMWAddress=dsSmoothDpParameters[1]+
															CMW_ADDRESS_CONFIG_SEPARATOR+
															dsSmoothDpParameters[3]+
															CMW_ADDRESS_CONFIG_SEPARATOR+
															dsSmoothDpParameters[4]+"$"+dsSmoothDpParameters[5]+"$"+dsSmoothDpParameters[6]+
															CMW_ADDRESS_CONFIG_SEPARATOR+
															dsSmoothDpParameters[7]+
															CMW_ADDRESS_CONFIG_SEPARATOR+
															dsSmoothDpParameters[8];
				dynAppend(dsDpParameters,sCompleteCMWAddress);
															
			  // HFStOrAuPosRSt		
				unConfigCMW_getAddressConfig(dsDpList[i]+".ProcessInput.HFStOrAuPosRSt", dsSmoothDpParameters );
				if (dsSmoothDpParameters[4]=="") // no address config
					sCompleteCMWAddress="";
				else
					sCompleteCMWAddress=dsSmoothDpParameters[1]+
															CMW_ADDRESS_CONFIG_SEPARATOR+
															dsSmoothDpParameters[3]+
															CMW_ADDRESS_CONFIG_SEPARATOR+
															dsSmoothDpParameters[4]+"$"+dsSmoothDpParameters[5]+"$"+dsSmoothDpParameters[6]+
															CMW_ADDRESS_CONFIG_SEPARATOR+
															dsSmoothDpParameters[7]+
															CMW_ADDRESS_CONFIG_SEPARATOR+
															dsSmoothDpParameters[8];
				dynAppend(dsDpParameters,sCompleteCMWAddress);
															
				// ManReg01		
				unConfigCMW_getAddressConfig(dsDpList[i]+".ProcessOutput.ManReg01", dsSmoothDpParameters );
				if (dsSmoothDpParameters[4]=="") // no address config
					sCompleteCMWAddress="";
				else
					sCompleteCMWAddress=dsSmoothDpParameters[1]+
															CMW_ADDRESS_CONFIG_SEPARATOR+
															dsSmoothDpParameters[3]+
															CMW_ADDRESS_CONFIG_SEPARATOR+
															dsSmoothDpParameters[4]+"$"+dsSmoothDpParameters[5]+"$"+dsSmoothDpParameters[6]+
															CMW_ADDRESS_CONFIG_SEPARATOR+
															dsSmoothDpParameters[7]+
															CMW_ADDRESS_CONFIG_SEPARATOR+
															dsSmoothDpParameters[8];
				dynAppend(dsDpParameters,sCompleteCMWAddress);
				
				// MPosR		
				unConfigCMW_getAddressConfig(dsDpList[i]+".ProcessOutput.MPosR", dsSmoothDpParameters );
				if (dsSmoothDpParameters[4]=="") // no address config
					sCompleteCMWAddress="";
				else
					sCompleteCMWAddress=dsSmoothDpParameters[1]+
															CMW_ADDRESS_CONFIG_SEPARATOR+
															dsSmoothDpParameters[3]+
															CMW_ADDRESS_CONFIG_SEPARATOR+
															dsSmoothDpParameters[4]+"$"+dsSmoothDpParameters[5]+"$"+dsSmoothDpParameters[6]+
															CMW_ADDRESS_CONFIG_SEPARATOR+
															dsSmoothDpParameters[7]+
															CMW_ADDRESS_CONFIG_SEPARATOR+
															dsSmoothDpParameters[8];
				dynAppend(dsDpParameters,sCompleteCMWAddress);
				
				iLenj=dynlen(dsArchiveNames);
				for(j=1; j<=iLenj ; j++)
					{
					dynAppend(dsDpParameters, dsArchiveNames[j]);
					}	
				unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
				}//dp doesn't exist
			}//end for
}

//------------------------------------------------------------------------------------------------------------------------
// CMW_DigitalInput_ExportConfig
/**
Purpose: Export DigitalInput Devices for _UnPlc front-end  

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
CMW_DigitalInput_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
string sDp, sArchiveActive, sObject, sArchiveName;
dyn_string dsAllCommonParameters, smoothDpParameters, dsDpParameters, dsArchiveNames;
float fArchiveTimeFilter ;
int i, j, iNormalPosition, iLeni, iLenj;
string sVersion, sMask;
int iMask;

		sVersion = frontEndVersion.text();
		sObject="DigitalInput";
		
		iLeni=dynlen(dsDpList);
		for(i=1; i<=iLeni; i++)
			{									
			if (dpExists(dsDpList[i]))
				{
				dynClear(dsAllCommonParameters);
				dynClear(smoothDpParameters);
				dynClear(dsDpParameters);
				dynClear(dsArchiveNames);
				
				//Get all common Parameters
				unExportDevice_getAllCommonParameters(dsDpList[i], sObject,  dsAllCommonParameters);
				dsDpParameters = dsAllCommonParameters;
												
				//Get Archive Parameters
				unExportDevice_getArchiveParameters(dsDpList[i], sObject, sArchiveActive, fArchiveTimeFilter, sVersion);
				
				//Get Archive Names
				unCPCExportDevice_getArchiveNames(dsDpList[i], sObject, dsArchiveNames);
											
				//Get address config		
				unConfigCMW_getAddressConfig(dsDpList[i]+".ProcessInput.PosSt", smoothDpParameters );
				
				//Get Normal Position
				unCPCExportDevice_getDigitalNormalPosition(dsDpList[i], sObject, iNormalPosition);
				
				dynAppend(dsDpParameters, sArchiveActive);
				
				dynAppend(dsDpParameters, iNormalPosition);
				
				dynAppend(dsDpParameters, smoothDpParameters[1]); // UN_CONFIG_CMW_DIGITALINPUT_TYPE
				dynAppend(dsDpParameters, smoothDpParameters[3]); // UN_CONFIG_CMW_DIGITALINPUT_GROUP
				dynAppend(dsDpParameters, smoothDpParameters[4]); // UN_CONFIG_CMW_DIGITALINPUT_ADDRESS_CMWDEVICE
				dynAppend(dsDpParameters, smoothDpParameters[7]); // UN_CONFIG_CMW_DIGITALINPUT_ADDRESS_CMW_FILTERNAME
				dynAppend(dsDpParameters, smoothDpParameters[8]); // UN_CONFIG_CMW_DIGITALINPUT_ADDRESS_CMW_FILTERVALUE
					
				iLenj=dynlen(dsArchiveNames);
				for(j=1; j<=iLenj ; j++)
					{
					dynAppend(dsDpParameters, dsArchiveNames[j]);
					}	
				unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
				}
			}	
}

//------------------------------------------------------------------------------------------------------------------------
// CMW_AnalogStatus_ExportConfig
/**
Purpose: Export AnalogStatus Devices for _UnPlc front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void CMW_AnalogStatus_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
dyn_string dsAllCommonParameters, dsSmoothDpParameters, dsLimits , dsDpParameters, dsArchiveNames;
string sObject, sUnit, sFormatToConvert, sFormatConverted, sArchiveActive, sMin, sMax, sArchiveName;
float fMinValue, fMaxValue, fDeadband, fDeadbandTemp, fTimeInterval, fArchiveTimeFilter ;
int iSmoothProcedure, iDeadBandType, i, j, iNormalPosition, iLeni, iLenj;
bool bDoesExist, bNegateRange, bIgnoreOutside,  bInclusiveMin , bInclusiveMax , bIsSmoothDefined ;
string sVersion, sMask;
int iMask;

		sVersion = frontEndVersion.text();
		sObject="AnalogStatus";
		g_iAlertRange = 0;
		
		iLeni=dynlen(dsDpList);
		for(i=1; i<=iLeni; i++)
			{									
			if (dpExists(dsDpList[i]))
				{
				dynClear(dsAllCommonParameters);
				dynClear(dsSmoothDpParameters);
				dynClear(dsLimits);		
				dynClear(dsDpParameters);
				dynClear(dsArchiveNames);
					
				//Get all common Parameters
				unExportDevice_getAllCommonParameters(dsDpList[i], sObject ,  dsAllCommonParameters);
				dsDpParameters = dsAllCommonParameters;
						
				//Get DP unit
				sUnit =  dpGetUnit(dsDpList[i] + ".ProcessInput.AuPosRSt" );
				
				//Get Format
				sFormatToConvert = dpGetFormat(dsDpList[i] + ".ProcessInput.AuPosRSt");
				sFormatConverted = unExportDevice_convertFormat(sFormatToConvert, "#");
				
				//Get RangeMax and RangeMin
				fwPvRange_get( dsDpList[i] + ".ProcessInput.AuPosRSt", bDoesExist , fMinValue , fMaxValue , bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax , exceptionInfo);
				
				sMax = unExportDevice_floatFormat(fMaxValue);			
				sMin = unExportDevice_floatFormat(fMinValue);
				
				//Get smoothProcedure, fDeadband & timeInterval
				unExportDevice_fwSmoothing_getTemp(dsDpList[i] + ".ProcessInput.AuPosRSt", bIsSmoothDefined, iSmoothProcedure, fDeadbandTemp, fTimeInterval);
				
				//for Max and Min: force convert from string to float
				//Get unicos DeadBand
				unExportDevice_getDeadBand(fDeadbandTemp, bIsSmoothDefined, sMax, sMin, iSmoothProcedure, fDeadband, iDeadBandType );	
								
				//Get Archive Parameters
				unExportDevice_getArchiveParametersDpe(dsDpList[i] + ".ProcessInput.AuPosRSt", sArchiveActive, fArchiveTimeFilter);
											
				//Get Archive Names
				unExportDevice_getArchiveNamesDpe(dsDpList[i], makeDynString(UN_CONFIG_EXPORT_UNDEF, ".ProcessInput.AuPosRSt", UN_CONFIG_EXPORT_UNDEF), 
																		dsArchiveNames);

				//Get address config		
				unConfigCMW_getAddressConfig(dsDpList[i]+".ProcessInput.AuPosRSt", dsSmoothDpParameters );
		
				dynAppend(dsDpParameters, sUnit);
				dynAppend(dsDpParameters, sFormatConverted);
								
				if (iDeadBandType==DPCONFIG_NONE)
					dynAppend(dsDpParameters, UN_CONFIG_EXPORT_UNDEF);
				else
					dynAppend(dsDpParameters, fDeadband);
				
				dynAppend(dsDpParameters, iDeadBandType);
				dynAppend(dsDpParameters, sMax);
				dynAppend(dsDpParameters, sMin);

				dynAppend(dsDpParameters, sArchiveActive);
				
				if(fArchiveTimeFilter < 0)
					dynAppend(dsDpParameters, UN_CONFIG_EXPORT_UNDEF);
				else	
					dynAppend(dsDpParameters, fArchiveTimeFilter);
				
				iLenj=dynlen(dsSmoothDpParameters);
				for(j=1; j<=iLenj ; j++)
					{
					dynAppend(dsDpParameters, dsSmoothDpParameters[j]);
					}		
				
				iLenj=dynlen(dsArchiveNames);
				for(j=1; j<=iLenj ; j++)
					{
					dynAppend(dsDpParameters, dsArchiveNames[j]);
					}	
				unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
				}//dp doesn't exist
			}//end for
}

//------------------------------------------------------------------------------------------------------------------------
// CMW_WordStatus_ExportConfig
/**
Purpose: Export WordStatus Devices for _UnPlc front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/

void CMW_WordStatus_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
dyn_string dsAllCommonParameters, dsSmoothDpParameters, dsLimits , dsDpParameters, dsArchiveNames;
string sObject, sUnit, sFormatToConvert, sFormatConverted, sArchiveActive, sMin, sMax, sArchiveName;
float fMinValue;
float fMaxValue, fDeadband, fDeadbandTemp, fTimeInterval, fArchiveTimeFilter ;
int iSmoothProcedure, iDeadBandType, i, j, iNormalPosition, iLeni, iLenj;
bool bDoesExist, bNegateRange, bIgnoreOutside,  bInclusiveMin , bInclusiveMax , bIsSmoothDefined ;
string sVersion, sMask;
int iMask;

		sVersion = frontEndVersion.text();
		sObject="WordStatus";
		g_iAlertRange = 0;
		
		iLeni=dynlen(dsDpList);
		for(i=1; i<=iLeni; i++)
			{									
			if (dpExists(dsDpList[i]))
				{
				dynClear(dsAllCommonParameters);
				dynClear(dsSmoothDpParameters);
				dynClear(dsLimits);		
				dynClear(dsDpParameters);
				dynClear(dsArchiveNames);
					
				//Get all common Parameters
				unExportDevice_getAllCommonParameters(dsDpList[i], sObject ,  dsAllCommonParameters);
				dsDpParameters = dsAllCommonParameters;
						
				//Get DP unit
				sUnit =  dpGetUnit(dsDpList[i] + ".ProcessInput.AuPosRSt" );
				
				//Get Format
				sFormatToConvert = dpGetFormat(dsDpList[i] + ".ProcessInput.AuPosRSt");
				sFormatConverted = unExportDevice_convertFormat(sFormatToConvert, "#");
				
				//Get RangeMax and RangeMin
				fwPvRange_get( dsDpList[i] + ".ProcessInput.AuPosRSt", bDoesExist , fMinValue , fMaxValue , bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax , exceptionInfo);
				
				sMax = unExportDevice_floatFormat(fMaxValue);			
				sMin = unExportDevice_floatFormat(fMinValue);
				
				//Get smoothProcedure, fDeadband & timeInterval
				unExportDevice_fwSmoothing_getTemp(dsDpList[i] + ".ProcessInput.AuPosRSt", bIsSmoothDefined, iSmoothProcedure, fDeadbandTemp, fTimeInterval);
				
				//for Max and Min: force convert from string to float
				//Get unicos DeadBand
				unExportDevice_getDeadBand(fDeadbandTemp, bIsSmoothDefined, sMax, sMin, iSmoothProcedure, fDeadband, iDeadBandType );	
								
				//Get Archive Parameters
				unExportDevice_getArchiveParametersDpe(dsDpList[i] + ".ProcessInput.AuPosRSt", sArchiveActive, fArchiveTimeFilter);
											
				//Get Archive Names
				unExportDevice_getArchiveNamesDpe(dsDpList[i], makeDynString(UN_CONFIG_EXPORT_UNDEF, ".ProcessInput.AuPosRSt", UN_CONFIG_EXPORT_UNDEF), 
																		dsArchiveNames);

				//Get address config		
				unConfigCMW_getAddressConfig(dsDpList[i]+".ProcessInput.AuPosRSt", dsSmoothDpParameters );
		
				dynAppend(dsDpParameters, sUnit);
				dynAppend(dsDpParameters, sFormatConverted);
								
				if (iDeadBandType==DPCONFIG_NONE)
					dynAppend(dsDpParameters, UN_CONFIG_EXPORT_UNDEF);
				else
					dynAppend(dsDpParameters, fDeadband);
				
				dynAppend(dsDpParameters, iDeadBandType);
				dynAppend(dsDpParameters, sMax);
				dynAppend(dsDpParameters, sMin);

				dynAppend(dsDpParameters, sArchiveActive);
				
				if(fArchiveTimeFilter < 0)
					dynAppend(dsDpParameters, UN_CONFIG_EXPORT_UNDEF);
				else	
					dynAppend(dsDpParameters, fArchiveTimeFilter);
				
				iLenj=dynlen(dsSmoothDpParameters);
				for(j=1; j<=iLenj ; j++)
					{
					dynAppend(dsDpParameters, dsSmoothDpParameters[j]);
					}		
				
				iLenj=dynlen(dsArchiveNames);
				for(j=1; j<=iLenj ; j++)
					{
					dynAppend(dsDpParameters, dsArchiveNames[j]);
					}	
				unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
				}//dp doesn't exist
			}//end for
}

//------------------------------------------------------------------------------------------------------------------------
// SOFT_FE_AnalogInput_ExportConfig
/**
Purpose: Export AnalogInput Devices for _UnPlc front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void SOFT_FE_AnalogInput_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	dyn_string dsAllCommonParameters, dsSmoothDpParameters, dsLimits , dsDpParameters, dsArchiveNames;
	string sObject, sUnit, sFormatToConvert, sFormatConverted, sArchiveActive, sMin, sMax, sArchiveName;
	float fMinValue, fMaxValue, fDeadband, fDeadbandTemp, fTimeInterval, fArchiveTimeFilter ;
	int iSmoothProcedure, iDeadBandType, i, j, iNormalPosition, iLeni, iLenj;
	bool bDoesExist, bNegateRange, bIgnoreOutside,  bInclusiveMin , bInclusiveMax , bIsSmoothDefined ;
	string sVersion, sMask;
	int iMask;

		sVersion = frontEndVersion.text();
		sObject="AnalogInput";
		g_iAlertRange = 0;
		
		iLeni=dynlen(dsDpList);
		for(i=1; i<=iLeni; i++)
			{									
			if (dpExists(dsDpList[i]))
				{
				dynClear(dsAllCommonParameters);
				dynClear(dsSmoothDpParameters);
				dynClear(dsLimits);		
				dynClear(dsDpParameters);
				dynClear(dsArchiveNames);
					
				//Get all common Parameters
				unExportDevice_getAllCommonParameters(dsDpList[i], sObject ,  dsAllCommonParameters);
				dsDpParameters = dsAllCommonParameters;
						
				//Get DP unit
				sUnit =  dpGetUnit(dsDpList[i] + ".ProcessInput.PosSt" );
				
				//Get Format
				sFormatToConvert = dpGetFormat(dsDpList[i] + ".ProcessInput.PosSt");
				sFormatConverted = unExportDevice_convertFormat(sFormatToConvert, "#");
				
				//Get RangeMax and RangeMin
				fwPvRange_get( dsDpList[i] + ".ProcessInput.PosSt", bDoesExist , fMinValue , fMaxValue , bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax , exceptionInfo);
				
				sMax = unExportDevice_floatFormat(fMaxValue);			
				sMin = unExportDevice_floatFormat(fMinValue);
				
				//Get Limits
				unExportDevice_getLimits(dsDpList[i], dsLimits, iNormalPosition, sMax, sMin);
				
				//Get Archive Parameters
				unExportDevice_getArchiveParameters(dsDpList[i], sObject, sArchiveActive, fArchiveTimeFilter);
											
				//Get Archive Names
				unCPCExportDevice_getArchiveNames(dsDpList[i], sObject, dsArchiveNames);
						
				dynAppend(dsDpParameters, sUnit);
				dynAppend(dsDpParameters, sFormatConverted);
				dynAppend(dsDpParameters, sMax);
				dynAppend(dsDpParameters, sMin);
								
				dynAppend(dsDpParameters, dsLimits[1]);
				dynAppend(dsDpParameters, dsLimits[2]);
				dynAppend(dsDpParameters, dsLimits[3]);
				dynAppend(dsDpParameters, dsLimits[4]);
								
				dynAppend(dsDpParameters, sArchiveActive);
				
				if(fArchiveTimeFilter < 0)
					dynAppend(dsDpParameters, UN_CONFIG_EXPORT_UNDEF);
				else	
					dynAppend(dsDpParameters, fArchiveTimeFilter);
				
				dynAppend(dsDpParameters, iNormalPosition);
				
				iLenj=dynlen(dsArchiveNames);
				for(j=1; j<=iLenj ; j++)
					{
					dynAppend(dsDpParameters, dsArchiveNames[j]);
					}	
				unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
				}//dp doesn't exist
			}//end for
}

//------------------------------------------------------------------------------------------------------------------------
// SOFT_FE_DigitalInput_ExportConfig
/**
Purpose: Export DigitalInput Devices for _UnPlc front-end  

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
SOFT_FE_DigitalInput_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
string sDp, sArchiveActive, sObject, sArchiveName;
dyn_string dsAllCommonParameters, smoothDpParameters, dsDpParameters, dsArchiveNames;
float fArchiveTimeFilter ;
int i, j, iNormalPosition, iLeni, iLenj;
string sVersion, sMask;
int iMask;

		sVersion = frontEndVersion.text();
		sObject="DigitalInput";
		
		iLeni=dynlen(dsDpList);
		for(i=1; i<=iLeni; i++)
			{									
			if (dpExists(dsDpList[i]))
				{
				dynClear(dsAllCommonParameters);
				dynClear(smoothDpParameters);
				dynClear(dsDpParameters);
				dynClear(dsArchiveNames);
				
				//Get all common Parameters
				unExportDevice_getAllCommonParameters(dsDpList[i], sObject,  dsAllCommonParameters);
				dsDpParameters = dsAllCommonParameters;
												
				//Get Archive Parameters
				unExportDevice_getArchiveParameters(dsDpList[i], sObject, sArchiveActive, fArchiveTimeFilter, sVersion);
				
				//Get Archive Names
				unCPCExportDevice_getArchiveNames(dsDpList[i], sObject, dsArchiveNames);
											
				//Get Normal Position
				unCPCExportDevice_getDigitalNormalPosition(dsDpList[i], sObject, iNormalPosition);
				
				dynAppend(dsDpParameters, sArchiveActive);
				
				dynAppend(dsDpParameters, iNormalPosition);
				
				iLenj=dynlen(dsArchiveNames);
				for(j=1; j<=iLenj ; j++)
					{
					dynAppend(dsDpParameters, dsArchiveNames[j]);
					}	
				unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
				}
			}	
}

//------------------------------------------------------------------------------------------------------------------------
// SOFT_FE_AnalogStatus_ExportConfig
/**
Purpose: Export AnalogStatus Devices for _UnPlc front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void SOFT_FE_AnalogStatus_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
dyn_string dsAllCommonParameters, dsSmoothDpParameters, dsLimits , dsDpParameters, dsArchiveNames;
string sObject, sUnit, sFormatToConvert, sFormatConverted, sArchiveActive, sMin, sMax, sArchiveName;
float fMinValue, fMaxValue, fDeadband, fDeadbandTemp, fTimeInterval, fArchiveTimeFilter ;
int iSmoothProcedure, iDeadBandType, i, j, iNormalPosition, iLeni, iLenj;
bool bDoesExist, bNegateRange, bIgnoreOutside,  bInclusiveMin , bInclusiveMax , bIsSmoothDefined ;
string sVersion, sMask;
int iMask;

		sVersion = frontEndVersion.text();
		sObject="AnalogStatus";
		g_iAlertRange = 0;
		
		iLeni=dynlen(dsDpList);
		for(i=1; i<=iLeni; i++)
			{									
			if (dpExists(dsDpList[i]))
				{
				dynClear(dsAllCommonParameters);
				dynClear(dsSmoothDpParameters);
				dynClear(dsLimits);		
				dynClear(dsDpParameters);
				dynClear(dsArchiveNames);
					
				//Get all common Parameters
				unExportDevice_getAllCommonParameters(dsDpList[i], sObject ,  dsAllCommonParameters);
				dsDpParameters = dsAllCommonParameters;
						
				//Get DP unit
				sUnit =  dpGetUnit(dsDpList[i] + ".ProcessInput.AuPosRSt" );
				
				//Get Format
				sFormatToConvert = dpGetFormat(dsDpList[i] + ".ProcessInput.AuPosRSt");
				sFormatConverted = unExportDevice_convertFormat(sFormatToConvert, "#");
				
				//Get RangeMax and RangeMin
				fwPvRange_get( dsDpList[i] + ".ProcessInput.AuPosRSt", bDoesExist , fMinValue , fMaxValue , bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax , exceptionInfo);
				
				sMax = unExportDevice_floatFormat(fMaxValue);			
				sMin = unExportDevice_floatFormat(fMinValue);
				
				//Get Archive Parameters
				unExportDevice_getArchiveParametersDpe(dsDpList[i] + ".ProcessInput.AuPosRSt", sArchiveActive, fArchiveTimeFilter);
											
				//Get Archive Names
				unExportDevice_getArchiveNamesDpe(dsDpList[i], makeDynString(UN_CONFIG_EXPORT_UNDEF, ".ProcessInput.AuPosRSt", UN_CONFIG_EXPORT_UNDEF), 
																		dsArchiveNames);

				dynAppend(dsDpParameters, sUnit);
				dynAppend(dsDpParameters, sFormatConverted);
								
				dynAppend(dsDpParameters, sMax);
				dynAppend(dsDpParameters, sMin);

				dynAppend(dsDpParameters, sArchiveActive);
				
				if(fArchiveTimeFilter < 0)
					dynAppend(dsDpParameters, UN_CONFIG_EXPORT_UNDEF);
				else	
					dynAppend(dsDpParameters, fArchiveTimeFilter);
				
				iLenj=dynlen(dsArchiveNames);
				for(j=1; j<=iLenj ; j++)
					{
					dynAppend(dsDpParameters, dsArchiveNames[j]);
					}	
				unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
				}//dp doesn't exist
			}//end for
}

//------------------------------------------------------------------------------------------------------------------------
// SOFT_FE_WordStatus_ExportConfig
/**
Purpose: Export WordStatus Devices for _UnPlc front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void SOFT_FE_WordStatus_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
dyn_string dsAllCommonParameters, dsSmoothDpParameters, dsLimits , dsDpParameters, dsArchiveNames;
string sObject, sUnit, sFormatToConvert, sFormatConverted, sArchiveActive, sMin, sMax, sArchiveName;
float fMinValue;
float fMaxValue, fDeadband, fDeadbandTemp, fTimeInterval, fArchiveTimeFilter ;
int iSmoothProcedure, iDeadBandType, i, j, iNormalPosition, iLeni, iLenj;
bool bDoesExist, bNegateRange, bIgnoreOutside,  bInclusiveMin , bInclusiveMax , bIsSmoothDefined ;
string sVersion, sMask;
int iMask;

		sVersion = frontEndVersion.text();
		sObject="WordStatus";
		g_iAlertRange = 0;
		
		iLeni=dynlen(dsDpList);
		for(i=1; i<=iLeni; i++)
			{									
			if (dpExists(dsDpList[i]))
				{
				dynClear(dsAllCommonParameters);
				dynClear(dsSmoothDpParameters);
				dynClear(dsLimits);		
				dynClear(dsDpParameters);
				dynClear(dsArchiveNames);
					
				//Get all common Parameters
				unExportDevice_getAllCommonParameters(dsDpList[i], sObject ,  dsAllCommonParameters);
				dsDpParameters = dsAllCommonParameters;
						
				//Get DP unit
				sUnit =  dpGetUnit(dsDpList[i] + ".ProcessInput.AuPosRSt" );
				
				//Get Format
				sFormatToConvert = dpGetFormat(dsDpList[i] + ".ProcessInput.AuPosRSt");
				sFormatConverted = unExportDevice_convertFormat(sFormatToConvert, "#");
				
				//Get RangeMax and RangeMin
				fwPvRange_get( dsDpList[i] + ".ProcessInput.AuPosRSt", bDoesExist , fMinValue , fMaxValue , bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax , exceptionInfo);
				
				sMax = unExportDevice_floatFormat(fMaxValue);			
				sMin = unExportDevice_floatFormat(fMinValue);
				
				//Get Archive Parameters
				unExportDevice_getArchiveParametersDpe(dsDpList[i] + ".ProcessInput.AuPosRSt", sArchiveActive, fArchiveTimeFilter);
											
				//Get Archive Names
				unExportDevice_getArchiveNamesDpe(dsDpList[i], makeDynString(UN_CONFIG_EXPORT_UNDEF, ".ProcessInput.AuPosRSt", UN_CONFIG_EXPORT_UNDEF), 
																		dsArchiveNames);

				dynAppend(dsDpParameters, sUnit);
				dynAppend(dsDpParameters, sFormatConverted);
								
				dynAppend(dsDpParameters, sMax);
				dynAppend(dsDpParameters, sMin);

				dynAppend(dsDpParameters, sArchiveActive);
				
				if(fArchiveTimeFilter < 0)
					dynAppend(dsDpParameters, UN_CONFIG_EXPORT_UNDEF);
				else	
					dynAppend(dsDpParameters, fArchiveTimeFilter);
      // get the bit description
      dynAppend(dsDpParameters, dpGetDescription(dsDpList[i]+".ProcessInput.AuPosRSt"));
				
				iLenj=dynlen(dsArchiveNames);
				for(j=1; j<=iLenj ; j++)
					{
					dynAppend(dsDpParameters, dsArchiveNames[j]);
					}	
				unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
				}//dp doesn't exist
			}//end for
}

//------------------------------------------------------------------------------------------------------------------------
// SOFT_FE_AnalogParameter_ExportConfig
/**
Purpose: Export AnalogParameter Devices for _UnPlc front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void SOFT_FE_AnalogParameter_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
dyn_string dsAllCommonParameters, dsSmoothDpParameters, dsLimits , dsDpParameters, dsArchiveNames;
string sObject, sUnit, sFormatToConvert, sFormatConverted, sArchiveActive, sMin, sMax, sDef, sAuto, sArchiveName;
float fMinValue, fMaxValue, fDefValue, fDeadband, fDeadbandTemp, fTimeInterval, fArchiveTimeFilter ;
int iSmoothProcedure, iDeadBandType, i, j, iNormalPosition, iLeni, iLenj, iAuto,iResult,iType;
bool bDoesExist, bNegateRange, bIgnoreOutside,  bInclusiveMin , bInclusiveMax , bIsSmoothDefined ;
string sVersion, sMask;
int iMask;

		sVersion = frontEndVersion.text();
		sObject="AnalogParameter";
		g_iAlertRange = 0;
		
		iLeni=dynlen(dsDpList);
		for(i=1; i<=iLeni; i++)
			{									
			if (dpExists(dsDpList[i]))
				{
				dynClear(dsAllCommonParameters);
				dynClear(dsSmoothDpParameters);
				dynClear(dsLimits);		
				dynClear(dsDpParameters);
				dynClear(dsArchiveNames);
					
				//Get all common Parameters
				unExportDevice_getAllCommonParameters(dsDpList[i], sObject ,  dsAllCommonParameters);
				dsDpParameters = dsAllCommonParameters;
						
				//Get DP unit
				sUnit =  dpGetUnit(dsDpList[i] + ".ProcessInput.CurValSt" );
				
				//Get Format
				sFormatToConvert = dpGetFormat(dsDpList[i] + ".ProcessInput.CurValSt");
				sFormatConverted = unExportDevice_convertFormat(sFormatToConvert, "#");
				
				//Get RangeMax and RangeMin
				fwPvRange_get( dsDpList[i] + ".ProcessInput.CurValSt", bDoesExist , fMinValue , fMaxValue , bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax , exceptionInfo);
				dpGet(dsDpList[i] + ".ProcessOutput.DfltVal", fDefValue);
				
				sMax = unExportDevice_floatFormat(fMaxValue);			
				sMin = unExportDevice_floatFormat(fMinValue);
				sDef = unExportDevice_floatFormat(fDefValue);
 
 
 				// iRes=dpGet(sDp + ":_dp_fct.._param", dsDpe);
 				iResult=dpGet(dsDpList[i] + ".ProcessInput.CurValSt:_dp_fct.._type", iType);
				// DebugN("&&&& dpGet: "+dsDpList[i] + ".ProcessInput.CurValSt:_dp_fct.._type");
				// DebugN("&&&& iResult= "+iType);
				
 				if (iType==DPCONFIG_DP_FUNCTION)
					sAuto="1";
				else
					sAuto="0";
				
				//Get Archive Parameters
				unExportDevice_getArchiveParametersDpe(dsDpList[i] + ".ProcessInput.CurValSt", sArchiveActive, fArchiveTimeFilter);
											
				//Get Archive Names
				unExportDevice_getArchiveNamesDpe(dsDpList[i], makeDynString(UN_CONFIG_EXPORT_UNDEF, ".ProcessInput.CurValSt", UN_CONFIG_EXPORT_UNDEF), 
																		dsArchiveNames);

				dynAppend(dsDpParameters, sUnit);
				dynAppend(dsDpParameters, sFormatConverted);
								
				dynAppend(dsDpParameters, sMax);
				dynAppend(dsDpParameters, sMin);
				dynAppend(dsDpParameters, sDef);
				
				dynAppend(dsDpParameters, sAuto);
				
				dynAppend(dsDpParameters, sArchiveActive);
				
				if(fArchiveTimeFilter < 0)
					dynAppend(dsDpParameters, UN_CONFIG_EXPORT_UNDEF);
				else	
					dynAppend(dsDpParameters, fArchiveTimeFilter);
				
				iLenj=dynlen(dsArchiveNames);
				for(j=1; j<=iLenj ; j++)
					{
					dynAppend(dsDpParameters, dsArchiveNames[j]);
					}	
				unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
				}//dp doesn't exist
			}//end for
}

//------------------------------------------------------------------------------------------------------------------------
// SOFT_FE_DigitalParameter_ExportConfig
/**
Purpose: Export DigitalParameter Devices for SOFT-FE front-end

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void SOFT_FE_DigitalParameter_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
dyn_string dsAllCommonParameters, dsSmoothDpParameters, dsLimits , dsDpParameters, dsArchiveNames;
string sObject, sUnit, sFormatToConvert, sFormatConverted, sArchiveActive, sMin, sMax, sDef, sAuto, sArchiveName;
float fMinValue, fMaxValue, fDefValue, fDeadband, fDeadbandTemp, fTimeInterval, fArchiveTimeFilter ;
int iSmoothProcedure, iDeadBandType, i, j, iNormalPosition, iLeni, iLenj, iAuto,iResult,iType;
bool bDoesExist, bNegateRange, bIgnoreOutside,  bInclusiveMin , bInclusiveMax , bIsSmoothDefined ;
string sVersion, sMask;
int iMask;

		sVersion = frontEndVersion.text();
		sObject="DigitalParameter";
		g_iAlertRange = 0;
		
		iLeni=dynlen(dsDpList);
		for(i=1; i<=iLeni; i++)
			{									
			if (dpExists(dsDpList[i]))
				{
				dynClear(dsAllCommonParameters);
				dynClear(dsSmoothDpParameters);
				dynClear(dsLimits);		
				dynClear(dsDpParameters);
				dynClear(dsArchiveNames);
					
				//Get all common Parameters
				unExportDevice_getAllCommonParameters(dsDpList[i], sObject ,  dsAllCommonParameters);
				dsDpParameters = dsAllCommonParameters;
						
				//Get DP unit
				sUnit =  dpGetUnit(dsDpList[i] + ".ProcessInput.CurValSt" );
				
				//Get Format
				sFormatToConvert = dpGetFormat(dsDpList[i] + ".ProcessInput.CurValSt");
				sFormatConverted = unExportDevice_convertFormat(sFormatToConvert, "#");
				
				//Get RangeMax and RangeMin
				fwPvRange_get( dsDpList[i] + ".ProcessInput.CurValSt", bDoesExist , fMinValue , fMaxValue , bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax , exceptionInfo);
				dpGet(dsDpList[i] + ".ProcessOutput.DfltVal", fDefValue);
				
				sMax = unExportDevice_floatFormat(fMaxValue);			
				sMin = unExportDevice_floatFormat(fMinValue);
				sDef = unExportDevice_floatFormat(fDefValue);
 
 
 				// iRes=dpGet(sDp + ":_dp_fct.._param", dsDpe);
 				iResult=dpGet(dsDpList[i] + ".ProcessInput.CurValSt:_dp_fct.._type", iType);
				// DebugN("&&&& dpGet: "+dsDpList[i] + ".ProcessInput.CurValSt:_dp_fct.._type");
				// DebugN("&&&& iResult= "+iType);
				
 				if (iType==DPCONFIG_DP_FUNCTION)
					sAuto="1";
				else
					sAuto="0";
				
				//Get Archive Parameters
				unExportDevice_getArchiveParametersDpe(dsDpList[i] + ".ProcessInput.CurValSt", sArchiveActive, fArchiveTimeFilter);
											
				//Get Archive Names
				unExportDevice_getArchiveNamesDpe(dsDpList[i], makeDynString(UN_CONFIG_EXPORT_UNDEF, ".ProcessInput.CurValSt", UN_CONFIG_EXPORT_UNDEF), 
																		dsArchiveNames);

				dynAppend(dsDpParameters, sUnit);
				dynAppend(dsDpParameters, sFormatConverted);
								
				dynAppend(dsDpParameters, sMax);
				dynAppend(dsDpParameters, sMin);
				dynAppend(dsDpParameters, sDef);
				
				dynAppend(dsDpParameters, sAuto);
				
				dynAppend(dsDpParameters, sArchiveActive);
				
				if(fArchiveTimeFilter < 0)
					dynAppend(dsDpParameters, UN_CONFIG_EXPORT_UNDEF);
				else	
					dynAppend(dsDpParameters, fArchiveTimeFilter);
				
				iLenj=dynlen(dsArchiveNames);
				for(j=1; j<=iLenj ; j++)
					{
					dynAppend(dsDpParameters, dsArchiveNames[j]);
					}	
				unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
				}//dp doesn't exist
			}//end for
}

//------------------------------------------------------------------------------------------------------------------------
// OPC_AnalogInput_ExportConfig
/**
Purpose: Export AnalogInput Devices 

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void OPC_AnalogInput_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	_UnPlc_AnalogInput_ExportConfig(dsDpList, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// OPC_DigitalInput_ExportConfig
/**
Purpose: Export DigitalInput Devices 

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void OPC_DigitalInput_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
	string sDp, sArchiveActive, sObject, sArchiveName;
	dyn_string dsAllCommonParameters, smoothDpParameters, dsDpParameters, dsArchiveNames;
	float fArchiveTimeFilter ;
	int i, j, iNormalPosition, iLeni, iLenj;

	sObject="DigitalInput";
	
	iLeni=dynlen(dsDpList);
	for(i=1; i<=iLeni; i++)
	{									
		if (dpExists(dsDpList[i]))
		{
			dynClear(dsAllCommonParameters);
			dynClear(smoothDpParameters);
			dynClear(dsDpParameters);
			dynClear(dsArchiveNames);
			
			//Get all common Parameters
			unExportDevice_getAllCommonParameters(dsDpList[i], sObject,  dsAllCommonParameters);
			dsDpParameters = dsAllCommonParameters;
											
			//Get Archive Parameters
			unExportDevice_getArchiveParameters(dsDpList[i], sObject, sArchiveActive, fArchiveTimeFilter);
			
			//Get Archive Names
			unCPCExportDevice_getArchiveNames(dsDpList[i], sObject, dsArchiveNames);
										
			//Get address config		
			unCPCExportDevice_getAddressConfig(dsDpList[i], sObject, smoothDpParameters );
			// get PosSt address config and insert it before the ManReg01 (last parameter)
			dynInsertAt(smoothDpParameters, 
									DRV_OPCCLIENT_convertToUnicosAddress(dsDpList[i] + ".ProcessInput.PosSt", "DigitalInput"),
									dynlen(smoothDpParameters));
			//Get Normal Position
			unCPCExportDevice_getDigitalNormalPosition(dsDpList[i], sObject, iNormalPosition);
			
			dynAppend(dsDpParameters, sArchiveActive);
			
			dynAppend(dsDpParameters, iNormalPosition);
			
			iLenj=dynlen(smoothDpParameters);
			for(j=1; j<=iLenj ; j++)
			{
				dynAppend(dsDpParameters, smoothDpParameters[j]);
			}
				
			iLenj=dynlen(dsArchiveNames);
			for(j=1; j<=iLenj ; j++)
			{
				dynAppend(dsDpParameters, dsArchiveNames[j]);
			}	
															
			unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
		}
	}	
}

//------------------------------------------------------------------------------------------------------------------------
// unCPCExportDevice_getDigitalNormalPosition
/**
Purpose: Get Normal Position of a Digital dp

Parameters:

Usage: Internal function

PVSS manager usage: NG, NV
*/
void unCPCExportDevice_getDigitalNormalPosition(string sDp, string sObject, int &iNormalPosition)
{
bool bAlarmMask, bOkRange, bBoolAlert,bSMS;
string sAlertClass1,sAlertClass2;
int iAlertType, iAlertType_2, iAlertType_3, iRes;
string sDataPoint;

	iNormalPosition = -1;
	
	switch (sObject)
		{
		case "Alarm":
		case "Local":
			sDataPoint = sDp + ".ProcessInput.PosAl";
			iRes = dpGet(sDataPoint + ":_alert_hdl.._type", iAlertType);
			break;
			
		case "OnOff":
		case "Analog":
		case "AnaDig":
			sDataPoint = sDp + ".ProcessInput.StartISt";
			iRes = dpGet(sDataPoint + ":_alert_hdl.._type", iAlertType);			
			sDataPoint = sDp + ".ProcessInput.StopISt";
			iRes = dpGet(sDataPoint + ":_alert_hdl.._type", iAlertType_2);
			
			if (iAlertType!=iAlertType_2)
				iAlertType == DPCONFIG_NONE;
			break;
		
		case "ProcessControlObject":
			sDataPoint = sDp + ".ProcessInput.FuStopAl";
			iRes = dpGet(sDataPoint + ":_alert_hdl.._type", iAlertType);
			sDataPoint = sDp + ".ProcessInput.TStopAl";
			iRes = dpGet(sDataPoint + ":_alert_hdl.._type", iAlertType_2);
			sDataPoint = sDp + ".ProcessInput.StartAl";
			iRes = dpGet(sDataPoint + ":_alert_hdl.._type", iAlertType_3);
			
			if ((iAlertType!=iAlertType_2) || (iAlertType!=iAlertType_3))
				iAlertType == DPCONFIG_NONE;
			break;

		default:
			sDataPoint = sDp + ".ProcessInput.PosSt";
			if (dpExists(sDataPoint))
				iRes = dpGet(sDataPoint + ":_alert_hdl.._type", iAlertType);
			else
				iAlertType == DPCONFIG_NONE;
			break;
		}
		
	if(iAlertType == DPCONFIG_NONE)
		{
		if(bSMS)iNormalPosition = 102;
		else iNormalPosition = 2;
		}
	else
		{
		bBoolAlert = ((iRes >= 0) && (iAlertType == DPCONFIG_ALERT_BINARYSIGNAL)); 
		
		if (bBoolAlert) 
			{ 
			iRes = dpGet(sDataPoint + ":_alert_hdl.._ok_range", bOkRange);
			}
		
		iRes = dpGet(sDataPoint + ":_alert_hdl.._active",  bAlarmMask);
		iRes = dpGet(sDataPoint + ":_alert_hdl.1._class",  sAlertClass1);
		iRes = dpGet(sDataPoint + ":_alert_hdl.2._class",  sAlertClass2);
		
	if((patternMatch(UN_PROCESSALARM_PATTERN,sAlertClass1))||(patternMatch(UN_PROCESSALARM_PATTERN,sAlertClass2)))
		bSMS = true;
	else bSMS = false;		
//	DebugN(bSMS, sAlertClass1,sAlertClass2);
	
		if(bAlarmMask)
			{ //Alarm unmasked
			if(bOkRange){//ON
			if(bSMS) iNormalPosition =101;
				else iNormalPosition = 1;
			}
			else //OFF
			{
			if(bSMS) iNormalPosition = 100;
			else iNormalPosition = 0;
			}
		}
		else
			{ //Alarm masked
			if(bOkRange)//ON
			{
			if (bSMS) iNormalPosition =104;
				else iNormalPosition = 4;
				}
			else//OFF
			if (bSMS) iNormalPosition = 103;
				else iNormalPosition = 3;
			}	
		}
} 

//------------------------------------------------------------------------------------------------------------------------
// unCPCExportDevice_getArchiveNames
/**
Purpose: Get Archives names (Bool, Analog and Event values)

Parameters:

Usage: Internal function

PVSS manager usage: NG, NV
*/
void unCPCExportDevice_getArchiveNames(string sDp, string sObject, dyn_string &dsNames)
{
dyn_string dsDpe;       
    
  switch(sObject)
    {
              case "Alarm":
			dynAppend(dsDpe, ".ProcessInput.PosAl" + DPE_ARCHIVE_DELIMITER);
			dynAppend(dsDpe, UN_CONFIG_EXPORT_UNDEF);			
			dynAppend(dsDpe, ".ProcessInput.evStsReg01" + DPE_ARCHIVE_DELIMITER);
			break;
                        
		case "Local":
		case "ProcessControlObject":
			dynAppend(dsDpe, UN_CONFIG_EXPORT_UNDEF);
			dynAppend(dsDpe, UN_CONFIG_EXPORT_UNDEF);			
			dynAppend(dsDpe, ".ProcessInput.evStsReg01" + DPE_ARCHIVE_DELIMITER);
			break;
			
		case "OnOff":
			dynAppend(dsDpe, UN_CONFIG_EXPORT_UNDEF);
			dynAppend(dsDpe, UN_CONFIG_EXPORT_UNDEF);			
			dynAppend(dsDpe, ".ProcessInput.evStsReg01" + DPE_ARCHIVE_DELIMITER + ".ProcessInput.evStsReg02");
			break;
		
		case "AnaDig":
			dynAppend(dsDpe, UN_CONFIG_EXPORT_UNDEF);
			dynAppend(dsDpe, ".ProcessInput.PosSt" + DPE_ARCHIVE_DELIMITER + ".ProcessInput.OutOVSt");			
			dynAppend(dsDpe, ".ProcessInput.evStsReg01" + DPE_ARCHIVE_DELIMITER + ".ProcessInput.evStsReg02");
			break;
		
		case "Analog":
			dynAppend(dsDpe, UN_CONFIG_EXPORT_UNDEF);
			dynAppend(dsDpe, ".ProcessInput.PosSt" + DPE_ARCHIVE_DELIMITER + ".ProcessInput.OutOVSt");			
			dynAppend(dsDpe, ".ProcessInput.evStsReg01" + DPE_ARCHIVE_DELIMITER);
			break;
		
		case "AnalogInput":
		case "AnalogOutput":
			dynAppend(dsDpe, UN_CONFIG_EXPORT_UNDEF);
			dynAppend(dsDpe, ".ProcessInput.PosSt" + DPE_ARCHIVE_DELIMITER);			
			dynAppend(dsDpe, ".ProcessInput.evStsReg01" + DPE_ARCHIVE_DELIMITER);
			break;
		
		case "DigitalInput":
		case "DigitalOutput":
			dynAppend(dsDpe, ".ProcessInput.PosSt" + DPE_ARCHIVE_DELIMITER);
			dynAppend(dsDpe, UN_CONFIG_EXPORT_UNDEF);			
			dynAppend(dsDpe, ".ProcessInput.evStsReg01" + DPE_ARCHIVE_DELIMITER);
			break;

		case "Controller":
			dynAppend(dsDpe, UN_CONFIG_EXPORT_UNDEF);
			dynAppend(dsDpe, ".ProcessInput.MV" + DPE_ARCHIVE_DELIMITER + ".ProcessInput.MV" + DPE_ARCHIVE_DELIMITER + ".ProcessInput.AtSPo");			
			dynAppend(dsDpe, ".ProcessInput.evStsReg01" + DPE_ARCHIVE_DELIMITER + ".ProcessInput.evStsReg02");
			break;
			
		default:
			dynAppend(dsDpe, UN_CONFIG_EXPORT_UNDEF);
			dynAppend(dsDpe, UN_CONFIG_EXPORT_UNDEF);			
			dynAppend(dsDpe, UN_CONFIG_EXPORT_UNDEF);			
			break;
		}
		
	unExportDevice_getArchiveNamesDpe(sDp, dsDpe, dsNames);
		
}

//------------------------------------------------------------------------------------------------------------------------
// unCPCExportDevice_getAddressConfig
/**
Purpose:  Get address config parameters of a dp

Parameters:

Usage: Internal function

PVSS manager usage: NG, NV
*/
void unCPCExportDevice_getAddressConfig(string sDp, string sObject, dyn_string &smoothDpParameters)
{
	int i=0, j=0, k, l, m, iDpError=-1, iLen, iLenInput, iTypeAddress;
	dyn_string dsInputValue, dsOutputValue,  deviceInputParameters, deviceOutputParameters;
	string temp, sFunction;
	bool bStsReg02Exist;
        
	dynClear(dsInputValue);
	dynClear(dsOutputValue);
	
        // DebugN("FUNCTION:: unCPCExportDevice_getAddressConfig(sDp=",sDp," sObject=",sObject);
        
	if(sObject=="AnalogInput" || sObject=="AnalogOutput")
		{
		//Input parameters
		dynAppend(deviceInputParameters, "StsReg01");
		dynAppend(deviceInputParameters, "evStsReg01");
		dynAppend(deviceInputParameters, "PosSt");
		dynAppend(deviceInputParameters, "HFStOrAuPosRSt");
		
		//Output parameters
		dynAppend(deviceOutputParameters, "ManReg01");
		dynAppend(deviceOutputParameters, "MPosR");	

		//Input parameters
		iLen=dynlen(deviceInputParameters);
		for(i=1; i<=iLen ; i++)
		{
			temp="";
			dpGet(sDp + ".ProcessInput." + deviceInputParameters[i] + ":_address.._drv_ident" , temp);
			sFunction = "DRV_"+temp+"_convertToUnicosAddress";
			if(isFunctionDefined(sFunction)) {
				evalScript(temp, "string main(string sDeviceDpeName, string sDeviceType) {"+
													"	string temp;"+
													"	temp = "+sFunction+"(sDeviceDpeName, sDeviceType);"+
													"	return temp;"+
													"}", makeDynString(), sDp + ".ProcessInput." + deviceInputParameters[i], sObject);
			}
			else
				temp = UN_CONFIG_EXPORT_UNDEF;
			dynAppend(smoothDpParameters, temp);
		}
		
		//Output parameters
		iLen=dynlen(deviceOutputParameters);
		for(j=1; j<=iLen; j++){
			temp="";
			dpGet(sDp + ".ProcessOutput." + deviceOutputParameters[j] + ":_address.._drv_ident" , temp);
			sFunction = "DRV_"+temp+"_convertToUnicosAddress";
			if(isFunctionDefined(sFunction)) {
				evalScript(temp, "string main(string sDeviceDpeName, string sDeviceType) {"+
													"	string temp;"+
													"	temp = "+sFunction+"(sDeviceDpeName, sDeviceType);"+
													"	return temp;"+
													"}", makeDynString(), sDp + ".ProcessOutput." + deviceOutputParameters[j], sObject);
			}
			else
				temp = UN_CONFIG_EXPORT_UNDEF;
			dynAppend(smoothDpParameters, temp);
		}
	}
	
	if(sObject=="DigitalInput"  || sObject=="DigitalOutput" || sObject=="Alarm" || sObject == "Local" )
	{
		//Input parameters
		dynAppend(deviceInputParameters, "StsReg01");
		dynAppend(deviceInputParameters, "evStsReg01");
		
		//Output parameters
		dynAppend(deviceOutputParameters, "ManReg01");
	
		//Input parameters
		iLen=dynlen(deviceInputParameters);
		for(i=1; i<=iLen ; i++)
		{
			temp="";
			dpGet(sDp + ".ProcessInput." + deviceInputParameters[i] + ":_address.._drv_ident" , temp);
			sFunction = "DRV_"+temp+"_convertToUnicosAddress";
			if(isFunctionDefined(sFunction)) {
				evalScript(temp, "string main(string sDeviceDpeName, string sDeviceType) {"+
													"	string temp;"+
													"	temp = "+sFunction+"(sDeviceDpeName, sDeviceType);"+
													"	return temp;"+
													"}", makeDynString(), sDp + ".ProcessInput." + deviceInputParameters[i], sObject);
			}
			else
				temp = UN_CONFIG_EXPORT_UNDEF;
			dynAppend(smoothDpParameters, temp);
		}
		
		//Output parameters
		iLen=dynlen(deviceOutputParameters);
		for(j=1; j<=iLen; j++)
		{
			temp="";
			dpGet(sDp + ".ProcessOutput." + deviceOutputParameters[j] + ":_address.._drv_ident" , temp);
			sFunction = "DRV_"+temp+"_convertToUnicosAddress";
			if(isFunctionDefined(sFunction)) {
				evalScript(temp, "string main(string sDeviceDpeName, string sDeviceType) {"+
													"	string temp;"+
													"	temp = "+sFunction+"(sDeviceDpeName, sDeviceType);"+
													"	return temp;"+
													"}", makeDynString(), sDp + ".ProcessOutput." + deviceOutputParameters[j], sObject);
			}
			else
				temp = UN_CONFIG_EXPORT_UNDEF;
			dynAppend(smoothDpParameters, temp);
		}
	}
	
	if(sObject=="OnOff")
	{
		//Input parameters
		dynAppend(deviceInputParameters, "StsReg01");
		dynAppend(deviceInputParameters, "StsReg02");
		dynAppend(deviceInputParameters, "evStsReg01");
		dynAppend(deviceInputParameters, "evStsReg02");
		
		//Output parameters
		dynAppend(deviceOutputParameters, "ManReg01");

		//Input parameters
		iLen=dynlen(deviceInputParameters);
		for(i=1; i<=iLen; i++)
		{
			temp="";
			dpGet(sDp + ".ProcessInput." + deviceInputParameters[i] + ":_address.._drv_ident" , temp);
			sFunction = "DRV_"+temp+"_convertToUnicosAddress";
			if(isFunctionDefined(sFunction)) {
				evalScript(temp, "string main(string sDeviceDpeName, string sDeviceType) {"+
													"	string temp;"+
													"	temp = "+sFunction+"(sDeviceDpeName, sDeviceType);"+
													"	return temp;"+
													"}", makeDynString(), sDp + ".ProcessInput." + deviceInputParameters[i], sObject);
			}
			else
				temp = UN_CONFIG_EXPORT_UNDEF;
			dynAppend(smoothDpParameters, temp);
		}
		
		//Output parameters
		iLen=dynlen(deviceOutputParameters);
		for(j=1; j<=iLen; j++)
		{
			temp="";
			dpGet(sDp + ".ProcessOutput." + deviceOutputParameters[j] + ":_address.._drv_ident" , temp);
			sFunction = "DRV_"+temp+"_convertToUnicosAddress";
			if(isFunctionDefined(sFunction)) {
				evalScript(temp, "string main(string sDeviceDpeName, string sDeviceType) {"+
													"	string temp;"+
													"	temp = "+sFunction+"(sDeviceDpeName, sDeviceType);"+
													"	return temp;"+
													"}", makeDynString(), sDp + ".ProcessOutput." + deviceOutputParameters[j], sObject);
			}
			else
				temp = UN_CONFIG_EXPORT_UNDEF;
			dynAppend(smoothDpParameters, temp);
		}
	}
	
	if(sObject=="Analog")
	{
		//Input parameters    
                bStsReg02Exist=false;
                dpGet(sDp + ".ProcessInput.StsReg02:_address.._type" , iTypeAddress);    
                // DebugN("dpGet("+sDp + ".ProcessInput.StsReg02:_address.._type)= ",iTypeAddress);  
                
                if (iTypeAddress != DPCONFIG_NONE)
                  bStsReg02Exist=true;
                                 
		dynAppend(deviceInputParameters, "StsReg01");
                if (bStsReg02Exist)
                  dynAppend(deviceInputParameters, "StsReg02");    
		dynAppend(deviceInputParameters, "evStsReg01");
                if (bStsReg02Exist)
                  dynAppend(deviceInputParameters, "evStsReg02");
		dynAppend(deviceInputParameters, "PosSt");
		dynAppend(deviceInputParameters, "AuPosRSt");
		dynAppend(deviceInputParameters, "MPosRSt");
		dynAppend(deviceInputParameters, "OutOVSt");
		
		//Output parameters
		dynAppend(deviceOutputParameters, "ManReg01");
		dynAppend(deviceOutputParameters, "MPosR");
		dynAppend(deviceOutputParameters, "PLiOn");
		dynAppend(deviceOutputParameters, "PLiOff");
		
		//Input parameters
		iLen=dynlen(deviceInputParameters);
		for(i=1; i<=iLen; i++)
		{
			temp="";
			dpGet(sDp + ".ProcessInput." + deviceInputParameters[i] + ":_address.._drv_ident" , temp);
			sFunction = "DRV_"+temp+"_convertToUnicosAddress";
			if(isFunctionDefined(sFunction)) {
				evalScript(temp, "string main(string sDeviceDpeName, string sDeviceType) {"+
													"	string temp;"+
													"	temp = "+sFunction+"(sDeviceDpeName, sDeviceType);"+
													"	return temp;"+
													"}", makeDynString(), sDp + ".ProcessInput." + deviceInputParameters[i], sObject);
			}
			else
				temp = UN_CONFIG_EXPORT_UNDEF;
			dynAppend(smoothDpParameters, temp);
		}
		
		//Output parameters
		iLen=dynlen(deviceOutputParameters);
		for(j=1; j<=iLen; j++)
		{
			temp="";
			dpGet(sDp + ".ProcessOutput." + deviceOutputParameters[j] + ":_address.._drv_ident" , temp);
			sFunction = "DRV_"+temp+"_convertToUnicosAddress";
			if(isFunctionDefined(sFunction)) {
				evalScript(temp, "string main(string sDeviceDpeName, string sDeviceType) {"+
													"	string temp;"+
													"	temp = "+sFunction+"(sDeviceDpeName, sDeviceType);"+
													"	return temp;"+
													"}", makeDynString(), sDp + ".ProcessOutput." + deviceOutputParameters[j], sObject);
			}
			else
				temp = UN_CONFIG_EXPORT_UNDEF;
			dynAppend(smoothDpParameters, temp);
		}
	}
	
	if(sObject=="AnaDig")
	{
		//Input parameters
		dynAppend(deviceInputParameters, "StsReg01");
		dynAppend(deviceInputParameters, "StsReg02");
		dynAppend(deviceInputParameters, "evStsReg01");
		dynAppend(deviceInputParameters, "evStsReg02");
		dynAppend(deviceInputParameters, "PosSt");
		dynAppend(deviceInputParameters, "AuPosRSt");
		dynAppend(deviceInputParameters, "MPosRSt");
		dynAppend(deviceInputParameters, "OutOVSt");
		
		//Output parameters
		dynAppend(deviceOutputParameters, "ManReg01");
		dynAppend(deviceOutputParameters, "MPosR");
		dynAppend(deviceOutputParameters, "PLiOn");
		dynAppend(deviceOutputParameters, "PLiOff");
		
		//Input parameters
		iLen=dynlen(deviceInputParameters);
		for(i=1; i<=iLen; i++)
		{
			temp="";
			dpGet(sDp + ".ProcessInput." + deviceInputParameters[i] + ":_address.._drv_ident" , temp);
			sFunction = "DRV_"+temp+"_convertToUnicosAddress";
			if(isFunctionDefined(sFunction)) {
				evalScript(temp, "string main(string sDeviceDpeName, string sDeviceType) {"+
													"	string temp;"+
													"	temp = "+sFunction+"(sDeviceDpeName, sDeviceType);"+
													"	return temp;"+
													"}", makeDynString(), sDp + ".ProcessInput." + deviceInputParameters[i], sObject);
			}
			else
				temp = UN_CONFIG_EXPORT_UNDEF;
			dynAppend(smoothDpParameters, temp);
		}
		
		//Output parameters
		iLen=dynlen(deviceOutputParameters);
		for(j=1; j<=iLen; j++)
		{
			temp="";
			dpGet(sDp + ".ProcessOutput." + deviceOutputParameters[j] + ":_address.._drv_ident" , temp);
			sFunction = "DRV_"+temp+"_convertToUnicosAddress";
			if(isFunctionDefined(sFunction)) {
				evalScript(temp, "string main(string sDeviceDpeName, string sDeviceType) {"+
													"	string temp;"+
													"	temp = "+sFunction+"(sDeviceDpeName, sDeviceType);"+
													"	return temp;"+
													"}", makeDynString(), sDp + ".ProcessOutput." + deviceOutputParameters[j], sObject);
			}
			else
				temp = UN_CONFIG_EXPORT_UNDEF;
			dynAppend(smoothDpParameters, temp);
		}
	}
	
	if(sObject=="ProcessControlObject")
	{
		//Input parameters
		dynAppend(deviceInputParameters, "StsReg01");
		dynAppend(deviceInputParameters, "StsReg02");
		dynAppend(deviceInputParameters, "evStsReg01");
		dynAppend(deviceInputParameters, "evStsReg02");
		dynAppend(deviceInputParameters, "AuOpMo");
		dynAppend(deviceInputParameters, "OpMoSt");	
		
		//Output parameters
		dynAppend(deviceOutputParameters, "ManReg01");
		dynAppend(deviceOutputParameters, "MOpMoR");		

		//Input parameters
		iLen=dynlen(deviceInputParameters);
		for(i=1; i<=iLen; i++)
		{
			temp="";
			dpGet(sDp + ".ProcessInput." + deviceInputParameters[i] + ":_address.._drv_ident" , temp);
			sFunction = "DRV_"+temp+"_convertToUnicosAddress";
			if(isFunctionDefined(sFunction)) {
				evalScript(temp, "string main(string sDeviceDpeName, string sDeviceType) {"+
													"	string temp;"+
													"	temp = "+sFunction+"(sDeviceDpeName, sDeviceType);"+
													"	return temp;"+
													"}", makeDynString(), sDp + ".ProcessInput." + deviceInputParameters[i], sObject);
			}
			else
				temp = UN_CONFIG_EXPORT_UNDEF;
			dynAppend(smoothDpParameters, temp);
		}
		
		//Output parameters
		iLen=dynlen(deviceOutputParameters);
		for(j=1; j<=iLen; j++)
		{
			temp="";
			dpGet(sDp + ".ProcessOutput." + deviceOutputParameters[j] + ":_address.._drv_ident" , temp);
			sFunction = "DRV_"+temp+"_convertToUnicosAddress";
			if(isFunctionDefined(sFunction)) {
				evalScript(temp, "string main(string sDeviceDpeName, string sDeviceType) {"+
													"	string temp;"+
													"	temp = "+sFunction+"(sDeviceDpeName, sDeviceType);"+
													"	return temp;"+
													"}", makeDynString(), sDp + ".ProcessOutput." + deviceOutputParameters[j], sObject);
			}
			else
				temp = UN_CONFIG_EXPORT_UNDEF;
			dynAppend(smoothDpParameters, temp);
		}
	}
	
	if(sObject=="Controller"){
		//Input parameters
		dynAppend(deviceInputParameters, "StsReg01");
		dynAppend(deviceInputParameters, "StsReg02");
		dynAppend(deviceInputParameters, "evStsReg01");
		dynAppend(deviceInputParameters, "evStsReg02");
		dynAppend(deviceInputParameters, "AtSPo");
		dynAppend(deviceInputParameters, "AtSPoL");	
		dynAppend(deviceInputParameters, "AtSPoH");
		dynAppend(deviceInputParameters, "OutOVSt");		
		dynAppend(deviceInputParameters, "AtOutOL");
		dynAppend(deviceInputParameters, "AtOutOH");
		dynAppend(deviceInputParameters, "MV");
		dynAppend(deviceInputParameters, "AtMSPo");
		dynAppend(deviceInputParameters, "AtAuSPo");
		dynAppend(deviceInputParameters, "MPosRSt");
		dynAppend(deviceInputParameters, "AuPosRSt");
		dynAppend(deviceInputParameters, "AtPKpSt");
		dynAppend(deviceInputParameters, "AtPTiSt");
		dynAppend(deviceInputParameters, "AtPTdSt");
		dynAppend(deviceInputParameters, "AtPKdSt");
		
		//Output parameters
		dynAppend(deviceOutputParameters, "ManReg01");
		dynAppend(deviceOutputParameters, "MOutPosR");	
		dynAppend(deviceOutputParameters, "MSPo");
		dynAppend(deviceOutputParameters, "MSPoL");
		dynAppend(deviceOutputParameters, "MSPoH");
		dynAppend(deviceOutputParameters, "MOutOL");
		dynAppend(deviceOutputParameters, "MOutOH");
		dynAppend(deviceOutputParameters, "MKp");
		dynAppend(deviceOutputParameters, "MTi");
		dynAppend(deviceOutputParameters, "MTd");				
		dynAppend(deviceOutputParameters, "MKd");

		//Input parameters
		iLen=dynlen(deviceInputParameters);
		for(i=1; i<= dynlen(deviceInputParameters) ; i++){
			temp="";
			dpGet(sDp + ".ProcessInput." + deviceInputParameters[i] + ":_address.._drv_ident" , temp);
			sFunction = "DRV_"+temp+"_convertToUnicosAddress";
			if(isFunctionDefined(sFunction)) {
				evalScript(temp, "string main(string sDeviceDpeName, string sDeviceType) {"+
													"	string temp;"+
													"	temp = "+sFunction+"(sDeviceDpeName, sDeviceType);"+
													"	return temp;"+
													"}", makeDynString(), sDp + ".ProcessInput." + deviceInputParameters[i], sObject);
			}
			else
				temp = UN_CONFIG_EXPORT_UNDEF;
			dynAppend(smoothDpParameters, temp);
		}
		
		//Output parameters
		iLen=dynlen(deviceOutputParameters);
		for(j=1; j<= dynlen(deviceOutputParameters) ; j++)
		{
			temp="";
			dpGet(sDp + ".ProcessOutput." + deviceOutputParameters[j] + ":_address.._drv_ident" , temp);
			sFunction = "DRV_"+temp+"_convertToUnicosAddress";
			if(isFunctionDefined(sFunction)) {
				evalScript(temp, "string main(string sDeviceDpeName, string sDeviceType) {"+
													"	string temp;"+
													"	temp = "+sFunction+"(sDeviceDpeName, sDeviceType);"+
													"	return temp;"+
													"}", makeDynString(), sDp + ".ProcessOutput." + deviceOutputParameters[j], sObject);
			}
			else
				temp = UN_CONFIG_EXPORT_UNDEF;
			dynAppend(smoothDpParameters, temp);			
		}
	}
}

//------------------------------------------------------------------------------------------------------------------------

//@}
