/**@name LIBRARY: unConfigAnalogInput.ctl

@author: Vincent Forest (LHC-IAS)

Creation Date: 31 10 2002

Modification History: 
  02/05/2011: Herve
  - IS-510: CPC 5 and CPC 6 compatibility of the export/import of the CPC device configuration with all the front-end types
  
	18/04/2006: Herve
		- implementation of mask event

	06/05/2005: Enrique
		- new function: S7_PLC_AnalogInput_checkConfig to check the S7_PLC configuration
		- new function: S7_PLC_AnalogInput_setConfig to set the S7_PLC configuration 
			
	22/04/2005: Herve
		- allow archive in import file per device

	15/04/2005: Herve
		- Modbus set and check front-end functions: _UnPlc_AnalogInput_checkConfig, _UnPlc_AnalogInput_setConfig

version 1.0

External Function : 
	. unConfigAnalogInput_setConfig :  set configuration of given Analog Input datapoint
	. unConfigAnalogInput_checkConfig : check data before configuration
	
Internal Functions :
	
Purpose: 
This library contains the function used to set Unicos Analog Input configuration.

Usage: Public

PVSS manager usage: Ctrl, NV, NG

Constraints:
	. JCOP framework
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

// Constants
const unsigned UN_CONFIG_ANALOGINPUT_LENGTH = 19;

const unsigned UN_CONFIG_ANALOGINPUT_UNIT = 1;
const unsigned UN_CONFIG_ANALOGINPUT_FORMAT = 2;
const unsigned UN_CONFIG_ANALOGINPUT_RANGEMAX = 3;
const unsigned UN_CONFIG_ANALOGINPUT_RANGEMIN = 4;
const unsigned UN_CONFIG_ANALOGINPUT_HHLIMIT = 5;
const unsigned UN_CONFIG_ANALOGINPUT_HLIMIT = 6;
const unsigned UN_CONFIG_ANALOGINPUT_LLIMIT = 7;
const unsigned UN_CONFIG_ANALOGINPUT_LLLIMIT = 8;
const unsigned UN_CONFIG_ANALOGINPUT_DEADBANDE = 9;
const unsigned UN_CONFIG_ANALOGINPUT_DEADBANDE_TYPE = 10;
const unsigned UN_CONFIG_ANALOGINPUT_ARCHIVE_ACTIVE = 11;
const unsigned UN_CONFIG_ANALOGINPUT_ARCHIVE_TIME_FILTER = 12;
const unsigned UN_CONFIG_ANALOGINPUT_NORMAL_POSITION = 13;
const unsigned UN_CONFIG_ANALOGINPUT_ADDRESS_STSREG01 = 14;
const unsigned UN_CONFIG_ANALOGINPUT_ADDRESS_EVSTSREG01 = 15;
const unsigned UN_CONFIG_ANALOGINPUT_ADDRESS_POSST = 16;
const unsigned UN_CONFIG_ANALOGINPUT_ADDRESS_HFST = 17;
const unsigned UN_CONFIG_ANALOGINPUT_ADDRESS_MANREG01 = 18;
const unsigned UN_CONFIG_ANALOGINPUT_ADDRESS_MPOSR = 19;

const unsigned UN_CONFIG_ANALOGINPUT_ADDITIONAL_LENGTH = 1;
const unsigned UN_CONFIG_ANALOGINPUT_MASKEVENT = 1;

// constant declaration for AnalogInput device used with CMW front-end
// const unsigned UN_CONFIG_CMW_ANALOGINPUT_LENGTH = 18;
const unsigned UN_CONFIG_CMW_ANALOGINPUT_LENGTH = 19;
const unsigned UN_CONFIG_CMW_ANALOGINPUT_UNIT = 1;
const unsigned UN_CONFIG_CMW_ANALOGINPUT_FORMAT = 2;
const unsigned UN_CONFIG_CMW_ANALOGINPUT_RANGEMAX = 3;
const unsigned UN_CONFIG_CMW_ANALOGINPUT_RANGEMIN = 4;
const unsigned UN_CONFIG_CMW_ANALOGINPUT_HHLIMIT = 5;
const unsigned UN_CONFIG_CMW_ANALOGINPUT_HLIMIT = 6;
const unsigned UN_CONFIG_CMW_ANALOGINPUT_LLIMIT = 7;
const unsigned UN_CONFIG_CMW_ANALOGINPUT_LLLIMIT = 8;
const unsigned UN_CONFIG_CMW_ANALOGINPUT_DEADBANDE = 9;
const unsigned UN_CONFIG_CMW_ANALOGINPUT_DEADBANDE_TYPE = 10;
const unsigned UN_CONFIG_CMW_ANALOGINPUT_ARCHIVE_ACTIVE = 11;
const unsigned UN_CONFIG_CMW_ANALOGINPUT_ARCHIVE_TIME_FILTER = 12;
const unsigned UN_CONFIG_CMW_ANALOGINPUT_NORMAL_POSITION = 13;
// const unsigned UN_CONFIG_CMW_ANALOGINPUT_TYPE = 14;
// const unsigned UN_CONFIG_CMW_ANALOGINPUT_GROUP = 15;
// const unsigned UN_CONFIG_CMW_ANALOGINPUT_ADDRESS_CMWDEVICE = 16;
// const unsigned UN_CONFIG_CMW_ANALOGINPUT_ADDRESS_CMW_FILTERNAME = 17;
// const unsigned UN_CONFIG_CMW_ANALOGINPUT_ADDRESS_CMW_FILTERVALUE = 18;

// Example: address CMW without filtering
// INPUT_SPONTANEUS: 2||device$property@tag||
// INPUT_POLLING: 4|pollingGroup,2000|device$property@tag||

const unsigned UN_CONFIG_CMW_ANALOGINPUT_ADDRESS_STSREG01 = 14;
const unsigned UN_CONFIG_CMW_ANALOGINPUT_ADDRESS_EVSTSREG01 = 15;
const unsigned UN_CONFIG_CMW_ANALOGINPUT_ADDRESS_POSST = 16;
const unsigned UN_CONFIG_CMW_ANALOGINPUT_ADDRESS_HFST = 17;
const unsigned UN_CONFIG_CMW_ANALOGINPUT_ADDRESS_MANREG = 18;
const unsigned UN_CONFIG_CMW_ANALOGINPUT_ADDRESS_MPOSR = 19;
const string CMW_ADDRESS_CONFIG_SEPARATOR = "|";
const unsigned CMW_ANALOGINPUT_STSREGE01_VALUE_NOINTERACT=  8196;

// constant declaration for AnalogInput device used with DIP front-end
const unsigned UN_CONFIG_DIP_ANALOGINPUT_LENGTH = 16;
const unsigned UN_CONFIG_DIP_ANALOGINPUT_UNIT = 1;
const unsigned UN_CONFIG_DIP_ANALOGINPUT_FORMAT = 2;
const unsigned UN_CONFIG_DIP_ANALOGINPUT_RANGEMAX = 3;
const unsigned UN_CONFIG_DIP_ANALOGINPUT_RANGEMIN = 4;
const unsigned UN_CONFIG_DIP_ANALOGINPUT_HHLIMIT = 5;
const unsigned UN_CONFIG_DIP_ANALOGINPUT_HLIMIT = 6;
const unsigned UN_CONFIG_DIP_ANALOGINPUT_LLIMIT = 7;
const unsigned UN_CONFIG_DIP_ANALOGINPUT_LLLIMIT = 8;
const unsigned UN_CONFIG_DIP_ANALOGINPUT_DEADBANDE = 9;
const unsigned UN_CONFIG_DIP_ANALOGINPUT_DEADBANDE_TYPE = 10;
const unsigned UN_CONFIG_DIP_ANALOGINPUT_ARCHIVE_ACTIVE = 11;
const unsigned UN_CONFIG_DIP_ANALOGINPUT_ARCHIVE_TIME_FILTER = 12;
const unsigned UN_CONFIG_DIP_ANALOGINPUT_NORMAL_POSITION = 13;
const unsigned UN_CONFIG_DIP_ANALOGINPUT_ADDRESS_DIPITEM_POSST = 14;
const unsigned UN_CONFIG_DIP_ANALOGINPUT_ADDRESS_DIPTAG_POSST = 15;
const unsigned UN_CONFIG_DIP_ANALOGINPUT_ADDRESS_DIPBUFFER_POSST = 16;

// constant declaration for AnalogInput device used with OPC front-end
const unsigned UN_CONFIG_OPC_ANALOGINPUT_LENGTH = 19;
const unsigned UN_CONFIG_OPC_ANALOGINPUT_UNIT = 1;
const unsigned UN_CONFIG_OPC_ANALOGINPUT_FORMAT = 2;
const unsigned UN_CONFIG_OPC_ANALOGINPUT_RANGEMAX = 3;
const unsigned UN_CONFIG_OPC_ANALOGINPUT_RANGEMIN = 4;
const unsigned UN_CONFIG_OPC_ANALOGINPUT_HHLIMIT = 5;
const unsigned UN_CONFIG_OPC_ANALOGINPUT_HLIMIT = 6;
const unsigned UN_CONFIG_OPC_ANALOGINPUT_LLIMIT = 7;
const unsigned UN_CONFIG_OPC_ANALOGINPUT_LLLIMIT = 8;
const unsigned UN_CONFIG_OPC_ANALOGINPUT_DEADBANDE = 9;
const unsigned UN_CONFIG_OPC_ANALOGINPUT_DEADBANDE_TYPE = 10;
const unsigned UN_CONFIG_OPC_ANALOGINPUT_ARCHIVE_ACTIVE = 11;
const unsigned UN_CONFIG_OPC_ANALOGINPUT_ARCHIVE_TIME_FILTER = 12;
const unsigned UN_CONFIG_OPC_ANALOGINPUT_NORMAL_POSITION = 13;
const unsigned UN_CONFIG_OPC_ANALOGINPUT_ADDRESS_OPCITEM_STSREG01 = 14;
const unsigned UN_CONFIG_OPC_ANALOGINPUT_ADDRESS_OPCITEM_EVSTSREG01 = 15;
const unsigned UN_CONFIG_OPC_ANALOGINPUT_ADDRESS_OPCITEM_POSST = 16;
const unsigned UN_CONFIG_OPC_ANALOGINPUT_ADDRESS_OPCITEM_HFST = 17;
const unsigned UN_CONFIG_OPC_ANALOGINPUT_ADDRESS_OPCITEM_MANREG01 = 18;
const unsigned UN_CONFIG_OPC_ANALOGINPUT_ADDRESS_OPCITEM_MPOSR = 19;

// constant declaration for AnalogInput device used with SOFT_FE front-end
const unsigned UN_CONFIG_SOFT_FE_ANALOGINPUT_LENGTH = 11;
const unsigned UN_CONFIG_SOFT_FE_ANALOGINPUT_UNIT = 1;
const unsigned UN_CONFIG_SOFT_FE_ANALOGINPUT_FORMAT = 2;
const unsigned UN_CONFIG_SOFT_FE_ANALOGINPUT_RANGEMAX = 3;
const unsigned UN_CONFIG_SOFT_FE_ANALOGINPUT_RANGEMIN = 4;
const unsigned UN_CONFIG_SOFT_FE_ANALOGINPUT_HHLIMIT = 5;
const unsigned UN_CONFIG_SOFT_FE_ANALOGINPUT_HLIMIT = 6;
const unsigned UN_CONFIG_SOFT_FE_ANALOGINPUT_LLIMIT = 7;
const unsigned UN_CONFIG_SOFT_FE_ANALOGINPUT_LLLIMIT = 8;
const unsigned UN_CONFIG_SOFT_FE_ANALOGINPUT_ARCHIVE_ACTIVE = 9;
const unsigned UN_CONFIG_SOFT_FE_ANALOGINPUT_ARCHIVE_TIME_FILTER = 10;
const unsigned UN_CONFIG_SOFT_FE_ANALOGINPUT_NORMAL_POSITION = 11;

//@{

//------------------------------------------------------------------------------------------------------------------------

// unConfigAnalogInput_setConfig
/**
Purpose: set configuration of given Analog Input datapoint

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigAnalogInput_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	
	// DebugN("FUNCTION: unConfigAnalogInput_setConfig(dsConfigs= "+dsConfigs+" )");
	
// 1. Initialization : checking + dpCreate
	unConfigGenericFunctions_checkAll(dsConfigs, "AnalogInput", exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)		// Good inputs
	{
		sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
		unConfigGenericFunctions_createDp(sDpName, "AnalogInput", exceptionInfo);
		if (dynlen(exceptionInfo) <= 0)	// Creation ok or dp already exists
		{
// 2. Generate device
// 2.1. Parameters
			unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
	// 2.2 call the device check function
			unConfigGenericFunctions_setDeviceConfig(dsConfigs, "AnalogInput", exceptionInfo);
		}		
	}
}

//------------------------------------------------------------------------------------------------------------------------

// unConfigAnalogInput_checkConfig
/**
Purpose: check data before configuration

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigAnalogInput_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	if (dynlen(configLine) >= UN_CONFIG_COMMON_LENGTH)
	{
	// 1. Common
		unConfigGenericFunctions_checkParameters(configLine, "AnalogInput", exceptionInfo);
	// 2. call the device check function
		unConfigGenericFunctions_checkDeviceConfig(configLine, "AnalogInput", exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","unConfigAnalogInput_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// _UnPlc_AnalogInput_setConfig
/**
Purpose: set configuration of given Analog Input datapoint for _UnPlc front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

_UnPlc_AnalogInput_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	string sMask = "0";
	int iMask;
	
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];

// if archive name are in the config->overwrite panel setting.	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if(dynlen(dsConfigs) == (UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOGINPUT_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOGINPUT_MASKEVENT];
		else if(dynlen(dsConfigs) == (UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANALOGINPUT_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANALOGINPUT_MASKEVENT];
		sscanf(sMask, "%d", iMask);

		if(dynlen(dsConfigs) == (UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANALOGINPUT_ADDITIONAL_LENGTH)) {
			if(dsConfigs[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}
	else {
		iMask = 1;
		if(dynlen(dsConfigs) == (UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			if(dsConfigs[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}

// 2.2. Registers
	unConfigGenericFunctions_setAllRegisters(dsConfigs,
					 makeDynString("StsReg01"),
					 makeDynInt(UN_CONFIG_ANALOGINPUT_ADDRESS_STSREG01),
					 makeDynInt(UN_CONFIG_ANALOGINPUT_ADDRESS_EVSTSREG01),
					 makeDynString("ManReg01"),
					 makeDynInt(UN_CONFIG_ANALOGINPUT_ADDRESS_MANREG01),
					 makeDynString(),
					 makeDynString(),
					 -1,
					 exceptionInfo);
// 2.3. Float values
	unConfigGenericFunctions_setFloatIntBlock(dsConfigs,
					PVSS_MODBUS_FLOAT,
					makeDynString("PosSt", "HFStOrAuPosRSt"),
					makeDynInt(UN_CONFIG_ANALOGINPUT_ADDRESS_POSST, UN_CONFIG_ANALOGINPUT_ADDRESS_HFST),
					makeDynString("MPosR"),
					makeDynInt(UN_CONFIG_ANALOGINPUT_ADDRESS_MPOSR),
					makeDynInt(UN_CONFIG_ANALOGINPUT_UNIT, UN_CONFIG_ANALOGINPUT_FORMAT, UN_CONFIG_ANALOGINPUT_DEADBANDE, UN_CONFIG_ANALOGINPUT_DEADBANDE_TYPE),		
					makeDynInt(UN_CONFIG_ANALOGINPUT_RANGEMIN, UN_CONFIG_ANALOGINPUT_RANGEMAX),
					makeDynString("PosSt"),
					makeDynInt(UN_CONFIG_ANALOGINPUT_ARCHIVE_ACTIVE, UN_CONFIG_ANALOGINPUT_ARCHIVE_TIME_FILTER),
					exceptionInfo);
	unConfigGenericFunctions_set5RangesAlert(dsConfigs, sDpName + ".ProcessInput.PosSt", "Position Status", UN_CONFIG_ANALOGINPUT_NORMAL_POSITION,
													 makeDynInt(UN_CONFIG_ANALOGINPUT_LLLIMIT, UN_CONFIG_ANALOGINPUT_LLIMIT, 
													 			UN_CONFIG_ANALOGINPUT_HLIMIT, UN_CONFIG_ANALOGINPUT_HHLIMIT),
													 exceptionInfo);

// 3. additional config
	unConfigGenericFunctions_setMaskEvent(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], iMask, 
																					makeDynString(".ProcessInput.evStsReg01"), exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------

// _UnPlc_AnalogInput_checkConfig
/**
Purpose: check data before configuration for _UnPlc front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

_UnPlc_AnalogInput_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	bool bOk=false;
	bool bCheckArchive;
	unsigned uUnsigned;
	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if (dynlen(configLine) == (UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANALOGINPUT_ADDITIONAL_LENGTH)) {
			bOk = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANALOGINPUT_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else if (dynlen(configLine) == (UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANALOGINPUT_ADDITIONAL_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANALOGINPUT_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else
			bOk = false;
	}
	else {
		if (dynlen(configLine) == (UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH))
			bOk = true;
		else if (dynlen(configLine) == (UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
		}
		else
			bOk = false;
	}
	
	if(bOk) {
		if(bCheckArchive) {
			unConfigGenericFunctions_checkArchive(configLine[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																						configLine[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																						configLine[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																						exceptionInfo);
		}
	// 2. Registers
		unConfigGenericFunctions_checkAllRegisters(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_ANALOGINPUT_ADDRESS_STSREG01),
				makeDynInt(UN_CONFIG_ANALOGINPUT_ADDRESS_EVSTSREG01),
				makeDynInt(UN_CONFIG_ANALOGINPUT_ADDRESS_MANREG01),
				-1,
				exceptionInfo);
	// 3. Float block
		unConfigGenericFunctions_checkFloatIntBlock(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_ANALOGINPUT_ADDRESS_POSST, UN_CONFIG_ANALOGINPUT_ADDRESS_HFST),
				makeDynInt(UN_CONFIG_ANALOGINPUT_ADDRESS_MPOSR), 
				makeDynInt(UN_CONFIG_ANALOGINPUT_UNIT, UN_CONFIG_ANALOGINPUT_FORMAT, UN_CONFIG_ANALOGINPUT_DEADBANDE, UN_CONFIG_ANALOGINPUT_DEADBANDE_TYPE),		
				makeDynInt(UN_CONFIG_ANALOGINPUT_RANGEMIN, UN_CONFIG_ANALOGINPUT_RANGEMAX),
				makeDynInt(UN_CONFIG_ANALOGINPUT_ARCHIVE_ACTIVE, UN_CONFIG_ANALOGINPUT_ARCHIVE_TIME_FILTER),
				exceptionInfo);
	// 4. Particular : limits
		unConfigGenericFunctions_check5RangesAlert(configLine, UN_CONFIG_ANALOGINPUT_NORMAL_POSITION,
												   makeDynInt(UN_CONFIG_ANALOGINPUT_LLLIMIT, UN_CONFIG_ANALOGINPUT_LLIMIT, 
												 			  UN_CONFIG_ANALOGINPUT_HLIMIT, UN_CONFIG_ANALOGINPUT_HHLIMIT),
												   exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","_UnPlc_AnalogInput_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// S7_PLC_AnalogInput_checkConfig
/**
Purpose: check data before configuration for S7_PLC front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: Win XP.
	. distributed system: yes.
*/

S7_PLC_AnalogInput_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	// DebugN("FUNCTION: S7_PLC_ANALOGINPUT_checkConfig(configLine="+configLine);
	bool bOk=false;
	bool bCheckArchive;
	unsigned uUnsigned;
	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if (dynlen(configLine) == (UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANALOGINPUT_ADDITIONAL_LENGTH)) {
			bOk = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ANALOGINPUT_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else if (dynlen(configLine) == (UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANALOGINPUT_ADDITIONAL_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANALOGINPUT_MASKEVENT], uUnsigned, exceptionInfo);
		}
		else
			bOk = false;
	}
	else {
		if (dynlen(configLine) == (UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH))
			bOk = true;
		else if (dynlen(configLine) == (UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			bOk = true;
			bCheckArchive = true;
		}
		else
			bOk = false;
	}
	
	if(bOk) {
		if(bCheckArchive) {
			unConfigGenericFunctions_checkArchive(configLine[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																						configLine[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																						configLine[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																						exceptionInfo);
		}
	// 2. Registers
		unConfigGenericFunctions_checkAllRegistersS7(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_ANALOGINPUT_ADDRESS_STSREG01),
				makeDynInt(UN_CONFIG_ANALOGINPUT_ADDRESS_EVSTSREG01),
				makeDynInt(UN_CONFIG_ANALOGINPUT_ADDRESS_MANREG01),
				-1,
				exceptionInfo);
	// 3. Float block	
		unConfigGenericFunctions_checkFloatIntBlockS7(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_ANALOGINPUT_ADDRESS_POSST, UN_CONFIG_ANALOGINPUT_ADDRESS_HFST),
				makeDynInt(UN_CONFIG_ANALOGINPUT_ADDRESS_MPOSR), 
				makeDynInt(UN_CONFIG_ANALOGINPUT_UNIT, UN_CONFIG_ANALOGINPUT_FORMAT, UN_CONFIG_ANALOGINPUT_DEADBANDE, UN_CONFIG_ANALOGINPUT_DEADBANDE_TYPE),		
				makeDynInt(UN_CONFIG_ANALOGINPUT_RANGEMIN, UN_CONFIG_ANALOGINPUT_RANGEMAX),
				makeDynInt(UN_CONFIG_ANALOGINPUT_ARCHIVE_ACTIVE, UN_CONFIG_ANALOGINPUT_ARCHIVE_TIME_FILTER),
				exceptionInfo);
				
	// 4. Particular : limits
		
		unConfigGenericFunctions_check5RangesAlert(configLine, UN_CONFIG_ANALOGINPUT_NORMAL_POSITION,
												   makeDynInt(UN_CONFIG_ANALOGINPUT_LLLIMIT, UN_CONFIG_ANALOGINPUT_LLIMIT, 
												 			  UN_CONFIG_ANALOGINPUT_HLIMIT, UN_CONFIG_ANALOGINPUT_HHLIMIT),
												   exceptionInfo);
		
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","S7_PLC_AnalogInput_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}


//------------------------------------------------------------------------------------------------------------------------

// S7_PLC_AnalogInput_setConfig
/**
Purpose: set configuration of given Analog Input datapoint for S7_PLC front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: Win XP
	. distributed system: yes.
*/

S7_PLC_AnalogInput_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	string sMask = "0";
	int iMask;
	
	// DebugN("FUNCTION: S7_PLC_AnalogInput_setConfig(dsConfigs= "+dsConfigs+" )");
	
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];

// if archive name are in the config->overwrite panel setting.	
	if(unConfigGenericFunctions_additionalDeviceInfo()) {
		if(dynlen(dsConfigs) == (UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOGINPUT_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ANALOGINPUT_MASKEVENT];
		else if(dynlen(dsConfigs) == (UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANALOGINPUT_ADDITIONAL_LENGTH))
			sMask = dsConfigs[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANALOGINPUT_MASKEVENT];
		sscanf(sMask, "%d", iMask);

		if(dynlen(dsConfigs) == (UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_ANALOGINPUT_ADDITIONAL_LENGTH)) {
			if(dsConfigs[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}
	else {
		iMask = 1;
		if(dynlen(dsConfigs) == (UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
			if(dsConfigs[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		}
	}
					
// 2.2. Registers
	unConfigGenericFunctions_setAllRegistersS7 (dsConfigs,
					 makeDynString("StsReg01"),
					 makeDynInt(UN_CONFIG_ANALOGINPUT_ADDRESS_STSREG01),
					 makeDynInt(UN_CONFIG_ANALOGINPUT_ADDRESS_EVSTSREG01),
					 makeDynString("ManReg01"),
					 makeDynInt(UN_CONFIG_ANALOGINPUT_ADDRESS_MANREG01),
					 makeDynString(),
					 makeDynString(),
					 -1,
					 exceptionInfo);
// 2.3. Float values
	unConfigGenericFunctions_setFloatIntBlockS7(dsConfigs,
					PVSS_S7_FLOAT,
					makeDynString("PosSt", "HFStOrAuPosRSt"),
					makeDynInt(UN_CONFIG_ANALOGINPUT_ADDRESS_POSST, UN_CONFIG_ANALOGINPUT_ADDRESS_HFST),
					makeDynString("MPosR"),
					makeDynInt(UN_CONFIG_ANALOGINPUT_ADDRESS_MPOSR),
					makeDynInt(UN_CONFIG_ANALOGINPUT_UNIT, UN_CONFIG_ANALOGINPUT_FORMAT, UN_CONFIG_ANALOGINPUT_DEADBANDE, UN_CONFIG_ANALOGINPUT_DEADBANDE_TYPE),		
					makeDynInt(UN_CONFIG_ANALOGINPUT_RANGEMIN, UN_CONFIG_ANALOGINPUT_RANGEMAX),
					makeDynString("PosSt"),
					makeDynInt(UN_CONFIG_ANALOGINPUT_ARCHIVE_ACTIVE, UN_CONFIG_ANALOGINPUT_ARCHIVE_TIME_FILTER),
					exceptionInfo);					
					
					
	unConfigGenericFunctions_set5RangesAlert (dsConfigs, sDpName + ".ProcessInput.PosSt", "Position Status", UN_CONFIG_ANALOGINPUT_NORMAL_POSITION,
													 makeDynInt(UN_CONFIG_ANALOGINPUT_LLLIMIT, UN_CONFIG_ANALOGINPUT_LLIMIT, 
													 			UN_CONFIG_ANALOGINPUT_HLIMIT, UN_CONFIG_ANALOGINPUT_HHLIMIT),
													 exceptionInfo);

// 3. additional config
	unConfigGenericFunctions_setMaskEvent(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], iMask, 
																					makeDynString(".ProcessInput.evStsReg01"), exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------

// CMW_AnalogInput_setConfig
/**
Purpose: set configuration of given Analog Input datapoint for CMW front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CMW_AnalogInput_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	bit32 b32;
	bool bDeleteConfigAdd=false;
		
	dyn_string dsAddressCMW,dsFullAddressCMW;
	string sPollGroup,sFilterName,sFilterValue,sType;
	
	setBit(b32, UN_STSREG01_AUMOST, true);
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
	
	// DebugN("FUNCTION: CMW_AnalogInput_setConfig(dsConfigs=",dsConfigs);
	
// if archive name are in the config->overwrite panel setting.	
	if(dynlen(dsConfigs) == (UN_CONFIG_CMW_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		if(dsConfigs[UN_CONFIG_CMW_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_CMW_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];

		if(dsConfigs[UN_CONFIG_CMW_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_CMW_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];

		if(dsConfigs[UN_CONFIG_CMW_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_CMW_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
	}

//DebugN(g_dsImport_UnCMW_config);
// set stsreg01 to automode
	dpSet(sDpName + ".ProcessInput.StsReg01", b32);
	dpSet(sDpName + ".eventMask", 1);
	dpSet(sDpName + ".ProcessInput.HFStOrAuPosRSt", 0);

// 2.2. Registers
	unConfigGenericFunctions_setCommonAllRegisters(dsConfigs,
					 makeDynString("StsReg01"),
					 makeDynString(),
					 makeDynString(),
					 -1,
					 exceptionInfo);
					 
// 2.3. Float values
	unConfigGenericFunctions_setCommonFloatIntBlock(dsConfigs,
					PVSS_MODBUS_FLOAT,
					makeDynString("PosSt", "HFStOrAuPosRSt"),
					makeDynInt(UN_CONFIG_CMW_ANALOGINPUT_UNIT, UN_CONFIG_CMW_ANALOGINPUT_FORMAT, 
											UN_CONFIG_CMW_ANALOGINPUT_DEADBANDE, UN_CONFIG_CMW_ANALOGINPUT_DEADBANDE_TYPE),		
					makeDynInt(UN_CONFIG_CMW_ANALOGINPUT_RANGEMIN, UN_CONFIG_CMW_ANALOGINPUT_RANGEMAX),
					makeDynString("PosSt"),
					makeDynInt(UN_CONFIG_CMW_ANALOGINPUT_ARCHIVE_ACTIVE, UN_CONFIG_CMW_ANALOGINPUT_ARCHIVE_TIME_FILTER),
					exceptionInfo);
					
	unConfigGenericFunctions_set5RangesAlert(dsConfigs, sDpName + ".ProcessInput.PosSt", "Position Status", UN_CONFIG_CMW_ANALOGINPUT_NORMAL_POSITION,
													 makeDynInt(UN_CONFIG_CMW_ANALOGINPUT_LLLIMIT, UN_CONFIG_CMW_ANALOGINPUT_LLIMIT, 
													 			UN_CONFIG_CMW_ANALOGINPUT_HLIMIT, UN_CONFIG_CMW_ANALOGINPUT_HHLIMIT),
													 exceptionInfo);
// input
	// 
	// unConfigCMW_setAddress(string sDpe, 
	//												string sDriver, 
	//												string sType, 
	//												string sTransformationType, 
	//												string sGroup, 
	//												string sFilterName, 
	//												string sFilterValue, 
  //												string sCMWDevice, 
  //												string sCMWProperty, 
  //												string sCMWTag, 
  //												bool bLLC, 
  //												dyn_string &exceptionInfo)

	// ************************************************************************************************

	// PosSt
	bDeleteConfigAdd=false;
	dsFullAddressCMW=strsplit(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CMW_ANALOGINPUT_ADDRESS_POSST],CMW_ADDRESS_CONFIG_SEPARATOR);
	if (dynlen(dsFullAddressCMW)==4) dsFullAddressCMW[5]="";
	if (dynlen(dsFullAddressCMW)==5)
		{
			sType=dsFullAddressCMW[1];
			sPollGroup=dsFullAddressCMW[2];
			sFilterName=dsFullAddressCMW[4];
			sFilterValue=dsFullAddressCMW[5];
			dsAddressCMW=strsplit(dsFullAddressCMW[3],"$");
			if (dynlen(dsAddressCMW) == 3)
				{
					unConfigCMW_setAddress(sDpName + ".ProcessInput.PosSt", 
													dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER], 
													sType, 
													CMW_FLOAT_TYPE,
													sPollGroup,
													sFilterName,
													sFilterValue,
													dsAddressCMW[1],
													dsAddressCMW[2],
													dsAddressCMW[3],
													true,
													exceptionInfo);
		 		}
		 	else
		 		bDeleteConfigAdd=true;
		 }
	 else
	 		bDeleteConfigAdd=true;
	 		
	 if (bDeleteConfigAdd)
	 	fwPeriphAddress_delete(sDpName+ ".ProcessInput.PosSt", exceptionInfo);	 
								
	// StsReg01 
	bDeleteConfigAdd=false;
	dsFullAddressCMW=strsplit(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CMW_ANALOGINPUT_ADDRESS_STSREG01],CMW_ADDRESS_CONFIG_SEPARATOR);
	// DebugN("StsReg01  =("+dynlen(dsFullAddressCMW)+")= dsFullAddressCMW="+dsFullAddressCMW);
	if (dynlen(dsFullAddressCMW)==4) dsFullAddressCMW[5]="";
	if (dynlen(dsFullAddressCMW)==5)
		{
			sType=dsFullAddressCMW[1];
			sPollGroup=dsFullAddressCMW[2];
			sFilterName=dsFullAddressCMW[4];
			sFilterValue=dsFullAddressCMW[5];
			dsAddressCMW=strsplit(dsFullAddressCMW[3],"$");
			if (dynlen(dsAddressCMW) == 3)
				{												
					unConfigCMW_setAddress(sDpName + ".ProcessInput.StsReg01", 
															dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER], 
															sType, 
															CMW_LONG_TYPE,
															sPollGroup,
															sFilterName,
															sFilterValue,
															dsAddressCMW[1],
															dsAddressCMW[2],
															dsAddressCMW[3],
															true,
															exceptionInfo);
				}
		 	else
		 		bDeleteConfigAdd=true;
		 }
	 else
	 			bDeleteConfigAdd=true;
	 		
	 	
	 if (bDeleteConfigAdd)		
	 		{
	 			fwPeriphAddress_delete(sDpName+ ".ProcessInput.StsReg01", exceptionInfo);
	 			
	 			// Sets the AI as not being forced allways
	 			int iRes=dpSet(sDpName+ ".ProcessInput.StsReg01:_original.._value",CMW_ANALOGINPUT_STSREGE01_VALUE_NOINTERACT);
	 			if (iRes<0)
	 				DebugN("ERROR: setting the CMW_ANALOGINPUT_STSREGE01_VALUE_NOINTERACT in dpe:"+sDpName+ ".ProcessInput.StsReg01");
	 				
	 		}
													
	// evStsReg01
	bDeleteConfigAdd=false;	
	dsFullAddressCMW=strsplit(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CMW_ANALOGINPUT_ADDRESS_EVSTSREG01],CMW_ADDRESS_CONFIG_SEPARATOR);
	// DebugN("evStsReg01  =("+dynlen(dsFullAddressCMW)+")= dsFullAddressCMW="+dsFullAddressCMW);	
	if (dynlen(dsFullAddressCMW)==4) dsFullAddressCMW[5]="";
	if (dynlen(dsFullAddressCMW)==5)
		{
			sType=dsFullAddressCMW[1];
			sPollGroup=dsFullAddressCMW[2];
			sFilterName=dsFullAddressCMW[4];
			sFilterValue=dsFullAddressCMW[5];
			dsAddressCMW=strsplit(dsFullAddressCMW[3],"$");
			if (dynlen(dsAddressCMW) == 3)
				{												
					unConfigCMW_setAddress(sDpName + ".ProcessInput.evStsReg01", 
															dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER], 
															sType, 
															CMW_LONG_TYPE,
															sPollGroup,
															sFilterName,
															sFilterValue,
															dsAddressCMW[1],
															dsAddressCMW[2],
															dsAddressCMW[3],
															true,
															exceptionInfo);
				}
		 	else
		 		bDeleteConfigAdd=true;
		 }
	 else
	 		bDeleteConfigAdd=true;
	 
	 if (bDeleteConfigAdd)
	 		fwPeriphAddress_delete(sDpName+ ".ProcessInput.evStsReg01", exceptionInfo);
													

	// HFStOrAuPosRSt
	bDeleteConfigAdd=false;
	dsFullAddressCMW=strsplit(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CMW_ANALOGINPUT_ADDRESS_HFST],CMW_ADDRESS_CONFIG_SEPARATOR);
	// DebugN("HFStOrAuPosRSt  =("+dynlen(dsFullAddressCMW)+")= dsFullAddressCMW="+dsFullAddressCMW);		
	// If the last element is empty strplit discards it. We have to add it.
	if (dynlen(dsFullAddressCMW)==4) dsFullAddressCMW[5]="";
	if (dynlen(dsFullAddressCMW)==5)
		{
			sType=dsFullAddressCMW[1];
			sPollGroup=dsFullAddressCMW[2];
			sFilterName=dsFullAddressCMW[4];
			sFilterValue=dsFullAddressCMW[5];
			dsAddressCMW=strsplit(dsFullAddressCMW[3],"$");
			if (dynlen(dsAddressCMW) == 3)
				{
					unConfigCMW_setAddress(sDpName + ".ProcessInput.HFStOrAuPosRSt", 
															dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER], 
															sType, 
															CMW_FLOAT_TYPE,
															sPollGroup,
															sFilterName,
															sFilterValue,
															dsAddressCMW[1],
															dsAddressCMW[2],
															dsAddressCMW[3],
															true,
															exceptionInfo);
				}
		 	else
		 		bDeleteConfigAdd=true;
		 }
	 else
	 		bDeleteConfigAdd=true;
	 
	 if (bDeleteConfigAdd)
	 		fwPeriphAddress_delete(sDpName+ ".ProcessInput.HFStOrAuPosRSt", exceptionInfo);
	 		
// output

	// ManReg01
	bDeleteConfigAdd=false;
	dsFullAddressCMW=strsplit(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CMW_ANALOGINPUT_ADDRESS_MANREG],CMW_ADDRESS_CONFIG_SEPARATOR);
	//DebugN("ManReg01  =("+dynlen(dsFullAddressCMW)+")= dsFullAddressCMW="+dsFullAddressCMW);	
	if (dynlen(dsFullAddressCMW)==4) dsFullAddressCMW[5]="";		
	if (dynlen(dsFullAddressCMW)==5)
		{
			sType=dsFullAddressCMW[1];
			sPollGroup=dsFullAddressCMW[2];
			sFilterName=dsFullAddressCMW[4];
			sFilterValue=dsFullAddressCMW[5];
			dsAddressCMW=strsplit(dsFullAddressCMW[3],"$");
			if (dynlen(dsAddressCMW) == 3)
				{
					unConfigCMW_setAddress(sDpName + ".ProcessOutput.ManReg01", 
																	dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER], 
																	DPATTR_ADDR_MODE_OUTPUT_SINGLE,
																	CMW_LONG_TYPE+CMW_NO_MONITORON,
																	sPollGroup,
																	sFilterName,
																	sFilterValue,
																	dsAddressCMW[1],
																	dsAddressCMW[2],
																	dsAddressCMW[3],
																	false,
																	exceptionInfo);
				}
		 	else
		 		bDeleteConfigAdd=true;
		 }
	 else
	 	bDeleteConfigAdd=true;
	 	
	 if (bDeleteConfigAdd)
		 fwPeriphAddress_delete(sDpName+ ".ProcessOutput.ManReg01", exceptionInfo);
													
	// MPosR
	bDeleteConfigAdd=false;
	dsFullAddressCMW=strsplit(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CMW_ANALOGINPUT_ADDRESS_MPOSR],CMW_ADDRESS_CONFIG_SEPARATOR);
	// DebugN("MPosR  =("+dynlen(dsFullAddressCMW)+")= dsFullAddressCMW="+dsFullAddressCMW);				
	if (dynlen(dsFullAddressCMW)==4) dsFullAddressCMW[5]="";
	if (dynlen(dsFullAddressCMW)==5)
		{
			sType=dsFullAddressCMW[1];
			sPollGroup=dsFullAddressCMW[2];
			sFilterName=dsFullAddressCMW[4];
			sFilterValue=dsFullAddressCMW[5];
			dsAddressCMW=strsplit(dsFullAddressCMW[3],"$");
			if (dynlen(dsAddressCMW) == 3)
				{												
					unConfigCMW_setAddress(sDpName + ".ProcessOutput.MPosR", 
																	dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER], 
																	DPATTR_ADDR_MODE_OUTPUT_SINGLE,
																	CMW_FLOAT_TYPE+CMW_NO_MONITORON,
																	sPollGroup,
																	sFilterName,
																	sFilterValue,
																	dsAddressCMW[1],
																	dsAddressCMW[2],
																	dsAddressCMW[3],
																	false,
																	exceptionInfo);
				}
		 	else
		 		bDeleteConfigAdd=true;
		 }
	 else
	 	bDeleteConfigAdd=true;
	 	
	 if (bDeleteConfigAdd)
	 		fwPeriphAddress_delete(sDpName+ ".ProcessOutput.MPosR", exceptionInfo);													
																	
}

//-----------------------------------------------------------------------------------------------------------

// CMW_AnalogInput_checkConfig
/**
Purpose: check data before configuration for CMW front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
CMW_AnalogInput_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	bool bOk=false;
	dyn_string dsAddressCMW;
	string sPollGroup,sType;
	string sAddressConfig;
	
	// DebugN("FUNCTION: CMW_AnalogInput_checkConfig( configLine="+configLine);
	
	if (dynlen(configLine) == (UN_CONFIG_CMW_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH))
		bOk = true;
	else if (dynlen(configLine) == (UN_CONFIG_CMW_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		bOk = true;
		unConfigGenericFunctions_checkArchive(configLine[UN_CONFIG_CMW_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																					configLine[UN_CONFIG_CMW_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																					configLine[UN_CONFIG_CMW_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																					exceptionInfo);
	}
	else
		bOk = false;
	
	if(bOk) {
	// 3. Float block
		unConfigGenericFunctions_checkCommonFloatIntBlock(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_CMW_ANALOGINPUT_UNIT, UN_CONFIG_CMW_ANALOGINPUT_FORMAT, UN_CONFIG_CMW_ANALOGINPUT_DEADBANDE, 
											UN_CONFIG_CMW_ANALOGINPUT_DEADBANDE_TYPE),		
				makeDynInt(UN_CONFIG_CMW_ANALOGINPUT_RANGEMIN, UN_CONFIG_CMW_ANALOGINPUT_RANGEMAX),
				makeDynInt(UN_CONFIG_CMW_ANALOGINPUT_ARCHIVE_ACTIVE, UN_CONFIG_CMW_ANALOGINPUT_ARCHIVE_TIME_FILTER),
				exceptionInfo);

	// 4. Particular : limits
		unConfigGenericFunctions_check5RangesAlert(configLine, UN_CONFIG_CMW_ANALOGINPUT_NORMAL_POSITION,
												   makeDynInt(UN_CONFIG_CMW_ANALOGINPUT_LLLIMIT, UN_CONFIG_CMW_ANALOGINPUT_LLIMIT, 
												 			  UN_CONFIG_CMW_ANALOGINPUT_HLIMIT, UN_CONFIG_CMW_ANALOGINPUT_HHLIMIT),
												   exceptionInfo);
	// check group 

			//DebugN(configLine, configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CMW_ANALOGINPUT_TYPE], configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CMW_ANALOGINPUT_GROUP]);
			
			// STSREG01
			dsAddressCMW=strsplit(configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CMW_ANALOGINPUT_ADDRESS_STSREG01],CMW_ADDRESS_CONFIG_SEPARATOR);
			if (dynlen(dsAddressCMW)>=3)
				{
					sType=dsAddressCMW[1];
					sPollGroup=dsAddressCMW[2];
					sAddressConfig=dsAddressCMW[3];
					unConfigCMW_checkType(sType, sPollGroup, exceptionInfo);
					unConfigCMW_checkFormatCMWAddress(sAddressConfig,exceptionInfo);
				}
			// EVSTSREG01
			dsAddressCMW=strsplit(configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CMW_ANALOGINPUT_ADDRESS_EVSTSREG01],CMW_ADDRESS_CONFIG_SEPARATOR);
			if (dynlen(dsAddressCMW)>=3)
				{
				  sType=dsAddressCMW[1];
					sPollGroup=dsAddressCMW[2];
					sAddressConfig=dsAddressCMW[3];
					unConfigCMW_checkType(sType, sPollGroup, exceptionInfo);
					unConfigCMW_checkFormatCMWAddress(sAddressConfig,exceptionInfo);
				}
			// POSST
			dsAddressCMW=strsplit(configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CMW_ANALOGINPUT_ADDRESS_POSST],CMW_ADDRESS_CONFIG_SEPARATOR);
			if (dynlen(dsAddressCMW)>=3)
				{
					sType=dsAddressCMW[1];
					sPollGroup=dsAddressCMW[2];
					sAddressConfig=dsAddressCMW[3];
					unConfigCMW_checkType(sType, sPollGroup, exceptionInfo);
					unConfigCMW_checkFormatCMWAddress(sAddressConfig,exceptionInfo);
				}
				
			// HFST
			dsAddressCMW=strsplit(configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CMW_ANALOGINPUT_ADDRESS_HFST],CMW_ADDRESS_CONFIG_SEPARATOR);
			if (dynlen(dsAddressCMW)>=3)
				{
					sType=dsAddressCMW[1];
					sPollGroup=dsAddressCMW[2];
					sAddressConfig=dsAddressCMW[3];
					unConfigCMW_checkType(sType, sPollGroup, exceptionInfo);
					unConfigCMW_checkFormatCMWAddress(sAddressConfig,exceptionInfo);
				}
				
			// MANREG
			dsAddressCMW=strsplit(configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CMW_ANALOGINPUT_ADDRESS_MANREG],CMW_ADDRESS_CONFIG_SEPARATOR);

			if (dynlen(dsAddressCMW)>=3)
				{
					sType=dsAddressCMW[1];
					sPollGroup=dsAddressCMW[2];
					sAddressConfig=dsAddressCMW[3];
					unConfigCMW_checkType(sType, sPollGroup, exceptionInfo);
					unConfigCMW_checkFormatCMWAddress(sAddressConfig,exceptionInfo);
				}
				
			// MPOSR
			dsAddressCMW=strsplit(configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CMW_ANALOGINPUT_ADDRESS_MPOSR],CMW_ADDRESS_CONFIG_SEPARATOR);
			if (dynlen(dsAddressCMW)>=3)
				{
					sType=dsAddressCMW[1];
					sPollGroup=dsAddressCMW[2];
					sAddressConfig=dsAddressCMW[3];
					unConfigCMW_checkType(sType, sPollGroup, exceptionInfo);
					unConfigCMW_checkFormatCMWAddress(sAddressConfig,exceptionInfo);
				}
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","CMW_AnalogInput_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// DIP_AnalogInput_setConfig
/**
Purpose: set configuration of given Analog Input datapoint for DIP front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

DIP_AnalogInput_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	int iDriverNumber;
	bit32 b32;
	string sDipSubscription;
	
	setBit(b32, UN_STSREG01_AUMOST, true);
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
	iDriverNumber = dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER];
	
// if archive name are in the config->overwrite panel setting.	
	if(dynlen(dsConfigs) == (UN_CONFIG_DIP_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		if(dsConfigs[UN_CONFIG_DIP_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_DIP_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];

		if(dsConfigs[UN_CONFIG_DIP_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_DIP_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];

		if(dsConfigs[UN_CONFIG_DIP_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_DIP_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
	}

//DebugN(g_dsImport_UnDIP_config);
// set stsreg01 to automode
	dpSet(sDpName + ".ProcessInput.StsReg01", b32);
	
// set periph address posst
	sDipSubscription = dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_DIP_ANALOGINPUT_ADDRESS_DIPITEM_POSST] +
											fwDIP_SEPERATOR +
											dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_DIP_ANALOGINPUT_ADDRESS_DIPTAG_POSST];
	fwPeriphAddress_set(sDpName + ".ProcessInput.PosSt",
						makeDynString("DIP",
									  iDriverNumber, 
									  sDipSubscription, 
									  0,
									  0,
									  0,
									  "",
									  "",
									  "",
									  "", 
									  g_dsImport_UnDIP_config[UNDIP_CLIENTLIST],
									  dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_DIP_ANALOGINPUT_ADDRESS_DIPBUFFER_POSST]),
						exceptionInfo);

// 2.2. Registers
	unConfigGenericFunctions_setCommonAllRegisters(dsConfigs,
					 makeDynString("StsReg01"),
					 makeDynString(),
					 makeDynString(),
					 -1,
					 exceptionInfo);
					 
// 2.3. Float values
// force the smooth config type to UN_CONFIG_DEADBAND_NONE
	dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_DIP_ANALOGINPUT_DEADBANDE_TYPE] = UN_CONFIG_DEADBAND_NONE;
	
	unConfigGenericFunctions_setCommonFloatIntBlock(dsConfigs,
					PVSS_MODBUS_FLOAT,
					makeDynString("PosSt", "HFStOrAuPosRSt"),
					makeDynInt(UN_CONFIG_DIP_ANALOGINPUT_UNIT, UN_CONFIG_DIP_ANALOGINPUT_FORMAT, 
											UN_CONFIG_DIP_ANALOGINPUT_DEADBANDE, UN_CONFIG_DIP_ANALOGINPUT_DEADBANDE_TYPE),		
					makeDynInt(UN_CONFIG_DIP_ANALOGINPUT_RANGEMIN, UN_CONFIG_DIP_ANALOGINPUT_RANGEMAX),
					makeDynString("PosSt"),
					makeDynInt(UN_CONFIG_DIP_ANALOGINPUT_ARCHIVE_ACTIVE, UN_CONFIG_DIP_ANALOGINPUT_ARCHIVE_TIME_FILTER),
					exceptionInfo);
					
	unConfigGenericFunctions_set5RangesAlert(dsConfigs, sDpName + ".ProcessInput.PosSt", "Position Status", UN_CONFIG_DIP_ANALOGINPUT_NORMAL_POSITION,
													 makeDynInt(UN_CONFIG_DIP_ANALOGINPUT_LLLIMIT, UN_CONFIG_DIP_ANALOGINPUT_LLIMIT, 
													 			UN_CONFIG_DIP_ANALOGINPUT_HLIMIT, UN_CONFIG_DIP_ANALOGINPUT_HHLIMIT),
													 exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------

// DIP_AnalogInput_checkConfig
/**
Purpose: check data before configuration for DIP front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

DIP_AnalogInput_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	bool bOk=false;
	
	if (dynlen(configLine) == (UN_CONFIG_DIP_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH))
		bOk = true;
	else if (dynlen(configLine) == (UN_CONFIG_DIP_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		bOk = true;
		unConfigGenericFunctions_checkArchive(configLine[UN_CONFIG_DIP_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																					configLine[UN_CONFIG_DIP_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																					configLine[UN_CONFIG_DIP_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																					exceptionInfo);
	}
	else
		bOk = false;
	
	if(bOk) {
	// 3. Float block
		unConfigGenericFunctions_checkCommonFloatIntBlock(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_DIP_ANALOGINPUT_UNIT, UN_CONFIG_DIP_ANALOGINPUT_FORMAT, UN_CONFIG_DIP_ANALOGINPUT_DEADBANDE, 
											UN_CONFIG_DIP_ANALOGINPUT_DEADBANDE_TYPE),		
				makeDynInt(UN_CONFIG_DIP_ANALOGINPUT_RANGEMIN, UN_CONFIG_DIP_ANALOGINPUT_RANGEMAX),
				makeDynInt(UN_CONFIG_DIP_ANALOGINPUT_ARCHIVE_ACTIVE, UN_CONFIG_DIP_ANALOGINPUT_ARCHIVE_TIME_FILTER),
				exceptionInfo);

	// 4. Particular : limits
		unConfigGenericFunctions_check5RangesAlert(configLine, UN_CONFIG_DIP_ANALOGINPUT_NORMAL_POSITION,
												   makeDynInt(UN_CONFIG_DIP_ANALOGINPUT_LLLIMIT, UN_CONFIG_DIP_ANALOGINPUT_LLIMIT, 
												 			  UN_CONFIG_DIP_ANALOGINPUT_HLIMIT, UN_CONFIG_DIP_ANALOGINPUT_HHLIMIT),
												   exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","DIP_AnalogInput_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// OPC_AnalogInput_setConfig
/**
Purpose: set configuration of given Analog Input datapoint for OPC front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

OPC_AnalogInput_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	int iDriverNumber;
	
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
	iDriverNumber = dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER];
	
// if archive name are in the config->overwrite panel setting.	
	if(dynlen(dsConfigs) == (UN_CONFIG_OPC_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		if(dsConfigs[UN_CONFIG_OPC_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_OPC_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];

		if(dsConfigs[UN_CONFIG_OPC_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_OPC_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];

		if(dsConfigs[UN_CONFIG_OPC_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_OPC_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
	}

//DebugN(g_dsImport_UnOPC_config);
// set periph address stsreg01, evstsreg01, posst, hfspostst
	fwPeriphAddress_set(sDpName + ".ProcessInput.StsReg01",
						makeDynString("OPC",
									  iDriverNumber, 
									  dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_OPC_ANALOGINPUT_ADDRESS_OPCITEM_STSREG01], 
									  DPATTR_ADDR_MODE_INPUT_SPONT,
									  OPC_UINT16,
									  UN_CONFIG_DEFAULT_ADDRESS_ACTIVE,
									  "",
									  "",
									  "",
									  "", 
									  true,
									  0,
									  g_dsImport_UnOPC_config[UNOPC_SERVERNAME], 
									  g_dsImport_UnOPC_config[UNOPC_GROUPINNAME],
									  g_dsImport_UnOPC_config[UNOPC_GROUPOUTNAME]
									  ),
						exceptionInfo);
	fwPeriphAddress_set(sDpName + ".ProcessInput.evStsReg01",
						makeDynString("OPC",
									  iDriverNumber, 
									  dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_OPC_ANALOGINPUT_ADDRESS_OPCITEM_EVSTSREG01], 
									  DPATTR_ADDR_MODE_INPUT_SPONT,
									  OPC_INT32,
									  UN_CONFIG_DEFAULT_ADDRESS_ACTIVE,
									  "",
									  "",
									  "",
									  "", 
									  true,
									  0,
									  g_dsImport_UnOPC_config[UNOPC_SERVERNAME], 
									  g_dsImport_UnOPC_config[UNOPC_GROUPINNAME],
									  g_dsImport_UnOPC_config[UNOPC_GROUPOUTNAME]
									  ),
						exceptionInfo);
	fwPeriphAddress_set(sDpName + ".ProcessInput.PosSt",
						makeDynString("OPC",
									  iDriverNumber, 
									  dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_OPC_ANALOGINPUT_ADDRESS_OPCITEM_POSST], 
									  DPATTR_ADDR_MODE_INPUT_SPONT,
									  OPC_FLOAT,
									  UN_CONFIG_DEFAULT_ADDRESS_ACTIVE,
									  "",
									  "",
									  "",
									  "", 
									  true,
									  0,
									  g_dsImport_UnOPC_config[UNOPC_SERVERNAME], 
									  g_dsImport_UnOPC_config[UNOPC_GROUPINNAME],
									  g_dsImport_UnOPC_config[UNOPC_GROUPOUTNAME]
									  ),
						exceptionInfo);
	fwPeriphAddress_set(sDpName + ".ProcessInput.HFStOrAuPosRSt",
						makeDynString("OPC",
									  iDriverNumber, 
									  dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_OPC_ANALOGINPUT_ADDRESS_OPCITEM_HFST], 
									  DPATTR_ADDR_MODE_INPUT_SPONT,
									  OPC_FLOAT,
									  UN_CONFIG_DEFAULT_ADDRESS_ACTIVE,
									  "",
									  "",
									  "",
									  "", 
									  true,
									  0,
									  g_dsImport_UnOPC_config[UNOPC_SERVERNAME], 
									  g_dsImport_UnOPC_config[UNOPC_GROUPINNAME],
									  g_dsImport_UnOPC_config[UNOPC_GROUPOUTNAME]
									  ),
						exceptionInfo);

// set periph address manreg01, mposr
	fwPeriphAddress_set(sDpName + ".ProcessOutput.ManReg01",
						makeDynString("OPC",
									  iDriverNumber, 
									  dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_OPC_ANALOGINPUT_ADDRESS_OPCITEM_MANREG01], 
									  DPATTR_ADDR_MODE_OUTPUT_SINGLE,
									  OPC_UINT16,
									  UN_CONFIG_DEFAULT_ADDRESS_ACTIVE,
									  "",
									  "",
									  "",
									  "", 
									  false,
									  0,
									  g_dsImport_UnOPC_config[UNOPC_SERVERNAME], 
									  g_dsImport_UnOPC_config[UNOPC_GROUPINNAME],
									  g_dsImport_UnOPC_config[UNOPC_GROUPOUTNAME]
										),
						exceptionInfo);
	fwPeriphAddress_set(sDpName + ".ProcessOutput.MPosR",
						makeDynString("OPC",
									  iDriverNumber, 
									  dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_OPC_ANALOGINPUT_ADDRESS_OPCITEM_MPOSR], 
									  DPATTR_ADDR_MODE_OUTPUT_SINGLE,
									  OPC_FLOAT,
									  UN_CONFIG_DEFAULT_ADDRESS_ACTIVE,
									  "",
									  "",
									  "",
									  "", 
									  false,
									  0,
									  g_dsImport_UnOPC_config[UNOPC_SERVERNAME], 
									  g_dsImport_UnOPC_config[UNOPC_GROUPINNAME],
									  g_dsImport_UnOPC_config[UNOPC_GROUPOUTNAME]
										),
						exceptionInfo);

// 2.2. Registers
	unConfigGenericFunctions_setCommonAllRegisters(dsConfigs,
					 makeDynString("StsReg01"),
					 makeDynString(),
					 makeDynString(),
					 -1,
					 exceptionInfo);
					 
// 2.3. Float values
	unConfigGenericFunctions_setCommonFloatIntBlock(dsConfigs,
					PVSS_MODBUS_FLOAT,
					makeDynString("PosSt", "HFStOrAuPosRSt"),
					makeDynInt(UN_CONFIG_OPC_ANALOGINPUT_UNIT, UN_CONFIG_OPC_ANALOGINPUT_FORMAT, 
											UN_CONFIG_OPC_ANALOGINPUT_DEADBANDE, UN_CONFIG_OPC_ANALOGINPUT_DEADBANDE_TYPE),		
					makeDynInt(UN_CONFIG_OPC_ANALOGINPUT_RANGEMIN, UN_CONFIG_OPC_ANALOGINPUT_RANGEMAX),
					makeDynString("PosSt"),
					makeDynInt(UN_CONFIG_OPC_ANALOGINPUT_ARCHIVE_ACTIVE, UN_CONFIG_OPC_ANALOGINPUT_ARCHIVE_TIME_FILTER),
					exceptionInfo);
					
	unConfigGenericFunctions_set5RangesAlert(dsConfigs, sDpName + ".ProcessInput.PosSt", "Position Status", UN_CONFIG_OPC_ANALOGINPUT_NORMAL_POSITION,
													 makeDynInt(UN_CONFIG_OPC_ANALOGINPUT_LLLIMIT, UN_CONFIG_OPC_ANALOGINPUT_LLIMIT, 
													 			UN_CONFIG_OPC_ANALOGINPUT_HLIMIT, UN_CONFIG_OPC_ANALOGINPUT_HHLIMIT),
													 exceptionInfo);
// 3. additional config
	unConfigGenericFunctions_setMaskEvent(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], 1, makeDynString(), exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------

// OPC_AnalogInput_checkConfig
/**
Purpose: check data before configuration for OPC front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

OPC_AnalogInput_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	bool bOk=false;
	
	if (dynlen(configLine) == (UN_CONFIG_OPC_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH))
		bOk = true;
	else if (dynlen(configLine) == (UN_CONFIG_OPC_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		bOk = true;
		unConfigGenericFunctions_checkArchive(configLine[UN_CONFIG_OPC_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																					configLine[UN_CONFIG_OPC_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																					configLine[UN_CONFIG_OPC_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																					exceptionInfo);
	}
	else
		bOk = false;
	
	if(bOk) {
	// 3. Float block
		unConfigGenericFunctions_checkCommonFloatIntBlock(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_OPC_ANALOGINPUT_UNIT, UN_CONFIG_OPC_ANALOGINPUT_FORMAT, UN_CONFIG_OPC_ANALOGINPUT_DEADBANDE, 
											UN_CONFIG_OPC_ANALOGINPUT_DEADBANDE_TYPE),		
				makeDynInt(UN_CONFIG_OPC_ANALOGINPUT_RANGEMIN, UN_CONFIG_OPC_ANALOGINPUT_RANGEMAX),
				makeDynInt(UN_CONFIG_OPC_ANALOGINPUT_ARCHIVE_ACTIVE, UN_CONFIG_OPC_ANALOGINPUT_ARCHIVE_TIME_FILTER),
				exceptionInfo);

	// 4. Particular : limits
		unConfigGenericFunctions_check5RangesAlert(configLine, UN_CONFIG_OPC_ANALOGINPUT_NORMAL_POSITION,
												   makeDynInt(UN_CONFIG_OPC_ANALOGINPUT_LLLIMIT, UN_CONFIG_OPC_ANALOGINPUT_LLIMIT, 
												 			  UN_CONFIG_OPC_ANALOGINPUT_HLIMIT, UN_CONFIG_OPC_ANALOGINPUT_HHLIMIT),
												   exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","OPC_AnalogInput_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------
// SOFT_FE_AnalogInput_setConfig
/**
Purpose: set configuration of given Analog Input datapoint for SOFT_FE front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
SOFT_FE_AnalogInput_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	bit32 b32;
	
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
	
// if archive name are in the config->overwrite panel setting.	
	if(dynlen(dsConfigs) == (UN_CONFIG_SOFT_FE_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		if(dsConfigs[UN_CONFIG_SOFT_FE_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_SOFT_FE_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];

		if(dsConfigs[UN_CONFIG_SOFT_FE_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_SOFT_FE_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];

		if(dsConfigs[UN_CONFIG_SOFT_FE_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_SOFT_FE_ANALOGINPUT_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
	}

//DebugN(g_dsImport_UnSOFT_FE_config);
// 2.2. Registers
	unConfigGenericFunctions_setCommonAllRegisters(dsConfigs,
					 makeDynString("StsReg01"),
					 makeDynString(),
					 makeDynString(),
					 -1,
					 exceptionInfo);
					 
// 2.3. Float values
	unConfigGenericFunctions_setCommonFloatIntBlock(dsConfigs,
					PVSS_MODBUS_FLOAT,
					makeDynString("PosSt", "HFStOrAuPosRSt"),
					makeDynInt(),		
					makeDynInt(UN_CONFIG_SOFT_FE_ANALOGINPUT_RANGEMIN, UN_CONFIG_SOFT_FE_ANALOGINPUT_RANGEMAX),
					makeDynString("PosSt"),
					makeDynInt(UN_CONFIG_SOFT_FE_ANALOGINPUT_ARCHIVE_ACTIVE, UN_CONFIG_SOFT_FE_ANALOGINPUT_ARCHIVE_TIME_FILTER),
					exceptionInfo);

	unConfigGenericFunctions_setUnit(sDpName + ".ProcessInput.PosSt", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGINPUT_UNIT], exceptionInfo);
	unConfigGenericFunctions_setFormat(sDpName + ".ProcessInput.PosSt", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGINPUT_FORMAT], exceptionInfo);
	unConfigGenericFunctions_setUnit(sDpName + ".ProcessInput.HFStOrAuPosRSt", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGINPUT_UNIT], exceptionInfo);
	unConfigGenericFunctions_setFormat(sDpName + ".ProcessInput.HFStOrAuPosRSt", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGINPUT_FORMAT], exceptionInfo);
					
	unConfigGenericFunctions_set5RangesAlert(dsConfigs, sDpName + ".ProcessInput.PosSt", "Position Status", UN_CONFIG_SOFT_FE_ANALOGINPUT_NORMAL_POSITION,
													 makeDynInt(UN_CONFIG_SOFT_FE_ANALOGINPUT_LLLIMIT, UN_CONFIG_SOFT_FE_ANALOGINPUT_LLIMIT, 
													 			UN_CONFIG_SOFT_FE_ANALOGINPUT_HLIMIT, UN_CONFIG_SOFT_FE_ANALOGINPUT_HHLIMIT),
													 exceptionInfo);
// set stsreg & evStsReg
// set mask
// set stsreg01 to automode
	setBit(b32, UN_STSREG01_AUMOST, true);
	dpSet(sDpName + ".ProcessInput.StsReg01", b32, sDpName + ".ProcessInput.evStsReg01", b32, 
			 sDpName + ".eventMask", 1, sDpName + ".ProcessInput.HFStOrAuPosRSt", 0);

}

//-----------------------------------------------------------------------------------------------------------

// SOFT_FE_AnalogInput_checkConfig
/**
Purpose: check data before configuration for SOFT_FE front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
SOFT_FE_AnalogInput_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	bool bOk=false;
	
	if (dynlen(configLine) == (UN_CONFIG_SOFT_FE_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH))
		bOk = true;
	else if (dynlen(configLine) == (UN_CONFIG_SOFT_FE_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		bOk = true;
		unConfigGenericFunctions_checkArchive(configLine[UN_CONFIG_SOFT_FE_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																					configLine[UN_CONFIG_SOFT_FE_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																					configLine[UN_CONFIG_SOFT_FE_ANALOGINPUT_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																					exceptionInfo);
	}
	else
		bOk = false;
	
	if(bOk) {
		unConfigGenericFunctions_checkUnitFormat(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGINPUT_UNIT], 
												 configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGINPUT_FORMAT], exceptionInfo);
		unConfigGenericFunctions_checkRange(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGINPUT_RANGEMIN], 
														configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGINPUT_RANGEMAX], exceptionInfo);
		unConfigGenericFunctions_checkArchiveParameters(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGINPUT_ARCHIVE_TIME_FILTER], 
														configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_SOFT_FE_ANALOGINPUT_ARCHIVE_ACTIVE], exceptionInfo);
				
	// 4. Particular : limits
		unConfigGenericFunctions_check5RangesAlert(configLine, UN_CONFIG_SOFT_FE_ANALOGINPUT_NORMAL_POSITION,
												   makeDynInt(UN_CONFIG_SOFT_FE_ANALOGINPUT_LLLIMIT, UN_CONFIG_SOFT_FE_ANALOGINPUT_LLIMIT, 
												 			  UN_CONFIG_SOFT_FE_ANALOGINPUT_HLIMIT, UN_CONFIG_SOFT_FE_ANALOGINPUT_HHLIMIT),
												   exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","SOFT_FE_AnalogInput_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}


//@}
