/**@name LIBRARY: unConfigController.ctl

@author: Vincent Forest (LHC-IAS)

Creation Date: 05 12 2002

Modification History:
  22/07/2009 Frederic
    - bug in the double definiton of the archive smoothing (especialy with VA,xx.x,A format)
    Add unConfigController_getMultiSmoothing()

  22/07/2009: Quique
   - unConfigController_setDefaultValuesAll includes the treatment of the exception in both check and set cases.

  14/07/2009: Herve
    - unConfigController_setDefaultValuesAll: missing exceptionInfo declaration
    
  04/11/2008 Quique
          - Double the definiton of the archive. The input can be ArchiveOUT,ArchiveMV_SP;ArchiveTimeOUT,ArchiveTimeMV_SP
            If only one is defined (former import files) the procedure assigns A;0 to the MV_SP case.
                  
  28/10/2008: Quique
          - Implementation of the new controller inputs (default values)
            Schneider case: keeps two cases, no parameters (former generator) and new parameters (9 default values)
            Siemens case: keeps two cases, some parameters with addressing (PID) and new parameters (9 default values)
                  
	25/04/2007: Herve
		- deadband and deadbandtype for SPMV and OUT instead of both
		
	15/11/2006: Herve
		- bug in case of _UnPlc type for event archive

	27/06/2006: Enrique
	 - Include the importation of the default PID parameters addresses 
	   (Functions: S7_PLC_Controller_checkConfig / S7_PLC_Controller_setConfig )
	 
	13/06/2006: Herve
		- case _UnPlc front-end device type, the default P, I, D and TDS are not used
		
	18/04/2006: Herve
		- implementation of mask event

	15/06/2005: Enrique
		- S7 adaptation
				
	22/04/2005: Herve
		- allow archive in import file per device

	15/04/2005: Herve
		- Modbus set and check front-end functions: _UnPlc_Controller_checkConfig, _UnPlc_Controller_setConfig

	22/01/2004 Frederic:
		in function unConfigController_setConfig(): Move "MV" in Float values SetPoint part

version 1.0

External Function : 
	. unConfigController_setConfig :  set configuration of given Controller datapoint
	. unConfigController_checkConfig : check data before configuration
	
Internal Functions :
	
Purpose: 
This library contains the function used to set Unicos Controller configuration.

Usage: Public

PVSS manager usage: Ctrl, NV, NG

Constraints:
	. JCOP framework
	. PVSS version: 2.12.1
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

// Constants
const unsigned UN_CONFIG_CONTROLLER_LENGTH = 43;

const unsigned UN_CONFIG_CONTROLLER_PIDNAME = 1;
const unsigned UN_CONFIG_CONTROLLER_UNIT_SP_MV = 2;
const unsigned UN_CONFIG_CONTROLLER_FORMAT_SP_MV = 3;
const unsigned UN_CONFIG_CONTROLLER_RANGEMAX_SP_MV = 4;
const unsigned UN_CONFIG_CONTROLLER_RANGEMIN_SP_MV = 5;
const unsigned UN_CONFIG_CONTROLLER_UNIT_OUT = 6;
const unsigned UN_CONFIG_CONTROLLER_FORMAT_OUT = 7;
const unsigned UN_CONFIG_CONTROLLER_RANGEMAX_OUT = 8;
const unsigned UN_CONFIG_CONTROLLER_RANGEMIN_OUT = 9;
const unsigned UN_CONFIG_CONTROLLER_DEADBANDE = 10;
const unsigned UN_CONFIG_CONTROLLER_DEADBANDE_TYPE = 11;
const unsigned UN_CONFIG_CONTROLLER_ARCHIVE_ACTIVE = 12;
const unsigned UN_CONFIG_CONTROLLER_ARCHIVE_TIME_FILTER = 13;
const unsigned UN_CONFIG_CONTROLLER_ADDRESS_STSREG01 = 14;
const unsigned UN_CONFIG_CONTROLLER_ADDRESS_STSREG02 = 15;
const unsigned UN_CONFIG_CONTROLLER_ADDRESS_EVSTSREG01 = 16;
const unsigned UN_CONFIG_CONTROLLER_ADDRESS_EVSTSREG02 = 17;
const unsigned UN_CONFIG_CONTROLLER_ADDRESS_ATSPO = 18;
const unsigned UN_CONFIG_CONTROLLER_ADDRESS_ATSPOL = 19;
const unsigned UN_CONFIG_CONTROLLER_ADDRESS_ATSPOH = 20;
const unsigned UN_CONFIG_CONTROLLER_ADDRESS_OUTOVST = 21;
const unsigned UN_CONFIG_CONTROLLER_ADDRESS_ATOUTOL = 22;
const unsigned UN_CONFIG_CONTROLLER_ADDRESS_ATOUTOH = 23;
const unsigned UN_CONFIG_CONTROLLER_ADDRESS_MV = 24;
const unsigned UN_CONFIG_CONTROLLER_ADDRESS_ATMSPO = 25;
const unsigned UN_CONFIG_CONTROLLER_ADDRESS_ATAUSPO = 26;
const unsigned UN_CONFIG_CONTROLLER_ADDRESS_MPOSRST = 27;
const unsigned UN_CONFIG_CONTROLLER_ADDRESS_AUPOSRST = 28;
const unsigned UN_CONFIG_CONTROLLER_ADDRESS_ATPKPST = 29;
const unsigned UN_CONFIG_CONTROLLER_ADDRESS_ATPTIST = 30;
const unsigned UN_CONFIG_CONTROLLER_ADDRESS_ATPTDST = 31;
const unsigned UN_CONFIG_CONTROLLER_ADDRESS_ATPKDST = 32;
const unsigned UN_CONFIG_CONTROLLER_ADDRESS_MANREG01 = 33;
const unsigned UN_CONFIG_CONTROLLER_ADDRESS_MOUTPOSR = 34;
const unsigned UN_CONFIG_CONTROLLER_ADDRESS_MSPO = 35;
const unsigned UN_CONFIG_CONTROLLER_ADDRESS_MSPOL = 36;
const unsigned UN_CONFIG_CONTROLLER_ADDRESS_MSPOH = 37;
const unsigned UN_CONFIG_CONTROLLER_ADDRESS_MOUTOL = 38;
const unsigned UN_CONFIG_CONTROLLER_ADDRESS_MOUTOH = 39;
const unsigned UN_CONFIG_CONTROLLER_ADDRESS_MKP = 40;
const unsigned UN_CONFIG_CONTROLLER_ADDRESS_MTI = 41;
const unsigned UN_CONFIG_CONTROLLER_ADDRESS_MTD = 42;
const unsigned UN_CONFIG_CONTROLLER_ADDRESS_MKD = 43;

// Format for parameters
const string UN_CONFIG_CONTROLLER_FORMAT_KP = "####.###";
const string UN_CONFIG_CONTROLLER_FORMAT_TI = "####.###";
const string UN_CONFIG_CONTROLLER_FORMAT_KD = "####.###";
const string UN_CONFIG_CONTROLLER_FORMAT_TD = "####.###";

const unsigned UN_CONFIG_CONTROLLER_ADDITIONAL_LENGTH_UNPLC = 1; // case _UnPlc, no default P, I, D and TDS

const unsigned UN_CONFIG_CONTROLLER_ADDITIONAL_LENGTH = 9;
const unsigned UN_CONFIG_CONTROLLER_MASKEVENT = 1;
const unsigned UN_CONFIG_CONTROLLER_ADDRESS_DEF_P=2;
const unsigned UN_CONFIG_CONTROLLER_DEFAULT_P = 3;
const unsigned UN_CONFIG_CONTROLLER_ADDRESS_DEF_I=4;
const unsigned UN_CONFIG_CONTROLLER_DEFAULT_I = 5;
const unsigned UN_CONFIG_CONTROLLER_ADDRESS_DEF_D=6;
const unsigned UN_CONFIG_CONTROLLER_DEFAULT_D = 7;
const unsigned UN_CONFIG_CONTROLLER_ADDRESS_DEF_TDS=8;
const unsigned UN_CONFIG_CONTROLLER_DEFAULT_TDS = 9;

// New implementation with all default values (PID, Spoint, Limits Sp, Limits OUT
const unsigned UN_CONFIG_CONTROLLER_ADDITIONAL_DEFV_ALL_LENGTH = 10;
// UN_CONFIG_CONTROLLER_MASKEVENT= 1 already defined
const unsigned UN_CONFIG_CONTROLLER_DEFV_KC =2;
const unsigned UN_CONFIG_CONTROLLER_DEFV_TI = 3;
const unsigned UN_CONFIG_CONTROLLER_DEFV_TD =4;
const unsigned UN_CONFIG_CONTROLLER_DEFV_TDS = 5;
const unsigned UN_CONFIG_CONTROLLER_DEFV_SPO=6;
const unsigned UN_CONFIG_CONTROLLER_DEFV_SPH = 7;
const unsigned UN_CONFIG_CONTROLLER_DEFV_SPL=8;
const unsigned UN_CONFIG_CONTROLLER_DEFV_OUTH = 9;
const unsigned UN_CONFIG_CONTROLLER_DEFV_OUTL = 10;

//@{

//------------------------------------------------------------------------------------------------------------------------

// unConfigController_setConfig
/**
Purpose:  set configuration of given Controller datapoint

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigController_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	
// 1. Initialization : checking + dpCreate
	unConfigGenericFunctions_checkAll(dsConfigs, "Controller", exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)		// Good inputs
	{
		sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
		unConfigGenericFunctions_createDp(sDpName, "Controller", exceptionInfo);
		if (dynlen(exceptionInfo) <= 0)	// Creation ok or dp already exists
		{
// 2. Generate device
// 2.1. Parameters
			unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
	// 2.2 call the device check function
			unConfigGenericFunctions_setDeviceConfig(dsConfigs, "Controller", exceptionInfo);
		}
	}
}

//------------------------------------------------------------------------------------------------------------------------

// unConfigController_checkConfig
/**
Purpose: check data before configuration

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigController_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	if (dynlen(configLine) >= UN_CONFIG_COMMON_LENGTH)
	{
	// 1. Common
		unConfigGenericFunctions_checkParameters(configLine, "Controller", exceptionInfo);
	// 2. call the device check function
		unConfigGenericFunctions_checkDeviceConfig(configLine, "Controller", exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","unConfigController_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// _UnPlc_Controller_setConfig
/**
Purpose:  set configuration of given Controller datapoint for _UnPlc front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

_UnPlc_Controller_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
string sDpName;
string sMask = "0";
int iMask;
float fDfltP, fDfltI, fDfltD, fDfltTDs;
float fDfltSp, fDfltSpH, fDfltSpL;
float fDfltOutH, fDlftOutL;
bool bSetDefaultAll;;
dyn_string dsDeadband, dsDeadbandType;
dyn_string dsArchiveActive, dsArchiveTimeFilter;
	
  // DebugN("FUNCTION: _UnPlc_Controller_setConfig(dsConfigs= "+dsConfigs);      
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];

  // if archive name are in the config->overwrite panel setting.	
	if(unConfigGenericFunctions_additionalDeviceInfo()) 
            {
		if(dynlen(dsConfigs) == (UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_ADDITIONAL_LENGTH_UNPLC)) 
                    {
			sMask = dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_MASKEVENT];
		    }
		else if(dynlen(dsConfigs) == (UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_CONTROLLER_ADDITIONAL_LENGTH_UNPLC)) 
                    {
			sMask = dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_CONTROLLER_MASKEVENT];

		    }
                // new case conf 9 default values without archives defined 
                else if (dynlen(dsConfigs) == (UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_ADDITIONAL_DEFV_ALL_LENGTH))
                    {
                        sMask = dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_MASKEVENT];
                        bSetDefaultAll = true;
                        sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_DEFV_KC], "%f", fDfltP);
			sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_DEFV_TI], "%f", fDfltI);
			sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_DEFV_TD], "%f", fDfltD);
			sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_DEFV_TDS], "%f", fDfltTDs);
                        
                        sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_DEFV_SPO], "%f", fDfltSp);
			sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_DEFV_SPH], "%f", fDfltSpH);
			sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_DEFV_SPL], "%f", fDfltSpL);
			sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_DEFV_OUTH], "%f", fDfltOutH);
                        sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_DEFV_OUTL], "%f", fDlftOutL);
                        
                    }
                // new case conf 9 default values / Archives defined
                else if (dynlen(dsConfigs) == (UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_CONTROLLER_ADDITIONAL_DEFV_ALL_LENGTH)) 
                    {
                        sMask = dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_CONTROLLER_MASKEVENT];
			bSetDefaultAll = true;
                        sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH +  UN_CONFIG_CONTROLLER_DEFV_KC], "%f", fDfltP);
			sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH +  UN_CONFIG_CONTROLLER_DEFV_TI], "%f", fDfltI);
			sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH +  UN_CONFIG_CONTROLLER_DEFV_TD], "%f", fDfltD);
			sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH +  UN_CONFIG_CONTROLLER_DEFV_TDS], "%f", fDfltTDs);
                        
                        sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH +  UN_CONFIG_CONTROLLER_DEFV_SPO], "%f", fDfltSp);
			sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH +  UN_CONFIG_CONTROLLER_DEFV_SPH], "%f", fDfltSpH);
			sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH +  UN_CONFIG_CONTROLLER_DEFV_SPL], "%f", fDfltSpL);
			sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH +  UN_CONFIG_CONTROLLER_DEFV_OUTH], "%f", fDfltOutH);
                        sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH +  UN_CONFIG_CONTROLLER_DEFV_OUTL], "%f", fDlftOutL);                        
                    }
                // mask event info
		sscanf(sMask, "%d", iMask);

		if((dynlen(dsConfigs) == (UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_CONTROLLER_ADDITIONAL_LENGTH_UNPLC)) ||
                   (dynlen(dsConfigs) == (UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_CONTROLLER_ADDITIONAL_DEFV_ALL_LENGTH)))                             
                    {
			if(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		    }
	    }
	else 
            {
		iMask = 1;
		if(dynlen(dsConfigs) == (UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) 
                    {
			if(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		    }
	     }

	unGenericDpFunctions_changeDpParameter(sDpName, UN_PARAMETER_NAME, dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_PIDNAME], exceptionInfo);
        
        // 2.2. Registers
	unConfigGenericFunctions_setAllRegisters(dsConfigs,
					 makeDynString("StsReg01","StsReg02"),
					 makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_STSREG01, UN_CONFIG_CONTROLLER_ADDRESS_STSREG02),
					 makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_EVSTSREG01, UN_CONFIG_CONTROLLER_ADDRESS_EVSTSREG02),
					 makeDynString("ManReg01"),
					 makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_MANREG01),
					 makeDynString(),
					 makeDynString(),
					 -1,
					 exceptionInfo);

	// keep the deadband and deadband type
	dsDeadband = unConfigGenericFunctions_getMultiGroup(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CONTROLLER_DEADBANDE]);
	while(dynlen(dsDeadband) <2)
		dynAppend(dsDeadband, "0");
	
  dsDeadbandType = unConfigGenericFunctions_getMultiGroup(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CONTROLLER_DEADBANDE_TYPE]);
	while(dynlen(dsDeadbandType) <2)
		dynAppend(dsDeadbandType, "0");
 
  //DebugN(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CONTROLLER_DEADBANDE], dsDeadband, dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CONTROLLER_DEADBANDE_TYPE], dsDeadbandType);

  // overwrite configLine UN_CONFIG_CONTROLLER_DEADBANDE and UN_CONFIG_CONTROLLER_DEADBANDE_TYPE for OUT
	dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CONTROLLER_DEADBANDE] = dsDeadband[1];
	dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CONTROLLER_DEADBANDE_TYPE] = dsDeadbandType[1];
        
  dsArchiveActive=unConfigController_getMultiSmoothing(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CONTROLLER_ARCHIVE_ACTIVE], exceptionInfo);
  // Archive treating (default for OUT is "A;0")       
  while(dynlen(dsArchiveActive) <2)
		dynAppend(dsArchiveActive, "A");
  
  dsArchiveTimeFilter=unConfigGenericFunctions_getMultiGroup(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CONTROLLER_ARCHIVE_TIME_FILTER]);
  while(dynlen(dsArchiveTimeFilter) <2)
    dynAppend(dsArchiveTimeFilter, "0");
        
  // Overwrite for the OutOVSt  
  dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CONTROLLER_ARCHIVE_ACTIVE]=dsArchiveActive[1];
  dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CONTROLLER_ARCHIVE_TIME_FILTER]=dsArchiveTimeFilter[1];
        
  // 2.4. Float values Output
	unConfigGenericFunctions_setFloatIntBlock(dsConfigs,
					PVSS_MODBUS_FLOAT,
					makeDynString("OutOVSt", "MPosRSt", "AuPosRSt", "AtOutOL", "AtOutOH"),
					makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_OUTOVST, UN_CONFIG_CONTROLLER_ADDRESS_MPOSRST, UN_CONFIG_CONTROLLER_ADDRESS_AUPOSRST, UN_CONFIG_CONTROLLER_ADDRESS_ATOUTOL, UN_CONFIG_CONTROLLER_ADDRESS_ATOUTOH),
					makeDynString("MOutPosR", "MOutOL", "MOutOH"),
					makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_MOUTPOSR, UN_CONFIG_CONTROLLER_ADDRESS_MOUTOL, UN_CONFIG_CONTROLLER_ADDRESS_MOUTOH),
					makeDynInt(UN_CONFIG_CONTROLLER_UNIT_OUT, UN_CONFIG_CONTROLLER_FORMAT_OUT, UN_CONFIG_CONTROLLER_DEADBANDE, UN_CONFIG_CONTROLLER_DEADBANDE_TYPE),		
					makeDynInt(UN_CONFIG_CONTROLLER_RANGEMIN_OUT, UN_CONFIG_CONTROLLER_RANGEMAX_OUT),
					makeDynString("OutOVSt"),
					makeDynInt(UN_CONFIG_CONTROLLER_ARCHIVE_ACTIVE, UN_CONFIG_CONTROLLER_ARCHIVE_TIME_FILTER),
					exceptionInfo);
        
	// overwrite configLine UN_CONFIG_CONTROLLER_DEADBANDE and UN_CONFIG_CONTROLLER_DEADBANDE_TYPE for SPMV
	dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CONTROLLER_DEADBANDE] = dsDeadband[2];
	dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CONTROLLER_DEADBANDE_TYPE] = dsDeadbandType[2];
        
  // Overwrite for the MV and SP
  dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CONTROLLER_ARCHIVE_ACTIVE]=dsArchiveActive[2];
  dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CONTROLLER_ARCHIVE_TIME_FILTER]=dsArchiveTimeFilter[2];

        // 2.3. Float values SetPoint
	unConfigGenericFunctions_setFloatIntBlock(dsConfigs,
					PVSS_MODBUS_FLOAT,
					makeDynString("AtSPo", "AtMSPo", "AtAuSPo", "AtSPoL", "AtSPoH", "MV"),
					makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_ATSPO, UN_CONFIG_CONTROLLER_ADDRESS_ATMSPO, UN_CONFIG_CONTROLLER_ADDRESS_ATAUSPO, UN_CONFIG_CONTROLLER_ADDRESS_ATSPOL, UN_CONFIG_CONTROLLER_ADDRESS_ATSPOH,UN_CONFIG_CONTROLLER_ADDRESS_MV),
					makeDynString("MSPo", "MSPoL", "MSPoH"),
					makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_MSPO, UN_CONFIG_CONTROLLER_ADDRESS_MSPOL, UN_CONFIG_CONTROLLER_ADDRESS_MSPOH),
					makeDynInt(UN_CONFIG_CONTROLLER_UNIT_SP_MV, UN_CONFIG_CONTROLLER_FORMAT_SP_MV, UN_CONFIG_CONTROLLER_DEADBANDE, UN_CONFIG_CONTROLLER_DEADBANDE_TYPE),		
					makeDynInt(UN_CONFIG_CONTROLLER_RANGEMIN_SP_MV, UN_CONFIG_CONTROLLER_RANGEMAX_SP_MV),
					makeDynString("AtSPo","MV"),
					makeDynInt(UN_CONFIG_CONTROLLER_ARCHIVE_ACTIVE, UN_CONFIG_CONTROLLER_ARCHIVE_TIME_FILTER),
					exceptionInfo);

        // 2.6. Float values Parameters
	unConfigGenericFunctions_setFloatIntBlock(dsConfigs,
					PVSS_MODBUS_FLOAT,
					makeDynString("AtPKpSt", "AtPTiSt", "AtPKdSt", "AtPTdSt"),
					makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_ATPKPST, UN_CONFIG_CONTROLLER_ADDRESS_ATPTIST, UN_CONFIG_CONTROLLER_ADDRESS_ATPKDST, UN_CONFIG_CONTROLLER_ADDRESS_ATPTDST),
					makeDynString("MKp", "MTi", "MKd", "MTd"),
					makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_MKP, UN_CONFIG_CONTROLLER_ADDRESS_MTI, UN_CONFIG_CONTROLLER_ADDRESS_MKD, UN_CONFIG_CONTROLLER_ADDRESS_MTD),
					makeDynInt(),		
					makeDynInt(),
					makeDynString(),
                                      // makeDynString("AtPKpSt", "AtPTiSt", "AtPKdSt", "AtPTdSt"),
					makeDynInt(),
                                      // makeDynInt(UN_CONFIG_ARCHIVE_ACTIVE_OLDNEW_AND_TIME, UN_CONFIG_CONTROLLER_ARCHIVE_TIME_FILTER),
					exceptionInfo);
	unConfigGenericFunctions_setFormat(sDpName + ".ProcessInput.AtPKpSt", UN_CONFIG_CONTROLLER_FORMAT_KP, exceptionInfo);
	unConfigGenericFunctions_setFormat(sDpName + ".ProcessInput.AtPTiSt", UN_CONFIG_CONTROLLER_FORMAT_TI, exceptionInfo);
	unConfigGenericFunctions_setFormat(sDpName + ".ProcessInput.AtPKdSt", UN_CONFIG_CONTROLLER_FORMAT_KD, exceptionInfo);
	unConfigGenericFunctions_setFormat(sDpName + ".ProcessInput.AtPTdSt", UN_CONFIG_CONTROLLER_FORMAT_TD, exceptionInfo);
	
  fwArchive_set(sDpName + ".ProcessInput.AtPKpSt", dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG], DPATTR_ARCH_PROC_SIMPLESM, DPATTR_COMPARE_OLD_NEW, 0, 0, exceptionInfo);
	fwArchive_set(sDpName + ".ProcessInput.AtPTiSt", dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG], DPATTR_ARCH_PROC_SIMPLESM, DPATTR_COMPARE_OLD_NEW, 0, 0, exceptionInfo);
	fwArchive_set(sDpName + ".ProcessInput.AtPKdSt", dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG], DPATTR_ARCH_PROC_SIMPLESM, DPATTR_COMPARE_OLD_NEW, 0, 0, exceptionInfo);
	fwArchive_set(sDpName + ".ProcessInput.AtPTdSt", dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG], DPATTR_ARCH_PROC_SIMPLESM, DPATTR_COMPARE_OLD_NEW, 0, 0, exceptionInfo);

  // 3. additional config
	unConfigGenericFunctions_setMaskEvent(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], iMask, 
																					makeDynString(".ProcessInput.evStsReg01", ".ProcessInput.evStsReg02"), exceptionInfo);
	if(bSetDefaultAll)
		unConfigController_setDefaultValuesAll(sDpName, fDfltP, fDfltI, fDfltD, fDfltTDs,
                                                   fDfltSp, fDfltSpH,  fDfltSpL,
                                                   fDfltOutH,  fDlftOutL, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// unConfigController_getMultiSmoothing
/**
Purpose: get the different smmothing

Parameters:
		sGroup: string, input, group name list "," separated
    exceptionInfo: dyn_string, input, Exception list
		return: dyn_string, output, the smoothing list

Usage: External function

PVSS manager usage: Ctrl, UI

Constraints:
	. PVSS version: 3.6 
	. operating system: XP, Linux
	. distributed system: yes.
*/

dyn_string unConfigController_getMultiSmoothing(string sGroup, dyn_string &exceptionInfo)
{
int iLen;  
dyn_string dsSplit, dsSmooth;  

//DebugN("unConfigController_getMultiSmoothing", strsplit(sGroup, UN_CONFIG_GROUP_SEPARATOR));

	dsSplit=strsplit(sGroup, UN_CONFIG_GROUP_SEPARATOR);
        
  iLen=dynlen(dsSplit);
  switch(iLen)
    {
    case 1:
    case 2:      
      dsSmooth=dsSplit;
      break;    

    //VA,xx.x,O            
    case 3:
      dsSmooth[1]=dsSplit[1]+UN_CONFIG_GROUP_SEPARATOR+dsSplit[2]+UN_CONFIG_GROUP_SEPARATOR+dsSplit[3];
      break;

    //VA,xx.x,O,Y   OR       N,VA,xx.x,O    
    case 4:
      if(dsSplit[1]=="VR" || dsSplit[1]=="VA")
        {
        dsSmooth[1]=dsSplit[1]+UN_CONFIG_GROUP_SEPARATOR+dsSplit[2]+UN_CONFIG_GROUP_SEPARATOR+dsSplit[3];
        dsSmooth[2]=dsSplit[4];
        }
      else
        {
        dsSmooth[1]=dsSplit[1];
        dsSmooth[2]=dsSplit[2]+UN_CONFIG_GROUP_SEPARATOR+dsSplit[3]+UN_CONFIG_GROUP_SEPARATOR+dsSplit[4];        
        }
      break;      
          
    //VA,xx.x,O,VA,XX.x,O      
    case 6:
      dsSmooth[1]=dsSplit[1]+UN_CONFIG_GROUP_SEPARATOR+dsSplit[2]+UN_CONFIG_GROUP_SEPARATOR+dsSplit[3];
      dsSmooth[2]=dsSplit[4]+UN_CONFIG_GROUP_SEPARATOR+dsSplit[5]+UN_CONFIG_GROUP_SEPARATOR+dsSplit[6];            
      break;
      
    default:
      fwException_raise(exceptionInfo,"ERROR","Wrong Archive Smoothing entry: " + sGroup,"");                           
    }
    
  return dsSmooth;         
        
}

//------------------------------------------------------------------------------------------------------------------------

// _UnPlc_Controller_checkConfig
/**
Purpose: check data before configuration for _UnPlc front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

_UnPlc_Controller_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	bool bOk=false;
	bool bCheckArchive;
	unsigned uUnsigned;
	float fFloat;
  int iBASE_INDEX;
	dyn_string dsDeadband, dsDeadbandType, dsArchiveActive, dsArchiveTimeFilter;
	
        // DebugN("FUNCTION::_UnPlc_Controller_setConfig(configLine=",configLine);
        
	if(unConfigGenericFunctions_additionalDeviceInfo()) 
            { 
                // Former definition (no parameters except the maskEvent)
		if (dynlen(configLine) == (UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CONTROLLER_ADDITIONAL_LENGTH_UNPLC)) 
                    {
                        // DebugN("FUNCTION: _UnPlc_CONTROLLER_checkConfig(case=Former definition (no parameters except the maskEvent)");
			bOk = true;          
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CONTROLLER_MASKEVENT], uUnsigned, exceptionInfo);
		    }
                // Former definition (no parameters except the maskEvent) + Archives
		else if (dynlen(configLine) == (UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_CONTROLLER_ADDITIONAL_LENGTH_UNPLC)) 
                    {
                        // DebugN("FUNCTION: _UnPlc_CONTROLLER_checkConfig(case=Former definition (no parameters except the maskEvent) + Archives");
			bOk = true;
			bCheckArchive = true;
			unConfigGenericFunctions_checkUnsigned(configLine[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_CONTROLLER_MASKEVENT], uUnsigned, exceptionInfo);
		    }
                // New all default values definition (9 parameters)
                else if (dynlen(configLine)== UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CONTROLLER_ADDITIONAL_DEFV_ALL_LENGTH)
                   { 
                        // DebugN("FUNCTION: _UnPlc_CONTROLLER_checkConfig(case=New all default values definition (without addressing)");
			bOk = true;
                        iBASE_INDEX=UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_COMMON_LENGTH;
                        
                        // Check default PID values          
			unConfigGenericFunctions_checkUnsigned(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_MASKEVENT], uUnsigned, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_KC], fFloat, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_TI], fFloat, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_TD], fFloat, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_TDS], fFloat, exceptionInfo);
                        // Check default SP & limits SP,OUT values  
                        unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_SPO], fFloat, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_SPH], fFloat, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_SPL], fFloat, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_OUTH], fFloat, exceptionInfo);
                        unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_OUTL], fFloat, exceptionInfo);
                    }
                // New case with Archives defined + new all Default values definition (without addressing)
                else if (dynlen(configLine)== UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_CONTROLLER_ADDITIONAL_DEFV_ALL_LENGTH) 
                   {
                        // DebugN("FUNCTION: _UnPlc_CONTROLLER_checkConfig(case=New case with Archives defined + new all Default values definition (without addressing)");
			bOk = true;
			bCheckArchive = true;
                        iBASE_INDEX=UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH;
                        
			unConfigGenericFunctions_checkUnsigned(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_MASKEVENT], uUnsigned, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_KC], fFloat, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_TI], fFloat, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_TD], fFloat, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_TDS], fFloat, exceptionInfo);
                        // Check default SP & limits SP,OUT values  
                        unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_SPO], fFloat, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_SPH], fFloat, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_SPL], fFloat, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_OUTH], fFloat, exceptionInfo);
                        unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_OUTL], fFloat, exceptionInfo);
                     } 
		else
			bOk = false;
	    }
	else 
            {
		if (dynlen(configLine) == (UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_COMMON_LENGTH))
			bOk = true;
		else if (dynlen(configLine) == (UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) 
                  {
			bOk = true;
			bCheckArchive = true;
		  }
		else
			bOk = false;
	    }
	
	if(bOk) 
            {
		if(bCheckArchive) 
                    {
			unConfigGenericFunctions_checkArchive(configLine[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
							      configLine[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
							      configLine[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
					                      exceptionInfo);
		    }
                
	  // 2. Registers
		unConfigGenericFunctions_checkAllRegisters(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_STSREG01, UN_CONFIG_CONTROLLER_ADDRESS_STSREG02),
				makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_EVSTSREG01, UN_CONFIG_CONTROLLER_ADDRESS_EVSTSREG02),
				makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_MANREG01),
				-1,
				exceptionInfo);

	  // keep the deadband and deadband type
		dsDeadband = unConfigGenericFunctions_getMultiGroup(configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CONTROLLER_DEADBANDE]);
		while(dynlen(dsDeadband) <2)
			dynAppend(dsDeadband, "0");
		dsDeadbandType = unConfigGenericFunctions_getMultiGroup(configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CONTROLLER_DEADBANDE_TYPE]);
		while(dynlen(dsDeadbandType) <2)
			dynAppend(dsDeadbandType, "0");

    // overwrite configLine UN_CONFIG_CONTROLLER_DEADBANDE and UN_CONFIG_CONTROLLER_DEADBANDE_TYPE for OUTPUT
		configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CONTROLLER_DEADBANDE] = dsDeadband[1];
		configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CONTROLLER_DEADBANDE_TYPE] = dsDeadbandType[1];

    dsArchiveActive=unConfigController_getMultiSmoothing(configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CONTROLLER_ARCHIVE_ACTIVE], exceptionInfo);
    
    // Archive treating (default for OUT is "A;0")       
    while(dynlen(dsArchiveActive) <2)
  		dynAppend(dsArchiveActive, "A");
  
    dsArchiveTimeFilter=unConfigGenericFunctions_getMultiGroup(configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CONTROLLER_ARCHIVE_TIME_FILTER]);
    while(dynlen(dsArchiveTimeFilter) <2)
      dynAppend(dsArchiveTimeFilter, "0");
        
    // Overwrite for the OutOVSt  
    configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CONTROLLER_ARCHIVE_ACTIVE]=dsArchiveActive[1];
    configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CONTROLLER_ARCHIVE_TIME_FILTER]=dsArchiveTimeFilter[1];                
                	        
    // 4. Float block Output
		unConfigGenericFunctions_checkFloatIntBlock(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_OUTOVST, UN_CONFIG_CONTROLLER_ADDRESS_MPOSRST, UN_CONFIG_CONTROLLER_ADDRESS_AUPOSRST, UN_CONFIG_CONTROLLER_ADDRESS_ATOUTOL, UN_CONFIG_CONTROLLER_ADDRESS_ATOUTOH),
				makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_MOUTPOSR, UN_CONFIG_CONTROLLER_ADDRESS_MOUTOL, UN_CONFIG_CONTROLLER_ADDRESS_MOUTOH),
				makeDynInt(UN_CONFIG_CONTROLLER_UNIT_OUT, UN_CONFIG_CONTROLLER_FORMAT_OUT, UN_CONFIG_CONTROLLER_DEADBANDE, UN_CONFIG_CONTROLLER_DEADBANDE_TYPE),		
				makeDynInt(UN_CONFIG_CONTROLLER_RANGEMIN_OUT, UN_CONFIG_CONTROLLER_RANGEMAX_OUT),
				makeDynInt(UN_CONFIG_CONTROLLER_ARCHIVE_ACTIVE, UN_CONFIG_CONTROLLER_ARCHIVE_TIME_FILTER),
				exceptionInfo);		
               
                
    // overwrite configLine UN_CONFIG_CONTROLLER_DEADBANDE and UN_CONFIG_CONTROLLER_DEADBANDE_TYPE for SPMV
		configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CONTROLLER_DEADBANDE] = dsDeadband[2];
		configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CONTROLLER_DEADBANDE_TYPE] = dsDeadbandType[2];
                
    // Overwrite for the MV and SP
    configLine[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CONTROLLER_ARCHIVE_ACTIVE]=dsArchiveActive[2];
    configLine[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CONTROLLER_ARCHIVE_TIME_FILTER]=dsArchiveTimeFilter[2];                
                

    // 3. Float block SetPoint
		unConfigGenericFunctions_checkFloatIntBlock(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_ATSPO, UN_CONFIG_CONTROLLER_ADDRESS_ATMSPO, UN_CONFIG_CONTROLLER_ADDRESS_ATAUSPO, UN_CONFIG_CONTROLLER_ADDRESS_ATSPOL, UN_CONFIG_CONTROLLER_ADDRESS_ATSPOH),
				makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_MSPO, UN_CONFIG_CONTROLLER_ADDRESS_MSPOL, UN_CONFIG_CONTROLLER_ADDRESS_MSPOH),
				makeDynInt(UN_CONFIG_CONTROLLER_UNIT_SP_MV, UN_CONFIG_CONTROLLER_FORMAT_SP_MV, UN_CONFIG_CONTROLLER_DEADBANDE, UN_CONFIG_CONTROLLER_DEADBANDE_TYPE),		
				makeDynInt(UN_CONFIG_CONTROLLER_RANGEMIN_SP_MV, UN_CONFIG_CONTROLLER_RANGEMAX_SP_MV),
				makeDynInt(UN_CONFIG_CONTROLLER_ARCHIVE_ACTIVE, UN_CONFIG_CONTROLLER_ARCHIVE_TIME_FILTER),
				exceptionInfo);
           
	  // 5. Float block MV
		unConfigGenericFunctions_checkAddresses(makeDynString(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CONTROLLER_ADDRESS_MV]), exceptionInfo);
	  
    // 6. Float block Parameters
		unConfigGenericFunctions_checkFloatIntBlock(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_ATPKPST, UN_CONFIG_CONTROLLER_ADDRESS_ATPTIST, UN_CONFIG_CONTROLLER_ADDRESS_ATPKDST, UN_CONFIG_CONTROLLER_ADDRESS_ATPTDST),
				makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_MKP, UN_CONFIG_CONTROLLER_ADDRESS_MTI, UN_CONFIG_CONTROLLER_ADDRESS_MKD, UN_CONFIG_CONTROLLER_ADDRESS_MTD),
				makeDynInt(),		
				makeDynInt(),
				makeDynInt(),
				exceptionInfo);
            }
	else
	    {
		fwException_raise(exceptionInfo,"ERROR","_UnPlc_Controller_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	    }
}

//------------------------------------------------------------------------------------------------------------------------

// S7_PLC_Controller_setConfig
/**
Purpose:  set configuration of given Controller datapoint for S7_PLC front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: win XP.
	. distributed system: yes.
*/

S7_PLC_Controller_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	string sMask = "0";
	int iMask;
	float fDfltP, fDfltI, fDfltD, fDfltTDs;
        float fDfltSp, fDfltSpH, fDfltSpL;
        float fDfltOutH, fDlftOutL; 
	bool bSetDefault,bSetDefaultAll;
	dyn_string dsDeadband, dsDeadbandType;
        dyn_string dsArchiveActive, dsArchiveTimeFilter;
	
	// DebugN("FUNCTION: S7_PLC_Controller_setConfig(dsConfigs= "+dsConfigs);
	
        
	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
        
        // Init values
        bSetDefault=false;
        bSetDefaultAll=false;
        
        // if archive name are in the config->overwrite panel setting.	
	if(unConfigGenericFunctions_additionalDeviceInfo()) 
          {
                // case former conf without archives defined 
		if(dynlen(dsConfigs) == (UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_ADDITIONAL_LENGTH)) 
                    {  
			sMask = dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_MASKEVENT];
			bSetDefault = true;
			sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_DEFAULT_P], "%f", fDfltP);
			sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_DEFAULT_I], "%f", fDfltI);
			sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_DEFAULT_D], "%f", fDfltD);
			sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_DEFAULT_TDS], "%f", fDfltTDs);
		    }
                // case former conf / with archives
		else if(dynlen(dsConfigs) == (UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_CONTROLLER_ADDITIONAL_LENGTH)) 
                    {  
			sMask = dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_CONTROLLER_MASKEVENT];
			bSetDefault = true;
			sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_CONTROLLER_DEFAULT_P], "%f", fDfltP);
			sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_CONTROLLER_DEFAULT_I], "%f", fDfltI);
			sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_CONTROLLER_DEFAULT_D], "%f", fDfltD);
			sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_CONTROLLER_DEFAULT_TDS], "%f", fDfltTDs);
		    }
                // new case conf 9 default values without archives defined 
                else if (dynlen(dsConfigs) == (UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_ADDITIONAL_DEFV_ALL_LENGTH))
                    {
                        sMask = dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_MASKEVENT];
                        bSetDefaultAll = true;
                        sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_DEFV_KC], "%f", fDfltP);
			sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_DEFV_TI], "%f", fDfltI);
			sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_DEFV_TD], "%f", fDfltD);
			sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_DEFV_TDS], "%f", fDfltTDs);
                        
                        sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_DEFV_SPO], "%f", fDfltSp);
			sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_DEFV_SPH], "%f", fDfltSpH);
			sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_DEFV_SPL], "%f", fDfltSpL);
			sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_DEFV_OUTH], "%f", fDfltOutH);
                        sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_DEFV_OUTL], "%f", fDlftOutL);
                        
                    }
                // new case conf 9 default values / Archives defined
                else if (dynlen(dsConfigs) == (UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_CONTROLLER_ADDITIONAL_DEFV_ALL_LENGTH)) 
                    {
                        sMask = dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_CONTROLLER_MASKEVENT];
			bSetDefaultAll = true;
                        sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH +  UN_CONFIG_CONTROLLER_DEFV_KC], "%f", fDfltP);
			sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH +  UN_CONFIG_CONTROLLER_DEFV_TI], "%f", fDfltI);
			sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH +  UN_CONFIG_CONTROLLER_DEFV_TD], "%f", fDfltD);
			sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH +  UN_CONFIG_CONTROLLER_DEFV_TDS], "%f", fDfltTDs);
                        
                        sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH +  UN_CONFIG_CONTROLLER_DEFV_SPO], "%f", fDfltSp);
			sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH +  UN_CONFIG_CONTROLLER_DEFV_SPH], "%f", fDfltSpH);
			sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH +  UN_CONFIG_CONTROLLER_DEFV_SPL], "%f", fDfltSpL);
			sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH +  UN_CONFIG_CONTROLLER_DEFV_OUTH], "%f", fDfltOutH);
                        sscanf(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH +  UN_CONFIG_CONTROLLER_DEFV_OUTL], "%f", fDlftOutL);                        
                    }
		sscanf(sMask, "%d", iMask);

		if((dynlen(dsConfigs) == (UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_CONTROLLER_ADDITIONAL_LENGTH)) || 
                   (dynlen(dsConfigs) == (UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_CONTROLLER_ADDITIONAL_DEFV_ALL_LENGTH)))
                    {
			if(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		    }
	  }
	else 
          {
		iMask = 1;
		if(dynlen(dsConfigs) == (UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) 
                  {
			if(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
	
			if(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
	
			if(dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
				dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
		  }
	  }

	unGenericDpFunctions_changeDpParameter(sDpName, UN_PARAMETER_NAME, dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_PIDNAME], exceptionInfo);

        // 2.2. Registers
	unConfigGenericFunctions_setAllRegistersS7(dsConfigs,
					 makeDynString("StsReg01","StsReg02"),
					 makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_STSREG01, UN_CONFIG_CONTROLLER_ADDRESS_STSREG02),
					 makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_EVSTSREG01, UN_CONFIG_CONTROLLER_ADDRESS_EVSTSREG02),
					 makeDynString("ManReg01"),
					 makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_MANREG01),
					 makeDynString(),
					 makeDynString(),
					 -1,
					 exceptionInfo);

	// keep the deadband and deadband type
	dsDeadband = unConfigGenericFunctions_getMultiGroup(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CONTROLLER_DEADBANDE]);
	while(dynlen(dsDeadband) <2)
		dynAppend(dsDeadband, "0");
	dsDeadbandType = unConfigGenericFunctions_getMultiGroup(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CONTROLLER_DEADBANDE_TYPE]);
	while(dynlen(dsDeadbandType) <2)
		dynAppend(dsDeadbandType, "0");
        //DebugN(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CONTROLLER_DEADBANDE], dsDeadband, dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CONTROLLER_DEADBANDE_TYPE], dsDeadbandType);
	// overwrite configLine UN_CONFIG_CONTROLLER_DEADBANDE and UN_CONFIG_CONTROLLER_DEADBANDE_TYPE for OUT
	dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CONTROLLER_DEADBANDE] = dsDeadband[1];
	dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CONTROLLER_DEADBANDE_TYPE] = dsDeadbandType[1];
        
  dsArchiveActive=unConfigController_getMultiSmoothing(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CONTROLLER_ARCHIVE_ACTIVE], exceptionInfo);
        
  // Archive treating (default for OUT is "A;0")
  while(dynlen(dsArchiveActive) <2)
		dynAppend(dsArchiveActive, "A");
        
  dsArchiveTimeFilter=unConfigGenericFunctions_getMultiGroup(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CONTROLLER_ARCHIVE_TIME_FILTER]);
  while(dynlen(dsArchiveTimeFilter) <2)
		dynAppend(dsArchiveTimeFilter, "0");
        
        // Overwrite for the OutOVSt
        dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CONTROLLER_ARCHIVE_ACTIVE]=dsArchiveActive[1];
        dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CONTROLLER_ARCHIVE_TIME_FILTER]=dsArchiveTimeFilter[1];
        
        // 2.4. Float values Output
	unConfigGenericFunctions_setFloatIntBlockS7(dsConfigs,
					PVSS_S7_FLOAT,
					makeDynString("OutOVSt", "MPosRSt", "AuPosRSt", "AtOutOL", "AtOutOH"),
					makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_OUTOVST, UN_CONFIG_CONTROLLER_ADDRESS_MPOSRST, UN_CONFIG_CONTROLLER_ADDRESS_AUPOSRST, UN_CONFIG_CONTROLLER_ADDRESS_ATOUTOL, UN_CONFIG_CONTROLLER_ADDRESS_ATOUTOH),
					makeDynString("MOutPosR", "MOutOL", "MOutOH"),
					makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_MOUTPOSR, UN_CONFIG_CONTROLLER_ADDRESS_MOUTOL, UN_CONFIG_CONTROLLER_ADDRESS_MOUTOH),
					makeDynInt(UN_CONFIG_CONTROLLER_UNIT_OUT, UN_CONFIG_CONTROLLER_FORMAT_OUT, UN_CONFIG_CONTROLLER_DEADBANDE, UN_CONFIG_CONTROLLER_DEADBANDE_TYPE),		
					makeDynInt(UN_CONFIG_CONTROLLER_RANGEMIN_OUT, UN_CONFIG_CONTROLLER_RANGEMAX_OUT),
					makeDynString("OutOVSt"),
					makeDynInt(UN_CONFIG_CONTROLLER_ARCHIVE_ACTIVE, UN_CONFIG_CONTROLLER_ARCHIVE_TIME_FILTER),
					exceptionInfo);

	// overwrite configLine UN_CONFIG_CONTROLLER_DEADBANDE and UN_CONFIG_CONTROLLER_DEADBANDE_TYPE for SPMV
	dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CONTROLLER_DEADBANDE] = dsDeadband[2];
	dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CONTROLLER_DEADBANDE_TYPE] = dsDeadbandType[2];
        
        // Overwrite for the MV and SP
        dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CONTROLLER_ARCHIVE_ACTIVE]=dsArchiveActive[2];
        dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CONTROLLER_ARCHIVE_TIME_FILTER]=dsArchiveTimeFilter[2];
        
        // 2.3. Float values SetPoint
	unConfigGenericFunctions_setFloatIntBlockS7(dsConfigs,
					PVSS_S7_FLOAT,
					makeDynString("AtSPo", "AtMSPo", "AtAuSPo", "AtSPoL", "AtSPoH", "MV"),
					makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_ATSPO, UN_CONFIG_CONTROLLER_ADDRESS_ATMSPO, UN_CONFIG_CONTROLLER_ADDRESS_ATAUSPO, UN_CONFIG_CONTROLLER_ADDRESS_ATSPOL, UN_CONFIG_CONTROLLER_ADDRESS_ATSPOH,UN_CONFIG_CONTROLLER_ADDRESS_MV),
					makeDynString("MSPo", "MSPoL", "MSPoH"),
					makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_MSPO, UN_CONFIG_CONTROLLER_ADDRESS_MSPOL, UN_CONFIG_CONTROLLER_ADDRESS_MSPOH),
					makeDynInt(UN_CONFIG_CONTROLLER_UNIT_SP_MV, UN_CONFIG_CONTROLLER_FORMAT_SP_MV, UN_CONFIG_CONTROLLER_DEADBANDE, UN_CONFIG_CONTROLLER_DEADBANDE_TYPE),		
					makeDynInt(UN_CONFIG_CONTROLLER_RANGEMIN_SP_MV, UN_CONFIG_CONTROLLER_RANGEMAX_SP_MV),
					makeDynString("AtSPo","MV"),
					makeDynInt(UN_CONFIG_CONTROLLER_ARCHIVE_ACTIVE, UN_CONFIG_CONTROLLER_ARCHIVE_TIME_FILTER),
					exceptionInfo);

        // 2.6. Float values Parameters
	unConfigGenericFunctions_setFloatIntBlockS7(dsConfigs,
					PVSS_S7_FLOAT,
					makeDynString("AtPKpSt", "AtPTiSt", "AtPKdSt", "AtPTdSt"),
					makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_ATPKPST, UN_CONFIG_CONTROLLER_ADDRESS_ATPTIST, UN_CONFIG_CONTROLLER_ADDRESS_ATPKDST, UN_CONFIG_CONTROLLER_ADDRESS_ATPTDST),
					makeDynString("MKp", "MTi", "MKd", "MTd"),
					makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_MKP, UN_CONFIG_CONTROLLER_ADDRESS_MTI, UN_CONFIG_CONTROLLER_ADDRESS_MKD, UN_CONFIG_CONTROLLER_ADDRESS_MTD),
					makeDynInt(),		
					makeDynInt(),
					makeDynString(),
					makeDynInt(),
                                        /* makeDynString("AtPKpSt", "AtPTiSt", "AtPKdSt", "AtPTdSt"),
					makeDynInt(),*/
					exceptionInfo);
	unConfigGenericFunctions_setFormat(sDpName + ".ProcessInput.AtPKpSt", UN_CONFIG_CONTROLLER_FORMAT_KP, exceptionInfo);
	unConfigGenericFunctions_setFormat(sDpName + ".ProcessInput.AtPTiSt", UN_CONFIG_CONTROLLER_FORMAT_TI, exceptionInfo);
	unConfigGenericFunctions_setFormat(sDpName + ".ProcessInput.AtPKdSt", UN_CONFIG_CONTROLLER_FORMAT_KD, exceptionInfo);
	unConfigGenericFunctions_setFormat(sDpName + ".ProcessInput.AtPTdSt", UN_CONFIG_CONTROLLER_FORMAT_TD, exceptionInfo);
	fwArchive_set(sDpName + ".ProcessInput.AtPKpSt", dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG], DPATTR_ARCH_PROC_SIMPLESM, DPATTR_COMPARE_OLD_NEW, 0, 0, exceptionInfo);
	fwArchive_set(sDpName + ".ProcessInput.AtPTiSt", dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG], DPATTR_ARCH_PROC_SIMPLESM, DPATTR_COMPARE_OLD_NEW, 0, 0, exceptionInfo);
	fwArchive_set(sDpName + ".ProcessInput.AtPKdSt", dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG], DPATTR_ARCH_PROC_SIMPLESM, DPATTR_COMPARE_OLD_NEW, 0, 0, exceptionInfo);
	fwArchive_set(sDpName + ".ProcessInput.AtPTdSt", dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG], DPATTR_ARCH_PROC_SIMPLESM, DPATTR_COMPARE_OLD_NEW, 0, 0, exceptionInfo);

        // 3. additional config
	unConfigGenericFunctions_setMaskEvent(dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME], iMask, 
																					makeDynString(".ProcessInput.evStsReg01", ".ProcessInput.evStsReg02"), exceptionInfo);
	if(bSetDefault)
		{
			unConfigController_setDefault(sDpName, fDfltP, fDfltI, fDfltD, fDfltTDs);
			
			// dsConfigs does not include archives			
			if(dynlen(dsConfigs) == (UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_ADDITIONAL_LENGTH)) 
				{
					
					unConfigGenericFunctions_setAddressDefaultPIDParam_S7(dsConfigs,				
						makeDynString("DfltP", "DfltI", "DfltD", "DfltTDs"),
						makeDynInt(UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_ADDRESS_DEF_P, 
											 UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_ADDRESS_DEF_I, 
											 UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_ADDRESS_DEF_D, 
											 UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CONTROLLER_ADDRESS_DEF_TDS),
						exceptionInfo);
				}
			// dsConfigs does not include archives	(include archives)
			else if(dynlen(dsConfigs) == (UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_CONTROLLER_ADDITIONAL_LENGTH)) 
				{
					unConfigGenericFunctions_setAddressDefaultPIDParam_S7(dsConfigs,				
						makeDynString("DfltP", "DfltI", "DfltD", "DfltTDs"),
						makeDynInt(UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_CONTROLLER_ADDRESS_DEF_P, 
										 	 UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_CONTROLLER_ADDRESS_DEF_I, 
										 	 UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_CONTROLLER_ADDRESS_DEF_D, 
										 	 UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_CONTROLLER_ADDRESS_DEF_TDS),
						exceptionInfo);
				}
		}
        else if (bSetDefaultAll)
            unConfigController_setDefaultValuesAll(sDpName, fDfltP, fDfltI, fDfltD, fDfltTDs,
                                                   fDfltSp, fDfltSpH,  fDfltSpL,
                                                   fDfltOutH,  fDlftOutL, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------

// S7_PLC_Controller_checkConfig
/**
Purpose: check data before configuration for _UnPlc front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

S7_PLC_Controller_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	// DebugN("FUNCTION: S7_PLC_CONTROLLER_checkConfig(configLine="+configLine);
	
	bool bOk=false;
	bool bCheckArchive;
	unsigned uUnsigned;
	float fFloat;
	dyn_string dsDeadband, dsDeadbandType, dsArchiveActive, dsArchiveTimeFilter;
	int iBASE_INDEX;
	dyn_string dsAddresses;
	
	if(unConfigGenericFunctions_additionalDeviceInfo()) 
            {
 
                // Former default values definition
                if (dynlen(configLine) == UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CONTROLLER_ADDITIONAL_LENGTH) 
                  {
                        // DebugN("FUNCTION: S7_PLC_CONTROLLER_checkConfig(case=Former default values definition)");
			bOk = true;
                        iBASE_INDEX=UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_COMMON_LENGTH;
                        
                        // Check default values          
			unConfigGenericFunctions_checkUnsigned(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_MASKEVENT], uUnsigned, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFAULT_P], fFloat, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFAULT_I], fFloat, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFAULT_D], fFloat, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFAULT_TDS], fFloat, exceptionInfo);
			
			// Check default values addresses			
			dynAppend(dsAddresses,configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_ADDRESS_DEF_P]);
			dynAppend(dsAddresses,configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_ADDRESS_DEF_I]);
			dynAppend(dsAddresses,configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_ADDRESS_DEF_D]);
			dynAppend(dsAddresses,configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_ADDRESS_DEF_TDS]);
			
			unConfigGenericFunctions_checkAddressS7(dsAddresses,UN_CONFIG_S7_PLC_DATATYPE_FLOAT,exceptionInfo);
                   }
                        
                // Archives defined + Former Default values definition
                else if (dynlen(configLine)== UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_CONTROLLER_ADDITIONAL_LENGTH) 
                   {
                        // DebugN("FUNCTION: S7_PLC_CONTROLLER_checkConfig(case=Archives defined + Former Default values definition");
			bOk = true;
			bCheckArchive = true;
                        iBASE_INDEX=UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH;
                        
			unConfigGenericFunctions_checkUnsigned(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_MASKEVENT], uUnsigned, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFAULT_P], fFloat, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFAULT_I], fFloat, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFAULT_D], fFloat, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFAULT_TDS], fFloat, exceptionInfo);

			// Check default values addresses
			dynAppend(dsAddresses,configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_ADDRESS_DEF_P]);
			dynAppend(dsAddresses,configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_ADDRESS_DEF_I]);
			dynAppend(dsAddresses,configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_ADDRESS_DEF_D]);
			dynAppend(dsAddresses,configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_ADDRESS_DEF_TDS]);

			unConfigGenericFunctions_checkAddressS7(dsAddresses,UN_CONFIG_S7_PLC_DATATYPE_FLOAT,exceptionInfo);
                    }  
                        
                // New all default values definition (without addressing)
                else if (dynlen(configLine)== UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CONTROLLER_ADDITIONAL_DEFV_ALL_LENGTH)
                   { 
                        // DebugN("FUNCTION: S7_PLC_CONTROLLER_checkConfig(case=New all default values definition (without addressing)");
			bOk = true;
                        iBASE_INDEX=UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_COMMON_LENGTH;
                        
                        // Check default PID values          
			unConfigGenericFunctions_checkUnsigned(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_MASKEVENT], uUnsigned, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_KC], fFloat, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_TI], fFloat, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_TD], fFloat, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_TDS], fFloat, exceptionInfo);
                        // Check default SP & limits SP,OUT values  
                        unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_SPO], fFloat, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_SPH], fFloat, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_SPL], fFloat, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_OUTH], fFloat, exceptionInfo);
                        unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_OUTL], fFloat, exceptionInfo);
                    }
                        
                // New case with Archives defined + new all Default values definition (without addressing)
                else if (dynlen(configLine)== UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH + UN_CONFIG_CONTROLLER_ADDITIONAL_DEFV_ALL_LENGTH) 
                   {
                        //DebugN("FUNCTION: S7_PLC_CONTROLLER_checkConfig(case=New case with Archives defined + new all Default values definition (without addressing)");
			bOk = true;
			bCheckArchive = true;
                        iBASE_INDEX=UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH;
                        
			unConfigGenericFunctions_checkUnsigned(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_MASKEVENT], uUnsigned, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_KC], fFloat, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_TI], fFloat, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_TD], fFloat, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_TDS], fFloat, exceptionInfo);
                        // Check default SP & limits SP,OUT values  
                        unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_SPO], fFloat, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_SPH], fFloat, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_SPL], fFloat, exceptionInfo);
			unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_OUTH], fFloat, exceptionInfo);
                        unConfigGenericFunctions_checkFloat(configLine[iBASE_INDEX + UN_CONFIG_CONTROLLER_DEFV_OUTL], fFloat, exceptionInfo);
                     }          
		 else
		        bOk = false;
	      }
	else // former generators without additional information
            {
		if (dynlen(configLine) == (UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_COMMON_LENGTH))
			bOk = true;
		else if (dynlen(configLine) == (UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) 
                    {
			bOk = true;
			bCheckArchive = true;
		    }
		else
			bOk = false;
	    }
	
	if(bOk) 
            {
		if(bCheckArchive) 
                    {
			unConfigGenericFunctions_checkArchive(configLine[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
							      configLine[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
							      configLine[UN_CONFIG_CONTROLLER_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
							      exceptionInfo);
		    }
	        // 2. Registers
		unConfigGenericFunctions_checkAllRegistersS7(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_STSREG01, UN_CONFIG_CONTROLLER_ADDRESS_STSREG02),
				makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_EVSTSREG01, UN_CONFIG_CONTROLLER_ADDRESS_EVSTSREG02),
				makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_MANREG01),
				-1,
				exceptionInfo);
                
     // keep the deadband and deadband type
		dsDeadband = unConfigGenericFunctions_getMultiGroup(configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CONTROLLER_DEADBANDE]);
		while(dynlen(dsDeadband) <2)
			dynAppend(dsDeadband, "0");
		dsDeadbandType = unConfigGenericFunctions_getMultiGroup(configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CONTROLLER_DEADBANDE_TYPE]);
		while(dynlen(dsDeadbandType) <2)
			dynAppend(dsDeadbandType, "0");

	  // overwrite configLine UN_CONFIG_CONTROLLER_DEADBANDE and UN_CONFIG_CONTROLLER_DEADBANDE_TYPE for OUTPUT
		configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CONTROLLER_DEADBANDE] = dsDeadband[1];
		configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CONTROLLER_DEADBANDE_TYPE] = dsDeadbandType[1];

    dsArchiveActive=unConfigController_getMultiSmoothing(configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CONTROLLER_ARCHIVE_ACTIVE], exceptionInfo);
    // Archive treating (default for OUT is "A;0")       
    while(dynlen(dsArchiveActive) <2)
  		dynAppend(dsArchiveActive, "A");
  
    dsArchiveTimeFilter=unConfigGenericFunctions_getMultiGroup(configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CONTROLLER_ARCHIVE_TIME_FILTER]);
    while(dynlen(dsArchiveTimeFilter) <2)
      dynAppend(dsArchiveTimeFilter, "0");
        
    // Overwrite for the OutOVSt  
    configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CONTROLLER_ARCHIVE_ACTIVE]=dsArchiveActive[1];
    configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CONTROLLER_ARCHIVE_TIME_FILTER]=dsArchiveTimeFilter[1];                
                                	
    // 4. Float block Output
		unConfigGenericFunctions_checkFloatIntBlockS7(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_OUTOVST, UN_CONFIG_CONTROLLER_ADDRESS_MPOSRST, UN_CONFIG_CONTROLLER_ADDRESS_AUPOSRST, UN_CONFIG_CONTROLLER_ADDRESS_ATOUTOL, UN_CONFIG_CONTROLLER_ADDRESS_ATOUTOH),
				makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_MOUTPOSR, UN_CONFIG_CONTROLLER_ADDRESS_MOUTOL, UN_CONFIG_CONTROLLER_ADDRESS_MOUTOH),
				makeDynInt(UN_CONFIG_CONTROLLER_UNIT_OUT, UN_CONFIG_CONTROLLER_FORMAT_OUT, UN_CONFIG_CONTROLLER_DEADBANDE, UN_CONFIG_CONTROLLER_DEADBANDE_TYPE),		
				makeDynInt(UN_CONFIG_CONTROLLER_RANGEMIN_OUT, UN_CONFIG_CONTROLLER_RANGEMAX_OUT),
				makeDynInt(),
				exceptionInfo);			
                
    // overwrite configLine UN_CONFIG_CONTROLLER_DEADBANDE and UN_CONFIG_CONTROLLER_DEADBANDE_TYPE for SPMV
		configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CONTROLLER_DEADBANDE] = dsDeadband[2];
		configLine[UN_CONFIG_COMMON_LENGTH+UN_CONFIG_CONTROLLER_DEADBANDE_TYPE] = dsDeadbandType[2];
                
    // Overwrite for the MV and SP
    configLine[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CONTROLLER_ARCHIVE_ACTIVE]=dsArchiveActive[2];
    configLine[UN_CONFIG_ALLCOMMON_LENGTH+UN_CONFIG_CONTROLLER_ARCHIVE_TIME_FILTER]=dsArchiveTimeFilter[2];
	
                // 3. Float block SetPoint
		unConfigGenericFunctions_checkFloatIntBlockS7(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_ATSPO, UN_CONFIG_CONTROLLER_ADDRESS_ATMSPO, UN_CONFIG_CONTROLLER_ADDRESS_ATAUSPO, UN_CONFIG_CONTROLLER_ADDRESS_ATSPOL, UN_CONFIG_CONTROLLER_ADDRESS_ATSPOH),
				makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_MSPO, UN_CONFIG_CONTROLLER_ADDRESS_MSPOL, UN_CONFIG_CONTROLLER_ADDRESS_MSPOH),
				makeDynInt(UN_CONFIG_CONTROLLER_UNIT_SP_MV, UN_CONFIG_CONTROLLER_FORMAT_SP_MV, UN_CONFIG_CONTROLLER_DEADBANDE, UN_CONFIG_CONTROLLER_DEADBANDE_TYPE),		
				makeDynInt(UN_CONFIG_CONTROLLER_RANGEMIN_SP_MV, UN_CONFIG_CONTROLLER_RANGEMAX_SP_MV),
				makeDynInt(UN_CONFIG_CONTROLLER_ARCHIVE_ACTIVE, UN_CONFIG_CONTROLLER_ARCHIVE_TIME_FILTER),
				exceptionInfo);
				
	        // 5. Float block MV
		unConfigGenericFunctions_checkAddressS7(makeDynString(configLine[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CONTROLLER_ADDRESS_MV]),
							UN_CONFIG_S7_PLC_DATATYPE_FLOAT, 
						        exceptionInfo);
			
	        // 6. Float block Parameters
		unConfigGenericFunctions_checkFloatIntBlockS7(configLine, UN_CONFIG_COMMON_LENGTH,
				makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_ATPKPST, UN_CONFIG_CONTROLLER_ADDRESS_ATPTIST, UN_CONFIG_CONTROLLER_ADDRESS_ATPKDST, UN_CONFIG_CONTROLLER_ADDRESS_ATPTDST),
				makeDynInt(UN_CONFIG_CONTROLLER_ADDRESS_MKP, UN_CONFIG_CONTROLLER_ADDRESS_MTI, UN_CONFIG_CONTROLLER_ADDRESS_MKD, UN_CONFIG_CONTROLLER_ADDRESS_MTD),
				makeDynInt(),		
				makeDynInt(),
				makeDynInt(),
				exceptionInfo);
	    }
	else
	    {
		fwException_raise(exceptionInfo,"ERROR","S7_PLC_Controller_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	    }
}

//------------------------------------------------------------------------------------------------------------------------

// unConfigController_setDefault
/**
Purpose:  set the default value for the Controller

Parameters:
		sDeviceDpName: string, input, the device DP
		fDfltP: float, input, the default P value
		fDfltI: float, input, the default I value
		fDfltD: float, input, the default D value
		fDfltTDs: float, input, the default TD value

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigController_setDefault(string sDeviceDpName, float fDfltP, float fDfltI, float fDfltD, float fDfltTDs)
{
	dpSet(sDeviceDpName+".ProcessOutput.DfltP", fDfltP, sDeviceDpName+".ProcessOutput.DfltI", fDfltI, 
						sDeviceDpName+".ProcessOutput.DfltD", fDfltD, sDeviceDpName+".ProcessOutput.DfltTDs", fDfltTDs);
}
//------------------------------------------------------------------------------------------------------------------------

// unConfigController_setDefaultValuesAll
/**
Purpose:  set all default values (PID, SP, SP & OUT limits) for the Controller

Parameters:
		sDeviceDpName: string, input, the device DP
		fDfltP: float, input, the default P value
		fDfltI: float, input, the default I value
		fDfltD: float, input, the default D value
		fDfltTDs: float, input, the default TD value
                fDfltSp: float, input, the default SP value
                fDfltSpH: float, input, the default SP High limit value
                fDfltSpL: float, input, the default SP Low limit value
                fDfltOutH: float, input, the default OUT High limit value
                fDlftOutL: float, input, the default OUT Low limit value
	     exceptionInfo: to catch the exceptions
Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.6 
	. operating system: XP, linux.
	. distributed system: yes.
*/

unConfigController_setDefaultValuesAll(string sDeviceDpName, 
                                       float fDfltP, float fDfltI, float fDfltD, float fDfltTDs,
                                       float fDfltSp, float fDfltSpH, float fDfltSpL,
                                       float fDfltOutH, float fDlftOutL, dyn_string &exceptionInfo)
{
  // dyn_string exceptionInfo;
  int iResult;
	// Check the existance of the dpes before setting. If not exist then make an exception
        if ( (dpExists(sDeviceDpName+".ProcessOutput.DfltP") == false)  || 
             (dpExists(sDeviceDpName+".ProcessOutput.DfltI") == false)  ||
             (dpExists(sDeviceDpName+".ProcessOutput.DfltD") == false)  || 
             (dpExists(sDeviceDpName+".ProcessOutput.DfltTDs") == false)  ||
             (dpExists(sDeviceDpName+".ProcessOutput.DfltSPo") == false)  || 
             (dpExists(sDeviceDpName+".ProcessOutput.DfltSPoH") == false)  ||
             (dpExists(sDeviceDpName+".ProcessOutput.DfltSPoL") == false)  || 
             (dpExists(sDeviceDpName+".ProcessOutput.DfltOutOH") == false) ||
             (dpExists(sDeviceDpName+".ProcessOutput.DfltOutOL") == false) )
          {
            // produce un exception
            fwException_raise(exceptionInfo,"ERROR","xxx_PLC_Controller_checkConfig: Controller Dflt DPEs non existing","");
          }
         
        else
          {
            // sets the existing dpes
            iResult = dpSet(sDeviceDpName+".ProcessOutput.DfltP", fDfltP, 
                  sDeviceDpName+".ProcessOutput.DfltI", fDfltI, 
    	          sDeviceDpName+".ProcessOutput.DfltD", fDfltD, 
                  sDeviceDpName+".ProcessOutput.DfltTDs", fDfltTDs,
                  sDeviceDpName+".ProcessOutput.DfltSPo", fDfltSp,
                  sDeviceDpName+".ProcessOutput.DfltSPoH", fDfltSpH,
                  sDeviceDpName+".ProcessOutput.DfltSPoL", fDfltSpL,
                  sDeviceDpName+".ProcessOutput.DfltOutOH", fDfltOutH,
                  sDeviceDpName+".ProcessOutput.DfltOutOL", fDlftOutL);
			if (iResult == -1)
				fwException_raise(exceptionInfo,"ERROR","xxx_PLC_Controller_setConfig: Controller Dflt DPEs not set","");
          }
}
//------------------------------------------------------------------------------------------------------------------------

// unConfigController_getDefault
/**
Purpose:  set the default value for the Controller

Parameters:
		sDeviceDpName: string, input, the device DP
		fDfltP: float, output, the default P value
		fDfltI: float, output, the default I value
		fDfltD: float, output, the default D value
		fDfltTDs: float, output, the default TD value

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.6 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigController_getDefault(string sDeviceDpName, float &fDfltP, float &fDfltI, float &fDfltD, float &fDfltTDs)
{
	dpGet(sDeviceDpName+".ProcessOutput.DfltP", fDfltP, sDeviceDpName+".ProcessOutput.DfltI", fDfltI, 
						sDeviceDpName+".ProcessOutput.DfltD", fDfltD, sDeviceDpName+".ProcessOutput.DfltTDs", fDfltTDs);
}

//------------------------------------------------------------------------------------------------------------------------

// unConfigController_getDefaultValuesAll
/**
Purpose:  set the all default values (PID, SP, SP & OUT limits) for the Controller

Parameters:
		sDeviceDpName: string, input, the device DP
		fDfltP: float, output, the default P value
		fDfltI: float, output, the default I value
		fDfltD: float, output, the default D value
		fDfltTDs: float, output, the default TD value
                fDfltSp: float, input, the default SP value
                fDfltSpH: float, input, the default SP High limit value
                fDfltSpL: float, input, the default SP Low limit value
                fDfltOutH: float, input, the default OUT High limit value
                fDlftOutL: float, input, the default OUT Low limit value
Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.6 
	. operating system: XP, linux..
	. distributed system: yes.
*/

unConfigController_getDefaultValuesAll(string sDeviceDpName, 
                                       float &fDfltP, float &fDfltI, float &fDfltD, float &fDfltTDs,
                                       float &fDfltSp, float &fDfltSpH, float &fDfltSpL,
                                       float &fDfltOutH, float &fDlftOutL)
{
        dpGet(sDeviceDpName+".ProcessOutput.DfltP", fDfltP, 
              sDeviceDpName+".ProcessOutput.DfltI", fDfltI, 
	      sDeviceDpName+".ProcessOutput.DfltD", fDfltD, 
              sDeviceDpName+".ProcessOutput.DfltTDs", fDfltTDs,
              sDeviceDpName+".ProcessOutput.DfltSPo", fDfltSp,
              sDeviceDpName+".ProcessOutput.DfltSPoH", fDfltSpH,
              sDeviceDpName+".ProcessOutput.DfltSPoL", fDfltSpL,
              sDeviceDpName+".ProcessOutput.DfltOutOH", fDfltOutH,
              sDeviceDpName+".ProcessOutput.DfltOutOL", fDlftOutL);
}

//@}
