/**@name LIBRARY: unDigitalInput.ctl

@author: Vincent Forest (AB-CO)

Creation Date: 04 06 2003

Modification History:
  27/03/2009: Frederic
	- unDigitalInput_WidgetRegisterCB(): fix problem with deviceType check during disconnection
	- unDigitalInput_ButtonRegisterCB(): remove fwException_raise(...) before g_dsUserAccess
	
  03/09/2008: Herve
    - allow rigth click if lock taken.
    
  28/12/2007: Herve
    - device acccess control
    
	02/05/2006: Herve
		- implementation of maskEvent in widget, faceplate and rigthClick
		
	11/04/2006: Herve
		- remove dynAppend(menuList, "SEPARATOR"); in unDigitalInput_MenuConfiguration function

	15/12/2005: Celine 
		- unDigitalInput_WidgetRegisterCB, unDigitalInput_FaceplateStatusRegisterCB,unDigitalInput_FaceplateStatusAnimationCB: 
		add g_bSystemConnected checking

	09/08/2005: Frederic
		- unDigitalInput_ButtonRegisterCB(), add evalScript to run dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS] function
 
	15/04/2005: Herve
		- unGenericObject_GetPLCNameFromDpName: return the PLCtype+"_"+plcName, in case of _UnPlc it returns _unPlc_+plcName
		
	05/11/2004: Herve
		- in the WidgetRegisterCB, faceplateRegiterCB: use the systemName of the $sIdentifier to dpConnect to the PLC

	26/10/2004: Herve
		- in the unDigitalInput_ButtonRegisterCB function: call the unDigitalInput_ButtonUserAccess to get the list of allowed buttons. This list is 
		reseted if the user logs in/out while the remote system is off.

	22/10/2004: Herve
		- modify the unGenericObject_FormatValue call

	18/10/2004: Herve
		- add to unDigitalInput_MenuConfiguration the function call unGenericObject_addTrendActionToMenu

	03/05/2004: Herve
		- during WidgetRegister and FaceplateRegister call unGenericObject_GetPLCNameFromDpName to get the PLC DPName
		- during WidgetRegister check if the dpType is correct

	28/04/2004: Herve
		- add function unDigitalInput_ObjectListGetValueTime

	14/04/2004: Herve
		- add the function unDigitalInput_HandleMenu
		- add the function unDigitalInput_UserLoginGetButtonState

	13/04/2004: Herve
		- in function unDigitalInput_ButtonRegisterCB add the set Manreg01 to 0 in case of dpConnect.
		- modification of the signature unDigitalInput_MenuConfiguration and add the creation of the menu.

	08/04/2004: Herve 
		modification of the signature of the function unDigitalInput_ButtonSetState

	25/03/2004: Herve
		in the function unDigitalInput_ButtonUserAccess, the set_alarm_limit is accessible only for expert (was before also operator)

	20/01/2004: Herve
		add in the function unDigitalInput_ButtonDisconnection the UN_FACEPLATE_BUTTON_SET_ALARM_LIMITS button.

	16/01/2004: Herve
		remove the dpConnect, dpDisconnect and callback function in the case of no PLC systemAlarm. Ease the development 
		and test.
		
	03/01/2004: Herve
		- add in all dpConnect and dpDisconnect the _unSystemAlarm_PLC_DS dp if the PLC is configured for systemIntegrity. 
		The g_bUnSystemAlarmPlc is set to true if the PLC is configured for systemIntegrity
		change the dpConnect and dpDisconnect of _aut_inv to _invalid

	06/10/2003: Herve
		add dpConnect on the _locl._alert_hdl._locked in order to detect when the alert_hdl config modified.

version 1.0

External Function : 
	. unDigitalInput_AcknowledgeAlarm : acknowledge DigitalInput alarms
	. unDigitalInput_FaceplateStatusDisconnection : animate faceplate disconnection
	. unDigitalInput_ButtonUserAccess : User access for DigitalInput buttons
	. unDigitalInput_ButtonSetState : set button state
	. unDigitalInput_WidgetDisconnection : animate widget disconnection
	. unDigitalInput_MenuConfiguration : configuration of popup menu

Internal Functions :
	. unDigitalInput_FaceplateStatusRegisterCB : register callback function
	. unDigitalInput_FaceplateLockCallBack : set up the lock callback function for the faceplate
	. unDigitalInput_FaceplateConnect: Connect function of the faceplate data
	. unDigitalInput_FaceplateDisconnect: Disconnect function of the faceplate data
	. unDigitalInput_FaceplateStatusAnimationCB_NoAlarm : animate faceplate
	. unDigitalInput_FaceplateStatusAnimationCB : animate faceplate
	. unDigitalInput_ButtonRegisterCB : register callback function
	. unDigitalInput_ButtonAnimationCB : animate buttons
	. unDigitalInput_ButtonDisconnection : animate buttons disconnection
	. unDigitalInput_WidgetRegisterCB : register callback function
	. unDigitalInput_WidgetLockCallBack : set up the lock callback function for the widget
	. unDigitalInput_WidgetConnect : Connect function of the widget data
	. unDigitalInput_WidgetDisconnect : Disconnect function of the widget data
	. unDigitalInput_WidgetAnimationCB : animate widget
	. unDigitalInput_WidgetAnimationCB_NoAlarm : animate widget without alarm
	
Purpose: This library contains DigitalInput object functions.

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. DigitalInput object (DPType etc.)
	. Global variables and $parameters defined in DigitalInput faceplate and widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

//@{

//---------------------------------------------------------------------------------------------------------------------------------------

// unDigitalInput_ObjectListGetValueTime
/**
Purpose: Function called from the objectList to return the time and value

Parameters:
	- sDeviceName, string, input, device name
	- sDeviceType, string, input, unicos object
	- dsReturnData, dyn_string, output, return value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unDigitalInput_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
	float value;
	string sTime;
	
	dpGet(sDeviceName+".ProcessInput.PosSt", value, sDeviceName+".ProcessInput.PosSt:_online.._stime", sTime);
	dsReturnData[1] = sTime;
	dsReturnData[2] = unGenericObject_FormatValue(dpGetFormat(sDeviceName+".ProcessInput.PosSt"), value) + " " 
							+ dpGetUnit(sDeviceName+".ProcessInput.PosSt");
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unDigitalInput_AcknowledgeAlarm
/**
Purpose: acknowledge DigitalInput alarms

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsNeedAck, dyn_string, output, DPEs names that need acknowledgement. 
									 The PLC acknowledgement is specified using UN_ACKNOWLEDGE_PLC string
	
Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in DigitalInput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unDigitalInput_AcknowledgeAlarm(string sDpName, string sDpType, dyn_string &dsNeedAck)
{
	dsNeedAck = makeDynString("ProcessInput.PosSt");
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unDigitalInput_MenuConfiguration
/**
Purpose: configuration of popup menu

Parameters:	
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsAccessOk, dyn_string, input, authorized action
	- menuList, dyn_string, output, menu to pop-up
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unDigitalInput_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{
	dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, UN_POPUPMENU_MASK_POSST_TEXT, 
								 UN_POPUPMENU_ACK_TEXT, UN_POPUPMENU_FACEPLATE_TEXT, UN_POPUPMENU_TREND_TEXT);

	unGenericObject_addSelectToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	unGenericObject_addUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	unGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	unGenericObject_addTrendActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unDigitalInput_HandleMenu
/**
Purpose: handle the answer of the popup menu

Parameters:
	- deviceName, string, input, the device dp name
	- sDpType, string, input, the device dp type
	- menuList, dyn_string, input, list of requested action
	- menuAnswer, int, input, selected action from the menuList
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unDigitalInput_HandleMenu(string deviceName, string sDpType, dyn_string menuList, int menuAnswer)
{
	dyn_string exceptionInfo;

	switch(menuAnswer) {
		case UN_POPUPMENU_UNMASKEVENT:				// unmask event
			unGenericObject_setMaskEvent(deviceName, 1, 
																							makeDynString(".ProcessInput.evStsReg01", ".ProcessInput.PosSt"), exceptionInfo);
			break;
		case UN_POPUPMENU_MASKEVENT:				// mask event
			unGenericObject_setMaskEvent(deviceName, 0, 
																							makeDynString(".ProcessInput.evStsReg01", ".ProcessInput.PosSt"), exceptionInfo);
			break;
		default:
			unGenericObject_HandleUnicosMenu(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);
			break;
	}

	if (dynlen(exceptionInfo) > 0)
	{
		unSendMessage_toExpertException("Right Click",exceptionInfo);
	}

}

//---------------------------------------------------------------------------------------------------------------------------------------

// unDigitalInput_FaceplateStatusRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in DigitalInput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unDigitalInput_FaceplateStatusRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, sSystemName, sPlcName;
	int iAction, iRes, iAlertType1;
	dyn_string exceptionInfo;
	bool bRemote;
	
	deviceName = unGenericDpFunctions_getDpName($sDpName);
// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);

	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;

// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	if(sPlcName != "") {
		if (dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;

	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			unDigitalInput_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
			g_bDisconnectCB = false;
			if(g_bUnSystemAlarmPlc) {
				iRes = dpConnect("unDigitalInput_FaceplateLockCallBack", deviceName + ".ProcessInput.PosSt:_lock._alert_hdl._locked");
	
				g_bCallbackConnected = (iRes >= 0);
			}
			else
				unDigitalInput_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			if(g_bUnSystemAlarmPlc)
				iRes = dpDisconnect("unDigitalInput_FaceplateLockCallBack", deviceName + ".ProcessInput.PosSt:_lock._alert_hdl._locked");

			if(g_bDisconnectCB)
				unDigitalInput_FaceplateDisconnect(deviceName, iRes);

			g_bCallbackConnected = !(iRes >= 0);
			g_bDisconnectCB = false;
			unDigitalInput_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------
// unDigitalInput_FaceplateLockCallBack
/**
Purpose: set up the lock callback function for the faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in DigitalInput widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unDigitalInput_FaceplateLockCallBack(string sdpe, bool bLockState)
{
	int iRes;
	string deviceName;
//DebugN("unDigitalInput_FaceplateLockCallBack", bLockState, g_bDisconnectCB);

	deviceName = unGenericDpFunctions_getDpName(sdpe);
	if(bLockState) { // lock set so disconnect if not already disconnected
// no need to check if it was already set before because it is not possible to do 2 dpSet on the lock the Ev reject the message
		if(g_bDisconnectCB) {
			unDigitalInput_FaceplateDisconnect(deviceName, iRes);
//DebugN("unDigitalInput_FaceplateLockCallBack: disconnect");
			g_bDisconnectCB = false;
		}
		unDigitalInput_FaceplateStatusDisconnection();
	}
	else { // lock not set so connect if not already connected
// no need to check if it was already set before because it is not possible to do 2 dpSet on the lock the Ev reject the message
		g_bDisconnectCB = true;
		unDigitalInput_FaceplateConnect(deviceName);
//DebugN("unDigitalInput_FaceplateLockCallBack: connect");
	}
}


//---------------------------------------------------------------------------------------------------------------------------------------
// unDigitalInput_FaceplateConnect
/**
Purpose: Connect function of the faceplate data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in DigitalInput widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unDigitalInput_FaceplateConnect(string deviceName)
{
	int iRes, iAlertType1, iRangeType;
	dyn_string exceptionInfo;
	string sPlcName, sSystemName;
		
// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	if(sPlcName != "") {
		if (dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;
		
	iRes = dpGet(deviceName + ".ProcessInput.PosSt:_alert_hdl.._type", iAlertType1);
	g_bBoolAlert = ((iRes >= 0) && (iAlertType1 == DPCONFIG_ALERT_BINARYSIGNAL));
	if(g_bUnSystemAlarmPlc) {
		if (g_bBoolAlert)
		{
			iRes = dpConnect("unDigitalInput_FaceplateStatusAnimationCB",
						 deviceName + ".ProcessInput.StsReg01", 
						 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
						 deviceName + ".ProcessInput.PosSt:_alert_hdl.._active",
						 deviceName + ".ProcessInput.PosSt:_alert_hdl.._act_state",
						 deviceName + ".ProcessInput.PosSt:_alert_hdl.._ack_possible",
						 deviceName + ".ProcessInput.PosSt:_alert_hdl.._ok_range",
						 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
						 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
						 deviceName + ".eventMask");
		}
		else
		{
			iRes = dpConnect("unDigitalInput_FaceplateStatusAnimationCB_NoAlarm",
						 deviceName + ".ProcessInput.StsReg01", 
						 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
						 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
						 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
						 deviceName + ".eventMask");
		}
	}
	else
		unDigitalInput_FaceplateStatusDisconnection();
}
//---------------------------------------------------------------------------------------------------------------------------------------
// unDigitalInput_FaceplateDisconnect
/**
Purpose: Disconnect function of the faceplate data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in DigitalInput widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unDigitalInput_FaceplateDisconnect(string deviceName, int &iRes)
{
	string sPlcName, sSystemName;

	if(g_bUnSystemAlarmPlc) {
// get the systemName
		sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
		sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);

		if (g_bBoolAlert)
		{
			iRes = dpDisconnect("unDigitalInput_FaceplateStatusAnimationCB",
						 deviceName + ".ProcessInput.StsReg01", 
						 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
						 deviceName + ".ProcessInput.PosSt:_alert_hdl.._active",
						 deviceName + ".ProcessInput.PosSt:_alert_hdl.._act_state",
						 deviceName + ".ProcessInput.PosSt:_alert_hdl.._ack_possible",
						 deviceName + ".ProcessInput.PosSt:_alert_hdl.._ok_range",
						 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
						 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
						 deviceName + ".eventMask");
		}
		else
		{
			iRes = dpDisconnect("unDigitalInput_FaceplateStatusAnimationCB_NoAlarm",
						 deviceName + ".ProcessInput.StsReg01", 
						 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
						 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
						 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
						 deviceName + ".eventMask");
		}
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unDigitalInput_FaceplateStatusAnimationCB_NoAlarm
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unDigitalInput_FaceplateStatusAnimationCB_NoAlarm(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid, 
																									string sDpAl, int iValAlarm, string sDpEnabled, bool bPlcEnabled,
																									string sDpEvt, int iMask)
{
	unDigitalInput_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid, "", 0, "", 0, "", 0, "", 0, 
																						sDpAl, iValAlarm, sDpEnabled, bPlcEnabled, sDpEvt, iMask);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unDigitalInput_FaceplateStatusAnimationCB
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in DigitalInput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unDigitalInput_FaceplateStatusAnimationCB(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
								   		  string sDp5, bool bIsAlarmActive, string sDp6, int iAlarmState, 
								   		  string sDp7, bool bAckPossible, string sDp8, bool bOkRange,
				 			        	 string sDpAl, int iValAlarm, string sDpEnabled, bool bSystemIntegrityAlarmEnabled,
				 			        	 string sDpEvt, int iMask)
{
dyn_bool isDpConfigured;
// 1. Animate colorboxes
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OperationMode_Auto", getBit(bit32StsReg01, UN_STSREG01_AUMOST), 1, "unFaceplate_OperationModeActive", 
				bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OperationMode_Forced", getBit(bit32StsReg01, UN_STSREG01_FOMOST), 1, "unFaceplate_OperationModeActive", 
				bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)	
	unGenericObject_ColorBoxAnimate("Warning_IOError", getBit(bit32StsReg01, UN_STSREG01_IOERRORW), 1, "unFaceplate_WarningActive", 
				bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Warning_IOSimulated", getBit(bit32StsReg01, UN_STSREG01_IOSIMUW), 1, "unFaceplate_WarningActive", 
				bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Warning_ForcedProcess", getBit(bit32StsReg01, UN_STSREG01_FODIPRO), 1, "unFaceplate_WarningActive", 
				bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)	
	unGenericObject_ColorBoxAnimate("Request_Process_On", getBit(bit32StsReg01, UN_STSREG01_HFST), 1, "unFaceplate_RequestActive", 
				bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Request_Process_Off", 1 - getBit(bit32StsReg01, UN_STSREG01_HFST), 1, "unFaceplate_RequestActive", 
				bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Request_Active_On", getBit(bit32StsReg01, UN_STSREG01_POSST), 1, "unFaceplate_RequestActive", 
				bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Request_Active_Off", 1 - getBit(bit32StsReg01, UN_STSREG01_POSST), 1, "unFaceplate_RequestActive", 
				bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));

// 2. Animate alarm colorboxes
	if (g_bBoolAlert && bIsAlarmActive)
		{
		if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimateAlarm("Alarm_Position", iAlarmState, 1, 
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));	// Normal alarm
		if(g_bSystemConnected)	
		unGenericObject_ColorBoxAnimate("Alarm_UnAck", bAckPossible, 1, "unFaceplate_AlarmNotAck", 
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
		unProcessAlarm_isDpConfigured(substr(sDp5,0,strpos(sDp5,":_alert_hdl.._active")),DPCONFIG_ALERT_BINARYSIGNAL,2,isDpConfigured);					
		if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("MailSMS", dynContains(isDpConfigured, 1)>0, bIsAlarmActive,"unFaceplate_OperationModeActive",
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));	
		}
	else
		{
		if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("Alarm_Position", false, false, "", false);	// Mask alarm
		if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("Alarm_UnAck", false, false, "", false);
		if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("MailSMS", false, false, "", false);
		}
	if(g_bBoolAlert) {
		if(bOkRange) 
			normalPosition.number(0);
		else
			normalPosition.number(1);
	}
	else {
		normalPosition.number(2);
	}

// display event state
	if(g_bSystemConnected) {
		unGenericObject_ColorBoxAnimate("eventState", iMask, 1, "", 
				bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value),
				"unFaceplate_OperationModeActive");
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unDigitalInput_FaceplateStatusDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unDigitalInput_FaceplateStatusDisconnection()
{
	unGenericObject_ColorBoxDisconnect(makeDynString("Alarm_Position", "Alarm_UnAck", "OperationMode_Auto", "OperationMode_Forced", 
													 "Warning_IOError", "Warning_IOSimulated", "Warning_ForcedProcess",
													 "Request_Process_On", "Request_Process_Off", "Request_Active_On", 
													 "Request_Active_Off","MailSMS"));
	normalPosition.number(2);
// set eventState
	unGenericObject_ColorBoxDisconnect(makeDynString("eventState"));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unDigitalInput_ButtonRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in DigitalInput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unDigitalInput_ButtonRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, sDpType, sFunction;
	int iAction, iRes, iManReg01;
	dyn_string exceptionInfo, dsFunctions, dsUserAccess;
	
	deviceName = unGenericDpFunctions_getDpName($sDpName);
	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			unDigitalInput_ButtonDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
// Send 0 to ManReg01 if necessary
			iRes = dpGet(deviceName + ".ProcessOutput.ManReg01", iManReg01);
			if ((iRes < 0) || (iManReg01 != 0))
			{
				dpSet(deviceName + ".ProcessOutput.ManReg01", 0);
			}
		
// call the unDigitalInput_ButtonUserAccess to get the list of allowed buttons.
//			unDigitalInput_ButtonUserAccess(deviceName, "DigitalInput", g_dsUserAccess);

			sDpType="DigitalInput";
			unGenericObject_GetFunctions(sDpType, dsFunctions, exceptionInfo);
			sFunction = dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS];
						 
			if (sFunction != "")
				{
				evalScript(dsUserAccess, "dyn_string main(string deviceNameTemp, string sDpTypeTemp) {" + 
							 "dyn_string dsUserAccessReturn;" +
							 "if (isFunctionDefined(\"" + sFunction  + "\"))" +
							 "    {" +
					 			sFunction  + "(deviceNameTemp, sDpTypeTemp, dsUserAccessReturn);" +
							 "    }" +
							 "return dsUserAccessReturn; }", makeDynString(), deviceName, sDpType);

					g_dsUserAccess=dsUserAccess;	
				}
			else
				fwException_raise(exceptionInfo,"ERROR", "unDigitalInput_ButtonRegisterCB: " + getCatStr("unGeneration","UNKNOWNFUNCTION"), "EMPTY");
					
			
			iRes = dpConnect("unDigitalInput_ButtonAnimationCB",
							 deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							 deviceName + ".statusInformation.selectedManager",
							 deviceName + ".ProcessInput.StsReg01",
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid");
			g_bCallbackConnected = (iRes >= 0);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			iRes = dpDisconnect("unDigitalInput_ButtonAnimationCB",
							 deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							 deviceName + ".statusInformation.selectedManager",
							 deviceName + ".ProcessInput.StsReg01",
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid");
			g_bCallbackConnected = !(iRes >= 0);
			unDigitalInput_ButtonDisconnection();
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unDigitalInput_ButtonAnimationCB
/**
Purpose: animate buttons

Parameters: 
	- dbPermissions, dyn_bool, input, list of permissions, length = 4, order : 1=monitor, 2=operator, 3=expert, 4=admin
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unDigitalInput_ButtonAnimationCB(string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager, 
						  		 string sDpStsReg01, bit32 stsReg01Value, string sDpStsReg01Bad, bool bStsReg01Bad)
{
	unDigitalInput_ButtonSetState(unGenericDpFunctions_getDpName(sDpStsReg01), "DigitalInput", g_dsUserAccess, 
						 		  makeDynString(bLocked, sSelectedManager, stsReg01Value, bStsReg01Bad, 0, 0));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unDigitalInput_ButtonUserAccess
/**
Purpose: User access for DigitalInput buttons

Parameters: 
	- sDpName, string, input, device name
	- sDpType, string, input, dptype
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unDigitalInput_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
	bool operator, expert, admin;
	int i, length;
	dyn_bool dbPermissions;
	dyn_string exceptionInfo;
        string sOperAction, sExpAction, sAdminAction;
        bool bActionDefined = false;

        unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo);
        if((sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME) ||
           (sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME))
          bActionDefined=true;
	operator = dbPermissions[2];
	expert = dbPermissions[3];
	admin = dbPermissions[4];
	dsAccess = makeDynString();
	if (operator)
	{
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
          else
	    dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_SELECT, UN_FACEPLATE_BUTTON_ONOPEN_REQUEST,
										  UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST, UN_FACEPLATE_BUTTON_ACK_ALARM, UN_FACEPLATE_BUTTON_MASK_ALARM));
	}
	if (expert)
	{
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
          else
	    dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_FORCED_MODE, UN_FACEPLATE_BUTTON_SET_ALARM_LIMITS, UN_FACEPLATE_MASKEVENT));
	}
        if(admin){
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
        }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unDigitalInput_UserLoginGetButtonState
/**
Purpose: function called before the button set state when a user logs in. 

Parameters:
	deviceName: string, input, the device name
	dsReturnData: dyn_string, output, the data are returned here

Usage: Internal function

PVSS manager usage:  Ctrl, NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
		-> $sDpName, string, the device DP name
		-> g_sFaceplateButtonType, string, faceplate type
	. PVSS version: 2.12.1 
	. operating system: NT, W2000 and Linux, but tested only under W2000.
	. distributed system: yes but not tested.
*/

unDigitalInput_UserLoginGetButtonState(string deviceName, string sType, dyn_string &dsReturnData)
{
	unGenericObject_UserCBGetButtonState(deviceName, sType, dsReturnData);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unDigitalInput_ButtonSetState
/**
Purpose: set button state

Parameters: 
	- deviceName, string, input, device
	- sDpType, string, input, dptype
	- dsUserAccess, dyn_string, input, user access
	- dsData, dyn_string, parameter
		
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unDigitalInput_ButtonSetState(string deviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
	bool bSelected, buttonEnabled, bit1, bit2;
	dyn_string dsButtons;
	int i, length;
	string localManager;
	bool bLocked, bStsReg01Bad, bStsReg02Bad;
	string sSelectedManager;
	bit32 stsReg01Value, stsReg02Value; 

	bLocked = dsData[1];
	sSelectedManager = dsData[2];
	stsReg01Value = dsData[3];
	bStsReg01Bad = dsData[4];
	stsReg02Value = dsData[5];
	bStsReg02Bad = dsData[6];

//DebugN("DigitalInput_SetState", bLocked, sSelectedManager, stsReg01Value, bStsReg01Bad, stsReg02Value, bStsReg02Bad);

	dsButtons = makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_FORCED_MODE, UN_FACEPLATE_BUTTON_ONOPEN_REQUEST, 
							  UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST, UN_FACEPLATE_BUTTON_ACK_ALARM, UN_FACEPLATE_BUTTON_SET_ALARM_LIMITS);
	length = dynlen(dsButtons);
// 1. Selection state
	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
	bSelected = (localManager == "S");
// 2. Button state
	for(i=1;i<=length;i++)
		{
		buttonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0);	// User access
		switch (dsButtons[i])
			{
			case UN_FACEPLATE_BUTTON_FORCED_MODE:
				bit1 = !(getBit(stsReg01Value, UN_STSREG01_FOMOST) == 0) && !(bStsReg01Bad || bStsReg02Bad);	
				bit2 = !(getBit(stsReg01Value, UN_STSREG01_AUINHFMO) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				buttonEnabled = buttonEnabled && bSelected && !(bit1 || bit2);
				break;
			case UN_FACEPLATE_BUTTON_ONOPEN_REQUEST:
			case UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST:
			case UN_FACEPLATE_BUTTON_AUTO_MODE:
				bit1 = !(getBit(stsReg01Value, UN_STSREG01_AUMOST) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				buttonEnabled = buttonEnabled && bSelected && !bit1;
				break;
			case UN_FACEPLATE_BUTTON_ACK_ALARM:
			case UN_FACEPLATE_BUTTON_SET_ALARM_LIMITS:
			default:
				break;
			}
		setValue(UN_FACEPLATE_BUTTON_PREFIX + dsButtons[i], "enabled", buttonEnabled);
		}
// 3. Button Select state
	unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unDigitalInput_ButtonDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unDigitalInput_ButtonDisconnection()
{
	unGenericObject_ButtonDisconnect(makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_SELECT,
												   UN_FACEPLATE_BUTTON_ONOPEN_REQUEST, UN_FACEPLATE_BUTTON_FORCED_MODE,
												   UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST, UN_FACEPLATE_BUTTON_ACK_ALARM,
												   UN_FACEPLATE_BUTTON_SET_ALARM_LIMITS));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unDigitalInput_WidgetRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in DigitalInput widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unDigitalInput_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, deviceType, sSystemName, sPlcName;
	int iAction, iRes, iAlertType1;
	dyn_string exceptionInfo;
	bool bRemote;
	
	deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
	if (deviceName == "")	// In case of disconnection
		{
		deviceName = g_sDpName;
		}
// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);

	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;

	if(sPlcName != "") {
		if (dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;

//  check if the device name is of type "DigitalInput" or "DigitalOutput" if not like of no PLC. -> not available.  
	if (deviceName != "")
		{
		deviceType = dpTypeName(deviceName);
		if((deviceType != "DigitalInput") && (deviceType != "DigitalOutput"))  
			g_bUnSystemAlarmPlc = false;
		}	  
    
	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			unDigitalInput_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPCONNECT:
			g_sDpName = deviceName;
			g_bDisconnectCB = false;
			
			if(g_bUnSystemAlarmPlc) {
  			iRes = dpConnect("unDigitalInput_WidgetLockCallBack", deviceName + ".ProcessInput.PosSt:_lock._alert_hdl._locked");	
				g_bCallbackConnected = (iRes >= 0);
			}
			else
				unDigitalInput_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			if(g_bUnSystemAlarmPlc) 
				iRes = dpDisconnect("unDigitalInput_WidgetLockCallBack", deviceName + ".ProcessInput.PosSt:_lock._alert_hdl._locked");

			if(g_bDisconnectCB)
				unDigitalInput_WidgetDisconnect(g_sDpName, iRes);

			g_bCallbackConnected = !(iRes >= 0);
			g_bDisconnectCB = false;
			unDigitalInput_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}
//---------------------------------------------------------------------------------------------------------------------------------------
// unDigitalInput_WidgetLockCallBack
/**
Purpose: set up the lock callback function for the widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in DigitalInput widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unDigitalInput_WidgetLockCallBack(string sdpe, bool bLockState)
{
	int iRes;
//DebugN("unDigitalInput_WidgetLockCallBack", bLockState, g_bDisconnectCB);

	if(bLockState) { // lock set so disconnect if not already disconnected
// no need to check if it was already set before because it is not possible to do 2 dpSet on the lock the Ev reject the message
		if(g_bDisconnectCB) {
			unDigitalInput_WidgetDisconnect(g_sDpName, iRes);
//DebugN("unDigitalInput_WidgetLockCallBack: disconnect");
			g_bDisconnectCB = false;
		}
		unDigitalInput_WidgetDisconnection(g_sWidgetType);
                setValue("WidgetArea", "visible", true);
	}
	else { // lock not set so connect if not already connected
// no need to check if it was already set before because it is not possible to do 2 dpSet on the lock the Ev reject the message
		g_bDisconnectCB = true;
		unDigitalInput_WidgetConnect(g_sDpName);
//DebugN("unDigitalInput_WidgetLockCallBack: connect");
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------
// unDigitalInput_WidgetConnect
/**
Purpose: Connect function of the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in DigitalInput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unDigitalInput_WidgetConnect(string deviceName)
{
	int iRes, iAlertType1;
	dyn_string exceptionInfo;
	string sPlcName, sSystemName;
		
// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	if(sPlcName != "") {
		if (dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;
		
	iRes = dpGet(deviceName + ".ProcessInput.PosSt:_alert_hdl.._type", iAlertType1);
	g_bBoolAlert = ((iRes >= 0) && (iAlertType1 == DPCONFIG_ALERT_BINARYSIGNAL));
	if(g_bUnSystemAlarmPlc) {
		if (g_bBoolAlert)
		{
			iRes = dpConnect("unDigitalInput_WidgetAnimationCB",
						 deviceName + ".statusInformation.selectedManager:_lock._original._locked",
						 deviceName + ".statusInformation.selectedManager",
						 deviceName + ".ProcessInput.StsReg01",
						 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
						 deviceName + ".ProcessInput.PosSt:_alert_hdl.._active",
						 deviceName + ".ProcessInput.PosSt:_alert_hdl.._act_state",
						 deviceName + ".ProcessInput.PosSt:_alert_hdl.._ack_possible",
						 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
						 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
						 deviceName + ".eventMask");
		}
		else
		{
			iRes = dpConnect("unDigitalInput_WidgetAnimationCB_NoAlarm",
						 deviceName + ".statusInformation.selectedManager:_lock._original._locked",
						 deviceName + ".statusInformation.selectedManager",
						 deviceName + ".ProcessInput.StsReg01",
						 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
						 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
						 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
						 deviceName + ".eventMask");
		}
	}
	else
		unDigitalInput_WidgetDisconnection(g_sWidgetType);
}
//---------------------------------------------------------------------------------------------------------------------------------------
// unDigitalInput_WidgetDisconnect
/**
Purpose: Disconnect function of the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in DigitalInput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unDigitalInput_WidgetDisconnect(string deviceName, int &iRes)
{
	string sPlcName, sSystemName;
		
	if(g_bUnSystemAlarmPlc) {
// get the systemName
		sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
		sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);

		if (g_bBoolAlert)
		{
			iRes = dpDisconnect("unDigitalInput_WidgetAnimationCB",
						 deviceName + ".statusInformation.selectedManager:_lock._original._locked",
						 deviceName + ".statusInformation.selectedManager",
						 deviceName + ".ProcessInput.StsReg01",
						 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
						 deviceName + ".ProcessInput.PosSt:_alert_hdl.._active",
						 deviceName + ".ProcessInput.PosSt:_alert_hdl.._act_state",
						 deviceName + ".ProcessInput.PosSt:_alert_hdl.._ack_possible",
						 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
						 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
						 deviceName + ".eventMask");
		}
		else
		{
			iRes = dpDisconnect("unDigitalInput_WidgetAnimationCB_NoAlarm",
						 deviceName + ".statusInformation.selectedManager:_lock._original._locked",
						 deviceName + ".statusInformation.selectedManager",
						 deviceName + ".ProcessInput.StsReg01",
						 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
						 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
						 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
						 deviceName + ".eventMask");
		}
	}
}
//---------------------------------------------------------------------------------------------------------------------------------------

// unDigitalInput_WidgetAnimationCB_NoAlarm
/**
Purpose: Animate widget without alarm

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in DigitalInput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unDigitalInput_WidgetAnimationCB_NoAlarm(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
						   				 string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid,
										string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled, string sEvtMask, int iMask)
{
	unDigitalInput_WidgetAnimationCB(sDpLock, bLocked, sDpManager, sSelectedManager,
						   			 sDpStsReg01, bit32StsReg01, sDp4, bStsReg01Invalid, "", 0, "", 0, "", 0, 
						   			 sDpAl, iAlValue, sDpEnabled, bPlcEnabled, sEvtMask, iMask);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unDigitalInput_WidgetAnimationCB
/**
Purpose: Animate widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in DigitalInput faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unDigitalInput_WidgetAnimationCB(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
						   		 string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid,
						   		 string sDp5, bool bIsAlarmActive, string sDp6, int iActState, string sDp7, bool bAckPossible,
						   		 string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled, string sEvtMask, int iMask)
{
	unGenericObject_WidgetDIDOAnimation(bLocked, sSelectedManager, bit32StsReg01, bStsReg01Invalid, 
										g_bBoolAlert, bIsAlarmActive, iActState, bAckPossible, iAlValue, bPlcEnabled);

// display event state
	if(g_bSystemConnected) {
		if(iMask == 0)
			eventState.visible	= true;
		else
			eventState.visible	= false;
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unDigitalInput_WidgetDisconnection
/**
Purpose: Animate widget disconnection

Parameters:
	- sWidgetType, string, input, widget type. ex: "heater"

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unDigitalInput_WidgetDisconnection(string sWidgetType)
{
	unGenericObject_WidgetDisconnection(sWidgetType, "DigitalInput");
	// set event state
	eventState.visible	= false;
}
//@}
