/**@name LIBRARY: unLineComm.ctl

@author: Arnaud BLAIN (AB-CO)

Creation Date: 25/01/2007

Modification History: 
		
version 1.0




Internal Functions :
	. unAlarm_LineCommWidgetRegisterCB : register callback Function
	. unAlarm_LineCommDisconnection : animate widget disconnection
	. unAlarm_LineCommAnimationCB : animate widget

		
Purpose: This library contains functions for the LineComm widget.

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Alarm object (DPType etc.)
	. UNICOS (DPType etc.)
	. PVSS version: 3.0 
	. operating system: WXP.
	. distributed system: yes.
*/

//---------------------------------------------------------------------------------------------------------------------------------------
// unAlarm_LineCommWidgetRegisterCB
/**
Purpose: register callback function

Parameters:
	- sDp, string, input, dp
	- bSystemConnected, bool, input, state of the remote system

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Line Comm
	. PVSS version: 3.0 
	. operating system: WXP.
	. distributed system: yes.
*/
unAlarm_LineCommWidgetRegisterCB(string sDp, bool bSystemConnected)
{
	dyn_string exceptionInfo;
	string deviceName;
	int iAction, iRes, iAlertType1;
	string sPlcName, sSystemName;
	bool bRemote;
		
	deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
	if (deviceName == "")	// In case of disconnection
		{
		deviceName = g_sDpName;
		}
// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	
	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;

	if(sPlcName != "") {
		if (dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;

	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction){
		case UN_ACTION_DISCONNECT:
			unAlarm_LineCommDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPCONNECT:
			g_sDpName = deviceName;
			
//  check if the device name is of type "Alarm" if not like of no PLC. -> not available.
			if(dpTypeName(deviceName) != "Alarm")
				g_bUnSystemAlarmPlc = false;
				
				if(g_bUnSystemAlarmPlc) {
					iRes = dpConnect("unAlarm_LineCommAnimationCB", 
								deviceName + ".ProcessInput.PosAl",
								deviceName + ".ProcessInput.StsReg01",
								deviceName + ".ProcessInput.StsReg01:_online.._invalid",
								sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
								sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled");
				}	
			else {
				unAlarm_LineCommDisconnection(g_sWidgetType);
				}
				
			g_bCallbackConnected = (iRes >= 0);
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}
//---------------------------------------------------------------------------------------------------------------------------------------

//unAlarm_LineCommDisconnection
/**
Purpose: Animate LineComm widget disconnection

Parameters:
	- sWidgetType, string, input, widget type. ex: "heater"

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: WXP.
	. distributed system: yes.
*/

unAlarm_LineCommDisconnection(string sWidgetType)
{
	setValue("LineComm", "foreCol", "_dpdoesnotexist");
}

//---------------------------------------------------------------------------------------------------------------------------------------

//unAlarm_LineCommAnimationCB
/**
Purpose: Animate widget through Alarm status

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: WXP.
	. distributed system: yes.
*/
unAlarm_LineCommAnimationCB(string sDeviceName, bool bPosAl, string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid, string sDpPLCAlarm, int iAlarm, string sDpPLCEnabled, bool bEnabled)
{						
	
	//if StsReg01 is invalid
	if (bStsReg01Invalid){
		setValue( "LineComm", "foreCol", "unDataNotValid");
	}
	else{
		if(bPosAl==true){
		//if Communication BAD : RED
		setValue( "LineComm", "foreCol", "{255,0,0}");
	}
	else{
		//if Communication OK : GREEN
		setValue( "LineComm", "foreCol", "{0,255,0}");
	}
	}	
}
