/**@name LIBRARY: unLocal.ctl

@author: Frederic Bernard (AB-CO-IS)

Creation Date: 05 06 2003

Modification History:
  27/03/2009: Frederic
	- unLocal_WidgetConnect(): move g_bCallbackConnected upper
	- unLocal_ButtonRegisterCB(): remove fwException_raise(...) before g_dsUserAccess
        
  28/12/2007: Herve
    - device acccess control
    
	02/05/2006: Herve
		- implementation of maskEvent in widget, faceplate and rigthClick
		
	11/04/2006: Herve
		- remove dynAppend(menuList, "SEPARATOR"); in unLocal_MenuConfiguration function

	15/12/2005: Celine
		- unLocal_FaceplateStatusRegisterCB, unLocal_FaceplateStatusAnimationCB, unLocal_WidgetRegisterCB,
		 : add g_bSystemConnected checking

	09/08/2005: Frederic
		- unLocal_ButtonRegisterCB(), add evalScript to run dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS] function

	15/04/2005: Herve
		- unGenericObject_GetPLCNameFromDpName: return the PLCtype+"_"+plcName, in case of _UnPlc it returns _unPlc_+plcName
		
	05/11/2004: Herve
		- in the WidgetRegisterCB, faceplateRegiterCB: use the systemName of the $sIdentifier to dpConnect to the PLC

	26/10/2004: Herve
		- in the unLocal_ButtonRegisterCB function: call the unLocal_ButtonUserAccess to get the list of allowed buttons. This list is 
		reseted if the user logs in/out while the remote system is off.

	03/05/2004: Herve
		- during WidgetRegister and FaceplateRegister call unGenericObject_GetPLCNameFromDpName to get the PLC DPName
		- during WidgetRegister check if the dpType is correct

	28/04/2004: Herve
		- add function unLocal_ObjectListGetValueTime

	14/04/2004: Herve
		- add the function unLocal_HandleMenu
		- add the function unLocal_UserLoginGetButtonState

	13/04/2004: Herve
		- in function unLocal_ButtonRegisterCB add the set Manreg01 to 0 in case of dpConnect.
		- modification of the signature unLocal_MenuConfiguration and add the creation of the menu.

	08/04/2004: Herve 
		modification of the signature of the function unLocal_ButtonSetState

	16/01/2004: Herve
		remove the dpConnect, dpDisconnect and callback function in the case of no PLC systemAlarm. Ease the development 
		and test.
		
	03/01/2004: Herve
		- add in all dpConnect and dpDisconnect the _unSystemAlarm_PLC_DS dp if the PLC is configured for systemIntegrity. 
		The g_bUnSystemAlarmPlc is set to true if the PLC is configured for systemIntegrity
		change the dpConnect and dpDisconnect of _aut_inv to _invalid

version 1.0

External Function : 
	. unLocal_AcknowledgeAlarm : acknowledge Local alarms
	. unLocal_FaceplateStatusDisconnection : animate faceplate disconnection
	. unLocal_ButtonUserAccess : User access for Local buttons
	. unLocal_ButtonSetState : set button state
	. unLocal_WidgetDisconnection : animate widget disconnection
	. unLocal_MenuConfiguration : configuration of popup menu

Internal Functions :
	. unLocal_FaceplateStatusRegisterCB : register callback function
	. unLocal_FaceplateStatusAnimationCB_NoAlarm : animate faceplate
	. unLocal_FaceplateStatusAnimationCB : animate faceplate
	. unLocal_ButtonRegisterCB : register callback function
	. unLocal_ButtonAnimationCB : animate buttons
	. unLocal_ButtonDisconnection : animate buttons disconnection
	. unLocal_WidgetRegisterCB : register callback function
	. unLocal_WidgetAnimationCB : animate widget
		
Purpose: This library contains Local object functions.

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Local object (DPType etc.)
	. Global variables and $parameters defined in Local faceplate and widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

//@{

//---------------------------------------------------------------------------------------------------------------------------------------

// unLocal_ObjectListGetValueTime
/**
Purpose: Function called from the objectList to return the time and value

Parameters:
	- sDeviceName, string, input, device name
	- sDeviceType, string, input, unicos object
	- dsReturnData, dyn_string, output, return value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unLocal_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
	float value;
	string sTime;
	
	dpGet(sDeviceName+".ProcessInput.PosAl", value, sDeviceName+".ProcessInput.PosAl:_online.._stime", sTime);
	dsReturnData[1] = sTime;
	dsReturnData[2] = value;
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unLocal_AcknowledgeAlarm
/**
Purpose: acknowledge Local alarms

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsNeedAck, dyn_string, output, DPEs names that need acknowledgement. 
									 The PLC acknowledgement is specified using UN_ACKNOWLEDGE_PLC string
	
Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Local faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unLocal_AcknowledgeAlarm(string sDpName, string sDpType, dyn_string &dsNeedAck)
{
	dsNeedAck = makeDynString("ProcessInput.PosAl", UN_ACKNOWLEDGE_PLC);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unLocal_MenuConfiguration
/**
Purpose: configuration of popup menu

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsAccessOk, dyn_string, input, authorized action
	- menuList, dyn_string, output, menu to pop-up
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unLocal_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{
	dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_ACK_TEXT, UN_POPUPMENU_FACEPLATE_TEXT);

	unGenericObject_addSelectToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	unGenericObject_addUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	unGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unLocal_HandleMenu
/**
Purpose: handle the answer of the popup menu

Parameters:
	- deviceName, string, input, the device dp name
	- sDpType, string, input, the device dp type
	- menuList, dyn_string, input, list of requested action
	- menuAnswer, int, input, selected action from the menuList
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unLocal_HandleMenu(string deviceName, string sDpType, dyn_string menuList, int menuAnswer)
{
	dyn_string exceptionInfo;

	switch(menuAnswer) {
		case UN_POPUPMENU_UNMASKEVENT:				// unmask event
			unGenericObject_setMaskEvent(deviceName, 1, 
																							makeDynString(".ProcessInput.evStsReg01", ".ProcessInput.PosAl"), exceptionInfo);
			break;
		case UN_POPUPMENU_MASKEVENT:				// mask event
			unGenericObject_setMaskEvent(deviceName, 0, 
																							makeDynString(".ProcessInput.evStsReg01", ".ProcessInput.PosAl"), exceptionInfo);
			break;
		default:
			unGenericObject_HandleUnicosMenu(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);
			break;
	}

	if (dynlen(exceptionInfo) > 0)
	{
		unSendMessage_toExpertException("Right Click",exceptionInfo);
	}

}

//---------------------------------------------------------------------------------------------------------------------------------------

// unLocal_FaceplateStatusRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Local faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unLocal_FaceplateStatusRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, sSystemName;
	int iAction;
	dyn_string exceptionInfo;
	bool bRemote;
	
	deviceName = unGenericDpFunctions_getDpName($sDpName);
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;


	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			unLocal_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
			unLocal_FaceplateConnect(deviceName);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			unLocal_FaceplateDisconnect(deviceName);
			unLocal_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}
//---------------------------------------------------------------------------------------------------------------------------------------

// unLocal_FaceplateConnect
/**
Purpose: connect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Local faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unLocal_FaceplateConnect(string deviceName)
{
	int iRes, iAlertType1;
	dyn_string exceptionInfo;
	string sPlcName, sSystemName;

// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	if(sPlcName != "") {
		if (dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;
		
	iRes = dpGet(deviceName + ".ProcessInput.PosAl:_alert_hdl.._type", iAlertType1);
	g_bBoolAlert = ((iRes >= 0) && (iAlertType1 == DPCONFIG_ALERT_BINARYSIGNAL));
	if(g_bUnSystemAlarmPlc) {
		if (g_bBoolAlert)
			{
			iRes = dpConnect("unLocal_FaceplateStatusAnimationCB",
								 deviceName + ".ProcessInput.StsReg01", 
								 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
								 deviceName + ".ProcessInput.PosAl:_alert_hdl.._act_state",
								 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
								 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
								 deviceName + ".eventMask");
			}
		else
			{
			iRes = dpConnect("unLocal_FaceplateStatusAnimationCB_NoAlarm",
								 deviceName + ".ProcessInput.StsReg01", 
								 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
								 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
								 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
								 deviceName + ".eventMask");
			}
	}
	else 
		unLocal_FaceplateStatusDisconnection();
	g_bCallbackConnected = (iRes >= 0);
}
//---------------------------------------------------------------------------------------------------------------------------------------

// unLocal_FaceplateDisconnect
/**
Purpose: disconnect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Local faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unLocal_FaceplateDisconnect(string deviceName)
{
	int iRes;
	string sPlcName, sSystemName;

	if(g_bUnSystemAlarmPlc) {
// get the systemName
		sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
		sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);

		if (g_bBoolAlert)
		{
			iRes = dpDisconnect("unLocal_FaceplateStatusAnimationCB",
								 deviceName + ".ProcessInput.StsReg01", 
								 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
								 deviceName + ".ProcessInput.PosAl:_alert_hdl.._act_state",
								 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
								 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
								 deviceName + ".eventMask");
		}
		else
		{
			iRes = dpDisconnect("unLocal_FaceplateStatusAnimationCB_NoAlarm",
								 deviceName + ".ProcessInput.StsReg01", 
								 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
								 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
								 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
								 deviceName + ".eventMask");
		}	
	}
	g_bCallbackConnected = !(iRes >= 0);
}
//---------------------------------------------------------------------------------------------------------------------------------------

// unLocal_FaceplateStatusAnimationCB_NoAlarm
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unLocal_FaceplateStatusAnimationCB_NoAlarm(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
				 			        string sDpAl, int iValAlarm, string sDpEnabled, bool bPlcEnabled, string sDpEvt, int iMask)
{
	unLocal_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid,"", 0,
					 			    sDpAl, iValAlarm, sDpEnabled, bPlcEnabled, sDpEvt, iMask);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unLocal_FaceplateStatusAnimationCB
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Local faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unLocal_FaceplateStatusAnimationCB(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
								   string sDp3, int iPosAl,
					 			   string sDpAl, int iValAlarm, string sDpEnabled, bool bSystemIntegrityAlarmEnabled, string sDpEvt, int iMask)
{
// 1. Animate colorboxes
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Warning_IOError", getBit(bit32StsReg01, UN_STSREG01_IOERRORW), 1, "unFaceplate_WarningActive",	
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)			
	unGenericObject_ColorBoxAnimate("Warning_IOSimulated", getBit(bit32StsReg01, UN_STSREG01_IOSIMUW), 1, "unFaceplate_WarningActive", 
			bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)	
	unGenericObject_ColorBoxAnimate("Status_OnOpened", getBit(bit32StsReg01, UN_STSREG01_ONST), getBit(bit32StsReg01, UN_STSREG01_PHFONST), 
			"unFaceplate_StatusActive", bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)			
	unGenericObject_ColorBoxAnimate("Status_OffClosed", getBit(bit32StsReg01, UN_STSREG01_OFFST), getBit(bit32StsReg01, UN_STSREG01_PHFOFFST), 
			"unFaceplate_StatusActive", bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
	
// 2. Animate alarm colorboxes
	if ((getBit(bit32StsReg01, UN_STSREG01_POSALE) == 0) && g_bBoolAlert)
		{
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimate("Alarm_Position", 0, 0, "", 
				bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
		}
	else
		{
	if(g_bSystemConnected)
		unGenericObject_ColorBoxAnimateAlarm("Alarm_Position", iPosAl, g_bBoolAlert, 
				bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
		}
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Alarm_UnAck", getBit(bit32StsReg01, UN_STSREG01_ALUNACK), getBit(bit32StsReg01, UN_STSREG01_POSALE), 
				"unFaceplate_AlarmNotAck", bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));

// display event state
	if(g_bSystemConnected) {
		unGenericObject_ColorBoxAnimate("eventState", iMask, 1, "", 
				bStsReg01Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value),
				"unFaceplate_OperationModeActive");
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unLocal_FaceplateStatusDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unLocal_FaceplateStatusDisconnection()
{
	unGenericObject_ColorBoxDisconnect(makeDynString("Alarm_Position", "Alarm_UnAck",
													 "Warning_IOError", "Warning_IOSimulated", 
													 "Status_OnOpened", "Status_OffClosed"));
// set eventState
	unGenericObject_ColorBoxDisconnect(makeDynString("eventState"));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unLocal_ButtonRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Local faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unLocal_ButtonRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, sFunction, sDpType;
	int iAction, iRes, iManReg01;
	dyn_string exceptionInfo, dsUserAccess, dsFunctions;
	
	deviceName = unGenericDpFunctions_getDpName($sDpName);
	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			unLocal_ButtonDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
// Send 0 to ManReg01 if necessary
			iRes = dpGet(deviceName + ".ProcessOutput.ManReg01", iManReg01);
			if ((iRes < 0) || (iManReg01 != 0))
			{
				dpSet(deviceName + ".ProcessOutput.ManReg01", 0);
			}
		
// call the unLocal_ButtonUserAccess to get the list of allowed buttons.
//			unLocal_ButtonUserAccess(deviceName, "Local", g_dsUserAccess);

			sDpType="Local";
			unGenericObject_GetFunctions(sDpType, dsFunctions, exceptionInfo);
			sFunction = dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS];
						 
			if (sFunction != "")
				{
				evalScript(dsUserAccess, "dyn_string main(string deviceNameTemp, string sDpTypeTemp) {" + 
							 "dyn_string dsUserAccessReturn;" +
							 "if (isFunctionDefined(\"" + sFunction  + "\"))" +
							 "    {" +
					 			sFunction  + "(deviceNameTemp, sDpTypeTemp, dsUserAccessReturn);" +
							 "    }" +
							 "return dsUserAccessReturn; }", makeDynString(), deviceName, sDpType);
						
				g_dsUserAccess=dsUserAccess;	
				}
			else
				fwException_raise(exceptionInfo,"ERROR", "unLocal_ButtonRegisterCB:" + getCatStr("unGeneration","UNKNOWNFUNCTION"), "EMPTY");
					
			
			iRes = dpConnect("unLocal_ButtonAnimationCB",
							 deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							 deviceName + ".statusInformation.selectedManager",
							 deviceName + ".ProcessInput.StsReg01",
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid");
			g_bCallbackConnected = (iRes >= 0);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			iRes = dpDisconnect("unLocal_ButtonAnimationCB",
							 deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							 deviceName + ".statusInformation.selectedManager",
							 deviceName + ".ProcessInput.StsReg01",
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid");
			g_bCallbackConnected = !(iRes >= 0);
			unLocal_ButtonDisconnection();
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unLocal_ButtonAnimationCB
/**
Purpose: animate buttons

Parameters: 
	- dbPermissions, dyn_bool, input, list of permissions, length = 4, order : 1=monitor, 2=operator, 3=expert, 4=admin
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unLocal_ButtonAnimationCB(string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager, 
						  string sDpStsReg01, bit32 stsReg01Value, string sDpStsReg01Bad, bool bStsReg01Bad)
{
	unLocal_ButtonSetState(unGenericDpFunctions_getDpName(sDpStsReg01), "Local", g_dsUserAccess, 
						   makeDynString(bLocked, sSelectedManager, stsReg01Value, bStsReg01Bad, 0, 0));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unLocal_ButtonUserAccess
/**
Purpose: User access for Local buttons

Parameters: 
	- sDpName, string, input, device name
	- sDpType, string, input, dptype
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unLocal_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
	bool operator, expert, admin;
	int i, length;
	dyn_bool dbPermissions;
	dyn_string exceptionInfo;
        string sOperAction, sExpAction, sAdminAction;
        bool bActionDefined = false;

        unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo);
        if((sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME) ||
           (sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME))
          bActionDefined=true;
	operator = dbPermissions[2];
	expert = dbPermissions[3];
	admin = dbPermissions[4];
	dsAccess = makeDynString();
	if (operator)
	{
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
          else
	    dynAppend(dsAccess,UN_FACEPLATE_BUTTON_ACK_ALARM);
	}
	if (expert)
	{
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
          else
	    dynAppend(dsAccess, UN_FACEPLATE_MASKEVENT);
	}
        if(admin){
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
        }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unLocal_UserLoginGetButtonState
/**
Purpose: function called before the button set state when a user logs in. 

Parameters:
	deviceName: string, input, the device name
	dsReturnData: dyn_string, output, the data are returned here

Usage: Internal function

PVSS manager usage:  Ctrl, NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
		-> $sDpName, string, the device DP name
		-> g_sFaceplateButtonType, string, faceplate type
	. PVSS version: 2.12.1 
	. operating system: NT, W2000 and Linux, but tested only under W2000.
	. distributed system: yes but not tested.
*/

unLocal_UserLoginGetButtonState(string deviceName, string sType, dyn_string &dsReturnData)
{
	unGenericObject_UserCBGetButtonState(deviceName, sType, dsReturnData);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unLocal_ButtonSetState
/**
Purpose: set button state

Parameters: 
	- deviceName, string, input, device
	- sDpType, string, input, dptype
	- dsUserAccess, dyn_string, input, user access
	- dsData, dyn_string, parameter
		
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unLocal_ButtonSetState(string deviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
	bool bSelected, buttonEnabled;
	string localManager;
	
// 1. Selection state
	//localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
	//bSelected = (localManager == "S");
	
//DebugN("Local_SetState", bLocked, sSelectedManager, stsReg01Value, bStsReg01Bad, stsReg02Value, bStsReg02Bad);
// 2. Button state
	buttonEnabled = (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_ACK_ALARM) > 0);	// User access
	setValue(UN_FACEPLATE_BUTTON_PREFIX + UN_FACEPLATE_BUTTON_ACK_ALARM, "enabled", buttonEnabled);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unLocal_ButtonDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unLocal_ButtonDisconnection()
{
	unGenericObject_ButtonDisconnect(makeDynString(UN_FACEPLATE_BUTTON_ACK_ALARM));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unLocal_WidgetRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Local widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unLocal_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName;
	int iAction;
	dyn_string exceptionInfo;
	string sPlcName, sSystemName;
	bool bRemote;

	deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
	if (deviceName == "")	// In case of disconnection
		{
		deviceName = g_sDpName;
		}
// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	
	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;

	if(sPlcName != "") {
		if (dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else 
		g_bUnSystemAlarmPlc = false;
		
	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			unLocal_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPCONNECT:
			g_sDpName = deviceName;
			unLocal_WidgetConnect(deviceName, sPlcName, sSystemName);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			unLocal_WidgetDisconnect(deviceName, sPlcName, sSystemName);
			unLocal_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}
//---------------------------------------------------------------------------------------------------------------------------------------

// unLocal_WidgetConnect
/**
Purpose: Connect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Local faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unLocal_WidgetConnect(string deviceName, string sPlcName, string sSystemName)
{
	int iRes;
	
//  check if the device name is of type "Local" if not like of no PLC. -> not available.
	if(dpTypeName(deviceName) != "Local")
		g_bUnSystemAlarmPlc = false;
				
	if(g_bUnSystemAlarmPlc)
	{
		iRes = dpConnect("unLocal_WidgetAnimationCB", 
							deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							deviceName + ".statusInformation.selectedManager",
							deviceName + ".ProcessInput.StsReg01",
							deviceName + ".ProcessInput.StsReg01:_online.._invalid",
							sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							deviceName + ".eventMask");
	g_bCallbackConnected = (iRes >= 0);	
  }
	else 
		unLocal_WidgetDisconnection(g_sWidgetType);

}

//---------------------------------------------------------------------------------------------------------------------------------------

// unLocal_WidgetDisconnect
/**
Purpose: Connect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Local faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unLocal_WidgetDisconnect(string deviceName, string sPlcName, string sSystemName)
{
	int iRes;

	if(g_bUnSystemAlarmPlc)
	{
		iRes = dpDisconnect("unLocal_WidgetAnimationCB", 
							deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							deviceName + ".statusInformation.selectedManager",
							deviceName + ".ProcessInput.StsReg01",
							deviceName + ".ProcessInput.StsReg01:_online.._invalid",
							sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							deviceName + ".eventMask");
	}
	g_bCallbackConnected = !(iRes >= 0);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unLocal_WidgetAnimationCB
/**
Purpose: Animate widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Local faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unLocal_WidgetAnimationCB(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
						string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid,
						string sDpAl, int iAlValue, string sDpEnabled, bool bPlcEnabled, string sEvtMask, int iMask)
{
	unGenericObject_WidgetAnimation(g_sWidgetType, "Local", 
		bLocked, sSelectedManager, bit32StsReg01,
		makeDynInt(UN_STSREG01_IOSIMUW, UN_STSREG01_IOERRORW),
		makeDynString(UN_WIDGET_TEXT_WARNING_SIMU, UN_WIDGET_TEXT_WARNING_ERROR),
		makeDynInt(UN_STSREG01_POSAL), 
		makeDynString(UN_WIDGET_TEXT_ALARM_POSAL),
		makeDynInt(),
		makeDynString(),
		UN_STSREG01_ONST, UN_STSREG01_OFFST, UN_STSREG01_ALUNACK,
		bStsReg01Invalid, bStsReg01Invalid,
		iAlValue, bPlcEnabled);

// display event state
	if(g_bSystemConnected) {
		if(iMask == 0)
			eventState.visible	= true;
		else
			eventState.visible	= false;
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unLocal_WidgetDisconnection
/**
Purpose: Animate widget disconnection

Parameters:
	- sWidgetType, string, input, widget type. ex: "heater"

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unLocal_WidgetDisconnection(string sWidgetType)
{
	unGenericObject_WidgetDisconnection(sWidgetType, "Local");
	
	// set event state
	eventState.visible	= false;
}
//@}
