/**@name LIBRARY: unProcessControlObject.ctl

@author: Vincent Forest (AB-CO)

Creation Date: 21 05 2003

Modification History: 
  27/03/09: Frederic
 	- unProcessControlObject_WidgetConnect(): move g_bCallbackConnected upper
	- unProcessControlObject_ButtonRegisterCB(): remove fwException_raise(...) before g_dsUserAccess
        
  06/11/08: Quique
    - New bit for block alarm information. (faceplate/widget animation)
    
  28/12/2007: Herve
    - device acccess control
    
	02/05/2006: Herve
		- implementation of maskEvent in widget, faceplate and rigthClick
		
	11/04/2006: Herve
		- remove dynAppend(menuList, "SEPARATOR"); in unProcessControlObject_MenuConfiguration function

	15/12/2005: Celine
		- unProcessControlObject_FaceplateStatusRegisterCB, unProcessControlObject_FaceplateStatusAnimationCB, 
		unProcessControlObject_WidgetRegisterCB, unProcessControlObject_WidgetAnimationCB : add g_bSystemConnected checking
		
	09/08/2005: Frederic
		- unProcessControlObject_ButtonRegisterCB(), add evalScript to run dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS] function

	15/04/2005: Herve
		- unGenericObject_GetPLCNameFromDpName: return the PLCtype+"_"+plcName, in case of _UnPlc it returns _unPlc_+plcName
		
	05/11/2004: Herve
		- in the WidgetRegisterCB, faceplateRegiterCB: use the systemName of the $sIdentifier to dpConnect to the PLC

	26/10/2004: Herve
		- in the unProcessControlObject_ButtonRegisterCB function: call the unProcessControlObject_ButtonUserAccess to get the list of allowed buttons. This list is 
		reseted if the user logs in/out while the remote system is off.

	03/05/2004: Herve
		- during WidgetRegister and FaceplateRegister call unGenericObject_GetPLCNameFromDpName to get the PLC DPName
		- during WidgetRegister check if the dpType is correct

	14/04/2004: Herve
		- add the function unProcessControlObject_HandleMenu
		- add the function unProcessControlObject_UserLoginGetButtonState

	13/04/2004: Herve
		- in function unProcessControlObject_ButtonRegisterCB add the set Manreg01 to 0 in case of dpConnect.
		- modification of the signature unProcessControlObject_MenuConfiguration and add the creation of the menu.

	08/04/2004: Herve 
		modification of the signature of the function unProcessControlObject_ButtonSetState

	16/01/2004: Herve
		remove the dpConnect, dpDisconnect and callback function in the case of no PLC systemAlarm. Ease the development 
		and test.
		
	01/05/2004: Herve
		- add in all dpConnect and dpDisconnect the _unSystemAlarm_PLC_DS dp if the PLC is configured for systemIntegrity. 
		The g_bUnSystemAlarmPlc is set to true if the PLC is configured for systemIntegrity
		change the dpConnect and dpDisconnect of _aut_inv to _invalid

version 1.0

External Function : 
	. unProcessControlObject_AcknowledgeAlarm : acknowledge ProcessControlObject alarms
	. unProcessControlObject_FaceplateStatusDisconnection : animate faceplate disconnection
	. unProcessControlObject_ButtonUserAccess : User access for ProcessControlObject buttons
	. unProcessControlObject_ButtonSetState : set button state
	. unProcessControlObject_WidgetDisconnection : animate widget disconnection
	. unProcessControlObject_MenuConfiguration : configuration of popup menu

	. unPCO_getName, to get PCO name
	. unPCO_getOptionModesLabel, to get an option mode label
	. unPCO_getOptionModesAllowance, to get an option mode allowance
	. unPCO_setName, to set PCO name
	. unPCO_setOptionModes, to set an option mode label & allowance
	. unPCO_getNextAllowedModes, to get the modes you can access from current mode (using allowance tables)

Internal Functions :
	. unProcessControlObject_FaceplateStatusRegisterCB : register callback function
	. unProcessControlObject_FaceplateStatusAnimationCB_NoAlarm : animate faceplate
	. unProcessControlObject_FaceplateStatusAnimationCB : animate faceplate
	. unProcessControlObject_ButtonRegisterCB : register callback function
	. unProcessControlObject_ButtonAnimationCB : animate buttons
	. unProcessControlObject_ButtonDisconnection : animate buttons disconnection
	. unProcessControlObject_WidgetRegisterCB : register callback function
	. unProcessControlObject_WidgetAnimationCB : animate widget

Purpose: This library contains ProcessControlObject object functions.

Usage: Public

PVSS manager usage: NG, NV

Option modes are saved in the DPE "<PCO name> + UN_PCO_OPTION_MODE_PATH" in a UN_PCO_OPTION_MODE_DELIMITER delimited string.
First UN_PCO_OPTION_MODE_NUMBER parameters are the labels. 
Next UN_PCO_OPTION_MODE_NUMBER parameters are the allowance parameter for each mode.
Example :
Physics;Standby;Capacity;Warm Up;;;;;11000000;01100000;01100000;00010000;0;0;0;0
While running, you can only switch from physics to standby, from standby to capacity, from capacity to standby.
Warm up can't be changed while running.

Constraints:
	. ProcessControlObject object (DPType etc.)
	. Global variables and $parameters defined in ProcessControlObject faceplate and widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

//@{

//---------------------------------------------------------------------------------------------------------------------------------------

// unProcessControlObject_AcknowledgeAlarm
/**
Purpose: acknowledge ProcessControlObject alarms

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsNeedAck, dyn_string, output, DPEs names that need acknowledgement. 
									 The PLC acknowledgement is specified using UN_ACKNOWLEDGE_PLC string
	
Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in ProcessControlObject faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unProcessControlObject_AcknowledgeAlarm(string sDpName, string sDpType, dyn_string &dsNeedAck)
{
	dsNeedAck = makeDynString("ProcessInput.FuStopAl", "ProcessInput.StartAl", "ProcessInput.TStopAl", UN_ACKNOWLEDGE_PLC);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unProcessControlObject_MenuConfiguration
/**
Purpose: configuration of popup menu

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsAccessOk, dyn_string, input, authorized action
	- menuList, dyn_string, output, menu to pop-up
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unProcessControlObject_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{
	dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, UN_POPUPMENU_ACK_TEXT, UN_POPUPMENU_FACEPLATE_TEXT);

	unGenericObject_addSelectToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	unGenericObject_addUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	unGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unProcessControlObject_HandleMenu
/**
Purpose: handle the answer of the popup menu

Parameters:
	- deviceName, string, input, the device dp name
	- sDpType, string, input, the device dp type
	- menuList, dyn_string, input, list of requested action
	- menuAnswer, int, input, selected action from the menuList
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unProcessControlObject_HandleMenu(string deviceName, string sDpType, dyn_string menuList, int menuAnswer)
{
	dyn_string exceptionInfo;

	switch(menuAnswer) {
		case UN_POPUPMENU_UNMASKEVENT:				// unmask event
			unGenericObject_setMaskEvent(deviceName, 1, 
																							makeDynString(".ProcessInput.evStsReg01", ".ProcessInput.evStsReg02",
																								".ProcessInput.FuStopAl", ".ProcessInput.TStopAl", ".ProcessInput.StartAl"), exceptionInfo);
			break;
		case UN_POPUPMENU_MASKEVENT:				// mask event
			unGenericObject_setMaskEvent(deviceName, 0, 
																							makeDynString(".ProcessInput.evStsReg01", ".ProcessInput.evStsReg02",
																								".ProcessInput.FuStopAl", ".ProcessInput.TStopAl", ".ProcessInput.StartAl"), exceptionInfo);
			break;
		default:
			unGenericObject_HandleUnicosMenu(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);
			break;
	}

	if (dynlen(exceptionInfo) > 0)
	{
		unSendMessage_toExpertException("Right Click",exceptionInfo);
	}

}

//---------------------------------------------------------------------------------------------------------------------------------------

// unProcessControlObject_FaceplateStatusRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in ProcessControlObject faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unProcessControlObject_FaceplateStatusRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, sSystemName;
	int iAction;
	dyn_string exceptionInfo;
	bool bRemote;
		
	deviceName = unGenericDpFunctions_getDpName($sDpName);
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;

	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			unProcessControlObject_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
			unProcessControlObject_FaceplateConnect(deviceName);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			unProcessControlObject_FaceplateDisconnect(deviceName);
			unProcessControlObject_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unProcessControlObject_FaceplateConnect
/**
Purpose: connect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in ProcessControlObject faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unProcessControlObject_FaceplateConnect(string deviceName)
{
	int iRes, iAlertType1, iAlertType2, iAlertType3;
	dyn_string exceptionInfo;
	string sPlcName, sSystemName;

// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	if(sPlcName != "") {
		if (dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;
		
	iRes = dpGet(deviceName + ".ProcessInput.FuStopAl:_alert_hdl.._type", iAlertType1,
				 deviceName + ".ProcessInput.StartAl:_alert_hdl.._type", iAlertType2,
				 deviceName + ".ProcessInput.TStopAl:_alert_hdl.._type", iAlertType3);
	g_bBoolAlert = ((iRes >= 0) && (iAlertType1 == DPCONFIG_ALERT_BINARYSIGNAL) && 
					(iAlertType2 == DPCONFIG_ALERT_BINARYSIGNAL) && (iAlertType3 == DPCONFIG_ALERT_BINARYSIGNAL));

	if(g_bUnSystemAlarmPlc) {
		if (g_bBoolAlert)
			{
			iRes = dpConnect("unProcessControlObject_FaceplateStatusAnimationCB",
							 deviceName + ".ProcessInput.StsReg01", 
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
							 deviceName + ".ProcessInput.StsReg02", 
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
							 deviceName + ".ProcessInput.OpMoSt",
							 deviceName + ".ProcessInput.OpMoSt:_online.._invalid",
							 deviceName + ".ProcessInput.AuOpMo",
							 deviceName + ".ProcessInput.AuOpMo:_online.._invalid",
							 deviceName + ".ProcessInput.StartAl:_alert_hdl.._act_state",
							 deviceName + ".ProcessInput.TStopAl:_alert_hdl.._act_state",
							 deviceName + ".ProcessInput.FuStopAl:_alert_hdl.._act_state",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							 deviceName + ".eventMask");
			}
		else
			{
			iRes = dpConnect("unProcessControlObject_FaceplateStatusAnimationCB_NoAlarm",
							 deviceName + ".ProcessInput.StsReg01", 
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
							 deviceName + ".ProcessInput.StsReg02", 
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
							 deviceName + ".ProcessInput.OpMoSt",
							 deviceName + ".ProcessInput.OpMoSt:_online.._invalid",
							 deviceName + ".ProcessInput.AuOpMo",
							 deviceName + ".ProcessInput.AuOpMo:_online.._invalid",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							 deviceName + ".eventMask");
			}
	}
	else 
		unProcessControlObject_FaceplateStatusDisconnection();
	g_bCallbackConnected = (iRes >= 0);
}
//---------------------------------------------------------------------------------------------------------------------------------------

// unProcessControlObject_FaceplateDisconnect
/**
Purpose: disconnect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in ProcessControlObject faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unProcessControlObject_FaceplateDisconnect(string deviceName)
{
	int iRes;
	string sPlcName, sSystemName;

	if(g_bUnSystemAlarmPlc) {
// get the systemName
		sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
		sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);

		if (g_bBoolAlert)
			{
			iRes = dpDisconnect("unProcessControlObject_FaceplateStatusAnimationCB",
							 deviceName + ".ProcessInput.StsReg01", 
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
							 deviceName + ".ProcessInput.StsReg02", 
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
							 deviceName + ".ProcessInput.OpMoSt",
							 deviceName + ".ProcessInput.OpMoSt:_online.._invalid",
							 deviceName + ".ProcessInput.AuOpMo",
							 deviceName + ".ProcessInput.AuOpMo:_online.._invalid",
							 deviceName + ".ProcessInput.StartAl:_alert_hdl.._act_state",
							 deviceName + ".ProcessInput.TStopAl:_alert_hdl.._act_state",
							 deviceName + ".ProcessInput.FuStopAl:_alert_hdl.._act_state",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							 deviceName + ".eventMask");
			}
		else
			{
			iRes = dpDisconnect("unProcessControlObject_FaceplateStatusAnimationCB_NoAlarm",
							 deviceName + ".ProcessInput.StsReg01", 
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
							 deviceName + ".ProcessInput.StsReg02", 
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid",
							 deviceName + ".ProcessInput.OpMoSt",
							 deviceName + ".ProcessInput.OpMoSt:_online.._invalid",
							 deviceName + ".ProcessInput.AuOpMo",
							 deviceName + ".ProcessInput.AuOpMo:_online.._invalid",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
							 sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
							 deviceName + ".eventMask");
			}
	}
	g_bCallbackConnected = !(iRes >= 0);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unProcessControlObject_FaceplateStatusAnimationCB_NoAlarm
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unProcessControlObject_FaceplateStatusAnimationCB_NoAlarm(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
									string sStsReg02, bit32 bit32StsReg02, string sDp4, bool bStsReg02Invalid,
									string sDpOpMoSt, int iOpMoSt, string sDp6, bool bOpMoStInvalid, 
				 			        string sDpAuOpMo, int iAuOpMo, string sDp8, bool bAuOpMoInvalid,
				 			        string sDpAl, int iValAlarm, string sDpEnabled, bool bPlcEnabled, string sDpEvt, int iMask)
{
	unProcessControlObject_FaceplateStatusAnimationCB(sStsReg01, bit32StsReg01, sDp2, bStsReg01Invalid, 
									    sStsReg02, bit32StsReg02, sDp4, bStsReg02Invalid,
										sDpOpMoSt, iOpMoSt, sDp6, bOpMoStInvalid,
					 			        sDpAuOpMo, iAuOpMo, sDp8, bAuOpMoInvalid, "", 0, "", 0, "", 0, sDpAl, iValAlarm, sDpEnabled, bPlcEnabled,
					 			        sDpEvt, iMask);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unProcessControlObject_FaceplateStatusAnimationCB
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in ProcessControlObject faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unProcessControlObject_FaceplateStatusAnimationCB(string sStsReg01, bit32 bit32StsReg01, string sDp2, bool bStsReg01Invalid,
									string sStsReg02, bit32 bit32StsReg02, string sDp4, bool bStsReg02Invalid,
									string sDpOpMoSt, int iOpMoSt, string sDp6, bool bOpMoStInvalid, 
				 			        string sDpAuOpMo, int iAuOpMo, string sDp8, bool bAuOpMoInvalid,
									string sDp19, int iStartAlState, string sDp20, int iTStopAlState, string sDp21, int iFuStopAlState,
				 			        string sDpAl, int iValAlarm, string sDpEnabled, bool bSystemIntegrityAlarmEnabled, string sDpEvt, int iMask)
{
	string result, deviceName;

// 1. Animate colorboxes
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("OperationMode_Auto", getBit(bit32StsReg01, UN_STSREG01_AUMOST), 1, "unFaceplate_OperationModeActive", 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)		
	unGenericObject_ColorBoxAnimate("OperationMode_Manual", getBit(bit32StsReg01, UN_STSREG01_MMOST), 1, "unFaceplate_OperationModeActive", 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)		
	unGenericObject_ColorBoxAnimate("OperationMode_Forced", getBit(bit32StsReg01, UN_STSREG01_FOMOST), 1, "unFaceplate_OperationModeActive", 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)		
	unGenericObject_ColorBoxAnimate("Warning_IOError", getBit(bit32StsReg01, UN_STSREG01_IOERRORW), 1, "unFaceplate_WarningActive", 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)		
	unGenericObject_ColorBoxAnimate("Warning_AlBW", getBit(bit32StsReg02, UN_STSREG02_ALBW), 1, "unFaceplate_WarningActive", 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Warning_IOSimulated", getBit(bit32StsReg01, UN_STSREG01_IOSIMUW), 1, "unFaceplate_WarningActive", 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimate("Warning_ManualAuto", getBit(bit32StsReg01, UN_STSREG01_AUMRW), 1, "unFaceplate_WarningActive", 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)		
	unGenericObject_ColorBoxAnimate("Status_OnOpened", getBit(bit32StsReg01, UN_STSREG01_ONST), 1, "unFaceplate_StatusActive", 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)		
	unGenericObject_ColorBoxAnimate("Status_OffClosed", getBit(bit32StsReg01, UN_STSREG01_OFFST), 1, "unFaceplate_StatusActive", 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)		
	unGenericObject_ColorBoxAnimate("Status_AlarmAction", getBit(bit32StsReg02, UN_STSREG02_ALACTB), 1, "unFaceplate_StatusAlarmActive", 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)		
	unGenericObject_ColorBoxAnimate("Status_Temporary", getBit(bit32StsReg02, UN_STSREG02_TFUSTOP), 1, "unFaceplate_StatusAlarmActive", 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)		
	unGenericObject_ColorBoxAnimate("Request_AutoOn", getBit(bit32StsReg02, UN_STSREG02_AUONRST), 1, "unFaceplate_RequestActive", 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)		
	unGenericObject_ColorBoxAnimate("Request_ManualOn", getBit(bit32StsReg02, UN_STSREG02_MONRST), 1, "unFaceplate_RequestActive", 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)		
	unGenericObject_ColorBoxAnimate("Request_AutoControlledStop", getBit(bit32StsReg02, UN_STSREG02_AUCSTOP), 1, "unFaceplate_RequestActive", 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)		
	unGenericObject_ColorBoxAnimate("Order_Run", getBit(bit32StsReg02, UN_STSREG02_RUNOST), 1, "unFaceplate_OrderActive", 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)		
	unGenericObject_ColorBoxAnimate("Order_ControlStop", getBit(bit32StsReg02, UN_STSREG02_CSTOPOST), 1, "unFaceplate_OrderActive", 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
// 2. Animate alarm colorboxes
if(g_bSystemConnected)
	unGenericObject_ColorBoxAnimateAlarm("Alarm_StartAl", iStartAlState, g_bBoolAlert, 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)		
	unGenericObject_ColorBoxAnimateAlarm("Alarm_TStopAl", iTStopAlState, g_bBoolAlert, 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)		
	unGenericObject_ColorBoxAnimateAlarm("Alarm_FuStopAl", iFuStopAlState, g_bBoolAlert, 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
if(g_bSystemConnected)		
	unGenericObject_ColorBoxAnimate("Alarm_UnAck", getBit(bit32StsReg01, UN_STSREG01_ALUNACK), 1, "unFaceplate_AlarmNotAck", 
		bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value));
// 3. Display option modes
	deviceName = unGenericDpFunctions_getDpName(sStsReg01);
	unPCO_getOptionModesLabel(deviceName, iOpMoSt, result);
if(g_bSystemConnected)	
	setValue("OrderActiveOption", "text", result);
if(g_bSystemConnected)	
	setValue("OrderActiveOption", "foreCol", "unDisplayValue_Output");
	if (bOpMoStInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value))
		{
		if(g_bSystemConnected)
		setValue("OrderActiveOption", "foreCol", "unDataNotValid");
		}
	unPCO_getOptionModesLabel(deviceName, iAuOpMo, result);
	if(g_bSystemConnected)
	setValue("RequestAutoOption", "text", result);
	if(g_bSystemConnected)
	setValue("RequestAutoOption", "foreCol", "unDisplayValue_Request");
	if (bAuOpMoInvalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value))
		{
		if(g_bSystemConnected)
		setValue("RequestAutoOption", "foreCol", "unDataNotValid");
		}

// display event state
	if(g_bSystemConnected) {
		unGenericObject_ColorBoxAnimate("eventState", iMask, 1, "", 
				bStsReg01Invalid || bStsReg02Invalid || (!bSystemIntegrityAlarmEnabled) || (iValAlarm > c_unSystemIntegrity_no_alarm_value),
				"unFaceplate_OperationModeActive");
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unProcessControlObject_FaceplateStatusDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unProcessControlObject_FaceplateStatusDisconnection()
{
	unGenericObject_ColorBoxDisconnect(makeDynString("Alarm_StartAl", "Alarm_TStopAl", "Alarm_FuStopAl", "Alarm_UnAck", 
													 "OperationMode_Auto", "OperationMode_Forced", "OperationMode_Manual",
													 "Warning_IOError", "Warning_IOSimulated", "Warning_ManualAuto", 
													 "Status_OnOpened", "Status_OffClosed", "Status_AlarmAction", "Status_Temporary", 
													 "Request_AutoOn", "Request_ManualOn", "Request_AutoControlledStop",
													 "Order_Run", "Order_ControlStop"));
	setValue("OrderActiveOption", "foreCol", "unDataNoAccess");
	setValue("RequestAutoOption", "foreCol", "unDataNoAccess");
// set eventState
	unGenericObject_ColorBoxDisconnect(makeDynString("eventState"));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unProcessControlObject_ButtonRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in ProcessControlObject faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unProcessControlObject_ButtonRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, sDpType, sFunction;
	int iAction, iRes, iManReg01;
	dyn_string exceptionInfo, dsUserAccess, dsFunctions;
	
	deviceName = unGenericDpFunctions_getDpName($sDpName);
	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			unProcessControlObject_ButtonDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
// Send 0 to ManReg01 if necessary
			iRes = dpGet(deviceName + ".ProcessOutput.ManReg01", iManReg01);
			if ((iRes < 0) || (iManReg01 != 0))
			{
				dpSet(deviceName + ".ProcessOutput.ManReg01", 0);
			}
		
// call the unProcessControlObject_ButtonUserAccess to get the list of allowed buttons.
//	unProcessControlObject_ButtonUserAccess(deviceName, "ProcessControlObject", g_dsUserAccess);
			sDpType="ProcessControlObject";
			unGenericObject_GetFunctions(sDpType, dsFunctions, exceptionInfo);
			sFunction = dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS];
						 
			if (sFunction != "")
				{
				evalScript(dsUserAccess, "dyn_string main(string deviceNameTemp, string sDpTypeTemp) {" + 
							 "dyn_string dsUserAccessReturn;" +
							 "if (isFunctionDefined(\"" + sFunction  + "\"))" +
							 "    {" +
					 			sFunction  + "(deviceNameTemp, sDpTypeTemp, dsUserAccessReturn);" +
							 "    }" +
							 "return dsUserAccessReturn; }", makeDynString(), deviceName, sDpType);
						
				g_dsUserAccess=dsUserAccess;	
				}
			else
				fwException_raise(exceptionInfo,"ERROR", "unProcessControlObject_ButtonRegisterCB:" + getCatStr("unGeneration","UNKNOWNFUNCTION"), "EMPTY");
					

			iRes = dpConnect("unProcessControlObject_ButtonAnimationCB",
							 deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							 deviceName + ".statusInformation.selectedManager",
							 deviceName + ".ProcessInput.StsReg01",
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
							 deviceName + ".ProcessInput.StsReg02",
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid");
			g_bCallbackConnected = (iRes >= 0);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			iRes = dpDisconnect("unProcessControlObject_ButtonAnimationCB",
							 deviceName + ".statusInformation.selectedManager:_lock._original._locked",
							 deviceName + ".statusInformation.selectedManager",
							 deviceName + ".ProcessInput.StsReg01",
							 deviceName + ".ProcessInput.StsReg01:_online.._invalid",
							 deviceName + ".ProcessInput.StsReg02",
							 deviceName + ".ProcessInput.StsReg02:_online.._invalid");
			g_bCallbackConnected = !(iRes >= 0);
			unProcessControlObject_ButtonDisconnection();
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unProcessControlObject_ButtonAnimationCB
/**
Purpose: animate buttons

Parameters: 
	- dbPermissions, dyn_bool, input, list of permissions, length = 4, order : 1=monitor, 2=operator, 3=expert, 4=admin
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unProcessControlObject_ButtonAnimationCB(string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager, 
						   string sDpStsReg01, bit32 stsReg01Value, string sDpStsReg01Bad, bool bStsReg01Bad,
						   string sDpStsReg02, bit32 stsReg02Value, string sDpStsReg02Bad, bool bStsReg02Bad)
{
	unProcessControlObject_ButtonSetState(unGenericDpFunctions_getDpName(sDpStsReg01), "ProcessControlObject", g_dsUserAccess, 
						 		  makeDynString(bLocked, sSelectedManager, stsReg01Value, bStsReg01Bad, stsReg02Value, bStsReg02Bad));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unProcessControlObject_ButtonUserAccess
/**
Purpose: User access for ProcessControlObject buttons

Parameters: 
	- sDpName, string, input, device name
	- sDpType, string, input, dptype
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unProcessControlObject_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
	bool operator, expert, admin;
	int i, length;
	dyn_bool dbPermissions;
	dyn_string exceptionInfo;
        string sOperAction, sExpAction, sAdminAction;
        bool bActionDefined = false;

        unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo);
        if((sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME) ||
           (sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME))
          bActionDefined=true;
	operator = dbPermissions[2];
	expert = dbPermissions[3];
	admin = dbPermissions[4];
	dsAccess = makeDynString(UN_FACEPLATE_BUTTON_NEXT, UN_FACEPLATE_BUTTON_BACK);
	if (operator)
	{
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
          else
	    dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_AUTO_MODE, UN_FACEPLATE_BUTTON_MANUAL_MODE,
							UN_FACEPLATE_BUTTON_ONOPEN_REQUEST, UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST, UN_FACEPLATE_BUTTON_ACK_ALARM,
							UN_FACEPLATE_BUTTON_SELECT, UN_FACEPLATE_BUTTON_CONTROLLED_STOP_REQUEST,
							UN_FACEPLATE_BUTTON_OPTION_MODES, UN_FACEPLATE_BUTTON_MANUAL_AUTO_TO_DEPENDENT_OBJECT));
	}
	if (expert)
	{
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
          else
	    dynAppend(dsAccess, makeDynString(UN_FACEPLATE_BUTTON_FORCED_MODE, UN_FACEPLATE_BUTTON_BLOCK_ALARM_ACTION,
							UN_FACEPLATE_BUTTON_DEBLOCK_ALARM_ACTION, UN_FACEPLATE_BUTTON_SET_TEMPORARY_AS_FULL_STOP,
							UN_FACEPLATE_BUTTON_RESET_TEMPORARY_AS_FULL_STOP, UN_FACEPLATE_MASKEVENT));
	}
        if(admin){
          if(bActionDefined)
            dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
        }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unProcessControlObject_UserLoginGetButtonState
/**
Purpose: function called before the button set state when a user logs in. 

Parameters:
	deviceName: string, input, the device name
	dsReturnData: dyn_string, output, the data are returned here

Usage: Internal function

PVSS manager usage:  Ctrl, NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
		-> $sDpName, string, the device DP name
		-> g_sFaceplateButtonType, string, faceplate type
	. PVSS version: 2.12.1 
	. operating system: NT, W2000 and Linux, but tested only under W2000.
	. distributed system: yes but not tested.
*/

unProcessControlObject_UserLoginGetButtonState(string deviceName, string sType, dyn_string &dsReturnData)
{
	unGenericObject_UserCBGetButtonState(deviceName, sType, dsReturnData);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unProcessControlObject_ButtonSetState
/**
Purpose: set button state

Parameters: 
	- deviceName, string, input, device
	- sDpType, string, input, dptype
	- dsUserAccess, dyn_string, input, user access
	- dsData, dyn_string, parameter
		
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unProcessControlObject_ButtonSetState(string deviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
	bool bSelected, buttonEnabled, bit1, bit2;
	dyn_string dsButtons;
	int i, length;
	string localManager;
	bool bLocked, bStsReg01Bad, bStsReg02Bad;
	string sSelectedManager;
	bit32 stsReg01Value, stsReg02Value; 

	bLocked = dsData[1];
	sSelectedManager = dsData[2];
	stsReg01Value = dsData[3];
	bStsReg01Bad = dsData[4];
	stsReg02Value = dsData[5];
	bStsReg02Bad = dsData[6];

//DebugN("ProcessControlObject_SetState", bLocked, sSelectedManager, stsReg01Value, bStsReg01Bad, stsReg02Value, bStsReg02Bad);

	dsButtons = makeDynString(UN_FACEPLATE_BUTTON_NEXT, UN_FACEPLATE_BUTTON_BACK, UN_FACEPLATE_BUTTON_AUTO_MODE,
							  UN_FACEPLATE_BUTTON_ONOPEN_REQUEST, UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST, UN_FACEPLATE_BUTTON_ACK_ALARM,
							  UN_FACEPLATE_BUTTON_CONTROLLED_STOP_REQUEST, UN_FACEPLATE_BUTTON_RESET_TEMPORARY_AS_FULL_STOP,
							  UN_FACEPLATE_BUTTON_OPTION_MODES, UN_FACEPLATE_BUTTON_MANUAL_AUTO_TO_DEPENDENT_OBJECT, 
							  UN_FACEPLATE_BUTTON_FORCED_MODE, UN_FACEPLATE_BUTTON_BLOCK_ALARM_ACTION, UN_FACEPLATE_BUTTON_MANUAL_MODE,
							  UN_FACEPLATE_BUTTON_DEBLOCK_ALARM_ACTION, UN_FACEPLATE_BUTTON_SET_TEMPORARY_AS_FULL_STOP);
	length = dynlen(dsButtons);
// 1. Selection state
	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
	bSelected = (localManager == "S");
// 2. Button state
	for(i=1;i<=length;i++)
		{
		buttonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0);	// User access
		switch (dsButtons[i])
			{
			case UN_FACEPLATE_BUTTON_SET_TEMPORARY_AS_FULL_STOP:
				bit1 = !(getBit(stsReg01Value, UN_STSREG01_AUMOST) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				bit2 = !(getBit(stsReg02Value, UN_STSREG02_TFUSTOP) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				buttonEnabled = buttonEnabled && bSelected && !(bit1 || bit2);
				break;
			case UN_FACEPLATE_BUTTON_RESET_TEMPORARY_AS_FULL_STOP:
				bit1 = !(getBit(stsReg01Value, UN_STSREG01_AUMOST) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				bit2 = (getBit(stsReg02Value, UN_STSREG02_TFUSTOP) == 1) || (bStsReg01Bad || bStsReg02Bad);
				buttonEnabled = buttonEnabled && bSelected && !(bit1) && bit2;
				break;
			case UN_FACEPLATE_BUTTON_BLOCK_ALARM_ACTION:
				bit1 = !(getBit(stsReg01Value, UN_STSREG01_AUMOST) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				bit2 = !(getBit(stsReg02Value, UN_STSREG02_ALACTB) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				buttonEnabled = buttonEnabled && bSelected && !(bit1 || bit2);
				break;
			case UN_FACEPLATE_BUTTON_DEBLOCK_ALARM_ACTION:
				bit1 = !(getBit(stsReg01Value, UN_STSREG01_AUMOST) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				bit2 = (getBit(stsReg02Value, UN_STSREG02_ALACTB) == 1) || (bStsReg01Bad || bStsReg02Bad);
				buttonEnabled = buttonEnabled && bSelected && !(bit1) && bit2;
				break;
			case UN_FACEPLATE_BUTTON_FORCED_MODE:
				bit1 = !(getBit(stsReg01Value, UN_STSREG01_FOMOST) == 0) && !(bStsReg01Bad || bStsReg02Bad);	
				bit2 = !(getBit(stsReg01Value, UN_STSREG01_AUINHFMO) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				buttonEnabled = buttonEnabled && bSelected && !(bit1 || bit2);
				break;
			case UN_FACEPLATE_BUTTON_MANUAL_MODE:
				bit1 = !(getBit(stsReg01Value, UN_STSREG01_MMOST) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				bit2 = !(getBit(stsReg01Value, UN_STSREG01_AUINHMMO) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				buttonEnabled = buttonEnabled && bSelected && !(bit1 || bit2);
				break;
			case UN_FACEPLATE_BUTTON_AUTO_MODE:
			case UN_FACEPLATE_BUTTON_ONOPEN_REQUEST:
			case UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST:
			case UN_FACEPLATE_BUTTON_CONTROLLED_STOP_REQUEST:
			case UN_FACEPLATE_BUTTON_OPTION_MODES:
			case UN_FACEPLATE_BUTTON_MANUAL_AUTO_TO_DEPENDENT_OBJECT:
				bit1 = !(getBit(stsReg01Value, UN_STSREG01_AUMOST) == 0) && !(bStsReg01Bad || bStsReg02Bad);
				buttonEnabled = buttonEnabled && bSelected && !(bit1);
				break;
			case UN_FACEPLATE_BUTTON_NEXT:
			case UN_FACEPLATE_BUTTON_BACK:
			case UN_FACEPLATE_BUTTON_ACK_ALARM:
			default:
				break;
			}
		setValue(UN_FACEPLATE_BUTTON_PREFIX + dsButtons[i], "enabled", buttonEnabled);
		}
// 3. Button Select state
	unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unProcessControlObject_ButtonDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unProcessControlObject_ButtonDisconnection()
{
	unGenericObject_ButtonDisconnect(makeDynString(UN_FACEPLATE_BUTTON_NEXT, UN_FACEPLATE_BUTTON_BACK, UN_FACEPLATE_BUTTON_AUTO_MODE,
							  UN_FACEPLATE_BUTTON_ONOPEN_REQUEST, UN_FACEPLATE_BUTTON_OFFCLOSE_REQUEST, UN_FACEPLATE_BUTTON_ACK_ALARM,
							  UN_FACEPLATE_BUTTON_CONTROLLED_STOP_REQUEST, UN_FACEPLATE_BUTTON_RESET_TEMPORARY_AS_FULL_STOP,
							  UN_FACEPLATE_BUTTON_OPTION_MODES, UN_FACEPLATE_BUTTON_MANUAL_AUTO_TO_DEPENDENT_OBJECT, 
							  UN_FACEPLATE_BUTTON_FORCED_MODE, UN_FACEPLATE_BUTTON_BLOCK_ALARM_ACTION, UN_FACEPLATE_BUTTON_MANUAL_MODE,
							  UN_FACEPLATE_BUTTON_DEBLOCK_ALARM_ACTION, UN_FACEPLATE_BUTTON_SET_TEMPORARY_AS_FULL_STOP,
							  UN_FACEPLATE_BUTTON_SELECT));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unProcessControlObject_WidgetRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in ProcessControlObject widget
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unProcessControlObject_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName;
	int iAction;
	dyn_string exceptionInfo;
	string sPlcName, sSystemName;
	bool bRemote;
		
	deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
	if (deviceName == "")	// In case of disconnection
		{
		deviceName = g_sDpName;
		}
// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sPlcName = unGenericObject_GetPLCNameFromDpName(deviceName);
	
	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;
	
	if(sPlcName != "") {
		if (dpExists(sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName)) {
			g_bUnSystemAlarmPlc = true;
		}
		else {
			g_bUnSystemAlarmPlc = false;
		}
	}
	else
		g_bUnSystemAlarmPlc = false;
		
	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			unProcessControlObject_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPCONNECT:
			g_sDpName = deviceName;
			unProcessControlObject_WidgetConnect(deviceName, sPlcName, sSystemName);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			unProcessControlObject_WidgetDisconnect(deviceName, sPlcName, sSystemName);
			unProcessControlObject_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}
//---------------------------------------------------------------------------------------------------------------------------------------

// unProcessControlObject_WidgetConnect
/**
Purpose: Connect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in ProcessControlObject faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unProcessControlObject_WidgetConnect(string deviceName, string sPlcName, string sSystemName)
{
	int iRes;
	
//  check if the device name is of type "ProcessControlObject" if not like of no PLC. -> not available.
	if(dpTypeName(deviceName) != "ProcessControlObject")
		g_bUnSystemAlarmPlc = false;
				
	if(g_bUnSystemAlarmPlc) {
		iRes = dpConnect("unProcessControlObject_WidgetAnimationCB", 
						deviceName + ".statusInformation.selectedManager:_lock._original._locked",
						deviceName + ".statusInformation.selectedManager",
						deviceName + ".ProcessInput.StsReg01",
						deviceName + ".ProcessInput.StsReg01:_online.._invalid",
						deviceName + ".ProcessInput.StsReg02",
						deviceName + ".ProcessInput.StsReg02:_online.._invalid",
						deviceName + ".ProcessInput.OpMoSt",
						deviceName + ".ProcessInput.OpMoSt:_online.._invalid",
						deviceName + ".ProcessInput.AuOpMo:_online.._invalid",
						sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
						sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
						deviceName + ".eventMask");
	g_bCallbackConnected = (iRes >= 0);	
  }
	else 
		unProcessControlObject_WidgetDisconnection(g_sWidgetType);

}
//---------------------------------------------------------------------------------------------------------------------------------------

// unProcessControlObject_WidgetDisconnect
/**
Purpose: Disconnect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in ProcessControlObject faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unProcessControlObject_WidgetDisconnect(string deviceName, string sPlcName, string sSystemName)
{
	int iRes;
	
	if(g_bUnSystemAlarmPlc)
	{
		iRes = dpDisconnect("unProcessControlObject_WidgetAnimationCB", 
						deviceName + ".statusInformation.selectedManager:_lock._original._locked",
						deviceName + ".statusInformation.selectedManager",
						deviceName + ".ProcessInput.StsReg01",
						deviceName + ".ProcessInput.StsReg01:_online.._invalid",
						deviceName + ".ProcessInput.StsReg02",
						deviceName + ".ProcessInput.StsReg02:_online.._invalid",
						deviceName + ".ProcessInput.OpMoSt",
						deviceName + ".ProcessInput.OpMoSt:_online.._invalid",
						deviceName + ".ProcessInput.AuOpMo:_online.._invalid",
						sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".alarm",
						sSystemName+c_unSystemAlarm_dpPattern+PLC_DS_pattern+sPlcName+".enabled",
						deviceName + ".eventMask");
	}
	g_bCallbackConnected = !(iRes >= 0);
}
//---------------------------------------------------------------------------------------------------------------------------------------

// unProcessControlObject_WidgetAnimationCB
/**
Purpose: Animate widget

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in ProcessControlObject faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unProcessControlObject_WidgetAnimationCB(string sDpLock, bool bLocked, string sDpManager, string sSelectedManager,
									     string sDpStsReg01, bit32 bit32StsReg01, string sDp4, bool bStsReg01Invalid,
									     string sDpStsReg02, bit32 bit32StsReg02, string sDp5, bool bStsReg02Invalid,
									     string sDpOpMoSt, int iOpMoSt, string sDp7, bool bOpMoStInvalid,
									     string sDp8, bool bAuOpMoInvalid,
										string sDpAl, int iSystemIntegrityAlarmValue, string sDpEnabled, bool bSystemIntegrityAlarmEnabled,
										string sEvtMask, int iMask)
{
	string deviceName, sFunction, blockColor, blockLetter, pcoColor, pcoName, sOpMo, displayColor;
	string sColorSelect, sWarningLetter, sWarningColor, sAlarmLetter, sAlarmColor, sControlLetter, sControlColor, sBodyColor;
	bool bLockVisible;
	dyn_string dsFunctions, exceptionInfo;
	
	deviceName = unGenericDpFunctions_getDpName(sDpStsReg01);
// 1. Select
	unGenericObject_WidgetSelectAnimation(bLocked, sSelectedManager, sColorSelect, bLockVisible);
// 2. Warning (Include the second stsReg: stsReg02 to check the warning letter)
	unGenericObject_WidgetWarningAnimationDoubleStsReg(bit32StsReg01, bit32StsReg02,
						makeDynInt(UN_STSREG01_AUMRW, UN_STSREG01_IOSIMUW, UN_STSREG01_IOERRORW),
                                                makeDynInt(UN_STSREG02_ALBW),
						makeDynString(UN_WIDGET_TEXT_WARNING_DEFAULT, UN_WIDGET_TEXT_WARNING_SIMU, UN_WIDGET_TEXT_WARNING_ERROR),
                                                makeDynString(UN_WIDGET_TEXT_WARNING_BLOCKED),
						sWarningLetter, sWarningColor);
// 2.1 if the device is not correclty running (the iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value) 
// or the checking is not enabled then set the letter to O and the color to not valid data

	if ((!bSystemIntegrityAlarmEnabled) || (iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value))
		{
		sWarningLetter = UN_WIDGET_TEXT_OLD_DATA;
		sWarningColor = "unDataNotValid";
		}

	if (bStsReg01Invalid || bStsReg02Invalid || bOpMoStInvalid || bAuOpMoInvalid)
		{
		sWarningLetter = UN_WIDGET_TEXT_INVALID;
		sWarningColor = "unDataNotValid";
		}
// 3. Alarm
	unGenericObject_WidgetAlarmTextAnimation(bit32StsReg01, 
						makeDynInt(UN_STSREG01_STARTAL, UN_STSREG01_TSTOPAL, UN_STSREG01_FUSTOPAL), 
						makeDynString(UN_WIDGET_TEXT_ALARM_STARTAL, UN_WIDGET_TEXT_ALARM_TSTOPAL, UN_WIDGET_TEXT_ALARM_FUSTOPAL),
						sAlarmLetter, sAlarmColor);
// 4. Control state
	unGenericObject_WidgetControlStateAnimation(bit32StsReg01,
						makeDynInt(UN_STSREG01_MMOST, UN_STSREG01_FOMOST),
						makeDynString(UN_WIDGET_TEXT_CONTROL_MANUAL, UN_WIDGET_TEXT_CONTROL_FORCED),
						sControlLetter, sControlColor);
// 5. Block text
	if (getBit(bit32StsReg02, UN_STSREG02_TFUSTOP) == 1)
		{
		blockLetter = UN_WIDGET_TEXT_BLOCK_TFUSTOP;
		blockColor = "unFaceplate_StatusAlarmActive";
		}
	if (getBit(bit32StsReg02, UN_STSREG02_ALACTB) == 1)
		{
		blockLetter = UN_WIDGET_TEXT_ALARM_BLOCKED;
		blockColor = "unFaceplate_StatusAlarmActive";
		}
// 6. Animate letters and select
if(g_bSystemConnected)
	setMultiValue("WarningText", "text", sWarningLetter, "WarningText", "foreCol", sWarningColor,
				  "AlarmText", "text", sAlarmLetter, "AlarmText", "foreCol", sAlarmColor,
				  "ControlStateText", "text", sControlLetter, "ControlStateText", "foreCol", sControlColor,
				  "SelectArea", "foreCol", sColorSelect, "LockBmp", "visible", bLockVisible,
				  "BlockText", "text", blockLetter, "BlockText", "foreCol", blockColor,
				  "WidgetArea", "visible", true);
// 7. Animate body
	sBodyColor = "unWidget_ControlStateAuto";
	if (bStsReg01Invalid || bStsReg02Invalid)
		{
		sBodyColor = "unDataNotValid";
		}
	else
		{
		if (sControlLetter == UN_WIDGET_TEXT_CONTROL_FORCED)
			{
			sBodyColor = "unWidget_ControlStateForced";
			}
		if (blockLetter == UN_WIDGET_TEXT_BLOCK_TFUSTOP)
			{
			sBodyColor = "unFaceplate_StatusAlarmActive";
			}
		if (sAlarmLetter != "")
			{
			sBodyColor = "unFaceplate_AlarmActive";
			}
		if (blockLetter == UN_WIDGET_TEXT_ALARM_BLOCKED)
			{
			sBodyColor = "unFaceplate_StatusAlarmActive";
			}	
		if (getBit(bit32StsReg01, UN_STSREG01_ALUNACK) == 1)
			{
			sBodyColor = "unWidget_AlarmNotAck";
			}
		}
	unGenericObject_WidgetGetFunctions(g_sWidgetType, "ProcessControlObject", dsFunctions, exceptionInfo);
	sFunction = dsFunctions[UN_GENERICWIDGET_FUNCTION_ANIMATION];
	if (isFunctionDefined(sFunction) && (sFunction != ""))
		{
		if(g_bSystemConnected)
			execScript("main(int iOn, int iOff, string sBodyColor) {" +
				   sFunction + "(iOn, iOff, sBodyColor);}", 
				   makeDynString(), getBit(bit32StsReg01, UN_STSREG01_ONST), getBit(bit32StsReg01, UN_STSREG01_OFFST), sBodyColor);
		}
// 8. PCO name
	pcoColor = sBodyColor;
	if (getBit(bit32StsReg01, UN_STSREG01_OFFST) == 0)
		{
		pcoColor = "unWidget_AlternativeColor";
		}
	if (bStsReg01Invalid || bStsReg02Invalid)
		{
		pcoColor = "unDataNotValid";
		}
	unPCO_getName(deviceName, pcoName);
	if(g_bSystemConnected)
		setMultiValue("PCOName", "text", pcoName, "PCOName", "foreCol", pcoColor);
// 9. Option mode
	unPCO_getOptionModesLabel(deviceName, iOpMoSt, sOpMo);
	displayColor = "unDisplayValue_Output";
	if (bOpMoStInvalid)
		{
		displayColor = "unDataNotValid";
		}
	if(g_bSystemConnected)	
		setMultiValue("OptionMode", "text", sOpMo, "OptionMode", "foreCol", displayColor);

// display event state
	if(g_bSystemConnected) {
		if(iMask == 0)
			eventState.visible	= true;
		else
			eventState.visible	= false;
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unProcessControlObject_WidgetDisconnection
/**
Purpose: Animate widget disconnection

Parameters:
	- sWidgetType, string, input, widget type. ex: "heater"

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unProcessControlObject_WidgetDisconnection(string sWidgetType)
{
	unGenericObject_WidgetDisconnection(sWidgetType, "ProcessControlObject");
	unGenericObject_WidgetDisplayValueDisconnect(makeDynString("OptionMode", "PCOName"));
	setValue("BlockText", "text", "");

	// set event state
	eventState.visible	= false;
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unPCO_getOptionModesLabel
/**
Purpose: this function is called by a PCO object to get an option mode label

Parameter:
	-> sDpName, string input, device name
	-> uLabel, unsigned input, label number	
	-> result, string output, result of the function								  

Usage: Public

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unPCO_getOptionModesLabel(string sDpName, unsigned uLabel, string &result)
{
	string sOptionModes;
	dyn_string dsOptionModes;
	
	result = "";
	sOptionModes = unGenericDpFunctions_getDescription(sDpName + UN_PCO_OPTION_MODE_PATH);
	dsOptionModes = strsplit(sOptionModes, UN_PCO_OPTION_MODE_DELIMITER);
	if ((dynlen(dsOptionModes) >= uLabel) && (uLabel > 0) && (UN_PCO_OPTION_MODE_NUMBER >= uLabel))
		{
		result = dsOptionModes[uLabel];
		}
	if (result == "")
		{
		//result = "Op Mode " + uLabel;
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unPCO_getOptionModesAllowance
/**
Purpose: this function is called by a PCO object to get an option mode allowance

Parameter:
	-> sDpName, string input, device name
	-> uLabel, unsigned input, label number	
	-> result, bit32 output, result of the function								  

Usage: Public

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unPCO_getOptionModesAllowance(string sDpName, unsigned uLabel, bit32 &result)
{
	string sOptionModes;
	dyn_string dsOptionModes;
	
	result = 0;
	sOptionModes = unGenericDpFunctions_getDescription(sDpName + UN_PCO_OPTION_MODE_PATH);
	dsOptionModes = strsplit(sOptionModes, UN_PCO_OPTION_MODE_DELIMITER);
	if ((dynlen(dsOptionModes) >= (UN_PCO_OPTION_MODE_NUMBER + uLabel)) && (uLabel > 0))
		{
		result = dsOptionModes[UN_PCO_OPTION_MODE_NUMBER + uLabel];
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unPCO_setOptionModes
/**
Purpose: this function is called by a PCO object to set the option mode labels & allowance

Parameter:
	-> sDpName, string input, device name
	-> dsLabels, dyn_string input, labels
	-> dsAllowances, dyn_string input, allowances (must be bit32 variables converted to string)
	-> iRes, int output, result of the function, < 0 if an error occured						  

Usage: Public

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unPCO_setOptionModes(string sDpName, dyn_string dsLabels, dyn_string dsAllowances, int &iRes)
{
	int i,length;
	string description;
	
	iRes = -1;
	length = dynlen(dsLabels);
	if ((length != dynlen(dsAllowances)) || (length == 0))
		{
		//error
		}
	else
		{
		for(i=1;i<=UN_PCO_OPTION_MODE_NUMBER;i++)			// Labels
			{
			if (i <= length)
				{
				description = description + UN_PCO_OPTION_MODE_DELIMITER + dsLabels[i];
				}
			else
				{
				description = description + UN_PCO_OPTION_MODE_DELIMITER;
				}
			}
		for(i=1;i<=UN_PCO_OPTION_MODE_NUMBER;i++)			// Allowances
			{
			if (i <= length)
				{
				description = description + UN_PCO_OPTION_MODE_DELIMITER + dsAllowances[i];
				}
			else
				{
				description = description + UN_PCO_OPTION_MODE_DELIMITER;
				}
			}
		description = substr(description, 1);
		iRes = dpSetDescription(sDpName + UN_PCO_OPTION_MODE_PATH, description);
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unPCO_getName
/**
Purpose: this function is called by a PCO object to get PCO name

Parameter:
	-> sDpName, string input, device name
	-> result, string output, result of the function								  

Usage: Public

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unPCO_getName(string sDpName, string &result)
{
	dyn_string dsParameters;
	
	unGenericDpFunctions_getParameters(sDpName, dsParameters);
	result = dsParameters[UN_PARAMETER_NAME];
	if (result == "")
		{
		result = "PCO";
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unPCO_setName
/**
Purpose: this function is called by a PCO object to set the PCO name

Parameter:
	-> sDpName, string input, device name
	-> name, string input, name to be set						  
	-> exceptionInfo, dyn_string output, for errors

Usage: Public

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unPCO_setName(string sDpName, string name, dyn_string &exceptionInfo)
{
	unGenericDpFunctions_changeDpParameter(sDpName, UN_PARAMETER_NAME, name, exceptionInfo);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unPCO_getNextAllowedModes
/**
Purpose: to get the modes you can access from current mode (using allowance tables)

Parameter:
	-> sDpName, string input, device name
	-> iCurrentMode, int input, current mode
	-> bRunning, bool input, true while running
	-> dsAllowedModes, dyn_string output, result of the function
	-> exceptionInfo, dyn_string output, for errors
	
Usage: Public

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unPCO_getNextAllowedModes(string sDpName, int iCurrentMode, bool bRunning, dyn_string &dsAllowedModes, dyn_string &exceptionInfo)
{
	string parameters;
	dyn_string dsParameters;
	bit32 allowance;
	int i;
	
	dynClear(dsAllowedModes);
/*	if ((iCurrentMode <= 0) || (iCurrentMode >  UN_PCO_OPTION_MODE_NUMBER))
		{
		fwException_raise(exceptionInfo,"ERROR","unPCO_getNextAllowedModes : " + getCatStr("unFunctions","BADINPUTS"),"");
		return;
		}
*/	
	// 1. Get parameters
	parameters = unGenericDpFunctions_getDescription(sDpName + UN_PCO_OPTION_MODE_PATH);
	dsParameters = strsplit(parameters, UN_PCO_OPTION_MODE_DELIMITER);
	while (dynlen(dsParameters) < (2*UN_PCO_OPTION_MODE_NUMBER))
		{
		dynAppend(dsParameters, "");
		}
	
	// 2. Not running -> each defined mode is allowed
	if (bRunning == false)
		{
		for(i=1;i<=UN_PCO_OPTION_MODE_NUMBER;i++)
			{
			if ((strltrim(dsParameters[i]) != "") && (i != iCurrentMode))
				{
				dynAppend(dsAllowedModes,dsParameters[i]);
				}
			}
		}
	else
		{
		
	// 3. While running -> allowed modes are coded in the allowance table
		allowance = dsParameters[iCurrentMode+UN_PCO_OPTION_MODE_NUMBER];
		for(i=1;i<=UN_PCO_OPTION_MODE_NUMBER;i++)
			{
			if ((strltrim(dsParameters[i]) != "") && (getBit(allowance,UN_PCO_OPTION_MODE_NUMBER-i) == 1) && (i != iCurrentMode))
				{
				dynAppend(dsAllowedModes,dsParameters[i]);
				}
			}
		}
}
//@}

